/*
 * Copyright (C) 2015 MediaTek Inc.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 */

/*****************************************************************************
 *
 * Filename:
 * ---------
 *   nt99141yuv_Sensor.c
 *
 * Project:
 * --------
 *   MAUI
 *
 * Description:
 * ------------
 *   Image sensor driver function
 *   V1.0.0
 *
 * Author:
 * -------
 *   Mormo
 *
 *=============================================================
 *             HISTORY
 * Below this line, this part is controlled by PVCS VM. DO NOT MODIFY!!
 *------------------------------------------------------------------------------
 * $Log$
 * 2011/11/03 Firsty Released By Mormo;
 *   
 *
 *------------------------------------------------------------------------------
 * Upper this line, this part is controlled by PVCS VM. DO NOT MODIFY!!
 *=============================================================
 ******************************************************************************/
#include <linux/videodev2.h>
#include <linux/i2c.h>
#include <linux/platform_device.h>
#include <linux/delay.h>
#include <linux/cdev.h>
#include <linux/uaccess.h>
#include <linux/fs.h>
#include <asm/atomic.h>
#include "kd_camera_hw.h"
#include "kd_imgsensor.h"
#include "kd_imgsensor_define.h"
#include "kd_imgsensor_errcode.h"
#include "kd_camera_feature.h"
#include "nt99141yuv_Sensor.h"
#include "nt99141yuv_Camera_Sensor_para.h"
#include "nt99141yuv_CameraCustomized.h"

/*******************************************************************************
 * // Adapter for Winmo typedef
 ********************************************************************************/
#define WINMO_USE 0

#define Sleep(ms) mdelay(ms)
#define RETAILMSG(x,...)
#define TEXT

#define PFX         "NT99141YUV"
#define DEBUG	1
#if DEBUG
    #define LOG_INF(format, args...)	pr_info( PFX "[%s] " format, __FUNCTION__, ##args)
#else
    #define LOG_INF(format, args...)   
#endif

//#define DEBUG_SENSOR_NT99141

//#define NT99141_24M_72M

#define PREVIEW_USE_VGA     0
#define PREVIEW_USE_SVGA    0 // 1
// don't use 720p, sensor may not output data
#define PREVIEW_USE_720P    1 // 0

#if PREVIEW_USE_VGA
#undef IMAGE_SENSOR_PV_WIDTH
#undef IMAGE_SENSOR_PV_HEIGHT
#define IMAGE_SENSOR_PV_WIDTH					(IMAGE_SENSOR_VGA_WIDTH - 8)
#define IMAGE_SENSOR_PV_HEIGHT					(IMAGE_SENSOR_VGA_HEIGHT - 6)
#endif

#if PREVIEW_USE_SVGA
#undef IMAGE_SENSOR_PV_WIDTH
#undef IMAGE_SENSOR_PV_HEIGHT
#define IMAGE_SENSOR_PV_WIDTH					(IMAGE_SENSOR_SVGA_WIDTH - 8)
#define IMAGE_SENSOR_PV_HEIGHT					(IMAGE_SENSOR_SVGA_HEIGHT - 6)
#endif

#if PREVIEW_USE_720P
#undef IMAGE_SENSOR_PV_WIDTH
#undef IMAGE_SENSOR_PV_HEIGHT
#define IMAGE_SENSOR_PV_WIDTH					(IMAGE_SENSOR_720P_WIDTH - 8)
#define IMAGE_SENSOR_PV_HEIGHT					(IMAGE_SENSOR_720P_HEIGHT - 6)
#endif


#ifdef DEBUG_SENSOR_NT99141

kal_uint8 fromsd = 0;
kal_uint16 NT99141_write_cmos_sensor(kal_uint8 addr, kal_uint8 para);
kal_uint16 NT99141_read_cmos_sensor(kal_uint8 addr);

#define NT99141_OP_CODE_INI		0x00		/* Initial value. */
#define NT99141_OP_CODE_REG		0x01		/* Register */
#define NT99141_OP_CODE_DLY		0x02		/* Delay */
#define NT99141_OP_CODE_END		0x03		/* End of initial setting. */
	

typedef struct
{
	u16 init_reg;
	u16 init_val;	/* Save the register value and delay tick */
	u8 op_code;		/* 0 - Initial value, 1 - Register, 2 - Delay, 3 - End of setting. */
} NT99141_initial_set_struct;

NT99141_initial_set_struct NT99141_Init_Reg[1000];

u32 strtol(const char *nptr, u8 base)
{
	u8 ret;
	if(!nptr || (base!=16 && base!=10 && base!=8))
	{
		printk("%s(): NULL pointer input\n", __FUNCTION__);
		return -1;
	}
	for(ret=0; *nptr; nptr++)
	{
		if((base==16 && *nptr>='A' && *nptr<='F') || 
				(base==16 && *nptr>='a' && *nptr<='f') || 
				(base>=10 && *nptr>='0' && *nptr<='9') ||
				(base>=8 && *nptr>='0' && *nptr<='7') )
		{
			ret *= base;
			if(base==16 && *nptr>='A' && *nptr<='F')
				ret += *nptr-'A'+10;
			else if(base==16 && *nptr>='a' && *nptr<='f')
				ret += *nptr-'a'+10;
			else if(base>=10 && *nptr>='0' && *nptr<='9')
				ret += *nptr-'0';
			else if(base>=8 && *nptr>='0' && *nptr<='7')
				ret += *nptr-'0';
		}
		else
			return ret;
	}
	return ret;
}

u8 NT99141_Initialize_from_T_Flash()
{
	//FS_HANDLE fp = -1;				/* Default, no file opened. */
	//u8 *data_buff = NULL;
	u8 *curr_ptr = NULL;
	u32 file_size = 0;
	//u32 bytes_read = 0;
	u32 i = 0, j = 0;
	u8 func_ind[4] = {0};	/* REG or DLY */


	struct file *fp; 
	mm_segment_t fs; 
	loff_t pos = 0; 
	static u8 data_buff[10*1024] ;

    fp = filp_open("/storage/sdcard1/nt99141_sd.dat", O_RDONLY , 0); 
    if (IS_ERR(fp)) { 
		printk("create file error 0\n");
		fp = filp_open("/storage/sdcard0/nt99141_sd.dat", O_RDONLY , 0); 
    	if (IS_ERR(fp)) { 
	        printk("create file error 1\n"); 
	        return 2;//-1; 
    	}
    } 
    else
		printk("NT99141_Initialize_from_T_Flash Open File Success\n");
	fs = get_fs(); 
	set_fs(KERNEL_DS); 

	file_size = vfs_llseek(fp, 0, SEEK_END);
	vfs_read(fp, data_buff, file_size, &pos); 
	//printk("%s %d %d\n", buf,iFileLen,pos); 
	filp_close(fp, NULL); 
	set_fs(fs);


	/* Start parse the setting witch read from t-flash. */
	curr_ptr = data_buff;
	while (curr_ptr < (data_buff + file_size))
	{
		while ((*curr_ptr == ' ') || (*curr_ptr == '\t'))/* Skip the Space & TAB */
			curr_ptr++;				

		if (((*curr_ptr) == '/') && ((*(curr_ptr + 1)) == '*'))
		{
			while (!(((*curr_ptr) == '*') && ((*(curr_ptr + 1)) == '/')))
			{
				curr_ptr++;		/* Skip block comment code. */
			}

			while (!((*curr_ptr == 0x0D) && (*(curr_ptr+1) == 0x0A)))
			{
				curr_ptr++;
			}

			curr_ptr += 2;						/* Skip the enter line */

			continue ;
		}

		if (((*curr_ptr) == '/') || ((*curr_ptr) == '{') || ((*curr_ptr) == '}'))		/* Comment line, skip it. */
		{
			while (!((*curr_ptr == 0x0D) && (*(curr_ptr+1) == 0x0A)))
			{
				curr_ptr++;
			}

			curr_ptr += 2;						/* Skip the enter line */

			continue ;
		}
		/* This just content one enter line. */
		if (((*curr_ptr) == 0x0D) && ((*(curr_ptr + 1)) == 0x0A))
		{
			curr_ptr += 2;
			continue ;
		}
		//printk(" curr_ptr1 = %s\n",curr_ptr);
		memcpy(func_ind, curr_ptr, 3);


		if (strcmp((const char *)func_ind, "REG") == 0)		/* REG */
		{
			curr_ptr += 6;				/* Skip "REG(0x" or "DLY(" */
			NT99141_Init_Reg[i].op_code = NT99141_OP_CODE_REG;

			NT99141_Init_Reg[i].init_reg = strtol((const char *)curr_ptr, 16);
			curr_ptr += 5;	/* Skip "00, 0x" */

			NT99141_Init_Reg[i].init_val = strtol((const char *)curr_ptr, 16);
			curr_ptr += 4;	/* Skip "00);" */

		}
		else									/* DLY */
		{
			/* Need add delay for this setting. */
			curr_ptr += 4;	
			NT99141_Init_Reg[i].op_code = NT99141_OP_CODE_DLY;

			NT99141_Init_Reg[i].init_reg = 0xFF;
			NT99141_Init_Reg[i].init_val = strtol((const char *)curr_ptr,  10);	/* Get the delay ticks, the delay should less then 50 */
		}
		i++;


		/* Skip to next line directly. */
		while (!((*curr_ptr == 0x0D) && (*(curr_ptr+1) == 0x0A)))
		{
			curr_ptr++;
		}
		curr_ptr += 2;
	}

	/* (0xFFFF, 0xFFFF) means the end of initial setting. */
	NT99141_Init_Reg[i].op_code = NT99141_OP_CODE_END;
	NT99141_Init_Reg[i].init_reg = 0xFF;
	NT99141_Init_Reg[i].init_val = 0xFF;
	i++;
	//for (j=0; j<i; j++)
	//printk(" %x  ==  %x\n",NT99141_Init_Reg[j].init_reg, NT99141_Init_Reg[j].init_val);

	/* Start apply the initial setting to sensor. */
#if 1
	for (j=0; j<i; j++)
	{
		if (NT99141_Init_Reg[j].op_code == NT99141_OP_CODE_END)	/* End of the setting. */
		{
			break ;
		}
		else if (NT99141_Init_Reg[j].op_code == NT99141_OP_CODE_DLY)
		{
			msleep(NT99141_Init_Reg[j].init_val);		/* Delay */
		}
		else if (NT99141_Init_Reg[j].op_code == NT99141_OP_CODE_REG)
		{

			NT99141_write_cmos_sensor((kal_uint8)NT99141_Init_Reg[j].init_reg, (kal_uint8)NT99141_Init_Reg[j].init_val);
		}
		else
		{
			printk("REG ERROR!\n");
		}
	}
#endif
	return 1;	
}

#endif
//auto lum
#define NT99141_NORMAL_Y0ffset  0x18 //0x10
#define NT99141_LOWLIGHT_Y0ffset  0x20

kal_bool   NT99141_MPEG4_encode_mode = KAL_FALSE;
kal_uint16 NT99141_dummy_pixels = 0, NT99141_dummy_lines = 0;
kal_bool   NT99141_MODE_CAPTURE = KAL_FALSE;
static kal_bool NT99141_gPVmode = KAL_TRUE; //PV size or Full size


static kal_int8 NT99141_DELAY_AFTER_PREVIEW = -1;

kal_bool   NT99141_CAM_BANDING_50HZ = KAL_TRUE;//lj_test KAL_FALSE;
kal_bool   NT99141_CAM_Nightmode = 0;
kal_bool	setshutter = KAL_FALSE;
kal_uint32 NT99141_isp_master_clock;
static kal_uint32 NT99141_g_fPV_PCLK = 24;

kal_uint8 NT99141_sensor_write_I2C_address = NT99141_WRITE_ID;
kal_uint8 NT99141_sensor_read_I2C_address = NT99141_READ_ID;

UINT8 NT99141PixelClockDivider=0;

MSDK_SENSOR_CONFIG_STRUCT NT99141SensorConfigData;

#define NT99141_SET_PAGE0 	NT99141_write_cmos_sensor(0xfd, 0x00)
#define NT99141_SET_PAGE1 	NT99141_write_cmos_sensor(0xfd, 0x01)

#define NT99141YUV_DEBUG
#ifdef NT99141YUV_DEBUG
#define SENSORDB(fmt, args...)  pr_err(PFX "%s: " fmt, __FUNCTION__ ,##args)
#else
#define SENSORDB(x,...)
#endif

#define WINDOW_SIZE_720P 	0
#define WINDOW_SIZE_SVGA 	1
#define WINDOW_SIZE_VGA	2

extern int iReadRegI2C(u8 *a_pSendData , u16 a_sizeSendData, u8 * a_pRecvData, u16 a_sizeRecvData, u16 i2cId);
extern int iWriteRegI2C(u8 *a_pSendData , u16 a_sizeSendData, u16 i2cId);


/* Sensor output window information */
static SENSOR_WINSIZE_INFO_STRUCT imgsensor_winsize_info[5]=
{{ 1280, 720,      0,    0, 1280, 720, 1280,  720,      0,   0, 1280,  720,  8,    6, 1264,  708},  // Preview
 { 1280, 720,      0,    0, 1280, 720, 1280,  720,      0,   0, 1280,  720,  8,    6, 1264,  708}, // capture
 { 1280, 720,      0,    0, 1280, 720, 1280,  720,      0,   0, 1280,  720,  8,    6, 1264,  708},  // video
 { 1280, 720,      0,    0, 1280, 720, 1280,  720,      0,   0, 1280,  720,  160,    240, 1280,  720}, //hight speed video
 { 1280, 720,      0,    0, 1280, 720, 1280,  720,      0,   0, 1280,  720,  160,    240, 1280,  720}}; // slim video


void NT99141_write_cmos_sensor(kal_uint16 addr, kal_uint8 para)
{
	char puSendCmd[4] = {(char)((addr >> 8) & 0xFF) ,(char)(addr & 0xFF), (char)(para & 0xFF)};

	kdSetI2CSpeed(400);
	iWriteRegI2C(puSendCmd , 3, NT99141_WRITE_ID);

}
kal_uint8 NT99141_read_cmos_sensor(kal_uint16 addr)
{
	kal_uint8 get_byte=0;
	char puSendCmd[2] = { (char)((addr >> 8) & 0xFF) ,(char)((addr  >> 0)& 0xFF) };

	kdSetI2CSpeed(400);
	iReadRegI2C(puSendCmd , 2, (u8*)&get_byte, 1, NT99141_WRITE_ID);
	
	return get_byte;
}

/*************************************************************************
 * FUNCTION
 *	NT99141_SetShutter
 *
 * DESCRIPTION
 *	This function set e-shutter of NT99141 to change exposure time.
 *
 * PARAMETERS
 *   iShutter : exposured lines
 *
 * RETURNS
 *	None
 *
 * GLOBALS AFFECTED
 *
 *************************************************************************/
void NT99141_Set_Shutter(kal_uint16 iShutter)
{
	SENSORDB("Ronlus NT99141_Set_Shutter\r\n");
} /* Set_NT99141_Shutter */


/*************************************************************************
 * FUNCTION
 *	NT99141_read_Shutter
 *
 * DESCRIPTION
 *	This function read e-shutter of NT99141 .
 *
 * PARAMETERS
 *  None
 *
 * RETURNS
 *	None
 *
 * GLOBALS AFFECTED
 *
 *************************************************************************/
kal_uint16 NT99141_Read_Shutter(void)
{
	//kal_uint8 temp_reg1, temp_reg2;
	kal_uint16 shutter = 0;

	//temp_reg1 = NT99141_read_cmos_sensor(0x04);
	//temp_reg2 = NT99141_read_cmos_sensor(0x03);

	//shutter = (temp_reg1 & 0xFF) | (temp_reg2 << 8);
	SENSORDB("Ronlus NT99141_Read_Shutter\r\n");
	return shutter;
} /* NT99141_read_shutter */

#if 0

/*************************************************************************
 * FUNCTION
 *	NT99141_ae_enable
 *
 * DESCRIPTION
 *	This function enable or disable the awb (Auto White Balance).
 *
 * PARAMETERS
 *	1. kal_bool : KAL_TRUE - enable awb, KAL_FALSE - disable awb.
 *
 * RETURNS
 *	kal_bool : It means set awb right or not.
 *
 *************************************************************************/

static void NT99141_ae_enable(kal_bool enable)
{	 
	kal_uint16 temp_AWB_reg = 0;
	SENSORDB("Ronlus NT99141_ae_enable\r\n");
	temp_AWB_reg = NT99141_read_cmos_sensor(0x32);//Ronlus 20120511
	if (enable == 1)
	{
		NT99141_write_cmos_sensor(0x32, (temp_AWB_reg |0x05));
	}
	else if(enable == 0)
	{
		NT99141_write_cmos_sensor(0x32, (temp_AWB_reg & (~0x05)));
	}
}

/*************************************************************************
 * FUNCTION
 *	NT99141_awb_enable
 *
 * DESCRIPTION
 *	This function enable or disable the awb (Auto White Balance).
 *
 * PARAMETERS
 *	1. kal_bool : KAL_TRUE - enable awb, KAL_FALSE - disable awb.
 *
 * RETURNS
 *	kal_bool : It means set awb right or not.
 *
 *************************************************************************/
static void NT99141_awb_enable(kal_bool enalbe)
{	 
	//kal_uint16 temp_AWB_reg = 0;
	SENSORDB("Ronlus NT99141_awb_enable\r\n");
}


/*************************************************************************
 * FUNCTION
 *	NT99141_set_hb_shutter
 *
 * DESCRIPTION
 *	This function set the dummy pixels(Horizontal Blanking) when capturing, it can be
 *	used to adjust the frame rate  for back-end process.
 *	
 *	IMPORTANT NOTICE: the base shutter need re-calculate for some sensor, or else flicker may occur.
 *
 * PARAMETERS
 *	1. kal_uint32 : Dummy Pixels (Horizontal Blanking)
 *	2. kal_uint32 : shutter (Vertical Blanking)
 *
 * RETURNS
 *	None
 *
 *************************************************************************/
static void NT99141_set_hb_shutter(kal_uint32 hb_add,  kal_uint32 shutter)
{
//	kal_uint32 hb_ori, hb_total;
//	kal_uint32 temp_reg, banding_step;
	SENSORDB("Ronlus NT99141_set_hb_shutter\r\n");
}    /* NT99141_set_dummy */
#endif

/*************************************************************************
 * FUNCTION
 *	NT99141_config_window
 *
 * DESCRIPTION
 *	This function config the hardware window of NT99141 for getting specified
 *  data of that window.
 *
 * PARAMETERS
 *	start_x : start column of the interested window
 *  start_y : start row of the interested window
 *  width  : column widht of the itnerested window
 *  height : row depth of the itnerested window
 *
 * RETURNS
 *	the data that read from NT99141
 *
 * GLOBALS AFFECTED
 *
 *************************************************************************/
//void NT99141_config_window(kal_uint16 startx, kal_uint16 starty, kal_uint16 width, kal_uint16 height)//Ronlus
void NT99141_config_window(kal_uint8 index)
{
	SENSORDB("Ronlus NT99141_config_window,index = %d\r\n",index);
	switch(index)
	{
		case WINDOW_SIZE_720P:  //720P@30
			SENSORDB("Ronlus NT99141_config_window,set WINDOW_SIZE_720P");
			NT99141_write_cmos_sensor(0x32BF, 0x60); 
			NT99141_write_cmos_sensor(0x32C0, 0x5B); 
			NT99141_write_cmos_sensor(0x32C1, 0x5B); 
			NT99141_write_cmos_sensor(0x32C2, 0x5B); 
			NT99141_write_cmos_sensor(0x32C3, 0x00); 
			NT99141_write_cmos_sensor(0x32C4, 0x27); 
			NT99141_write_cmos_sensor(0x32C5, 0x28); 
			NT99141_write_cmos_sensor(0x32C6, 0x28); 
			NT99141_write_cmos_sensor(0x32C7, 0x00); 
			NT99141_write_cmos_sensor(0x32C8, 0xD8); 
			NT99141_write_cmos_sensor(0x32C9, 0x5B); 
			NT99141_write_cmos_sensor(0x32CA, 0x83); 
			NT99141_write_cmos_sensor(0x32CB, 0x83); 
			NT99141_write_cmos_sensor(0x32CC, 0x83); 
			NT99141_write_cmos_sensor(0x32CD, 0x83); 
			NT99141_write_cmos_sensor(0x32DB, 0x7B); 
			NT99141_write_cmos_sensor(0x32E0, 0x05); 
			NT99141_write_cmos_sensor(0x32E1, 0x00); 
			NT99141_write_cmos_sensor(0x32E2, 0x02); 
			NT99141_write_cmos_sensor(0x32E3, 0xD0); 
			NT99141_write_cmos_sensor(0x32E4, 0x00); 
			NT99141_write_cmos_sensor(0x32E5, 0x00); 
			NT99141_write_cmos_sensor(0x32E6, 0x00); 
			NT99141_write_cmos_sensor(0x32E7, 0x00); 
			NT99141_write_cmos_sensor(0x3200, 0x3E); 
			//NT99141_write_cmos_sensor(0x3201, 0x0F); 
			NT99141_write_cmos_sensor(0x3028, 0x0B); 
			NT99141_write_cmos_sensor(0x3029, 0x00); 
			NT99141_write_cmos_sensor(0x302A, 0x04); 
			NT99141_write_cmos_sensor(0x3022, 0x24); 
			NT99141_write_cmos_sensor(0x3023, 0x24); 
			NT99141_write_cmos_sensor(0x3002, 0x00); 
			NT99141_write_cmos_sensor(0x3003, 0x04); 
			NT99141_write_cmos_sensor(0x3004, 0x00); 
			NT99141_write_cmos_sensor(0x3005, 0x04); 
			NT99141_write_cmos_sensor(0x3006, 0x05); 
			NT99141_write_cmos_sensor(0x3007, 0x03); 
			NT99141_write_cmos_sensor(0x3008, 0x02); 
			NT99141_write_cmos_sensor(0x3009, 0xD3); 
			NT99141_write_cmos_sensor(0x300A, 0x06); 
			NT99141_write_cmos_sensor(0x300B, 0x7F); 
			NT99141_write_cmos_sensor(0x300C, 0x02); 
			NT99141_write_cmos_sensor(0x300D, 0xE0); 
			NT99141_write_cmos_sensor(0x300E, 0x05); 
			NT99141_write_cmos_sensor(0x300F, 0x00); 
			NT99141_write_cmos_sensor(0x3010, 0x02); 
			NT99141_write_cmos_sensor(0x3011, 0xD0); 
			NT99141_write_cmos_sensor(0x32B8, 0x3F); 
			NT99141_write_cmos_sensor(0x32B9, 0x31); 
			NT99141_write_cmos_sensor(0x32BB, 0x87); 
			NT99141_write_cmos_sensor(0x32BC, 0x38); 
			NT99141_write_cmos_sensor(0x32BD, 0x3C); 
			NT99141_write_cmos_sensor(0x32BE, 0x34); 
			//NT99141_write_cmos_sensor(0x3201, 0x7F); 
			NT99141_write_cmos_sensor(0x3021, 0x06); 
			NT99141_write_cmos_sensor(0x320A, 0x00);
			NT99141_write_cmos_sensor(0x3060, 0x01); 
			/**1280*720  this is for crop para*/
			break;
		case WINDOW_SIZE_SVGA:
			/*SVGA this is for resize para*/							 			
			break;
		case WINDOW_SIZE_VGA:
			/*VGA this is for resize para*/							 
			break;
		default:
			break;
	}
} /* NT99141_config_window */



/*************************************************************************
 * FUNCTION
 *	NT99141_SetGain
 *
 * DESCRIPTION
 *	This function is to set global gain to sensor.
 *
 * PARAMETERS
 *   iGain : sensor global gain(base: 0x40)
 *
 * RETURNS
 *	the actually gain set to sensor.
 *
 * GLOBALS AFFECTED
 *
 *************************************************************************/
kal_uint16 NT99141_SetGain(kal_uint16 iGain)
{
	SENSORDB("Ronlus NT99141_SetGain\r\n");
	return iGain;
}


/*************************************************************************
 * FUNCTION
 *	NT99141_NightMode
 *
 * DESCRIPTION
 *	This function night mode of NT99141.
 *
 * PARAMETERS
 *	bEnable: KAL_TRUE -> enable night mode, otherwise, disable night mode
 *
 * RETURNS
 *	None
 *
 * GLOBALS AFFECTED
 *
 *************************************************************************/
void NT99141_night_mode(kal_bool bEnable)
{
} 

/*************************************************************************
 * FUNCTION
 *	NT99141_Write_More_Registers
 *
 * DESCRIPTION
 *	This function is served for FAE to modify the necessary Init Regs. Do not modify the regs
 *     in init_NT99141() directly.
 *
 * PARAMETERS
 *	None
 *
 * RETURNS
 *	None
 *
 * GLOBALS AFFECTED
 *
 *************************************************************************/
void NT99141_Write_More_Registers(void)
{
	//Ronlus this function is used for FAE debug
}

#if 0

/*************************************************************************
 * FUNCTION
 *	NT99141_PV_setting
 *
 * DESCRIPTION
 *	This function apply the preview mode setting, normal the preview size is 1/4 of full size.
 *	Ex. 2M (1600 x 1200)
 *	Preview: 800 x 600 (Use sub-sample or binning to acheive it)
 *	Full Size: 1600 x 1200 (Output every effective pixels.)
 *
 * PARAMETERS
 *	1. image_sensor_exposure_window_struct : Set the grab start x,y and width,height.
 *	2. image_sensor_config_struct : Current operation mode.
 *
 * RETURNS
 *	None
 *
 *************************************************************************/
static void NT99141_PV_setting(void)
{
	SENSORDB("Ronlus NT99141_PV_setting\r\n");

} /* NT99141_PV_setting */


/*************************************************************************
 * FUNCTION
 *	NT99141_CAP_setting
 *
 * DESCRIPTION
 *	This function apply the full size mode setting.
 *	Ex. 2M (1600 x 1200)
 *	Preview: 800 x 600 (Use sub-sample or binning to acheive it)
 *	Full Size: 1600 x 1200 (Output every effective pixels.)
 *
 * PARAMETERS
 *	1. image_sensor_exposure_window_struct : Set the grab start x,y and width,height.
 *	2. image_sensor_config_struct : Current operation mode.
 *
 * RETURNS
 *	None
 *
 *************************************************************************/
static void NT99141_CAP_setting(void)
{
	//kal_uint16 corr_r_offset,corr_g_offset,corr_b_offset,temp = 0;
	SENSORDB("Ronlus NT99141_CAP_setting\r\n");	
} /* NT99141_CAP_setting */

#endif




/*************************************************************************
 * FUNCTION
 *	NT99141_Sensor_Init
 *
 * DESCRIPTION
 *	This function apply all of the initial setting to sensor.
 *
 * PARAMETERS
 *	None
 *
 * RETURNS
 *	None
 *
 *************************************************************************/

void NT99141_Sensor_Init(void)
{
	NT99141_write_cmos_sensor(0x3024,0x08);
	NT99141_write_cmos_sensor(0x3040,0x04);
	NT99141_write_cmos_sensor(0x3041,0x02);
	NT99141_write_cmos_sensor(0x3042,0xFF);
	NT99141_write_cmos_sensor(0x3043,0x08);
	NT99141_write_cmos_sensor(0x3052,0xE0);
	NT99141_write_cmos_sensor(0x305F,0x33);
	NT99141_write_cmos_sensor(0x3069,0x01);
	NT99141_write_cmos_sensor(0x306A,0x02); 
	NT99141_write_cmos_sensor(0x3100,0x07);
	NT99141_write_cmos_sensor(0x3105,0x01);
	NT99141_write_cmos_sensor(0x3106,0x03);
	NT99141_write_cmos_sensor(0x3108,0x05);
	NT99141_write_cmos_sensor(0x3110,0x22);
	NT99141_write_cmos_sensor(0x3111,0x57);
	NT99141_write_cmos_sensor(0x3112,0x22);
	NT99141_write_cmos_sensor(0x3113,0x55);
	NT99141_write_cmos_sensor(0x3114,0x05);
	NT99141_write_cmos_sensor(0x3135,0x00);
	NT99141_write_cmos_sensor(0x32F0,0x01);
	NT99141_write_cmos_sensor(0x3290,0x01);
	NT99141_write_cmos_sensor(0x3291,0xA0);
	NT99141_write_cmos_sensor(0x3296,0x01);
	NT99141_write_cmos_sensor(0x3297,0x73);
#if 1 // for 3208B
	NT99141_write_cmos_sensor(0x3250,0xC8); 
	NT99141_write_cmos_sensor(0x3251,0x01);
	NT99141_write_cmos_sensor(0x3252,0x1B);
	NT99141_write_cmos_sensor(0x3253,0x85);
	NT99141_write_cmos_sensor(0x3254,0x00);
	NT99141_write_cmos_sensor(0x3255,0xC9);
	NT99141_write_cmos_sensor(0x3256,0x91);
	NT99141_write_cmos_sensor(0x3257,0x48);
	NT99141_write_cmos_sensor(0x3258,0x0A);
	NT99141_write_cmos_sensor(0x329B,0x31);
	NT99141_write_cmos_sensor(0x32A1,0x00);
	NT99141_write_cmos_sensor(0x32A2,0xFF);
	NT99141_write_cmos_sensor(0x32A3,0x01);
	NT99141_write_cmos_sensor(0x32A4,0xB0);
	NT99141_write_cmos_sensor(0x32A5,0x01);
	NT99141_write_cmos_sensor(0x32A6,0x50);
	NT99141_write_cmos_sensor(0x32A7,0x01);
	NT99141_write_cmos_sensor(0x32A8,0xE0);
	NT99141_write_cmos_sensor(0x32A9,0x11); // +R  -B
	NT99141_write_cmos_sensor(0x32AA,0x01); 
#else // for 3309A
	NT99141_write_cmos_sensor(0x3250,0xC0);
	NT99141_write_cmos_sensor(0x3251,0x00);
	NT99141_write_cmos_sensor(0x3252,0xDF);
	NT99141_write_cmos_sensor(0x3253,0x0E);
	NT99141_write_cmos_sensor(0x3254,0x00);
	NT99141_write_cmos_sensor(0x3255,0xEB);
	NT99141_write_cmos_sensor(0x3256,0x81);
	NT99141_write_cmos_sensor(0x3257,0x40);	
	NT99141_write_cmos_sensor(0x329B,0x31);
	NT99141_write_cmos_sensor(0x32A1,0x00);
	NT99141_write_cmos_sensor(0x32A2,0xA0);
	NT99141_write_cmos_sensor(0x32A3,0x01);
	NT99141_write_cmos_sensor(0x32A4,0xA0);
	NT99141_write_cmos_sensor(0x32A5,0x01);
	NT99141_write_cmos_sensor(0x32A6,0x18);
	NT99141_write_cmos_sensor(0x32A7,0x01);
	NT99141_write_cmos_sensor(0x32A8,0xE0);
#endif
	NT99141_write_cmos_sensor(0x3210,0x16);
	NT99141_write_cmos_sensor(0x3211,0x19);
	NT99141_write_cmos_sensor(0x3212,0x16);
	NT99141_write_cmos_sensor(0x3213,0x14);
	NT99141_write_cmos_sensor(0x3214,0x15);
	NT99141_write_cmos_sensor(0x3215,0x18);
	NT99141_write_cmos_sensor(0x3216,0x15);
	NT99141_write_cmos_sensor(0x3217,0x14);
	NT99141_write_cmos_sensor(0x3218,0x15);
	NT99141_write_cmos_sensor(0x3219,0x18);
	NT99141_write_cmos_sensor(0x321A,0x15);
	NT99141_write_cmos_sensor(0x321B,0x14);
	NT99141_write_cmos_sensor(0x321C,0x14);
	NT99141_write_cmos_sensor(0x321D,0x17);
	NT99141_write_cmos_sensor(0x321E,0x14);
	NT99141_write_cmos_sensor(0x321F,0x12);
	NT99141_write_cmos_sensor(0x3231,0x74);
	NT99141_write_cmos_sensor(0x3232,0xC4);
	NT99141_write_cmos_sensor(0x3326,0x02);
	NT99141_write_cmos_sensor(0x3327,0x0A);
	NT99141_write_cmos_sensor(0x3328,0x0A);
	NT99141_write_cmos_sensor(0x3329,0x06);
	NT99141_write_cmos_sensor(0x332A,0x06);
	NT99141_write_cmos_sensor(0x332B,0x1C);
	NT99141_write_cmos_sensor(0x332C,0x1C);
	NT99141_write_cmos_sensor(0x332D,0x00);
	NT99141_write_cmos_sensor(0x332E,0x1D);
	NT99141_write_cmos_sensor(0x332F,0x1F);
	NT99141_write_cmos_sensor(0x32F6,0x0F);
	NT99141_write_cmos_sensor(0x32F9,0x63);
	NT99141_write_cmos_sensor(0x32FA,0x36);
	NT99141_write_cmos_sensor(0x3325,0x4A);
	NT99141_write_cmos_sensor(0x3330,0x00);
	NT99141_write_cmos_sensor(0x3331,0x0A);
	NT99141_write_cmos_sensor(0x3332,0xFF);
	NT99141_write_cmos_sensor(0x3338,0x30);
	NT99141_write_cmos_sensor(0x3339,0x84);
	NT99141_write_cmos_sensor(0x333A,0x48);
	NT99141_write_cmos_sensor(0x333F,0x07);
	NT99141_write_cmos_sensor(0x3360,0x10);
	NT99141_write_cmos_sensor(0x3361,0x18);
	NT99141_write_cmos_sensor(0x3362,0x1F);
	NT99141_write_cmos_sensor(0x3363,0x37);
	NT99141_write_cmos_sensor(0x3364,0x84);
	NT99141_write_cmos_sensor(0x3365,0x80);
	NT99141_write_cmos_sensor(0x3366,0x70);
	NT99141_write_cmos_sensor(0x3367,0x64);
	NT99141_write_cmos_sensor(0x3368,0x30);
	NT99141_write_cmos_sensor(0x3369,0x28);
	NT99141_write_cmos_sensor(0x336A,0x20);
	NT99141_write_cmos_sensor(0x336B,0x10);
	NT99141_write_cmos_sensor(0x336C,0x00);
	NT99141_write_cmos_sensor(0x336D,0x20);
	NT99141_write_cmos_sensor(0x336E,0x1C);
	NT99141_write_cmos_sensor(0x336F,0x18);
	NT99141_write_cmos_sensor(0x3370,0x10);
	NT99141_write_cmos_sensor(0x3371,0x38);
	NT99141_write_cmos_sensor(0x3372,0x3C);
	NT99141_write_cmos_sensor(0x3373,0x3F);
	NT99141_write_cmos_sensor(0x3374,0x3F);
	NT99141_write_cmos_sensor(0x338A,0x34);
	NT99141_write_cmos_sensor(0x338B,0x7F);
	NT99141_write_cmos_sensor(0x338C,0x10);
	NT99141_write_cmos_sensor(0x338D,0x23);
	NT99141_write_cmos_sensor(0x338E,0x7F);
	NT99141_write_cmos_sensor(0x338F,0x14);
	NT99141_write_cmos_sensor(0x3375,0x0A);
	NT99141_write_cmos_sensor(0x3376,0x0C);
	NT99141_write_cmos_sensor(0x3377,0x10);
	NT99141_write_cmos_sensor(0x3378,0x14);

#if 1 // for 3208B
	NT99141_write_cmos_sensor(0x3270,0x08);//YW_TUNE
	NT99141_write_cmos_sensor(0x3271,0x16);
	NT99141_write_cmos_sensor(0x3272,0x22);
	NT99141_write_cmos_sensor(0x3273,0x37);
	NT99141_write_cmos_sensor(0x3274,0x49);
	NT99141_write_cmos_sensor(0x3275,0x5A);
	NT99141_write_cmos_sensor(0x3276,0x76);
	NT99141_write_cmos_sensor(0x3277,0x8B);
	NT99141_write_cmos_sensor(0x3278,0x9F);
	NT99141_write_cmos_sensor(0x3279,0xB0);
	NT99141_write_cmos_sensor(0x327A,0xCD);
	NT99141_write_cmos_sensor(0x327B,0xE2);
	NT99141_write_cmos_sensor(0x327C,0xEF);
	NT99141_write_cmos_sensor(0x327D,0xF9);
	NT99141_write_cmos_sensor(0x327E,0xFF);
	NT99141_write_cmos_sensor(0x3302,0x00); //CC CAL 160Tool
	NT99141_write_cmos_sensor(0x3303,0x4C);
	NT99141_write_cmos_sensor(0x3304,0x00);
	NT99141_write_cmos_sensor(0x3305,0x96);
	NT99141_write_cmos_sensor(0x3306,0x00);
	NT99141_write_cmos_sensor(0x3307,0x1D);
	NT99141_write_cmos_sensor(0x3308,0x07);
	NT99141_write_cmos_sensor(0x3309,0xBB);
	NT99141_write_cmos_sensor(0x330A,0x06);
	NT99141_write_cmos_sensor(0x330B,0xDC);
	NT99141_write_cmos_sensor(0x330C,0x01);
	NT99141_write_cmos_sensor(0x330D,0x6A);
	NT99141_write_cmos_sensor(0x330E,0x01);
	NT99141_write_cmos_sensor(0x330F,0x10);
	NT99141_write_cmos_sensor(0x3310,0x07);
	NT99141_write_cmos_sensor(0x3311,0x20);
	NT99141_write_cmos_sensor(0x3312,0x07);
	NT99141_write_cmos_sensor(0x3313,0xD0);
#else	// for 3309A
	NT99141_write_cmos_sensor(0x3270,0x10);
	NT99141_write_cmos_sensor(0x3271,0x1A);
	NT99141_write_cmos_sensor(0x3272,0x25);
	NT99141_write_cmos_sensor(0x3273,0x38);
	NT99141_write_cmos_sensor(0x3274,0x49);
	NT99141_write_cmos_sensor(0x3275,0x5A);
	NT99141_write_cmos_sensor(0x3276,0x71);
	NT99141_write_cmos_sensor(0x3277,0x87);
	NT99141_write_cmos_sensor(0x3278,0x99);
	NT99141_write_cmos_sensor(0x3279,0xA6);
	NT99141_write_cmos_sensor(0x327A,0xBC);
	NT99141_write_cmos_sensor(0x327B,0xCC);
	NT99141_write_cmos_sensor(0x327C,0xDD);
	NT99141_write_cmos_sensor(0x327D,0xEE);
	NT99141_write_cmos_sensor(0x327E,0xFF);
	NT99141_write_cmos_sensor(0x3302,0x00);
	NT99141_write_cmos_sensor(0x3303,0x40);
	NT99141_write_cmos_sensor(0x3304,0x00);
	NT99141_write_cmos_sensor(0x3305,0x96);
	NT99141_write_cmos_sensor(0x3306,0x00);
	NT99141_write_cmos_sensor(0x3307,0x29);
	NT99141_write_cmos_sensor(0x3308,0x07);
	NT99141_write_cmos_sensor(0x3309,0xBA);
	NT99141_write_cmos_sensor(0x330A,0x06);
	NT99141_write_cmos_sensor(0x330B,0xF5);
	NT99141_write_cmos_sensor(0x330C,0x01);
	NT99141_write_cmos_sensor(0x330D,0x51);
	NT99141_write_cmos_sensor(0x330E,0x01);
	NT99141_write_cmos_sensor(0x330F,0x30);
	NT99141_write_cmos_sensor(0x3310,0x07);
	NT99141_write_cmos_sensor(0x3311,0x16);
	NT99141_write_cmos_sensor(0x3312,0x07);
	NT99141_write_cmos_sensor(0x3313,0xBA);
#endif
	//CY Hsu 20131014 Start
	NT99141_write_cmos_sensor(0x3201,0x7F);
	NT99141_write_cmos_sensor(0x32B1,0x7D);
	NT99141_write_cmos_sensor(0x32B2,0x7D);
	NT99141_write_cmos_sensor(0x3257,0x60);
    //CY Hsu 20131014 End
} 

/*************************************************************************
 * FUNCTION
 *	NT99141Open
 *
 * DESCRIPTION
 *	This function initialize the registers of CMOS sensor
 *
 * PARAMETERS
 *	None
 *
 * RETURNS
 *	None
 *
 * GLOBALS AFFECTED
 *
 *************************************************************************/
UINT32 NT99141Open(void)
{
	kal_uint16 sensor_id=0;
	int i;
	printk(KERN_EMERG"NT99141 Open File Start\n");
	SENSORDB("@@@SP:Ronlus NT99141Open\r\n");
	// check if sensor ID correct
	for(i = 0; i < 3; i++)
	{
		sensor_id = NT99141_read_cmos_sensor(0x3000);
		sensor_id = (sensor_id << 8) | NT99141_read_cmos_sensor(0x3001);
		//SENSORDB("@@@SP:Ronlus NT99141 Sensor id = %x\n", sensor_id);

		if (sensor_id == NT99141_YUV_SENSOR_ID)
		{
			break;
		}
	}
	//mdelay(50);
	if(sensor_id != NT99141_YUV_SENSOR_ID)
	{
		SENSORDB("@@@SP:NT99141 Sensor id read failed, ID = %x\n", sensor_id);
		return ERROR_SENSOR_CONNECT_FAIL;
	}
	SENSORDB("@@@SP:NT99141 Sensor id read, ID = %x\n", sensor_id);

#ifdef DEBUG_SENSOR_NT99141  //gepeiwei   120903
	struct file *fp; 
	mm_segment_t fs; 
	loff_t pos = 0; 
	//static char buf[10*1024] ;

	printk("NT99141 Open File Start\n");
	fp = filp_open("/storage/sdcard1/nt99141_sd.dat", O_RDONLY , 0); 
	if (IS_ERR(fp)) 
	{ 
		printk("open file error 0\n");
		fp = filp_open("/storage/sdcard0/nt99141_sd.dat", O_RDONLY , 0); 
		if (IS_ERR(fp)) { 
			fromsd = 0;   
			printk("open file error 1\n");
		}
		else{
			printk("open file success 1\n");
			fromsd = 1;
			filp_close(fp, NULL); 
		}
	} 
	else 
	{
		printk("open file success 0\n");
		fromsd = 1;
		printk("open file ok\n");

		//NT99141_Initialize_from_T_Flash();


		filp_close(fp, NULL); 
	}

	if(fromsd == 1)//Ƿ\u017dSDȡ//gepeiwei   120903
	{
		printk("________________from t!\n");
		NT99141_Initialize_from_T_Flash();//\u017dSD\u0161ȡҪ
	}
	else
	{
		NT99141_Sensor_Init();
		NT99141_Write_More_Registers();//added for FAE to debug
	}
#else  
	//RETAILMSG(1, (TEXT("Sensor Read ID OK \r\n")));
	// initail sequence write in
	NT99141_Sensor_Init();
	NT99141_Write_More_Registers();//added for FAE to debut
#endif	
return ERROR_NONE;
} /* NT99141Open */


/*************************************************************************
 * FUNCTION
 *	NT99141Close
 *
 * DESCRIPTION
 *	This function is to turn off sensor module power.
 *
 * PARAMETERS
 *	None
 *
 * RETURNS
 *	None
 *
 * GLOBALS AFFECTED
 *
 *************************************************************************/
UINT32 NT99141Close(void)
{
	SENSORDB("@@@SP:NT99141Close\r\n");
	return ERROR_NONE;
} /* NT99141Close */

void NT99141_AfterSnapshot(void)
{

}  


void NT99141_BeforeSnapshot(void)
{
//	return 0;  
} 

/*************************************************************************
 * FUNCTION
 * NT99141Preview
 *
 * DESCRIPTION
 *	This function start the sensor preview.
 *
 * PARAMETERS
 *	*image_window : address pointer of pixel numbers in one period of HSYNC
 *  *sensor_config_data : address pointer of line numbers in one period of VSYNC
 *
 * RETURNS
 *	None
 *
 * GLOBALS AFFECTED
 *
 *************************************************************************/
UINT32 NT99141Preview(MSDK_SENSOR_EXPOSURE_WINDOW_STRUCT *image_window,
		MSDK_SENSOR_CONFIG_STRUCT *sensor_config_data)
{
	//kal_uint32 iTemp;
	//kal_uint16 iStartX = 0, iStartY = 1;
	printk("@@@SP:Ronlus NT99141Preview fun start\r\n");

    NT99141_MODE_CAPTURE = KAL_FALSE;
    NT99141_gPVmode = KAL_TRUE;

    #if PREVIEW_USE_VGA
        NT99141_config_window(WINDOW_SIZE_VGA);
    #elif PREVIEW_USE_SVGA
        NT99141_config_window(WINDOW_SIZE_SVGA);
    #elif PREVIEW_USE_720P
        NT99141_config_window(WINDOW_SIZE_720P);
    #else
        NT99141_config_window(WINDOW_SIZE_720P);
    #endif
    
    if (setshutter) 
    {
	    ///NT99141_AfterSnapshot(); 
    }
	setshutter = KAL_FALSE;
	if(sensor_config_data->SensorOperationMode == MSDK_SENSOR_OPERATION_MODE_VIDEO)		// MPEG4 Encode Mode
	{
		printk("@@@SP:Ronlus video preview\r\n");
		NT99141_MPEG4_encode_mode = KAL_TRUE;
	}
	else
	{
		printk("@@@SP:Ronlus capture preview\r\n");
		NT99141_MPEG4_encode_mode = KAL_FALSE;
	}
	//NT99141_config_window(WINDOW_SIZE_SVGA);//add zch test(use this for SVGA)
    image_window->GrabStartX= 1;
    image_window->GrabStartY= 1;
    image_window->ExposureWindowWidth = IMAGE_SENSOR_PV_WIDTH;//800-16; //1280-16; //1600-32 ; //800-16; 
    image_window->ExposureWindowHeight = IMAGE_SENSOR_PV_HEIGHT;//600-12;//720-12; //1200-24; //600-12;

    NT99141_DELAY_AFTER_PREVIEW = 1;
	// copy sensor_config_data
	memcpy(&NT99141SensorConfigData, sensor_config_data, sizeof(MSDK_SENSOR_CONFIG_STRUCT));//rotation
	printk("@@@SP:Ronlus NT99141Preview fun end\r\n");
	//Sleep(300);
	printk("@@@SP:555555\r\n");
	return ERROR_NONE;
} /* NT99141Preview */


/*************************************************************************
 * FUNCTION
 *	NT99141Capture
 *
 * DESCRIPTION
 *	This function setup the CMOS sensor in capture MY_OUTPUT mode
 *
 * PARAMETERS
 *
 * RETURNS
 *	None
 *
 * GLOBALS AFFECTED
 *
 *************************************************************************/
UINT32 NT99141Capture(MSDK_SENSOR_EXPOSURE_WINDOW_STRUCT *image_window,
		MSDK_SENSOR_CONFIG_STRUCT *sensor_config_data)

{
	SENSORDB("Ronlus NT99141Capture fun start\r\n");
	NT99141_MODE_CAPTURE=KAL_TRUE;

    // config: UXGA for capture
    NT99141_config_window(WINDOW_SIZE_720P);
	
     if ((image_window->ImageTargetWidth<=640-16)&&
        (image_window->ImageTargetHeight<=480-12))
		{    /* Less than PV Mode */
			image_window->GrabStartX = 0;
			image_window->GrabStartY = 1;
			image_window->ExposureWindowWidth= 640-16;
			image_window->ExposureWindowHeight = 480-12;
		}
     else

	{
//		kal_uint32 shutter, cap_dummy_pixels = 0; 
		if(!setshutter)
		{
			//// NT99141_BeforeSnapshot();
			image_window->GrabStartX = 1;//iStartX;
			image_window->GrabStartY = 1;//iStartY;
			image_window->ExposureWindowWidth = IMAGE_SENSOR_FULL_WIDTH; //16 ;
			image_window->ExposureWindowHeight = IMAGE_SENSOR_FULL_HEIGHT; //12;
		}
		else
		{
			image_window->GrabStartX = 1;
			image_window->GrabStartY = 1;
			image_window->ExposureWindowWidth = IMAGE_SENSOR_FULL_WIDTH; //16 ;
			image_window->ExposureWindowHeight = IMAGE_SENSOR_FULL_HEIGHT; //12;
		}

		setshutter = KAL_TRUE;
		
	}

	 NT99141_DELAY_AFTER_PREVIEW = 2;
	// copy sensor_config_data
	memcpy(&NT99141SensorConfigData, sensor_config_data, sizeof(MSDK_SENSOR_CONFIG_STRUCT));
	SENSORDB("Ronlus NT99141Capture fun end\r\n");
	return ERROR_NONE;
} /* NT99141_Capture() */




UINT32 NT99141GetResolution(MSDK_SENSOR_RESOLUTION_INFO_STRUCT *pSensorResolution)
{
	SENSORDB("Ronlus NT99141GetResolution\r\n");
    pSensorResolution->SensorFullWidth = IMAGE_SENSOR_FULL_WIDTH; //1600-96; //-16-32;
    pSensorResolution->SensorFullHeight = IMAGE_SENSOR_FULL_HEIGHT; //1200-72; //12-24;
    pSensorResolution->SensorPreviewWidth = IMAGE_SENSOR_PV_WIDTH; //1600-64; //800-16-8;//IMAGE_SENSOR_SVGA_WIDTH;(user this for SVGA)
    pSensorResolution->SensorPreviewHeight = IMAGE_SENSOR_PV_HEIGHT; //1200-48; //600-12-6;//IMAGE_SIMAGE_SENSOR_SVGA_HEIGHT;(use this for SVGA)
	pSensorResolution->SensorVideoWidth = IMAGE_SENSOR_FULL_WIDTH; //1600-64;  //800-16-8;
	pSensorResolution->SensorVideoHeight = IMAGE_SENSOR_FULL_HEIGHT; //1200-48; //600-12-6;
	
    pSensorResolution->SensorHighSpeedVideoWidth = IMAGE_SENSOR_PV_WIDTH;
    pSensorResolution->SensorHighSpeedVideoHeight = IMAGE_SENSOR_PV_HEIGHT;
    
    pSensorResolution->SensorSlimVideoWidth = IMAGE_SENSOR_PV_WIDTH;
    pSensorResolution->SensorSlimVideoHeight = IMAGE_SENSOR_PV_HEIGHT;  
	return ERROR_NONE;
} /* NT99141GetResolution() */ 


UINT32 NT99141GetInfo(MSDK_SCENARIO_ID_ENUM ScenarioId,
		MSDK_SENSOR_INFO_STRUCT *pSensorInfo,
		MSDK_SENSOR_CONFIG_STRUCT *pSensorConfigData)
{
	SENSORDB("@@@SP:Ronlus NT99141GetInfo\r\n");
    pSensorInfo->SensorPreviewResolutionX = IMAGE_SENSOR_PV_WIDTH; //1600-64; //800-16-8;//IMAGE_SENSOR_PV_WIDTH(use this for SVGA);
    pSensorInfo->SensorPreviewResolutionY = IMAGE_SENSOR_PV_HEIGHT; //1200-48; //600-12-6;//IMAGE_SENSOR_PV_HEIGHT(user this for SVGA);
    pSensorInfo->SensorFullResolutionX = IMAGE_SENSOR_FULL_WIDTH; //1600-96; //-16-32;
    pSensorInfo->SensorFullResolutionY = IMAGE_SENSOR_FULL_HEIGHT; //1200-72; //-12-24;

	pSensorInfo->SensorCameraPreviewFrameRate=30;  
	pSensorInfo->SensorVideoFrameRate=30;
	pSensorInfo->SensorStillCaptureFrameRate=10;
	pSensorInfo->SensorWebCamCaptureFrameRate=15; 
	pSensorInfo->SensorResetActiveHigh=TRUE;
	pSensorInfo->SensorResetDelayCount=1;
	pSensorInfo->SensorOutputDataFormat=SENSOR_OUTPUT_FORMAT_UYVY;
	pSensorInfo->SensorClockPolarity=SENSOR_CLOCK_POLARITY_LOW;
	pSensorInfo->SensorClockFallingPolarity=SENSOR_CLOCK_POLARITY_LOW;
	
	pSensorInfo->SensorHsyncPolarity =SENSOR_CLOCK_POLARITY_LOW;
	pSensorInfo->SensorVsyncPolarity = SENSOR_CLOCK_POLARITY_HIGH;
	pSensorInfo->SensorInterruptDelayLines = 1;
	pSensorInfo->SensroInterfaceType=SENSOR_INTERFACE_TYPE_PARALLEL;
  
	pSensorInfo->CaptureDelayFrame = 1;
	pSensorInfo->PreviewDelayFrame = 1;
	pSensorInfo->VideoDelayFrame = 4;
	pSensorInfo->SensorMasterClockSwitch = 0;
	pSensorInfo->SensorDrivingCurrent = ISP_DRIVING_8MA;///ISP_DRIVING_2MA; //modify by sp_yjp,20130813
	
	pSensorInfo->HighSpeedVideoDelayFrame = 4;
	pSensorInfo->SlimVideoDelayFrame = 4;
 	pSensorInfo->SensorModeNum = 5;


	switch (ScenarioId)
	{
		case MSDK_SCENARIO_ID_CAMERA_PREVIEW:
		case MSDK_SCENARIO_ID_VIDEO_PREVIEW:
			//case MSDK_SCENARIO_ID_VIDEO_CAPTURE_MPEG4:
			pSensorInfo->SensorClockFreq=24;
			pSensorInfo->SensorClockDividCount=	3;
			pSensorInfo->SensorClockRisingCount= 0;
			pSensorInfo->SensorClockFallingCount= 2;
			pSensorInfo->SensorPixelClockCount= 3;
			pSensorInfo->SensorDataLatchCount= 2;
        	pSensorInfo->SensorGrabStartX = 1;
        	pSensorInfo->SensorGrabStartY = 1;

			break;
		case MSDK_SCENARIO_ID_CAMERA_CAPTURE_JPEG:
			//  case MSDK_SCENARIO_ID_CAMERA_CAPTURE_MEM:
			pSensorInfo->SensorClockFreq=24;
			pSensorInfo->SensorClockDividCount= 3;
			pSensorInfo->SensorClockRisingCount=0;
			pSensorInfo->SensorClockFallingCount=2;
			pSensorInfo->SensorPixelClockCount=3;
			pSensorInfo->SensorDataLatchCount=2;
        	pSensorInfo->SensorGrabStartX = 1;
        	pSensorInfo->SensorGrabStartY = 1;
			break;
		default:
			pSensorInfo->SensorClockFreq=24;
			pSensorInfo->SensorClockDividCount= 3;
			pSensorInfo->SensorClockRisingCount=0;
			pSensorInfo->SensorClockFallingCount=2;
			pSensorInfo->SensorPixelClockCount=3;
			pSensorInfo->SensorDataLatchCount=2;
        	pSensorInfo->SensorGrabStartX = 1;
        	pSensorInfo->SensorGrabStartY = 1;
			break;
	}
	NT99141PixelClockDivider=pSensorInfo->SensorPixelClockCount;
	memcpy(pSensorConfigData, &NT99141SensorConfigData, sizeof(MSDK_SENSOR_CONFIG_STRUCT));
	return ERROR_NONE;
} /* NT99141GetInfo() */


UINT32 NT99141Control(MSDK_SCENARIO_ID_ENUM ScenarioId, MSDK_SENSOR_EXPOSURE_WINDOW_STRUCT *pImageWindow,
		MSDK_SENSOR_CONFIG_STRUCT *pSensorConfigData)
{
	//pr_debug("@@@SP:Ronlus NT99141Control\r\n");
	
	switch (ScenarioId)
	{
		case MSDK_SCENARIO_ID_CAMERA_PREVIEW:
		case MSDK_SCENARIO_ID_VIDEO_PREVIEW:
			//case MSDK_SCENARIO_ID_VIDEO_CAPTURE_MPEG4:
			NT99141Preview(pImageWindow, pSensorConfigData);
			break;
		case MSDK_SCENARIO_ID_CAMERA_CAPTURE_JPEG:
			//case MSDK_SCENARIO_ID_CAMERA_CAPTURE_MEM:
			NT99141Capture(pImageWindow, pSensorConfigData);
			break;
		default:
            SENSORDB("Error ScenarioId setting");
            NT99141Preview(pImageWindow, pSensorConfigData);
            return ERROR_INVALID_SCENARIO_ID;
	}


	return TRUE;
}	/* NT99141Control() */

BOOL NT99141_set_param_wb(UINT16 para)
{
	SENSORDB("Ronlus NT99141_set_param_wb para = %d\r\n",para);
	switch (para)
	{
		case AWB_MODE_OFF:
			NT99141_write_cmos_sensor(0x3201,0x6F);//bit4
			break; 					
		case AWB_MODE_AUTO:
			NT99141_write_cmos_sensor(0x3201,0x7F);
			break;
		case AWB_MODE_CLOUDY_DAYLIGHT: //cloudy
			NT99141_write_cmos_sensor(0x3290,0x01);	// 337 / 256 = 1.316								 
			NT99141_write_cmos_sensor(0x3291,0x51);													   
			NT99141_write_cmos_sensor(0x3296,0x01);	// 256 / 256 = 1.0									 
			NT99141_write_cmos_sensor(0x3297,0x00);													   
			NT99141_write_cmos_sensor(0x3060,0x01);

			break;
		case AWB_MODE_DAYLIGHT: //sunny
			NT99141_write_cmos_sensor(0x3290,0x01);	// 312 / 256 = 1.218								 
			NT99141_write_cmos_sensor(0x3291,0x67);													   
			NT99141_write_cmos_sensor(0x3296,0x01);	// 360 / 256 = 1.406								 
			NT99141_write_cmos_sensor(0x3297,0x3d);													   
			NT99141_write_cmos_sensor(0x3060,0x01); 

			break;
		case AWB_MODE_INCANDESCENT: //office
			NT99141_write_cmos_sensor(0x3290,0x00);	// 304 / 256 = 1.187								 
			NT99141_write_cmos_sensor(0x3291,0xe4);													   
			NT99141_write_cmos_sensor(0x3296,0x01);	// 459 / 256 = 1.793								 
			NT99141_write_cmos_sensor(0x3297,0xe0);													   
			NT99141_write_cmos_sensor(0x3060,0x01);

			break;
		case AWB_MODE_TUNGSTEN: //home
			NT99141_write_cmos_sensor(0x3290,0x00);	// 256 / 256 = 1.0									 
			NT99141_write_cmos_sensor(0x3291,0xe6);													   
			NT99141_write_cmos_sensor(0x3296,0x02);	// 560 / 256 = 2.1875								 
			NT99141_write_cmos_sensor(0x3297,0xe0);													   
			NT99141_write_cmos_sensor(0x3060,0x01);

			break;
		case AWB_MODE_FLUORESCENT:
			NT99141_write_cmos_sensor(0x3290,0x01);	// 368 / 256 = 1.437								 
			NT99141_write_cmos_sensor(0x3291,0x1d);													   
			NT99141_write_cmos_sensor(0x3296,0x01);	// 511 / 256 = 1.996								 
			NT99141_write_cmos_sensor(0x3297,0xb4);													   
			NT99141_write_cmos_sensor(0x3060,0x01); 

		default:
			return FALSE;
	}  
	return TRUE;
} /* NT99141_set_param_wb */


BOOL NT99141_set_param_effect(UINT16 para)
{
	SENSORDB("Ronlus NT99141_set_param_effect\r\n");
	switch (para)
	{
		case MEFFECT_OFF:

			break;
		case MEFFECT_SEPIA:

			break;
		case MEFFECT_NEGATIVE://----datasheet

			break;
		case MEFFECT_SEPIAGREEN://----datasheet aqua

			break;
		case MEFFECT_SEPIABLUE:

			break;
		case MEFFECT_MONO: //----datasheet black & white

			break;
		default:
			return FALSE;
	}
	return TRUE;
} /* NT99141_set_param_effect */

//UINT8 index = 1;
BOOL NT99141_set_param_banding(UINT16 para)
{
	//UINT16 buffer = 0;
	UINT8 index = 1;
	SENSORDB("Ronlus NT99141_set_param_banding para = %d ---- index = %d\r\n",para,index); 
	SENSORDB("Ronlus NT99141_set_param_banding ---- NT99141_MPEG4_encode_mode = %d\r\n",NT99141_MPEG4_encode_mode);
	switch (para)
	{
		case AE_FLICKER_MODE_50HZ:
			NT99141_write_cmos_sensor(0x32BF,0x60); 
			NT99141_write_cmos_sensor(0x32C0,0x5B); 
			NT99141_write_cmos_sensor(0x32C1,0x5B); 
			NT99141_write_cmos_sensor(0x32C2,0x5B); 
			NT99141_write_cmos_sensor(0x32C3,0x00); 
			NT99141_write_cmos_sensor(0x32C4,0x27); 
			NT99141_write_cmos_sensor(0x32C5,0x28); 
			NT99141_write_cmos_sensor(0x32C6,0x28); 
			NT99141_write_cmos_sensor(0x32C7,0x00); 
			NT99141_write_cmos_sensor(0x32C8,0xD8); 
			NT99141_write_cmos_sensor(0x32C9,0x5B); 
			NT99141_write_cmos_sensor(0x32CA,0x83); 
			NT99141_write_cmos_sensor(0x32CB,0x83); 
			NT99141_write_cmos_sensor(0x32CC,0x83); 
			NT99141_write_cmos_sensor(0x32CD,0x83); 
			NT99141_write_cmos_sensor(0x32DB,0x7B); 
			NT99141_CAM_BANDING_50HZ = KAL_TRUE;
			break;
		case AE_FLICKER_MODE_60HZ:
			NT99141_write_cmos_sensor(0x32BF,0x60); 
			NT99141_write_cmos_sensor(0x32C0,0x60); 
			NT99141_write_cmos_sensor(0x32C1,0x60); 
			NT99141_write_cmos_sensor(0x32C2,0x60); 
			NT99141_write_cmos_sensor(0x32C3,0x00); 
			NT99141_write_cmos_sensor(0x32C4,0x27); 
			NT99141_write_cmos_sensor(0x32C5,0x28); 
			NT99141_write_cmos_sensor(0x32C6,0x28); 
			NT99141_write_cmos_sensor(0x32C7,0x00); 
			NT99141_write_cmos_sensor(0x32C8,0xB4); 
			NT99141_write_cmos_sensor(0x32C9,0x60); 
			NT99141_write_cmos_sensor(0x32CA,0x88); 
			NT99141_write_cmos_sensor(0x32CB,0x88); 
			NT99141_write_cmos_sensor(0x32CC,0x88); 
			NT99141_write_cmos_sensor(0x32CD,0x88); 
			NT99141_write_cmos_sensor(0x32DB,0x76);
			NT99141_CAM_BANDING_50HZ = KAL_FALSE;
			break;
		default:
			//NT99141_write_cmos_sensor(0xfd,0x00);//ronlus test
			//buffer = NT99141_read_cmos_sensor(0x35);
			return FALSE;
	}
#if 0/*Superpix Ronlus some vertical line when switching*/
	NT99141_config_window(WINDOW_SIZE_SVGA);
	NT99141_write_cmos_sensor(0xfd,0x00);
	NT99141_write_cmos_sensor(0x35,0x00);
#endif
	return TRUE;
} /* NT99141_set_param_banding */


BOOL NT99141_set_param_exposure(UINT16 para)
{
	SENSORDB("Ronlus NT99141_set_param_exposure\r\n");
	switch (para)
	{
		case AE_EV_COMP_n30:              /* EV -2 */

			break;
		case AE_EV_COMP_n20:              /* EV -1.5 */

			break;
		case AE_EV_COMP_n10:              /* EV -1 */

			break;
		case AE_EV_COMP_n03:              
			break;  
		
		case AE_EV_COMP_00:                /* EV 0 */

			break;
		case AE_EV_COMP_10:              /* EV +0.5 */

			break;
		case AE_EV_COMP_20:              /* EV +1 */

			break;
		case AE_EV_COMP_30:              /* EV +1.5 */

			break;
		case AE_EV_COMP_13:             
			break;  
		default:
			return FALSE;
	}
	return TRUE;

} /* NT99141_set_param_exposure */


UINT32 NT99141YUVSensorSetting(FEATURE_ID iCmd, UINT16 iPara)
{

#ifdef DEBUG_SENSOR_NT99141
	return TRUE;
#endif
	//SENSORDB("Ronlus NT99141YUVSensorSetting\r\n");
	switch (iCmd) 
	{
		case FID_SCENE_MODE:	
			#if 0 //del by sp_yjp for test,20130822
			if (iPara == SCENE_MODE_OFF)
			{
				NT99141_night_mode(0); 
			}
			else if (iPara == SCENE_MODE_NIGHTSCENE)
			{
				NT99141_night_mode(1); 
			}
			#endif
			break; 	    
		case FID_AWB_MODE:
			NT99141_set_param_wb(iPara);
			break;
		case FID_COLOR_EFFECT:
			NT99141_set_param_effect(iPara);
			break;
		case FID_AE_EV:
			NT99141_set_param_exposure(iPara);
			break;
		case FID_AE_FLICKER:
			NT99141_set_param_banding(iPara);
			NT99141_night_mode(NT99141_CAM_Nightmode); 
			break;
		default:
			break;
	}
	return TRUE;
} /* NT99141YUVSensorSetting */

UINT32 nt99141_get_sensor_id(UINT32 *sensorID) 
{
	volatile signed char i;
	kal_uint16 sensor_id=0;
	printk("@@@SP:xieyang NT99141GetSensorID ");
	///SENSORDB("@@@SP:xieyang in GPIO_CAMERA_CMPDN_PIN=%d,GPIO_CAMERA_CMPDN1_PIN=%d"
	///		, mt_get_gpio_out(GPIO_CAMERA_CMPDN_PIN),mt_get_gpio_out(GPIO_CAMERA_CMPDN1_PIN));
	for(i=0;i<10;i++)
	{
		sensor_id = NT99141_read_cmos_sensor(0x3000);
		sensor_id = (sensor_id << 8) | NT99141_read_cmos_sensor(0x3001);
		if(sensor_id == NT99141_YUV_SENSOR_ID)
			break;
	}
	printk("@@@SP:NT99141GetSensorID=%#x\r\n ",sensor_id);

	while(0)	//add by sp_yjp for test,20130813
	{
		mdelay(50);	
		sensor_id = NT99141_read_cmos_sensor(0x3000);
		printk("@@@SP:DeadReadNT99141ID=%#x\r\n ",sensor_id);
		if(sensor_id == NT99141_YUV_SENSOR_ID)
			break;
	}
	
	if(sensor_id != NT99141_YUV_SENSOR_ID)
	{
		printk("@@@SP:>>>>>>>>>>NT99141 read ID error<<<<<<<<<<<");			
		*sensorID = 0xFFFFFFFF;
		return ERROR_SENSOR_CONNECT_FAIL;
	}
	
	*sensorID = sensor_id;
	return ERROR_NONE;
	
}


UINT32 NT99141_SetTestPatternMode(kal_bool bEnable)
{
 //   kal_uint16 temp;

    SENSORDB("bEnable: %d", bEnable);

    if(bEnable) 
	{
	}
	else        
	{
	}
    return ERROR_NONE;
}

//#define NT99141_TEST_PATTERN_CHECKSUM (0x4d1d696d)
//#define NT99141_TEST_PATTERN_CHECKSUM (0x96f31f11)
#define NT99141_TEST_PATTERN_CHECKSUM (0x3f4b965a)


UINT32 NT99141FeatureControl(MSDK_SENSOR_FEATURE_ENUM FeatureId,
		UINT8 *pFeaturePara,UINT32 *pFeatureParaLen)
{
	UINT16 *pFeatureReturnPara16=(UINT16 *) pFeaturePara;
//	UINT16 *pFeatureData16=(UINT16 *) pFeaturePara;
	UINT32 *pFeatureReturnPara32=(UINT32 *) pFeaturePara;
	UINT32 *pFeatureData32=(UINT32 *) pFeaturePara;
	
    unsigned long long *pFeatureData=(unsigned long long *) pFeaturePara;
//  unsigned long long *pFeatureReturnPara=(unsigned long long *) pFeaturePara;
//	UINT32 NT99141SensorRegNumber;
//	UINT32 i;

	SENSOR_WINSIZE_INFO_STRUCT *wininfo;
	
	
	MSDK_SENSOR_CONFIG_STRUCT *pSensorConfigData=(MSDK_SENSOR_CONFIG_STRUCT *) pFeaturePara;
	MSDK_SENSOR_REG_INFO_STRUCT *pSensorRegData=(MSDK_SENSOR_REG_INFO_STRUCT *) pFeaturePara;

	//pr_debug("@@@SP:Ronlus NT99141FeatureControl.---FeatureId = %d\r\n",FeatureId);
	RETAILMSG(1, (_T("gaiyang NT99141FeatureControl FeatureId=%d\r\n"), FeatureId));

	switch (FeatureId)
	{
		case SENSOR_FEATURE_GET_RESOLUTION:
			*pFeatureReturnPara16++=N720P_PERIOD_PIXEL_NUMS;
			*pFeatureReturnPara16=N720P_PERIOD_LINE_NUMS;
			*pFeatureParaLen=4;
			break;
		case SENSOR_FEATURE_GET_PERIOD:
			*pFeatureReturnPara16++=(SVGA_PERIOD_PIXEL_NUMS)+NT99141_dummy_pixels;
			*pFeatureReturnPara16=(SVGA_PERIOD_LINE_NUMS)+NT99141_dummy_lines;
			*pFeatureParaLen=4;
			break;
		case SENSOR_FEATURE_GET_PIXEL_CLOCK_FREQ:
			*pFeatureReturnPara32 = NT99141_g_fPV_PCLK;
			*pFeatureParaLen=4;
			break;
		case SENSOR_FEATURE_SET_ESHUTTER:
			break;
		case SENSOR_FEATURE_SET_NIGHTMODE:
#ifndef DEBUG_SENSOR_NT99141		
			NT99141_night_mode((BOOL) *pFeatureData);
#endif
			break;
		case SENSOR_FEATURE_SET_GAIN:
		case SENSOR_FEATURE_SET_FLASHLIGHT:
			break;
		case SENSOR_FEATURE_SET_ISP_MASTER_CLOCK_FREQ:
			NT99141_isp_master_clock=*pFeatureData32;
			break;
		case SENSOR_FEATURE_SET_REGISTER:
			NT99141_write_cmos_sensor(pSensorRegData->RegAddr, pSensorRegData->RegData);
			break;
		case SENSOR_FEATURE_GET_REGISTER:
			pSensorRegData->RegData = NT99141_read_cmos_sensor(pSensorRegData->RegAddr);
			break;
		case SENSOR_FEATURE_GET_CONFIG_PARA:
			memcpy(pSensorConfigData, &NT99141SensorConfigData, sizeof(MSDK_SENSOR_CONFIG_STRUCT));
			*pFeatureParaLen=sizeof(MSDK_SENSOR_CONFIG_STRUCT);
			break;
		case SENSOR_FEATURE_SET_CCT_REGISTER:
		case SENSOR_FEATURE_GET_CCT_REGISTER:
		case SENSOR_FEATURE_SET_ENG_REGISTER:
		case SENSOR_FEATURE_GET_ENG_REGISTER:
		case SENSOR_FEATURE_GET_REGISTER_DEFAULT:
		case SENSOR_FEATURE_CAMERA_PARA_TO_SENSOR:
		case SENSOR_FEATURE_SENSOR_TO_CAMERA_PARA:
		case SENSOR_FEATURE_GET_GROUP_COUNT:
		case SENSOR_FEATURE_GET_GROUP_INFO:
		case SENSOR_FEATURE_GET_ITEM_INFO:
		case SENSOR_FEATURE_SET_ITEM_INFO:
		case SENSOR_FEATURE_GET_ENG_INFO:
			break;
		case SENSOR_FEATURE_CHECK_SENSOR_ID:
			nt99141_get_sensor_id(pFeatureReturnPara32); 
			break; 
		case SENSOR_FEATURE_GET_LENS_DRIVER_ID:
			// get the lens driver ID from EEPROM or just return LENS_DRIVER_ID_DO_NOT_CARE
			// if EEPROM does not exist in camera module.
			*pFeatureReturnPara32=LENS_DRIVER_ID_DO_NOT_CARE;
			*pFeatureParaLen=4;
			break;
		case SENSOR_FEATURE_SET_YUV_CMD:
			NT99141YUVSensorSetting((FEATURE_ID)*pFeatureData, *(pFeatureData+1));//test ,del by sp_yjp,20130813
			break;
		case SENSOR_FEATURE_SET_TEST_PATTERN:
        	NT99141_SetTestPatternMode((BOOL)*pFeatureData);
       		break;
    	case SENSOR_FEATURE_GET_TEST_PATTERN_CHECKSUM_VALUE://for factory mode auto testing             
        	*pFeatureReturnPara32= NT99141_TEST_PATTERN_CHECKSUM;
        	*pFeatureParaLen=4; 
			break;
		case SENSOR_FEATURE_GET_CROP_INFO:
            //LOG_INF("SENSOR_FEATURE_GET_CROP_INFO scenarioId:%d\n", *pFeatureData);
            // For pointer address in user space is 32-bit, need use compat_ptr to convert it to 64-bit in kernel space    

            wininfo = (SENSOR_WINSIZE_INFO_STRUCT *)(uintptr_t)(*(pFeatureData+1));

            switch (*pFeatureData32) {
                case MSDK_SCENARIO_ID_CAMERA_CAPTURE_JPEG:
                    memcpy((void *)wininfo,(void *)&imgsensor_winsize_info[1],sizeof(SENSOR_WINSIZE_INFO_STRUCT));
                    break;
                case MSDK_SCENARIO_ID_VIDEO_PREVIEW:
                    memcpy((void *)wininfo,(void *)&imgsensor_winsize_info[2],sizeof(SENSOR_WINSIZE_INFO_STRUCT));
                    break;
                case MSDK_SCENARIO_ID_HIGH_SPEED_VIDEO:
                    memcpy((void *)wininfo,(void *)&imgsensor_winsize_info[3],sizeof(SENSOR_WINSIZE_INFO_STRUCT));
                    break;
                case MSDK_SCENARIO_ID_SLIM_VIDEO:
                    memcpy((void *)wininfo,(void *)&imgsensor_winsize_info[4],sizeof(SENSOR_WINSIZE_INFO_STRUCT));
                    break;
                case MSDK_SCENARIO_ID_CAMERA_PREVIEW:
                default:
                    memcpy((void *)wininfo,(void *)&imgsensor_winsize_info[0],sizeof(SENSOR_WINSIZE_INFO_STRUCT));
                    break;
            }
			break;
			
		default:
			break;
	}
	return ERROR_NONE;
}	/* NT99141FeatureControl() */


SENSOR_FUNCTION_STRUCT	SensorFuncNT99141YUV=
{
	NT99141Open,
	NT99141GetInfo,
	NT99141GetResolution,
	NT99141FeatureControl,
	NT99141Control,
	NT99141Close
};


UINT32 NT99141_YUV_SensorInit(PSENSOR_FUNCTION_STRUCT *pfFunc)
{
	/* To Do : Check Sensor status here */	
	if (pfFunc!=NULL)
	{
		SENSORDB("@@@SP:NT99141_YUV_SensorInit\r\n");
		*pfFunc=&SensorFuncNT99141YUV;
	}
	return ERROR_NONE;
} /* SensorInit() */
