#include <linux/string.h>
#include <linux/gpio.h>
#include <mt-plat/mt_gpio.h>
#include "lcm_drv.h"
#include <linux/leds.h>

enum mt65xx_led_type {
	MT65XX_LED_TYPE_RED = 0,
	MT65XX_LED_TYPE_GREEN,
	MT65XX_LED_TYPE_BLUE,
	MT65XX_LED_TYPE_JOGBALL,
	MT65XX_LED_TYPE_KEYBOARD,
	MT65XX_LED_TYPE_BUTTON,
	MT65XX_LED_TYPE_LCD,
	MT65XX_LED_TYPE_TOTAL,
};
extern int mt65xx_leds_brightness_set(enum mt65xx_led_type type, enum led_brightness level);


// ---------------------------------------------------------------------------
//  Local Constants
// ---------------------------------------------------------------------------

#define FRAME_HEIGHT (600)
#define FRAME_WIDTH  (1024)


// ---------------------------------------------------------------------------
//  Local Variables
// ---------------------------------------------------------------------------

static LCM_UTIL_FUNCS lcm_util = {0};

#define SET_RESET_PIN(v)    (lcm_util.set_reset_pin((v)))

#define UDELAY(n) (lcm_util.udelay(n))
#define MDELAY(n) (lcm_util.mdelay(n))


// ---------------------------------------------------------------------------
//  LCM Driver Implementations
// ---------------------------------------------------------------------------

static void lcm_set_util_funcs(const LCM_UTIL_FUNCS *util)
{
	memcpy(&lcm_util, util, sizeof(LCM_UTIL_FUNCS));
}


static void lcm_get_params(LCM_PARAMS *params)
{
	memset(params, 0, sizeof(LCM_PARAMS));

	params->type   = LCM_TYPE_DPI;
	params->width  = FRAME_WIDTH;
	params->height = FRAME_HEIGHT;

	params->dpi.width = FRAME_WIDTH;
	params->dpi.height = FRAME_HEIGHT;

	params->dpi.PLL_CLOCK = 52;

	params->dpi.clk_pol           = LCM_POLARITY_RISING;
	params->dpi.de_pol            = LCM_POLARITY_RISING;
	params->dpi.vsync_pol         = LCM_POLARITY_RISING;
	params->dpi.hsync_pol         = LCM_POLARITY_RISING;

	params->dpi.hsync_pulse_width = 128;
	params->dpi.hsync_back_porch  = 160;
	params->dpi.hsync_front_porch = 160;
	params->dpi.vsync_pulse_width = 3;
	params->dpi.vsync_back_porch  = 23;
	params->dpi.vsync_front_porch = 12;

	params->dpi.lvds_tx_en = 1;
	params->dpi.format = LCM_DPI_FORMAT_RGB888;
	params->dpi.rgb_order = LCM_COLOR_ORDER_RGB;

}


static void lcm_init(void)
{
	mt65xx_leds_brightness_set (6, LED_HALF);
}


static void lcm_suspend(void)
{

	mt65xx_leds_brightness_set (6, 0);
	MDELAY(60); // avoid LCD transient
}


static void lcm_resume(void)
{
	SET_RESET_PIN(1);

	mt65xx_leds_brightness_set (6, LED_HALF);
	MDELAY(120); // avoid LCD resume transient
}


LCM_DRIVER lvds_wsvga_lcm_drv =
{
	.name			= "lvds_wsvga",
	.set_util_funcs = lcm_set_util_funcs,
	.get_params     = lcm_get_params,
	.init           = lcm_init,
	.suspend        = lcm_suspend,
	.resume         = lcm_resume,
};
