#include <linux/init.h>
#include <linux/module.h>
#include <linux/ioctl.h>
#include <linux/fs.h>
#include <linux/device.h>
#include <linux/err.h>
#include <linux/mutex.h>
#include <linux/compat.h>
#include <linux/delay.h>
#include <linux/gpio.h>
#include <asm/uaccess.h>
#include <linux/errno.h>
#include <linux/spi/spi.h>
#include <linux/gpio.h>
#include <linux/io.h>
#include <linux/cdev.h>
#include <asm/atomic.h>
#include <linux/poll.h>
#include <linux/interrupt.h>
#include <linux/workqueue.h>
#include <linux/of.h>
#include <linux/of_gpio.h>
#include <linux/proc_fs.h>
#include <linux/of_platform.h>
#include <linux/platform_device.h>
#include <linux/platform_data/spi-mt65xx.h>

struct gs_codec_data {
	dev_t			devt;
	spinlock_t		spi_lock;
	struct cdev 		c_dev;
	struct spi_device	*spidev;
	struct mutex		buf_lock;
	unsigned int 		mode;
	unsigned int 		speed;
	unsigned int 		mlsb;
	unsigned int  		rxlen;
	unsigned int 		gpio;
	u8			*spi_buffer;
	struct completion   	completed;
	struct work_struct  	fasync_wq;
	struct fasync_struct 	*fasync_q;
	struct class		*dev_class;
	struct device 		*dev;

	/*for debug*/
	struct proc_dir_entry *dir;
	struct proc_dir_entry *debug;
};
struct mtk_chip_config gs_spi_conf = {
		.tx_mlsb = 1,
		.rx_mlsb = 1,
};

static atomic_t codec_usage_cnt;
struct gs_codec_data *gs_codec;
#define SPI_SPEED	5*1000*1000
#define SPI_BUF_LEN 4*1024						
/*---------------------------spi transfer api----------------------------------------------*/
static void gs_codec_complete(void *arg)
{
	complete(arg);
}
//spi_write_then_read


static ssize_t
gs_codec_sync(struct gs_codec_data *gs_codec, struct spi_message *message)
{
	DECLARE_COMPLETION_ONSTACK(done);
	int status;

	message->complete = gs_codec_complete;
	message->context = &done;

	spin_lock_irq(&gs_codec->spi_lock);
	if (gs_codec->spidev == NULL)
		status = -ESHUTDOWN;
	else
		status = spi_async(gs_codec->spidev, message);
	spin_unlock_irq(&gs_codec->spi_lock);

	if (status == 0) {
		wait_for_completion(&done);
		status = message->status;
		if (status == 0)
			status = message->actual_length;
	}
	
	return status;
}

static inline ssize_t
gs_codec_sync_write(struct gs_codec_data *gs_codec, size_t len)
{
	struct spi_transfer	t = {
			.tx_buf		= gs_codec->spi_buffer,
			.len		= len,
		};
	struct spi_message	m;

	spi_message_init(&m);
	spi_message_add_tail(&t, &m);
	return gs_codec_sync(gs_codec, &m);
}

static inline ssize_t
gs_codec_sync_read(struct gs_codec_data *gs_codec, size_t len)
{
	struct spi_transfer	t = {
			.rx_buf		= gs_codec->spi_buffer,
			.len		= len,
		};
	struct spi_message	m;

	spi_message_init(&m);
	spi_message_add_tail(&t, &m);
	return gs_codec_sync(gs_codec, &m);
}
/*---------------------------the end----------------------------------------------*/
__weak int get_gs_device(void)
{
	if (atomic_cmpxchg(&codec_usage_cnt, 0, 1) != 0)
		return -EBUSY;
	return 0;
}

__weak int put_gs_device(void)
{
	if (atomic_cmpxchg(&codec_usage_cnt, 1, 0) != 1)
		return -EFAULT;
	return 0;
}

/* Read-only message with current device setup */
static ssize_t
gs_codec_read(struct file *filp, char __user *buf, size_t count, loff_t *f_pos)
{
	struct gs_codec_data	*gs_codec;
	ssize_t			status = 0;

	/* chipselect only toggles at start or end of operation */
	if (count > SPI_BUF_LEN)
		return -EMSGSIZE;

	gs_codec = filp->private_data;

	mutex_lock(&gs_codec->buf_lock);
	status = gs_codec_sync_read(gs_codec, count);
	if (status > 0) {
		unsigned long	missing;

		missing = copy_to_user(buf, gs_codec->spi_buffer, status);
		if (missing == status)
			status = -EFAULT;
		else
			status = status - missing;
	}
	mutex_unlock(&gs_codec->buf_lock);

	return status;
}
/* Write-only message with current device setup */
static ssize_t
gs_codec_write(struct file *filp, const char __user *buf,
		size_t count, loff_t *f_pos)
{
	struct gs_codec_data	*gs_codec;
	ssize_t			status = 0;
	unsigned long		missing;

	/* chipselect only toggles at start or end of operation */
	if (count > SPI_BUF_LEN)
		return -EMSGSIZE;

	gs_codec = filp->private_data;

	mutex_lock(&gs_codec->buf_lock);
	missing = copy_from_user(gs_codec->spi_buffer, buf, count);
	if (missing == 0) {
		status = gs_codec_sync_write(gs_codec, count);
	} else
		status = -EFAULT;
	mutex_unlock(&gs_codec->buf_lock);

	return status;
}


static int gs_codec_open(struct inode *inode, struct file *filp)
{
	int ret = 0;
	ret = get_gs_device();
	if (ret) {
		pr_err("[GSCODEC] device busy\n");
		goto exit;
	}

	pr_debug("[GSCODEC] open by %s\n", current->comm);
	filp->private_data = gs_codec;
	nonseekable_open(inode, filp);
exit:
	return ret;
}

static int gs_codec_release(struct inode *inode, struct file *filp)
{
	int ret = 0;
	ret = put_gs_device();
	if (ret) {
		pr_err("[GSCODEC] device close without open\n");
		goto exit;
	}

	filp->private_data = NULL;
	pr_debug("[GSCODEC] close by %s\n", current->comm);
exit:
	return ret;
}


static int gs_codec_fasync(int fd, struct file * filp, int on) 
{
    int retval;
    struct gs_codec_data	*gs_codec;
    gs_codec = filp->private_data;
	
    retval=fasync_helper(fd,filp,on,&gs_codec->fasync_q);
    if(retval<0)
      return retval;
	
    return 0;
}

static void gs_codec_fasync_work(struct work_struct *work )
{
	struct gs_codec_data	*gs_codec;
	gs_codec = container_of(work,struct gs_codec_data,fasync_wq);
	
    if (gs_codec->fasync_q)
      kill_fasync(&gs_codec->fasync_q, SIGIO, POLL_IN);
	
}

#define CONFIG_FPGA_TEST 1
#ifdef CONFIG_FPGA_TEST
static void gs_fpga_test(void)
{
	int ret;
	char txcmd[32]={0};
	char rxdat[32]={0};

	/*step 1 send start up */
	txcmd[0] = 0xF1;
	txcmd[1] = 0x00;
	memcpy(gs_codec->spi_buffer,&txcmd,2);
	gs_codec_sync_write(gs_codec,2);


	txcmd[0] = 0x00;
	txcmd[1] = 0xF1;
	memcpy(gs_codec->spi_buffer,&txcmd,2);
	gs_codec_sync_write(gs_codec,2);

	/*step 2.1 read active flag*/
	txcmd[0] = 0xF2;
	txcmd[1] = 0x00;
	ret = spi_write_then_read(gs_codec->spidev,&txcmd,2,&rxdat,1);
	if(ret != 0){
		pr_err("[GSCODEC] read active flag error\n");
	}
	pr_info("[GSCODEC] get active flag %d\n",rxdat[0]);

	txcmd[0] = 0x00;
	txcmd[1] = 0xF2;
	ret = spi_write_then_read(gs_codec->spidev,&txcmd,2,&rxdat,1);
	if(ret != 0){
		pr_err("[GSCODEC] read active flag error\n");
	}
	pr_info("[GSCODEC] get active flag %d\n",rxdat[0]);


	/*step 2.2 read active flag*/
	txcmd[0] = 0xF2;
	txcmd[1] = 0x00;
	memcpy(gs_codec->spi_buffer,txcmd,2);
	gs_codec_sync_write(gs_codec,2);
	gs_codec_sync_read(gs_codec,1);
	pr_info("[GSCODEC] get active flag %d\n",gs_codec->spi_buffer[0]);

	/*step 3 */
	txcmd[0] = 0x00;
	txcmd[1] = 0xF3;
	ret = spi_write_then_read(gs_codec->spidev,&txcmd,2,&rxdat,18);
	if(ret != 0){
		pr_err("[GSCODEC] read audio data error\n");
	}

	for(ret = 0; ret < 18; ret++)
		pr_info("[GSCODEC] get audio data %d:0x%x\n",ret,rxdat[ret]);
	
}
#endif

static const struct file_operations char_dev_fops = {
	.owner 			=	THIS_MODULE,
	.write 			=	gs_codec_write,
	.read 			=	gs_codec_read,
	.open	 		=	gs_codec_open,
	.release 		=	gs_codec_release,
	.fasync			=	gs_codec_fasync,
};

/*---------------------------the end----------------------------------------*/

#ifdef CONFIG_FPGA_TEST

static int gs_codec_proc_show(struct seq_file *m, void *v)
{

	struct gs_codec_data	*dev = (struct gs_codec_data *)m->private;
	seq_printf(m,"\n[GSCODEC] set mode:%d speed:%d mlsb:%d\n",dev->mode,dev->speed,dev->mlsb);
	return 0;
}


static int gs_codec_proc_open(struct inode *inode, struct file *file)
{
    return single_open(file, gs_codec_proc_show, PDE_DATA(inode));
}

static ssize_t gs_codec_proc_write (struct file *file,
	const char __user *buffer, size_t count, loff_t *pos)

{
	int ret = 0;
	char buff[32] = {0};
	int mode = 0; // 0 register, 1 fram rate set
	int speed = 0;
	int mlsb = 0;

	struct gs_codec_data *dev = PDE_DATA(file_inode(file));
	struct spi_device	*spi = dev->spidev;

	memset(&buff,0,32);
	if (copy_from_user(&buff, buffer,32))
	{
		pr_err("%s: copy_from_user error!",__func__);
		return (-EFAULT);
	}
	
	ret = sscanf(buff,"%d:%d=%d",&mode,&speed,&mlsb);
	if(ret != 3){
		pr_err("%s:input error,cmd[mode:speed=mlsb]\n",__func__);
		return -EFAULT;
	}

	pr_info("[GSCODEC] set mode:%d speed:%d mlsb:%d\n",mode,speed,mlsb);
	dev->mlsb = mlsb;
	spi->mode = dev->mode = mode; /* FIXME */
	spi->max_speed_hz = dev->speed = speed;
	ret = spi_setup(spi);
	if (ret < 0) {
		pr_err("[GSCODEC] spi_setup fail ret=%d\n", ret);
		return -1;
	}
	
	gs_fpga_test();

	return (count);
}

static const struct file_operations debug_proc_fops = {
	.open = gs_codec_proc_open,
	.read = seq_read,
	.write  = gs_codec_proc_write,
	.release = single_release,
};

static void  gs_codec_proc_init(struct gs_codec_data *dev )
{
    dev->dir = proc_mkdir("gscodec", NULL);
    if(dev->dir){ 
		dev->debug = proc_create_data("debug",0644,dev->dir,&debug_proc_fops,dev);
		return;
    }

    pr_info("%s:creat proc dir error!\n",__func__);
}
#endif 
static int __init gs_spi_codec_probe(struct spi_device *spi)
{
	int ret = 0;


	printk(KERN_EMERG"[GSCODEC] gs_spi_codec_probe START\n");
	if(!gs_codec){
		pr_err("[GSCODEC] device resource is NULL\n");
		return -EFAULT;	
	}

	spin_lock_init(&gs_codec->spi_lock);
	mutex_init(&gs_codec->buf_lock);

	/* create char device */
	ret = alloc_chrdev_region(&gs_codec->devt, 0, 1, "gs_codec");
	if (ret) {
		pr_err("[GSCODEC] alloc_chrdev_region fail ret=%d\n", ret);
		goto exit;
	}

	gs_codec->c_dev.owner = THIS_MODULE;
	cdev_init(&gs_codec->c_dev, &char_dev_fops);
	ret = cdev_add(&gs_codec->c_dev, gs_codec->devt, 1);
	if (ret) {
		pr_err("[GSCODEC] cdev_add fail ret=%d\n", ret);
		goto exit;
	}

	gs_codec->dev_class = class_create(THIS_MODULE, "gs_codec");
	gs_codec->dev = device_create(gs_codec->dev_class, NULL, gs_codec->devt, NULL, "gs_codec");
	if (IS_ERR(gs_codec->dev)) {
		ret = PTR_ERR(gs_codec->dev);
		pr_err("[GSCODEC] device_create fail ret=%d\n", ret);
		goto exit;
	}

	/* alloc buffer */
	gs_codec->spi_buffer = kzalloc(SPI_BUF_LEN, GFP_KERNEL);
	if (!gs_codec->spi_buffer) {
		pr_err("[GSCODEC] device malloc memory fail\n");
		ret = -ENOMEM;
		goto exit;
	}

	spi->controller_data = (void *)&gs_spi_conf;
	spi->mode = SPI_MODE_1; /* FIXME */
	spi->bits_per_word = 8;
	spi->max_speed_hz = SPI_SPEED;
	spi->cs_gpio = gs_codec->gpio;
	ret = spi_setup(spi);
	if (ret < 0) {
		pr_err("[GSCODEC] spi_setup fail ret=%d\n", ret);
		goto exit;
	}
	gs_codec->spidev = spi;
	spi_set_drvdata(spi, gs_codec);
	INIT_WORK(&gs_codec->fasync_wq,gs_codec_fasync_work);
	
#ifdef CONFIG_FPGA_TEST
	gs_fpga_test();
	gs_codec_proc_init(gs_codec);

#endif 
	pr_info("[GSCODEC] gs_spi_codec_probe END\n");
 exit:
	return ret;
}

static int __init gs_spi_codec_remove(struct spi_device *spi)
{
	struct gs_codec_data *priv = spi_get_drvdata(spi);

	/* make sure ops on existing fds can abort cleanly */
	spin_lock_irq(&priv->spi_lock);
	priv->spidev = NULL;
	spi_set_drvdata(spi, NULL);
	spin_unlock_irq(&priv->spi_lock);

	cdev_del (&priv->c_dev);
	device_destroy(priv->dev_class,  priv->devt);
    class_destroy(priv->dev_class);                               //delete class created by us
    unregister_chrdev_region (priv->devt, 1);

	return 0;
}
/*-------------------------------------------------------------------------*/
static struct spi_device_id spi_id_table = {"gs_codec", 0};

#if 1
static const struct of_device_id spidev_dt_ids[] = {
	{	.compatible = "gscodec"},
	{},
};

MODULE_DEVICE_TABLE(of,spidev_dt_ids);
#endif 

static struct spi_driver gs_spi_codec_driver = {
	.driver = {
		.name = "gs_codec",
		.bus = &spi_bus_type,
		.owner = THIS_MODULE,
		.of_match_table = of_match_ptr(spidev_dt_ids),
	},
	.probe = gs_spi_codec_probe,
	.remove = gs_spi_codec_remove,
	.id_table = &spi_id_table,
};


static int gs_codec_probe(struct platform_device *pdev)
{
	int err = 0;
	//int gpio_cs;
	if (pdev->dev.of_node == NULL) {
		pr_err("[GSCODEC] OF node is NULL\n");
		return -ENODEV;
	}
	pr_info("[GSCODEC] gs_codec_probe  %s START\n",pdev->name);

	gs_codec = devm_kzalloc(&pdev->dev, sizeof(*gs_codec),GFP_KERNEL);
	if (!gs_codec)
		return -ENOMEM;

	gs_codec->gpio = of_get_gpio(pdev->dev.of_node , 0);
	if (!gpio_is_valid(gs_codec->gpio)) {
		err = -EINVAL;
		dev_err(&pdev->dev, "gpio%d is invalid.\n", gs_codec->gpio);
		goto fail;
	}

	err = devm_gpio_request_one(&pdev->dev, gs_codec->gpio, GPIOF_DIR_OUT, "CODEC_CS");
	if (err < 0) {
		err = -EFAULT;
		dev_err(&pdev->dev, "request gpio%d failed.\n",  gs_codec->gpio);
		goto fail;
	}

	gpio_set_value(gs_codec->gpio,1);
	/* create SPI device */
	err = spi_register_driver(&gs_spi_codec_driver);
	if (err) {
		pr_err("[GSCODEC] spi_register_driver fail ret=%d\n", err);
		goto fail;
	}

	platform_set_drvdata(pdev, gs_codec);
	printk(KERN_EMERG"[GSCODEC] gs_codec_probe enD\n");
	return 0;
fail:
	kfree(gs_codec);
	return err;
	
}

static int gs_codec_remove(struct platform_device *pdev)
{
	struct gs_codec_data *priv = platform_get_drvdata(pdev);

	platform_set_drvdata(pdev, NULL);
	kfree(priv);
	return 0;
}


static const struct of_device_id of_gs_codec_match[] = {
	{ .compatible = "grandstream,codec", },
	{},
};

MODULE_DEVICE_TABLE(of, of_gs_codec_match);

static struct platform_driver gs_codec_driver = {
	.probe		 = gs_codec_probe,
	.remove		 = gs_codec_remove,
	.driver		 = {
		.name	 = "gs_codec",
		.owner	 = THIS_MODULE,
		.of_match_table = of_match_ptr(of_gs_codec_match),
	},
};

module_platform_driver(gs_codec_driver);
MODULE_AUTHOR("spchen@grandstream.cn");
MODULE_DESCRIPTION("User mode SPI device interface");
MODULE_LICENSE("GPL");
MODULE_ALIAS("Grandstream Audio codec");
