#!/system/xbin/sh
#@20111020
#@20111029 fix, apply different policy according to nvram.wan_ip_option
#@20111102 fix, pppoe_device=`wan_ifname`
#@20111108 fix, override dscp(nvram.38) with MED network policy's DSCP
#          lldpd disable old network policy first when LLDP packet changes.
#@20120202 fix, enalbe MED Network Policy TLVs use lldpctl 
#@20120607 remove (nvram.38) modification
#@20120926 fix static ip dns1 dns2 is not set bug,
#          GXP [Bug 12104] Phone loose VLAN tag randomly and loose the network connection.
# ANSI/TIA-1057- 2006
# ENABLE: 1 or 0
# DTYPE:
# 0-not defined
# 1-class I
# 2-class II
# 3-class III
# 4-Network Connectivity
# 5-255 Reserved
#
# ATYPE:
# 0-Reserved
# 1-Voice
# 2-Voice Signaling
# 3-Guest Voice
# 4-Guest Voice Signaling
# 5-Softphone Voice
# 6-Video Conferencing
# 7-Streaming Video
# 8-Video Signaling
# 9-255 Reserved
#
# UNKNOWN: 0 or 1
# TAGGED: 0 or 1
# VID: 0-Priority Tagged, 1-4094 VLAN ID, 4095 Reserved
# PRIORITY: 0-7
# DSCP: 0-63
#

ENABLE=$1
TAGGED=$2
VID=$3
PRIORITY=$4
AUDIO_DSCP=$5
SIP_DSCP=$6
VIDEO_DSCP=$7

VLAN_PROP="net.vlan.tag"
LLDP_PROP="net.lldp.running"
VLAN_PRI_PROP="net.vlan.pri"
VLAN_OLD_PROP="net.vlan.old.tag"
VLAN_OLD_PRI_PROP="net.vlan.old.pri"
VLAN_OLD_AUDIO_DSCP="new.vlan.old.audio"
VLAN_OLD_SIP_DSCP="new.vlan.old.sip"
VLAN_OLD_VIDEO_DSCP="new.vlan.old.video"
NET_SAVE_PROP="net.qos.saved"
NET_STORE_PROP="net.qos.stored"
VLAN_ID=`nvram get :vlan_id`

syslog() {
	if [ "`nvram get 208`" != "0" ]; then
		logger -p kern.info -t "`nvram get syslog_id`" "[`cat /proc/gsboard/dev_info/dev_mac`]" "[VOIP VLAN]" $*
	else
		log -t "VOIP_VLAN" $*
	fi
}

send_cfupdate()
{
	/system/xbin/dbus-send  --system --type=signal /com/grandstream/dbus/gui com.grandstream.dbus.signal.cfupdated
	syslog "send cfupdated"
}

get_vlan_tag(){
	vlan_tag="0"
	wifi_switch=`nvram get :wifi_switch`
	if [ "${wifi_switch}"x = "1x" ]; then
		vlan_tag=`nvram get 22047`
	else
    if [ -z $VLAN_ID ] || [$VLAN_ID -eq 0]; then
			vlan_tag=`nvram get 51`
		else 
			vlan_tag=$VLAN_ID
		fi       
	fi

	return ${vlan_tag}
}

med_need_join(){
	vlan_id=$1
	rval=1

	get_vlan_tag
	tag=$?

	if [ -e /sys/class/net/eth0.${vlan_id} ] || [ -e /sys/class/net/wlan0.${vlan_id} ]; then
		#VLAN ID is not equal to nvram.51 and VLAN interface exist
		rval=0
	else 
		rval=1
	fi
	syslog "med vid ${vlan_id}, nvram 51(or 22047)=${tag}, need join ${rval}"
	return $rval
}

med_need_leave(){
	vlan_id=$1
	rval=0

	if [ -e /sys/class/net/eth0.${vlan_id} ] || [ -e /sys/class/net/wlan0.${vlan_id} ]; then
		#VLAN ID is not equal to nvram.51 and VLAN interface exist
		rval=1
	fi

	syslog "med vid ${vlan_id}, need leave ${rval}"
	return $rval
}


store_cfg()
{
	#setprop ${NET_SAVE_PROP} "0"
	vlan_tag="0"
	qos_priority="0"
	vlan_tag=`nvram get 51`
	qos_priority=`nvram get 87`
	setprop $NET_STORE_PROP "eth0"

	sip_dscp=`nvram get 1558`
	audio_dscp=`nvram get 1559`
	video_dscp=`nvram get 1560`
	setprop $VLAN_OLD_PROP $vlan_tag
	setprop $VLAN_OLD_PRI_PROP $qos_priority
	setprop $VLAN_OLD_AUDIO_DSCP $audio_dscp
	setprop $VLAN_OLD_SIP_DSCP $sip_dscp
	setprop $VLAN_OLD_VIDEO_DSCP $video_dscp

	syslog "store vlan_tag=${vlan_tag}, qos_priority=${qos_priority},sip_dscp=${sip_dscp},audio_dscp=${audio_dscp},video_dscp=${video_dscp}"
}

restore_dscp()
{

	sip_dscp=`getprop ${VLAN_OLD_SIP_DSCP}`
	audio_dscp=`getprop ${VLAN_OLD_AUDIO_DSCP}`
	video_dscp=`getprop ${VLAN_OLD_VIDEO_DSCP}`
	nvram unset :1558
	nvram unset :1559
	nvram unset :1560

	nvram set 1558=$sip_dscp
	nvram set 1559=$audio_dscp
	nvram set 1560=$video_dscp
	nvram commit
	send_cfupdate
	setprop $VLAN_OLD_PROP "0"
	setprop $VLAN_OLD_PRI_PROP "0"
	setprop $VLAN_OLD_AUDIO_DSCP "0"
	setprop $VLAN_OLD_SIP_DSCP "0"
	setprop $VLAN_OLD_VIDEO_DSCP "0"
}

restore_cfg()
{
	vlan_tag=`getprop ${VLAN_OLD_PROP}`
	qos_priority=`getprop ${VLAN_OLD_PRI_PROP}`
	sip_dscp=`getprop ${VLAN_OLD_SIP_DSCP}`
	audio_dscp=`getprop ${VLAN_OLD_AUDIO_DSCP}`
	video_dscp=`getprop ${VLAN_OLD_VIDEO_DSCP}`

	saved=`getprop ${NET_SAVE_PROP}`
	stored=`getprop ${NET_STORE_PROP}`

	if [ $saved -eq 0 ];then
		nvram unset :51
		nvram unset :87
		nvram set 51=$vlan_tag
		nvram set 87=$qos_priority
		restore_dscp
		setprop $NET_STORE_PROP "0"
		syslog "restore vlan_tag=${vlan_tag}, qos_priority=${qos_priority},sip_dscp=${sip_dscp},audio_dscp=${audio_dscp},video_dscp=${video_dscp}"
	else
		setprop ${NET_SAVE_PROP} "0"
		setprop $NET_STORE_PROP "0"
	fi
}

join_med_vlan(){
	vlan_tag=$1
	qos_priority=$2
	audio_dscp=$3
	sip_dscp=$4
	video_dscp=$5

	setprop ${NET_SAVE_PROP} "0"
	setprop $VLAN_PROP $vlan_tag
	setprop $VLAN_PRI_PROP ${qos_priority} 

	nvram set :vlan_id=$1
	nvram set :vlan_qos=$2

	if [ $audio_dscp -gt 0 ];then
		nvram set :dscp1=$audio_dscp
		#nvram set :1559=$audio_dscp
	fi
	if [ $sip_dscp -gt 0 ];then
		nvram set :dscp2=$sip_dscp
		#		nvram set :1558=$sip_dscp
	fi
	if [ $video_dscp -gt 0 ];then
		nvram set :dscp7=$video_dscp
		#		nvram set :1560=$video_dscp
	fi
	send_cfupdate
}

leave_med_vlan(){
	setprop $VLAN_PROP "0"
	setprop $VLAN_PRI_PROP "0"
	#		nvram set :vlan_id=
	#       nvram set :vlan_qos=
}


#LLDP_TAG=`nvram get :in_lldp`

if [ $ENABLE -eq 1 ]; then
	if [ $TAGGED -eq 1 ];then
		med_need_join ${VID}
		if [ 1 -eq $? ]; then
			/system/bin/stop provision

			join_med_vlan ${VID} ${PRIORITY} ${AUDIO_DSCP} ${SIP_DSCP} ${VIDEO_DSCP}
			nvram set :in_lldp="1"
			/system/bin/start provision
		fi
	else
		nvram set :lldp_fast_timeout="1"
	fi
	MED_NP="1:0:${TAGGED}:${VID}:${PRIORITY}:${AUDIO_DSCP}"
	lldpctl -P ${MED_NP} &
else 
	get_vlan_tag
	LAST_VID=$?
	med_need_leave ${LAST_VID}
	if [ 1 -eq $? ]; then
		/system/bin/stop provision
		
		leave_med_vlan 
        nvram set :vlan_id=""
        nvram set :vlan_qos=""
        nvram set :dscp1=""
        nvram set :dscp2=""
        nvram set :dscp7=""
		nvram set :in_lldp="0"
		/system/bin/start provision
	else
        nvram set :vlan_id=""
        nvram set :vlan_qos=""
        nvram set :dscp1=""
        nvram set :dscp2=""
        nvram set :dscp7=""
		nvram set :in_lldp="0"

	fi
	lldpctl -P 1:0:0:0:0:0 &
fi
