/*
 * Copyright (C) 2016 MediaTek Inc.

 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See http://www.gnu.org/licenses/gpl-2.0.html for more details.
 */

#include <linux/kernel.h>
#include <linux/module.h>

#include <mach/upmu_sw.h>
#include <mach/upmu_hw.h>
#include <mt-plat/upmu_common.h>

const PMU_FLAG_TABLE_ENTRY pmu_flags_table[] = {
	{PMIC_RG_THR_DET_DIS, PMIC_RG_THR_DET_DIS_ADDR, PMIC_RG_THR_DET_DIS_MASK,
	 PMIC_RG_THR_DET_DIS_SHIFT},
	{PMIC_RG_THR_TEST, PMIC_RG_THR_TEST_ADDR, PMIC_RG_THR_TEST_MASK, PMIC_RG_THR_TEST_SHIFT},
	{PMIC_RG_STRUP_THER_DEB_RMAX, PMIC_RG_STRUP_THER_DEB_RMAX_ADDR,
	 PMIC_RG_STRUP_THER_DEB_RMAX_MASK, PMIC_RG_STRUP_THER_DEB_RMAX_SHIFT},
	{PMIC_RG_STRUP_THER_DEB_FMAX, PMIC_RG_STRUP_THER_DEB_FMAX_ADDR,
	 PMIC_RG_STRUP_THER_DEB_FMAX_MASK, PMIC_RG_STRUP_THER_DEB_FMAX_SHIFT},
	{PMIC_DDUVLO_DEB_EN, PMIC_DDUVLO_DEB_EN_ADDR, PMIC_DDUVLO_DEB_EN_MASK,
	 PMIC_DDUVLO_DEB_EN_SHIFT},
	{PMIC_RG_STRUP_OSC_EN, PMIC_RG_STRUP_OSC_EN_ADDR, PMIC_RG_STRUP_OSC_EN_MASK,
	 PMIC_RG_STRUP_OSC_EN_SHIFT},
	{PMIC_RG_STRUP_OSC_EN_SEL, PMIC_RG_STRUP_OSC_EN_SEL_ADDR, PMIC_RG_STRUP_OSC_EN_SEL_MASK,
	 PMIC_RG_STRUP_OSC_EN_SEL_SHIFT},
	{PMIC_RG_STRUP_FT_CTRL, PMIC_RG_STRUP_FT_CTRL_ADDR, PMIC_RG_STRUP_FT_CTRL_MASK,
	 PMIC_RG_STRUP_FT_CTRL_SHIFT},
	{PMIC_RG_STRUP_PWRON_FORCE, PMIC_RG_STRUP_PWRON_FORCE_ADDR, PMIC_RG_STRUP_PWRON_FORCE_MASK,
	 PMIC_RG_STRUP_PWRON_FORCE_SHIFT},
	{PMIC_RG_BIASGEN_FORCE, PMIC_RG_BIASGEN_FORCE_ADDR, PMIC_RG_BIASGEN_FORCE_MASK,
	 PMIC_RG_BIASGEN_FORCE_SHIFT},
	{PMIC_RG_STRUP_PWRON, PMIC_RG_STRUP_PWRON_ADDR, PMIC_RG_STRUP_PWRON_MASK,
	 PMIC_RG_STRUP_PWRON_SHIFT},
	{PMIC_RG_STRUP_PWRON_SEL, PMIC_RG_STRUP_PWRON_SEL_ADDR, PMIC_RG_STRUP_PWRON_SEL_MASK,
	 PMIC_RG_STRUP_PWRON_SEL_SHIFT},
	{PMIC_RG_BIASGEN, PMIC_RG_BIASGEN_ADDR, PMIC_RG_BIASGEN_MASK, PMIC_RG_BIASGEN_SHIFT},
	{PMIC_RG_BIASGEN_SEL, PMIC_RG_BIASGEN_SEL_ADDR, PMIC_RG_BIASGEN_SEL_MASK,
	 PMIC_RG_BIASGEN_SEL_SHIFT},
	{PMIC_RG_RTC_XOSC32_ENB, PMIC_RG_RTC_XOSC32_ENB_ADDR, PMIC_RG_RTC_XOSC32_ENB_MASK,
	 PMIC_RG_RTC_XOSC32_ENB_SHIFT},
	{PMIC_RG_RTC_XOSC32_ENB_SEL, PMIC_RG_RTC_XOSC32_ENB_SEL_ADDR,
	 PMIC_RG_RTC_XOSC32_ENB_SEL_MASK, PMIC_RG_RTC_XOSC32_ENB_SEL_SHIFT},
	{PMIC_STRUP_DIG_IO_PG_FORCE, PMIC_STRUP_DIG_IO_PG_FORCE_ADDR,
	 PMIC_STRUP_DIG_IO_PG_FORCE_MASK, PMIC_STRUP_DIG_IO_PG_FORCE_SHIFT},
	{PMIC_CLR_JUST_RST, PMIC_CLR_JUST_RST_ADDR, PMIC_CLR_JUST_RST_MASK,
	 PMIC_CLR_JUST_RST_SHIFT},
	{PMIC_UVLO_L2H_DEB_EN, PMIC_UVLO_L2H_DEB_EN_ADDR, PMIC_UVLO_L2H_DEB_EN_MASK,
	 PMIC_UVLO_L2H_DEB_EN_SHIFT},
	{PMIC_JUST_PWRKEY_RST, PMIC_JUST_PWRKEY_RST_ADDR, PMIC_JUST_PWRKEY_RST_MASK,
	 PMIC_JUST_PWRKEY_RST_SHIFT},
	{PMIC_DA_QI_OSC_EN, PMIC_DA_QI_OSC_EN_ADDR, PMIC_DA_QI_OSC_EN_MASK,
	 PMIC_DA_QI_OSC_EN_SHIFT},
	{PMIC_RG_STRUP_EXT_PMIC_EN, PMIC_RG_STRUP_EXT_PMIC_EN_ADDR, PMIC_RG_STRUP_EXT_PMIC_EN_MASK,
	 PMIC_RG_STRUP_EXT_PMIC_EN_SHIFT},
	{PMIC_RG_STRUP_EXT_PMIC_SEL, PMIC_RG_STRUP_EXT_PMIC_SEL_ADDR,
	 PMIC_RG_STRUP_EXT_PMIC_SEL_MASK, PMIC_RG_STRUP_EXT_PMIC_SEL_SHIFT},
	{PMIC_STRUP_CON8_RSV0, PMIC_STRUP_CON8_RSV0_ADDR, PMIC_STRUP_CON8_RSV0_MASK,
	 PMIC_STRUP_CON8_RSV0_SHIFT},
	{PMIC_DA_QI_EXT_PMIC_EN, PMIC_DA_QI_EXT_PMIC_EN_ADDR, PMIC_DA_QI_EXT_PMIC_EN_MASK,
	 PMIC_DA_QI_EXT_PMIC_EN_SHIFT},
	{PMIC_RG_STRUP_AUXADC_START_SW, PMIC_RG_STRUP_AUXADC_START_SW_ADDR,
	 PMIC_RG_STRUP_AUXADC_START_SW_MASK, PMIC_RG_STRUP_AUXADC_START_SW_SHIFT},
	{PMIC_RG_STRUP_AUXADC_RSTB_SW, PMIC_RG_STRUP_AUXADC_RSTB_SW_ADDR,
	 PMIC_RG_STRUP_AUXADC_RSTB_SW_MASK, PMIC_RG_STRUP_AUXADC_RSTB_SW_SHIFT},
	{PMIC_RG_STRUP_AUXADC_START_SEL, PMIC_RG_STRUP_AUXADC_START_SEL_ADDR,
	 PMIC_RG_STRUP_AUXADC_START_SEL_MASK, PMIC_RG_STRUP_AUXADC_START_SEL_SHIFT},
	{PMIC_RG_STRUP_AUXADC_RSTB_SEL, PMIC_RG_STRUP_AUXADC_RSTB_SEL_ADDR,
	 PMIC_RG_STRUP_AUXADC_RSTB_SEL_MASK, PMIC_RG_STRUP_AUXADC_RSTB_SEL_SHIFT},
	{PMIC_RG_STRUP_AUXADC_RPCNT_MAX, PMIC_RG_STRUP_AUXADC_RPCNT_MAX_ADDR,
	 PMIC_RG_STRUP_AUXADC_RPCNT_MAX_MASK, PMIC_RG_STRUP_AUXADC_RPCNT_MAX_SHIFT},
	{PMIC_STRUP_PWROFF_SEQ_EN, PMIC_STRUP_PWROFF_SEQ_EN_ADDR, PMIC_STRUP_PWROFF_SEQ_EN_MASK,
	 PMIC_STRUP_PWROFF_SEQ_EN_SHIFT},
	{PMIC_STRUP_PWROFF_PREOFF_EN, PMIC_STRUP_PWROFF_PREOFF_EN_ADDR,
	 PMIC_STRUP_PWROFF_PREOFF_EN_MASK, PMIC_STRUP_PWROFF_PREOFF_EN_SHIFT},
	{PMIC_STRUP_DIG0_RSV0, PMIC_STRUP_DIG0_RSV0_ADDR, PMIC_STRUP_DIG0_RSV0_MASK,
	 PMIC_STRUP_DIG0_RSV0_SHIFT},
	{PMIC_STRUP_DIG1_RSV0, PMIC_STRUP_DIG1_RSV0_ADDR, PMIC_STRUP_DIG1_RSV0_MASK,
	 PMIC_STRUP_DIG1_RSV0_SHIFT},
	{PMIC_RG_RSV_SWREG, PMIC_RG_RSV_SWREG_ADDR, PMIC_RG_RSV_SWREG_MASK,
	 PMIC_RG_RSV_SWREG_SHIFT},
	{PMIC_RG_STRUP_UVLO_U1U2_SEL, PMIC_RG_STRUP_UVLO_U1U2_SEL_ADDR,
	 PMIC_RG_STRUP_UVLO_U1U2_SEL_MASK, PMIC_RG_STRUP_UVLO_U1U2_SEL_SHIFT},
	{PMIC_RG_STRUP_UVLO_U1U2_SEL_SWCTRL, PMIC_RG_STRUP_UVLO_U1U2_SEL_SWCTRL_ADDR,
	 PMIC_RG_STRUP_UVLO_U1U2_SEL_SWCTRL_MASK, PMIC_RG_STRUP_UVLO_U1U2_SEL_SWCTRL_SHIFT},
	{PMIC_RG_STRUP_THR_CLR, PMIC_RG_STRUP_THR_CLR_ADDR, PMIC_RG_STRUP_THR_CLR_MASK,
	 PMIC_RG_STRUP_THR_CLR_SHIFT},
	{PMIC_RG_STRUP_LONG_PRESS_EXT_SEL, PMIC_RG_STRUP_LONG_PRESS_EXT_SEL_ADDR,
	 PMIC_RG_STRUP_LONG_PRESS_EXT_SEL_MASK, PMIC_RG_STRUP_LONG_PRESS_EXT_SEL_SHIFT},
	{PMIC_RG_STRUP_LONG_PRESS_EXT_TD, PMIC_RG_STRUP_LONG_PRESS_EXT_TD_ADDR,
	 PMIC_RG_STRUP_LONG_PRESS_EXT_TD_MASK, PMIC_RG_STRUP_LONG_PRESS_EXT_TD_SHIFT},
	{PMIC_RG_STRUP_LONG_PRESS_EXT_EN, PMIC_RG_STRUP_LONG_PRESS_EXT_EN_ADDR,
	 PMIC_RG_STRUP_LONG_PRESS_EXT_EN_MASK, PMIC_RG_STRUP_LONG_PRESS_EXT_EN_SHIFT},
	{PMIC_RG_STRUP_LONG_PRESS_EXT_CHR_CTRL, PMIC_RG_STRUP_LONG_PRESS_EXT_CHR_CTRL_ADDR,
	 PMIC_RG_STRUP_LONG_PRESS_EXT_CHR_CTRL_MASK, PMIC_RG_STRUP_LONG_PRESS_EXT_CHR_CTRL_SHIFT},
	{PMIC_RG_STRUP_LONG_PRESS_EXT_PWRKEY_CTRL, PMIC_RG_STRUP_LONG_PRESS_EXT_PWRKEY_CTRL_ADDR,
	 PMIC_RG_STRUP_LONG_PRESS_EXT_PWRKEY_CTRL_MASK,
	 PMIC_RG_STRUP_LONG_PRESS_EXT_PWRKEY_CTRL_SHIFT},
	{PMIC_RG_STRUP_LONG_PRESS_EXT_SPAR_CTRL, PMIC_RG_STRUP_LONG_PRESS_EXT_SPAR_CTRL_ADDR,
	 PMIC_RG_STRUP_LONG_PRESS_EXT_SPAR_CTRL_MASK, PMIC_RG_STRUP_LONG_PRESS_EXT_SPAR_CTRL_SHIFT},
	{PMIC_RG_STRUP_LONG_PRESS_EXT_RTCA_CTRL, PMIC_RG_STRUP_LONG_PRESS_EXT_RTCA_CTRL_ADDR,
	 PMIC_RG_STRUP_LONG_PRESS_EXT_RTCA_CTRL_MASK, PMIC_RG_STRUP_LONG_PRESS_EXT_RTCA_CTRL_SHIFT},
	{PMIC_RG_STRUP_ENVTEM, PMIC_RG_STRUP_ENVTEM_ADDR, PMIC_RG_STRUP_ENVTEM_MASK,
	 PMIC_RG_STRUP_ENVTEM_SHIFT},
	{PMIC_RG_STRUP_ENVTEM_CTRL, PMIC_RG_STRUP_ENVTEM_CTRL_ADDR, PMIC_RG_STRUP_ENVTEM_CTRL_MASK,
	 PMIC_RG_STRUP_ENVTEM_CTRL_SHIFT},
	{PMIC_RG_STRUP_PWRKEY_COUNT_RESET, PMIC_RG_STRUP_PWRKEY_COUNT_RESET_ADDR,
	 PMIC_RG_STRUP_PWRKEY_COUNT_RESET_MASK, PMIC_RG_STRUP_PWRKEY_COUNT_RESET_SHIFT},
	{PMIC_RG_STRUP_VSRAM_DVFS2_PG_H2L_EN, PMIC_RG_STRUP_VSRAM_DVFS2_PG_H2L_EN_ADDR,
	 PMIC_RG_STRUP_VSRAM_DVFS2_PG_H2L_EN_MASK, PMIC_RG_STRUP_VSRAM_DVFS2_PG_H2L_EN_SHIFT},
	{PMIC_RG_STRUP_VSRAM_DVFS1_PG_H2L_EN, PMIC_RG_STRUP_VSRAM_DVFS1_PG_H2L_EN_ADDR,
	 PMIC_RG_STRUP_VSRAM_DVFS1_PG_H2L_EN_MASK, PMIC_RG_STRUP_VSRAM_DVFS1_PG_H2L_EN_SHIFT},
	{PMIC_RG_STRUP_VSRAM_CORE_PG_H2L_EN, PMIC_RG_STRUP_VSRAM_CORE_PG_H2L_EN_ADDR,
	 PMIC_RG_STRUP_VSRAM_CORE_PG_H2L_EN_MASK, PMIC_RG_STRUP_VSRAM_CORE_PG_H2L_EN_SHIFT},
	{PMIC_RG_STRUP_VSRAM_GPU_PG_H2L_EN, PMIC_RG_STRUP_VSRAM_GPU_PG_H2L_EN_ADDR,
	 PMIC_RG_STRUP_VSRAM_GPU_PG_H2L_EN_MASK, PMIC_RG_STRUP_VSRAM_GPU_PG_H2L_EN_SHIFT},
	{PMIC_RG_STRUP_VSRAM_MD_PG_H2L_EN, PMIC_RG_STRUP_VSRAM_MD_PG_H2L_EN_ADDR,
	 PMIC_RG_STRUP_VSRAM_MD_PG_H2L_EN_MASK, PMIC_RG_STRUP_VSRAM_MD_PG_H2L_EN_SHIFT},
	{PMIC_RG_STRUP_VUFS18_PG_H2L_EN, PMIC_RG_STRUP_VUFS18_PG_H2L_EN_ADDR,
	 PMIC_RG_STRUP_VUFS18_PG_H2L_EN_MASK, PMIC_RG_STRUP_VUFS18_PG_H2L_EN_SHIFT},
	{PMIC_RG_STRUP_VEMC_PG_H2L_EN, PMIC_RG_STRUP_VEMC_PG_H2L_EN_ADDR,
	 PMIC_RG_STRUP_VEMC_PG_H2L_EN_MASK, PMIC_RG_STRUP_VEMC_PG_H2L_EN_SHIFT},
	{PMIC_RG_STRUP_VA12_PG_H2L_EN, PMIC_RG_STRUP_VA12_PG_H2L_EN_ADDR,
	 PMIC_RG_STRUP_VA12_PG_H2L_EN_MASK, PMIC_RG_STRUP_VA12_PG_H2L_EN_SHIFT},
	{PMIC_RG_STRUP_VA10_PG_H2L_EN, PMIC_RG_STRUP_VA10_PG_H2L_EN_ADDR,
	 PMIC_RG_STRUP_VA10_PG_H2L_EN_MASK, PMIC_RG_STRUP_VA10_PG_H2L_EN_SHIFT},
	{PMIC_RG_STRUP_VA18_PG_H2L_EN, PMIC_RG_STRUP_VA18_PG_H2L_EN_ADDR,
	 PMIC_RG_STRUP_VA18_PG_H2L_EN_MASK, PMIC_RG_STRUP_VA18_PG_H2L_EN_SHIFT},
	{PMIC_RG_STRUP_VDRAM_PG_H2L_EN, PMIC_RG_STRUP_VDRAM_PG_H2L_EN_ADDR,
	 PMIC_RG_STRUP_VDRAM_PG_H2L_EN_MASK, PMIC_RG_STRUP_VDRAM_PG_H2L_EN_SHIFT},
	{PMIC_RG_STRUP_VMODEM_PG_H2L_EN, PMIC_RG_STRUP_VMODEM_PG_H2L_EN_ADDR,
	 PMIC_RG_STRUP_VMODEM_PG_H2L_EN_MASK, PMIC_RG_STRUP_VMODEM_PG_H2L_EN_SHIFT},
	{PMIC_RG_STRUP_VMD1_PG_H2L_EN, PMIC_RG_STRUP_VMD1_PG_H2L_EN_ADDR,
	 PMIC_RG_STRUP_VMD1_PG_H2L_EN_MASK, PMIC_RG_STRUP_VMD1_PG_H2L_EN_SHIFT},
	{PMIC_RG_STRUP_VS2_PG_H2L_EN, PMIC_RG_STRUP_VS2_PG_H2L_EN_ADDR,
	 PMIC_RG_STRUP_VS2_PG_H2L_EN_MASK, PMIC_RG_STRUP_VS2_PG_H2L_EN_SHIFT},
	{PMIC_RG_STRUP_VS1_PG_H2L_EN, PMIC_RG_STRUP_VS1_PG_H2L_EN_ADDR,
	 PMIC_RG_STRUP_VS1_PG_H2L_EN_MASK, PMIC_RG_STRUP_VS1_PG_H2L_EN_SHIFT},
	{PMIC_RG_STRUP_VCORE_PG_H2L_EN, PMIC_RG_STRUP_VCORE_PG_H2L_EN_ADDR,
	 PMIC_RG_STRUP_VCORE_PG_H2L_EN_MASK, PMIC_RG_STRUP_VCORE_PG_H2L_EN_SHIFT},
	{PMIC_RG_STRUP_EXT_PMIC_PG_H2L_EN, PMIC_RG_STRUP_EXT_PMIC_PG_H2L_EN_ADDR,
	 PMIC_RG_STRUP_EXT_PMIC_PG_H2L_EN_MASK, PMIC_RG_STRUP_EXT_PMIC_PG_H2L_EN_SHIFT},
	{PMIC_RG_STRUP_VUSB33_PG_H2L_EN, PMIC_RG_STRUP_VUSB33_PG_H2L_EN_ADDR,
	 PMIC_RG_STRUP_VUSB33_PG_H2L_EN_MASK, PMIC_RG_STRUP_VUSB33_PG_H2L_EN_SHIFT},
	{PMIC_RG_STRUP_VSRAM_DVFS2_PG_ENB, PMIC_RG_STRUP_VSRAM_DVFS2_PG_ENB_ADDR,
	 PMIC_RG_STRUP_VSRAM_DVFS2_PG_ENB_MASK, PMIC_RG_STRUP_VSRAM_DVFS2_PG_ENB_SHIFT},
	{PMIC_RG_STRUP_VSRAM_DVFS1_PG_ENB, PMIC_RG_STRUP_VSRAM_DVFS1_PG_ENB_ADDR,
	 PMIC_RG_STRUP_VSRAM_DVFS1_PG_ENB_MASK, PMIC_RG_STRUP_VSRAM_DVFS1_PG_ENB_SHIFT},
	{PMIC_RG_STRUP_VSRAM_CORE_PG_ENB, PMIC_RG_STRUP_VSRAM_CORE_PG_ENB_ADDR,
	 PMIC_RG_STRUP_VSRAM_CORE_PG_ENB_MASK, PMIC_RG_STRUP_VSRAM_CORE_PG_ENB_SHIFT},
	{PMIC_RG_STRUP_VSRAM_GPU_PG_ENB, PMIC_RG_STRUP_VSRAM_GPU_PG_ENB_ADDR,
	 PMIC_RG_STRUP_VSRAM_GPU_PG_ENB_MASK, PMIC_RG_STRUP_VSRAM_GPU_PG_ENB_SHIFT},
	{PMIC_RG_STRUP_VSRAM_MD_PG_ENB, PMIC_RG_STRUP_VSRAM_MD_PG_ENB_ADDR,
	 PMIC_RG_STRUP_VSRAM_MD_PG_ENB_MASK, PMIC_RG_STRUP_VSRAM_MD_PG_ENB_SHIFT},
	{PMIC_RG_STRUP_VUFS18_PG_ENB, PMIC_RG_STRUP_VUFS18_PG_ENB_ADDR,
	 PMIC_RG_STRUP_VUFS18_PG_ENB_MASK, PMIC_RG_STRUP_VUFS18_PG_ENB_SHIFT},
	{PMIC_RG_STRUP_VEMC_PG_ENB, PMIC_RG_STRUP_VEMC_PG_ENB_ADDR, PMIC_RG_STRUP_VEMC_PG_ENB_MASK,
	 PMIC_RG_STRUP_VEMC_PG_ENB_SHIFT},
	{PMIC_RG_STRUP_VA12_PG_ENB, PMIC_RG_STRUP_VA12_PG_ENB_ADDR, PMIC_RG_STRUP_VA12_PG_ENB_MASK,
	 PMIC_RG_STRUP_VA12_PG_ENB_SHIFT},
	{PMIC_RG_STRUP_VA10_PG_ENB, PMIC_RG_STRUP_VA10_PG_ENB_ADDR, PMIC_RG_STRUP_VA10_PG_ENB_MASK,
	 PMIC_RG_STRUP_VA10_PG_ENB_SHIFT},
	{PMIC_RG_STRUP_VA18_PG_ENB, PMIC_RG_STRUP_VA18_PG_ENB_ADDR, PMIC_RG_STRUP_VA18_PG_ENB_MASK,
	 PMIC_RG_STRUP_VA18_PG_ENB_SHIFT},
	{PMIC_RG_STRUP_VDRAM_PG_ENB, PMIC_RG_STRUP_VDRAM_PG_ENB_ADDR,
	 PMIC_RG_STRUP_VDRAM_PG_ENB_MASK, PMIC_RG_STRUP_VDRAM_PG_ENB_SHIFT},
	{PMIC_RG_STRUP_VMODEM_PG_ENB, PMIC_RG_STRUP_VMODEM_PG_ENB_ADDR,
	 PMIC_RG_STRUP_VMODEM_PG_ENB_MASK, PMIC_RG_STRUP_VMODEM_PG_ENB_SHIFT},
	{PMIC_RG_STRUP_VMD1_PG_ENB, PMIC_RG_STRUP_VMD1_PG_ENB_ADDR, PMIC_RG_STRUP_VMD1_PG_ENB_MASK,
	 PMIC_RG_STRUP_VMD1_PG_ENB_SHIFT},
	{PMIC_RG_STRUP_VS2_PG_ENB, PMIC_RG_STRUP_VS2_PG_ENB_ADDR, PMIC_RG_STRUP_VS2_PG_ENB_MASK,
	 PMIC_RG_STRUP_VS2_PG_ENB_SHIFT},
	{PMIC_RG_STRUP_VS1_PG_ENB, PMIC_RG_STRUP_VS1_PG_ENB_ADDR, PMIC_RG_STRUP_VS1_PG_ENB_MASK,
	 PMIC_RG_STRUP_VS1_PG_ENB_SHIFT},
	{PMIC_RG_STRUP_VCORE_PG_ENB, PMIC_RG_STRUP_VCORE_PG_ENB_ADDR,
	 PMIC_RG_STRUP_VCORE_PG_ENB_MASK, PMIC_RG_STRUP_VCORE_PG_ENB_SHIFT},
	{PMIC_RG_STRUP_VXO22_PG_ENB, PMIC_RG_STRUP_VXO22_PG_ENB_ADDR,
	 PMIC_RG_STRUP_VXO22_PG_ENB_MASK, PMIC_RG_STRUP_VXO22_PG_ENB_SHIFT},
	{PMIC_RG_STRUP_VIO28_PG_ENB, PMIC_RG_STRUP_VIO28_PG_ENB_ADDR,
	 PMIC_RG_STRUP_VIO28_PG_ENB_MASK, PMIC_RG_STRUP_VIO28_PG_ENB_SHIFT},
	{PMIC_RG_STRUP_VIO18_PG_ENB, PMIC_RG_STRUP_VIO18_PG_ENB_ADDR,
	 PMIC_RG_STRUP_VIO18_PG_ENB_MASK, PMIC_RG_STRUP_VIO18_PG_ENB_SHIFT},
	{PMIC_RG_STRUP_EXT_PMIC_PG_ENB, PMIC_RG_STRUP_EXT_PMIC_PG_ENB_ADDR,
	 PMIC_RG_STRUP_EXT_PMIC_PG_ENB_MASK, PMIC_RG_STRUP_EXT_PMIC_PG_ENB_SHIFT},
	{PMIC_RG_STRUP_VUSB33_PG_ENB, PMIC_RG_STRUP_VUSB33_PG_ENB_ADDR,
	 PMIC_RG_STRUP_VUSB33_PG_ENB_MASK, PMIC_RG_STRUP_VUSB33_PG_ENB_SHIFT},
	{PMIC_RG_STRUP_VDRAM_OC_ENB, PMIC_RG_STRUP_VDRAM_OC_ENB_ADDR,
	 PMIC_RG_STRUP_VDRAM_OC_ENB_MASK, PMIC_RG_STRUP_VDRAM_OC_ENB_SHIFT},
	{PMIC_RG_STRUP_VMODEM_OC_ENB, PMIC_RG_STRUP_VMODEM_OC_ENB_ADDR,
	 PMIC_RG_STRUP_VMODEM_OC_ENB_MASK, PMIC_RG_STRUP_VMODEM_OC_ENB_SHIFT},
	{PMIC_RG_STRUP_VMD1_OC_ENB, PMIC_RG_STRUP_VMD1_OC_ENB_ADDR, PMIC_RG_STRUP_VMD1_OC_ENB_MASK,
	 PMIC_RG_STRUP_VMD1_OC_ENB_SHIFT},
	{PMIC_RG_STRUP_VS2_OC_ENB, PMIC_RG_STRUP_VS2_OC_ENB_ADDR, PMIC_RG_STRUP_VS2_OC_ENB_MASK,
	 PMIC_RG_STRUP_VS2_OC_ENB_SHIFT},
	{PMIC_RG_STRUP_VS1_OC_ENB, PMIC_RG_STRUP_VS1_OC_ENB_ADDR, PMIC_RG_STRUP_VS1_OC_ENB_MASK,
	 PMIC_RG_STRUP_VS1_OC_ENB_SHIFT},
	{PMIC_RG_STRUP_VCORE_OC_ENB, PMIC_RG_STRUP_VCORE_OC_ENB_ADDR,
	 PMIC_RG_STRUP_VCORE_OC_ENB_MASK, PMIC_RG_STRUP_VCORE_OC_ENB_SHIFT},
	{PMIC_RG_STRUP_LONG_PRESS_RESET_EXTEND, PMIC_RG_STRUP_LONG_PRESS_RESET_EXTEND_ADDR,
	 PMIC_RG_STRUP_LONG_PRESS_RESET_EXTEND_MASK, PMIC_RG_STRUP_LONG_PRESS_RESET_EXTEND_SHIFT},
	{PMIC_RG_EXT_PMIC_PG_DEBTD, PMIC_RG_EXT_PMIC_PG_DEBTD_ADDR, PMIC_RG_EXT_PMIC_PG_DEBTD_MASK,
	 PMIC_RG_EXT_PMIC_PG_DEBTD_SHIFT},
	{PMIC_RG_RTC_SPAR_DEB_EN, PMIC_RG_RTC_SPAR_DEB_EN_ADDR, PMIC_RG_RTC_SPAR_DEB_EN_MASK,
	 PMIC_RG_RTC_SPAR_DEB_EN_SHIFT},
	{PMIC_RG_RTC_ALARM_DEB_EN, PMIC_RG_RTC_ALARM_DEB_EN_ADDR, PMIC_RG_RTC_ALARM_DEB_EN_MASK,
	 PMIC_RG_RTC_ALARM_DEB_EN_SHIFT},
	{PMIC_RGS_ANA_CHIP_ID, PMIC_RGS_ANA_CHIP_ID_ADDR, PMIC_RGS_ANA_CHIP_ID_MASK,
	 PMIC_RGS_ANA_CHIP_ID_SHIFT},
	{PMIC_RG_PP_EN_DRVSEL, PMIC_RG_PP_EN_DRVSEL_ADDR, PMIC_RG_PP_EN_DRVSEL_MASK,
	 PMIC_RG_PP_EN_DRVSEL_SHIFT},
	{PMIC_RG_FCHR_PU_EN, PMIC_RG_FCHR_PU_EN_ADDR, PMIC_RG_FCHR_PU_EN_MASK,
	 PMIC_RG_FCHR_PU_EN_SHIFT},
	{PMIC_RG_PMU_RSV, PMIC_RG_PMU_RSV_ADDR, PMIC_RG_PMU_RSV_MASK, PMIC_RG_PMU_RSV_SHIFT},
	{PMIC_RG_FCHR_KEYDET_EN, PMIC_RG_FCHR_KEYDET_EN_ADDR, PMIC_RG_FCHR_KEYDET_EN_MASK,
	 PMIC_RG_FCHR_KEYDET_EN_SHIFT},
	{PMIC_RG_EN_DRVSEL, PMIC_RG_EN_DRVSEL_ADDR, PMIC_RG_EN_DRVSEL_MASK,
	 PMIC_RG_EN_DRVSEL_SHIFT},
	{PMIC_RG_RST_DRVSEL, PMIC_RG_RST_DRVSEL_ADDR, PMIC_RG_RST_DRVSEL_MASK,
	 PMIC_RG_RST_DRVSEL_SHIFT},
	{PMIC_RG_STRUP_IREF_TRIM, PMIC_RG_STRUP_IREF_TRIM_ADDR, PMIC_RG_STRUP_IREF_TRIM_MASK,
	 PMIC_RG_STRUP_IREF_TRIM_SHIFT},
	{PMIC_RG_VREF_BG, PMIC_RG_VREF_BG_ADDR, PMIC_RG_VREF_BG_MASK, PMIC_RG_VREF_BG_SHIFT},
	{PMIC_RG_THR_TMODE, PMIC_RG_THR_TMODE_ADDR, PMIC_RG_THR_TMODE_MASK,
	 PMIC_RG_THR_TMODE_SHIFT},
	{PMIC_RG_STRUP_THR_SEL, PMIC_RG_STRUP_THR_SEL_ADDR, PMIC_RG_STRUP_THR_SEL_MASK,
	 PMIC_RG_STRUP_THR_SEL_SHIFT},
	{PMIC_RG_THRDET_SEL, PMIC_RG_THRDET_SEL_ADDR, PMIC_RG_THRDET_SEL_MASK,
	 PMIC_RG_THRDET_SEL_SHIFT},
	{PMIC_RG_PWRHOLD, PMIC_RG_PWRHOLD_ADDR, PMIC_RG_PWRHOLD_MASK, PMIC_RG_PWRHOLD_SHIFT},
	{PMIC_RG_CRST, PMIC_RG_CRST_ADDR, PMIC_RG_CRST_MASK, PMIC_RG_CRST_SHIFT},
	{PMIC_RG_WRST, PMIC_RG_WRST_ADDR, PMIC_RG_WRST_MASK, PMIC_RG_WRST_SHIFT},
	{PMIC_RG_RSTB_ONINTV, PMIC_RG_RSTB_ONINTV_ADDR, PMIC_RG_RSTB_ONINTV_MASK,
	 PMIC_RG_RSTB_ONINTV_SHIFT},
	{PMIC_RG_CRST_INTV, PMIC_RG_CRST_INTV_ADDR, PMIC_RG_CRST_INTV_MASK,
	 PMIC_RG_CRST_INTV_SHIFT},
	{PMIC_RG_WRST_INTV, PMIC_RG_WRST_INTV_ADDR, PMIC_RG_WRST_INTV_MASK,
	 PMIC_RG_WRST_INTV_SHIFT},
	{PMIC_RG_PSEQ_IVGEN_SEL, PMIC_RG_PSEQ_IVGEN_SEL_ADDR, PMIC_RG_PSEQ_IVGEN_SEL_MASK,
	 PMIC_RG_PSEQ_IVGEN_SEL_SHIFT},
	{PMIC_RG_PSEQ_FSM_RST_SEL, PMIC_RG_PSEQ_FSM_RST_SEL_ADDR, PMIC_RG_PSEQ_FSM_RST_SEL_MASK,
	 PMIC_RG_PSEQ_FSM_RST_SEL_SHIFT},
	{PMIC_RG_PSEQ_PG_CK_SEL, PMIC_RG_PSEQ_PG_CK_SEL_ADDR, PMIC_RG_PSEQ_PG_CK_SEL_MASK,
	 PMIC_RG_PSEQ_PG_CK_SEL_SHIFT},
	{PMIC_RG_PSEQ_1MS_TK_EXT, PMIC_RG_PSEQ_1MS_TK_EXT_ADDR, PMIC_RG_PSEQ_1MS_TK_EXT_MASK,
	 PMIC_RG_PSEQ_1MS_TK_EXT_SHIFT},
	{PMIC_RG_PSEQ_SPAR_XCPT_MASK, PMIC_RG_PSEQ_SPAR_XCPT_MASK_ADDR,
	 PMIC_RG_PSEQ_SPAR_XCPT_MASK_MASK, PMIC_RG_PSEQ_SPAR_XCPT_MASK_SHIFT},
	{PMIC_RG_PSEQ_RTCA_XCPT_MASK, PMIC_RG_PSEQ_RTCA_XCPT_MASK_ADDR,
	 PMIC_RG_PSEQ_RTCA_XCPT_MASK_MASK, PMIC_RG_PSEQ_RTCA_XCPT_MASK_SHIFT},
	{PMIC_RG_WDTRST_EN, PMIC_RG_WDTRST_EN_ADDR, PMIC_RG_WDTRST_EN_MASK,
	 PMIC_RG_WDTRST_EN_SHIFT},
	{PMIC_RG_WDTRST_ACT, PMIC_RG_WDTRST_ACT_ADDR, PMIC_RG_WDTRST_ACT_MASK,
	 PMIC_RG_WDTRST_ACT_SHIFT},
	{PMIC_RG_PSPG_SHDN_EN, PMIC_RG_PSPG_SHDN_EN_ADDR, PMIC_RG_PSPG_SHDN_EN_MASK,
	 PMIC_RG_PSPG_SHDN_EN_SHIFT},
	{PMIC_RG_THM_SHDN_EN, PMIC_RG_THM_SHDN_EN_ADDR, PMIC_RG_THM_SHDN_EN_MASK,
	 PMIC_RG_THM_SHDN_EN_SHIFT},
	{PMIC_RG_PSEQ_FORCE_ON, PMIC_RG_PSEQ_FORCE_ON_ADDR, PMIC_RG_PSEQ_FORCE_ON_MASK,
	 PMIC_RG_PSEQ_FORCE_ON_SHIFT},
	{PMIC_RG_PSEQ_FORCE_ALL_DOFF, PMIC_RG_PSEQ_FORCE_ALL_DOFF_ADDR,
	 PMIC_RG_PSEQ_FORCE_ALL_DOFF_MASK, PMIC_RG_PSEQ_FORCE_ALL_DOFF_SHIFT},
	{PMIC_RG_PSEQ_F75K_FORCE, PMIC_RG_PSEQ_F75K_FORCE_ADDR, PMIC_RG_PSEQ_F75K_FORCE_MASK,
	 PMIC_RG_PSEQ_F75K_FORCE_SHIFT},
	{PMIC_RG_PSEQ_RSV0, PMIC_RG_PSEQ_RSV0_ADDR, PMIC_RG_PSEQ_RSV0_MASK,
	 PMIC_RG_PSEQ_RSV0_SHIFT},
	{PMIC_RG_PSEQ_RSV1, PMIC_RG_PSEQ_RSV1_ADDR, PMIC_RG_PSEQ_RSV1_MASK,
	 PMIC_RG_PSEQ_RSV1_SHIFT},
	{PMIC_RG_PSEQ_RSV2, PMIC_RG_PSEQ_RSV2_ADDR, PMIC_RG_PSEQ_RSV2_MASK,
	 PMIC_RG_PSEQ_RSV2_SHIFT},
	{PMIC_RG_BWDT_EN, PMIC_RG_BWDT_EN_ADDR, PMIC_RG_BWDT_EN_MASK, PMIC_RG_BWDT_EN_SHIFT},
	{PMIC_RG_BWDT_TD, PMIC_RG_BWDT_TD_ADDR, PMIC_RG_BWDT_TD_MASK, PMIC_RG_BWDT_TD_SHIFT},
	{PMIC_RG_BWDT_CHRTD, PMIC_RG_BWDT_CHRTD_ADDR, PMIC_RG_BWDT_CHRTD_MASK,
	 PMIC_RG_BWDT_CHRTD_SHIFT},
	{PMIC_RG_BWDT_DDLO_TD, PMIC_RG_BWDT_DDLO_TD_ADDR, PMIC_RG_BWDT_DDLO_TD_MASK,
	 PMIC_RG_BWDT_DDLO_TD_SHIFT},
	{PMIC_RG_BWDT_SRCSEL, PMIC_RG_BWDT_SRCSEL_ADDR, PMIC_RG_BWDT_SRCSEL_MASK,
	 PMIC_RG_BWDT_SRCSEL_SHIFT},
	{PMIC_RG_CPS_W_KEY, PMIC_RG_CPS_W_KEY_ADDR, PMIC_RG_CPS_W_KEY_MASK,
	 PMIC_RG_CPS_W_KEY_SHIFT},
	{PMIC_RG_SLOT_INTV_UP, PMIC_RG_SLOT_INTV_UP_ADDR, PMIC_RG_SLOT_INTV_UP_MASK,
	 PMIC_RG_SLOT_INTV_UP_SHIFT},
	{PMIC_RG_SEQ_LEN, PMIC_RG_SEQ_LEN_ADDR, PMIC_RG_SEQ_LEN_MASK, PMIC_RG_SEQ_LEN_SHIFT},
	{PMIC_RG_SLOT_INTV_DOWN, PMIC_RG_SLOT_INTV_DOWN_ADDR, PMIC_RG_SLOT_INTV_DOWN_MASK,
	 PMIC_RG_SLOT_INTV_DOWN_SHIFT},
	{PMIC_RG_DSEQ_LEN, PMIC_RG_DSEQ_LEN_ADDR, PMIC_RG_DSEQ_LEN_MASK, PMIC_RG_DSEQ_LEN_SHIFT},
	{PMIC_RG_VCORE_USA, PMIC_RG_VCORE_USA_ADDR, PMIC_RG_VCORE_USA_MASK,
	 PMIC_RG_VCORE_USA_SHIFT},
	{PMIC_RG_VS1_USA, PMIC_RG_VS1_USA_ADDR, PMIC_RG_VS1_USA_MASK, PMIC_RG_VS1_USA_SHIFT},
	{PMIC_RG_VS2_USA, PMIC_RG_VS2_USA_ADDR, PMIC_RG_VS2_USA_MASK, PMIC_RG_VS2_USA_SHIFT},
	{PMIC_RG_VMD1_USA, PMIC_RG_VMD1_USA_ADDR, PMIC_RG_VMD1_USA_MASK, PMIC_RG_VMD1_USA_SHIFT},
	{PMIC_RG_VMODEM_USA, PMIC_RG_VMODEM_USA_ADDR, PMIC_RG_VMODEM_USA_MASK,
	 PMIC_RG_VMODEM_USA_SHIFT},
	{PMIC_RG_VDRAM_USA, PMIC_RG_VDRAM_USA_ADDR, PMIC_RG_VDRAM_USA_MASK,
	 PMIC_RG_VDRAM_USA_SHIFT},
	{PMIC_RG_VA10_USA, PMIC_RG_VA10_USA_ADDR, PMIC_RG_VA10_USA_MASK, PMIC_RG_VA10_USA_SHIFT},
	{PMIC_RG_VA12_USA, PMIC_RG_VA12_USA_ADDR, PMIC_RG_VA12_USA_MASK, PMIC_RG_VA12_USA_SHIFT},
	{PMIC_RG_VEMC_USA, PMIC_RG_VEMC_USA_ADDR, PMIC_RG_VEMC_USA_MASK, PMIC_RG_VEMC_USA_SHIFT},
	{PMIC_RG_VUFS18_USA, PMIC_RG_VUFS18_USA_ADDR, PMIC_RG_VUFS18_USA_MASK,
	 PMIC_RG_VUFS18_USA_SHIFT},
	{PMIC_RG_VSRAM_MD_USA, PMIC_RG_VSRAM_MD_USA_ADDR, PMIC_RG_VSRAM_MD_USA_MASK,
	 PMIC_RG_VSRAM_MD_USA_SHIFT},
	{PMIC_RG_VSRAM_GPU_USA, PMIC_RG_VSRAM_GPU_USA_ADDR, PMIC_RG_VSRAM_GPU_USA_MASK,
	 PMIC_RG_VSRAM_GPU_USA_SHIFT},
	{PMIC_RG_VSRAM_CORE_USA, PMIC_RG_VSRAM_CORE_USA_ADDR, PMIC_RG_VSRAM_CORE_USA_MASK,
	 PMIC_RG_VSRAM_CORE_USA_SHIFT},
	{PMIC_RG_VSRAM_DVFS1_USA, PMIC_RG_VSRAM_DVFS1_USA_ADDR, PMIC_RG_VSRAM_DVFS1_USA_MASK,
	 PMIC_RG_VSRAM_DVFS1_USA_SHIFT},
	{PMIC_RG_VSRAM_DVFS2_USA, PMIC_RG_VSRAM_DVFS2_USA_ADDR, PMIC_RG_VSRAM_DVFS2_USA_MASK,
	 PMIC_RG_VSRAM_DVFS2_USA_SHIFT},
	{PMIC_RG_VUSB33_USA, PMIC_RG_VUSB33_USA_ADDR, PMIC_RG_VUSB33_USA_MASK,
	 PMIC_RG_VUSB33_USA_SHIFT},
	{PMIC_RG_EXTPMIC_USA, PMIC_RG_EXTPMIC_USA_ADDR, PMIC_RG_EXTPMIC_USA_MASK,
	 PMIC_RG_EXTPMIC_USA_SHIFT},
	{PMIC_RG_VIO18_USA, PMIC_RG_VIO18_USA_ADDR, PMIC_RG_VIO18_USA_MASK,
	 PMIC_RG_VIO18_USA_SHIFT},
	{PMIC_RG_VIO28_USA, PMIC_RG_VIO28_USA_ADDR, PMIC_RG_VIO28_USA_MASK,
	 PMIC_RG_VIO28_USA_SHIFT},
	{PMIC_RG_VXO22_USA, PMIC_RG_VXO22_USA_ADDR, PMIC_RG_VXO22_USA_MASK,
	 PMIC_RG_VXO22_USA_SHIFT},
	{PMIC_RG_VRSV0_USA, PMIC_RG_VRSV0_USA_ADDR, PMIC_RG_VRSV0_USA_MASK,
	 PMIC_RG_VRSV0_USA_SHIFT},
	{PMIC_RG_VRSV1_USA, PMIC_RG_VRSV1_USA_ADDR, PMIC_RG_VRSV1_USA_MASK,
	 PMIC_RG_VRSV1_USA_SHIFT},
	{PMIC_RG_VCORE_DSA, PMIC_RG_VCORE_DSA_ADDR, PMIC_RG_VCORE_DSA_MASK,
	 PMIC_RG_VCORE_DSA_SHIFT},
	{PMIC_RG_VS1_DSA, PMIC_RG_VS1_DSA_ADDR, PMIC_RG_VS1_DSA_MASK, PMIC_RG_VS1_DSA_SHIFT},
	{PMIC_RG_VS2_DSA, PMIC_RG_VS2_DSA_ADDR, PMIC_RG_VS2_DSA_MASK, PMIC_RG_VS2_DSA_SHIFT},
	{PMIC_RG_VMD1_DSA, PMIC_RG_VMD1_DSA_ADDR, PMIC_RG_VMD1_DSA_MASK, PMIC_RG_VMD1_DSA_SHIFT},
	{PMIC_RG_VMODEM_DSA, PMIC_RG_VMODEM_DSA_ADDR, PMIC_RG_VMODEM_DSA_MASK,
	 PMIC_RG_VMODEM_DSA_SHIFT},
	{PMIC_RG_VDRAM_DSA, PMIC_RG_VDRAM_DSA_ADDR, PMIC_RG_VDRAM_DSA_MASK,
	 PMIC_RG_VDRAM_DSA_SHIFT},
	{PMIC_RG_VA10_DSA, PMIC_RG_VA10_DSA_ADDR, PMIC_RG_VA10_DSA_MASK, PMIC_RG_VA10_DSA_SHIFT},
	{PMIC_RG_VA12_DSA, PMIC_RG_VA12_DSA_ADDR, PMIC_RG_VA12_DSA_MASK, PMIC_RG_VA12_DSA_SHIFT},
	{PMIC_RG_VEMC_DSA, PMIC_RG_VEMC_DSA_ADDR, PMIC_RG_VEMC_DSA_MASK, PMIC_RG_VEMC_DSA_SHIFT},
	{PMIC_RG_VUFS18_DSA, PMIC_RG_VUFS18_DSA_ADDR, PMIC_RG_VUFS18_DSA_MASK,
	 PMIC_RG_VUFS18_DSA_SHIFT},
	{PMIC_RG_VSRAM_MD_DSA, PMIC_RG_VSRAM_MD_DSA_ADDR, PMIC_RG_VSRAM_MD_DSA_MASK,
	 PMIC_RG_VSRAM_MD_DSA_SHIFT},
	{PMIC_RG_VSRAM_GPU_DSA, PMIC_RG_VSRAM_GPU_DSA_ADDR, PMIC_RG_VSRAM_GPU_DSA_MASK,
	 PMIC_RG_VSRAM_GPU_DSA_SHIFT},
	{PMIC_RG_VSRAM_CORE_DSA, PMIC_RG_VSRAM_CORE_DSA_ADDR, PMIC_RG_VSRAM_CORE_DSA_MASK,
	 PMIC_RG_VSRAM_CORE_DSA_SHIFT},
	{PMIC_RG_VSRAM_DVFS1_DSA, PMIC_RG_VSRAM_DVFS1_DSA_ADDR, PMIC_RG_VSRAM_DVFS1_DSA_MASK,
	 PMIC_RG_VSRAM_DVFS1_DSA_SHIFT},
	{PMIC_RG_VSRAM_DVFS2_DSA, PMIC_RG_VSRAM_DVFS2_DSA_ADDR, PMIC_RG_VSRAM_DVFS2_DSA_MASK,
	 PMIC_RG_VSRAM_DVFS2_DSA_SHIFT},
	{PMIC_RG_VUSB33_DSA, PMIC_RG_VUSB33_DSA_ADDR, PMIC_RG_VUSB33_DSA_MASK,
	 PMIC_RG_VUSB33_DSA_SHIFT},
	{PMIC_RG_EXTPMIC_DSA, PMIC_RG_EXTPMIC_DSA_ADDR, PMIC_RG_EXTPMIC_DSA_MASK,
	 PMIC_RG_EXTPMIC_DSA_SHIFT},
	{PMIC_RG_VIO18_DSA, PMIC_RG_VIO18_DSA_ADDR, PMIC_RG_VIO18_DSA_MASK,
	 PMIC_RG_VIO18_DSA_SHIFT},
	{PMIC_RG_VIO28_DSA, PMIC_RG_VIO28_DSA_ADDR, PMIC_RG_VIO28_DSA_MASK,
	 PMIC_RG_VIO28_DSA_SHIFT},
	{PMIC_RG_VXO22_DSA, PMIC_RG_VXO22_DSA_ADDR, PMIC_RG_VXO22_DSA_MASK,
	 PMIC_RG_VXO22_DSA_SHIFT},
	{PMIC_RG_VRSV0_DSA, PMIC_RG_VRSV0_DSA_ADDR, PMIC_RG_VRSV0_DSA_MASK,
	 PMIC_RG_VRSV0_DSA_SHIFT},
	{PMIC_RG_VRSV1_DSA, PMIC_RG_VRSV1_DSA_ADDR, PMIC_RG_VRSV1_DSA_MASK,
	 PMIC_RG_VRSV1_DSA_SHIFT},
	{PMIC_RG_POR_FLAG, PMIC_RG_POR_FLAG_ADDR, PMIC_RG_POR_FLAG_MASK, PMIC_RG_POR_FLAG_SHIFT},
	{PMIC_STS_PWRKEY, PMIC_STS_PWRKEY_ADDR, PMIC_STS_PWRKEY_MASK, PMIC_STS_PWRKEY_SHIFT},
	{PMIC_STS_RTCA, PMIC_STS_RTCA_ADDR, PMIC_STS_RTCA_MASK, PMIC_STS_RTCA_SHIFT},
	{PMIC_STS_CHRIN, PMIC_STS_CHRIN_ADDR, PMIC_STS_CHRIN_MASK, PMIC_STS_CHRIN_SHIFT},
	{PMIC_STS_SPAR, PMIC_STS_SPAR_ADDR, PMIC_STS_SPAR_MASK, PMIC_STS_SPAR_SHIFT},
	{PMIC_STS_RBOOT, PMIC_STS_RBOOT_ADDR, PMIC_STS_RBOOT_MASK, PMIC_STS_RBOOT_SHIFT},
	{PMIC_STS_UVLO, PMIC_STS_UVLO_ADDR, PMIC_STS_UVLO_MASK, PMIC_STS_UVLO_SHIFT},
	{PMIC_STS_PGFAIL, PMIC_STS_PGFAIL_ADDR, PMIC_STS_PGFAIL_MASK, PMIC_STS_PGFAIL_SHIFT},
	{PMIC_STS_PSOC, PMIC_STS_PSOC_ADDR, PMIC_STS_PSOC_MASK, PMIC_STS_PSOC_SHIFT},
	{PMIC_STS_THRDN, PMIC_STS_THRDN_ADDR, PMIC_STS_THRDN_MASK, PMIC_STS_THRDN_SHIFT},
	{PMIC_STS_WRST, PMIC_STS_WRST_ADDR, PMIC_STS_WRST_MASK, PMIC_STS_WRST_SHIFT},
	{PMIC_STS_CRST, PMIC_STS_CRST_ADDR, PMIC_STS_CRST_MASK, PMIC_STS_CRST_SHIFT},
	{PMIC_STS_PKEYLP, PMIC_STS_PKEYLP_ADDR, PMIC_STS_PKEYLP_MASK, PMIC_STS_PKEYLP_SHIFT},
	{PMIC_STS_NORMOFF, PMIC_STS_NORMOFF_ADDR, PMIC_STS_NORMOFF_MASK, PMIC_STS_NORMOFF_SHIFT},
	{PMIC_STS_BWDT, PMIC_STS_BWDT_ADDR, PMIC_STS_BWDT_MASK, PMIC_STS_BWDT_SHIFT},
	{PMIC_STS_DDLO, PMIC_STS_DDLO_ADDR, PMIC_STS_DDLO_MASK, PMIC_STS_DDLO_SHIFT},
	{PMIC_STS_WDT, PMIC_STS_WDT_ADDR, PMIC_STS_WDT_MASK, PMIC_STS_WDT_SHIFT},
	{PMIC_STS_PUPSRC, PMIC_STS_PUPSRC_ADDR, PMIC_STS_PUPSRC_MASK, PMIC_STS_PUPSRC_SHIFT},
	{PMIC_RG_POFFSTS_CLR, PMIC_RG_POFFSTS_CLR_ADDR, PMIC_RG_POFFSTS_CLR_MASK,
	 PMIC_RG_POFFSTS_CLR_SHIFT},
	{PMIC_RG_PONSTS_CLR, PMIC_RG_PONSTS_CLR_ADDR, PMIC_RG_PONSTS_CLR_MASK,
	 PMIC_RG_PONSTS_CLR_SHIFT},
	{PMIC_HWCID, PMIC_HWCID_ADDR, PMIC_HWCID_MASK, PMIC_HWCID_SHIFT},
	{PMIC_SWCID, PMIC_SWCID_ADDR, PMIC_SWCID_MASK, PMIC_SWCID_SHIFT},
	{PMIC_RG_SRCLKEN_IN0_EN, PMIC_RG_SRCLKEN_IN0_EN_ADDR, PMIC_RG_SRCLKEN_IN0_EN_MASK,
	 PMIC_RG_SRCLKEN_IN0_EN_SHIFT},
	{PMIC_RG_SRCLKEN_IN1_EN, PMIC_RG_SRCLKEN_IN1_EN_ADDR, PMIC_RG_SRCLKEN_IN1_EN_MASK,
	 PMIC_RG_SRCLKEN_IN1_EN_SHIFT},
	{PMIC_RG_OSC_SEL, PMIC_RG_OSC_SEL_ADDR, PMIC_RG_OSC_SEL_MASK, PMIC_RG_OSC_SEL_SHIFT},
	{PMIC_RG_SRCLKEN_IN0_HW_MODE, PMIC_RG_SRCLKEN_IN0_HW_MODE_ADDR,
	 PMIC_RG_SRCLKEN_IN0_HW_MODE_MASK, PMIC_RG_SRCLKEN_IN0_HW_MODE_SHIFT},
	{PMIC_RG_SRCLKEN_IN1_HW_MODE, PMIC_RG_SRCLKEN_IN1_HW_MODE_ADDR,
	 PMIC_RG_SRCLKEN_IN1_HW_MODE_MASK, PMIC_RG_SRCLKEN_IN1_HW_MODE_SHIFT},
	{PMIC_RG_OSC_SEL_HW_MODE, PMIC_RG_OSC_SEL_HW_MODE_ADDR, PMIC_RG_OSC_SEL_HW_MODE_MASK,
	 PMIC_RG_OSC_SEL_HW_MODE_SHIFT},
	{PMIC_RG_SRCLKEN_IN_SYNC_EN, PMIC_RG_SRCLKEN_IN_SYNC_EN_ADDR,
	 PMIC_RG_SRCLKEN_IN_SYNC_EN_MASK, PMIC_RG_SRCLKEN_IN_SYNC_EN_SHIFT},
	{PMIC_RG_OSC_EN_AUTO_OFF, PMIC_RG_OSC_EN_AUTO_OFF_ADDR, PMIC_RG_OSC_EN_AUTO_OFF_MASK,
	 PMIC_RG_OSC_EN_AUTO_OFF_SHIFT},
	{PMIC_TEST_OUT, PMIC_TEST_OUT_ADDR, PMIC_TEST_OUT_MASK, PMIC_TEST_OUT_SHIFT},
	{PMIC_RG_MON_FLAG_SEL, PMIC_RG_MON_FLAG_SEL_ADDR, PMIC_RG_MON_FLAG_SEL_MASK,
	 PMIC_RG_MON_FLAG_SEL_SHIFT},
	{PMIC_RG_MON_GRP_SEL, PMIC_RG_MON_GRP_SEL_ADDR, PMIC_RG_MON_GRP_SEL_MASK,
	 PMIC_RG_MON_GRP_SEL_SHIFT},
	{PMIC_RG_NANDTREE_MODE, PMIC_RG_NANDTREE_MODE_ADDR, PMIC_RG_NANDTREE_MODE_MASK,
	 PMIC_RG_NANDTREE_MODE_SHIFT},
	{PMIC_RG_TEST_AUXADC, PMIC_RG_TEST_AUXADC_ADDR, PMIC_RG_TEST_AUXADC_MASK,
	 PMIC_RG_TEST_AUXADC_SHIFT},
	{PMIC_RG_EFUSE_MODE, PMIC_RG_EFUSE_MODE_ADDR, PMIC_RG_EFUSE_MODE_MASK,
	 PMIC_RG_EFUSE_MODE_SHIFT},
	{PMIC_RG_TEST_STRUP, PMIC_RG_TEST_STRUP_ADDR, PMIC_RG_TEST_STRUP_MASK,
	 PMIC_RG_TEST_STRUP_SHIFT},
	{PMIC_TESTMODE_SW, PMIC_TESTMODE_SW_ADDR, PMIC_TESTMODE_SW_MASK, PMIC_TESTMODE_SW_SHIFT},
	{PMIC_VSRAM_DVFS2_PG_DEB, PMIC_VSRAM_DVFS2_PG_DEB_ADDR, PMIC_VSRAM_DVFS2_PG_DEB_MASK,
	 PMIC_VSRAM_DVFS2_PG_DEB_SHIFT},
	{PMIC_VSRAM_DVFS1_PG_DEB, PMIC_VSRAM_DVFS1_PG_DEB_ADDR, PMIC_VSRAM_DVFS1_PG_DEB_MASK,
	 PMIC_VSRAM_DVFS1_PG_DEB_SHIFT},
	{PMIC_VSRAM_CORE_PG_DEB, PMIC_VSRAM_CORE_PG_DEB_ADDR, PMIC_VSRAM_CORE_PG_DEB_MASK,
	 PMIC_VSRAM_CORE_PG_DEB_SHIFT},
	{PMIC_VSRAM_GPU_PG_DEB, PMIC_VSRAM_GPU_PG_DEB_ADDR, PMIC_VSRAM_GPU_PG_DEB_MASK,
	 PMIC_VSRAM_GPU_PG_DEB_SHIFT},
	{PMIC_VSRAM_MD_PG_DEB, PMIC_VSRAM_MD_PG_DEB_ADDR, PMIC_VSRAM_MD_PG_DEB_MASK,
	 PMIC_VSRAM_MD_PG_DEB_SHIFT},
	{PMIC_VUFS18_PG_DEB, PMIC_VUFS18_PG_DEB_ADDR, PMIC_VUFS18_PG_DEB_MASK,
	 PMIC_VUFS18_PG_DEB_SHIFT},
	{PMIC_VEMC_PG_DEB, PMIC_VEMC_PG_DEB_ADDR, PMIC_VEMC_PG_DEB_MASK, PMIC_VEMC_PG_DEB_SHIFT},
	{PMIC_VA12_PG_DEB, PMIC_VA12_PG_DEB_ADDR, PMIC_VA12_PG_DEB_MASK, PMIC_VA12_PG_DEB_SHIFT},
	{PMIC_VA10_PG_DEB, PMIC_VA10_PG_DEB_ADDR, PMIC_VA10_PG_DEB_MASK, PMIC_VA10_PG_DEB_SHIFT},
	{PMIC_VA18_PG_DEB, PMIC_VA18_PG_DEB_ADDR, PMIC_VA18_PG_DEB_MASK, PMIC_VA18_PG_DEB_SHIFT},
	{PMIC_VDRAM_PG_DEB, PMIC_VDRAM_PG_DEB_ADDR, PMIC_VDRAM_PG_DEB_MASK,
	 PMIC_VDRAM_PG_DEB_SHIFT},
	{PMIC_VMODEM_PG_DEB, PMIC_VMODEM_PG_DEB_ADDR, PMIC_VMODEM_PG_DEB_MASK,
	 PMIC_VMODEM_PG_DEB_SHIFT},
	{PMIC_VMD1_PG_DEB, PMIC_VMD1_PG_DEB_ADDR, PMIC_VMD1_PG_DEB_MASK, PMIC_VMD1_PG_DEB_SHIFT},
	{PMIC_VS2_PG_DEB, PMIC_VS2_PG_DEB_ADDR, PMIC_VS2_PG_DEB_MASK, PMIC_VS2_PG_DEB_SHIFT},
	{PMIC_VS1_PG_DEB, PMIC_VS1_PG_DEB_ADDR, PMIC_VS1_PG_DEB_MASK, PMIC_VS1_PG_DEB_SHIFT},
	{PMIC_VCORE_PG_DEB, PMIC_VCORE_PG_DEB_ADDR, PMIC_VCORE_PG_DEB_MASK,
	 PMIC_VCORE_PG_DEB_SHIFT},
	{PMIC_VXO22_PG_DEB, PMIC_VXO22_PG_DEB_ADDR, PMIC_VXO22_PG_DEB_MASK,
	 PMIC_VXO22_PG_DEB_SHIFT},
	{PMIC_VIO28_PG_DEB, PMIC_VIO28_PG_DEB_ADDR, PMIC_VIO28_PG_DEB_MASK,
	 PMIC_VIO28_PG_DEB_SHIFT},
	{PMIC_VIO18_PG_DEB, PMIC_VIO18_PG_DEB_ADDR, PMIC_VIO18_PG_DEB_MASK,
	 PMIC_VIO18_PG_DEB_SHIFT},
	{PMIC_EXT_PMIC_PG_DEB, PMIC_EXT_PMIC_PG_DEB_ADDR, PMIC_EXT_PMIC_PG_DEB_MASK,
	 PMIC_EXT_PMIC_PG_DEB_SHIFT},
	{PMIC_VUSB33_PG_DEB, PMIC_VUSB33_PG_DEB_ADDR, PMIC_VUSB33_PG_DEB_MASK,
	 PMIC_VUSB33_PG_DEB_SHIFT},
	{PMIC_STRUP_VSRAM_DVFS2_PG_STATUS, PMIC_STRUP_VSRAM_DVFS2_PG_STATUS_ADDR,
	 PMIC_STRUP_VSRAM_DVFS2_PG_STATUS_MASK, PMIC_STRUP_VSRAM_DVFS2_PG_STATUS_SHIFT},
	{PMIC_STRUP_VSRAM_DVFS1_PG_STATUS, PMIC_STRUP_VSRAM_DVFS1_PG_STATUS_ADDR,
	 PMIC_STRUP_VSRAM_DVFS1_PG_STATUS_MASK, PMIC_STRUP_VSRAM_DVFS1_PG_STATUS_SHIFT},
	{PMIC_STRUP_VSRAM_CORE_PG_STATUS, PMIC_STRUP_VSRAM_CORE_PG_STATUS_ADDR,
	 PMIC_STRUP_VSRAM_CORE_PG_STATUS_MASK, PMIC_STRUP_VSRAM_CORE_PG_STATUS_SHIFT},
	{PMIC_STRUP_VSRAM_GPU_PG_STATUS, PMIC_STRUP_VSRAM_GPU_PG_STATUS_ADDR,
	 PMIC_STRUP_VSRAM_GPU_PG_STATUS_MASK, PMIC_STRUP_VSRAM_GPU_PG_STATUS_SHIFT},
	{PMIC_STRUP_VSRAM_MD_PG_STATUS, PMIC_STRUP_VSRAM_MD_PG_STATUS_ADDR,
	 PMIC_STRUP_VSRAM_MD_PG_STATUS_MASK, PMIC_STRUP_VSRAM_MD_PG_STATUS_SHIFT},
	{PMIC_STRUP_VUFS18_PG_STATUS, PMIC_STRUP_VUFS18_PG_STATUS_ADDR,
	 PMIC_STRUP_VUFS18_PG_STATUS_MASK, PMIC_STRUP_VUFS18_PG_STATUS_SHIFT},
	{PMIC_STRUP_VEMC_PG_STATUS, PMIC_STRUP_VEMC_PG_STATUS_ADDR, PMIC_STRUP_VEMC_PG_STATUS_MASK,
	 PMIC_STRUP_VEMC_PG_STATUS_SHIFT},
	{PMIC_STRUP_VA12_PG_STATUS, PMIC_STRUP_VA12_PG_STATUS_ADDR, PMIC_STRUP_VA12_PG_STATUS_MASK,
	 PMIC_STRUP_VA12_PG_STATUS_SHIFT},
	{PMIC_STRUP_VA10_PG_STATUS, PMIC_STRUP_VA10_PG_STATUS_ADDR, PMIC_STRUP_VA10_PG_STATUS_MASK,
	 PMIC_STRUP_VA10_PG_STATUS_SHIFT},
	{PMIC_STRUP_VA18_PG_STATUS, PMIC_STRUP_VA18_PG_STATUS_ADDR, PMIC_STRUP_VA18_PG_STATUS_MASK,
	 PMIC_STRUP_VA18_PG_STATUS_SHIFT},
	{PMIC_STRUP_VDRAM_PG_STATUS, PMIC_STRUP_VDRAM_PG_STATUS_ADDR,
	 PMIC_STRUP_VDRAM_PG_STATUS_MASK, PMIC_STRUP_VDRAM_PG_STATUS_SHIFT},
	{PMIC_STRUP_VMODEM_PG_STATUS, PMIC_STRUP_VMODEM_PG_STATUS_ADDR,
	 PMIC_STRUP_VMODEM_PG_STATUS_MASK, PMIC_STRUP_VMODEM_PG_STATUS_SHIFT},
	{PMIC_STRUP_VMD1_PG_STATUS, PMIC_STRUP_VMD1_PG_STATUS_ADDR, PMIC_STRUP_VMD1_PG_STATUS_MASK,
	 PMIC_STRUP_VMD1_PG_STATUS_SHIFT},
	{PMIC_STRUP_VS2_PG_STATUS, PMIC_STRUP_VS2_PG_STATUS_ADDR, PMIC_STRUP_VS2_PG_STATUS_MASK,
	 PMIC_STRUP_VS2_PG_STATUS_SHIFT},
	{PMIC_STRUP_VS1_PG_STATUS, PMIC_STRUP_VS1_PG_STATUS_ADDR, PMIC_STRUP_VS1_PG_STATUS_MASK,
	 PMIC_STRUP_VS1_PG_STATUS_SHIFT},
	{PMIC_STRUP_VCORE_PG_STATUS, PMIC_STRUP_VCORE_PG_STATUS_ADDR,
	 PMIC_STRUP_VCORE_PG_STATUS_MASK, PMIC_STRUP_VCORE_PG_STATUS_SHIFT},
	{PMIC_STRUP_VXO22_PG_STATUS, PMIC_STRUP_VXO22_PG_STATUS_ADDR,
	 PMIC_STRUP_VXO22_PG_STATUS_MASK, PMIC_STRUP_VXO22_PG_STATUS_SHIFT},
	{PMIC_STRUP_VIO28_PG_STATUS, PMIC_STRUP_VIO28_PG_STATUS_ADDR,
	 PMIC_STRUP_VIO28_PG_STATUS_MASK, PMIC_STRUP_VIO28_PG_STATUS_SHIFT},
	{PMIC_STRUP_VIO18_PG_STATUS, PMIC_STRUP_VIO18_PG_STATUS_ADDR,
	 PMIC_STRUP_VIO18_PG_STATUS_MASK, PMIC_STRUP_VIO18_PG_STATUS_SHIFT},
	{PMIC_STRUP_EXT_PMIC_PG_STATUS, PMIC_STRUP_EXT_PMIC_PG_STATUS_ADDR,
	 PMIC_STRUP_EXT_PMIC_PG_STATUS_MASK, PMIC_STRUP_EXT_PMIC_PG_STATUS_SHIFT},
	{PMIC_STRUP_VUSB33_PG_STATUS, PMIC_STRUP_VUSB33_PG_STATUS_ADDR,
	 PMIC_STRUP_VUSB33_PG_STATUS_MASK, PMIC_STRUP_VUSB33_PG_STATUS_SHIFT},
	{PMIC_STRUP_VDRAM_OC_STATUS, PMIC_STRUP_VDRAM_OC_STATUS_ADDR,
	 PMIC_STRUP_VDRAM_OC_STATUS_MASK, PMIC_STRUP_VDRAM_OC_STATUS_SHIFT},
	{PMIC_STRUP_VMODEM_OC_STATUS, PMIC_STRUP_VMODEM_OC_STATUS_ADDR,
	 PMIC_STRUP_VMODEM_OC_STATUS_MASK, PMIC_STRUP_VMODEM_OC_STATUS_SHIFT},
	{PMIC_STRUP_VMD1_OC_STATUS, PMIC_STRUP_VMD1_OC_STATUS_ADDR, PMIC_STRUP_VMD1_OC_STATUS_MASK,
	 PMIC_STRUP_VMD1_OC_STATUS_SHIFT},
	{PMIC_STRUP_VS2_OC_STATUS, PMIC_STRUP_VS2_OC_STATUS_ADDR, PMIC_STRUP_VS2_OC_STATUS_MASK,
	 PMIC_STRUP_VS2_OC_STATUS_SHIFT},
	{PMIC_STRUP_VS1_OC_STATUS, PMIC_STRUP_VS1_OC_STATUS_ADDR, PMIC_STRUP_VS1_OC_STATUS_MASK,
	 PMIC_STRUP_VS1_OC_STATUS_SHIFT},
	{PMIC_STRUP_VCORE_OC_STATUS, PMIC_STRUP_VCORE_OC_STATUS_ADDR,
	 PMIC_STRUP_VCORE_OC_STATUS_MASK, PMIC_STRUP_VCORE_OC_STATUS_SHIFT},
	{PMIC_PMU_THERMAL_DEB, PMIC_PMU_THERMAL_DEB_ADDR, PMIC_PMU_THERMAL_DEB_MASK,
	 PMIC_PMU_THERMAL_DEB_SHIFT},
	{PMIC_STRUP_THERMAL_STATUS, PMIC_STRUP_THERMAL_STATUS_ADDR, PMIC_STRUP_THERMAL_STATUS_MASK,
	 PMIC_STRUP_THERMAL_STATUS_SHIFT},
	{PMIC_PMU_TEST_MODE_SCAN, PMIC_PMU_TEST_MODE_SCAN_ADDR, PMIC_PMU_TEST_MODE_SCAN_MASK,
	 PMIC_PMU_TEST_MODE_SCAN_SHIFT},
	{PMIC_PWRKEY_DEB, PMIC_PWRKEY_DEB_ADDR, PMIC_PWRKEY_DEB_MASK, PMIC_PWRKEY_DEB_SHIFT},
	{PMIC_HOMEKEY_DEB, PMIC_HOMEKEY_DEB_ADDR, PMIC_HOMEKEY_DEB_MASK, PMIC_HOMEKEY_DEB_SHIFT},
	{PMIC_RTC_XTAL_DET_DONE, PMIC_RTC_XTAL_DET_DONE_ADDR, PMIC_RTC_XTAL_DET_DONE_MASK,
	 PMIC_RTC_XTAL_DET_DONE_SHIFT},
	{PMIC_XOSC32_ENB_DET, PMIC_XOSC32_ENB_DET_ADDR, PMIC_XOSC32_ENB_DET_MASK,
	 PMIC_XOSC32_ENB_DET_SHIFT},
	{PMIC_RTC_XTAL_DET_RSV, PMIC_RTC_XTAL_DET_RSV_ADDR, PMIC_RTC_XTAL_DET_RSV_MASK,
	 PMIC_RTC_XTAL_DET_RSV_SHIFT},
	{PMIC_RG_PMU_TDSEL, PMIC_RG_PMU_TDSEL_ADDR, PMIC_RG_PMU_TDSEL_MASK,
	 PMIC_RG_PMU_TDSEL_SHIFT},
	{PMIC_RG_SPI_TDSEL, PMIC_RG_SPI_TDSEL_ADDR, PMIC_RG_SPI_TDSEL_MASK,
	 PMIC_RG_SPI_TDSEL_SHIFT},
	{PMIC_RG_AUD_TDSEL, PMIC_RG_AUD_TDSEL_ADDR, PMIC_RG_AUD_TDSEL_MASK,
	 PMIC_RG_AUD_TDSEL_SHIFT},
	{PMIC_RG_E32CAL_TDSEL, PMIC_RG_E32CAL_TDSEL_ADDR, PMIC_RG_E32CAL_TDSEL_MASK,
	 PMIC_RG_E32CAL_TDSEL_SHIFT},
	{PMIC_RG_PMU_RDSEL, PMIC_RG_PMU_RDSEL_ADDR, PMIC_RG_PMU_RDSEL_MASK,
	 PMIC_RG_PMU_RDSEL_SHIFT},
	{PMIC_RG_SPI_RDSEL, PMIC_RG_SPI_RDSEL_ADDR, PMIC_RG_SPI_RDSEL_MASK,
	 PMIC_RG_SPI_RDSEL_SHIFT},
	{PMIC_RG_AUD_RDSEL, PMIC_RG_AUD_RDSEL_ADDR, PMIC_RG_AUD_RDSEL_MASK,
	 PMIC_RG_AUD_RDSEL_SHIFT},
	{PMIC_RG_E32CAL_RDSEL, PMIC_RG_E32CAL_RDSEL_ADDR, PMIC_RG_E32CAL_RDSEL_MASK,
	 PMIC_RG_E32CAL_RDSEL_SHIFT},
	{PMIC_RG_SMT_WDTRSTB_IN, PMIC_RG_SMT_WDTRSTB_IN_ADDR, PMIC_RG_SMT_WDTRSTB_IN_MASK,
	 PMIC_RG_SMT_WDTRSTB_IN_SHIFT},
	{PMIC_RG_SMT_HOMEKEY, PMIC_RG_SMT_HOMEKEY_ADDR, PMIC_RG_SMT_HOMEKEY_MASK,
	 PMIC_RG_SMT_HOMEKEY_SHIFT},
	{PMIC_RG_SMT_SRCLKEN_IN0, PMIC_RG_SMT_SRCLKEN_IN0_ADDR, PMIC_RG_SMT_SRCLKEN_IN0_MASK,
	 PMIC_RG_SMT_SRCLKEN_IN0_SHIFT},
	{PMIC_RG_SMT_SRCLKEN_IN1, PMIC_RG_SMT_SRCLKEN_IN1_ADDR, PMIC_RG_SMT_SRCLKEN_IN1_MASK,
	 PMIC_RG_SMT_SRCLKEN_IN1_SHIFT},
	{PMIC_RG_SMT_RTC_32K1V8_0, PMIC_RG_SMT_RTC_32K1V8_0_ADDR, PMIC_RG_SMT_RTC_32K1V8_0_MASK,
	 PMIC_RG_SMT_RTC_32K1V8_0_SHIFT},
	{PMIC_RG_SMT_RTC_32K1V8_1, PMIC_RG_SMT_RTC_32K1V8_1_ADDR, PMIC_RG_SMT_RTC_32K1V8_1_MASK,
	 PMIC_RG_SMT_RTC_32K1V8_1_SHIFT},
	{PMIC_RG_SMT_SCP_VREQ_VAO, PMIC_RG_SMT_SCP_VREQ_VAO_ADDR, PMIC_RG_SMT_SCP_VREQ_VAO_MASK,
	 PMIC_RG_SMT_SCP_VREQ_VAO_SHIFT},
	{PMIC_RG_SMT_SPI_CLK, PMIC_RG_SMT_SPI_CLK_ADDR, PMIC_RG_SMT_SPI_CLK_MASK,
	 PMIC_RG_SMT_SPI_CLK_SHIFT},
	{PMIC_RG_SMT_SPI_CSN, PMIC_RG_SMT_SPI_CSN_ADDR, PMIC_RG_SMT_SPI_CSN_MASK,
	 PMIC_RG_SMT_SPI_CSN_SHIFT},
	{PMIC_RG_SMT_SPI_MOSI, PMIC_RG_SMT_SPI_MOSI_ADDR, PMIC_RG_SMT_SPI_MOSI_MASK,
	 PMIC_RG_SMT_SPI_MOSI_SHIFT},
	{PMIC_RG_SMT_SPI_MISO, PMIC_RG_SMT_SPI_MISO_ADDR, PMIC_RG_SMT_SPI_MISO_MASK,
	 PMIC_RG_SMT_SPI_MISO_SHIFT},
	{PMIC_RG_OCTL_SRCLKEN_IN0, PMIC_RG_OCTL_SRCLKEN_IN0_ADDR, PMIC_RG_OCTL_SRCLKEN_IN0_MASK,
	 PMIC_RG_OCTL_SRCLKEN_IN0_SHIFT},
	{PMIC_RG_OCTL_SRCLKEN_IN1, PMIC_RG_OCTL_SRCLKEN_IN1_ADDR, PMIC_RG_OCTL_SRCLKEN_IN1_MASK,
	 PMIC_RG_OCTL_SRCLKEN_IN1_SHIFT},
	{PMIC_RG_OCTL_RTC_32K1V8_0, PMIC_RG_OCTL_RTC_32K1V8_0_ADDR, PMIC_RG_OCTL_RTC_32K1V8_0_MASK,
	 PMIC_RG_OCTL_RTC_32K1V8_0_SHIFT},
	{PMIC_RG_OCTL_RTC_32K1V8_1, PMIC_RG_OCTL_RTC_32K1V8_1_ADDR, PMIC_RG_OCTL_RTC_32K1V8_1_MASK,
	 PMIC_RG_OCTL_RTC_32K1V8_1_SHIFT},
	{PMIC_RG_OCTL_SPI_CLK, PMIC_RG_OCTL_SPI_CLK_ADDR, PMIC_RG_OCTL_SPI_CLK_MASK,
	 PMIC_RG_OCTL_SPI_CLK_SHIFT},
	{PMIC_RG_OCTL_SPI_CSN, PMIC_RG_OCTL_SPI_CSN_ADDR, PMIC_RG_OCTL_SPI_CSN_MASK,
	 PMIC_RG_OCTL_SPI_CSN_SHIFT},
	{PMIC_RG_OCTL_SPI_MOSI, PMIC_RG_OCTL_SPI_MOSI_ADDR, PMIC_RG_OCTL_SPI_MOSI_MASK,
	 PMIC_RG_OCTL_SPI_MOSI_SHIFT},
	{PMIC_RG_OCTL_SPI_MISO, PMIC_RG_OCTL_SPI_MISO_ADDR, PMIC_RG_OCTL_SPI_MISO_MASK,
	 PMIC_RG_OCTL_SPI_MISO_SHIFT},
	{PMIC_RG_OCTL_HOMEKEY, PMIC_RG_OCTL_HOMEKEY_ADDR, PMIC_RG_OCTL_HOMEKEY_MASK,
	 PMIC_RG_OCTL_HOMEKEY_SHIFT},
	{PMIC_RG_OCTL_SCP_VREQ_VAO, PMIC_RG_OCTL_SCP_VREQ_VAO_ADDR, PMIC_RG_OCTL_SCP_VREQ_VAO_MASK,
	 PMIC_RG_OCTL_SCP_VREQ_VAO_SHIFT},
	{PMIC_RG_SPI_CLK_FILTER_EN, PMIC_RG_SPI_CLK_FILTER_EN_ADDR, PMIC_RG_SPI_CLK_FILTER_EN_MASK,
	 PMIC_RG_SPI_CLK_FILTER_EN_SHIFT},
	{PMIC_RG_SPI_CSN_FILTER_EN, PMIC_RG_SPI_CSN_FILTER_EN_ADDR, PMIC_RG_SPI_CSN_FILTER_EN_MASK,
	 PMIC_RG_SPI_CSN_FILTER_EN_SHIFT},
	{PMIC_RG_SPI_MOSI_FILTER_EN, PMIC_RG_SPI_MOSI_FILTER_EN_ADDR,
	 PMIC_RG_SPI_MOSI_FILTER_EN_MASK, PMIC_RG_SPI_MOSI_FILTER_EN_SHIFT},
	{PMIC_RG_SPI_MISO_FILTER_EN, PMIC_RG_SPI_MISO_FILTER_EN_ADDR,
	 PMIC_RG_SPI_MISO_FILTER_EN_MASK, PMIC_RG_SPI_MISO_FILTER_EN_SHIFT},
	{PMIC_TOP_STATUS, PMIC_TOP_STATUS_ADDR, PMIC_TOP_STATUS_MASK, PMIC_TOP_STATUS_SHIFT},
	{PMIC_TOP_STATUS_SET, PMIC_TOP_STATUS_SET_ADDR, PMIC_TOP_STATUS_SET_MASK,
	 PMIC_TOP_STATUS_SET_SHIFT},
	{PMIC_TOP_STATUS_CLR, PMIC_TOP_STATUS_CLR_ADDR, PMIC_TOP_STATUS_CLR_MASK,
	 PMIC_TOP_STATUS_CLR_SHIFT},
	{PMIC_RG_SRCLKEN_IN2_EN, PMIC_RG_SRCLKEN_IN2_EN_ADDR, PMIC_RG_SRCLKEN_IN2_EN_MASK,
	 PMIC_RG_SRCLKEN_IN2_EN_SHIFT},
	{PMIC_RG_SRCLKEN_IN3_EN, PMIC_RG_SRCLKEN_IN3_EN_ADDR, PMIC_RG_SRCLKEN_IN3_EN_MASK,
	 PMIC_RG_SRCLKEN_IN3_EN_SHIFT},
	{PMIC_RG_G_SMPS_CK_PDN, PMIC_RG_G_SMPS_CK_PDN_ADDR, PMIC_RG_G_SMPS_CK_PDN_MASK,
	 PMIC_RG_G_SMPS_CK_PDN_SHIFT},
	{PMIC_RG_G_SMPS_TEST_CK_PDN, PMIC_RG_G_SMPS_TEST_CK_PDN_ADDR,
	 PMIC_RG_G_SMPS_TEST_CK_PDN_MASK, PMIC_RG_G_SMPS_TEST_CK_PDN_SHIFT},
	{PMIC_RG_INTRP_CK_PDN, PMIC_RG_INTRP_CK_PDN_ADDR, PMIC_RG_INTRP_CK_PDN_MASK,
	 PMIC_RG_INTRP_CK_PDN_SHIFT},
	{PMIC_RG_INTRP_PRE_OC_CK_PDN, PMIC_RG_INTRP_PRE_OC_CK_PDN_ADDR,
	 PMIC_RG_INTRP_PRE_OC_CK_PDN_MASK, PMIC_RG_INTRP_PRE_OC_CK_PDN_SHIFT},
	{PMIC_RG_G_BIF_1M_CK_PDN, PMIC_RG_G_BIF_1M_CK_PDN_ADDR, PMIC_RG_G_BIF_1M_CK_PDN_MASK,
	 PMIC_RG_G_BIF_1M_CK_PDN_SHIFT},
	{PMIC_RG_BIF_X1_CK_PDN, PMIC_RG_BIF_X1_CK_PDN_ADDR, PMIC_RG_BIF_X1_CK_PDN_MASK,
	 PMIC_RG_BIF_X1_CK_PDN_SHIFT},
	{PMIC_RG_BIF_X4_CK_PDN, PMIC_RG_BIF_X4_CK_PDN_ADDR, PMIC_RG_BIF_X4_CK_PDN_MASK,
	 PMIC_RG_BIF_X4_CK_PDN_SHIFT},
	{PMIC_RG_BIF_X72_CK_PDN, PMIC_RG_BIF_X72_CK_PDN_ADDR, PMIC_RG_BIF_X72_CK_PDN_MASK,
	 PMIC_RG_BIF_X72_CK_PDN_SHIFT},
	{PMIC_RG_AUXADC_1M_CK_PDN, PMIC_RG_AUXADC_1M_CK_PDN_ADDR, PMIC_RG_AUXADC_1M_CK_PDN_MASK,
	 PMIC_RG_AUXADC_1M_CK_PDN_SHIFT},
	{PMIC_RG_AUXADC_CK_PDN, PMIC_RG_AUXADC_CK_PDN_ADDR, PMIC_RG_AUXADC_CK_PDN_MASK,
	 PMIC_RG_AUXADC_CK_PDN_SHIFT},
	{PMIC_RG_AUXADC_RNG_CK_PDN, PMIC_RG_AUXADC_RNG_CK_PDN_ADDR, PMIC_RG_AUXADC_RNG_CK_PDN_MASK,
	 PMIC_RG_AUXADC_RNG_CK_PDN_SHIFT},
	{PMIC_RG_DRV_32K_CK_PDN, PMIC_RG_DRV_32K_CK_PDN_ADDR, PMIC_RG_DRV_32K_CK_PDN_MASK,
	 PMIC_RG_DRV_32K_CK_PDN_SHIFT},
	{PMIC_RG_DRV_ISINK3_CK_PDN, PMIC_RG_DRV_ISINK3_CK_PDN_ADDR, PMIC_RG_DRV_ISINK3_CK_PDN_MASK,
	 PMIC_RG_DRV_ISINK3_CK_PDN_SHIFT},
	{PMIC_RG_DRV_ISINK2_CK_PDN, PMIC_RG_DRV_ISINK2_CK_PDN_ADDR, PMIC_RG_DRV_ISINK2_CK_PDN_MASK,
	 PMIC_RG_DRV_ISINK2_CK_PDN_SHIFT},
	{PMIC_RG_DRV_ISINK1_CK_PDN, PMIC_RG_DRV_ISINK1_CK_PDN_ADDR, PMIC_RG_DRV_ISINK1_CK_PDN_MASK,
	 PMIC_RG_DRV_ISINK1_CK_PDN_SHIFT},
	{PMIC_RG_DRV_ISINK0_CK_PDN, PMIC_RG_DRV_ISINK0_CK_PDN_ADDR, PMIC_RG_DRV_ISINK0_CK_PDN_MASK,
	 PMIC_RG_DRV_ISINK0_CK_PDN_SHIFT},
	{PMIC_TOP_CKPDN_CON0_SET, PMIC_TOP_CKPDN_CON0_SET_ADDR, PMIC_TOP_CKPDN_CON0_SET_MASK,
	 PMIC_TOP_CKPDN_CON0_SET_SHIFT},
	{PMIC_TOP_CKPDN_CON0_CLR, PMIC_TOP_CKPDN_CON0_CLR_ADDR, PMIC_TOP_CKPDN_CON0_CLR_MASK,
	 PMIC_TOP_CKPDN_CON0_CLR_SHIFT},
	{PMIC_RG_BUCK_9M_CK_PDN, PMIC_RG_BUCK_9M_CK_PDN_ADDR, PMIC_RG_BUCK_9M_CK_PDN_MASK,
	 PMIC_RG_BUCK_9M_CK_PDN_SHIFT},
	{PMIC_RG_BUCK_1M_CK_PDN, PMIC_RG_BUCK_1M_CK_PDN_ADDR, PMIC_RG_BUCK_1M_CK_PDN_MASK,
	 PMIC_RG_BUCK_1M_CK_PDN_SHIFT},
	{PMIC_RG_BUCK_18M_CK_PDN, PMIC_RG_BUCK_18M_CK_PDN_ADDR, PMIC_RG_BUCK_18M_CK_PDN_MASK,
	 PMIC_RG_BUCK_18M_CK_PDN_SHIFT},
	{PMIC_RG_BUCK_32K_CK_PDN, PMIC_RG_BUCK_32K_CK_PDN_ADDR, PMIC_RG_BUCK_32K_CK_PDN_MASK,
	 PMIC_RG_BUCK_32K_CK_PDN_SHIFT},
	{PMIC_RG_LDO_9M_CK_PDN, PMIC_RG_LDO_9M_CK_PDN_ADDR, PMIC_RG_LDO_9M_CK_PDN_MASK,
	 PMIC_RG_LDO_9M_CK_PDN_SHIFT},
	{PMIC_RG_LDO_1M_CK_PDN, PMIC_RG_LDO_1M_CK_PDN_ADDR, PMIC_RG_LDO_1M_CK_PDN_MASK,
	 PMIC_RG_LDO_1M_CK_PDN_SHIFT},
	{PMIC_RG_LDO_CALI_75K_CK_PDN, PMIC_RG_LDO_CALI_75K_CK_PDN_ADDR,
	 PMIC_RG_LDO_CALI_75K_CK_PDN_MASK, PMIC_RG_LDO_CALI_75K_CK_PDN_SHIFT},
	{PMIC_RG_BUCK_ANA_CLK_PDN, PMIC_RG_BUCK_ANA_CLK_PDN_ADDR, PMIC_RG_BUCK_ANA_CLK_PDN_MASK,
	 PMIC_RG_BUCK_ANA_CLK_PDN_SHIFT},
	{PMIC_RG_TRIM_75K_CK_PDN, PMIC_RG_TRIM_75K_CK_PDN_ADDR, PMIC_RG_TRIM_75K_CK_PDN_MASK,
	 PMIC_RG_TRIM_75K_CK_PDN_SHIFT},
	{PMIC_RG_CHDET_75K_CK_PDN, PMIC_RG_CHDET_75K_CK_PDN_ADDR, PMIC_RG_CHDET_75K_CK_PDN_MASK,
	 PMIC_RG_CHDET_75K_CK_PDN_SHIFT},
	{PMIC_RG_SPI_CK_PDN, PMIC_RG_SPI_CK_PDN_ADDR, PMIC_RG_SPI_CK_PDN_MASK,
	 PMIC_RG_SPI_CK_PDN_SHIFT},
	{PMIC_RG_REG_CK_PDN, PMIC_RG_REG_CK_PDN_ADDR, PMIC_RG_REG_CK_PDN_MASK,
	 PMIC_RG_REG_CK_PDN_SHIFT},
	{PMIC_RG_BGR_TEST_CK_PDN, PMIC_RG_BGR_TEST_CK_PDN_ADDR, PMIC_RG_BGR_TEST_CK_PDN_MASK,
	 PMIC_RG_BGR_TEST_CK_PDN_SHIFT},
	{PMIC_RG_FQMTR_32K_CK_PDN, PMIC_RG_FQMTR_32K_CK_PDN_ADDR, PMIC_RG_FQMTR_32K_CK_PDN_MASK,
	 PMIC_RG_FQMTR_32K_CK_PDN_SHIFT},
	{PMIC_RG_FQMTR_CK_PDN, PMIC_RG_FQMTR_CK_PDN_ADDR, PMIC_RG_FQMTR_CK_PDN_MASK,
	 PMIC_RG_FQMTR_CK_PDN_SHIFT},
	{PMIC_RG_BUCK_ANA_CK_PDN, PMIC_RG_BUCK_ANA_CK_PDN_ADDR, PMIC_RG_BUCK_ANA_CK_PDN_MASK,
	 PMIC_RG_BUCK_ANA_CK_PDN_SHIFT},
	{PMIC_TOP_CKPDN_CON1_SET, PMIC_TOP_CKPDN_CON1_SET_ADDR, PMIC_TOP_CKPDN_CON1_SET_MASK,
	 PMIC_TOP_CKPDN_CON1_SET_SHIFT},
	{PMIC_TOP_CKPDN_CON1_CLR, PMIC_TOP_CKPDN_CON1_CLR_ADDR, PMIC_TOP_CKPDN_CON1_CLR_MASK,
	 PMIC_TOP_CKPDN_CON1_CLR_SHIFT},
	{PMIC_RG_EOSC_CALI_TEST_CK_PDN, PMIC_RG_EOSC_CALI_TEST_CK_PDN_ADDR,
	 PMIC_RG_EOSC_CALI_TEST_CK_PDN_MASK, PMIC_RG_EOSC_CALI_TEST_CK_PDN_SHIFT},
	{PMIC_RG_RTC_EOSC32_CK_PDN, PMIC_RG_RTC_EOSC32_CK_PDN_ADDR, PMIC_RG_RTC_EOSC32_CK_PDN_MASK,
	 PMIC_RG_RTC_EOSC32_CK_PDN_SHIFT},
	{PMIC_RG_RTCDET_CK_PDN, PMIC_RG_RTCDET_CK_PDN_ADDR, PMIC_RG_RTCDET_CK_PDN_MASK,
	 PMIC_RG_RTCDET_CK_PDN_SHIFT},
	{PMIC_RG_RTC_75K_CK_PDN, PMIC_RG_RTC_75K_CK_PDN_ADDR, PMIC_RG_RTC_75K_CK_PDN_MASK,
	 PMIC_RG_RTC_75K_CK_PDN_SHIFT},
	{PMIC_RG_RTC_MCLK_PDN, PMIC_RG_RTC_MCLK_PDN_ADDR, PMIC_RG_RTC_MCLK_PDN_MASK,
	 PMIC_RG_RTC_MCLK_PDN_SHIFT},
	{PMIC_RG_RTC_32K_CK_PDN, PMIC_RG_RTC_32K_CK_PDN_ADDR, PMIC_RG_RTC_32K_CK_PDN_MASK,
	 PMIC_RG_RTC_32K_CK_PDN_SHIFT},
	{PMIC_RG_RTC_26M_CK_PDN, PMIC_RG_RTC_26M_CK_PDN_ADDR, PMIC_RG_RTC_26M_CK_PDN_MASK,
	 PMIC_RG_RTC_26M_CK_PDN_SHIFT},
	{PMIC_RG_FGADC_FT_CK_PDN, PMIC_RG_FGADC_FT_CK_PDN_ADDR, PMIC_RG_FGADC_FT_CK_PDN_MASK,
	 PMIC_RG_FGADC_FT_CK_PDN_SHIFT},
	{PMIC_RG_FGADC_DIG_CK_PDN, PMIC_RG_FGADC_DIG_CK_PDN_ADDR, PMIC_RG_FGADC_DIG_CK_PDN_MASK,
	 PMIC_RG_FGADC_DIG_CK_PDN_SHIFT},
	{PMIC_RG_FGADC_ANA_CK_PDN, PMIC_RG_FGADC_ANA_CK_PDN_ADDR, PMIC_RG_FGADC_ANA_CK_PDN_MASK,
	 PMIC_RG_FGADC_ANA_CK_PDN_SHIFT},
	{PMIC_RG_STRUP_32K_CK_PDN, PMIC_RG_STRUP_32K_CK_PDN_ADDR, PMIC_RG_STRUP_32K_CK_PDN_MASK,
	 PMIC_RG_STRUP_32K_CK_PDN_SHIFT},
	{PMIC_RG_STRUP_75K_CK_PDN, PMIC_RG_STRUP_75K_CK_PDN_ADDR, PMIC_RG_STRUP_75K_CK_PDN_MASK,
	 PMIC_RG_STRUP_75K_CK_PDN_SHIFT},
	{PMIC_RG_RTC32K_1V8_0_O_PDN, PMIC_RG_RTC32K_1V8_0_O_PDN_ADDR,
	 PMIC_RG_RTC32K_1V8_0_O_PDN_MASK, PMIC_RG_RTC32K_1V8_0_O_PDN_SHIFT},
	{PMIC_RG_RTC32K_1V8_1_O_PDN, PMIC_RG_RTC32K_1V8_1_O_PDN_ADDR,
	 PMIC_RG_RTC32K_1V8_1_O_PDN_MASK, PMIC_RG_RTC32K_1V8_1_O_PDN_SHIFT},
	{PMIC_RG_RTC_2SEC_OFF_DET_PDN, PMIC_RG_RTC_2SEC_OFF_DET_PDN_ADDR,
	 PMIC_RG_RTC_2SEC_OFF_DET_PDN_MASK, PMIC_RG_RTC_2SEC_OFF_DET_PDN_SHIFT},
	{PMIC_RG_SMPS_CK_DIV_PDN, PMIC_RG_SMPS_CK_DIV_PDN_ADDR, PMIC_RG_SMPS_CK_DIV_PDN_MASK,
	 PMIC_RG_SMPS_CK_DIV_PDN_SHIFT},
	{PMIC_TOP_CKPDN_CON2_SET, PMIC_TOP_CKPDN_CON2_SET_ADDR, PMIC_TOP_CKPDN_CON2_SET_MASK,
	 PMIC_TOP_CKPDN_CON2_SET_SHIFT},
	{PMIC_TOP_CKPDN_CON2_CLR, PMIC_TOP_CKPDN_CON2_CLR_ADDR, PMIC_TOP_CKPDN_CON2_CLR_MASK,
	 PMIC_TOP_CKPDN_CON2_CLR_SHIFT},
	{PMIC_RG_BATON_75K_CK_PDN, PMIC_RG_BATON_75K_CK_PDN_ADDR, PMIC_RG_BATON_75K_CK_PDN_MASK,
	 PMIC_RG_BATON_75K_CK_PDN_SHIFT},
	{PMIC_RG_EFUSE_CK_PDN, PMIC_RG_EFUSE_CK_PDN_ADDR, PMIC_RG_EFUSE_CK_PDN_MASK,
	 PMIC_RG_EFUSE_CK_PDN_SHIFT},
	{PMIC_RG_TOP_CKPDN_CON3_RSV, PMIC_RG_TOP_CKPDN_CON3_RSV_ADDR,
	 PMIC_RG_TOP_CKPDN_CON3_RSV_MASK, PMIC_RG_TOP_CKPDN_CON3_RSV_SHIFT},
	{PMIC_TOP_CKPDN_CON3_SET, PMIC_TOP_CKPDN_CON3_SET_ADDR, PMIC_TOP_CKPDN_CON3_SET_MASK,
	 PMIC_TOP_CKPDN_CON3_SET_SHIFT},
	{PMIC_TOP_CKPDN_CON3_CLR, PMIC_TOP_CKPDN_CON3_CLR_ADDR, PMIC_TOP_CKPDN_CON3_CLR_MASK,
	 PMIC_TOP_CKPDN_CON3_CLR_SHIFT},
	{PMIC_RG_FQMTR_CK_CKSEL, PMIC_RG_FQMTR_CK_CKSEL_ADDR, PMIC_RG_FQMTR_CK_CKSEL_MASK,
	 PMIC_RG_FQMTR_CK_CKSEL_SHIFT},
	{PMIC_RG_75K_32K_SEL, PMIC_RG_75K_32K_SEL_ADDR, PMIC_RG_75K_32K_SEL_MASK,
	 PMIC_RG_75K_32K_SEL_SHIFT},
	{PMIC_RG_FGADC_ANA_CK_CKSEL, PMIC_RG_FGADC_ANA_CK_CKSEL_ADDR,
	 PMIC_RG_FGADC_ANA_CK_CKSEL_MASK, PMIC_RG_FGADC_ANA_CK_CKSEL_SHIFT},
	{PMIC_RG_BGR_TEST_CK_CKSEL, PMIC_RG_BGR_TEST_CK_CKSEL_ADDR, PMIC_RG_BGR_TEST_CK_CKSEL_MASK,
	 PMIC_RG_BGR_TEST_CK_CKSEL_SHIFT},
	{PMIC_RG_TOP_CKSEL_CON0_RSV, PMIC_RG_TOP_CKSEL_CON0_RSV_ADDR,
	 PMIC_RG_TOP_CKSEL_CON0_RSV_MASK, PMIC_RG_TOP_CKSEL_CON0_RSV_SHIFT},
	{PMIC_TOP_CKSEL_CON0_SET, PMIC_TOP_CKSEL_CON0_SET_ADDR, PMIC_TOP_CKSEL_CON0_SET_MASK,
	 PMIC_TOP_CKSEL_CON0_SET_SHIFT},
	{PMIC_TOP_CKSEL_CON0_CLR, PMIC_TOP_CKSEL_CON0_CLR_ADDR, PMIC_TOP_CKSEL_CON0_CLR_MASK,
	 PMIC_TOP_CKSEL_CON0_CLR_SHIFT},
	{PMIC_RG_SRCVOLTEN_SW, PMIC_RG_SRCVOLTEN_SW_ADDR, PMIC_RG_SRCVOLTEN_SW_MASK,
	 PMIC_RG_SRCVOLTEN_SW_SHIFT},
	{PMIC_RG_BUCK_OSC_SEL_SW, PMIC_RG_BUCK_OSC_SEL_SW_ADDR, PMIC_RG_BUCK_OSC_SEL_SW_MASK,
	 PMIC_RG_BUCK_OSC_SEL_SW_SHIFT},
	{PMIC_RG_VOWEN_SW, PMIC_RG_VOWEN_SW_ADDR, PMIC_RG_VOWEN_SW_MASK, PMIC_RG_VOWEN_SW_SHIFT},
	{PMIC_RG_SRCVOLTEN_MODE, PMIC_RG_SRCVOLTEN_MODE_ADDR, PMIC_RG_SRCVOLTEN_MODE_MASK,
	 PMIC_RG_SRCVOLTEN_MODE_SHIFT},
	{PMIC_RG_BUCK_OSC_SEL_MODE, PMIC_RG_BUCK_OSC_SEL_MODE_ADDR, PMIC_RG_BUCK_OSC_SEL_MODE_MASK,
	 PMIC_RG_BUCK_OSC_SEL_MODE_SHIFT},
	{PMIC_RG_VOWEN_MODE, PMIC_RG_VOWEN_MODE_ADDR, PMIC_RG_VOWEN_MODE_MASK,
	 PMIC_RG_VOWEN_MODE_SHIFT},
	{PMIC_RG_TOP_CKSEL_CON2_RSV, PMIC_RG_TOP_CKSEL_CON2_RSV_ADDR,
	 PMIC_RG_TOP_CKSEL_CON2_RSV_MASK, PMIC_RG_TOP_CKSEL_CON2_RSV_SHIFT},
	{PMIC_TOP_CKSEL_CON2_SET, PMIC_TOP_CKSEL_CON2_SET_ADDR, PMIC_TOP_CKSEL_CON2_SET_MASK,
	 PMIC_TOP_CKSEL_CON2_SET_SHIFT},
	{PMIC_TOP_CKSEL_CON2_CLR, PMIC_TOP_CKSEL_CON2_CLR_ADDR, PMIC_TOP_CKSEL_CON2_CLR_MASK,
	 PMIC_TOP_CKSEL_CON2_CLR_SHIFT},
	{PMIC_RG_AUXADC_CK_DIVSEL, PMIC_RG_AUXADC_CK_DIVSEL_ADDR, PMIC_RG_AUXADC_CK_DIVSEL_MASK,
	 PMIC_RG_AUXADC_CK_DIVSEL_SHIFT},
	{PMIC_RG_LDO_9M_CK_DIVSEL, PMIC_RG_LDO_9M_CK_DIVSEL_ADDR, PMIC_RG_LDO_9M_CK_DIVSEL_MASK,
	 PMIC_RG_LDO_9M_CK_DIVSEL_SHIFT},
	{PMIC_RG_BUCK_9M_CK_DIVSEL, PMIC_RG_BUCK_9M_CK_DIVSEL_ADDR, PMIC_RG_BUCK_9M_CK_DIVSEL_MASK,
	 PMIC_RG_BUCK_9M_CK_DIVSEL_SHIFT},
	{PMIC_RG_BIF_X4_CK_DIVSEL, PMIC_RG_BIF_X4_CK_DIVSEL_ADDR, PMIC_RG_BIF_X4_CK_DIVSEL_MASK,
	 PMIC_RG_BIF_X4_CK_DIVSEL_SHIFT},
	{PMIC_RG_REG_CK_DIVSEL, PMIC_RG_REG_CK_DIVSEL_ADDR, PMIC_RG_REG_CK_DIVSEL_MASK,
	 PMIC_RG_REG_CK_DIVSEL_SHIFT},
	{PMIC_TOP_CKDIVSEL_CON0_RSV, PMIC_TOP_CKDIVSEL_CON0_RSV_ADDR,
	 PMIC_TOP_CKDIVSEL_CON0_RSV_MASK, PMIC_TOP_CKDIVSEL_CON0_RSV_SHIFT},
	{PMIC_TOP_CKDIVSEL_CON0_SET, PMIC_TOP_CKDIVSEL_CON0_SET_ADDR,
	 PMIC_TOP_CKDIVSEL_CON0_SET_MASK, PMIC_TOP_CKDIVSEL_CON0_SET_SHIFT},
	{PMIC_TOP_CKDIVSEL_CON0_CLR, PMIC_TOP_CKDIVSEL_CON0_CLR_ADDR,
	 PMIC_TOP_CKDIVSEL_CON0_CLR_MASK, PMIC_TOP_CKDIVSEL_CON0_CLR_SHIFT},
	{PMIC_RG_G_SMPS_CK_PDN_HWEN, PMIC_RG_G_SMPS_CK_PDN_HWEN_ADDR,
	 PMIC_RG_G_SMPS_CK_PDN_HWEN_MASK, PMIC_RG_G_SMPS_CK_PDN_HWEN_SHIFT},
	{PMIC_RG_SMPS_PD_1M_CK_PDN_HWEN, PMIC_RG_SMPS_PD_1M_CK_PDN_HWEN_ADDR,
	 PMIC_RG_SMPS_PD_1M_CK_PDN_HWEN_MASK, PMIC_RG_SMPS_PD_1M_CK_PDN_HWEN_SHIFT},
	{PMIC_RG_AUXADC_CK_PDN_HWEN, PMIC_RG_AUXADC_CK_PDN_HWEN_ADDR,
	 PMIC_RG_AUXADC_CK_PDN_HWEN_MASK, PMIC_RG_AUXADC_CK_PDN_HWEN_SHIFT},
	{PMIC_RG_AUXADC_RNG_CK_PDN_HWEN, PMIC_RG_AUXADC_RNG_CK_PDN_HWEN_ADDR,
	 PMIC_RG_AUXADC_RNG_CK_PDN_HWEN_MASK, PMIC_RG_AUXADC_RNG_CK_PDN_HWEN_SHIFT},
	{PMIC_RG_BIF_X4_CK_PDN_HWEN, PMIC_RG_BIF_X4_CK_PDN_HWEN_ADDR,
	 PMIC_RG_BIF_X4_CK_PDN_HWEN_MASK, PMIC_RG_BIF_X4_CK_PDN_HWEN_SHIFT},
	{PMIC_RG_BIF_X72_CK_PDN_HWEN, PMIC_RG_BIF_X72_CK_PDN_HWEN_ADDR,
	 PMIC_RG_BIF_X72_CK_PDN_HWEN_MASK, PMIC_RG_BIF_X72_CK_PDN_HWEN_SHIFT},
	{PMIC_RG_RTC_26M_CK_PDN_HWEN, PMIC_RG_RTC_26M_CK_PDN_HWEN_ADDR,
	 PMIC_RG_RTC_26M_CK_PDN_HWEN_MASK, PMIC_RG_RTC_26M_CK_PDN_HWEN_SHIFT},
	{PMIC_RG_REG_CK_PDN_HWEN, PMIC_RG_REG_CK_PDN_HWEN_ADDR, PMIC_RG_REG_CK_PDN_HWEN_MASK,
	 PMIC_RG_REG_CK_PDN_HWEN_SHIFT},
	{PMIC_RG_EFUSE_CK_PDN_HWEN, PMIC_RG_EFUSE_CK_PDN_HWEN_ADDR, PMIC_RG_EFUSE_CK_PDN_HWEN_MASK,
	 PMIC_RG_EFUSE_CK_PDN_HWEN_SHIFT},
	{PMIC_TOP_CKHWEN_CON0_RSV, PMIC_TOP_CKHWEN_CON0_RSV_ADDR, PMIC_TOP_CKHWEN_CON0_RSV_MASK,
	 PMIC_TOP_CKHWEN_CON0_RSV_SHIFT},
	{PMIC_TOP_CKHWEN_CON0_SET, PMIC_TOP_CKHWEN_CON0_SET_ADDR, PMIC_TOP_CKHWEN_CON0_SET_MASK,
	 PMIC_TOP_CKHWEN_CON0_SET_SHIFT},
	{PMIC_TOP_CKHWEN_CON0_CLR, PMIC_TOP_CKHWEN_CON0_CLR_ADDR, PMIC_TOP_CKHWEN_CON0_CLR_MASK,
	 PMIC_TOP_CKHWEN_CON0_CLR_SHIFT},
	{PMIC_RG_BUCK_9M_CK_PDN_HWEN, PMIC_RG_BUCK_9M_CK_PDN_HWEN_ADDR,
	 PMIC_RG_BUCK_9M_CK_PDN_HWEN_MASK, PMIC_RG_BUCK_9M_CK_PDN_HWEN_SHIFT},
	{PMIC_RG_BUCK_1M_CK_PDN_HWEN, PMIC_RG_BUCK_1M_CK_PDN_HWEN_ADDR,
	 PMIC_RG_BUCK_1M_CK_PDN_HWEN_MASK, PMIC_RG_BUCK_1M_CK_PDN_HWEN_SHIFT},
	{PMIC_RG_BUCK_18M_CK_PDN_HWEN, PMIC_RG_BUCK_18M_CK_PDN_HWEN_ADDR,
	 PMIC_RG_BUCK_18M_CK_PDN_HWEN_MASK, PMIC_RG_BUCK_18M_CK_PDN_HWEN_SHIFT},
	{PMIC_RG_LDO_1M_CK_PDN_HWEN, PMIC_RG_LDO_1M_CK_PDN_HWEN_ADDR,
	 PMIC_RG_LDO_1M_CK_PDN_HWEN_MASK, PMIC_RG_LDO_1M_CK_PDN_HWEN_SHIFT},
	{PMIC_RG_LDO_9M_CK_PDN_HWEN, PMIC_RG_LDO_9M_CK_PDN_HWEN_ADDR,
	 PMIC_RG_LDO_9M_CK_PDN_HWEN_MASK, PMIC_RG_LDO_9M_CK_PDN_HWEN_SHIFT},
	{PMIC_RG_BUCK_VMODEM_1M_CK_PDN_HWEN, PMIC_RG_BUCK_VMODEM_1M_CK_PDN_HWEN_ADDR,
	 PMIC_RG_BUCK_VMODEM_1M_CK_PDN_HWEN_MASK, PMIC_RG_BUCK_VMODEM_1M_CK_PDN_HWEN_SHIFT},
	{PMIC_TOP_CKHWEN_RSV, PMIC_TOP_CKHWEN_RSV_ADDR, PMIC_TOP_CKHWEN_RSV_MASK,
	 PMIC_TOP_CKHWEN_RSV_SHIFT},
	{PMIC_TOP_CKHWEN_CON1_SET, PMIC_TOP_CKHWEN_CON1_SET_ADDR, PMIC_TOP_CKHWEN_CON1_SET_MASK,
	 PMIC_TOP_CKHWEN_CON1_SET_SHIFT},
	{PMIC_TOP_CKHWEN_CON1_CLR, PMIC_TOP_CKHWEN_CON1_CLR_ADDR, PMIC_TOP_CKHWEN_CON1_CLR_MASK,
	 PMIC_TOP_CKHWEN_CON1_CLR_SHIFT},
	{PMIC_RG_VCORE_FREQ_SEL, PMIC_RG_VCORE_FREQ_SEL_ADDR, PMIC_RG_VCORE_FREQ_SEL_MASK,
	 PMIC_RG_VCORE_FREQ_SEL_SHIFT},
	{PMIC_RG_VDRAM_FREQ_SEL, PMIC_RG_VDRAM_FREQ_SEL_ADDR, PMIC_RG_VDRAM_FREQ_SEL_MASK,
	 PMIC_RG_VDRAM_FREQ_SEL_SHIFT},
	{PMIC_RG_VMODEM_FREQ_SEL, PMIC_RG_VMODEM_FREQ_SEL_ADDR, PMIC_RG_VMODEM_FREQ_SEL_MASK,
	 PMIC_RG_VMODEM_FREQ_SEL_SHIFT},
	{PMIC_RG_VMD1_FREQ_SEL, PMIC_RG_VMD1_FREQ_SEL_ADDR, PMIC_RG_VMD1_FREQ_SEL_MASK,
	 PMIC_RG_VMD1_FREQ_SEL_SHIFT},
	{PMIC_TOP_BUCK_ANACK_FREQ_SEL_CON0_RSV, PMIC_TOP_BUCK_ANACK_FREQ_SEL_CON0_RSV_ADDR,
	 PMIC_TOP_BUCK_ANACK_FREQ_SEL_CON0_RSV_MASK, PMIC_TOP_BUCK_ANACK_FREQ_SEL_CON0_RSV_SHIFT},
	{PMIC_TOP_BUCK_ANACK_FREQ_SEL_CON0_SET, PMIC_TOP_BUCK_ANACK_FREQ_SEL_CON0_SET_ADDR,
	 PMIC_TOP_BUCK_ANACK_FREQ_SEL_CON0_SET_MASK, PMIC_TOP_BUCK_ANACK_FREQ_SEL_CON0_SET_SHIFT},
	{PMIC_TOP_BUCK_ANACK_FREQ_SEL_CON0_CLR, PMIC_TOP_BUCK_ANACK_FREQ_SEL_CON0_CLR_ADDR,
	 PMIC_TOP_BUCK_ANACK_FREQ_SEL_CON0_CLR_MASK, PMIC_TOP_BUCK_ANACK_FREQ_SEL_CON0_CLR_SHIFT},
	{PMIC_RG_PMU75K_CK_TST_DIS, PMIC_RG_PMU75K_CK_TST_DIS_ADDR, PMIC_RG_PMU75K_CK_TST_DIS_MASK,
	 PMIC_RG_PMU75K_CK_TST_DIS_SHIFT},
	{PMIC_RG_SMPS_CK_TST_DIS, PMIC_RG_SMPS_CK_TST_DIS_ADDR, PMIC_RG_SMPS_CK_TST_DIS_MASK,
	 PMIC_RG_SMPS_CK_TST_DIS_SHIFT},
	{PMIC_RG_RTC32K_CK_TST_DIS, PMIC_RG_RTC32K_CK_TST_DIS_ADDR, PMIC_RG_RTC32K_CK_TST_DIS_MASK,
	 PMIC_RG_RTC32K_CK_TST_DIS_SHIFT},
	{PMIC_RG_FG_CK_TST_DIS, PMIC_RG_FG_CK_TST_DIS_ADDR, PMIC_RG_FG_CK_TST_DIS_MASK,
	 PMIC_RG_FG_CK_TST_DIS_SHIFT},
	{PMIC_RG_RTC26M_CK_TST_DIS, PMIC_RG_RTC26M_CK_TST_DIS_ADDR, PMIC_RG_RTC26M_CK_TST_DIS_MASK,
	 PMIC_RG_RTC26M_CK_TST_DIS_SHIFT},
	{PMIC_TOP_CKTST_CON0_RSV, PMIC_TOP_CKTST_CON0_RSV_ADDR, PMIC_TOP_CKTST_CON0_RSV_MASK,
	 PMIC_TOP_CKTST_CON0_RSV_SHIFT},
	{PMIC_RG_BUCK_ANA_AUTO_OFF_DIS, PMIC_RG_BUCK_ANA_AUTO_OFF_DIS_ADDR,
	 PMIC_RG_BUCK_ANA_AUTO_OFF_DIS_MASK, PMIC_RG_BUCK_ANA_AUTO_OFF_DIS_SHIFT},
	{PMIC_RG_AUXADC_CK_TSTSEL, PMIC_RG_AUXADC_CK_TSTSEL_ADDR, PMIC_RG_AUXADC_CK_TSTSEL_MASK,
	 PMIC_RG_AUXADC_CK_TSTSEL_SHIFT},
	{PMIC_RG_FQMTR_CK_TSTSEL, PMIC_RG_FQMTR_CK_TSTSEL_ADDR, PMIC_RG_FQMTR_CK_TSTSEL_MASK,
	 PMIC_RG_FQMTR_CK_TSTSEL_SHIFT},
	{PMIC_RG_RTCDET_CK_TSTSEL, PMIC_RG_RTCDET_CK_TSTSEL_ADDR, PMIC_RG_RTCDET_CK_TSTSEL_MASK,
	 PMIC_RG_RTCDET_CK_TSTSEL_SHIFT},
	{PMIC_RG_RTC_EOSC32_CK_TSTSEL, PMIC_RG_RTC_EOSC32_CK_TSTSEL_ADDR,
	 PMIC_RG_RTC_EOSC32_CK_TSTSEL_MASK, PMIC_RG_RTC_EOSC32_CK_TSTSEL_SHIFT},
	{PMIC_RG_EOSC_CALI_TEST_CK_TSTSEL, PMIC_RG_EOSC_CALI_TEST_CK_TSTSEL_ADDR,
	 PMIC_RG_EOSC_CALI_TEST_CK_TSTSEL_MASK, PMIC_RG_EOSC_CALI_TEST_CK_TSTSEL_SHIFT},
	{PMIC_RG_RTC26M_CK_TSTSEL, PMIC_RG_RTC26M_CK_TSTSEL_ADDR, PMIC_RG_RTC26M_CK_TSTSEL_MASK,
	 PMIC_RG_RTC26M_CK_TSTSEL_SHIFT},
	{PMIC_RG_RTC32K_CK_TSTSEL, PMIC_RG_RTC32K_CK_TSTSEL_ADDR, PMIC_RG_RTC32K_CK_TSTSEL_MASK,
	 PMIC_RG_RTC32K_CK_TSTSEL_SHIFT},
	{PMIC_RG_FG_CK_TSTSEL, PMIC_RG_FG_CK_TSTSEL_ADDR, PMIC_RG_FG_CK_TSTSEL_MASK,
	 PMIC_RG_FG_CK_TSTSEL_SHIFT},
	{PMIC_RG_FGADC_ANA_CK_TSTSEL, PMIC_RG_FGADC_ANA_CK_TSTSEL_ADDR,
	 PMIC_RG_FGADC_ANA_CK_TSTSEL_MASK, PMIC_RG_FGADC_ANA_CK_TSTSEL_SHIFT},
	{PMIC_RG_BGR_TEST_CK_TSTSEL, PMIC_RG_BGR_TEST_CK_TSTSEL_ADDR,
	 PMIC_RG_BGR_TEST_CK_TSTSEL_MASK, PMIC_RG_BGR_TEST_CK_TSTSEL_SHIFT},
	{PMIC_RG_PMU75K_CK_TSTSEL, PMIC_RG_PMU75K_CK_TSTSEL_ADDR, PMIC_RG_PMU75K_CK_TSTSEL_MASK,
	 PMIC_RG_PMU75K_CK_TSTSEL_SHIFT},
	{PMIC_RG_SMPS_CK_TSTSEL, PMIC_RG_SMPS_CK_TSTSEL_ADDR, PMIC_RG_SMPS_CK_TSTSEL_MASK,
	 PMIC_RG_SMPS_CK_TSTSEL_SHIFT},
	{PMIC_TOP_CKTST_CON1_RSV, PMIC_TOP_CKTST_CON1_RSV_ADDR, PMIC_TOP_CKTST_CON1_RSV_MASK,
	 PMIC_TOP_CKTST_CON1_RSV_SHIFT},
	{PMIC_RG_CLKSQ_EN_AUD, PMIC_RG_CLKSQ_EN_AUD_ADDR, PMIC_RG_CLKSQ_EN_AUD_MASK,
	 PMIC_RG_CLKSQ_EN_AUD_SHIFT},
	{PMIC_RG_CLKSQ_EN_FQR, PMIC_RG_CLKSQ_EN_FQR_ADDR, PMIC_RG_CLKSQ_EN_FQR_MASK,
	 PMIC_RG_CLKSQ_EN_FQR_SHIFT},
	{PMIC_RG_CLKSQ_EN_AUX_AP, PMIC_RG_CLKSQ_EN_AUX_AP_ADDR, PMIC_RG_CLKSQ_EN_AUX_AP_MASK,
	 PMIC_RG_CLKSQ_EN_AUX_AP_SHIFT},
	{PMIC_RG_CLKSQ_EN_AUX_MD, PMIC_RG_CLKSQ_EN_AUX_MD_ADDR, PMIC_RG_CLKSQ_EN_AUX_MD_MASK,
	 PMIC_RG_CLKSQ_EN_AUX_MD_SHIFT},
	{PMIC_RG_CLKSQ_EN_AUX_GPS, PMIC_RG_CLKSQ_EN_AUX_GPS_ADDR, PMIC_RG_CLKSQ_EN_AUX_GPS_MASK,
	 PMIC_RG_CLKSQ_EN_AUX_GPS_SHIFT},
	{PMIC_RG_CLKSQ_EN_AUX_RSV, PMIC_RG_CLKSQ_EN_AUX_RSV_ADDR, PMIC_RG_CLKSQ_EN_AUX_RSV_MASK,
	 PMIC_RG_CLKSQ_EN_AUX_RSV_SHIFT},
	{PMIC_RG_CLKSQ_EN_AUX_AP_MODE, PMIC_RG_CLKSQ_EN_AUX_AP_MODE_ADDR,
	 PMIC_RG_CLKSQ_EN_AUX_AP_MODE_MASK, PMIC_RG_CLKSQ_EN_AUX_AP_MODE_SHIFT},
	{PMIC_RG_CLKSQ_EN_AUX_MD_MODE, PMIC_RG_CLKSQ_EN_AUX_MD_MODE_ADDR,
	 PMIC_RG_CLKSQ_EN_AUX_MD_MODE_MASK, PMIC_RG_CLKSQ_EN_AUX_MD_MODE_SHIFT},
	{PMIC_RG_CLKSQ_IN_SEL_VA18, PMIC_RG_CLKSQ_IN_SEL_VA18_ADDR, PMIC_RG_CLKSQ_IN_SEL_VA18_MASK,
	 PMIC_RG_CLKSQ_IN_SEL_VA18_SHIFT},
	{PMIC_RG_CLKSQ_IN_SEL_VA18_SWCTRL, PMIC_RG_CLKSQ_IN_SEL_VA18_SWCTRL_ADDR,
	 PMIC_RG_CLKSQ_IN_SEL_VA18_SWCTRL_MASK, PMIC_RG_CLKSQ_IN_SEL_VA18_SWCTRL_SHIFT},
	{PMIC_TOP_CLKSQ_RSV, PMIC_TOP_CLKSQ_RSV_ADDR, PMIC_TOP_CLKSQ_RSV_MASK,
	 PMIC_TOP_CLKSQ_RSV_SHIFT},
	{PMIC_DA_CLKSQ_EN_VA18, PMIC_DA_CLKSQ_EN_VA18_ADDR, PMIC_DA_CLKSQ_EN_VA18_MASK,
	 PMIC_DA_CLKSQ_EN_VA18_SHIFT},
	{PMIC_TOP_CLKSQ_SET, PMIC_TOP_CLKSQ_SET_ADDR, PMIC_TOP_CLKSQ_SET_MASK,
	 PMIC_TOP_CLKSQ_SET_SHIFT},
	{PMIC_TOP_CLKSQ_CLR, PMIC_TOP_CLKSQ_CLR_ADDR, PMIC_TOP_CLKSQ_CLR_MASK,
	 PMIC_TOP_CLKSQ_CLR_SHIFT},
	{PMIC_RG_CLKSQ_RTC_EN, PMIC_RG_CLKSQ_RTC_EN_ADDR, PMIC_RG_CLKSQ_RTC_EN_MASK,
	 PMIC_RG_CLKSQ_RTC_EN_SHIFT},
	{PMIC_RG_CLKSQ_RTC_EN_HW_MODE, PMIC_RG_CLKSQ_RTC_EN_HW_MODE_ADDR,
	 PMIC_RG_CLKSQ_RTC_EN_HW_MODE_MASK, PMIC_RG_CLKSQ_RTC_EN_HW_MODE_SHIFT},
	{PMIC_TOP_CLKSQ_RTC_RSV0, PMIC_TOP_CLKSQ_RTC_RSV0_ADDR, PMIC_TOP_CLKSQ_RTC_RSV0_MASK,
	 PMIC_TOP_CLKSQ_RTC_RSV0_SHIFT},
	{PMIC_RG_ENBB_SEL, PMIC_RG_ENBB_SEL_ADDR, PMIC_RG_ENBB_SEL_MASK, PMIC_RG_ENBB_SEL_SHIFT},
	{PMIC_RG_XOSC_EN_SEL, PMIC_RG_XOSC_EN_SEL_ADDR, PMIC_RG_XOSC_EN_SEL_MASK,
	 PMIC_RG_XOSC_EN_SEL_SHIFT},
	{PMIC_TOP_CLKSQ_RTC_RSV1, PMIC_TOP_CLKSQ_RTC_RSV1_ADDR, PMIC_TOP_CLKSQ_RTC_RSV1_MASK,
	 PMIC_TOP_CLKSQ_RTC_RSV1_SHIFT},
	{PMIC_DA_CLKSQ_EN_VDIG18, PMIC_DA_CLKSQ_EN_VDIG18_ADDR, PMIC_DA_CLKSQ_EN_VDIG18_MASK,
	 PMIC_DA_CLKSQ_EN_VDIG18_SHIFT},
	{PMIC_TOP_CLKSQ_RTC_SET, PMIC_TOP_CLKSQ_RTC_SET_ADDR, PMIC_TOP_CLKSQ_RTC_SET_MASK,
	 PMIC_TOP_CLKSQ_RTC_SET_SHIFT},
	{PMIC_TOP_CLKSQ_RTC_CLR, PMIC_TOP_CLKSQ_RTC_CLR_ADDR, PMIC_TOP_CLKSQ_RTC_CLR_MASK,
	 PMIC_TOP_CLKSQ_RTC_CLR_SHIFT},
	{PMIC_RG_OSC_75K_TRIM, PMIC_RG_OSC_75K_TRIM_ADDR, PMIC_RG_OSC_75K_TRIM_MASK,
	 PMIC_RG_OSC_75K_TRIM_SHIFT},
	{PMIC_RG_OSC_75K_TRIM_EN, PMIC_RG_OSC_75K_TRIM_EN_ADDR, PMIC_RG_OSC_75K_TRIM_EN_MASK,
	 PMIC_RG_OSC_75K_TRIM_EN_SHIFT},
	{PMIC_RG_OSC_75K_TRIM_RATE, PMIC_RG_OSC_75K_TRIM_RATE_ADDR, PMIC_RG_OSC_75K_TRIM_RATE_MASK,
	 PMIC_RG_OSC_75K_TRIM_RATE_SHIFT},
	{PMIC_DA_OSC_75K_TRIM, PMIC_DA_OSC_75K_TRIM_ADDR, PMIC_DA_OSC_75K_TRIM_MASK,
	 PMIC_DA_OSC_75K_TRIM_SHIFT},
	{PMIC_RG_G_SMPS_CK_PDN_SRCLKEN0_EN, PMIC_RG_G_SMPS_CK_PDN_SRCLKEN0_EN_ADDR,
	 PMIC_RG_G_SMPS_CK_PDN_SRCLKEN0_EN_MASK, PMIC_RG_G_SMPS_CK_PDN_SRCLKEN0_EN_SHIFT},
	{PMIC_RG_G_SMPS_CK_PDN_SRCLKEN1_EN, PMIC_RG_G_SMPS_CK_PDN_SRCLKEN1_EN_ADDR,
	 PMIC_RG_G_SMPS_CK_PDN_SRCLKEN1_EN_MASK, PMIC_RG_G_SMPS_CK_PDN_SRCLKEN1_EN_SHIFT},
	{PMIC_RG_G_SMPS_CK_PDN_SRCLKEN2_EN, PMIC_RG_G_SMPS_CK_PDN_SRCLKEN2_EN_ADDR,
	 PMIC_RG_G_SMPS_CK_PDN_SRCLKEN2_EN_MASK, PMIC_RG_G_SMPS_CK_PDN_SRCLKEN2_EN_SHIFT},
	{PMIC_RG_G_SMPS_CK_PDN_BUCK_OSC_SEL_EN, PMIC_RG_G_SMPS_CK_PDN_BUCK_OSC_SEL_EN_ADDR,
	 PMIC_RG_G_SMPS_CK_PDN_BUCK_OSC_SEL_EN_MASK, PMIC_RG_G_SMPS_CK_PDN_BUCK_OSC_SEL_EN_SHIFT},
	{PMIC_RG_G_SMPS_CK_PDN_VOWEN_EN, PMIC_RG_G_SMPS_CK_PDN_VOWEN_EN_ADDR,
	 PMIC_RG_G_SMPS_CK_PDN_VOWEN_EN_MASK, PMIC_RG_G_SMPS_CK_PDN_VOWEN_EN_SHIFT},
	{PMIC_RG_OSC_SEL_SRCLKEN0_EN, PMIC_RG_OSC_SEL_SRCLKEN0_EN_ADDR,
	 PMIC_RG_OSC_SEL_SRCLKEN0_EN_MASK, PMIC_RG_OSC_SEL_SRCLKEN0_EN_SHIFT},
	{PMIC_RG_OSC_SEL_SRCLKEN1_EN, PMIC_RG_OSC_SEL_SRCLKEN1_EN_ADDR,
	 PMIC_RG_OSC_SEL_SRCLKEN1_EN_MASK, PMIC_RG_OSC_SEL_SRCLKEN1_EN_SHIFT},
	{PMIC_RG_OSC_SEL_SRCLKEN2_EN, PMIC_RG_OSC_SEL_SRCLKEN2_EN_ADDR,
	 PMIC_RG_OSC_SEL_SRCLKEN2_EN_MASK, PMIC_RG_OSC_SEL_SRCLKEN2_EN_SHIFT},
	{PMIC_RG_OSC_SEL_BUCK_LDO_EN, PMIC_RG_OSC_SEL_BUCK_LDO_EN_ADDR,
	 PMIC_RG_OSC_SEL_BUCK_LDO_EN_MASK, PMIC_RG_OSC_SEL_BUCK_LDO_EN_SHIFT},
	{PMIC_RG_OSC_SEL_VOWEN_EN, PMIC_RG_OSC_SEL_VOWEN_EN_ADDR, PMIC_RG_OSC_SEL_VOWEN_EN_MASK,
	 PMIC_RG_OSC_SEL_VOWEN_EN_SHIFT},
	{PMIC_RG_OSC_SEL_SPI_EN, PMIC_RG_OSC_SEL_SPI_EN_ADDR, PMIC_RG_OSC_SEL_SPI_EN_MASK,
	 PMIC_RG_OSC_SEL_SPI_EN_SHIFT},
	{PMIC_RG_CLK_RSV, PMIC_RG_CLK_RSV_ADDR, PMIC_RG_CLK_RSV_MASK, PMIC_RG_CLK_RSV_SHIFT},
	{PMIC_TOP_CLK_CON0_SET, PMIC_TOP_CLK_CON0_SET_ADDR, PMIC_TOP_CLK_CON0_SET_MASK,
	 PMIC_TOP_CLK_CON0_SET_SHIFT},
	{PMIC_TOP_CLK_CON0_CLR, PMIC_TOP_CLK_CON0_CLR_ADDR, PMIC_TOP_CLK_CON0_CLR_MASK,
	 PMIC_TOP_CLK_CON0_CLR_SHIFT},
	{PMIC_RG_CLKCTL_RSV0, PMIC_RG_CLKCTL_RSV0_ADDR, PMIC_RG_CLKCTL_RSV0_MASK,
	 PMIC_RG_CLKCTL_RSV0_SHIFT},
	{PMIC_TOP_CLK_CON1_SET, PMIC_TOP_CLK_CON1_SET_ADDR, PMIC_TOP_CLK_CON1_SET_MASK,
	 PMIC_TOP_CLK_CON1_SET_SHIFT},
	{PMIC_TOP_CLK_CON1_CLR, PMIC_TOP_CLK_CON1_CLR_ADDR, PMIC_TOP_CLK_CON1_CLR_MASK,
	 PMIC_TOP_CLK_CON1_CLR_SHIFT},
	{PMIC_RG_DCXO_PWRKEY_RSTB_SEL, PMIC_RG_DCXO_PWRKEY_RSTB_SEL_ADDR,
	 PMIC_RG_DCXO_PWRKEY_RSTB_SEL_MASK, PMIC_RG_DCXO_PWRKEY_RSTB_SEL_SHIFT},
	{PMIC_RG_EFUSE_MAN_RST, PMIC_RG_EFUSE_MAN_RST_ADDR, PMIC_RG_EFUSE_MAN_RST_MASK,
	 PMIC_RG_EFUSE_MAN_RST_SHIFT},
	{PMIC_RG_AUXADC_RST, PMIC_RG_AUXADC_RST_ADDR, PMIC_RG_AUXADC_RST_MASK,
	 PMIC_RG_AUXADC_RST_SHIFT},
	{PMIC_RG_AUXADC_REG_RST, PMIC_RG_AUXADC_REG_RST_ADDR, PMIC_RG_AUXADC_REG_RST_MASK,
	 PMIC_RG_AUXADC_REG_RST_SHIFT},
	{PMIC_RG_AUDIO_RST, PMIC_RG_AUDIO_RST_ADDR, PMIC_RG_AUDIO_RST_MASK,
	 PMIC_RG_AUDIO_RST_SHIFT},
	{PMIC_RG_ACCDET_RST, PMIC_RG_ACCDET_RST_ADDR, PMIC_RG_ACCDET_RST_MASK,
	 PMIC_RG_ACCDET_RST_SHIFT},
	{PMIC_RG_BIF_RST, PMIC_RG_BIF_RST_ADDR, PMIC_RG_BIF_RST_MASK, PMIC_RG_BIF_RST_SHIFT},
	{PMIC_RG_DRIVER_RST, PMIC_RG_DRIVER_RST_ADDR, PMIC_RG_DRIVER_RST_MASK,
	 PMIC_RG_DRIVER_RST_SHIFT},
	{PMIC_RG_FGADC_RST, PMIC_RG_FGADC_RST_ADDR, PMIC_RG_FGADC_RST_MASK,
	 PMIC_RG_FGADC_RST_SHIFT},
	{PMIC_RG_FQMTR_RST, PMIC_RG_FQMTR_RST_ADDR, PMIC_RG_FQMTR_RST_MASK,
	 PMIC_RG_FQMTR_RST_SHIFT},
	{PMIC_RG_RTC_RST, PMIC_RG_RTC_RST_ADDR, PMIC_RG_RTC_RST_MASK, PMIC_RG_RTC_RST_SHIFT},
	{PMIC_RG_TYPE_C_CC_RST, PMIC_RG_TYPE_C_CC_RST_ADDR, PMIC_RG_TYPE_C_CC_RST_MASK,
	 PMIC_RG_TYPE_C_CC_RST_SHIFT},
	{PMIC_RG_CHRWDT_RST, PMIC_RG_CHRWDT_RST_ADDR, PMIC_RG_CHRWDT_RST_MASK,
	 PMIC_RG_CHRWDT_RST_SHIFT},
	{PMIC_RG_ZCD_RST, PMIC_RG_ZCD_RST_ADDR, PMIC_RG_ZCD_RST_MASK, PMIC_RG_ZCD_RST_SHIFT},
	{PMIC_RG_AUDNCP_RST, PMIC_RG_AUDNCP_RST_ADDR, PMIC_RG_AUDNCP_RST_MASK,
	 PMIC_RG_AUDNCP_RST_SHIFT},
	{PMIC_RG_CLK_TRIM_RST, PMIC_RG_CLK_TRIM_RST_ADDR, PMIC_RG_CLK_TRIM_RST_MASK,
	 PMIC_RG_CLK_TRIM_RST_SHIFT},
	{PMIC_RG_BUCK_SRCLKEN_RST, PMIC_RG_BUCK_SRCLKEN_RST_ADDR, PMIC_RG_BUCK_SRCLKEN_RST_MASK,
	 PMIC_RG_BUCK_SRCLKEN_RST_SHIFT},
	{PMIC_TOP_RST_CON0_SET, PMIC_TOP_RST_CON0_SET_ADDR, PMIC_TOP_RST_CON0_SET_MASK,
	 PMIC_TOP_RST_CON0_SET_SHIFT},
	{PMIC_TOP_RST_CON0_CLR, PMIC_TOP_RST_CON0_CLR_ADDR, PMIC_TOP_RST_CON0_CLR_MASK,
	 PMIC_TOP_RST_CON0_CLR_SHIFT},
	{PMIC_RG_STRUP_LONG_PRESS_RST, PMIC_RG_STRUP_LONG_PRESS_RST_ADDR,
	 PMIC_RG_STRUP_LONG_PRESS_RST_MASK, PMIC_RG_STRUP_LONG_PRESS_RST_SHIFT},
	{PMIC_RG_BUCK_PROT_PMPP_RST, PMIC_RG_BUCK_PROT_PMPP_RST_ADDR,
	 PMIC_RG_BUCK_PROT_PMPP_RST_MASK, PMIC_RG_BUCK_PROT_PMPP_RST_SHIFT},
	{PMIC_RG_SPK_RST, PMIC_RG_SPK_RST_ADDR, PMIC_RG_SPK_RST_MASK, PMIC_RG_SPK_RST_SHIFT},
	{PMIC_RG_CHRDET_RST, PMIC_RG_CHRDET_RST_ADDR, PMIC_RG_CHRDET_RST_MASK,
	 PMIC_RG_CHRDET_RST_SHIFT},
	{PMIC_RG_BUCK_LDO_FT_TESTMODE_RST, PMIC_RG_BUCK_LDO_FT_TESTMODE_RST_ADDR,
	 PMIC_RG_BUCK_LDO_FT_TESTMODE_RST_MASK, PMIC_RG_BUCK_LDO_FT_TESTMODE_RST_SHIFT},
	{PMIC_RG_BATON_RST, PMIC_RG_BATON_RST_ADDR, PMIC_RG_BATON_RST_MASK,
	 PMIC_RG_BATON_RST_SHIFT},
	{PMIC_RG_FGADC_RST_SRC_SEL, PMIC_RG_FGADC_RST_SRC_SEL_ADDR, PMIC_RG_FGADC_RST_SRC_SEL_MASK,
	 PMIC_RG_FGADC_RST_SRC_SEL_SHIFT},
	{PMIC_RG_LDO_CALI_RST, PMIC_RG_LDO_CALI_RST_ADDR, PMIC_RG_LDO_CALI_RST_MASK,
	 PMIC_RG_LDO_CALI_RST_SHIFT},
	{PMIC_TOP_RST_CON1_RSV, PMIC_TOP_RST_CON1_RSV_ADDR, PMIC_TOP_RST_CON1_RSV_MASK,
	 PMIC_TOP_RST_CON1_RSV_SHIFT},
	{PMIC_TOP_RST_CON1_SET, PMIC_TOP_RST_CON1_SET_ADDR, PMIC_TOP_RST_CON1_SET_MASK,
	 PMIC_TOP_RST_CON1_SET_SHIFT},
	{PMIC_TOP_RST_CON1_CLR, PMIC_TOP_RST_CON1_CLR_ADDR, PMIC_TOP_RST_CON1_CLR_MASK,
	 PMIC_TOP_RST_CON1_CLR_SHIFT},
	{PMIC_RG_CHR_LDO_DET_MODE, PMIC_RG_CHR_LDO_DET_MODE_ADDR, PMIC_RG_CHR_LDO_DET_MODE_MASK,
	 PMIC_RG_CHR_LDO_DET_MODE_SHIFT},
	{PMIC_RG_CHR_LDO_DET_SW, PMIC_RG_CHR_LDO_DET_SW_ADDR, PMIC_RG_CHR_LDO_DET_SW_MASK,
	 PMIC_RG_CHR_LDO_DET_SW_SHIFT},
	{PMIC_RG_CHRWDT_FLAG_MODE, PMIC_RG_CHRWDT_FLAG_MODE_ADDR, PMIC_RG_CHRWDT_FLAG_MODE_MASK,
	 PMIC_RG_CHRWDT_FLAG_MODE_SHIFT},
	{PMIC_RG_CHRWDT_FLAG_SW, PMIC_RG_CHRWDT_FLAG_SW_ADDR, PMIC_RG_CHRWDT_FLAG_SW_MASK,
	 PMIC_RG_CHRWDT_FLAG_SW_SHIFT},
	{PMIC_TOP_RST_CON2_RSV, PMIC_TOP_RST_CON2_RSV_ADDR, PMIC_TOP_RST_CON2_RSV_MASK,
	 PMIC_TOP_RST_CON2_RSV_SHIFT},
	{PMIC_RG_WDTRSTB_EN, PMIC_RG_WDTRSTB_EN_ADDR, PMIC_RG_WDTRSTB_EN_MASK,
	 PMIC_RG_WDTRSTB_EN_SHIFT},
	{PMIC_RG_WDTRSTB_MODE, PMIC_RG_WDTRSTB_MODE_ADDR, PMIC_RG_WDTRSTB_MODE_MASK,
	 PMIC_RG_WDTRSTB_MODE_SHIFT},
	{PMIC_WDTRSTB_STATUS, PMIC_WDTRSTB_STATUS_ADDR, PMIC_WDTRSTB_STATUS_MASK,
	 PMIC_WDTRSTB_STATUS_SHIFT},
	{PMIC_WDTRSTB_STATUS_CLR, PMIC_WDTRSTB_STATUS_CLR_ADDR, PMIC_WDTRSTB_STATUS_CLR_MASK,
	 PMIC_WDTRSTB_STATUS_CLR_SHIFT},
	{PMIC_RG_WDTRSTB_FB_EN, PMIC_RG_WDTRSTB_FB_EN_ADDR, PMIC_RG_WDTRSTB_FB_EN_MASK,
	 PMIC_RG_WDTRSTB_FB_EN_SHIFT},
	{PMIC_RG_WDTRSTB_DEB, PMIC_RG_WDTRSTB_DEB_ADDR, PMIC_RG_WDTRSTB_DEB_MASK,
	 PMIC_RG_WDTRSTB_DEB_SHIFT},
	{PMIC_RG_HOMEKEY_RST_EN, PMIC_RG_HOMEKEY_RST_EN_ADDR, PMIC_RG_HOMEKEY_RST_EN_MASK,
	 PMIC_RG_HOMEKEY_RST_EN_SHIFT},
	{PMIC_RG_PWRKEY_RST_EN, PMIC_RG_PWRKEY_RST_EN_ADDR, PMIC_RG_PWRKEY_RST_EN_MASK,
	 PMIC_RG_PWRKEY_RST_EN_SHIFT},
	{PMIC_RG_PWRRST_TMR_DIS, PMIC_RG_PWRRST_TMR_DIS_ADDR, PMIC_RG_PWRRST_TMR_DIS_MASK,
	 PMIC_RG_PWRRST_TMR_DIS_SHIFT},
	{PMIC_RG_PWRKEY_RST_TD, PMIC_RG_PWRKEY_RST_TD_ADDR, PMIC_RG_PWRKEY_RST_TD_MASK,
	 PMIC_RG_PWRKEY_RST_TD_SHIFT},
	{PMIC_TOP_RST_MISC_RSV, PMIC_TOP_RST_MISC_RSV_ADDR, PMIC_TOP_RST_MISC_RSV_MASK,
	 PMIC_TOP_RST_MISC_RSV_SHIFT},
	{PMIC_TOP_RST_MISC_SET, PMIC_TOP_RST_MISC_SET_ADDR, PMIC_TOP_RST_MISC_SET_MASK,
	 PMIC_TOP_RST_MISC_SET_SHIFT},
	{PMIC_TOP_RST_MISC_CLR, PMIC_TOP_RST_MISC_CLR_ADDR, PMIC_TOP_RST_MISC_CLR_MASK,
	 PMIC_TOP_RST_MISC_CLR_SHIFT},
	{PMIC_VPWRIN_RSTB_STATUS, PMIC_VPWRIN_RSTB_STATUS_ADDR, PMIC_VPWRIN_RSTB_STATUS_MASK,
	 PMIC_VPWRIN_RSTB_STATUS_SHIFT},
	{PMIC_DDLO_RSTB_STATUS, PMIC_DDLO_RSTB_STATUS_ADDR, PMIC_DDLO_RSTB_STATUS_MASK,
	 PMIC_DDLO_RSTB_STATUS_SHIFT},
	{PMIC_UVLO_RSTB_STATUS, PMIC_UVLO_RSTB_STATUS_ADDR, PMIC_UVLO_RSTB_STATUS_MASK,
	 PMIC_UVLO_RSTB_STATUS_SHIFT},
	{PMIC_RTC_DDLO_RSTB_STATUS, PMIC_RTC_DDLO_RSTB_STATUS_ADDR, PMIC_RTC_DDLO_RSTB_STATUS_MASK,
	 PMIC_RTC_DDLO_RSTB_STATUS_SHIFT},
	{PMIC_CHRWDT_REG_RSTB_STATUS, PMIC_CHRWDT_REG_RSTB_STATUS_ADDR,
	 PMIC_CHRWDT_REG_RSTB_STATUS_MASK, PMIC_CHRWDT_REG_RSTB_STATUS_SHIFT},
	{PMIC_CHRDET_REG_RSTB_STATUS, PMIC_CHRDET_REG_RSTB_STATUS_ADDR,
	 PMIC_CHRDET_REG_RSTB_STATUS_MASK, PMIC_CHRDET_REG_RSTB_STATUS_SHIFT},
	{PMIC_BWDT_DDLO_RSTB_STATUS, PMIC_BWDT_DDLO_RSTB_STATUS_ADDR,
	 PMIC_BWDT_DDLO_RSTB_STATUS_MASK, PMIC_BWDT_DDLO_RSTB_STATUS_SHIFT},
	{PMIC_TOP_RST_STATUS_RSV, PMIC_TOP_RST_STATUS_RSV_ADDR, PMIC_TOP_RST_STATUS_RSV_MASK,
	 PMIC_TOP_RST_STATUS_RSV_SHIFT},
	{PMIC_TOP_RST_STATUS_SET, PMIC_TOP_RST_STATUS_SET_ADDR, PMIC_TOP_RST_STATUS_SET_MASK,
	 PMIC_TOP_RST_STATUS_SET_SHIFT},
	{PMIC_TOP_RST_STATUS_CLR, PMIC_TOP_RST_STATUS_CLR_ADDR, PMIC_TOP_RST_STATUS_CLR_MASK,
	 PMIC_TOP_RST_STATUS_CLR_SHIFT},
	{PMIC_TOP_RST_RSV_CON0, PMIC_TOP_RST_RSV_CON0_ADDR, PMIC_TOP_RST_RSV_CON0_MASK,
	 PMIC_TOP_RST_RSV_CON0_SHIFT},
	{PMIC_TOP_RST_RSV_CON1, PMIC_TOP_RST_RSV_CON1_ADDR, PMIC_TOP_RST_RSV_CON1_MASK,
	 PMIC_TOP_RST_RSV_CON1_SHIFT},
	{PMIC_RG_INT_EN_PWRKEY, PMIC_RG_INT_EN_PWRKEY_ADDR, PMIC_RG_INT_EN_PWRKEY_MASK,
	 PMIC_RG_INT_EN_PWRKEY_SHIFT},
	{PMIC_RG_INT_EN_HOMEKEY, PMIC_RG_INT_EN_HOMEKEY_ADDR, PMIC_RG_INT_EN_HOMEKEY_MASK,
	 PMIC_RG_INT_EN_HOMEKEY_SHIFT},
	{PMIC_RG_INT_EN_PWRKEY_R, PMIC_RG_INT_EN_PWRKEY_R_ADDR, PMIC_RG_INT_EN_PWRKEY_R_MASK,
	 PMIC_RG_INT_EN_PWRKEY_R_SHIFT},
	{PMIC_RG_INT_EN_HOMEKEY_R, PMIC_RG_INT_EN_HOMEKEY_R_ADDR, PMIC_RG_INT_EN_HOMEKEY_R_MASK,
	 PMIC_RG_INT_EN_HOMEKEY_R_SHIFT},
	{PMIC_RG_INT_EN_NI_LBAT_INT, PMIC_RG_INT_EN_NI_LBAT_INT_ADDR,
	 PMIC_RG_INT_EN_NI_LBAT_INT_MASK, PMIC_RG_INT_EN_NI_LBAT_INT_SHIFT},
	{PMIC_RG_INT_EN_CHRDET, PMIC_RG_INT_EN_CHRDET_ADDR, PMIC_RG_INT_EN_CHRDET_MASK,
	 PMIC_RG_INT_EN_CHRDET_SHIFT},
	{PMIC_RG_INT_EN_CHRDET_EDGE, PMIC_RG_INT_EN_CHRDET_EDGE_ADDR,
	 PMIC_RG_INT_EN_CHRDET_EDGE_MASK, PMIC_RG_INT_EN_CHRDET_EDGE_SHIFT},
	{PMIC_RG_INT_EN_BATON_LV, PMIC_RG_INT_EN_BATON_LV_ADDR, PMIC_RG_INT_EN_BATON_LV_MASK,
	 PMIC_RG_INT_EN_BATON_LV_SHIFT},
	{PMIC_RG_INT_EN_BATON_HV, PMIC_RG_INT_EN_BATON_HV_ADDR, PMIC_RG_INT_EN_BATON_HV_MASK,
	 PMIC_RG_INT_EN_BATON_HV_SHIFT},
	{PMIC_RG_INT_EN_BATON_BAT_IN, PMIC_RG_INT_EN_BATON_BAT_IN_ADDR,
	 PMIC_RG_INT_EN_BATON_BAT_IN_MASK, PMIC_RG_INT_EN_BATON_BAT_IN_SHIFT},
	{PMIC_RG_INT_EN_BATON_BAT_OUT, PMIC_RG_INT_EN_BATON_BAT_OUT_ADDR,
	 PMIC_RG_INT_EN_BATON_BAT_OUT_MASK, PMIC_RG_INT_EN_BATON_BAT_OUT_SHIFT},
	{PMIC_RG_INT_EN_RTC, PMIC_RG_INT_EN_RTC_ADDR, PMIC_RG_INT_EN_RTC_MASK,
	 PMIC_RG_INT_EN_RTC_SHIFT},
	{PMIC_RG_INT_EN_BIF, PMIC_RG_INT_EN_BIF_ADDR, PMIC_RG_INT_EN_BIF_MASK,
	 PMIC_RG_INT_EN_BIF_SHIFT},
	{PMIC_RG_INT_EN_VCDT_HV_DET, PMIC_RG_INT_EN_VCDT_HV_DET_ADDR,
	 PMIC_RG_INT_EN_VCDT_HV_DET_MASK, PMIC_RG_INT_EN_VCDT_HV_DET_SHIFT},
	{PMIC_INT_CON0_SET, PMIC_INT_CON0_SET_ADDR, PMIC_INT_CON0_SET_MASK,
	 PMIC_INT_CON0_SET_SHIFT},
	{PMIC_INT_CON0_CLR, PMIC_INT_CON0_CLR_ADDR, PMIC_INT_CON0_CLR_MASK,
	 PMIC_INT_CON0_CLR_SHIFT},
	{PMIC_RG_INT_EN_THR_H, PMIC_RG_INT_EN_THR_H_ADDR, PMIC_RG_INT_EN_THR_H_MASK,
	 PMIC_RG_INT_EN_THR_H_SHIFT},
	{PMIC_RG_INT_EN_THR_L, PMIC_RG_INT_EN_THR_L_ADDR, PMIC_RG_INT_EN_THR_L_MASK,
	 PMIC_RG_INT_EN_THR_L_SHIFT},
	{PMIC_RG_INT_EN_BAT_H, PMIC_RG_INT_EN_BAT_H_ADDR, PMIC_RG_INT_EN_BAT_H_MASK,
	 PMIC_RG_INT_EN_BAT_H_SHIFT},
	{PMIC_RG_INT_EN_BAT_L, PMIC_RG_INT_EN_BAT_L_ADDR, PMIC_RG_INT_EN_BAT_L_MASK,
	 PMIC_RG_INT_EN_BAT_L_SHIFT},
	{PMIC_RG_INT_EN_BAT2_H, PMIC_RG_INT_EN_BAT2_H_ADDR, PMIC_RG_INT_EN_BAT2_H_MASK,
	 PMIC_RG_INT_EN_BAT2_H_SHIFT},
	{PMIC_RG_INT_EN_BAT2_L, PMIC_RG_INT_EN_BAT2_L_ADDR, PMIC_RG_INT_EN_BAT2_L_MASK,
	 PMIC_RG_INT_EN_BAT2_L_SHIFT},
	{PMIC_RG_INT_EN_BAT_TEMP_H, PMIC_RG_INT_EN_BAT_TEMP_H_ADDR, PMIC_RG_INT_EN_BAT_TEMP_H_MASK,
	 PMIC_RG_INT_EN_BAT_TEMP_H_SHIFT},
	{PMIC_RG_INT_EN_BAT_TEMP_L, PMIC_RG_INT_EN_BAT_TEMP_L_ADDR, PMIC_RG_INT_EN_BAT_TEMP_L_MASK,
	 PMIC_RG_INT_EN_BAT_TEMP_L_SHIFT},
	{PMIC_RG_INT_EN_AUXADC_IMP, PMIC_RG_INT_EN_AUXADC_IMP_ADDR, PMIC_RG_INT_EN_AUXADC_IMP_MASK,
	 PMIC_RG_INT_EN_AUXADC_IMP_SHIFT},
	{PMIC_RG_INT_EN_NAG_C_DLTV, PMIC_RG_INT_EN_NAG_C_DLTV_ADDR, PMIC_RG_INT_EN_NAG_C_DLTV_MASK,
	 PMIC_RG_INT_EN_NAG_C_DLTV_SHIFT},
	{PMIC_RG_INT_EN_JEITA_HOT, PMIC_RG_INT_EN_JEITA_HOT_ADDR, PMIC_RG_INT_EN_JEITA_HOT_MASK,
	 PMIC_RG_INT_EN_JEITA_HOT_SHIFT},
	{PMIC_RG_INT_EN_JEITA_WARM, PMIC_RG_INT_EN_JEITA_WARM_ADDR, PMIC_RG_INT_EN_JEITA_WARM_MASK,
	 PMIC_RG_INT_EN_JEITA_WARM_SHIFT},
	{PMIC_RG_INT_EN_JEITA_COOL, PMIC_RG_INT_EN_JEITA_COOL_ADDR, PMIC_RG_INT_EN_JEITA_COOL_MASK,
	 PMIC_RG_INT_EN_JEITA_COOL_SHIFT},
	{PMIC_RG_INT_EN_JEITA_COLD, PMIC_RG_INT_EN_JEITA_COLD_ADDR, PMIC_RG_INT_EN_JEITA_COLD_MASK,
	 PMIC_RG_INT_EN_JEITA_COLD_SHIFT},
	{PMIC_INT_CON1_SET, PMIC_INT_CON1_SET_ADDR, PMIC_INT_CON1_SET_MASK,
	 PMIC_INT_CON1_SET_SHIFT},
	{PMIC_INT_CON1_CLR, PMIC_INT_CON1_CLR_ADDR, PMIC_INT_CON1_CLR_MASK,
	 PMIC_INT_CON1_CLR_SHIFT},
	{PMIC_RG_INT_EN_VCORE_OC, PMIC_RG_INT_EN_VCORE_OC_ADDR, PMIC_RG_INT_EN_VCORE_OC_MASK,
	 PMIC_RG_INT_EN_VCORE_OC_SHIFT},
	{PMIC_RG_INT_EN_VMD1_OC, PMIC_RG_INT_EN_VMD1_OC_ADDR, PMIC_RG_INT_EN_VMD1_OC_MASK,
	 PMIC_RG_INT_EN_VMD1_OC_SHIFT},
	{PMIC_RG_INT_EN_VMODEM_OC, PMIC_RG_INT_EN_VMODEM_OC_ADDR, PMIC_RG_INT_EN_VMODEM_OC_MASK,
	 PMIC_RG_INT_EN_VMODEM_OC_SHIFT},
	{PMIC_RG_INT_EN_VS1_OC, PMIC_RG_INT_EN_VS1_OC_ADDR, PMIC_RG_INT_EN_VS1_OC_MASK,
	 PMIC_RG_INT_EN_VS1_OC_SHIFT},
	{PMIC_RG_INT_EN_VS2_OC, PMIC_RG_INT_EN_VS2_OC_ADDR, PMIC_RG_INT_EN_VS2_OC_MASK,
	 PMIC_RG_INT_EN_VS2_OC_SHIFT},
	{PMIC_RG_INT_EN_VDRAM_OC, PMIC_RG_INT_EN_VDRAM_OC_ADDR, PMIC_RG_INT_EN_VDRAM_OC_MASK,
	 PMIC_RG_INT_EN_VDRAM_OC_SHIFT},
	{PMIC_RG_INT_EN_VPA1_OC, PMIC_RG_INT_EN_VPA1_OC_ADDR, PMIC_RG_INT_EN_VPA1_OC_MASK,
	 PMIC_RG_INT_EN_VPA1_OC_SHIFT},
	{PMIC_RG_INT_EN_VPA2_OC, PMIC_RG_INT_EN_VPA2_OC_ADDR, PMIC_RG_INT_EN_VPA2_OC_MASK,
	 PMIC_RG_INT_EN_VPA2_OC_SHIFT},
	{PMIC_RG_INT_EN_VCORE_PREOC, PMIC_RG_INT_EN_VCORE_PREOC_ADDR,
	 PMIC_RG_INT_EN_VCORE_PREOC_MASK, PMIC_RG_INT_EN_VCORE_PREOC_SHIFT},
	{PMIC_RG_INT_EN_VA10_OC, PMIC_RG_INT_EN_VA10_OC_ADDR, PMIC_RG_INT_EN_VA10_OC_MASK,
	 PMIC_RG_INT_EN_VA10_OC_SHIFT},
	{PMIC_RG_INT_EN_VA12_OC, PMIC_RG_INT_EN_VA12_OC_ADDR, PMIC_RG_INT_EN_VA12_OC_MASK,
	 PMIC_RG_INT_EN_VA12_OC_SHIFT},
	{PMIC_RG_INT_EN_VA18_OC, PMIC_RG_INT_EN_VA18_OC_ADDR, PMIC_RG_INT_EN_VA18_OC_MASK,
	 PMIC_RG_INT_EN_VA18_OC_SHIFT},
	{PMIC_RG_INT_EN_VBIF28_OC, PMIC_RG_INT_EN_VBIF28_OC_ADDR, PMIC_RG_INT_EN_VBIF28_OC_MASK,
	 PMIC_RG_INT_EN_VBIF28_OC_SHIFT},
	{PMIC_RG_INT_EN_VCAMA1_OC, PMIC_RG_INT_EN_VCAMA1_OC_ADDR, PMIC_RG_INT_EN_VCAMA1_OC_MASK,
	 PMIC_RG_INT_EN_VCAMA1_OC_SHIFT},
	{PMIC_RG_INT_EN_VCAMA2_OC, PMIC_RG_INT_EN_VCAMA2_OC_ADDR, PMIC_RG_INT_EN_VCAMA2_OC_MASK,
	 PMIC_RG_INT_EN_VCAMA2_OC_SHIFT},
	{PMIC_RG_INT_EN_VCAMAF_OC, PMIC_RG_INT_EN_VCAMAF_OC_ADDR, PMIC_RG_INT_EN_VCAMAF_OC_MASK,
	 PMIC_RG_INT_EN_VCAMAF_OC_SHIFT},
	{PMIC_INT_CON2_SET, PMIC_INT_CON2_SET_ADDR, PMIC_INT_CON2_SET_MASK,
	 PMIC_INT_CON2_SET_SHIFT},
	{PMIC_INT_CON2_CLR, PMIC_INT_CON2_CLR_ADDR, PMIC_INT_CON2_CLR_MASK,
	 PMIC_INT_CON2_CLR_SHIFT},
	{PMIC_RG_INT_EN_VCAMD1_OC, PMIC_RG_INT_EN_VCAMD1_OC_ADDR, PMIC_RG_INT_EN_VCAMD1_OC_MASK,
	 PMIC_RG_INT_EN_VCAMD1_OC_SHIFT},
	{PMIC_RG_INT_EN_VCAMD2_OC, PMIC_RG_INT_EN_VCAMD2_OC_ADDR, PMIC_RG_INT_EN_VCAMD2_OC_MASK,
	 PMIC_RG_INT_EN_VCAMD2_OC_SHIFT},
	{PMIC_RG_INT_EN_VCAMIO_OC, PMIC_RG_INT_EN_VCAMIO_OC_ADDR, PMIC_RG_INT_EN_VCAMIO_OC_MASK,
	 PMIC_RG_INT_EN_VCAMIO_OC_SHIFT},
	{PMIC_RG_INT_EN_VCN18_OC, PMIC_RG_INT_EN_VCN18_OC_ADDR, PMIC_RG_INT_EN_VCN18_OC_MASK,
	 PMIC_RG_INT_EN_VCN18_OC_SHIFT},
	{PMIC_RG_INT_EN_VCN28_OC, PMIC_RG_INT_EN_VCN28_OC_ADDR, PMIC_RG_INT_EN_VCN28_OC_MASK,
	 PMIC_RG_INT_EN_VCN28_OC_SHIFT},
	{PMIC_RG_INT_EN_VCN33_OC, PMIC_RG_INT_EN_VCN33_OC_ADDR, PMIC_RG_INT_EN_VCN33_OC_MASK,
	 PMIC_RG_INT_EN_VCN33_OC_SHIFT},
	{PMIC_RG_INT_EN_VEFUSE_OC, PMIC_RG_INT_EN_VEFUSE_OC_ADDR, PMIC_RG_INT_EN_VEFUSE_OC_MASK,
	 PMIC_RG_INT_EN_VEFUSE_OC_SHIFT},
	{PMIC_RG_INT_EN_VEMC_OC, PMIC_RG_INT_EN_VEMC_OC_ADDR, PMIC_RG_INT_EN_VEMC_OC_MASK,
	 PMIC_RG_INT_EN_VEMC_OC_SHIFT},
	{PMIC_RG_INT_EN_VFE28_OC, PMIC_RG_INT_EN_VFE28_OC_ADDR, PMIC_RG_INT_EN_VFE28_OC_MASK,
	 PMIC_RG_INT_EN_VFE28_OC_SHIFT},
	{PMIC_RG_INT_EN_VGP3_OC, PMIC_RG_INT_EN_VGP3_OC_ADDR, PMIC_RG_INT_EN_VGP3_OC_MASK,
	 PMIC_RG_INT_EN_VGP3_OC_SHIFT},
	{PMIC_RG_INT_EN_VIBR_OC, PMIC_RG_INT_EN_VIBR_OC_ADDR, PMIC_RG_INT_EN_VIBR_OC_MASK,
	 PMIC_RG_INT_EN_VIBR_OC_SHIFT},
	{PMIC_RG_INT_EN_VIO18_OC, PMIC_RG_INT_EN_VIO18_OC_ADDR, PMIC_RG_INT_EN_VIO18_OC_MASK,
	 PMIC_RG_INT_EN_VIO18_OC_SHIFT},
	{PMIC_RG_INT_EN_VIO28_OC, PMIC_RG_INT_EN_VIO28_OC_ADDR, PMIC_RG_INT_EN_VIO28_OC_MASK,
	 PMIC_RG_INT_EN_VIO28_OC_SHIFT},
	{PMIC_RG_INT_EN_VMC_OC, PMIC_RG_INT_EN_VMC_OC_ADDR, PMIC_RG_INT_EN_VMC_OC_MASK,
	 PMIC_RG_INT_EN_VMC_OC_SHIFT},
	{PMIC_RG_INT_EN_VMCH_OC, PMIC_RG_INT_EN_VMCH_OC_ADDR, PMIC_RG_INT_EN_VMCH_OC_MASK,
	 PMIC_RG_INT_EN_VMCH_OC_SHIFT},
	{PMIC_RG_INT_EN_VMIPI_OC, PMIC_RG_INT_EN_VMIPI_OC_ADDR, PMIC_RG_INT_EN_VMIPI_OC_MASK,
	 PMIC_RG_INT_EN_VMIPI_OC_SHIFT},
	{PMIC_INT_CON3_SET, PMIC_INT_CON3_SET_ADDR, PMIC_INT_CON3_SET_MASK,
	 PMIC_INT_CON3_SET_SHIFT},
	{PMIC_INT_CON3_CLR, PMIC_INT_CON3_CLR_ADDR, PMIC_INT_CON3_CLR_MASK,
	 PMIC_INT_CON3_CLR_SHIFT},
	{PMIC_RG_INT_EN_VRF12_OC, PMIC_RG_INT_EN_VRF12_OC_ADDR, PMIC_RG_INT_EN_VRF12_OC_MASK,
	 PMIC_RG_INT_EN_VRF12_OC_SHIFT},
	{PMIC_RG_INT_EN_VRF18_1_OC, PMIC_RG_INT_EN_VRF18_1_OC_ADDR, PMIC_RG_INT_EN_VRF18_1_OC_MASK,
	 PMIC_RG_INT_EN_VRF18_1_OC_SHIFT},
	{PMIC_RG_INT_EN_VRF18_2_OC, PMIC_RG_INT_EN_VRF18_2_OC_ADDR, PMIC_RG_INT_EN_VRF18_2_OC_MASK,
	 PMIC_RG_INT_EN_VRF18_2_OC_SHIFT},
	{PMIC_RG_INT_EN_VSIM1_OC, PMIC_RG_INT_EN_VSIM1_OC_ADDR, PMIC_RG_INT_EN_VSIM1_OC_MASK,
	 PMIC_RG_INT_EN_VSIM1_OC_SHIFT},
	{PMIC_RG_INT_EN_VSIM2_OC, PMIC_RG_INT_EN_VSIM2_OC_ADDR, PMIC_RG_INT_EN_VSIM2_OC_MASK,
	 PMIC_RG_INT_EN_VSIM2_OC_SHIFT},
	{PMIC_RG_INT_EN_VSRAM_CORE_OC, PMIC_RG_INT_EN_VSRAM_CORE_OC_ADDR,
	 PMIC_RG_INT_EN_VSRAM_CORE_OC_MASK, PMIC_RG_INT_EN_VSRAM_CORE_OC_SHIFT},
	{PMIC_RG_INT_EN_VSRAM_DVFS1_OC, PMIC_RG_INT_EN_VSRAM_DVFS1_OC_ADDR,
	 PMIC_RG_INT_EN_VSRAM_DVFS1_OC_MASK, PMIC_RG_INT_EN_VSRAM_DVFS1_OC_SHIFT},
	{PMIC_RG_INT_EN_VSRAM_DVFS2_OC, PMIC_RG_INT_EN_VSRAM_DVFS2_OC_ADDR,
	 PMIC_RG_INT_EN_VSRAM_DVFS2_OC_MASK, PMIC_RG_INT_EN_VSRAM_DVFS2_OC_SHIFT},
	{PMIC_RG_INT_EN_VSRAM_GPU_OC, PMIC_RG_INT_EN_VSRAM_GPU_OC_ADDR,
	 PMIC_RG_INT_EN_VSRAM_GPU_OC_MASK, PMIC_RG_INT_EN_VSRAM_GPU_OC_SHIFT},
	{PMIC_RG_INT_EN_VSRAM_MD_OC, PMIC_RG_INT_EN_VSRAM_MD_OC_ADDR,
	 PMIC_RG_INT_EN_VSRAM_MD_OC_MASK, PMIC_RG_INT_EN_VSRAM_MD_OC_SHIFT},
	{PMIC_RG_INT_EN_VUFS18_OC, PMIC_RG_INT_EN_VUFS18_OC_ADDR, PMIC_RG_INT_EN_VUFS18_OC_MASK,
	 PMIC_RG_INT_EN_VUFS18_OC_SHIFT},
	{PMIC_RG_INT_EN_VUSB33_OC, PMIC_RG_INT_EN_VUSB33_OC_ADDR, PMIC_RG_INT_EN_VUSB33_OC_MASK,
	 PMIC_RG_INT_EN_VUSB33_OC_SHIFT},
	{PMIC_RG_INT_EN_VXO22_OC, PMIC_RG_INT_EN_VXO22_OC_ADDR, PMIC_RG_INT_EN_VXO22_OC_MASK,
	 PMIC_RG_INT_EN_VXO22_OC_SHIFT},
	{PMIC_RG_INT_EN_CON4, PMIC_RG_INT_EN_CON4_ADDR, PMIC_RG_INT_EN_CON4_MASK,
	 PMIC_RG_INT_EN_CON4_SHIFT},
	{PMIC_INT_CON4_SET, PMIC_INT_CON4_SET_ADDR, PMIC_INT_CON4_SET_MASK,
	 PMIC_INT_CON4_SET_SHIFT},
	{PMIC_INT_CON4_CLR, PMIC_INT_CON4_CLR_ADDR, PMIC_INT_CON4_CLR_MASK,
	 PMIC_INT_CON4_CLR_SHIFT},
	{PMIC_RG_INT_EN_FG_BAT0_H, PMIC_RG_INT_EN_FG_BAT0_H_ADDR, PMIC_RG_INT_EN_FG_BAT0_H_MASK,
	 PMIC_RG_INT_EN_FG_BAT0_H_SHIFT},
	{PMIC_RG_INT_EN_FG_BAT0_L, PMIC_RG_INT_EN_FG_BAT0_L_ADDR, PMIC_RG_INT_EN_FG_BAT0_L_MASK,
	 PMIC_RG_INT_EN_FG_BAT0_L_SHIFT},
	{PMIC_RG_INT_EN_FG_CUR_H, PMIC_RG_INT_EN_FG_CUR_H_ADDR, PMIC_RG_INT_EN_FG_CUR_H_MASK,
	 PMIC_RG_INT_EN_FG_CUR_H_SHIFT},
	{PMIC_RG_INT_EN_FG_CUR_L, PMIC_RG_INT_EN_FG_CUR_L_ADDR, PMIC_RG_INT_EN_FG_CUR_L_MASK,
	 PMIC_RG_INT_EN_FG_CUR_L_SHIFT},
	{PMIC_RG_INT_EN_FG_ZCV, PMIC_RG_INT_EN_FG_ZCV_ADDR, PMIC_RG_INT_EN_FG_ZCV_MASK,
	 PMIC_RG_INT_EN_FG_ZCV_SHIFT},
	{PMIC_RG_INT_EN_FG_BAT1_H, PMIC_RG_INT_EN_FG_BAT1_H_ADDR, PMIC_RG_INT_EN_FG_BAT1_H_MASK,
	 PMIC_RG_INT_EN_FG_BAT1_H_SHIFT},
	{PMIC_RG_INT_EN_FG_BAT1_L, PMIC_RG_INT_EN_FG_BAT1_L_ADDR, PMIC_RG_INT_EN_FG_BAT1_L_MASK,
	 PMIC_RG_INT_EN_FG_BAT1_L_SHIFT},
	{PMIC_RG_INT_EN_FG_N_CHARGE_L, PMIC_RG_INT_EN_FG_N_CHARGE_L_ADDR,
	 PMIC_RG_INT_EN_FG_N_CHARGE_L_MASK, PMIC_RG_INT_EN_FG_N_CHARGE_L_SHIFT},
	{PMIC_RG_INT_EN_FG_IAVG_H, PMIC_RG_INT_EN_FG_IAVG_H_ADDR, PMIC_RG_INT_EN_FG_IAVG_H_MASK,
	 PMIC_RG_INT_EN_FG_IAVG_H_SHIFT},
	{PMIC_RG_INT_EN_FG_IAVG_L, PMIC_RG_INT_EN_FG_IAVG_L_ADDR, PMIC_RG_INT_EN_FG_IAVG_L_MASK,
	 PMIC_RG_INT_EN_FG_IAVG_L_SHIFT},
	{PMIC_RG_INT_EN_FG_TIME_H, PMIC_RG_INT_EN_FG_TIME_H_ADDR, PMIC_RG_INT_EN_FG_TIME_H_MASK,
	 PMIC_RG_INT_EN_FG_TIME_H_SHIFT},
	{PMIC_RG_INT_EN_FG_DISCHARGE, PMIC_RG_INT_EN_FG_DISCHARGE_ADDR,
	 PMIC_RG_INT_EN_FG_DISCHARGE_MASK, PMIC_RG_INT_EN_FG_DISCHARGE_SHIFT},
	{PMIC_RG_INT_EN_FG_CHARGE, PMIC_RG_INT_EN_FG_CHARGE_ADDR, PMIC_RG_INT_EN_FG_CHARGE_MASK,
	 PMIC_RG_INT_EN_FG_CHARGE_SHIFT},
	{PMIC_RG_INT_EN_CON5, PMIC_RG_INT_EN_CON5_ADDR, PMIC_RG_INT_EN_CON5_MASK,
	 PMIC_RG_INT_EN_CON5_SHIFT},
	{PMIC_INT_CON5_SET, PMIC_INT_CON5_SET_ADDR, PMIC_INT_CON5_SET_MASK,
	 PMIC_INT_CON5_SET_SHIFT},
	{PMIC_INT_CON5_CLR, PMIC_INT_CON5_CLR_ADDR, PMIC_INT_CON5_CLR_MASK,
	 PMIC_INT_CON5_CLR_SHIFT},
	{PMIC_RG_INT_MASK_PWRKEY, PMIC_RG_INT_MASK_PWRKEY_ADDR, PMIC_RG_INT_MASK_PWRKEY_MASK,
	 PMIC_RG_INT_MASK_PWRKEY_SHIFT},
	{PMIC_RG_INT_MASK_HOMEKEY, PMIC_RG_INT_MASK_HOMEKEY_ADDR, PMIC_RG_INT_MASK_HOMEKEY_MASK,
	 PMIC_RG_INT_MASK_HOMEKEY_SHIFT},
	{PMIC_RG_INT_MASK_PWRKEY_R, PMIC_RG_INT_MASK_PWRKEY_R_ADDR, PMIC_RG_INT_MASK_PWRKEY_R_MASK,
	 PMIC_RG_INT_MASK_PWRKEY_R_SHIFT},
	{PMIC_RG_INT_MASK_HOMEKEY_R, PMIC_RG_INT_MASK_HOMEKEY_R_ADDR,
	 PMIC_RG_INT_MASK_HOMEKEY_R_MASK, PMIC_RG_INT_MASK_HOMEKEY_R_SHIFT},
	{PMIC_RG_INT_MASK_NI_LBAT_INT, PMIC_RG_INT_MASK_NI_LBAT_INT_ADDR,
	 PMIC_RG_INT_MASK_NI_LBAT_INT_MASK, PMIC_RG_INT_MASK_NI_LBAT_INT_SHIFT},
	{PMIC_RG_INT_MASK_CHRDET, PMIC_RG_INT_MASK_CHRDET_ADDR, PMIC_RG_INT_MASK_CHRDET_MASK,
	 PMIC_RG_INT_MASK_CHRDET_SHIFT},
	{PMIC_RG_INT_MASK_CHRDET_EDGE, PMIC_RG_INT_MASK_CHRDET_EDGE_ADDR,
	 PMIC_RG_INT_MASK_CHRDET_EDGE_MASK, PMIC_RG_INT_MASK_CHRDET_EDGE_SHIFT},
	{PMIC_RG_INT_MASK_BATON_LV, PMIC_RG_INT_MASK_BATON_LV_ADDR, PMIC_RG_INT_MASK_BATON_LV_MASK,
	 PMIC_RG_INT_MASK_BATON_LV_SHIFT},
	{PMIC_RG_INT_MASK_BATON_HV, PMIC_RG_INT_MASK_BATON_HV_ADDR, PMIC_RG_INT_MASK_BATON_HV_MASK,
	 PMIC_RG_INT_MASK_BATON_HV_SHIFT},
	{PMIC_RG_INT_MASK_BATON_BAT_IN, PMIC_RG_INT_MASK_BATON_BAT_IN_ADDR,
	 PMIC_RG_INT_MASK_BATON_BAT_IN_MASK, PMIC_RG_INT_MASK_BATON_BAT_IN_SHIFT},
	{PMIC_RG_INT_MASK_BATON_BAT_OUT, PMIC_RG_INT_MASK_BATON_BAT_OUT_ADDR,
	 PMIC_RG_INT_MASK_BATON_BAT_OUT_MASK, PMIC_RG_INT_MASK_BATON_BAT_OUT_SHIFT},
	{PMIC_RG_INT_MASK_RTC, PMIC_RG_INT_MASK_RTC_ADDR, PMIC_RG_INT_MASK_RTC_MASK,
	 PMIC_RG_INT_MASK_RTC_SHIFT},
	{PMIC_RG_INT_MASK_BIF, PMIC_RG_INT_MASK_BIF_ADDR, PMIC_RG_INT_MASK_BIF_MASK,
	 PMIC_RG_INT_MASK_BIF_SHIFT},
	{PMIC_RG_INT_MASK_VCDT_HV_DET, PMIC_RG_INT_MASK_VCDT_HV_DET_ADDR,
	 PMIC_RG_INT_MASK_VCDT_HV_DET_MASK, PMIC_RG_INT_MASK_VCDT_HV_DET_SHIFT},
	{PMIC_INT_MASK_CON0_SET, PMIC_INT_MASK_CON0_SET_ADDR, PMIC_INT_MASK_CON0_SET_MASK,
	 PMIC_INT_MASK_CON0_SET_SHIFT},
	{PMIC_INT_MASK_CON0_CLR, PMIC_INT_MASK_CON0_CLR_ADDR, PMIC_INT_MASK_CON0_CLR_MASK,
	 PMIC_INT_MASK_CON0_CLR_SHIFT},
	{PMIC_RG_INT_MASK_THR_H, PMIC_RG_INT_MASK_THR_H_ADDR, PMIC_RG_INT_MASK_THR_H_MASK,
	 PMIC_RG_INT_MASK_THR_H_SHIFT},
	{PMIC_RG_INT_MASK_THR_L, PMIC_RG_INT_MASK_THR_L_ADDR, PMIC_RG_INT_MASK_THR_L_MASK,
	 PMIC_RG_INT_MASK_THR_L_SHIFT},
	{PMIC_RG_INT_MASK_BAT_H, PMIC_RG_INT_MASK_BAT_H_ADDR, PMIC_RG_INT_MASK_BAT_H_MASK,
	 PMIC_RG_INT_MASK_BAT_H_SHIFT},
	{PMIC_RG_INT_MASK_BAT_L, PMIC_RG_INT_MASK_BAT_L_ADDR, PMIC_RG_INT_MASK_BAT_L_MASK,
	 PMIC_RG_INT_MASK_BAT_L_SHIFT},
	{PMIC_RG_INT_MASK_BAT2_H, PMIC_RG_INT_MASK_BAT2_H_ADDR, PMIC_RG_INT_MASK_BAT2_H_MASK,
	 PMIC_RG_INT_MASK_BAT2_H_SHIFT},
	{PMIC_RG_INT_MASK_BAT2_L, PMIC_RG_INT_MASK_BAT2_L_ADDR, PMIC_RG_INT_MASK_BAT2_L_MASK,
	 PMIC_RG_INT_MASK_BAT2_L_SHIFT},
	{PMIC_RG_INT_MASK_BAT_TEMP_H, PMIC_RG_INT_MASK_BAT_TEMP_H_ADDR,
	 PMIC_RG_INT_MASK_BAT_TEMP_H_MASK, PMIC_RG_INT_MASK_BAT_TEMP_H_SHIFT},
	{PMIC_RG_INT_MASK_BAT_TEMP_L, PMIC_RG_INT_MASK_BAT_TEMP_L_ADDR,
	 PMIC_RG_INT_MASK_BAT_TEMP_L_MASK, PMIC_RG_INT_MASK_BAT_TEMP_L_SHIFT},
	{PMIC_RG_INT_MASK_AUXADC_IMP, PMIC_RG_INT_MASK_AUXADC_IMP_ADDR,
	 PMIC_RG_INT_MASK_AUXADC_IMP_MASK, PMIC_RG_INT_MASK_AUXADC_IMP_SHIFT},
	{PMIC_RG_INT_MASK_NAG_C_DLTV, PMIC_RG_INT_MASK_NAG_C_DLTV_ADDR,
	 PMIC_RG_INT_MASK_NAG_C_DLTV_MASK, PMIC_RG_INT_MASK_NAG_C_DLTV_SHIFT},
	{PMIC_RG_INT_MASK_JEITA_HOT, PMIC_RG_INT_MASK_JEITA_HOT_ADDR,
	 PMIC_RG_INT_MASK_JEITA_HOT_MASK, PMIC_RG_INT_MASK_JEITA_HOT_SHIFT},
	{PMIC_RG_INT_MASK_JEITA_WARM, PMIC_RG_INT_MASK_JEITA_WARM_ADDR,
	 PMIC_RG_INT_MASK_JEITA_WARM_MASK, PMIC_RG_INT_MASK_JEITA_WARM_SHIFT},
	{PMIC_RG_INT_MASK_JEITA_COOL, PMIC_RG_INT_MASK_JEITA_COOL_ADDR,
	 PMIC_RG_INT_MASK_JEITA_COOL_MASK, PMIC_RG_INT_MASK_JEITA_COOL_SHIFT},
	{PMIC_RG_INT_MASK_JEITA_COLD, PMIC_RG_INT_MASK_JEITA_COLD_ADDR,
	 PMIC_RG_INT_MASK_JEITA_COLD_MASK, PMIC_RG_INT_MASK_JEITA_COLD_SHIFT},
	{PMIC_INT_MASK_CON1_SET, PMIC_INT_MASK_CON1_SET_ADDR, PMIC_INT_MASK_CON1_SET_MASK,
	 PMIC_INT_MASK_CON1_SET_SHIFT},
	{PMIC_INT_MASK_CON1_CLR, PMIC_INT_MASK_CON1_CLR_ADDR, PMIC_INT_MASK_CON1_CLR_MASK,
	 PMIC_INT_MASK_CON1_CLR_SHIFT},
	{PMIC_RG_INT_MASK_VCORE_OC, PMIC_RG_INT_MASK_VCORE_OC_ADDR, PMIC_RG_INT_MASK_VCORE_OC_MASK,
	 PMIC_RG_INT_MASK_VCORE_OC_SHIFT},
	{PMIC_RG_INT_MASK_VMD1_OC, PMIC_RG_INT_MASK_VMD1_OC_ADDR, PMIC_RG_INT_MASK_VMD1_OC_MASK,
	 PMIC_RG_INT_MASK_VMD1_OC_SHIFT},
	{PMIC_RG_INT_MASK_VMODEM_OC, PMIC_RG_INT_MASK_VMODEM_OC_ADDR,
	 PMIC_RG_INT_MASK_VMODEM_OC_MASK, PMIC_RG_INT_MASK_VMODEM_OC_SHIFT},
	{PMIC_RG_INT_MASK_VS1_OC, PMIC_RG_INT_MASK_VS1_OC_ADDR, PMIC_RG_INT_MASK_VS1_OC_MASK,
	 PMIC_RG_INT_MASK_VS1_OC_SHIFT},
	{PMIC_RG_INT_MASK_VS2_OC, PMIC_RG_INT_MASK_VS2_OC_ADDR, PMIC_RG_INT_MASK_VS2_OC_MASK,
	 PMIC_RG_INT_MASK_VS2_OC_SHIFT},
	{PMIC_RG_INT_MASK_VDRAM_OC, PMIC_RG_INT_MASK_VDRAM_OC_ADDR, PMIC_RG_INT_MASK_VDRAM_OC_MASK,
	 PMIC_RG_INT_MASK_VDRAM_OC_SHIFT},
	{PMIC_RG_INT_MASK_VPA1_OC, PMIC_RG_INT_MASK_VPA1_OC_ADDR, PMIC_RG_INT_MASK_VPA1_OC_MASK,
	 PMIC_RG_INT_MASK_VPA1_OC_SHIFT},
	{PMIC_RG_INT_MASK_VPA2_OC, PMIC_RG_INT_MASK_VPA2_OC_ADDR, PMIC_RG_INT_MASK_VPA2_OC_MASK,
	 PMIC_RG_INT_MASK_VPA2_OC_SHIFT},
	{PMIC_RG_INT_MASK_VCORE_PREOC, PMIC_RG_INT_MASK_VCORE_PREOC_ADDR,
	 PMIC_RG_INT_MASK_VCORE_PREOC_MASK, PMIC_RG_INT_MASK_VCORE_PREOC_SHIFT},
	{PMIC_RG_INT_MASK_VA10_OC, PMIC_RG_INT_MASK_VA10_OC_ADDR, PMIC_RG_INT_MASK_VA10_OC_MASK,
	 PMIC_RG_INT_MASK_VA10_OC_SHIFT},
	{PMIC_RG_INT_MASK_VA12_OC, PMIC_RG_INT_MASK_VA12_OC_ADDR, PMIC_RG_INT_MASK_VA12_OC_MASK,
	 PMIC_RG_INT_MASK_VA12_OC_SHIFT},
	{PMIC_RG_INT_MASK_VA18_OC, PMIC_RG_INT_MASK_VA18_OC_ADDR, PMIC_RG_INT_MASK_VA18_OC_MASK,
	 PMIC_RG_INT_MASK_VA18_OC_SHIFT},
	{PMIC_RG_INT_MASK_VBIF28_OC, PMIC_RG_INT_MASK_VBIF28_OC_ADDR,
	 PMIC_RG_INT_MASK_VBIF28_OC_MASK, PMIC_RG_INT_MASK_VBIF28_OC_SHIFT},
	{PMIC_RG_INT_MASK_VCAMA1_OC, PMIC_RG_INT_MASK_VCAMA1_OC_ADDR,
	 PMIC_RG_INT_MASK_VCAMA1_OC_MASK, PMIC_RG_INT_MASK_VCAMA1_OC_SHIFT},
	{PMIC_RG_INT_MASK_VCAMA2_OC, PMIC_RG_INT_MASK_VCAMA2_OC_ADDR,
	 PMIC_RG_INT_MASK_VCAMA2_OC_MASK, PMIC_RG_INT_MASK_VCAMA2_OC_SHIFT},
	{PMIC_RG_INT_MASK_VCAMAF_OC, PMIC_RG_INT_MASK_VCAMAF_OC_ADDR,
	 PMIC_RG_INT_MASK_VCAMAF_OC_MASK, PMIC_RG_INT_MASK_VCAMAF_OC_SHIFT},
	{PMIC_INT_MASK_CON2_SET, PMIC_INT_MASK_CON2_SET_ADDR, PMIC_INT_MASK_CON2_SET_MASK,
	 PMIC_INT_MASK_CON2_SET_SHIFT},
	{PMIC_INT_MASK_CON2_CLR, PMIC_INT_MASK_CON2_CLR_ADDR, PMIC_INT_MASK_CON2_CLR_MASK,
	 PMIC_INT_MASK_CON2_CLR_SHIFT},
	{PMIC_RG_INT_MASK_VCAMD1_OC, PMIC_RG_INT_MASK_VCAMD1_OC_ADDR,
	 PMIC_RG_INT_MASK_VCAMD1_OC_MASK, PMIC_RG_INT_MASK_VCAMD1_OC_SHIFT},
	{PMIC_RG_INT_MASK_VCAMD2_OC, PMIC_RG_INT_MASK_VCAMD2_OC_ADDR,
	 PMIC_RG_INT_MASK_VCAMD2_OC_MASK, PMIC_RG_INT_MASK_VCAMD2_OC_SHIFT},
	{PMIC_RG_INT_MASK_VCAMIO_OC, PMIC_RG_INT_MASK_VCAMIO_OC_ADDR,
	 PMIC_RG_INT_MASK_VCAMIO_OC_MASK, PMIC_RG_INT_MASK_VCAMIO_OC_SHIFT},
	{PMIC_RG_INT_MASK_VCN18_OC, PMIC_RG_INT_MASK_VCN18_OC_ADDR, PMIC_RG_INT_MASK_VCN18_OC_MASK,
	 PMIC_RG_INT_MASK_VCN18_OC_SHIFT},
	{PMIC_RG_INT_MASK_VCN28_OC, PMIC_RG_INT_MASK_VCN28_OC_ADDR, PMIC_RG_INT_MASK_VCN28_OC_MASK,
	 PMIC_RG_INT_MASK_VCN28_OC_SHIFT},
	{PMIC_RG_INT_MASK_VCN33_OC, PMIC_RG_INT_MASK_VCN33_OC_ADDR, PMIC_RG_INT_MASK_VCN33_OC_MASK,
	 PMIC_RG_INT_MASK_VCN33_OC_SHIFT},
	{PMIC_RG_INT_MASK_VEFUSE_OC, PMIC_RG_INT_MASK_VEFUSE_OC_ADDR,
	 PMIC_RG_INT_MASK_VEFUSE_OC_MASK, PMIC_RG_INT_MASK_VEFUSE_OC_SHIFT},
	{PMIC_RG_INT_MASK_VEMC_OC, PMIC_RG_INT_MASK_VEMC_OC_ADDR, PMIC_RG_INT_MASK_VEMC_OC_MASK,
	 PMIC_RG_INT_MASK_VEMC_OC_SHIFT},
	{PMIC_RG_INT_MASK_VFE28_OC, PMIC_RG_INT_MASK_VFE28_OC_ADDR, PMIC_RG_INT_MASK_VFE28_OC_MASK,
	 PMIC_RG_INT_MASK_VFE28_OC_SHIFT},
	{PMIC_RG_INT_MASK_VGP3_OC, PMIC_RG_INT_MASK_VGP3_OC_ADDR, PMIC_RG_INT_MASK_VGP3_OC_MASK,
	 PMIC_RG_INT_MASK_VGP3_OC_SHIFT},
	{PMIC_RG_INT_MASK_VIBR_OC, PMIC_RG_INT_MASK_VIBR_OC_ADDR, PMIC_RG_INT_MASK_VIBR_OC_MASK,
	 PMIC_RG_INT_MASK_VIBR_OC_SHIFT},
	{PMIC_RG_INT_MASK_VIO18_OC, PMIC_RG_INT_MASK_VIO18_OC_ADDR, PMIC_RG_INT_MASK_VIO18_OC_MASK,
	 PMIC_RG_INT_MASK_VIO18_OC_SHIFT},
	{PMIC_RG_INT_MASK_VIO28_OC, PMIC_RG_INT_MASK_VIO28_OC_ADDR, PMIC_RG_INT_MASK_VIO28_OC_MASK,
	 PMIC_RG_INT_MASK_VIO28_OC_SHIFT},
	{PMIC_RG_INT_MASK_VMC_OC, PMIC_RG_INT_MASK_VMC_OC_ADDR, PMIC_RG_INT_MASK_VMC_OC_MASK,
	 PMIC_RG_INT_MASK_VMC_OC_SHIFT},
	{PMIC_RG_INT_MASK_VMCH_OC, PMIC_RG_INT_MASK_VMCH_OC_ADDR, PMIC_RG_INT_MASK_VMCH_OC_MASK,
	 PMIC_RG_INT_MASK_VMCH_OC_SHIFT},
	{PMIC_RG_INT_MASK_VMIPI_OC, PMIC_RG_INT_MASK_VMIPI_OC_ADDR, PMIC_RG_INT_MASK_VMIPI_OC_MASK,
	 PMIC_RG_INT_MASK_VMIPI_OC_SHIFT},
	{PMIC_INT_MASK_CON3_SET, PMIC_INT_MASK_CON3_SET_ADDR, PMIC_INT_MASK_CON3_SET_MASK,
	 PMIC_INT_MASK_CON3_SET_SHIFT},
	{PMIC_INT_MASK_CON3_CLR, PMIC_INT_MASK_CON3_CLR_ADDR, PMIC_INT_MASK_CON3_CLR_MASK,
	 PMIC_INT_MASK_CON3_CLR_SHIFT},
	{PMIC_RG_INT_MASK_VRF12_OC, PMIC_RG_INT_MASK_VRF12_OC_ADDR, PMIC_RG_INT_MASK_VRF12_OC_MASK,
	 PMIC_RG_INT_MASK_VRF12_OC_SHIFT},
	{PMIC_RG_INT_MASK_VRF18_1_OC, PMIC_RG_INT_MASK_VRF18_1_OC_ADDR,
	 PMIC_RG_INT_MASK_VRF18_1_OC_MASK, PMIC_RG_INT_MASK_VRF18_1_OC_SHIFT},
	{PMIC_RG_INT_MASK_VRF18_2_OC, PMIC_RG_INT_MASK_VRF18_2_OC_ADDR,
	 PMIC_RG_INT_MASK_VRF18_2_OC_MASK, PMIC_RG_INT_MASK_VRF18_2_OC_SHIFT},
	{PMIC_RG_INT_MASK_VSIM1_OC, PMIC_RG_INT_MASK_VSIM1_OC_ADDR, PMIC_RG_INT_MASK_VSIM1_OC_MASK,
	 PMIC_RG_INT_MASK_VSIM1_OC_SHIFT},
	{PMIC_RG_INT_MASK_VSIM2_OC, PMIC_RG_INT_MASK_VSIM2_OC_ADDR, PMIC_RG_INT_MASK_VSIM2_OC_MASK,
	 PMIC_RG_INT_MASK_VSIM2_OC_SHIFT},
	{PMIC_RG_INT_MASK_VSRAM_CORE_OC, PMIC_RG_INT_MASK_VSRAM_CORE_OC_ADDR,
	 PMIC_RG_INT_MASK_VSRAM_CORE_OC_MASK, PMIC_RG_INT_MASK_VSRAM_CORE_OC_SHIFT},
	{PMIC_RG_INT_MASK_VSRAM_DVFS1_OC, PMIC_RG_INT_MASK_VSRAM_DVFS1_OC_ADDR,
	 PMIC_RG_INT_MASK_VSRAM_DVFS1_OC_MASK, PMIC_RG_INT_MASK_VSRAM_DVFS1_OC_SHIFT},
	{PMIC_RG_INT_MASK_VSRAM_DVFS2_OC, PMIC_RG_INT_MASK_VSRAM_DVFS2_OC_ADDR,
	 PMIC_RG_INT_MASK_VSRAM_DVFS2_OC_MASK, PMIC_RG_INT_MASK_VSRAM_DVFS2_OC_SHIFT},
	{PMIC_RG_INT_MASK_VSRAM_GPU_OC, PMIC_RG_INT_MASK_VSRAM_GPU_OC_ADDR,
	 PMIC_RG_INT_MASK_VSRAM_GPU_OC_MASK, PMIC_RG_INT_MASK_VSRAM_GPU_OC_SHIFT},
	{PMIC_RG_INT_MASK_VSRAM_MD_OC, PMIC_RG_INT_MASK_VSRAM_MD_OC_ADDR,
	 PMIC_RG_INT_MASK_VSRAM_MD_OC_MASK, PMIC_RG_INT_MASK_VSRAM_MD_OC_SHIFT},
	{PMIC_RG_INT_MASK_VUFS18_OC, PMIC_RG_INT_MASK_VUFS18_OC_ADDR,
	 PMIC_RG_INT_MASK_VUFS18_OC_MASK, PMIC_RG_INT_MASK_VUFS18_OC_SHIFT},
	{PMIC_RG_INT_MASK_VUSB33_OC, PMIC_RG_INT_MASK_VUSB33_OC_ADDR,
	 PMIC_RG_INT_MASK_VUSB33_OC_MASK, PMIC_RG_INT_MASK_VUSB33_OC_SHIFT},
	{PMIC_RG_INT_MASK_VXO22_OC, PMIC_RG_INT_MASK_VXO22_OC_ADDR, PMIC_RG_INT_MASK_VXO22_OC_MASK,
	 PMIC_RG_INT_MASK_VXO22_OC_SHIFT},
	{PMIC_RG_INT_MASK_CON4, PMIC_RG_INT_MASK_CON4_ADDR, PMIC_RG_INT_MASK_CON4_MASK,
	 PMIC_RG_INT_MASK_CON4_SHIFT},
	{PMIC_INT_MASK_CON4_SET, PMIC_INT_MASK_CON4_SET_ADDR, PMIC_INT_MASK_CON4_SET_MASK,
	 PMIC_INT_MASK_CON4_SET_SHIFT},
	{PMIC_INT_MASK_CON4_CLR, PMIC_INT_MASK_CON4_CLR_ADDR, PMIC_INT_MASK_CON4_CLR_MASK,
	 PMIC_INT_MASK_CON4_CLR_SHIFT},
	{PMIC_RG_INT_MASK_FG_BAT0_H, PMIC_RG_INT_MASK_FG_BAT0_H_ADDR,
	 PMIC_RG_INT_MASK_FG_BAT0_H_MASK, PMIC_RG_INT_MASK_FG_BAT0_H_SHIFT},
	{PMIC_RG_INT_MASK_FG_BAT0_L, PMIC_RG_INT_MASK_FG_BAT0_L_ADDR,
	 PMIC_RG_INT_MASK_FG_BAT0_L_MASK, PMIC_RG_INT_MASK_FG_BAT0_L_SHIFT},
	{PMIC_RG_INT_MASK_FG_CUR_H, PMIC_RG_INT_MASK_FG_CUR_H_ADDR, PMIC_RG_INT_MASK_FG_CUR_H_MASK,
	 PMIC_RG_INT_MASK_FG_CUR_H_SHIFT},
	{PMIC_RG_INT_MASK_FG_CUR_L, PMIC_RG_INT_MASK_FG_CUR_L_ADDR, PMIC_RG_INT_MASK_FG_CUR_L_MASK,
	 PMIC_RG_INT_MASK_FG_CUR_L_SHIFT},
	{PMIC_RG_INT_MASK_FG_ZCV, PMIC_RG_INT_MASK_FG_ZCV_ADDR, PMIC_RG_INT_MASK_FG_ZCV_MASK,
	 PMIC_RG_INT_MASK_FG_ZCV_SHIFT},
	{PMIC_RG_INT_MASK_FG_BAT1_H, PMIC_RG_INT_MASK_FG_BAT1_H_ADDR,
	 PMIC_RG_INT_MASK_FG_BAT1_H_MASK, PMIC_RG_INT_MASK_FG_BAT1_H_SHIFT},
	{PMIC_RG_INT_MASK_FG_BAT1_L, PMIC_RG_INT_MASK_FG_BAT1_L_ADDR,
	 PMIC_RG_INT_MASK_FG_BAT1_L_MASK, PMIC_RG_INT_MASK_FG_BAT1_L_SHIFT},
	{PMIC_RG_INT_MASK_FG_N_CHARGE_L, PMIC_RG_INT_MASK_FG_N_CHARGE_L_ADDR,
	 PMIC_RG_INT_MASK_FG_N_CHARGE_L_MASK, PMIC_RG_INT_MASK_FG_N_CHARGE_L_SHIFT},
	{PMIC_RG_INT_MASK_FG_IAVG_H, PMIC_RG_INT_MASK_FG_IAVG_H_ADDR,
	 PMIC_RG_INT_MASK_FG_IAVG_H_MASK, PMIC_RG_INT_MASK_FG_IAVG_H_SHIFT},
	{PMIC_RG_INT_MASK_FG_IAVG_L, PMIC_RG_INT_MASK_FG_IAVG_L_ADDR,
	 PMIC_RG_INT_MASK_FG_IAVG_L_MASK, PMIC_RG_INT_MASK_FG_IAVG_L_SHIFT},
	{PMIC_RG_INT_MASK_FG_TIME_H, PMIC_RG_INT_MASK_FG_TIME_H_ADDR,
	 PMIC_RG_INT_MASK_FG_TIME_H_MASK, PMIC_RG_INT_MASK_FG_TIME_H_SHIFT},
	{PMIC_RG_INT_MASK_FG_DISCHARGE, PMIC_RG_INT_MASK_FG_DISCHARGE_ADDR,
	 PMIC_RG_INT_MASK_FG_DISCHARGE_MASK, PMIC_RG_INT_MASK_FG_DISCHARGE_SHIFT},
	{PMIC_RG_INT_MASK_FG_CHARGE, PMIC_RG_INT_MASK_FG_CHARGE_ADDR,
	 PMIC_RG_INT_MASK_FG_CHARGE_MASK, PMIC_RG_INT_MASK_FG_CHARGE_SHIFT},
	{PMIC_RG_INT_MASK_CON5, PMIC_RG_INT_MASK_CON5_ADDR, PMIC_RG_INT_MASK_CON5_MASK,
	 PMIC_RG_INT_MASK_CON5_SHIFT},
	{PMIC_INT_MASK_CON5_SET, PMIC_INT_MASK_CON5_SET_ADDR, PMIC_INT_MASK_CON5_SET_MASK,
	 PMIC_INT_MASK_CON5_SET_SHIFT},
	{PMIC_INT_MASK_CON5_CLR, PMIC_INT_MASK_CON5_CLR_ADDR, PMIC_INT_MASK_CON5_CLR_MASK,
	 PMIC_INT_MASK_CON5_CLR_SHIFT},
	{PMIC_RG_INT_STATUS_PWRKEY, PMIC_RG_INT_STATUS_PWRKEY_ADDR, PMIC_RG_INT_STATUS_PWRKEY_MASK,
	 PMIC_RG_INT_STATUS_PWRKEY_SHIFT},
	{PMIC_RG_INT_STATUS_HOMEKEY, PMIC_RG_INT_STATUS_HOMEKEY_ADDR,
	 PMIC_RG_INT_STATUS_HOMEKEY_MASK, PMIC_RG_INT_STATUS_HOMEKEY_SHIFT},
	{PMIC_RG_INT_STATUS_PWRKEY_R, PMIC_RG_INT_STATUS_PWRKEY_R_ADDR,
	 PMIC_RG_INT_STATUS_PWRKEY_R_MASK, PMIC_RG_INT_STATUS_PWRKEY_R_SHIFT},
	{PMIC_RG_INT_STATUS_HOMEKEY_R, PMIC_RG_INT_STATUS_HOMEKEY_R_ADDR,
	 PMIC_RG_INT_STATUS_HOMEKEY_R_MASK, PMIC_RG_INT_STATUS_HOMEKEY_R_SHIFT},
	{PMIC_RG_INT_STATUS_NI_LBAT_INT, PMIC_RG_INT_STATUS_NI_LBAT_INT_ADDR,
	 PMIC_RG_INT_STATUS_NI_LBAT_INT_MASK, PMIC_RG_INT_STATUS_NI_LBAT_INT_SHIFT},
	{PMIC_RG_INT_STATUS_CHRDET, PMIC_RG_INT_STATUS_CHRDET_ADDR, PMIC_RG_INT_STATUS_CHRDET_MASK,
	 PMIC_RG_INT_STATUS_CHRDET_SHIFT},
	{PMIC_RG_INT_STATUS_CHRDET_EDGE, PMIC_RG_INT_STATUS_CHRDET_EDGE_ADDR,
	 PMIC_RG_INT_STATUS_CHRDET_EDGE_MASK, PMIC_RG_INT_STATUS_CHRDET_EDGE_SHIFT},
	{PMIC_RG_INT_STATUS_BATON_LV, PMIC_RG_INT_STATUS_BATON_LV_ADDR,
	 PMIC_RG_INT_STATUS_BATON_LV_MASK, PMIC_RG_INT_STATUS_BATON_LV_SHIFT},
	{PMIC_RG_INT_STATUS_BATON_HV, PMIC_RG_INT_STATUS_BATON_HV_ADDR,
	 PMIC_RG_INT_STATUS_BATON_HV_MASK, PMIC_RG_INT_STATUS_BATON_HV_SHIFT},
	{PMIC_RG_INT_STATUS_BATON_BAT_IN, PMIC_RG_INT_STATUS_BATON_BAT_IN_ADDR,
	 PMIC_RG_INT_STATUS_BATON_BAT_IN_MASK, PMIC_RG_INT_STATUS_BATON_BAT_IN_SHIFT},
	{PMIC_RG_INT_STATUS_BATON_BAT_OUT, PMIC_RG_INT_STATUS_BATON_BAT_OUT_ADDR,
	 PMIC_RG_INT_STATUS_BATON_BAT_OUT_MASK, PMIC_RG_INT_STATUS_BATON_BAT_OUT_SHIFT},
	{PMIC_RG_INT_STATUS_RTC, PMIC_RG_INT_STATUS_RTC_ADDR, PMIC_RG_INT_STATUS_RTC_MASK,
	 PMIC_RG_INT_STATUS_RTC_SHIFT},
	{PMIC_RG_INT_STATUS_BIF, PMIC_RG_INT_STATUS_BIF_ADDR, PMIC_RG_INT_STATUS_BIF_MASK,
	 PMIC_RG_INT_STATUS_BIF_SHIFT},
	{PMIC_RG_INT_STATUS_VCDT_HV_DET, PMIC_RG_INT_STATUS_VCDT_HV_DET_ADDR,
	 PMIC_RG_INT_STATUS_VCDT_HV_DET_MASK, PMIC_RG_INT_STATUS_VCDT_HV_DET_SHIFT},
	{PMIC_RG_INT_STATUS_THR_H, PMIC_RG_INT_STATUS_THR_H_ADDR, PMIC_RG_INT_STATUS_THR_H_MASK,
	 PMIC_RG_INT_STATUS_THR_H_SHIFT},
	{PMIC_RG_INT_STATUS_THR_L, PMIC_RG_INT_STATUS_THR_L_ADDR, PMIC_RG_INT_STATUS_THR_L_MASK,
	 PMIC_RG_INT_STATUS_THR_L_SHIFT},
	{PMIC_RG_INT_STATUS_BAT_H, PMIC_RG_INT_STATUS_BAT_H_ADDR, PMIC_RG_INT_STATUS_BAT_H_MASK,
	 PMIC_RG_INT_STATUS_BAT_H_SHIFT},
	{PMIC_RG_INT_STATUS_BAT_L, PMIC_RG_INT_STATUS_BAT_L_ADDR, PMIC_RG_INT_STATUS_BAT_L_MASK,
	 PMIC_RG_INT_STATUS_BAT_L_SHIFT},
	{PMIC_RG_INT_STATUS_BAT2_H, PMIC_RG_INT_STATUS_BAT2_H_ADDR, PMIC_RG_INT_STATUS_BAT2_H_MASK,
	 PMIC_RG_INT_STATUS_BAT2_H_SHIFT},
	{PMIC_RG_INT_STATUS_BAT2_L, PMIC_RG_INT_STATUS_BAT2_L_ADDR, PMIC_RG_INT_STATUS_BAT2_L_MASK,
	 PMIC_RG_INT_STATUS_BAT2_L_SHIFT},
	{PMIC_RG_INT_STATUS_BAT_TEMP_H, PMIC_RG_INT_STATUS_BAT_TEMP_H_ADDR,
	 PMIC_RG_INT_STATUS_BAT_TEMP_H_MASK, PMIC_RG_INT_STATUS_BAT_TEMP_H_SHIFT},
	{PMIC_RG_INT_STATUS_BAT_TEMP_L, PMIC_RG_INT_STATUS_BAT_TEMP_L_ADDR,
	 PMIC_RG_INT_STATUS_BAT_TEMP_L_MASK, PMIC_RG_INT_STATUS_BAT_TEMP_L_SHIFT},
	{PMIC_RG_INT_STATUS_AUXADC_IMP, PMIC_RG_INT_STATUS_AUXADC_IMP_ADDR,
	 PMIC_RG_INT_STATUS_AUXADC_IMP_MASK, PMIC_RG_INT_STATUS_AUXADC_IMP_SHIFT},
	{PMIC_RG_INT_STATUS_NAG_C_DLTV, PMIC_RG_INT_STATUS_NAG_C_DLTV_ADDR,
	 PMIC_RG_INT_STATUS_NAG_C_DLTV_MASK, PMIC_RG_INT_STATUS_NAG_C_DLTV_SHIFT},
	{PMIC_RG_INT_STATUS_JEITA_HOT, PMIC_RG_INT_STATUS_JEITA_HOT_ADDR,
	 PMIC_RG_INT_STATUS_JEITA_HOT_MASK, PMIC_RG_INT_STATUS_JEITA_HOT_SHIFT},
	{PMIC_RG_INT_STATUS_JEITA_WARM, PMIC_RG_INT_STATUS_JEITA_WARM_ADDR,
	 PMIC_RG_INT_STATUS_JEITA_WARM_MASK, PMIC_RG_INT_STATUS_JEITA_WARM_SHIFT},
	{PMIC_RG_INT_STATUS_JEITA_COOL, PMIC_RG_INT_STATUS_JEITA_COOL_ADDR,
	 PMIC_RG_INT_STATUS_JEITA_COOL_MASK, PMIC_RG_INT_STATUS_JEITA_COOL_SHIFT},
	{PMIC_RG_INT_STATUS_JEITA_COLD, PMIC_RG_INT_STATUS_JEITA_COLD_ADDR,
	 PMIC_RG_INT_STATUS_JEITA_COLD_MASK, PMIC_RG_INT_STATUS_JEITA_COLD_SHIFT},
	{PMIC_RG_INT_STATUS_VCORE_OC, PMIC_RG_INT_STATUS_VCORE_OC_ADDR,
	 PMIC_RG_INT_STATUS_VCORE_OC_MASK, PMIC_RG_INT_STATUS_VCORE_OC_SHIFT},
	{PMIC_RG_INT_STATUS_VMD1_OC, PMIC_RG_INT_STATUS_VMD1_OC_ADDR,
	 PMIC_RG_INT_STATUS_VMD1_OC_MASK, PMIC_RG_INT_STATUS_VMD1_OC_SHIFT},
	{PMIC_RG_INT_STATUS_VMODEM_OC, PMIC_RG_INT_STATUS_VMODEM_OC_ADDR,
	 PMIC_RG_INT_STATUS_VMODEM_OC_MASK, PMIC_RG_INT_STATUS_VMODEM_OC_SHIFT},
	{PMIC_RG_INT_STATUS_VS1_OC, PMIC_RG_INT_STATUS_VS1_OC_ADDR, PMIC_RG_INT_STATUS_VS1_OC_MASK,
	 PMIC_RG_INT_STATUS_VS1_OC_SHIFT},
	{PMIC_RG_INT_STATUS_VS2_OC, PMIC_RG_INT_STATUS_VS2_OC_ADDR, PMIC_RG_INT_STATUS_VS2_OC_MASK,
	 PMIC_RG_INT_STATUS_VS2_OC_SHIFT},
	{PMIC_RG_INT_STATUS_VDRAM_OC, PMIC_RG_INT_STATUS_VDRAM_OC_ADDR,
	 PMIC_RG_INT_STATUS_VDRAM_OC_MASK, PMIC_RG_INT_STATUS_VDRAM_OC_SHIFT},
	{PMIC_RG_INT_STATUS_VPA1_OC, PMIC_RG_INT_STATUS_VPA1_OC_ADDR,
	 PMIC_RG_INT_STATUS_VPA1_OC_MASK, PMIC_RG_INT_STATUS_VPA1_OC_SHIFT},
	{PMIC_RG_INT_STATUS_VPA2_OC, PMIC_RG_INT_STATUS_VPA2_OC_ADDR,
	 PMIC_RG_INT_STATUS_VPA2_OC_MASK, PMIC_RG_INT_STATUS_VPA2_OC_SHIFT},
	{PMIC_RG_INT_STATUS_VCORE_PREOC, PMIC_RG_INT_STATUS_VCORE_PREOC_ADDR,
	 PMIC_RG_INT_STATUS_VCORE_PREOC_MASK, PMIC_RG_INT_STATUS_VCORE_PREOC_SHIFT},
	{PMIC_RG_INT_STATUS_VA10_OC, PMIC_RG_INT_STATUS_VA10_OC_ADDR,
	 PMIC_RG_INT_STATUS_VA10_OC_MASK, PMIC_RG_INT_STATUS_VA10_OC_SHIFT},
	{PMIC_RG_INT_STATUS_VA12_OC, PMIC_RG_INT_STATUS_VA12_OC_ADDR,
	 PMIC_RG_INT_STATUS_VA12_OC_MASK, PMIC_RG_INT_STATUS_VA12_OC_SHIFT},
	{PMIC_RG_INT_STATUS_VA18_OC, PMIC_RG_INT_STATUS_VA18_OC_ADDR,
	 PMIC_RG_INT_STATUS_VA18_OC_MASK, PMIC_RG_INT_STATUS_VA18_OC_SHIFT},
	{PMIC_RG_INT_STATUS_VBIF28_OC, PMIC_RG_INT_STATUS_VBIF28_OC_ADDR,
	 PMIC_RG_INT_STATUS_VBIF28_OC_MASK, PMIC_RG_INT_STATUS_VBIF28_OC_SHIFT},
	{PMIC_RG_INT_STATUS_VCAMA1_OC, PMIC_RG_INT_STATUS_VCAMA1_OC_ADDR,
	 PMIC_RG_INT_STATUS_VCAMA1_OC_MASK, PMIC_RG_INT_STATUS_VCAMA1_OC_SHIFT},
	{PMIC_RG_INT_STATUS_VCAMA2_OC, PMIC_RG_INT_STATUS_VCAMA2_OC_ADDR,
	 PMIC_RG_INT_STATUS_VCAMA2_OC_MASK, PMIC_RG_INT_STATUS_VCAMA2_OC_SHIFT},
	{PMIC_RG_INT_STATUS_VCAMAF_OC, PMIC_RG_INT_STATUS_VCAMAF_OC_ADDR,
	 PMIC_RG_INT_STATUS_VCAMAF_OC_MASK, PMIC_RG_INT_STATUS_VCAMAF_OC_SHIFT},
	{PMIC_RG_INT_STATUS_VCAMD1_OC, PMIC_RG_INT_STATUS_VCAMD1_OC_ADDR,
	 PMIC_RG_INT_STATUS_VCAMD1_OC_MASK, PMIC_RG_INT_STATUS_VCAMD1_OC_SHIFT},
	{PMIC_RG_INT_STATUS_VCAMD2_OC, PMIC_RG_INT_STATUS_VCAMD2_OC_ADDR,
	 PMIC_RG_INT_STATUS_VCAMD2_OC_MASK, PMIC_RG_INT_STATUS_VCAMD2_OC_SHIFT},
	{PMIC_RG_INT_STATUS_VCAMIO_OC, PMIC_RG_INT_STATUS_VCAMIO_OC_ADDR,
	 PMIC_RG_INT_STATUS_VCAMIO_OC_MASK, PMIC_RG_INT_STATUS_VCAMIO_OC_SHIFT},
	{PMIC_RG_INT_STATUS_VCN18_OC, PMIC_RG_INT_STATUS_VCN18_OC_ADDR,
	 PMIC_RG_INT_STATUS_VCN18_OC_MASK, PMIC_RG_INT_STATUS_VCN18_OC_SHIFT},
	{PMIC_RG_INT_STATUS_VCN28_OC, PMIC_RG_INT_STATUS_VCN28_OC_ADDR,
	 PMIC_RG_INT_STATUS_VCN28_OC_MASK, PMIC_RG_INT_STATUS_VCN28_OC_SHIFT},
	{PMIC_RG_INT_STATUS_VCN33_OC, PMIC_RG_INT_STATUS_VCN33_OC_ADDR,
	 PMIC_RG_INT_STATUS_VCN33_OC_MASK, PMIC_RG_INT_STATUS_VCN33_OC_SHIFT},
	{PMIC_RG_INT_STATUS_VEFUSE_OC, PMIC_RG_INT_STATUS_VEFUSE_OC_ADDR,
	 PMIC_RG_INT_STATUS_VEFUSE_OC_MASK, PMIC_RG_INT_STATUS_VEFUSE_OC_SHIFT},
	{PMIC_RG_INT_STATUS_VEMC_OC, PMIC_RG_INT_STATUS_VEMC_OC_ADDR,
	 PMIC_RG_INT_STATUS_VEMC_OC_MASK, PMIC_RG_INT_STATUS_VEMC_OC_SHIFT},
	{PMIC_RG_INT_STATUS_VFE28_OC, PMIC_RG_INT_STATUS_VFE28_OC_ADDR,
	 PMIC_RG_INT_STATUS_VFE28_OC_MASK, PMIC_RG_INT_STATUS_VFE28_OC_SHIFT},
	{PMIC_RG_INT_STATUS_VGP3_OC, PMIC_RG_INT_STATUS_VGP3_OC_ADDR,
	 PMIC_RG_INT_STATUS_VGP3_OC_MASK, PMIC_RG_INT_STATUS_VGP3_OC_SHIFT},
	{PMIC_RG_INT_STATUS_VIBR_OC, PMIC_RG_INT_STATUS_VIBR_OC_ADDR,
	 PMIC_RG_INT_STATUS_VIBR_OC_MASK, PMIC_RG_INT_STATUS_VIBR_OC_SHIFT},
	{PMIC_RG_INT_STATUS_VIO18_OC, PMIC_RG_INT_STATUS_VIO18_OC_ADDR,
	 PMIC_RG_INT_STATUS_VIO18_OC_MASK, PMIC_RG_INT_STATUS_VIO18_OC_SHIFT},
	{PMIC_RG_INT_STATUS_VIO28_OC, PMIC_RG_INT_STATUS_VIO28_OC_ADDR,
	 PMIC_RG_INT_STATUS_VIO28_OC_MASK, PMIC_RG_INT_STATUS_VIO28_OC_SHIFT},
	{PMIC_RG_INT_STATUS_VMC_OC, PMIC_RG_INT_STATUS_VMC_OC_ADDR, PMIC_RG_INT_STATUS_VMC_OC_MASK,
	 PMIC_RG_INT_STATUS_VMC_OC_SHIFT},
	{PMIC_RG_INT_STATUS_VMCH_OC, PMIC_RG_INT_STATUS_VMCH_OC_ADDR,
	 PMIC_RG_INT_STATUS_VMCH_OC_MASK, PMIC_RG_INT_STATUS_VMCH_OC_SHIFT},
	{PMIC_RG_INT_STATUS_VMIPI_OC, PMIC_RG_INT_STATUS_VMIPI_OC_ADDR,
	 PMIC_RG_INT_STATUS_VMIPI_OC_MASK, PMIC_RG_INT_STATUS_VMIPI_OC_SHIFT},
	{PMIC_RG_INT_STATUS_VRF12_OC, PMIC_RG_INT_STATUS_VRF12_OC_ADDR,
	 PMIC_RG_INT_STATUS_VRF12_OC_MASK, PMIC_RG_INT_STATUS_VRF12_OC_SHIFT},
	{PMIC_RG_INT_STATUS_VRF18_1_OC, PMIC_RG_INT_STATUS_VRF18_1_OC_ADDR,
	 PMIC_RG_INT_STATUS_VRF18_1_OC_MASK, PMIC_RG_INT_STATUS_VRF18_1_OC_SHIFT},
	{PMIC_RG_INT_STATUS_VRF18_2_OC, PMIC_RG_INT_STATUS_VRF18_2_OC_ADDR,
	 PMIC_RG_INT_STATUS_VRF18_2_OC_MASK, PMIC_RG_INT_STATUS_VRF18_2_OC_SHIFT},
	{PMIC_RG_INT_STATUS_VSIM1_OC, PMIC_RG_INT_STATUS_VSIM1_OC_ADDR,
	 PMIC_RG_INT_STATUS_VSIM1_OC_MASK, PMIC_RG_INT_STATUS_VSIM1_OC_SHIFT},
	{PMIC_RG_INT_STATUS_VSIM2_OC, PMIC_RG_INT_STATUS_VSIM2_OC_ADDR,
	 PMIC_RG_INT_STATUS_VSIM2_OC_MASK, PMIC_RG_INT_STATUS_VSIM2_OC_SHIFT},
	{PMIC_RG_INT_STATUS_VSRAM_CORE_OC, PMIC_RG_INT_STATUS_VSRAM_CORE_OC_ADDR,
	 PMIC_RG_INT_STATUS_VSRAM_CORE_OC_MASK, PMIC_RG_INT_STATUS_VSRAM_CORE_OC_SHIFT},
	{PMIC_RG_INT_STATUS_VSRAM_DVFS1_OC, PMIC_RG_INT_STATUS_VSRAM_DVFS1_OC_ADDR,
	 PMIC_RG_INT_STATUS_VSRAM_DVFS1_OC_MASK, PMIC_RG_INT_STATUS_VSRAM_DVFS1_OC_SHIFT},
	{PMIC_RG_INT_STATUS_VSRAM_DVFS2_OC, PMIC_RG_INT_STATUS_VSRAM_DVFS2_OC_ADDR,
	 PMIC_RG_INT_STATUS_VSRAM_DVFS2_OC_MASK, PMIC_RG_INT_STATUS_VSRAM_DVFS2_OC_SHIFT},
	{PMIC_RG_INT_STATUS_VSRAM_GPU_OC, PMIC_RG_INT_STATUS_VSRAM_GPU_OC_ADDR,
	 PMIC_RG_INT_STATUS_VSRAM_GPU_OC_MASK, PMIC_RG_INT_STATUS_VSRAM_GPU_OC_SHIFT},
	{PMIC_RG_INT_STATUS_VSRAM_MD_OC, PMIC_RG_INT_STATUS_VSRAM_MD_OC_ADDR,
	 PMIC_RG_INT_STATUS_VSRAM_MD_OC_MASK, PMIC_RG_INT_STATUS_VSRAM_MD_OC_SHIFT},
	{PMIC_RG_INT_STATUS_VUFS18_OC, PMIC_RG_INT_STATUS_VUFS18_OC_ADDR,
	 PMIC_RG_INT_STATUS_VUFS18_OC_MASK, PMIC_RG_INT_STATUS_VUFS18_OC_SHIFT},
	{PMIC_RG_INT_STATUS_VUSB33_OC, PMIC_RG_INT_STATUS_VUSB33_OC_ADDR,
	 PMIC_RG_INT_STATUS_VUSB33_OC_MASK, PMIC_RG_INT_STATUS_VUSB33_OC_SHIFT},
	{PMIC_RG_INT_STATUS_VXO22_OC, PMIC_RG_INT_STATUS_VXO22_OC_ADDR,
	 PMIC_RG_INT_STATUS_VXO22_OC_MASK, PMIC_RG_INT_STATUS_VXO22_OC_SHIFT},
	{PMIC_RG_INT_STATUS_CON4, PMIC_RG_INT_STATUS_CON4_ADDR, PMIC_RG_INT_STATUS_CON4_MASK,
	 PMIC_RG_INT_STATUS_CON4_SHIFT},
	{PMIC_RG_INT_STATUS_FG_BAT0_H, PMIC_RG_INT_STATUS_FG_BAT0_H_ADDR,
	 PMIC_RG_INT_STATUS_FG_BAT0_H_MASK, PMIC_RG_INT_STATUS_FG_BAT0_H_SHIFT},
	{PMIC_RG_INT_STATUS_FG_BAT0_L, PMIC_RG_INT_STATUS_FG_BAT0_L_ADDR,
	 PMIC_RG_INT_STATUS_FG_BAT0_L_MASK, PMIC_RG_INT_STATUS_FG_BAT0_L_SHIFT},
	{PMIC_RG_INT_STATUS_FG_CUR_H, PMIC_RG_INT_STATUS_FG_CUR_H_ADDR,
	 PMIC_RG_INT_STATUS_FG_CUR_H_MASK, PMIC_RG_INT_STATUS_FG_CUR_H_SHIFT},
	{PMIC_RG_INT_STATUS_FG_CUR_L, PMIC_RG_INT_STATUS_FG_CUR_L_ADDR,
	 PMIC_RG_INT_STATUS_FG_CUR_L_MASK, PMIC_RG_INT_STATUS_FG_CUR_L_SHIFT},
	{PMIC_RG_INT_STATUS_FG_ZCV, PMIC_RG_INT_STATUS_FG_ZCV_ADDR, PMIC_RG_INT_STATUS_FG_ZCV_MASK,
	 PMIC_RG_INT_STATUS_FG_ZCV_SHIFT},
	{PMIC_RG_INT_STATUS_FG_BAT1_H, PMIC_RG_INT_STATUS_FG_BAT1_H_ADDR,
	 PMIC_RG_INT_STATUS_FG_BAT1_H_MASK, PMIC_RG_INT_STATUS_FG_BAT1_H_SHIFT},
	{PMIC_RG_INT_STATUS_FG_BAT1_L, PMIC_RG_INT_STATUS_FG_BAT1_L_ADDR,
	 PMIC_RG_INT_STATUS_FG_BAT1_L_MASK, PMIC_RG_INT_STATUS_FG_BAT1_L_SHIFT},
	{PMIC_RG_INT_STATUS_FG_N_CHARGE_L, PMIC_RG_INT_STATUS_FG_N_CHARGE_L_ADDR,
	 PMIC_RG_INT_STATUS_FG_N_CHARGE_L_MASK, PMIC_RG_INT_STATUS_FG_N_CHARGE_L_SHIFT},
	{PMIC_RG_INT_STATUS_FG_IAVG_H, PMIC_RG_INT_STATUS_FG_IAVG_H_ADDR,
	 PMIC_RG_INT_STATUS_FG_IAVG_H_MASK, PMIC_RG_INT_STATUS_FG_IAVG_H_SHIFT},
	{PMIC_RG_INT_STATUS_FG_IAVG_L, PMIC_RG_INT_STATUS_FG_IAVG_L_ADDR,
	 PMIC_RG_INT_STATUS_FG_IAVG_L_MASK, PMIC_RG_INT_STATUS_FG_IAVG_L_SHIFT},
	{PMIC_RG_INT_STATUS_FG_TIME_H, PMIC_RG_INT_STATUS_FG_TIME_H_ADDR,
	 PMIC_RG_INT_STATUS_FG_TIME_H_MASK, PMIC_RG_INT_STATUS_FG_TIME_H_SHIFT},
	{PMIC_RG_INT_STATUS_FG_DISCHARGE, PMIC_RG_INT_STATUS_FG_DISCHARGE_ADDR,
	 PMIC_RG_INT_STATUS_FG_DISCHARGE_MASK, PMIC_RG_INT_STATUS_FG_DISCHARGE_SHIFT},
	{PMIC_RG_INT_STATUS_FG_CHARGE, PMIC_RG_INT_STATUS_FG_CHARGE_ADDR,
	 PMIC_RG_INT_STATUS_FG_CHARGE_MASK, PMIC_RG_INT_STATUS_FG_CHARGE_SHIFT},
	{PMIC_RG_INT_STATUS_CON5, PMIC_RG_INT_STATUS_CON5_ADDR, PMIC_RG_INT_STATUS_CON5_MASK,
	 PMIC_RG_INT_STATUS_CON5_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_PWRKEY, PMIC_RG_INT_RAW_STATUS_PWRKEY_ADDR,
	 PMIC_RG_INT_RAW_STATUS_PWRKEY_MASK, PMIC_RG_INT_RAW_STATUS_PWRKEY_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_HOMEKEY, PMIC_RG_INT_RAW_STATUS_HOMEKEY_ADDR,
	 PMIC_RG_INT_RAW_STATUS_HOMEKEY_MASK, PMIC_RG_INT_RAW_STATUS_HOMEKEY_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_PWRKEY_R, PMIC_RG_INT_RAW_STATUS_PWRKEY_R_ADDR,
	 PMIC_RG_INT_RAW_STATUS_PWRKEY_R_MASK, PMIC_RG_INT_RAW_STATUS_PWRKEY_R_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_HOMEKEY_R, PMIC_RG_INT_RAW_STATUS_HOMEKEY_R_ADDR,
	 PMIC_RG_INT_RAW_STATUS_HOMEKEY_R_MASK, PMIC_RG_INT_RAW_STATUS_HOMEKEY_R_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_NI_LBAT_INT, PMIC_RG_INT_RAW_STATUS_NI_LBAT_INT_ADDR,
	 PMIC_RG_INT_RAW_STATUS_NI_LBAT_INT_MASK, PMIC_RG_INT_RAW_STATUS_NI_LBAT_INT_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_CHRDET, PMIC_RG_INT_RAW_STATUS_CHRDET_ADDR,
	 PMIC_RG_INT_RAW_STATUS_CHRDET_MASK, PMIC_RG_INT_RAW_STATUS_CHRDET_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_CHRDET_EDGE, PMIC_RG_INT_RAW_STATUS_CHRDET_EDGE_ADDR,
	 PMIC_RG_INT_RAW_STATUS_CHRDET_EDGE_MASK, PMIC_RG_INT_RAW_STATUS_CHRDET_EDGE_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_BATON_LV, PMIC_RG_INT_RAW_STATUS_BATON_LV_ADDR,
	 PMIC_RG_INT_RAW_STATUS_BATON_LV_MASK, PMIC_RG_INT_RAW_STATUS_BATON_LV_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_BATON_HV, PMIC_RG_INT_RAW_STATUS_BATON_HV_ADDR,
	 PMIC_RG_INT_RAW_STATUS_BATON_HV_MASK, PMIC_RG_INT_RAW_STATUS_BATON_HV_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_BATON_BAT_IN, PMIC_RG_INT_RAW_STATUS_BATON_BAT_IN_ADDR,
	 PMIC_RG_INT_RAW_STATUS_BATON_BAT_IN_MASK, PMIC_RG_INT_RAW_STATUS_BATON_BAT_IN_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_BATON_BAT_OUT, PMIC_RG_INT_RAW_STATUS_BATON_BAT_OUT_ADDR,
	 PMIC_RG_INT_RAW_STATUS_BATON_BAT_OUT_MASK, PMIC_RG_INT_RAW_STATUS_BATON_BAT_OUT_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_RTC, PMIC_RG_INT_RAW_STATUS_RTC_ADDR,
	 PMIC_RG_INT_RAW_STATUS_RTC_MASK, PMIC_RG_INT_RAW_STATUS_RTC_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_BIF, PMIC_RG_INT_RAW_STATUS_BIF_ADDR,
	 PMIC_RG_INT_RAW_STATUS_BIF_MASK, PMIC_RG_INT_RAW_STATUS_BIF_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_VCDT_HV_DET, PMIC_RG_INT_RAW_STATUS_VCDT_HV_DET_ADDR,
	 PMIC_RG_INT_RAW_STATUS_VCDT_HV_DET_MASK, PMIC_RG_INT_RAW_STATUS_VCDT_HV_DET_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_THR_H, PMIC_RG_INT_RAW_STATUS_THR_H_ADDR,
	 PMIC_RG_INT_RAW_STATUS_THR_H_MASK, PMIC_RG_INT_RAW_STATUS_THR_H_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_THR_L, PMIC_RG_INT_RAW_STATUS_THR_L_ADDR,
	 PMIC_RG_INT_RAW_STATUS_THR_L_MASK, PMIC_RG_INT_RAW_STATUS_THR_L_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_BAT_H, PMIC_RG_INT_RAW_STATUS_BAT_H_ADDR,
	 PMIC_RG_INT_RAW_STATUS_BAT_H_MASK, PMIC_RG_INT_RAW_STATUS_BAT_H_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_BAT_L, PMIC_RG_INT_RAW_STATUS_BAT_L_ADDR,
	 PMIC_RG_INT_RAW_STATUS_BAT_L_MASK, PMIC_RG_INT_RAW_STATUS_BAT_L_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_BAT2_H, PMIC_RG_INT_RAW_STATUS_BAT2_H_ADDR,
	 PMIC_RG_INT_RAW_STATUS_BAT2_H_MASK, PMIC_RG_INT_RAW_STATUS_BAT2_H_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_BAT2_L, PMIC_RG_INT_RAW_STATUS_BAT2_L_ADDR,
	 PMIC_RG_INT_RAW_STATUS_BAT2_L_MASK, PMIC_RG_INT_RAW_STATUS_BAT2_L_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_BAT_TEMP_H, PMIC_RG_INT_RAW_STATUS_BAT_TEMP_H_ADDR,
	 PMIC_RG_INT_RAW_STATUS_BAT_TEMP_H_MASK, PMIC_RG_INT_RAW_STATUS_BAT_TEMP_H_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_BAT_TEMP_L, PMIC_RG_INT_RAW_STATUS_BAT_TEMP_L_ADDR,
	 PMIC_RG_INT_RAW_STATUS_BAT_TEMP_L_MASK, PMIC_RG_INT_RAW_STATUS_BAT_TEMP_L_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_AUXADC_IMP, PMIC_RG_INT_RAW_STATUS_AUXADC_IMP_ADDR,
	 PMIC_RG_INT_RAW_STATUS_AUXADC_IMP_MASK, PMIC_RG_INT_RAW_STATUS_AUXADC_IMP_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_NAG_C_DLTV, PMIC_RG_INT_RAW_STATUS_NAG_C_DLTV_ADDR,
	 PMIC_RG_INT_RAW_STATUS_NAG_C_DLTV_MASK, PMIC_RG_INT_RAW_STATUS_NAG_C_DLTV_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_JEITA_HOT, PMIC_RG_INT_RAW_STATUS_JEITA_HOT_ADDR,
	 PMIC_RG_INT_RAW_STATUS_JEITA_HOT_MASK, PMIC_RG_INT_RAW_STATUS_JEITA_HOT_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_JEITA_WARM, PMIC_RG_INT_RAW_STATUS_JEITA_WARM_ADDR,
	 PMIC_RG_INT_RAW_STATUS_JEITA_WARM_MASK, PMIC_RG_INT_RAW_STATUS_JEITA_WARM_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_JEITA_COOL, PMIC_RG_INT_RAW_STATUS_JEITA_COOL_ADDR,
	 PMIC_RG_INT_RAW_STATUS_JEITA_COOL_MASK, PMIC_RG_INT_RAW_STATUS_JEITA_COOL_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_JEITA_COLD, PMIC_RG_INT_RAW_STATUS_JEITA_COLD_ADDR,
	 PMIC_RG_INT_RAW_STATUS_JEITA_COLD_MASK, PMIC_RG_INT_RAW_STATUS_JEITA_COLD_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_VCORE_OC, PMIC_RG_INT_RAW_STATUS_VCORE_OC_ADDR,
	 PMIC_RG_INT_RAW_STATUS_VCORE_OC_MASK, PMIC_RG_INT_RAW_STATUS_VCORE_OC_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_VMD1_OC, PMIC_RG_INT_RAW_STATUS_VMD1_OC_ADDR,
	 PMIC_RG_INT_RAW_STATUS_VMD1_OC_MASK, PMIC_RG_INT_RAW_STATUS_VMD1_OC_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_VMODEM_OC, PMIC_RG_INT_RAW_STATUS_VMODEM_OC_ADDR,
	 PMIC_RG_INT_RAW_STATUS_VMODEM_OC_MASK, PMIC_RG_INT_RAW_STATUS_VMODEM_OC_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_VS1_OC, PMIC_RG_INT_RAW_STATUS_VS1_OC_ADDR,
	 PMIC_RG_INT_RAW_STATUS_VS1_OC_MASK, PMIC_RG_INT_RAW_STATUS_VS1_OC_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_VS2_OC, PMIC_RG_INT_RAW_STATUS_VS2_OC_ADDR,
	 PMIC_RG_INT_RAW_STATUS_VS2_OC_MASK, PMIC_RG_INT_RAW_STATUS_VS2_OC_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_VDRAM_OC, PMIC_RG_INT_RAW_STATUS_VDRAM_OC_ADDR,
	 PMIC_RG_INT_RAW_STATUS_VDRAM_OC_MASK, PMIC_RG_INT_RAW_STATUS_VDRAM_OC_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_VPA1_OC, PMIC_RG_INT_RAW_STATUS_VPA1_OC_ADDR,
	 PMIC_RG_INT_RAW_STATUS_VPA1_OC_MASK, PMIC_RG_INT_RAW_STATUS_VPA1_OC_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_VPA2_OC, PMIC_RG_INT_RAW_STATUS_VPA2_OC_ADDR,
	 PMIC_RG_INT_RAW_STATUS_VPA2_OC_MASK, PMIC_RG_INT_RAW_STATUS_VPA2_OC_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_VCORE_PREOC, PMIC_RG_INT_RAW_STATUS_VCORE_PREOC_ADDR,
	 PMIC_RG_INT_RAW_STATUS_VCORE_PREOC_MASK, PMIC_RG_INT_RAW_STATUS_VCORE_PREOC_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_VA10_OC, PMIC_RG_INT_RAW_STATUS_VA10_OC_ADDR,
	 PMIC_RG_INT_RAW_STATUS_VA10_OC_MASK, PMIC_RG_INT_RAW_STATUS_VA10_OC_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_VA12_OC, PMIC_RG_INT_RAW_STATUS_VA12_OC_ADDR,
	 PMIC_RG_INT_RAW_STATUS_VA12_OC_MASK, PMIC_RG_INT_RAW_STATUS_VA12_OC_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_VA18_OC, PMIC_RG_INT_RAW_STATUS_VA18_OC_ADDR,
	 PMIC_RG_INT_RAW_STATUS_VA18_OC_MASK, PMIC_RG_INT_RAW_STATUS_VA18_OC_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_VBIF28_OC, PMIC_RG_INT_RAW_STATUS_VBIF28_OC_ADDR,
	 PMIC_RG_INT_RAW_STATUS_VBIF28_OC_MASK, PMIC_RG_INT_RAW_STATUS_VBIF28_OC_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_VCAMA1_OC, PMIC_RG_INT_RAW_STATUS_VCAMA1_OC_ADDR,
	 PMIC_RG_INT_RAW_STATUS_VCAMA1_OC_MASK, PMIC_RG_INT_RAW_STATUS_VCAMA1_OC_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_VCAMA2_OC, PMIC_RG_INT_RAW_STATUS_VCAMA2_OC_ADDR,
	 PMIC_RG_INT_RAW_STATUS_VCAMA2_OC_MASK, PMIC_RG_INT_RAW_STATUS_VCAMA2_OC_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_VCAMAF_OC, PMIC_RG_INT_RAW_STATUS_VCAMAF_OC_ADDR,
	 PMIC_RG_INT_RAW_STATUS_VCAMAF_OC_MASK, PMIC_RG_INT_RAW_STATUS_VCAMAF_OC_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_VCAMD1_OC, PMIC_RG_INT_RAW_STATUS_VCAMD1_OC_ADDR,
	 PMIC_RG_INT_RAW_STATUS_VCAMD1_OC_MASK, PMIC_RG_INT_RAW_STATUS_VCAMD1_OC_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_VCAMD2_OC, PMIC_RG_INT_RAW_STATUS_VCAMD2_OC_ADDR,
	 PMIC_RG_INT_RAW_STATUS_VCAMD2_OC_MASK, PMIC_RG_INT_RAW_STATUS_VCAMD2_OC_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_VCAMIO_OC, PMIC_RG_INT_RAW_STATUS_VCAMIO_OC_ADDR,
	 PMIC_RG_INT_RAW_STATUS_VCAMIO_OC_MASK, PMIC_RG_INT_RAW_STATUS_VCAMIO_OC_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_VCN18_OC, PMIC_RG_INT_RAW_STATUS_VCN18_OC_ADDR,
	 PMIC_RG_INT_RAW_STATUS_VCN18_OC_MASK, PMIC_RG_INT_RAW_STATUS_VCN18_OC_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_VCN28_OC, PMIC_RG_INT_RAW_STATUS_VCN28_OC_ADDR,
	 PMIC_RG_INT_RAW_STATUS_VCN28_OC_MASK, PMIC_RG_INT_RAW_STATUS_VCN28_OC_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_VCN33_OC, PMIC_RG_INT_RAW_STATUS_VCN33_OC_ADDR,
	 PMIC_RG_INT_RAW_STATUS_VCN33_OC_MASK, PMIC_RG_INT_RAW_STATUS_VCN33_OC_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_VEFUSE_OC, PMIC_RG_INT_RAW_STATUS_VEFUSE_OC_ADDR,
	 PMIC_RG_INT_RAW_STATUS_VEFUSE_OC_MASK, PMIC_RG_INT_RAW_STATUS_VEFUSE_OC_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_VEMC_OC, PMIC_RG_INT_RAW_STATUS_VEMC_OC_ADDR,
	 PMIC_RG_INT_RAW_STATUS_VEMC_OC_MASK, PMIC_RG_INT_RAW_STATUS_VEMC_OC_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_VFE28_OC, PMIC_RG_INT_RAW_STATUS_VFE28_OC_ADDR,
	 PMIC_RG_INT_RAW_STATUS_VFE28_OC_MASK, PMIC_RG_INT_RAW_STATUS_VFE28_OC_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_VGP3_OC, PMIC_RG_INT_RAW_STATUS_VGP3_OC_ADDR,
	 PMIC_RG_INT_RAW_STATUS_VGP3_OC_MASK, PMIC_RG_INT_RAW_STATUS_VGP3_OC_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_VIBR_OC, PMIC_RG_INT_RAW_STATUS_VIBR_OC_ADDR,
	 PMIC_RG_INT_RAW_STATUS_VIBR_OC_MASK, PMIC_RG_INT_RAW_STATUS_VIBR_OC_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_VIO18_OC, PMIC_RG_INT_RAW_STATUS_VIO18_OC_ADDR,
	 PMIC_RG_INT_RAW_STATUS_VIO18_OC_MASK, PMIC_RG_INT_RAW_STATUS_VIO18_OC_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_VIO28_OC, PMIC_RG_INT_RAW_STATUS_VIO28_OC_ADDR,
	 PMIC_RG_INT_RAW_STATUS_VIO28_OC_MASK, PMIC_RG_INT_RAW_STATUS_VIO28_OC_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_VMC_OC, PMIC_RG_INT_RAW_STATUS_VMC_OC_ADDR,
	 PMIC_RG_INT_RAW_STATUS_VMC_OC_MASK, PMIC_RG_INT_RAW_STATUS_VMC_OC_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_VMCH_OC, PMIC_RG_INT_RAW_STATUS_VMCH_OC_ADDR,
	 PMIC_RG_INT_RAW_STATUS_VMCH_OC_MASK, PMIC_RG_INT_RAW_STATUS_VMCH_OC_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_VMIPI_OC, PMIC_RG_INT_RAW_STATUS_VMIPI_OC_ADDR,
	 PMIC_RG_INT_RAW_STATUS_VMIPI_OC_MASK, PMIC_RG_INT_RAW_STATUS_VMIPI_OC_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_VRF12_OC, PMIC_RG_INT_RAW_STATUS_VRF12_OC_ADDR,
	 PMIC_RG_INT_RAW_STATUS_VRF12_OC_MASK, PMIC_RG_INT_RAW_STATUS_VRF12_OC_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_VRF18_1_OC, PMIC_RG_INT_RAW_STATUS_VRF18_1_OC_ADDR,
	 PMIC_RG_INT_RAW_STATUS_VRF18_1_OC_MASK, PMIC_RG_INT_RAW_STATUS_VRF18_1_OC_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_VRF18_2_OC, PMIC_RG_INT_RAW_STATUS_VRF18_2_OC_ADDR,
	 PMIC_RG_INT_RAW_STATUS_VRF18_2_OC_MASK, PMIC_RG_INT_RAW_STATUS_VRF18_2_OC_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_VSIM1_OC, PMIC_RG_INT_RAW_STATUS_VSIM1_OC_ADDR,
	 PMIC_RG_INT_RAW_STATUS_VSIM1_OC_MASK, PMIC_RG_INT_RAW_STATUS_VSIM1_OC_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_VSIM2_OC, PMIC_RG_INT_RAW_STATUS_VSIM2_OC_ADDR,
	 PMIC_RG_INT_RAW_STATUS_VSIM2_OC_MASK, PMIC_RG_INT_RAW_STATUS_VSIM2_OC_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_VSRAM_CORE_OC, PMIC_RG_INT_RAW_STATUS_VSRAM_CORE_OC_ADDR,
	 PMIC_RG_INT_RAW_STATUS_VSRAM_CORE_OC_MASK, PMIC_RG_INT_RAW_STATUS_VSRAM_CORE_OC_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_VSRAM_DVFS1_OC, PMIC_RG_INT_RAW_STATUS_VSRAM_DVFS1_OC_ADDR,
	 PMIC_RG_INT_RAW_STATUS_VSRAM_DVFS1_OC_MASK, PMIC_RG_INT_RAW_STATUS_VSRAM_DVFS1_OC_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_VSRAM_DVFS2_OC, PMIC_RG_INT_RAW_STATUS_VSRAM_DVFS2_OC_ADDR,
	 PMIC_RG_INT_RAW_STATUS_VSRAM_DVFS2_OC_MASK, PMIC_RG_INT_RAW_STATUS_VSRAM_DVFS2_OC_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_VSRAM_GPU_OC, PMIC_RG_INT_RAW_STATUS_VSRAM_GPU_OC_ADDR,
	 PMIC_RG_INT_RAW_STATUS_VSRAM_GPU_OC_MASK, PMIC_RG_INT_RAW_STATUS_VSRAM_GPU_OC_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_VSRAM_MD_OC, PMIC_RG_INT_RAW_STATUS_VSRAM_MD_OC_ADDR,
	 PMIC_RG_INT_RAW_STATUS_VSRAM_MD_OC_MASK, PMIC_RG_INT_RAW_STATUS_VSRAM_MD_OC_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_VUFS18_OC, PMIC_RG_INT_RAW_STATUS_VUFS18_OC_ADDR,
	 PMIC_RG_INT_RAW_STATUS_VUFS18_OC_MASK, PMIC_RG_INT_RAW_STATUS_VUFS18_OC_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_VUSB33_OC, PMIC_RG_INT_RAW_STATUS_VUSB33_OC_ADDR,
	 PMIC_RG_INT_RAW_STATUS_VUSB33_OC_MASK, PMIC_RG_INT_RAW_STATUS_VUSB33_OC_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_VXO22_OC, PMIC_RG_INT_RAW_STATUS_VXO22_OC_ADDR,
	 PMIC_RG_INT_RAW_STATUS_VXO22_OC_MASK, PMIC_RG_INT_RAW_STATUS_VXO22_OC_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_CON4, PMIC_RG_INT_RAW_STATUS_CON4_ADDR,
	 PMIC_RG_INT_RAW_STATUS_CON4_MASK, PMIC_RG_INT_RAW_STATUS_CON4_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_FG_BAT0_H, PMIC_RG_INT_RAW_STATUS_FG_BAT0_H_ADDR,
	 PMIC_RG_INT_RAW_STATUS_FG_BAT0_H_MASK, PMIC_RG_INT_RAW_STATUS_FG_BAT0_H_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_FG_BAT0_L, PMIC_RG_INT_RAW_STATUS_FG_BAT0_L_ADDR,
	 PMIC_RG_INT_RAW_STATUS_FG_BAT0_L_MASK, PMIC_RG_INT_RAW_STATUS_FG_BAT0_L_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_FG_CUR_H, PMIC_RG_INT_RAW_STATUS_FG_CUR_H_ADDR,
	 PMIC_RG_INT_RAW_STATUS_FG_CUR_H_MASK, PMIC_RG_INT_RAW_STATUS_FG_CUR_H_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_FG_CUR_L, PMIC_RG_INT_RAW_STATUS_FG_CUR_L_ADDR,
	 PMIC_RG_INT_RAW_STATUS_FG_CUR_L_MASK, PMIC_RG_INT_RAW_STATUS_FG_CUR_L_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_FG_ZCV, PMIC_RG_INT_RAW_STATUS_FG_ZCV_ADDR,
	 PMIC_RG_INT_RAW_STATUS_FG_ZCV_MASK, PMIC_RG_INT_RAW_STATUS_FG_ZCV_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_FG_BAT1_H, PMIC_RG_INT_RAW_STATUS_FG_BAT1_H_ADDR,
	 PMIC_RG_INT_RAW_STATUS_FG_BAT1_H_MASK, PMIC_RG_INT_RAW_STATUS_FG_BAT1_H_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_FG_BAT1_L, PMIC_RG_INT_RAW_STATUS_FG_BAT1_L_ADDR,
	 PMIC_RG_INT_RAW_STATUS_FG_BAT1_L_MASK, PMIC_RG_INT_RAW_STATUS_FG_BAT1_L_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_FG_N_CHARGE_L, PMIC_RG_INT_RAW_STATUS_FG_N_CHARGE_L_ADDR,
	 PMIC_RG_INT_RAW_STATUS_FG_N_CHARGE_L_MASK, PMIC_RG_INT_RAW_STATUS_FG_N_CHARGE_L_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_FG_IAVG_H, PMIC_RG_INT_RAW_STATUS_FG_IAVG_H_ADDR,
	 PMIC_RG_INT_RAW_STATUS_FG_IAVG_H_MASK, PMIC_RG_INT_RAW_STATUS_FG_IAVG_H_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_FG_IAVG_L, PMIC_RG_INT_RAW_STATUS_FG_IAVG_L_ADDR,
	 PMIC_RG_INT_RAW_STATUS_FG_IAVG_L_MASK, PMIC_RG_INT_RAW_STATUS_FG_IAVG_L_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_FG_TIME_H, PMIC_RG_INT_RAW_STATUS_FG_TIME_H_ADDR,
	 PMIC_RG_INT_RAW_STATUS_FG_TIME_H_MASK, PMIC_RG_INT_RAW_STATUS_FG_TIME_H_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_FG_DISCHARGE, PMIC_RG_INT_RAW_STATUS_FG_DISCHARGE_ADDR,
	 PMIC_RG_INT_RAW_STATUS_FG_DISCHARGE_MASK, PMIC_RG_INT_RAW_STATUS_FG_DISCHARGE_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_FG_CHARGE, PMIC_RG_INT_RAW_STATUS_FG_CHARGE_ADDR,
	 PMIC_RG_INT_RAW_STATUS_FG_CHARGE_MASK, PMIC_RG_INT_RAW_STATUS_FG_CHARGE_SHIFT},
	{PMIC_RG_INT_RAW_STATUS_CON5, PMIC_RG_INT_RAW_STATUS_CON5_ADDR,
	 PMIC_RG_INT_RAW_STATUS_CON5_MASK, PMIC_RG_INT_RAW_STATUS_CON5_SHIFT},
	{PMIC_POLARITY, PMIC_POLARITY_ADDR, PMIC_POLARITY_MASK, PMIC_POLARITY_SHIFT},
	{PMIC_RG_HOMEKEY_INT_SEL, PMIC_RG_HOMEKEY_INT_SEL_ADDR, PMIC_RG_HOMEKEY_INT_SEL_MASK,
	 PMIC_RG_HOMEKEY_INT_SEL_SHIFT},
	{PMIC_RG_PWRKEY_INT_SEL, PMIC_RG_PWRKEY_INT_SEL_ADDR, PMIC_RG_PWRKEY_INT_SEL_MASK,
	 PMIC_RG_PWRKEY_INT_SEL_SHIFT},
	{PMIC_RG_CHRDET_INT_SEL, PMIC_RG_CHRDET_INT_SEL_ADDR, PMIC_RG_CHRDET_INT_SEL_MASK,
	 PMIC_RG_CHRDET_INT_SEL_SHIFT},
	{PMIC_RG_PCHR_CM_VINC_POLARITY_RSV, PMIC_RG_PCHR_CM_VINC_POLARITY_RSV_ADDR,
	 PMIC_RG_PCHR_CM_VINC_POLARITY_RSV_MASK, PMIC_RG_PCHR_CM_VINC_POLARITY_RSV_SHIFT},
	{PMIC_RG_PCHR_CM_VDEC_POLARITY_RSV, PMIC_RG_PCHR_CM_VDEC_POLARITY_RSV_ADDR,
	 PMIC_RG_PCHR_CM_VDEC_POLARITY_RSV_MASK, PMIC_RG_PCHR_CM_VDEC_POLARITY_RSV_SHIFT},
	{PMIC_INT_MISC_CON_SET, PMIC_INT_MISC_CON_SET_ADDR, PMIC_INT_MISC_CON_SET_MASK,
	 PMIC_INT_MISC_CON_SET_SHIFT},
	{PMIC_INT_MISC_CON_CLR, PMIC_INT_MISC_CON_CLR_ADDR, PMIC_INT_MISC_CON_CLR_MASK,
	 PMIC_INT_MISC_CON_CLR_SHIFT},
	{PMIC_FQMTR_TCKSEL, PMIC_FQMTR_TCKSEL_ADDR, PMIC_FQMTR_TCKSEL_MASK,
	 PMIC_FQMTR_TCKSEL_SHIFT},
	{PMIC_FQMTR_BUSY, PMIC_FQMTR_BUSY_ADDR, PMIC_FQMTR_BUSY_MASK, PMIC_FQMTR_BUSY_SHIFT},
	{PMIC_FQMTR_DCXO26M_EN, PMIC_FQMTR_DCXO26M_EN_ADDR, PMIC_FQMTR_DCXO26M_EN_MASK,
	 PMIC_FQMTR_DCXO26M_EN_SHIFT},
	{PMIC_FQMTR_EN, PMIC_FQMTR_EN_ADDR, PMIC_FQMTR_EN_MASK, PMIC_FQMTR_EN_SHIFT},
	{PMIC_FQMTR_WINSET, PMIC_FQMTR_WINSET_ADDR, PMIC_FQMTR_WINSET_MASK,
	 PMIC_FQMTR_WINSET_SHIFT},
	{PMIC_FQMTR_DATA, PMIC_FQMTR_DATA_ADDR, PMIC_FQMTR_DATA_MASK, PMIC_FQMTR_DATA_SHIFT},
	{PMIC_RG_SLP_RW_EN, PMIC_RG_SLP_RW_EN_ADDR, PMIC_RG_SLP_RW_EN_MASK,
	 PMIC_RG_SLP_RW_EN_SHIFT},
	{PMIC_RG_SPI_RSV, PMIC_RG_SPI_RSV_ADDR, PMIC_RG_SPI_RSV_MASK, PMIC_RG_SPI_RSV_SHIFT},
	{PMIC_DEW_DIO_EN, PMIC_DEW_DIO_EN_ADDR, PMIC_DEW_DIO_EN_MASK, PMIC_DEW_DIO_EN_SHIFT},
	{PMIC_DEW_READ_TEST, PMIC_DEW_READ_TEST_ADDR, PMIC_DEW_READ_TEST_MASK,
	 PMIC_DEW_READ_TEST_SHIFT},
	{PMIC_DEW_WRITE_TEST, PMIC_DEW_WRITE_TEST_ADDR, PMIC_DEW_WRITE_TEST_MASK,
	 PMIC_DEW_WRITE_TEST_SHIFT},
	{PMIC_DEW_CRC_SWRST, PMIC_DEW_CRC_SWRST_ADDR, PMIC_DEW_CRC_SWRST_MASK,
	 PMIC_DEW_CRC_SWRST_SHIFT},
	{PMIC_DEW_CRC_EN, PMIC_DEW_CRC_EN_ADDR, PMIC_DEW_CRC_EN_MASK, PMIC_DEW_CRC_EN_SHIFT},
	{PMIC_DEW_CRC_VAL, PMIC_DEW_CRC_VAL_ADDR, PMIC_DEW_CRC_VAL_MASK, PMIC_DEW_CRC_VAL_SHIFT},
	{PMIC_DEW_DBG_MON_SEL, PMIC_DEW_DBG_MON_SEL_ADDR, PMIC_DEW_DBG_MON_SEL_MASK,
	 PMIC_DEW_DBG_MON_SEL_SHIFT},
	{PMIC_DEW_CIPHER_KEY_SEL, PMIC_DEW_CIPHER_KEY_SEL_ADDR, PMIC_DEW_CIPHER_KEY_SEL_MASK,
	 PMIC_DEW_CIPHER_KEY_SEL_SHIFT},
	{PMIC_DEW_CIPHER_IV_SEL, PMIC_DEW_CIPHER_IV_SEL_ADDR, PMIC_DEW_CIPHER_IV_SEL_MASK,
	 PMIC_DEW_CIPHER_IV_SEL_SHIFT},
	{PMIC_DEW_CIPHER_EN, PMIC_DEW_CIPHER_EN_ADDR, PMIC_DEW_CIPHER_EN_MASK,
	 PMIC_DEW_CIPHER_EN_SHIFT},
	{PMIC_DEW_CIPHER_RDY, PMIC_DEW_CIPHER_RDY_ADDR, PMIC_DEW_CIPHER_RDY_MASK,
	 PMIC_DEW_CIPHER_RDY_SHIFT},
	{PMIC_DEW_CIPHER_MODE, PMIC_DEW_CIPHER_MODE_ADDR, PMIC_DEW_CIPHER_MODE_MASK,
	 PMIC_DEW_CIPHER_MODE_SHIFT},
	{PMIC_DEW_CIPHER_SWRST, PMIC_DEW_CIPHER_SWRST_ADDR, PMIC_DEW_CIPHER_SWRST_MASK,
	 PMIC_DEW_CIPHER_SWRST_SHIFT},
	{PMIC_DEW_RDDMY_NO, PMIC_DEW_RDDMY_NO_ADDR, PMIC_DEW_RDDMY_NO_MASK,
	 PMIC_DEW_RDDMY_NO_SHIFT},
	{PMIC_INT_TYPE_CON0, PMIC_INT_TYPE_CON0_ADDR, PMIC_INT_TYPE_CON0_MASK,
	 PMIC_INT_TYPE_CON0_SHIFT},
	{PMIC_INT_TYPE_CON0_SET, PMIC_INT_TYPE_CON0_SET_ADDR, PMIC_INT_TYPE_CON0_SET_MASK,
	 PMIC_INT_TYPE_CON0_SET_SHIFT},
	{PMIC_INT_TYPE_CON0_CLR, PMIC_INT_TYPE_CON0_CLR_ADDR, PMIC_INT_TYPE_CON0_CLR_MASK,
	 PMIC_INT_TYPE_CON0_CLR_SHIFT},
	{PMIC_INT_TYPE_CON1, PMIC_INT_TYPE_CON1_ADDR, PMIC_INT_TYPE_CON1_MASK,
	 PMIC_INT_TYPE_CON1_SHIFT},
	{PMIC_INT_TYPE_CON1_SET, PMIC_INT_TYPE_CON1_SET_ADDR, PMIC_INT_TYPE_CON1_SET_MASK,
	 PMIC_INT_TYPE_CON1_SET_SHIFT},
	{PMIC_INT_TYPE_CON1_CLR, PMIC_INT_TYPE_CON1_CLR_ADDR, PMIC_INT_TYPE_CON1_CLR_MASK,
	 PMIC_INT_TYPE_CON1_CLR_SHIFT},
	{PMIC_INT_TYPE_CON2, PMIC_INT_TYPE_CON2_ADDR, PMIC_INT_TYPE_CON2_MASK,
	 PMIC_INT_TYPE_CON2_SHIFT},
	{PMIC_INT_TYPE_CON2_SET, PMIC_INT_TYPE_CON2_SET_ADDR, PMIC_INT_TYPE_CON2_SET_MASK,
	 PMIC_INT_TYPE_CON2_SET_SHIFT},
	{PMIC_INT_TYPE_CON2_CLR, PMIC_INT_TYPE_CON2_CLR_ADDR, PMIC_INT_TYPE_CON2_CLR_MASK,
	 PMIC_INT_TYPE_CON2_CLR_SHIFT},
	{PMIC_INT_TYPE_CON3, PMIC_INT_TYPE_CON3_ADDR, PMIC_INT_TYPE_CON3_MASK,
	 PMIC_INT_TYPE_CON3_SHIFT},
	{PMIC_INT_TYPE_CON3_SET, PMIC_INT_TYPE_CON3_SET_ADDR, PMIC_INT_TYPE_CON3_SET_MASK,
	 PMIC_INT_TYPE_CON3_SET_SHIFT},
	{PMIC_INT_TYPE_CON3_CLR, PMIC_INT_TYPE_CON3_CLR_ADDR, PMIC_INT_TYPE_CON3_CLR_MASK,
	 PMIC_INT_TYPE_CON3_CLR_SHIFT},
	{PMIC_INT_TYPE_CON4, PMIC_INT_TYPE_CON4_ADDR, PMIC_INT_TYPE_CON4_MASK,
	 PMIC_INT_TYPE_CON4_SHIFT},
	{PMIC_INT_TYPE_CON4_SET, PMIC_INT_TYPE_CON4_SET_ADDR, PMIC_INT_TYPE_CON4_SET_MASK,
	 PMIC_INT_TYPE_CON4_SET_SHIFT},
	{PMIC_INT_TYPE_CON4_CLR, PMIC_INT_TYPE_CON4_CLR_ADDR, PMIC_INT_TYPE_CON4_CLR_MASK,
	 PMIC_INT_TYPE_CON4_CLR_SHIFT},
	{PMIC_INT_TYPE_CON5, PMIC_INT_TYPE_CON5_ADDR, PMIC_INT_TYPE_CON5_MASK,
	 PMIC_INT_TYPE_CON5_SHIFT},
	{PMIC_INT_TYPE_CON5_SET, PMIC_INT_TYPE_CON5_SET_ADDR, PMIC_INT_TYPE_CON5_SET_MASK,
	 PMIC_INT_TYPE_CON5_SET_SHIFT},
	{PMIC_INT_TYPE_CON5_CLR, PMIC_INT_TYPE_CON5_CLR_ADDR, PMIC_INT_TYPE_CON5_CLR_MASK,
	 PMIC_INT_TYPE_CON5_CLR_SHIFT},
	{PMIC_CPU_INT_STA, PMIC_CPU_INT_STA_ADDR, PMIC_CPU_INT_STA_MASK, PMIC_CPU_INT_STA_SHIFT},
	{PMIC_MD32_INT_STA, PMIC_MD32_INT_STA_ADDR, PMIC_MD32_INT_STA_MASK,
	 PMIC_MD32_INT_STA_SHIFT},
	{PMIC_RG_SRCLKEN_IN3_SMPS_CLK_MODE, PMIC_RG_SRCLKEN_IN3_SMPS_CLK_MODE_ADDR,
	 PMIC_RG_SRCLKEN_IN3_SMPS_CLK_MODE_MASK, PMIC_RG_SRCLKEN_IN3_SMPS_CLK_MODE_SHIFT},
	{PMIC_RG_SRCLKEN_IN3_EN_SMPS_TEST, PMIC_RG_SRCLKEN_IN3_EN_SMPS_TEST_ADDR,
	 PMIC_RG_SRCLKEN_IN3_EN_SMPS_TEST_MASK, PMIC_RG_SRCLKEN_IN3_EN_SMPS_TEST_SHIFT},
	{PMIC_RG_SRCLKEN_IN2_SMPS_CLK_MODE, PMIC_RG_SRCLKEN_IN2_SMPS_CLK_MODE_ADDR,
	 PMIC_RG_SRCLKEN_IN2_SMPS_CLK_MODE_MASK, PMIC_RG_SRCLKEN_IN2_SMPS_CLK_MODE_SHIFT},
	{PMIC_RG_SRCLKEN_IN2_EN_SMPS_TEST, PMIC_RG_SRCLKEN_IN2_EN_SMPS_TEST_ADDR,
	 PMIC_RG_SRCLKEN_IN2_EN_SMPS_TEST_MASK, PMIC_RG_SRCLKEN_IN2_EN_SMPS_TEST_SHIFT},
	{PMIC_RG_SPI_DLY_SEL, PMIC_RG_SPI_DLY_SEL_ADDR, PMIC_RG_SPI_DLY_SEL_MASK,
	 PMIC_RG_SPI_DLY_SEL_SHIFT},
	{PMIC_RG_BUCK_LDO_FT_EN, PMIC_RG_BUCK_LDO_FT_EN_ADDR, PMIC_RG_BUCK_LDO_FT_EN_MASK,
	 PMIC_RG_BUCK_LDO_FT_EN_SHIFT},
	{PMIC_RG_BUCK_DCM_MODE, PMIC_RG_BUCK_DCM_MODE_ADDR, PMIC_RG_BUCK_DCM_MODE_MASK,
	 PMIC_RG_BUCK_DCM_MODE_SHIFT},
	{PMIC_RG_BUCK_ALL_CON0_RSV, PMIC_RG_BUCK_ALL_CON0_RSV_ADDR, PMIC_RG_BUCK_ALL_CON0_RSV_MASK,
	 PMIC_RG_BUCK_ALL_CON0_RSV_SHIFT},
	{PMIC_RG_BUCK_STB_MAX, PMIC_RG_BUCK_STB_MAX_ADDR, PMIC_RG_BUCK_STB_MAX_MASK,
	 PMIC_RG_BUCK_STB_MAX_SHIFT},
	{PMIC_RG_BUCK_LP_PROT_DISABLE, PMIC_RG_BUCK_LP_PROT_DISABLE_ADDR,
	 PMIC_RG_BUCK_LP_PROT_DISABLE_MASK, PMIC_RG_BUCK_LP_PROT_DISABLE_SHIFT},
	{PMIC_RG_BUCK_VSLEEP_SRC0, PMIC_RG_BUCK_VSLEEP_SRC0_ADDR, PMIC_RG_BUCK_VSLEEP_SRC0_MASK,
	 PMIC_RG_BUCK_VSLEEP_SRC0_SHIFT},
	{PMIC_RG_BUCK_VSLEEP_SRC1, PMIC_RG_BUCK_VSLEEP_SRC1_ADDR, PMIC_RG_BUCK_VSLEEP_SRC1_MASK,
	 PMIC_RG_BUCK_VSLEEP_SRC1_SHIFT},
	{PMIC_RG_BUCK_R2R_SRC0, PMIC_RG_BUCK_R2R_SRC0_ADDR, PMIC_RG_BUCK_R2R_SRC0_MASK,
	 PMIC_RG_BUCK_R2R_SRC0_SHIFT},
	{PMIC_RG_BUCK_R2R_SRC1, PMIC_RG_BUCK_R2R_SRC1_ADDR, PMIC_RG_BUCK_R2R_SRC1_MASK,
	 PMIC_RG_BUCK_R2R_SRC1_SHIFT},
	{PMIC_RG_BUCK_LP_SEQ_COUNT, PMIC_RG_BUCK_LP_SEQ_COUNT_ADDR, PMIC_RG_BUCK_LP_SEQ_COUNT_MASK,
	 PMIC_RG_BUCK_LP_SEQ_COUNT_SHIFT},
	{PMIC_RG_BUCK_ON_SEQ_COUNT, PMIC_RG_BUCK_ON_SEQ_COUNT_ADDR, PMIC_RG_BUCK_ON_SEQ_COUNT_MASK,
	 PMIC_RG_BUCK_ON_SEQ_COUNT_SHIFT},
	{PMIC_RG_BUCK_VCORE_OC_SDN_STATUS, PMIC_RG_BUCK_VCORE_OC_SDN_STATUS_ADDR,
	 PMIC_RG_BUCK_VCORE_OC_SDN_STATUS_MASK, PMIC_RG_BUCK_VCORE_OC_SDN_STATUS_SHIFT},
	{PMIC_RG_BUCK_VDRAM_OC_SDN_STATUS, PMIC_RG_BUCK_VDRAM_OC_SDN_STATUS_ADDR,
	 PMIC_RG_BUCK_VDRAM_OC_SDN_STATUS_MASK, PMIC_RG_BUCK_VDRAM_OC_SDN_STATUS_SHIFT},
	{PMIC_RG_BUCK_VMODEM_OC_SDN_STATUS, PMIC_RG_BUCK_VMODEM_OC_SDN_STATUS_ADDR,
	 PMIC_RG_BUCK_VMODEM_OC_SDN_STATUS_MASK, PMIC_RG_BUCK_VMODEM_OC_SDN_STATUS_SHIFT},
	{PMIC_RG_BUCK_VMD1_OC_SDN_STATUS, PMIC_RG_BUCK_VMD1_OC_SDN_STATUS_ADDR,
	 PMIC_RG_BUCK_VMD1_OC_SDN_STATUS_MASK, PMIC_RG_BUCK_VMD1_OC_SDN_STATUS_SHIFT},
	{PMIC_RG_BUCK_VS1_OC_SDN_STATUS, PMIC_RG_BUCK_VS1_OC_SDN_STATUS_ADDR,
	 PMIC_RG_BUCK_VS1_OC_SDN_STATUS_MASK, PMIC_RG_BUCK_VS1_OC_SDN_STATUS_SHIFT},
	{PMIC_RG_BUCK_VS2_OC_SDN_STATUS, PMIC_RG_BUCK_VS2_OC_SDN_STATUS_ADDR,
	 PMIC_RG_BUCK_VS2_OC_SDN_STATUS_MASK, PMIC_RG_BUCK_VS2_OC_SDN_STATUS_SHIFT},
	{PMIC_RG_BUCK_VPA1_OC_SDN_STATUS, PMIC_RG_BUCK_VPA1_OC_SDN_STATUS_ADDR,
	 PMIC_RG_BUCK_VPA1_OC_SDN_STATUS_MASK, PMIC_RG_BUCK_VPA1_OC_SDN_STATUS_SHIFT},
	{PMIC_RG_BUCK_VPA2_OC_SDN_STATUS, PMIC_RG_BUCK_VPA2_OC_SDN_STATUS_ADDR,
	 PMIC_RG_BUCK_VPA2_OC_SDN_STATUS_MASK, PMIC_RG_BUCK_VPA2_OC_SDN_STATUS_SHIFT},
	{PMIC_RG_BUCK_VCORE_OC_SDN_EN, PMIC_RG_BUCK_VCORE_OC_SDN_EN_ADDR,
	 PMIC_RG_BUCK_VCORE_OC_SDN_EN_MASK, PMIC_RG_BUCK_VCORE_OC_SDN_EN_SHIFT},
	{PMIC_RG_BUCK_VDRAM_OC_SDN_EN, PMIC_RG_BUCK_VDRAM_OC_SDN_EN_ADDR,
	 PMIC_RG_BUCK_VDRAM_OC_SDN_EN_MASK, PMIC_RG_BUCK_VDRAM_OC_SDN_EN_SHIFT},
	{PMIC_RG_BUCK_VMODEM_OC_SDN_EN, PMIC_RG_BUCK_VMODEM_OC_SDN_EN_ADDR,
	 PMIC_RG_BUCK_VMODEM_OC_SDN_EN_MASK, PMIC_RG_BUCK_VMODEM_OC_SDN_EN_SHIFT},
	{PMIC_RG_BUCK_VMD1_OC_SDN_EN, PMIC_RG_BUCK_VMD1_OC_SDN_EN_ADDR,
	 PMIC_RG_BUCK_VMD1_OC_SDN_EN_MASK, PMIC_RG_BUCK_VMD1_OC_SDN_EN_SHIFT},
	{PMIC_RG_BUCK_VS1_OC_SDN_EN, PMIC_RG_BUCK_VS1_OC_SDN_EN_ADDR,
	 PMIC_RG_BUCK_VS1_OC_SDN_EN_MASK, PMIC_RG_BUCK_VS1_OC_SDN_EN_SHIFT},
	{PMIC_RG_BUCK_VS2_OC_SDN_EN, PMIC_RG_BUCK_VS2_OC_SDN_EN_ADDR,
	 PMIC_RG_BUCK_VS2_OC_SDN_EN_MASK, PMIC_RG_BUCK_VS2_OC_SDN_EN_SHIFT},
	{PMIC_RG_BUCK_VPA1_OC_SDN_EN, PMIC_RG_BUCK_VPA1_OC_SDN_EN_ADDR,
	 PMIC_RG_BUCK_VPA1_OC_SDN_EN_MASK, PMIC_RG_BUCK_VPA1_OC_SDN_EN_SHIFT},
	{PMIC_RG_BUCK_VPA2_OC_SDN_EN, PMIC_RG_BUCK_VPA2_OC_SDN_EN_ADDR,
	 PMIC_RG_BUCK_VPA2_OC_SDN_EN_MASK, PMIC_RG_BUCK_VPA2_OC_SDN_EN_SHIFT},
	{PMIC_RG_BUCK_K_RST_DONE, PMIC_RG_BUCK_K_RST_DONE_ADDR, PMIC_RG_BUCK_K_RST_DONE_MASK,
	 PMIC_RG_BUCK_K_RST_DONE_SHIFT},
	{PMIC_RG_BUCK_K_MAP_SEL, PMIC_RG_BUCK_K_MAP_SEL_ADDR, PMIC_RG_BUCK_K_MAP_SEL_MASK,
	 PMIC_RG_BUCK_K_MAP_SEL_SHIFT},
	{PMIC_RG_BUCK_K_ONCE_EN, PMIC_RG_BUCK_K_ONCE_EN_ADDR, PMIC_RG_BUCK_K_ONCE_EN_MASK,
	 PMIC_RG_BUCK_K_ONCE_EN_SHIFT},
	{PMIC_RG_BUCK_K_ONCE, PMIC_RG_BUCK_K_ONCE_ADDR, PMIC_RG_BUCK_K_ONCE_MASK,
	 PMIC_RG_BUCK_K_ONCE_SHIFT},
	{PMIC_RG_BUCK_K_START_MANUAL, PMIC_RG_BUCK_K_START_MANUAL_ADDR,
	 PMIC_RG_BUCK_K_START_MANUAL_MASK, PMIC_RG_BUCK_K_START_MANUAL_SHIFT},
	{PMIC_RG_BUCK_K_SRC_SEL, PMIC_RG_BUCK_K_SRC_SEL_ADDR, PMIC_RG_BUCK_K_SRC_SEL_MASK,
	 PMIC_RG_BUCK_K_SRC_SEL_SHIFT},
	{PMIC_RG_BUCK_K_AUTO_EN, PMIC_RG_BUCK_K_AUTO_EN_ADDR, PMIC_RG_BUCK_K_AUTO_EN_MASK,
	 PMIC_RG_BUCK_K_AUTO_EN_SHIFT},
	{PMIC_RG_BUCK_K_INV, PMIC_RG_BUCK_K_INV_ADDR, PMIC_RG_BUCK_K_INV_MASK,
	 PMIC_RG_BUCK_K_INV_SHIFT},
	{PMIC_RG_BUCK_K_CK_EN, PMIC_RG_BUCK_K_CK_EN_ADDR, PMIC_RG_BUCK_K_CK_EN_MASK,
	 PMIC_RG_BUCK_K_CK_EN_SHIFT},
	{PMIC_RG_BUCK_K_CONTROL_SMPS, PMIC_RG_BUCK_K_CONTROL_SMPS_ADDR,
	 PMIC_RG_BUCK_K_CONTROL_SMPS_MASK, PMIC_RG_BUCK_K_CONTROL_SMPS_SHIFT},
	{PMIC_BUCK_K_RESULT, PMIC_BUCK_K_RESULT_ADDR, PMIC_BUCK_K_RESULT_MASK,
	 PMIC_BUCK_K_RESULT_SHIFT},
	{PMIC_BUCK_K_DONE, PMIC_BUCK_K_DONE_ADDR, PMIC_BUCK_K_DONE_MASK, PMIC_BUCK_K_DONE_SHIFT},
	{PMIC_BUCK_K_CONTROL, PMIC_BUCK_K_CONTROL_ADDR, PMIC_BUCK_K_CONTROL_MASK,
	 PMIC_BUCK_K_CONTROL_SHIFT},
	{PMIC_DA_QI_SMPS_OSC_CAL, PMIC_DA_QI_SMPS_OSC_CAL_ADDR, PMIC_DA_QI_SMPS_OSC_CAL_MASK,
	 PMIC_DA_QI_SMPS_OSC_CAL_SHIFT},
	{PMIC_RG_BUCK_K_BUCK_CK_CNT, PMIC_RG_BUCK_K_BUCK_CK_CNT_ADDR,
	 PMIC_RG_BUCK_K_BUCK_CK_CNT_MASK, PMIC_RG_BUCK_K_BUCK_CK_CNT_SHIFT},
	{PMIC_RG_BUCK_VCORE_SSHUB_MODE, PMIC_RG_BUCK_VCORE_SSHUB_MODE_ADDR,
	 PMIC_RG_BUCK_VCORE_SSHUB_MODE_MASK, PMIC_RG_BUCK_VCORE_SSHUB_MODE_SHIFT},
	{PMIC_RG_BUCK_VCORE_SSHUB_ON, PMIC_RG_BUCK_VCORE_SSHUB_ON_ADDR,
	 PMIC_RG_BUCK_VCORE_SSHUB_ON_MASK, PMIC_RG_BUCK_VCORE_SSHUB_ON_SHIFT},
	{PMIC_RG_BUCK_VCORE_SSHUB_VOSEL, PMIC_RG_BUCK_VCORE_SSHUB_VOSEL_ADDR,
	 PMIC_RG_BUCK_VCORE_SSHUB_VOSEL_MASK, PMIC_RG_BUCK_VCORE_SSHUB_VOSEL_SHIFT},
	{PMIC_RG_BUCK_VCORE_EN, PMIC_RG_BUCK_VCORE_EN_ADDR, PMIC_RG_BUCK_VCORE_EN_MASK,
	 PMIC_RG_BUCK_VCORE_EN_SHIFT},
	{PMIC_RG_BUCK_VCORE_LP, PMIC_RG_BUCK_VCORE_LP_ADDR, PMIC_RG_BUCK_VCORE_LP_MASK,
	 PMIC_RG_BUCK_VCORE_LP_SHIFT},
	{PMIC_RG_BUCK_VCORE_VOSEL, PMIC_RG_BUCK_VCORE_VOSEL_ADDR, PMIC_RG_BUCK_VCORE_VOSEL_MASK,
	 PMIC_RG_BUCK_VCORE_VOSEL_SHIFT},
	{PMIC_RG_BUCK_VCORE_VOSEL_SLEEP, PMIC_RG_BUCK_VCORE_VOSEL_SLEEP_ADDR,
	 PMIC_RG_BUCK_VCORE_VOSEL_SLEEP_MASK, PMIC_RG_BUCK_VCORE_VOSEL_SLEEP_SHIFT},
	{PMIC_RG_BUCK_VCORE_SFCHG_FRATE, PMIC_RG_BUCK_VCORE_SFCHG_FRATE_ADDR,
	 PMIC_RG_BUCK_VCORE_SFCHG_FRATE_MASK, PMIC_RG_BUCK_VCORE_SFCHG_FRATE_SHIFT},
	{PMIC_RG_BUCK_VCORE_SFCHG_FEN, PMIC_RG_BUCK_VCORE_SFCHG_FEN_ADDR,
	 PMIC_RG_BUCK_VCORE_SFCHG_FEN_MASK, PMIC_RG_BUCK_VCORE_SFCHG_FEN_SHIFT},
	{PMIC_RG_BUCK_VCORE_SFCHG_RRATE, PMIC_RG_BUCK_VCORE_SFCHG_RRATE_ADDR,
	 PMIC_RG_BUCK_VCORE_SFCHG_RRATE_MASK, PMIC_RG_BUCK_VCORE_SFCHG_RRATE_SHIFT},
	{PMIC_RG_BUCK_VCORE_SFCHG_REN, PMIC_RG_BUCK_VCORE_SFCHG_REN_ADDR,
	 PMIC_RG_BUCK_VCORE_SFCHG_REN_MASK, PMIC_RG_BUCK_VCORE_SFCHG_REN_SHIFT},
	{PMIC_RG_BUCK_VCORE_DVS_TRANS_TD, PMIC_RG_BUCK_VCORE_DVS_TRANS_TD_ADDR,
	 PMIC_RG_BUCK_VCORE_DVS_TRANS_TD_MASK, PMIC_RG_BUCK_VCORE_DVS_TRANS_TD_SHIFT},
	{PMIC_RG_BUCK_VCORE_DVS_TRANS_CTRL, PMIC_RG_BUCK_VCORE_DVS_TRANS_CTRL_ADDR,
	 PMIC_RG_BUCK_VCORE_DVS_TRANS_CTRL_MASK, PMIC_RG_BUCK_VCORE_DVS_TRANS_CTRL_SHIFT},
	{PMIC_RG_BUCK_VCORE_DVS_TRANS_ONCE, PMIC_RG_BUCK_VCORE_DVS_TRANS_ONCE_ADDR,
	 PMIC_RG_BUCK_VCORE_DVS_TRANS_ONCE_MASK, PMIC_RG_BUCK_VCORE_DVS_TRANS_ONCE_SHIFT},
	{PMIC_RG_BUCK_VCORE_SW_OP_EN, PMIC_RG_BUCK_VCORE_SW_OP_EN_ADDR,
	 PMIC_RG_BUCK_VCORE_SW_OP_EN_MASK, PMIC_RG_BUCK_VCORE_SW_OP_EN_SHIFT},
	{PMIC_RG_BUCK_VCORE_HW0_OP_EN, PMIC_RG_BUCK_VCORE_HW0_OP_EN_ADDR,
	 PMIC_RG_BUCK_VCORE_HW0_OP_EN_MASK, PMIC_RG_BUCK_VCORE_HW0_OP_EN_SHIFT},
	{PMIC_RG_BUCK_VCORE_HW1_OP_EN, PMIC_RG_BUCK_VCORE_HW1_OP_EN_ADDR,
	 PMIC_RG_BUCK_VCORE_HW1_OP_EN_MASK, PMIC_RG_BUCK_VCORE_HW1_OP_EN_SHIFT},
	{PMIC_RG_BUCK_VCORE_HW2_OP_EN, PMIC_RG_BUCK_VCORE_HW2_OP_EN_ADDR,
	 PMIC_RG_BUCK_VCORE_HW2_OP_EN_MASK, PMIC_RG_BUCK_VCORE_HW2_OP_EN_SHIFT},
	{PMIC_RG_BUCK_VCORE_OP_EN_SET, PMIC_RG_BUCK_VCORE_OP_EN_SET_ADDR,
	 PMIC_RG_BUCK_VCORE_OP_EN_SET_MASK, PMIC_RG_BUCK_VCORE_OP_EN_SET_SHIFT},
	{PMIC_RG_BUCK_VCORE_OP_EN_CLR, PMIC_RG_BUCK_VCORE_OP_EN_CLR_ADDR,
	 PMIC_RG_BUCK_VCORE_OP_EN_CLR_MASK, PMIC_RG_BUCK_VCORE_OP_EN_CLR_SHIFT},
	{PMIC_RG_BUCK_VCORE_HW0_OP_CFG, PMIC_RG_BUCK_VCORE_HW0_OP_CFG_ADDR,
	 PMIC_RG_BUCK_VCORE_HW0_OP_CFG_MASK, PMIC_RG_BUCK_VCORE_HW0_OP_CFG_SHIFT},
	{PMIC_RG_BUCK_VCORE_HW1_OP_CFG, PMIC_RG_BUCK_VCORE_HW1_OP_CFG_ADDR,
	 PMIC_RG_BUCK_VCORE_HW1_OP_CFG_MASK, PMIC_RG_BUCK_VCORE_HW1_OP_CFG_SHIFT},
	{PMIC_RG_BUCK_VCORE_HW2_OP_CFG, PMIC_RG_BUCK_VCORE_HW2_OP_CFG_ADDR,
	 PMIC_RG_BUCK_VCORE_HW2_OP_CFG_MASK, PMIC_RG_BUCK_VCORE_HW2_OP_CFG_SHIFT},
	{PMIC_RG_BUCK_VCORE_ON_OP, PMIC_RG_BUCK_VCORE_ON_OP_ADDR, PMIC_RG_BUCK_VCORE_ON_OP_MASK,
	 PMIC_RG_BUCK_VCORE_ON_OP_SHIFT},
	{PMIC_RG_BUCK_VCORE_LP_OP, PMIC_RG_BUCK_VCORE_LP_OP_ADDR, PMIC_RG_BUCK_VCORE_LP_OP_MASK,
	 PMIC_RG_BUCK_VCORE_LP_OP_SHIFT},
	{PMIC_RG_BUCK_VCORE_OP_CFG_SET, PMIC_RG_BUCK_VCORE_OP_CFG_SET_ADDR,
	 PMIC_RG_BUCK_VCORE_OP_CFG_SET_MASK, PMIC_RG_BUCK_VCORE_OP_CFG_SET_SHIFT},
	{PMIC_RG_BUCK_VCORE_OP_CFG_CLR, PMIC_RG_BUCK_VCORE_OP_CFG_CLR_ADDR,
	 PMIC_RG_BUCK_VCORE_OP_CFG_CLR_MASK, PMIC_RG_BUCK_VCORE_OP_CFG_CLR_SHIFT},
	{PMIC_RG_BUCK_VCORE_SP_SW_VOSEL, PMIC_RG_BUCK_VCORE_SP_SW_VOSEL_ADDR,
	 PMIC_RG_BUCK_VCORE_SP_SW_VOSEL_MASK, PMIC_RG_BUCK_VCORE_SP_SW_VOSEL_SHIFT},
	{PMIC_RG_BUCK_VCORE_SP_SW_VOSEL_EN, PMIC_RG_BUCK_VCORE_SP_SW_VOSEL_EN_ADDR,
	 PMIC_RG_BUCK_VCORE_SP_SW_VOSEL_EN_MASK, PMIC_RG_BUCK_VCORE_SP_SW_VOSEL_EN_SHIFT},
	{PMIC_RG_BUCK_VCORE_SP_ON_VOSEL_MUX_SEL, PMIC_RG_BUCK_VCORE_SP_ON_VOSEL_MUX_SEL_ADDR,
	 PMIC_RG_BUCK_VCORE_SP_ON_VOSEL_MUX_SEL_MASK, PMIC_RG_BUCK_VCORE_SP_ON_VOSEL_MUX_SEL_SHIFT},
	{PMIC_RG_BUCK_VCORE_OC_DEG_EN, PMIC_RG_BUCK_VCORE_OC_DEG_EN_ADDR,
	 PMIC_RG_BUCK_VCORE_OC_DEG_EN_MASK, PMIC_RG_BUCK_VCORE_OC_DEG_EN_SHIFT},
	{PMIC_RG_BUCK_VCORE_OC_WND, PMIC_RG_BUCK_VCORE_OC_WND_ADDR, PMIC_RG_BUCK_VCORE_OC_WND_MASK,
	 PMIC_RG_BUCK_VCORE_OC_WND_SHIFT},
	{PMIC_RG_BUCK_VCORE_OC_THD, PMIC_RG_BUCK_VCORE_OC_THD_ADDR, PMIC_RG_BUCK_VCORE_OC_THD_MASK,
	 PMIC_RG_BUCK_VCORE_OC_THD_SHIFT},
	{PMIC_DA_NI_VCORE_VOSEL, PMIC_DA_NI_VCORE_VOSEL_ADDR, PMIC_DA_NI_VCORE_VOSEL_MASK,
	 PMIC_DA_NI_VCORE_VOSEL_SHIFT},
	{PMIC_DA_NI_VCORE_VOSEL_GRAY, PMIC_DA_NI_VCORE_VOSEL_GRAY_ADDR,
	 PMIC_DA_NI_VCORE_VOSEL_GRAY_MASK, PMIC_DA_NI_VCORE_VOSEL_GRAY_SHIFT},
	{PMIC_DA_QI_VCORE_EN, PMIC_DA_QI_VCORE_EN_ADDR, PMIC_DA_QI_VCORE_EN_MASK,
	 PMIC_DA_QI_VCORE_EN_SHIFT},
	{PMIC_DA_VCORE_STB, PMIC_DA_VCORE_STB_ADDR, PMIC_DA_VCORE_STB_MASK,
	 PMIC_DA_VCORE_STB_SHIFT},
	{PMIC_DA_NI_VCORE_VSLEEP_SEL, PMIC_DA_NI_VCORE_VSLEEP_SEL_ADDR,
	 PMIC_DA_NI_VCORE_VSLEEP_SEL_MASK, PMIC_DA_NI_VCORE_VSLEEP_SEL_SHIFT},
	{PMIC_DA_NI_VCORE_R2R_PDN, PMIC_DA_NI_VCORE_R2R_PDN_ADDR, PMIC_DA_NI_VCORE_R2R_PDN_MASK,
	 PMIC_DA_NI_VCORE_R2R_PDN_SHIFT},
	{PMIC_DA_QI_VCORE_DVS_EN, PMIC_DA_QI_VCORE_DVS_EN_ADDR, PMIC_DA_QI_VCORE_DVS_EN_MASK,
	 PMIC_DA_QI_VCORE_DVS_EN_SHIFT},
	{PMIC_RG_BUCK_VCORE_LP_AUTO_EN, PMIC_RG_BUCK_VCORE_LP_AUTO_EN_ADDR,
	 PMIC_RG_BUCK_VCORE_LP_AUTO_EN_MASK, PMIC_RG_BUCK_VCORE_LP_AUTO_EN_SHIFT},
	{PMIC_RG_BUCK_VCORE_LP_DBG_EN, PMIC_RG_BUCK_VCORE_LP_DBG_EN_ADDR,
	 PMIC_RG_BUCK_VCORE_LP_DBG_EN_MASK, PMIC_RG_BUCK_VCORE_LP_DBG_EN_SHIFT},
	{PMIC_RG_BUCK_VCORE_VSLEEP_SEL_DBG, PMIC_RG_BUCK_VCORE_VSLEEP_SEL_DBG_ADDR,
	 PMIC_RG_BUCK_VCORE_VSLEEP_SEL_DBG_MASK, PMIC_RG_BUCK_VCORE_VSLEEP_SEL_DBG_SHIFT},
	{PMIC_RG_BUCK_VCORE_R2R_PDN_DBG, PMIC_RG_BUCK_VCORE_R2R_PDN_DBG_ADDR,
	 PMIC_RG_BUCK_VCORE_R2R_PDN_DBG_MASK, PMIC_RG_BUCK_VCORE_R2R_PDN_DBG_SHIFT},
	{PMIC_RG_BUCK_VCORE_OC_FLAG_CLR_SEL, PMIC_RG_BUCK_VCORE_OC_FLAG_CLR_SEL_ADDR,
	 PMIC_RG_BUCK_VCORE_OC_FLAG_CLR_SEL_MASK, PMIC_RG_BUCK_VCORE_OC_FLAG_CLR_SEL_SHIFT},
	{PMIC_RG_BUCK_VCORE_OSC_SEL_DIS, PMIC_RG_BUCK_VCORE_OSC_SEL_DIS_ADDR,
	 PMIC_RG_BUCK_VCORE_OSC_SEL_DIS_MASK, PMIC_RG_BUCK_VCORE_OSC_SEL_DIS_SHIFT},
	{PMIC_RG_BUCK_VCORE_CK_SW_MODE, PMIC_RG_BUCK_VCORE_CK_SW_MODE_ADDR,
	 PMIC_RG_BUCK_VCORE_CK_SW_MODE_MASK, PMIC_RG_BUCK_VCORE_CK_SW_MODE_SHIFT},
	{PMIC_RG_BUCK_VCORE_CK_SW_EN, PMIC_RG_BUCK_VCORE_CK_SW_EN_ADDR,
	 PMIC_RG_BUCK_VCORE_CK_SW_EN_MASK, PMIC_RG_BUCK_VCORE_CK_SW_EN_SHIFT},
	{PMIC_RG_BUCK_VDRAM_EN, PMIC_RG_BUCK_VDRAM_EN_ADDR, PMIC_RG_BUCK_VDRAM_EN_MASK,
	 PMIC_RG_BUCK_VDRAM_EN_SHIFT},
	{PMIC_RG_BUCK_VDRAM_LP, PMIC_RG_BUCK_VDRAM_LP_ADDR, PMIC_RG_BUCK_VDRAM_LP_MASK,
	 PMIC_RG_BUCK_VDRAM_LP_SHIFT},
	{PMIC_RG_BUCK_VDRAM_VOSEL, PMIC_RG_BUCK_VDRAM_VOSEL_ADDR, PMIC_RG_BUCK_VDRAM_VOSEL_MASK,
	 PMIC_RG_BUCK_VDRAM_VOSEL_SHIFT},
	{PMIC_RG_BUCK_VDRAM_VOSEL_SLEEP, PMIC_RG_BUCK_VDRAM_VOSEL_SLEEP_ADDR,
	 PMIC_RG_BUCK_VDRAM_VOSEL_SLEEP_MASK, PMIC_RG_BUCK_VDRAM_VOSEL_SLEEP_SHIFT},
	{PMIC_RG_BUCK_VDRAM_SFCHG_FRATE, PMIC_RG_BUCK_VDRAM_SFCHG_FRATE_ADDR,
	 PMIC_RG_BUCK_VDRAM_SFCHG_FRATE_MASK, PMIC_RG_BUCK_VDRAM_SFCHG_FRATE_SHIFT},
	{PMIC_RG_BUCK_VDRAM_SFCHG_FEN, PMIC_RG_BUCK_VDRAM_SFCHG_FEN_ADDR,
	 PMIC_RG_BUCK_VDRAM_SFCHG_FEN_MASK, PMIC_RG_BUCK_VDRAM_SFCHG_FEN_SHIFT},
	{PMIC_RG_BUCK_VDRAM_SFCHG_RRATE, PMIC_RG_BUCK_VDRAM_SFCHG_RRATE_ADDR,
	 PMIC_RG_BUCK_VDRAM_SFCHG_RRATE_MASK, PMIC_RG_BUCK_VDRAM_SFCHG_RRATE_SHIFT},
	{PMIC_RG_BUCK_VDRAM_SFCHG_REN, PMIC_RG_BUCK_VDRAM_SFCHG_REN_ADDR,
	 PMIC_RG_BUCK_VDRAM_SFCHG_REN_MASK, PMIC_RG_BUCK_VDRAM_SFCHG_REN_SHIFT},
	{PMIC_RG_BUCK_VDRAM_DVS_TRANS_TD, PMIC_RG_BUCK_VDRAM_DVS_TRANS_TD_ADDR,
	 PMIC_RG_BUCK_VDRAM_DVS_TRANS_TD_MASK, PMIC_RG_BUCK_VDRAM_DVS_TRANS_TD_SHIFT},
	{PMIC_RG_BUCK_VDRAM_DVS_TRANS_CTRL, PMIC_RG_BUCK_VDRAM_DVS_TRANS_CTRL_ADDR,
	 PMIC_RG_BUCK_VDRAM_DVS_TRANS_CTRL_MASK, PMIC_RG_BUCK_VDRAM_DVS_TRANS_CTRL_SHIFT},
	{PMIC_RG_BUCK_VDRAM_DVS_TRANS_ONCE, PMIC_RG_BUCK_VDRAM_DVS_TRANS_ONCE_ADDR,
	 PMIC_RG_BUCK_VDRAM_DVS_TRANS_ONCE_MASK, PMIC_RG_BUCK_VDRAM_DVS_TRANS_ONCE_SHIFT},
	{PMIC_RG_BUCK_VDRAM_SW_OP_EN, PMIC_RG_BUCK_VDRAM_SW_OP_EN_ADDR,
	 PMIC_RG_BUCK_VDRAM_SW_OP_EN_MASK, PMIC_RG_BUCK_VDRAM_SW_OP_EN_SHIFT},
	{PMIC_RG_BUCK_VDRAM_HW0_OP_EN, PMIC_RG_BUCK_VDRAM_HW0_OP_EN_ADDR,
	 PMIC_RG_BUCK_VDRAM_HW0_OP_EN_MASK, PMIC_RG_BUCK_VDRAM_HW0_OP_EN_SHIFT},
	{PMIC_RG_BUCK_VDRAM_HW1_OP_EN, PMIC_RG_BUCK_VDRAM_HW1_OP_EN_ADDR,
	 PMIC_RG_BUCK_VDRAM_HW1_OP_EN_MASK, PMIC_RG_BUCK_VDRAM_HW1_OP_EN_SHIFT},
	{PMIC_RG_BUCK_VDRAM_HW2_OP_EN, PMIC_RG_BUCK_VDRAM_HW2_OP_EN_ADDR,
	 PMIC_RG_BUCK_VDRAM_HW2_OP_EN_MASK, PMIC_RG_BUCK_VDRAM_HW2_OP_EN_SHIFT},
	{PMIC_RG_BUCK_VDRAM_OP_EN_SET, PMIC_RG_BUCK_VDRAM_OP_EN_SET_ADDR,
	 PMIC_RG_BUCK_VDRAM_OP_EN_SET_MASK, PMIC_RG_BUCK_VDRAM_OP_EN_SET_SHIFT},
	{PMIC_RG_BUCK_VDRAM_OP_EN_CLR, PMIC_RG_BUCK_VDRAM_OP_EN_CLR_ADDR,
	 PMIC_RG_BUCK_VDRAM_OP_EN_CLR_MASK, PMIC_RG_BUCK_VDRAM_OP_EN_CLR_SHIFT},
	{PMIC_RG_BUCK_VDRAM_HW0_OP_CFG, PMIC_RG_BUCK_VDRAM_HW0_OP_CFG_ADDR,
	 PMIC_RG_BUCK_VDRAM_HW0_OP_CFG_MASK, PMIC_RG_BUCK_VDRAM_HW0_OP_CFG_SHIFT},
	{PMIC_RG_BUCK_VDRAM_HW1_OP_CFG, PMIC_RG_BUCK_VDRAM_HW1_OP_CFG_ADDR,
	 PMIC_RG_BUCK_VDRAM_HW1_OP_CFG_MASK, PMIC_RG_BUCK_VDRAM_HW1_OP_CFG_SHIFT},
	{PMIC_RG_BUCK_VDRAM_HW2_OP_CFG, PMIC_RG_BUCK_VDRAM_HW2_OP_CFG_ADDR,
	 PMIC_RG_BUCK_VDRAM_HW2_OP_CFG_MASK, PMIC_RG_BUCK_VDRAM_HW2_OP_CFG_SHIFT},
	{PMIC_RG_BUCK_VDRAM_ON_OP, PMIC_RG_BUCK_VDRAM_ON_OP_ADDR, PMIC_RG_BUCK_VDRAM_ON_OP_MASK,
	 PMIC_RG_BUCK_VDRAM_ON_OP_SHIFT},
	{PMIC_RG_BUCK_VDRAM_LP_OP, PMIC_RG_BUCK_VDRAM_LP_OP_ADDR, PMIC_RG_BUCK_VDRAM_LP_OP_MASK,
	 PMIC_RG_BUCK_VDRAM_LP_OP_SHIFT},
	{PMIC_RG_BUCK_VDRAM_OP_CFG_SET, PMIC_RG_BUCK_VDRAM_OP_CFG_SET_ADDR,
	 PMIC_RG_BUCK_VDRAM_OP_CFG_SET_MASK, PMIC_RG_BUCK_VDRAM_OP_CFG_SET_SHIFT},
	{PMIC_RG_BUCK_VDRAM_OP_CFG_CLR, PMIC_RG_BUCK_VDRAM_OP_CFG_CLR_ADDR,
	 PMIC_RG_BUCK_VDRAM_OP_CFG_CLR_MASK, PMIC_RG_BUCK_VDRAM_OP_CFG_CLR_SHIFT},
	{PMIC_RG_BUCK_VDRAM_SP_SW_VOSEL, PMIC_RG_BUCK_VDRAM_SP_SW_VOSEL_ADDR,
	 PMIC_RG_BUCK_VDRAM_SP_SW_VOSEL_MASK, PMIC_RG_BUCK_VDRAM_SP_SW_VOSEL_SHIFT},
	{PMIC_RG_BUCK_VDRAM_SP_SW_VOSEL_EN, PMIC_RG_BUCK_VDRAM_SP_SW_VOSEL_EN_ADDR,
	 PMIC_RG_BUCK_VDRAM_SP_SW_VOSEL_EN_MASK, PMIC_RG_BUCK_VDRAM_SP_SW_VOSEL_EN_SHIFT},
	{PMIC_RG_BUCK_VDRAM_SP_ON_VOSEL_MUX_SEL, PMIC_RG_BUCK_VDRAM_SP_ON_VOSEL_MUX_SEL_ADDR,
	 PMIC_RG_BUCK_VDRAM_SP_ON_VOSEL_MUX_SEL_MASK, PMIC_RG_BUCK_VDRAM_SP_ON_VOSEL_MUX_SEL_SHIFT},
	{PMIC_RG_BUCK_VDRAM_OC_DEG_EN, PMIC_RG_BUCK_VDRAM_OC_DEG_EN_ADDR,
	 PMIC_RG_BUCK_VDRAM_OC_DEG_EN_MASK, PMIC_RG_BUCK_VDRAM_OC_DEG_EN_SHIFT},
	{PMIC_RG_BUCK_VDRAM_OC_WND, PMIC_RG_BUCK_VDRAM_OC_WND_ADDR, PMIC_RG_BUCK_VDRAM_OC_WND_MASK,
	 PMIC_RG_BUCK_VDRAM_OC_WND_SHIFT},
	{PMIC_RG_BUCK_VDRAM_OC_THD, PMIC_RG_BUCK_VDRAM_OC_THD_ADDR, PMIC_RG_BUCK_VDRAM_OC_THD_MASK,
	 PMIC_RG_BUCK_VDRAM_OC_THD_SHIFT},
	{PMIC_DA_NI_VDRAM_VOSEL, PMIC_DA_NI_VDRAM_VOSEL_ADDR, PMIC_DA_NI_VDRAM_VOSEL_MASK,
	 PMIC_DA_NI_VDRAM_VOSEL_SHIFT},
	{PMIC_DA_NI_VDRAM_VOSEL_GRAY, PMIC_DA_NI_VDRAM_VOSEL_GRAY_ADDR,
	 PMIC_DA_NI_VDRAM_VOSEL_GRAY_MASK, PMIC_DA_NI_VDRAM_VOSEL_GRAY_SHIFT},
	{PMIC_DA_QI_VDRAM_EN, PMIC_DA_QI_VDRAM_EN_ADDR, PMIC_DA_QI_VDRAM_EN_MASK,
	 PMIC_DA_QI_VDRAM_EN_SHIFT},
	{PMIC_DA_VDRAM_STB, PMIC_DA_VDRAM_STB_ADDR, PMIC_DA_VDRAM_STB_MASK,
	 PMIC_DA_VDRAM_STB_SHIFT},
	{PMIC_DA_NI_VDRAM_VSLEEP_SEL, PMIC_DA_NI_VDRAM_VSLEEP_SEL_ADDR,
	 PMIC_DA_NI_VDRAM_VSLEEP_SEL_MASK, PMIC_DA_NI_VDRAM_VSLEEP_SEL_SHIFT},
	{PMIC_DA_NI_VDRAM_R2R_PDN, PMIC_DA_NI_VDRAM_R2R_PDN_ADDR, PMIC_DA_NI_VDRAM_R2R_PDN_MASK,
	 PMIC_DA_NI_VDRAM_R2R_PDN_SHIFT},
	{PMIC_DA_QI_VDRAM_DVS_EN, PMIC_DA_QI_VDRAM_DVS_EN_ADDR, PMIC_DA_QI_VDRAM_DVS_EN_MASK,
	 PMIC_DA_QI_VDRAM_DVS_EN_SHIFT},
	{PMIC_RG_BUCK_VDRAM_LP_AUTO_EN, PMIC_RG_BUCK_VDRAM_LP_AUTO_EN_ADDR,
	 PMIC_RG_BUCK_VDRAM_LP_AUTO_EN_MASK, PMIC_RG_BUCK_VDRAM_LP_AUTO_EN_SHIFT},
	{PMIC_RG_BUCK_VDRAM_LP_DBG_EN, PMIC_RG_BUCK_VDRAM_LP_DBG_EN_ADDR,
	 PMIC_RG_BUCK_VDRAM_LP_DBG_EN_MASK, PMIC_RG_BUCK_VDRAM_LP_DBG_EN_SHIFT},
	{PMIC_RG_BUCK_VDRAM_VSLEEP_SEL_DBG, PMIC_RG_BUCK_VDRAM_VSLEEP_SEL_DBG_ADDR,
	 PMIC_RG_BUCK_VDRAM_VSLEEP_SEL_DBG_MASK, PMIC_RG_BUCK_VDRAM_VSLEEP_SEL_DBG_SHIFT},
	{PMIC_RG_BUCK_VDRAM_R2R_PDN_DBG, PMIC_RG_BUCK_VDRAM_R2R_PDN_DBG_ADDR,
	 PMIC_RG_BUCK_VDRAM_R2R_PDN_DBG_MASK, PMIC_RG_BUCK_VDRAM_R2R_PDN_DBG_SHIFT},
	{PMIC_RG_BUCK_VDRAM_OC_FLAG_CLR_SEL, PMIC_RG_BUCK_VDRAM_OC_FLAG_CLR_SEL_ADDR,
	 PMIC_RG_BUCK_VDRAM_OC_FLAG_CLR_SEL_MASK, PMIC_RG_BUCK_VDRAM_OC_FLAG_CLR_SEL_SHIFT},
	{PMIC_RG_BUCK_VDRAM_OSC_SEL_DIS, PMIC_RG_BUCK_VDRAM_OSC_SEL_DIS_ADDR,
	 PMIC_RG_BUCK_VDRAM_OSC_SEL_DIS_MASK, PMIC_RG_BUCK_VDRAM_OSC_SEL_DIS_SHIFT},
	{PMIC_RG_BUCK_VDRAM_CK_SW_MODE, PMIC_RG_BUCK_VDRAM_CK_SW_MODE_ADDR,
	 PMIC_RG_BUCK_VDRAM_CK_SW_MODE_MASK, PMIC_RG_BUCK_VDRAM_CK_SW_MODE_SHIFT},
	{PMIC_RG_BUCK_VDRAM_CK_SW_EN, PMIC_RG_BUCK_VDRAM_CK_SW_EN_ADDR,
	 PMIC_RG_BUCK_VDRAM_CK_SW_EN_MASK, PMIC_RG_BUCK_VDRAM_CK_SW_EN_SHIFT},
	{PMIC_RG_BUCK_VMD1_EN, PMIC_RG_BUCK_VMD1_EN_ADDR, PMIC_RG_BUCK_VMD1_EN_MASK,
	 PMIC_RG_BUCK_VMD1_EN_SHIFT},
	{PMIC_RG_BUCK_VMD1_LP, PMIC_RG_BUCK_VMD1_LP_ADDR, PMIC_RG_BUCK_VMD1_LP_MASK,
	 PMIC_RG_BUCK_VMD1_LP_SHIFT},
	{PMIC_RG_BUCK_VMD1_VOSEL, PMIC_RG_BUCK_VMD1_VOSEL_ADDR, PMIC_RG_BUCK_VMD1_VOSEL_MASK,
	 PMIC_RG_BUCK_VMD1_VOSEL_SHIFT},
	{PMIC_RG_BUCK_VMD1_VOSEL_SLEEP, PMIC_RG_BUCK_VMD1_VOSEL_SLEEP_ADDR,
	 PMIC_RG_BUCK_VMD1_VOSEL_SLEEP_MASK, PMIC_RG_BUCK_VMD1_VOSEL_SLEEP_SHIFT},
	{PMIC_RG_BUCK_VMD1_SFCHG_FRATE, PMIC_RG_BUCK_VMD1_SFCHG_FRATE_ADDR,
	 PMIC_RG_BUCK_VMD1_SFCHG_FRATE_MASK, PMIC_RG_BUCK_VMD1_SFCHG_FRATE_SHIFT},
	{PMIC_RG_BUCK_VMD1_SFCHG_FEN, PMIC_RG_BUCK_VMD1_SFCHG_FEN_ADDR,
	 PMIC_RG_BUCK_VMD1_SFCHG_FEN_MASK, PMIC_RG_BUCK_VMD1_SFCHG_FEN_SHIFT},
	{PMIC_RG_BUCK_VMD1_SFCHG_RRATE, PMIC_RG_BUCK_VMD1_SFCHG_RRATE_ADDR,
	 PMIC_RG_BUCK_VMD1_SFCHG_RRATE_MASK, PMIC_RG_BUCK_VMD1_SFCHG_RRATE_SHIFT},
	{PMIC_RG_BUCK_VMD1_SFCHG_REN, PMIC_RG_BUCK_VMD1_SFCHG_REN_ADDR,
	 PMIC_RG_BUCK_VMD1_SFCHG_REN_MASK, PMIC_RG_BUCK_VMD1_SFCHG_REN_SHIFT},
	{PMIC_RG_BUCK_VMD1_DVS_TRANS_TD, PMIC_RG_BUCK_VMD1_DVS_TRANS_TD_ADDR,
	 PMIC_RG_BUCK_VMD1_DVS_TRANS_TD_MASK, PMIC_RG_BUCK_VMD1_DVS_TRANS_TD_SHIFT},
	{PMIC_RG_BUCK_VMD1_DVS_TRANS_CTRL, PMIC_RG_BUCK_VMD1_DVS_TRANS_CTRL_ADDR,
	 PMIC_RG_BUCK_VMD1_DVS_TRANS_CTRL_MASK, PMIC_RG_BUCK_VMD1_DVS_TRANS_CTRL_SHIFT},
	{PMIC_RG_BUCK_VMD1_DVS_TRANS_ONCE, PMIC_RG_BUCK_VMD1_DVS_TRANS_ONCE_ADDR,
	 PMIC_RG_BUCK_VMD1_DVS_TRANS_ONCE_MASK, PMIC_RG_BUCK_VMD1_DVS_TRANS_ONCE_SHIFT},
	{PMIC_RG_BUCK_VMD1_SW_OP_EN, PMIC_RG_BUCK_VMD1_SW_OP_EN_ADDR,
	 PMIC_RG_BUCK_VMD1_SW_OP_EN_MASK, PMIC_RG_BUCK_VMD1_SW_OP_EN_SHIFT},
	{PMIC_RG_BUCK_VMD1_HW0_OP_EN, PMIC_RG_BUCK_VMD1_HW0_OP_EN_ADDR,
	 PMIC_RG_BUCK_VMD1_HW0_OP_EN_MASK, PMIC_RG_BUCK_VMD1_HW0_OP_EN_SHIFT},
	{PMIC_RG_BUCK_VMD1_HW1_OP_EN, PMIC_RG_BUCK_VMD1_HW1_OP_EN_ADDR,
	 PMIC_RG_BUCK_VMD1_HW1_OP_EN_MASK, PMIC_RG_BUCK_VMD1_HW1_OP_EN_SHIFT},
	{PMIC_RG_BUCK_VMD1_HW2_OP_EN, PMIC_RG_BUCK_VMD1_HW2_OP_EN_ADDR,
	 PMIC_RG_BUCK_VMD1_HW2_OP_EN_MASK, PMIC_RG_BUCK_VMD1_HW2_OP_EN_SHIFT},
	{PMIC_RG_BUCK_VMD1_OP_EN_SET, PMIC_RG_BUCK_VMD1_OP_EN_SET_ADDR,
	 PMIC_RG_BUCK_VMD1_OP_EN_SET_MASK, PMIC_RG_BUCK_VMD1_OP_EN_SET_SHIFT},
	{PMIC_RG_BUCK_VMD1_OP_EN_CLR, PMIC_RG_BUCK_VMD1_OP_EN_CLR_ADDR,
	 PMIC_RG_BUCK_VMD1_OP_EN_CLR_MASK, PMIC_RG_BUCK_VMD1_OP_EN_CLR_SHIFT},
	{PMIC_RG_BUCK_VMD1_HW0_OP_CFG, PMIC_RG_BUCK_VMD1_HW0_OP_CFG_ADDR,
	 PMIC_RG_BUCK_VMD1_HW0_OP_CFG_MASK, PMIC_RG_BUCK_VMD1_HW0_OP_CFG_SHIFT},
	{PMIC_RG_BUCK_VMD1_HW1_OP_CFG, PMIC_RG_BUCK_VMD1_HW1_OP_CFG_ADDR,
	 PMIC_RG_BUCK_VMD1_HW1_OP_CFG_MASK, PMIC_RG_BUCK_VMD1_HW1_OP_CFG_SHIFT},
	{PMIC_RG_BUCK_VMD1_HW2_OP_CFG, PMIC_RG_BUCK_VMD1_HW2_OP_CFG_ADDR,
	 PMIC_RG_BUCK_VMD1_HW2_OP_CFG_MASK, PMIC_RG_BUCK_VMD1_HW2_OP_CFG_SHIFT},
	{PMIC_RG_BUCK_VMD1_ON_OP, PMIC_RG_BUCK_VMD1_ON_OP_ADDR, PMIC_RG_BUCK_VMD1_ON_OP_MASK,
	 PMIC_RG_BUCK_VMD1_ON_OP_SHIFT},
	{PMIC_RG_BUCK_VMD1_LP_OP, PMIC_RG_BUCK_VMD1_LP_OP_ADDR, PMIC_RG_BUCK_VMD1_LP_OP_MASK,
	 PMIC_RG_BUCK_VMD1_LP_OP_SHIFT},
	{PMIC_RG_BUCK_VMD1_OP_CFG_SET, PMIC_RG_BUCK_VMD1_OP_CFG_SET_ADDR,
	 PMIC_RG_BUCK_VMD1_OP_CFG_SET_MASK, PMIC_RG_BUCK_VMD1_OP_CFG_SET_SHIFT},
	{PMIC_RG_BUCK_VMD1_OP_CFG_CLR, PMIC_RG_BUCK_VMD1_OP_CFG_CLR_ADDR,
	 PMIC_RG_BUCK_VMD1_OP_CFG_CLR_MASK, PMIC_RG_BUCK_VMD1_OP_CFG_CLR_SHIFT},
	{PMIC_RG_BUCK_VMD1_SP_SW_VOSEL, PMIC_RG_BUCK_VMD1_SP_SW_VOSEL_ADDR,
	 PMIC_RG_BUCK_VMD1_SP_SW_VOSEL_MASK, PMIC_RG_BUCK_VMD1_SP_SW_VOSEL_SHIFT},
	{PMIC_RG_BUCK_VMD1_SP_SW_VOSEL_EN, PMIC_RG_BUCK_VMD1_SP_SW_VOSEL_EN_ADDR,
	 PMIC_RG_BUCK_VMD1_SP_SW_VOSEL_EN_MASK, PMIC_RG_BUCK_VMD1_SP_SW_VOSEL_EN_SHIFT},
	{PMIC_RG_BUCK_VMD1_SP_ON_VOSEL_MUX_SEL, PMIC_RG_BUCK_VMD1_SP_ON_VOSEL_MUX_SEL_ADDR,
	 PMIC_RG_BUCK_VMD1_SP_ON_VOSEL_MUX_SEL_MASK, PMIC_RG_BUCK_VMD1_SP_ON_VOSEL_MUX_SEL_SHIFT},
	{PMIC_RG_BUCK_VMD1_OC_DEG_EN, PMIC_RG_BUCK_VMD1_OC_DEG_EN_ADDR,
	 PMIC_RG_BUCK_VMD1_OC_DEG_EN_MASK, PMIC_RG_BUCK_VMD1_OC_DEG_EN_SHIFT},
	{PMIC_RG_BUCK_VMD1_OC_WND, PMIC_RG_BUCK_VMD1_OC_WND_ADDR, PMIC_RG_BUCK_VMD1_OC_WND_MASK,
	 PMIC_RG_BUCK_VMD1_OC_WND_SHIFT},
	{PMIC_RG_BUCK_VMD1_OC_THD, PMIC_RG_BUCK_VMD1_OC_THD_ADDR, PMIC_RG_BUCK_VMD1_OC_THD_MASK,
	 PMIC_RG_BUCK_VMD1_OC_THD_SHIFT},
	{PMIC_DA_NI_VMD1_VOSEL, PMIC_DA_NI_VMD1_VOSEL_ADDR, PMIC_DA_NI_VMD1_VOSEL_MASK,
	 PMIC_DA_NI_VMD1_VOSEL_SHIFT},
	{PMIC_DA_NI_VMD1_VOSEL_GRAY, PMIC_DA_NI_VMD1_VOSEL_GRAY_ADDR,
	 PMIC_DA_NI_VMD1_VOSEL_GRAY_MASK, PMIC_DA_NI_VMD1_VOSEL_GRAY_SHIFT},
	{PMIC_DA_QI_VMD1_EN, PMIC_DA_QI_VMD1_EN_ADDR, PMIC_DA_QI_VMD1_EN_MASK,
	 PMIC_DA_QI_VMD1_EN_SHIFT},
	{PMIC_DA_VMD1_STB, PMIC_DA_VMD1_STB_ADDR, PMIC_DA_VMD1_STB_MASK, PMIC_DA_VMD1_STB_SHIFT},
	{PMIC_DA_NI_VMD1_VSLEEP_SEL, PMIC_DA_NI_VMD1_VSLEEP_SEL_ADDR,
	 PMIC_DA_NI_VMD1_VSLEEP_SEL_MASK, PMIC_DA_NI_VMD1_VSLEEP_SEL_SHIFT},
	{PMIC_DA_NI_VMD1_R2R_PDN, PMIC_DA_NI_VMD1_R2R_PDN_ADDR, PMIC_DA_NI_VMD1_R2R_PDN_MASK,
	 PMIC_DA_NI_VMD1_R2R_PDN_SHIFT},
	{PMIC_DA_QI_VMD1_DVS_EN, PMIC_DA_QI_VMD1_DVS_EN_ADDR, PMIC_DA_QI_VMD1_DVS_EN_MASK,
	 PMIC_DA_QI_VMD1_DVS_EN_SHIFT},
	{PMIC_RG_BUCK_VMD1_LP_AUTO_EN, PMIC_RG_BUCK_VMD1_LP_AUTO_EN_ADDR,
	 PMIC_RG_BUCK_VMD1_LP_AUTO_EN_MASK, PMIC_RG_BUCK_VMD1_LP_AUTO_EN_SHIFT},
	{PMIC_RG_BUCK_VMD1_LP_DBG_EN, PMIC_RG_BUCK_VMD1_LP_DBG_EN_ADDR,
	 PMIC_RG_BUCK_VMD1_LP_DBG_EN_MASK, PMIC_RG_BUCK_VMD1_LP_DBG_EN_SHIFT},
	{PMIC_RG_BUCK_VMD1_VSLEEP_SEL_DBG, PMIC_RG_BUCK_VMD1_VSLEEP_SEL_DBG_ADDR,
	 PMIC_RG_BUCK_VMD1_VSLEEP_SEL_DBG_MASK, PMIC_RG_BUCK_VMD1_VSLEEP_SEL_DBG_SHIFT},
	{PMIC_RG_BUCK_VMD1_R2R_PDN_DBG, PMIC_RG_BUCK_VMD1_R2R_PDN_DBG_ADDR,
	 PMIC_RG_BUCK_VMD1_R2R_PDN_DBG_MASK, PMIC_RG_BUCK_VMD1_R2R_PDN_DBG_SHIFT},
	{PMIC_RG_BUCK_VMD1_OC_FLAG_CLR_SEL, PMIC_RG_BUCK_VMD1_OC_FLAG_CLR_SEL_ADDR,
	 PMIC_RG_BUCK_VMD1_OC_FLAG_CLR_SEL_MASK, PMIC_RG_BUCK_VMD1_OC_FLAG_CLR_SEL_SHIFT},
	{PMIC_RG_BUCK_VMD1_OSC_SEL_DIS, PMIC_RG_BUCK_VMD1_OSC_SEL_DIS_ADDR,
	 PMIC_RG_BUCK_VMD1_OSC_SEL_DIS_MASK, PMIC_RG_BUCK_VMD1_OSC_SEL_DIS_SHIFT},
	{PMIC_RG_BUCK_VMD1_CK_SW_MODE, PMIC_RG_BUCK_VMD1_CK_SW_MODE_ADDR,
	 PMIC_RG_BUCK_VMD1_CK_SW_MODE_MASK, PMIC_RG_BUCK_VMD1_CK_SW_MODE_SHIFT},
	{PMIC_RG_BUCK_VMD1_CK_SW_EN, PMIC_RG_BUCK_VMD1_CK_SW_EN_ADDR,
	 PMIC_RG_BUCK_VMD1_CK_SW_EN_MASK, PMIC_RG_BUCK_VMD1_CK_SW_EN_SHIFT},
	{PMIC_RG_BUCK_VMODEM_EN, PMIC_RG_BUCK_VMODEM_EN_ADDR, PMIC_RG_BUCK_VMODEM_EN_MASK,
	 PMIC_RG_BUCK_VMODEM_EN_SHIFT},
	{PMIC_RG_BUCK_VMODEM_LP, PMIC_RG_BUCK_VMODEM_LP_ADDR, PMIC_RG_BUCK_VMODEM_LP_MASK,
	 PMIC_RG_BUCK_VMODEM_LP_SHIFT},
	{PMIC_RG_BUCK_VMODEM_VOSEL, PMIC_RG_BUCK_VMODEM_VOSEL_ADDR, PMIC_RG_BUCK_VMODEM_VOSEL_MASK,
	 PMIC_RG_BUCK_VMODEM_VOSEL_SHIFT},
	{PMIC_RG_BUCK_VMODEM_VOSEL_SLEEP, PMIC_RG_BUCK_VMODEM_VOSEL_SLEEP_ADDR,
	 PMIC_RG_BUCK_VMODEM_VOSEL_SLEEP_MASK, PMIC_RG_BUCK_VMODEM_VOSEL_SLEEP_SHIFT},
	{PMIC_RG_BUCK_VMODEM_SFCHG_FRATE, PMIC_RG_BUCK_VMODEM_SFCHG_FRATE_ADDR,
	 PMIC_RG_BUCK_VMODEM_SFCHG_FRATE_MASK, PMIC_RG_BUCK_VMODEM_SFCHG_FRATE_SHIFT},
	{PMIC_RG_BUCK_VMODEM_SFCHG_FEN, PMIC_RG_BUCK_VMODEM_SFCHG_FEN_ADDR,
	 PMIC_RG_BUCK_VMODEM_SFCHG_FEN_MASK, PMIC_RG_BUCK_VMODEM_SFCHG_FEN_SHIFT},
	{PMIC_RG_BUCK_VMODEM_SFCHG_RRATE, PMIC_RG_BUCK_VMODEM_SFCHG_RRATE_ADDR,
	 PMIC_RG_BUCK_VMODEM_SFCHG_RRATE_MASK, PMIC_RG_BUCK_VMODEM_SFCHG_RRATE_SHIFT},
	{PMIC_RG_BUCK_VMODEM_SFCHG_REN, PMIC_RG_BUCK_VMODEM_SFCHG_REN_ADDR,
	 PMIC_RG_BUCK_VMODEM_SFCHG_REN_MASK, PMIC_RG_BUCK_VMODEM_SFCHG_REN_SHIFT},
	{PMIC_RG_BUCK_VMODEM_DVS_TRANS_TD, PMIC_RG_BUCK_VMODEM_DVS_TRANS_TD_ADDR,
	 PMIC_RG_BUCK_VMODEM_DVS_TRANS_TD_MASK, PMIC_RG_BUCK_VMODEM_DVS_TRANS_TD_SHIFT},
	{PMIC_RG_BUCK_VMODEM_DVS_TRANS_CTRL, PMIC_RG_BUCK_VMODEM_DVS_TRANS_CTRL_ADDR,
	 PMIC_RG_BUCK_VMODEM_DVS_TRANS_CTRL_MASK, PMIC_RG_BUCK_VMODEM_DVS_TRANS_CTRL_SHIFT},
	{PMIC_RG_BUCK_VMODEM_DVS_TRANS_ONCE, PMIC_RG_BUCK_VMODEM_DVS_TRANS_ONCE_ADDR,
	 PMIC_RG_BUCK_VMODEM_DVS_TRANS_ONCE_MASK, PMIC_RG_BUCK_VMODEM_DVS_TRANS_ONCE_SHIFT},
	{PMIC_RG_BUCK_VMODEM_SW_OP_EN, PMIC_RG_BUCK_VMODEM_SW_OP_EN_ADDR,
	 PMIC_RG_BUCK_VMODEM_SW_OP_EN_MASK, PMIC_RG_BUCK_VMODEM_SW_OP_EN_SHIFT},
	{PMIC_RG_BUCK_VMODEM_HW0_OP_EN, PMIC_RG_BUCK_VMODEM_HW0_OP_EN_ADDR,
	 PMIC_RG_BUCK_VMODEM_HW0_OP_EN_MASK, PMIC_RG_BUCK_VMODEM_HW0_OP_EN_SHIFT},
	{PMIC_RG_BUCK_VMODEM_HW1_OP_EN, PMIC_RG_BUCK_VMODEM_HW1_OP_EN_ADDR,
	 PMIC_RG_BUCK_VMODEM_HW1_OP_EN_MASK, PMIC_RG_BUCK_VMODEM_HW1_OP_EN_SHIFT},
	{PMIC_RG_BUCK_VMODEM_HW2_OP_EN, PMIC_RG_BUCK_VMODEM_HW2_OP_EN_ADDR,
	 PMIC_RG_BUCK_VMODEM_HW2_OP_EN_MASK, PMIC_RG_BUCK_VMODEM_HW2_OP_EN_SHIFT},
	{PMIC_RG_BUCK_VMODEM_OP_EN_SET, PMIC_RG_BUCK_VMODEM_OP_EN_SET_ADDR,
	 PMIC_RG_BUCK_VMODEM_OP_EN_SET_MASK, PMIC_RG_BUCK_VMODEM_OP_EN_SET_SHIFT},
	{PMIC_RG_BUCK_VMODEM_OP_EN_CLR, PMIC_RG_BUCK_VMODEM_OP_EN_CLR_ADDR,
	 PMIC_RG_BUCK_VMODEM_OP_EN_CLR_MASK, PMIC_RG_BUCK_VMODEM_OP_EN_CLR_SHIFT},
	{PMIC_RG_BUCK_VMODEM_HW0_OP_CFG, PMIC_RG_BUCK_VMODEM_HW0_OP_CFG_ADDR,
	 PMIC_RG_BUCK_VMODEM_HW0_OP_CFG_MASK, PMIC_RG_BUCK_VMODEM_HW0_OP_CFG_SHIFT},
	{PMIC_RG_BUCK_VMODEM_HW1_OP_CFG, PMIC_RG_BUCK_VMODEM_HW1_OP_CFG_ADDR,
	 PMIC_RG_BUCK_VMODEM_HW1_OP_CFG_MASK, PMIC_RG_BUCK_VMODEM_HW1_OP_CFG_SHIFT},
	{PMIC_RG_BUCK_VMODEM_HW2_OP_CFG, PMIC_RG_BUCK_VMODEM_HW2_OP_CFG_ADDR,
	 PMIC_RG_BUCK_VMODEM_HW2_OP_CFG_MASK, PMIC_RG_BUCK_VMODEM_HW2_OP_CFG_SHIFT},
	{PMIC_RG_BUCK_VMODEM_ON_OP, PMIC_RG_BUCK_VMODEM_ON_OP_ADDR, PMIC_RG_BUCK_VMODEM_ON_OP_MASK,
	 PMIC_RG_BUCK_VMODEM_ON_OP_SHIFT},
	{PMIC_RG_BUCK_VMODEM_LP_OP, PMIC_RG_BUCK_VMODEM_LP_OP_ADDR, PMIC_RG_BUCK_VMODEM_LP_OP_MASK,
	 PMIC_RG_BUCK_VMODEM_LP_OP_SHIFT},
	{PMIC_RG_BUCK_VMODEM_OP_CFG_SET, PMIC_RG_BUCK_VMODEM_OP_CFG_SET_ADDR,
	 PMIC_RG_BUCK_VMODEM_OP_CFG_SET_MASK, PMIC_RG_BUCK_VMODEM_OP_CFG_SET_SHIFT},
	{PMIC_RG_BUCK_VMODEM_OP_CFG_CLR, PMIC_RG_BUCK_VMODEM_OP_CFG_CLR_ADDR,
	 PMIC_RG_BUCK_VMODEM_OP_CFG_CLR_MASK, PMIC_RG_BUCK_VMODEM_OP_CFG_CLR_SHIFT},
	{PMIC_RG_BUCK_VMODEM_SP_SW_VOSEL, PMIC_RG_BUCK_VMODEM_SP_SW_VOSEL_ADDR,
	 PMIC_RG_BUCK_VMODEM_SP_SW_VOSEL_MASK, PMIC_RG_BUCK_VMODEM_SP_SW_VOSEL_SHIFT},
	{PMIC_RG_BUCK_VMODEM_SP_SW_VOSEL_EN, PMIC_RG_BUCK_VMODEM_SP_SW_VOSEL_EN_ADDR,
	 PMIC_RG_BUCK_VMODEM_SP_SW_VOSEL_EN_MASK, PMIC_RG_BUCK_VMODEM_SP_SW_VOSEL_EN_SHIFT},
	{PMIC_RG_BUCK_VMODEM_SP_ON_VOSEL_MUX_SEL, PMIC_RG_BUCK_VMODEM_SP_ON_VOSEL_MUX_SEL_ADDR,
	 PMIC_RG_BUCK_VMODEM_SP_ON_VOSEL_MUX_SEL_MASK,
	 PMIC_RG_BUCK_VMODEM_SP_ON_VOSEL_MUX_SEL_SHIFT},
	{PMIC_RG_BUCK_VMODEM_OC_DEG_EN, PMIC_RG_BUCK_VMODEM_OC_DEG_EN_ADDR,
	 PMIC_RG_BUCK_VMODEM_OC_DEG_EN_MASK, PMIC_RG_BUCK_VMODEM_OC_DEG_EN_SHIFT},
	{PMIC_RG_BUCK_VMODEM_OC_WND, PMIC_RG_BUCK_VMODEM_OC_WND_ADDR,
	 PMIC_RG_BUCK_VMODEM_OC_WND_MASK, PMIC_RG_BUCK_VMODEM_OC_WND_SHIFT},
	{PMIC_RG_BUCK_VMODEM_OC_THD, PMIC_RG_BUCK_VMODEM_OC_THD_ADDR,
	 PMIC_RG_BUCK_VMODEM_OC_THD_MASK, PMIC_RG_BUCK_VMODEM_OC_THD_SHIFT},
	{PMIC_DA_NI_VMODEM_VOSEL, PMIC_DA_NI_VMODEM_VOSEL_ADDR, PMIC_DA_NI_VMODEM_VOSEL_MASK,
	 PMIC_DA_NI_VMODEM_VOSEL_SHIFT},
	{PMIC_DA_NI_VMODEM_VOSEL_GRAY, PMIC_DA_NI_VMODEM_VOSEL_GRAY_ADDR,
	 PMIC_DA_NI_VMODEM_VOSEL_GRAY_MASK, PMIC_DA_NI_VMODEM_VOSEL_GRAY_SHIFT},
	{PMIC_DA_QI_VMODEM_EN, PMIC_DA_QI_VMODEM_EN_ADDR, PMIC_DA_QI_VMODEM_EN_MASK,
	 PMIC_DA_QI_VMODEM_EN_SHIFT},
	{PMIC_DA_VMODEM_STB, PMIC_DA_VMODEM_STB_ADDR, PMIC_DA_VMODEM_STB_MASK,
	 PMIC_DA_VMODEM_STB_SHIFT},
	{PMIC_DA_NI_VMODEM_VSLEEP_SEL, PMIC_DA_NI_VMODEM_VSLEEP_SEL_ADDR,
	 PMIC_DA_NI_VMODEM_VSLEEP_SEL_MASK, PMIC_DA_NI_VMODEM_VSLEEP_SEL_SHIFT},
	{PMIC_DA_NI_VMODEM_R2R_PDN, PMIC_DA_NI_VMODEM_R2R_PDN_ADDR, PMIC_DA_NI_VMODEM_R2R_PDN_MASK,
	 PMIC_DA_NI_VMODEM_R2R_PDN_SHIFT},
	{PMIC_DA_QI_VMODEM_DVS_EN, PMIC_DA_QI_VMODEM_DVS_EN_ADDR, PMIC_DA_QI_VMODEM_DVS_EN_MASK,
	 PMIC_DA_QI_VMODEM_DVS_EN_SHIFT},
	{PMIC_RG_BUCK_VMODEM_LP_AUTO_EN, PMIC_RG_BUCK_VMODEM_LP_AUTO_EN_ADDR,
	 PMIC_RG_BUCK_VMODEM_LP_AUTO_EN_MASK, PMIC_RG_BUCK_VMODEM_LP_AUTO_EN_SHIFT},
	{PMIC_RG_BUCK_VMODEM_LP_DBG_EN, PMIC_RG_BUCK_VMODEM_LP_DBG_EN_ADDR,
	 PMIC_RG_BUCK_VMODEM_LP_DBG_EN_MASK, PMIC_RG_BUCK_VMODEM_LP_DBG_EN_SHIFT},
	{PMIC_RG_BUCK_VMODEM_VSLEEP_SEL_DBG, PMIC_RG_BUCK_VMODEM_VSLEEP_SEL_DBG_ADDR,
	 PMIC_RG_BUCK_VMODEM_VSLEEP_SEL_DBG_MASK, PMIC_RG_BUCK_VMODEM_VSLEEP_SEL_DBG_SHIFT},
	{PMIC_RG_BUCK_VMODEM_R2R_PDN_DBG, PMIC_RG_BUCK_VMODEM_R2R_PDN_DBG_ADDR,
	 PMIC_RG_BUCK_VMODEM_R2R_PDN_DBG_MASK, PMIC_RG_BUCK_VMODEM_R2R_PDN_DBG_SHIFT},
	{PMIC_RG_BUCK_VMODEM_OC_FLAG_CLR_SEL, PMIC_RG_BUCK_VMODEM_OC_FLAG_CLR_SEL_ADDR,
	 PMIC_RG_BUCK_VMODEM_OC_FLAG_CLR_SEL_MASK, PMIC_RG_BUCK_VMODEM_OC_FLAG_CLR_SEL_SHIFT},
	{PMIC_RG_BUCK_VMODEM_OSC_SEL_DIS, PMIC_RG_BUCK_VMODEM_OSC_SEL_DIS_ADDR,
	 PMIC_RG_BUCK_VMODEM_OSC_SEL_DIS_MASK, PMIC_RG_BUCK_VMODEM_OSC_SEL_DIS_SHIFT},
	{PMIC_RG_BUCK_VMODEM_CK_SW_MODE, PMIC_RG_BUCK_VMODEM_CK_SW_MODE_ADDR,
	 PMIC_RG_BUCK_VMODEM_CK_SW_MODE_MASK, PMIC_RG_BUCK_VMODEM_CK_SW_MODE_SHIFT},
	{PMIC_RG_BUCK_VMODEM_CK_SW_EN, PMIC_RG_BUCK_VMODEM_CK_SW_EN_ADDR,
	 PMIC_RG_BUCK_VMODEM_CK_SW_EN_MASK, PMIC_RG_BUCK_VMODEM_CK_SW_EN_SHIFT},
	{PMIC_RG_BUCK_VS1_EN, PMIC_RG_BUCK_VS1_EN_ADDR, PMIC_RG_BUCK_VS1_EN_MASK,
	 PMIC_RG_BUCK_VS1_EN_SHIFT},
	{PMIC_RG_BUCK_VS1_LP, PMIC_RG_BUCK_VS1_LP_ADDR, PMIC_RG_BUCK_VS1_LP_MASK,
	 PMIC_RG_BUCK_VS1_LP_SHIFT},
	{PMIC_RG_BUCK_VS1_VOSEL, PMIC_RG_BUCK_VS1_VOSEL_ADDR, PMIC_RG_BUCK_VS1_VOSEL_MASK,
	 PMIC_RG_BUCK_VS1_VOSEL_SHIFT},
	{PMIC_RG_BUCK_VS1_VOSEL_SLEEP, PMIC_RG_BUCK_VS1_VOSEL_SLEEP_ADDR,
	 PMIC_RG_BUCK_VS1_VOSEL_SLEEP_MASK, PMIC_RG_BUCK_VS1_VOSEL_SLEEP_SHIFT},
	{PMIC_RG_BUCK_VS1_SFCHG_FRATE, PMIC_RG_BUCK_VS1_SFCHG_FRATE_ADDR,
	 PMIC_RG_BUCK_VS1_SFCHG_FRATE_MASK, PMIC_RG_BUCK_VS1_SFCHG_FRATE_SHIFT},
	{PMIC_RG_BUCK_VS1_SFCHG_FEN, PMIC_RG_BUCK_VS1_SFCHG_FEN_ADDR,
	 PMIC_RG_BUCK_VS1_SFCHG_FEN_MASK, PMIC_RG_BUCK_VS1_SFCHG_FEN_SHIFT},
	{PMIC_RG_BUCK_VS1_SFCHG_RRATE, PMIC_RG_BUCK_VS1_SFCHG_RRATE_ADDR,
	 PMIC_RG_BUCK_VS1_SFCHG_RRATE_MASK, PMIC_RG_BUCK_VS1_SFCHG_RRATE_SHIFT},
	{PMIC_RG_BUCK_VS1_SFCHG_REN, PMIC_RG_BUCK_VS1_SFCHG_REN_ADDR,
	 PMIC_RG_BUCK_VS1_SFCHG_REN_MASK, PMIC_RG_BUCK_VS1_SFCHG_REN_SHIFT},
	{PMIC_RG_BUCK_VS1_DVS_TRANS_TD, PMIC_RG_BUCK_VS1_DVS_TRANS_TD_ADDR,
	 PMIC_RG_BUCK_VS1_DVS_TRANS_TD_MASK, PMIC_RG_BUCK_VS1_DVS_TRANS_TD_SHIFT},
	{PMIC_RG_BUCK_VS1_DVS_TRANS_CTRL, PMIC_RG_BUCK_VS1_DVS_TRANS_CTRL_ADDR,
	 PMIC_RG_BUCK_VS1_DVS_TRANS_CTRL_MASK, PMIC_RG_BUCK_VS1_DVS_TRANS_CTRL_SHIFT},
	{PMIC_RG_BUCK_VS1_DVS_TRANS_ONCE, PMIC_RG_BUCK_VS1_DVS_TRANS_ONCE_ADDR,
	 PMIC_RG_BUCK_VS1_DVS_TRANS_ONCE_MASK, PMIC_RG_BUCK_VS1_DVS_TRANS_ONCE_SHIFT},
	{PMIC_RG_BUCK_VS1_SW_OP_EN, PMIC_RG_BUCK_VS1_SW_OP_EN_ADDR, PMIC_RG_BUCK_VS1_SW_OP_EN_MASK,
	 PMIC_RG_BUCK_VS1_SW_OP_EN_SHIFT},
	{PMIC_RG_BUCK_VS1_HW0_OP_EN, PMIC_RG_BUCK_VS1_HW0_OP_EN_ADDR,
	 PMIC_RG_BUCK_VS1_HW0_OP_EN_MASK, PMIC_RG_BUCK_VS1_HW0_OP_EN_SHIFT},
	{PMIC_RG_BUCK_VS1_HW1_OP_EN, PMIC_RG_BUCK_VS1_HW1_OP_EN_ADDR,
	 PMIC_RG_BUCK_VS1_HW1_OP_EN_MASK, PMIC_RG_BUCK_VS1_HW1_OP_EN_SHIFT},
	{PMIC_RG_BUCK_VS1_HW2_OP_EN, PMIC_RG_BUCK_VS1_HW2_OP_EN_ADDR,
	 PMIC_RG_BUCK_VS1_HW2_OP_EN_MASK, PMIC_RG_BUCK_VS1_HW2_OP_EN_SHIFT},
	{PMIC_RG_BUCK_VS1_OP_EN_SET, PMIC_RG_BUCK_VS1_OP_EN_SET_ADDR,
	 PMIC_RG_BUCK_VS1_OP_EN_SET_MASK, PMIC_RG_BUCK_VS1_OP_EN_SET_SHIFT},
	{PMIC_RG_BUCK_VS1_OP_EN_CLR, PMIC_RG_BUCK_VS1_OP_EN_CLR_ADDR,
	 PMIC_RG_BUCK_VS1_OP_EN_CLR_MASK, PMIC_RG_BUCK_VS1_OP_EN_CLR_SHIFT},
	{PMIC_RG_BUCK_VS1_HW0_OP_CFG, PMIC_RG_BUCK_VS1_HW0_OP_CFG_ADDR,
	 PMIC_RG_BUCK_VS1_HW0_OP_CFG_MASK, PMIC_RG_BUCK_VS1_HW0_OP_CFG_SHIFT},
	{PMIC_RG_BUCK_VS1_HW1_OP_CFG, PMIC_RG_BUCK_VS1_HW1_OP_CFG_ADDR,
	 PMIC_RG_BUCK_VS1_HW1_OP_CFG_MASK, PMIC_RG_BUCK_VS1_HW1_OP_CFG_SHIFT},
	{PMIC_RG_BUCK_VS1_HW2_OP_CFG, PMIC_RG_BUCK_VS1_HW2_OP_CFG_ADDR,
	 PMIC_RG_BUCK_VS1_HW2_OP_CFG_MASK, PMIC_RG_BUCK_VS1_HW2_OP_CFG_SHIFT},
	{PMIC_RG_BUCK_VS1_ON_OP, PMIC_RG_BUCK_VS1_ON_OP_ADDR, PMIC_RG_BUCK_VS1_ON_OP_MASK,
	 PMIC_RG_BUCK_VS1_ON_OP_SHIFT},
	{PMIC_RG_BUCK_VS1_LP_OP, PMIC_RG_BUCK_VS1_LP_OP_ADDR, PMIC_RG_BUCK_VS1_LP_OP_MASK,
	 PMIC_RG_BUCK_VS1_LP_OP_SHIFT},
	{PMIC_RG_BUCK_VS1_OP_CFG_SET, PMIC_RG_BUCK_VS1_OP_CFG_SET_ADDR,
	 PMIC_RG_BUCK_VS1_OP_CFG_SET_MASK, PMIC_RG_BUCK_VS1_OP_CFG_SET_SHIFT},
	{PMIC_RG_BUCK_VS1_OP_CFG_CLR, PMIC_RG_BUCK_VS1_OP_CFG_CLR_ADDR,
	 PMIC_RG_BUCK_VS1_OP_CFG_CLR_MASK, PMIC_RG_BUCK_VS1_OP_CFG_CLR_SHIFT},
	{PMIC_RG_BUCK_VS1_SP_SW_VOSEL, PMIC_RG_BUCK_VS1_SP_SW_VOSEL_ADDR,
	 PMIC_RG_BUCK_VS1_SP_SW_VOSEL_MASK, PMIC_RG_BUCK_VS1_SP_SW_VOSEL_SHIFT},
	{PMIC_RG_BUCK_VS1_SP_SW_VOSEL_EN, PMIC_RG_BUCK_VS1_SP_SW_VOSEL_EN_ADDR,
	 PMIC_RG_BUCK_VS1_SP_SW_VOSEL_EN_MASK, PMIC_RG_BUCK_VS1_SP_SW_VOSEL_EN_SHIFT},
	{PMIC_RG_BUCK_VS1_SP_ON_VOSEL_MUX_SEL, PMIC_RG_BUCK_VS1_SP_ON_VOSEL_MUX_SEL_ADDR,
	 PMIC_RG_BUCK_VS1_SP_ON_VOSEL_MUX_SEL_MASK, PMIC_RG_BUCK_VS1_SP_ON_VOSEL_MUX_SEL_SHIFT},
	{PMIC_RG_BUCK_VS1_OC_DEG_EN, PMIC_RG_BUCK_VS1_OC_DEG_EN_ADDR,
	 PMIC_RG_BUCK_VS1_OC_DEG_EN_MASK, PMIC_RG_BUCK_VS1_OC_DEG_EN_SHIFT},
	{PMIC_RG_BUCK_VS1_OC_WND, PMIC_RG_BUCK_VS1_OC_WND_ADDR, PMIC_RG_BUCK_VS1_OC_WND_MASK,
	 PMIC_RG_BUCK_VS1_OC_WND_SHIFT},
	{PMIC_RG_BUCK_VS1_OC_THD, PMIC_RG_BUCK_VS1_OC_THD_ADDR, PMIC_RG_BUCK_VS1_OC_THD_MASK,
	 PMIC_RG_BUCK_VS1_OC_THD_SHIFT},
	{PMIC_DA_NI_VS1_VOSEL, PMIC_DA_NI_VS1_VOSEL_ADDR, PMIC_DA_NI_VS1_VOSEL_MASK,
	 PMIC_DA_NI_VS1_VOSEL_SHIFT},
	{PMIC_DA_NI_VS1_VOSEL_GRAY, PMIC_DA_NI_VS1_VOSEL_GRAY_ADDR, PMIC_DA_NI_VS1_VOSEL_GRAY_MASK,
	 PMIC_DA_NI_VS1_VOSEL_GRAY_SHIFT},
	{PMIC_DA_QI_VS1_EN, PMIC_DA_QI_VS1_EN_ADDR, PMIC_DA_QI_VS1_EN_MASK,
	 PMIC_DA_QI_VS1_EN_SHIFT},
	{PMIC_DA_VS1_STB, PMIC_DA_VS1_STB_ADDR, PMIC_DA_VS1_STB_MASK, PMIC_DA_VS1_STB_SHIFT},
	{PMIC_DA_NI_VS1_VSLEEP_SEL, PMIC_DA_NI_VS1_VSLEEP_SEL_ADDR, PMIC_DA_NI_VS1_VSLEEP_SEL_MASK,
	 PMIC_DA_NI_VS1_VSLEEP_SEL_SHIFT},
	{PMIC_DA_NI_VS1_R2R_PDN, PMIC_DA_NI_VS1_R2R_PDN_ADDR, PMIC_DA_NI_VS1_R2R_PDN_MASK,
	 PMIC_DA_NI_VS1_R2R_PDN_SHIFT},
	{PMIC_DA_QI_VS1_DVS_EN, PMIC_DA_QI_VS1_DVS_EN_ADDR, PMIC_DA_QI_VS1_DVS_EN_MASK,
	 PMIC_DA_QI_VS1_DVS_EN_SHIFT},
	{PMIC_RG_BUCK_VS1_LP_AUTO_EN, PMIC_RG_BUCK_VS1_LP_AUTO_EN_ADDR,
	 PMIC_RG_BUCK_VS1_LP_AUTO_EN_MASK, PMIC_RG_BUCK_VS1_LP_AUTO_EN_SHIFT},
	{PMIC_RG_BUCK_VS1_LP_DBG_EN, PMIC_RG_BUCK_VS1_LP_DBG_EN_ADDR,
	 PMIC_RG_BUCK_VS1_LP_DBG_EN_MASK, PMIC_RG_BUCK_VS1_LP_DBG_EN_SHIFT},
	{PMIC_RG_BUCK_VS1_VSLEEP_SEL_DBG, PMIC_RG_BUCK_VS1_VSLEEP_SEL_DBG_ADDR,
	 PMIC_RG_BUCK_VS1_VSLEEP_SEL_DBG_MASK, PMIC_RG_BUCK_VS1_VSLEEP_SEL_DBG_SHIFT},
	{PMIC_RG_BUCK_VS1_R2R_PDN_DBG, PMIC_RG_BUCK_VS1_R2R_PDN_DBG_ADDR,
	 PMIC_RG_BUCK_VS1_R2R_PDN_DBG_MASK, PMIC_RG_BUCK_VS1_R2R_PDN_DBG_SHIFT},
	{PMIC_RG_BUCK_VS1_OC_FLAG_CLR_SEL, PMIC_RG_BUCK_VS1_OC_FLAG_CLR_SEL_ADDR,
	 PMIC_RG_BUCK_VS1_OC_FLAG_CLR_SEL_MASK, PMIC_RG_BUCK_VS1_OC_FLAG_CLR_SEL_SHIFT},
	{PMIC_RG_BUCK_VS1_OSC_SEL_DIS, PMIC_RG_BUCK_VS1_OSC_SEL_DIS_ADDR,
	 PMIC_RG_BUCK_VS1_OSC_SEL_DIS_MASK, PMIC_RG_BUCK_VS1_OSC_SEL_DIS_SHIFT},
	{PMIC_RG_BUCK_VS1_CK_SW_MODE, PMIC_RG_BUCK_VS1_CK_SW_MODE_ADDR,
	 PMIC_RG_BUCK_VS1_CK_SW_MODE_MASK, PMIC_RG_BUCK_VS1_CK_SW_MODE_SHIFT},
	{PMIC_RG_BUCK_VS1_CK_SW_EN, PMIC_RG_BUCK_VS1_CK_SW_EN_ADDR, PMIC_RG_BUCK_VS1_CK_SW_EN_MASK,
	 PMIC_RG_BUCK_VS1_CK_SW_EN_SHIFT},
	{PMIC_RG_BUCK_VS2_EN, PMIC_RG_BUCK_VS2_EN_ADDR, PMIC_RG_BUCK_VS2_EN_MASK,
	 PMIC_RG_BUCK_VS2_EN_SHIFT},
	{PMIC_RG_BUCK_VS2_LP, PMIC_RG_BUCK_VS2_LP_ADDR, PMIC_RG_BUCK_VS2_LP_MASK,
	 PMIC_RG_BUCK_VS2_LP_SHIFT},
	{PMIC_RG_BUCK_VS2_VOSEL, PMIC_RG_BUCK_VS2_VOSEL_ADDR, PMIC_RG_BUCK_VS2_VOSEL_MASK,
	 PMIC_RG_BUCK_VS2_VOSEL_SHIFT},
	{PMIC_RG_BUCK_VS2_VOSEL_SLEEP, PMIC_RG_BUCK_VS2_VOSEL_SLEEP_ADDR,
	 PMIC_RG_BUCK_VS2_VOSEL_SLEEP_MASK, PMIC_RG_BUCK_VS2_VOSEL_SLEEP_SHIFT},
	{PMIC_RG_BUCK_VS2_SFCHG_FRATE, PMIC_RG_BUCK_VS2_SFCHG_FRATE_ADDR,
	 PMIC_RG_BUCK_VS2_SFCHG_FRATE_MASK, PMIC_RG_BUCK_VS2_SFCHG_FRATE_SHIFT},
	{PMIC_RG_BUCK_VS2_SFCHG_FEN, PMIC_RG_BUCK_VS2_SFCHG_FEN_ADDR,
	 PMIC_RG_BUCK_VS2_SFCHG_FEN_MASK, PMIC_RG_BUCK_VS2_SFCHG_FEN_SHIFT},
	{PMIC_RG_BUCK_VS2_SFCHG_RRATE, PMIC_RG_BUCK_VS2_SFCHG_RRATE_ADDR,
	 PMIC_RG_BUCK_VS2_SFCHG_RRATE_MASK, PMIC_RG_BUCK_VS2_SFCHG_RRATE_SHIFT},
	{PMIC_RG_BUCK_VS2_SFCHG_REN, PMIC_RG_BUCK_VS2_SFCHG_REN_ADDR,
	 PMIC_RG_BUCK_VS2_SFCHG_REN_MASK, PMIC_RG_BUCK_VS2_SFCHG_REN_SHIFT},
	{PMIC_RG_BUCK_VS2_DVS_TRANS_TD, PMIC_RG_BUCK_VS2_DVS_TRANS_TD_ADDR,
	 PMIC_RG_BUCK_VS2_DVS_TRANS_TD_MASK, PMIC_RG_BUCK_VS2_DVS_TRANS_TD_SHIFT},
	{PMIC_RG_BUCK_VS2_DVS_TRANS_CTRL, PMIC_RG_BUCK_VS2_DVS_TRANS_CTRL_ADDR,
	 PMIC_RG_BUCK_VS2_DVS_TRANS_CTRL_MASK, PMIC_RG_BUCK_VS2_DVS_TRANS_CTRL_SHIFT},
	{PMIC_RG_BUCK_VS2_DVS_TRANS_ONCE, PMIC_RG_BUCK_VS2_DVS_TRANS_ONCE_ADDR,
	 PMIC_RG_BUCK_VS2_DVS_TRANS_ONCE_MASK, PMIC_RG_BUCK_VS2_DVS_TRANS_ONCE_SHIFT},
	{PMIC_RG_BUCK_VS2_SW_OP_EN, PMIC_RG_BUCK_VS2_SW_OP_EN_ADDR, PMIC_RG_BUCK_VS2_SW_OP_EN_MASK,
	 PMIC_RG_BUCK_VS2_SW_OP_EN_SHIFT},
	{PMIC_RG_BUCK_VS2_HW0_OP_EN, PMIC_RG_BUCK_VS2_HW0_OP_EN_ADDR,
	 PMIC_RG_BUCK_VS2_HW0_OP_EN_MASK, PMIC_RG_BUCK_VS2_HW0_OP_EN_SHIFT},
	{PMIC_RG_BUCK_VS2_HW1_OP_EN, PMIC_RG_BUCK_VS2_HW1_OP_EN_ADDR,
	 PMIC_RG_BUCK_VS2_HW1_OP_EN_MASK, PMIC_RG_BUCK_VS2_HW1_OP_EN_SHIFT},
	{PMIC_RG_BUCK_VS2_HW2_OP_EN, PMIC_RG_BUCK_VS2_HW2_OP_EN_ADDR,
	 PMIC_RG_BUCK_VS2_HW2_OP_EN_MASK, PMIC_RG_BUCK_VS2_HW2_OP_EN_SHIFT},
	{PMIC_RG_BUCK_VS2_OP_EN_SET, PMIC_RG_BUCK_VS2_OP_EN_SET_ADDR,
	 PMIC_RG_BUCK_VS2_OP_EN_SET_MASK, PMIC_RG_BUCK_VS2_OP_EN_SET_SHIFT},
	{PMIC_RG_BUCK_VS2_OP_EN_CLR, PMIC_RG_BUCK_VS2_OP_EN_CLR_ADDR,
	 PMIC_RG_BUCK_VS2_OP_EN_CLR_MASK, PMIC_RG_BUCK_VS2_OP_EN_CLR_SHIFT},
	{PMIC_RG_BUCK_VS2_HW0_OP_CFG, PMIC_RG_BUCK_VS2_HW0_OP_CFG_ADDR,
	 PMIC_RG_BUCK_VS2_HW0_OP_CFG_MASK, PMIC_RG_BUCK_VS2_HW0_OP_CFG_SHIFT},
	{PMIC_RG_BUCK_VS2_HW1_OP_CFG, PMIC_RG_BUCK_VS2_HW1_OP_CFG_ADDR,
	 PMIC_RG_BUCK_VS2_HW1_OP_CFG_MASK, PMIC_RG_BUCK_VS2_HW1_OP_CFG_SHIFT},
	{PMIC_RG_BUCK_VS2_HW2_OP_CFG, PMIC_RG_BUCK_VS2_HW2_OP_CFG_ADDR,
	 PMIC_RG_BUCK_VS2_HW2_OP_CFG_MASK, PMIC_RG_BUCK_VS2_HW2_OP_CFG_SHIFT},
	{PMIC_RG_BUCK_VS2_ON_OP, PMIC_RG_BUCK_VS2_ON_OP_ADDR, PMIC_RG_BUCK_VS2_ON_OP_MASK,
	 PMIC_RG_BUCK_VS2_ON_OP_SHIFT},
	{PMIC_RG_BUCK_VS2_LP_OP, PMIC_RG_BUCK_VS2_LP_OP_ADDR, PMIC_RG_BUCK_VS2_LP_OP_MASK,
	 PMIC_RG_BUCK_VS2_LP_OP_SHIFT},
	{PMIC_RG_BUCK_VS2_OP_CFG_SET, PMIC_RG_BUCK_VS2_OP_CFG_SET_ADDR,
	 PMIC_RG_BUCK_VS2_OP_CFG_SET_MASK, PMIC_RG_BUCK_VS2_OP_CFG_SET_SHIFT},
	{PMIC_RG_BUCK_VS2_OP_CFG_CLR, PMIC_RG_BUCK_VS2_OP_CFG_CLR_ADDR,
	 PMIC_RG_BUCK_VS2_OP_CFG_CLR_MASK, PMIC_RG_BUCK_VS2_OP_CFG_CLR_SHIFT},
	{PMIC_RG_BUCK_VS2_SP_SW_VOSEL, PMIC_RG_BUCK_VS2_SP_SW_VOSEL_ADDR,
	 PMIC_RG_BUCK_VS2_SP_SW_VOSEL_MASK, PMIC_RG_BUCK_VS2_SP_SW_VOSEL_SHIFT},
	{PMIC_RG_BUCK_VS2_SP_SW_VOSEL_EN, PMIC_RG_BUCK_VS2_SP_SW_VOSEL_EN_ADDR,
	 PMIC_RG_BUCK_VS2_SP_SW_VOSEL_EN_MASK, PMIC_RG_BUCK_VS2_SP_SW_VOSEL_EN_SHIFT},
	{PMIC_RG_BUCK_VS2_SP_ON_VOSEL_MUX_SEL, PMIC_RG_BUCK_VS2_SP_ON_VOSEL_MUX_SEL_ADDR,
	 PMIC_RG_BUCK_VS2_SP_ON_VOSEL_MUX_SEL_MASK, PMIC_RG_BUCK_VS2_SP_ON_VOSEL_MUX_SEL_SHIFT},
	{PMIC_RG_BUCK_VS2_OC_DEG_EN, PMIC_RG_BUCK_VS2_OC_DEG_EN_ADDR,
	 PMIC_RG_BUCK_VS2_OC_DEG_EN_MASK, PMIC_RG_BUCK_VS2_OC_DEG_EN_SHIFT},
	{PMIC_RG_BUCK_VS2_OC_WND, PMIC_RG_BUCK_VS2_OC_WND_ADDR, PMIC_RG_BUCK_VS2_OC_WND_MASK,
	 PMIC_RG_BUCK_VS2_OC_WND_SHIFT},
	{PMIC_RG_BUCK_VS2_OC_THD, PMIC_RG_BUCK_VS2_OC_THD_ADDR, PMIC_RG_BUCK_VS2_OC_THD_MASK,
	 PMIC_RG_BUCK_VS2_OC_THD_SHIFT},
	{PMIC_DA_NI_VS2_VOSEL, PMIC_DA_NI_VS2_VOSEL_ADDR, PMIC_DA_NI_VS2_VOSEL_MASK,
	 PMIC_DA_NI_VS2_VOSEL_SHIFT},
	{PMIC_DA_NI_VS2_VOSEL_GRAY, PMIC_DA_NI_VS2_VOSEL_GRAY_ADDR, PMIC_DA_NI_VS2_VOSEL_GRAY_MASK,
	 PMIC_DA_NI_VS2_VOSEL_GRAY_SHIFT},
	{PMIC_DA_QI_VS2_EN, PMIC_DA_QI_VS2_EN_ADDR, PMIC_DA_QI_VS2_EN_MASK,
	 PMIC_DA_QI_VS2_EN_SHIFT},
	{PMIC_DA_VS2_STB, PMIC_DA_VS2_STB_ADDR, PMIC_DA_VS2_STB_MASK, PMIC_DA_VS2_STB_SHIFT},
	{PMIC_DA_NI_VS2_VSLEEP_SEL, PMIC_DA_NI_VS2_VSLEEP_SEL_ADDR, PMIC_DA_NI_VS2_VSLEEP_SEL_MASK,
	 PMIC_DA_NI_VS2_VSLEEP_SEL_SHIFT},
	{PMIC_DA_NI_VS2_R2R_PDN, PMIC_DA_NI_VS2_R2R_PDN_ADDR, PMIC_DA_NI_VS2_R2R_PDN_MASK,
	 PMIC_DA_NI_VS2_R2R_PDN_SHIFT},
	{PMIC_DA_QI_VS2_DVS_EN, PMIC_DA_QI_VS2_DVS_EN_ADDR, PMIC_DA_QI_VS2_DVS_EN_MASK,
	 PMIC_DA_QI_VS2_DVS_EN_SHIFT},
	{PMIC_RG_BUCK_VS2_LP_AUTO_EN, PMIC_RG_BUCK_VS2_LP_AUTO_EN_ADDR,
	 PMIC_RG_BUCK_VS2_LP_AUTO_EN_MASK, PMIC_RG_BUCK_VS2_LP_AUTO_EN_SHIFT},
	{PMIC_RG_BUCK_VS2_LP_DBG_EN, PMIC_RG_BUCK_VS2_LP_DBG_EN_ADDR,
	 PMIC_RG_BUCK_VS2_LP_DBG_EN_MASK, PMIC_RG_BUCK_VS2_LP_DBG_EN_SHIFT},
	{PMIC_RG_BUCK_VS2_VSLEEP_SEL_DBG, PMIC_RG_BUCK_VS2_VSLEEP_SEL_DBG_ADDR,
	 PMIC_RG_BUCK_VS2_VSLEEP_SEL_DBG_MASK, PMIC_RG_BUCK_VS2_VSLEEP_SEL_DBG_SHIFT},
	{PMIC_RG_BUCK_VS2_R2R_PDN_DBG, PMIC_RG_BUCK_VS2_R2R_PDN_DBG_ADDR,
	 PMIC_RG_BUCK_VS2_R2R_PDN_DBG_MASK, PMIC_RG_BUCK_VS2_R2R_PDN_DBG_SHIFT},
	{PMIC_RG_BUCK_VS2_OC_FLAG_CLR_SEL, PMIC_RG_BUCK_VS2_OC_FLAG_CLR_SEL_ADDR,
	 PMIC_RG_BUCK_VS2_OC_FLAG_CLR_SEL_MASK, PMIC_RG_BUCK_VS2_OC_FLAG_CLR_SEL_SHIFT},
	{PMIC_RG_BUCK_VS2_OSC_SEL_DIS, PMIC_RG_BUCK_VS2_OSC_SEL_DIS_ADDR,
	 PMIC_RG_BUCK_VS2_OSC_SEL_DIS_MASK, PMIC_RG_BUCK_VS2_OSC_SEL_DIS_SHIFT},
	{PMIC_RG_BUCK_VS2_CK_SW_MODE, PMIC_RG_BUCK_VS2_CK_SW_MODE_ADDR,
	 PMIC_RG_BUCK_VS2_CK_SW_MODE_MASK, PMIC_RG_BUCK_VS2_CK_SW_MODE_SHIFT},
	{PMIC_RG_BUCK_VS2_CK_SW_EN, PMIC_RG_BUCK_VS2_CK_SW_EN_ADDR, PMIC_RG_BUCK_VS2_CK_SW_EN_MASK,
	 PMIC_RG_BUCK_VS2_CK_SW_EN_SHIFT},
	{PMIC_RG_BUCK_VPA1_EN, PMIC_RG_BUCK_VPA1_EN_ADDR, PMIC_RG_BUCK_VPA1_EN_MASK,
	 PMIC_RG_BUCK_VPA1_EN_SHIFT},
	{PMIC_RG_BUCK_VPA1_VOSEL, PMIC_RG_BUCK_VPA1_VOSEL_ADDR, PMIC_RG_BUCK_VPA1_VOSEL_MASK,
	 PMIC_RG_BUCK_VPA1_VOSEL_SHIFT},
	{PMIC_RG_BUCK_VPA1_SFCHG_FRATE, PMIC_RG_BUCK_VPA1_SFCHG_FRATE_ADDR,
	 PMIC_RG_BUCK_VPA1_SFCHG_FRATE_MASK, PMIC_RG_BUCK_VPA1_SFCHG_FRATE_SHIFT},
	{PMIC_RG_BUCK_VPA1_SFCHG_FEN, PMIC_RG_BUCK_VPA1_SFCHG_FEN_ADDR,
	 PMIC_RG_BUCK_VPA1_SFCHG_FEN_MASK, PMIC_RG_BUCK_VPA1_SFCHG_FEN_SHIFT},
	{PMIC_RG_BUCK_VPA1_SFCHG_RRATE, PMIC_RG_BUCK_VPA1_SFCHG_RRATE_ADDR,
	 PMIC_RG_BUCK_VPA1_SFCHG_RRATE_MASK, PMIC_RG_BUCK_VPA1_SFCHG_RRATE_SHIFT},
	{PMIC_RG_BUCK_VPA1_SFCHG_REN, PMIC_RG_BUCK_VPA1_SFCHG_REN_ADDR,
	 PMIC_RG_BUCK_VPA1_SFCHG_REN_MASK, PMIC_RG_BUCK_VPA1_SFCHG_REN_SHIFT},
	{PMIC_RG_BUCK_VPA1_DVS_TRANS_TD, PMIC_RG_BUCK_VPA1_DVS_TRANS_TD_ADDR,
	 PMIC_RG_BUCK_VPA1_DVS_TRANS_TD_MASK, PMIC_RG_BUCK_VPA1_DVS_TRANS_TD_SHIFT},
	{PMIC_RG_BUCK_VPA1_DVS_TRANS_CTRL, PMIC_RG_BUCK_VPA1_DVS_TRANS_CTRL_ADDR,
	 PMIC_RG_BUCK_VPA1_DVS_TRANS_CTRL_MASK, PMIC_RG_BUCK_VPA1_DVS_TRANS_CTRL_SHIFT},
	{PMIC_RG_BUCK_VPA1_DVS_TRANS_ONCE, PMIC_RG_BUCK_VPA1_DVS_TRANS_ONCE_ADDR,
	 PMIC_RG_BUCK_VPA1_DVS_TRANS_ONCE_MASK, PMIC_RG_BUCK_VPA1_DVS_TRANS_ONCE_SHIFT},
	{PMIC_RG_BUCK_VPA1_DVS_BW_TD, PMIC_RG_BUCK_VPA1_DVS_BW_TD_ADDR,
	 PMIC_RG_BUCK_VPA1_DVS_BW_TD_MASK, PMIC_RG_BUCK_VPA1_DVS_BW_TD_SHIFT},
	{PMIC_RG_BUCK_VPA1_DVS_BW_CTRL, PMIC_RG_BUCK_VPA1_DVS_BW_CTRL_ADDR,
	 PMIC_RG_BUCK_VPA1_DVS_BW_CTRL_MASK, PMIC_RG_BUCK_VPA1_DVS_BW_CTRL_SHIFT},
	{PMIC_RG_BUCK_VPA1_DVS_BW_ONCE, PMIC_RG_BUCK_VPA1_DVS_BW_ONCE_ADDR,
	 PMIC_RG_BUCK_VPA1_DVS_BW_ONCE_MASK, PMIC_RG_BUCK_VPA1_DVS_BW_ONCE_SHIFT},
	{PMIC_RG_BUCK_VPA1_OC_DEG_EN, PMIC_RG_BUCK_VPA1_OC_DEG_EN_ADDR,
	 PMIC_RG_BUCK_VPA1_OC_DEG_EN_MASK, PMIC_RG_BUCK_VPA1_OC_DEG_EN_SHIFT},
	{PMIC_RG_BUCK_VPA1_OC_WND, PMIC_RG_BUCK_VPA1_OC_WND_ADDR, PMIC_RG_BUCK_VPA1_OC_WND_MASK,
	 PMIC_RG_BUCK_VPA1_OC_WND_SHIFT},
	{PMIC_RG_BUCK_VPA1_OC_THD, PMIC_RG_BUCK_VPA1_OC_THD_ADDR, PMIC_RG_BUCK_VPA1_OC_THD_MASK,
	 PMIC_RG_BUCK_VPA1_OC_THD_SHIFT},
	{PMIC_DA_NI_VPA1_VOSEL, PMIC_DA_NI_VPA1_VOSEL_ADDR, PMIC_DA_NI_VPA1_VOSEL_MASK,
	 PMIC_DA_NI_VPA1_VOSEL_SHIFT},
	{PMIC_DA_NI_VPA1_VOSEL_GRAY, PMIC_DA_NI_VPA1_VOSEL_GRAY_ADDR,
	 PMIC_DA_NI_VPA1_VOSEL_GRAY_MASK, PMIC_DA_NI_VPA1_VOSEL_GRAY_SHIFT},
	{PMIC_DA_QI_VPA1_EN, PMIC_DA_QI_VPA1_EN_ADDR, PMIC_DA_QI_VPA1_EN_MASK,
	 PMIC_DA_QI_VPA1_EN_SHIFT},
	{PMIC_DA_VPA1_STB, PMIC_DA_VPA1_STB_ADDR, PMIC_DA_VPA1_STB_MASK, PMIC_DA_VPA1_STB_SHIFT},
	{PMIC_DA_NI_VPA1_DVS_TRANST, PMIC_DA_NI_VPA1_DVS_TRANST_ADDR,
	 PMIC_DA_NI_VPA1_DVS_TRANST_MASK, PMIC_DA_NI_VPA1_DVS_TRANST_SHIFT},
	{PMIC_DA_NI_VPA1_DVS_BW, PMIC_DA_NI_VPA1_DVS_BW_ADDR, PMIC_DA_NI_VPA1_DVS_BW_MASK,
	 PMIC_DA_NI_VPA1_DVS_BW_SHIFT},
	{PMIC_DA_NI_VPA1_DLC, PMIC_DA_NI_VPA1_DLC_ADDR, PMIC_DA_NI_VPA1_DLC_MASK,
	 PMIC_DA_NI_VPA1_DLC_SHIFT},
	{PMIC_RG_BUCK_VPA1_LP_DBG_EN, PMIC_RG_BUCK_VPA1_LP_DBG_EN_ADDR,
	 PMIC_RG_BUCK_VPA1_LP_DBG_EN_MASK, PMIC_RG_BUCK_VPA1_LP_DBG_EN_SHIFT},
	{PMIC_RG_BUCK_VPA1_OC_FLAG_CLR_SEL, PMIC_RG_BUCK_VPA1_OC_FLAG_CLR_SEL_ADDR,
	 PMIC_RG_BUCK_VPA1_OC_FLAG_CLR_SEL_MASK, PMIC_RG_BUCK_VPA1_OC_FLAG_CLR_SEL_SHIFT},
	{PMIC_RG_BUCK_VPA1_OSC_SEL_DIS, PMIC_RG_BUCK_VPA1_OSC_SEL_DIS_ADDR,
	 PMIC_RG_BUCK_VPA1_OSC_SEL_DIS_MASK, PMIC_RG_BUCK_VPA1_OSC_SEL_DIS_SHIFT},
	{PMIC_RG_BUCK_VPA1_CK_SW_MODE, PMIC_RG_BUCK_VPA1_CK_SW_MODE_ADDR,
	 PMIC_RG_BUCK_VPA1_CK_SW_MODE_MASK, PMIC_RG_BUCK_VPA1_CK_SW_MODE_SHIFT},
	{PMIC_RG_BUCK_VPA1_CK_SW_EN, PMIC_RG_BUCK_VPA1_CK_SW_EN_ADDR,
	 PMIC_RG_BUCK_VPA1_CK_SW_EN_MASK, PMIC_RG_BUCK_VPA1_CK_SW_EN_SHIFT},
	{PMIC_RG_BUCK_VPA1_VOSEL_DLC011, PMIC_RG_BUCK_VPA1_VOSEL_DLC011_ADDR,
	 PMIC_RG_BUCK_VPA1_VOSEL_DLC011_MASK, PMIC_RG_BUCK_VPA1_VOSEL_DLC011_SHIFT},
	{PMIC_RG_BUCK_VPA1_VOSEL_DLC111, PMIC_RG_BUCK_VPA1_VOSEL_DLC111_ADDR,
	 PMIC_RG_BUCK_VPA1_VOSEL_DLC111_MASK, PMIC_RG_BUCK_VPA1_VOSEL_DLC111_SHIFT},
	{PMIC_RG_BUCK_VPA1_VOSEL_DLC001, PMIC_RG_BUCK_VPA1_VOSEL_DLC001_ADDR,
	 PMIC_RG_BUCK_VPA1_VOSEL_DLC001_MASK, PMIC_RG_BUCK_VPA1_VOSEL_DLC001_SHIFT},
	{PMIC_RG_BUCK_VPA1_DLC_MAP_EN, PMIC_RG_BUCK_VPA1_DLC_MAP_EN_ADDR,
	 PMIC_RG_BUCK_VPA1_DLC_MAP_EN_MASK, PMIC_RG_BUCK_VPA1_DLC_MAP_EN_SHIFT},
	{PMIC_RG_BUCK_VPA1_DLC, PMIC_RG_BUCK_VPA1_DLC_ADDR, PMIC_RG_BUCK_VPA1_DLC_MASK,
	 PMIC_RG_BUCK_VPA1_DLC_SHIFT},
	{PMIC_RG_BUCK_VPA1_MSFG_EN, PMIC_RG_BUCK_VPA1_MSFG_EN_ADDR, PMIC_RG_BUCK_VPA1_MSFG_EN_MASK,
	 PMIC_RG_BUCK_VPA1_MSFG_EN_SHIFT},
	{PMIC_RG_BUCK_VPA1_MSFG_RDELTA2GO, PMIC_RG_BUCK_VPA1_MSFG_RDELTA2GO_ADDR,
	 PMIC_RG_BUCK_VPA1_MSFG_RDELTA2GO_MASK, PMIC_RG_BUCK_VPA1_MSFG_RDELTA2GO_SHIFT},
	{PMIC_RG_BUCK_VPA1_MSFG_FDELTA2GO, PMIC_RG_BUCK_VPA1_MSFG_FDELTA2GO_ADDR,
	 PMIC_RG_BUCK_VPA1_MSFG_FDELTA2GO_MASK, PMIC_RG_BUCK_VPA1_MSFG_FDELTA2GO_SHIFT},
	{PMIC_RG_BUCK_VPA1_MSFG_RRATE0, PMIC_RG_BUCK_VPA1_MSFG_RRATE0_ADDR,
	 PMIC_RG_BUCK_VPA1_MSFG_RRATE0_MASK, PMIC_RG_BUCK_VPA1_MSFG_RRATE0_SHIFT},
	{PMIC_RG_BUCK_VPA1_MSFG_RRATE1, PMIC_RG_BUCK_VPA1_MSFG_RRATE1_ADDR,
	 PMIC_RG_BUCK_VPA1_MSFG_RRATE1_MASK, PMIC_RG_BUCK_VPA1_MSFG_RRATE1_SHIFT},
	{PMIC_RG_BUCK_VPA1_MSFG_RRATE2, PMIC_RG_BUCK_VPA1_MSFG_RRATE2_ADDR,
	 PMIC_RG_BUCK_VPA1_MSFG_RRATE2_MASK, PMIC_RG_BUCK_VPA1_MSFG_RRATE2_SHIFT},
	{PMIC_RG_BUCK_VPA1_MSFG_RRATE3, PMIC_RG_BUCK_VPA1_MSFG_RRATE3_ADDR,
	 PMIC_RG_BUCK_VPA1_MSFG_RRATE3_MASK, PMIC_RG_BUCK_VPA1_MSFG_RRATE3_SHIFT},
	{PMIC_RG_BUCK_VPA1_MSFG_RRATE4, PMIC_RG_BUCK_VPA1_MSFG_RRATE4_ADDR,
	 PMIC_RG_BUCK_VPA1_MSFG_RRATE4_MASK, PMIC_RG_BUCK_VPA1_MSFG_RRATE4_SHIFT},
	{PMIC_RG_BUCK_VPA1_MSFG_RRATE5, PMIC_RG_BUCK_VPA1_MSFG_RRATE5_ADDR,
	 PMIC_RG_BUCK_VPA1_MSFG_RRATE5_MASK, PMIC_RG_BUCK_VPA1_MSFG_RRATE5_SHIFT},
	{PMIC_RG_BUCK_VPA1_MSFG_RTHD0, PMIC_RG_BUCK_VPA1_MSFG_RTHD0_ADDR,
	 PMIC_RG_BUCK_VPA1_MSFG_RTHD0_MASK, PMIC_RG_BUCK_VPA1_MSFG_RTHD0_SHIFT},
	{PMIC_RG_BUCK_VPA1_MSFG_RTHD1, PMIC_RG_BUCK_VPA1_MSFG_RTHD1_ADDR,
	 PMIC_RG_BUCK_VPA1_MSFG_RTHD1_MASK, PMIC_RG_BUCK_VPA1_MSFG_RTHD1_SHIFT},
	{PMIC_RG_BUCK_VPA1_MSFG_RTHD2, PMIC_RG_BUCK_VPA1_MSFG_RTHD2_ADDR,
	 PMIC_RG_BUCK_VPA1_MSFG_RTHD2_MASK, PMIC_RG_BUCK_VPA1_MSFG_RTHD2_SHIFT},
	{PMIC_RG_BUCK_VPA1_MSFG_RTHD3, PMIC_RG_BUCK_VPA1_MSFG_RTHD3_ADDR,
	 PMIC_RG_BUCK_VPA1_MSFG_RTHD3_MASK, PMIC_RG_BUCK_VPA1_MSFG_RTHD3_SHIFT},
	{PMIC_RG_BUCK_VPA1_MSFG_RTHD4, PMIC_RG_BUCK_VPA1_MSFG_RTHD4_ADDR,
	 PMIC_RG_BUCK_VPA1_MSFG_RTHD4_MASK, PMIC_RG_BUCK_VPA1_MSFG_RTHD4_SHIFT},
	{PMIC_RG_BUCK_VPA1_MSFG_FRATE0, PMIC_RG_BUCK_VPA1_MSFG_FRATE0_ADDR,
	 PMIC_RG_BUCK_VPA1_MSFG_FRATE0_MASK, PMIC_RG_BUCK_VPA1_MSFG_FRATE0_SHIFT},
	{PMIC_RG_BUCK_VPA1_MSFG_FRATE1, PMIC_RG_BUCK_VPA1_MSFG_FRATE1_ADDR,
	 PMIC_RG_BUCK_VPA1_MSFG_FRATE1_MASK, PMIC_RG_BUCK_VPA1_MSFG_FRATE1_SHIFT},
	{PMIC_RG_BUCK_VPA1_MSFG_FRATE2, PMIC_RG_BUCK_VPA1_MSFG_FRATE2_ADDR,
	 PMIC_RG_BUCK_VPA1_MSFG_FRATE2_MASK, PMIC_RG_BUCK_VPA1_MSFG_FRATE2_SHIFT},
	{PMIC_RG_BUCK_VPA1_MSFG_FRATE3, PMIC_RG_BUCK_VPA1_MSFG_FRATE3_ADDR,
	 PMIC_RG_BUCK_VPA1_MSFG_FRATE3_MASK, PMIC_RG_BUCK_VPA1_MSFG_FRATE3_SHIFT},
	{PMIC_RG_BUCK_VPA1_MSFG_FRATE4, PMIC_RG_BUCK_VPA1_MSFG_FRATE4_ADDR,
	 PMIC_RG_BUCK_VPA1_MSFG_FRATE4_MASK, PMIC_RG_BUCK_VPA1_MSFG_FRATE4_SHIFT},
	{PMIC_RG_BUCK_VPA1_MSFG_FRATE5, PMIC_RG_BUCK_VPA1_MSFG_FRATE5_ADDR,
	 PMIC_RG_BUCK_VPA1_MSFG_FRATE5_MASK, PMIC_RG_BUCK_VPA1_MSFG_FRATE5_SHIFT},
	{PMIC_RG_BUCK_VPA1_MSFG_FTHD0, PMIC_RG_BUCK_VPA1_MSFG_FTHD0_ADDR,
	 PMIC_RG_BUCK_VPA1_MSFG_FTHD0_MASK, PMIC_RG_BUCK_VPA1_MSFG_FTHD0_SHIFT},
	{PMIC_RG_BUCK_VPA1_MSFG_FTHD1, PMIC_RG_BUCK_VPA1_MSFG_FTHD1_ADDR,
	 PMIC_RG_BUCK_VPA1_MSFG_FTHD1_MASK, PMIC_RG_BUCK_VPA1_MSFG_FTHD1_SHIFT},
	{PMIC_RG_BUCK_VPA1_MSFG_FTHD2, PMIC_RG_BUCK_VPA1_MSFG_FTHD2_ADDR,
	 PMIC_RG_BUCK_VPA1_MSFG_FTHD2_MASK, PMIC_RG_BUCK_VPA1_MSFG_FTHD2_SHIFT},
	{PMIC_RG_BUCK_VPA1_MSFG_FTHD3, PMIC_RG_BUCK_VPA1_MSFG_FTHD3_ADDR,
	 PMIC_RG_BUCK_VPA1_MSFG_FTHD3_MASK, PMIC_RG_BUCK_VPA1_MSFG_FTHD3_SHIFT},
	{PMIC_RG_BUCK_VPA1_MSFG_FTHD4, PMIC_RG_BUCK_VPA1_MSFG_FTHD4_ADDR,
	 PMIC_RG_BUCK_VPA1_MSFG_FTHD4_MASK, PMIC_RG_BUCK_VPA1_MSFG_FTHD4_SHIFT},
	{PMIC_RG_BUCK_VPA2_EN, PMIC_RG_BUCK_VPA2_EN_ADDR, PMIC_RG_BUCK_VPA2_EN_MASK,
	 PMIC_RG_BUCK_VPA2_EN_SHIFT},
	{PMIC_RG_BUCK_VPA2_VOSEL, PMIC_RG_BUCK_VPA2_VOSEL_ADDR, PMIC_RG_BUCK_VPA2_VOSEL_MASK,
	 PMIC_RG_BUCK_VPA2_VOSEL_SHIFT},
	{PMIC_RG_BUCK_VPA2_SFCHG_FRATE, PMIC_RG_BUCK_VPA2_SFCHG_FRATE_ADDR,
	 PMIC_RG_BUCK_VPA2_SFCHG_FRATE_MASK, PMIC_RG_BUCK_VPA2_SFCHG_FRATE_SHIFT},
	{PMIC_RG_BUCK_VPA2_SFCHG_FEN, PMIC_RG_BUCK_VPA2_SFCHG_FEN_ADDR,
	 PMIC_RG_BUCK_VPA2_SFCHG_FEN_MASK, PMIC_RG_BUCK_VPA2_SFCHG_FEN_SHIFT},
	{PMIC_RG_BUCK_VPA2_SFCHG_RRATE, PMIC_RG_BUCK_VPA2_SFCHG_RRATE_ADDR,
	 PMIC_RG_BUCK_VPA2_SFCHG_RRATE_MASK, PMIC_RG_BUCK_VPA2_SFCHG_RRATE_SHIFT},
	{PMIC_RG_BUCK_VPA2_SFCHG_REN, PMIC_RG_BUCK_VPA2_SFCHG_REN_ADDR,
	 PMIC_RG_BUCK_VPA2_SFCHG_REN_MASK, PMIC_RG_BUCK_VPA2_SFCHG_REN_SHIFT},
	{PMIC_RG_BUCK_VPA2_DVS_TRANS_TD, PMIC_RG_BUCK_VPA2_DVS_TRANS_TD_ADDR,
	 PMIC_RG_BUCK_VPA2_DVS_TRANS_TD_MASK, PMIC_RG_BUCK_VPA2_DVS_TRANS_TD_SHIFT},
	{PMIC_RG_BUCK_VPA2_DVS_TRANS_CTRL, PMIC_RG_BUCK_VPA2_DVS_TRANS_CTRL_ADDR,
	 PMIC_RG_BUCK_VPA2_DVS_TRANS_CTRL_MASK, PMIC_RG_BUCK_VPA2_DVS_TRANS_CTRL_SHIFT},
	{PMIC_RG_BUCK_VPA2_DVS_TRANS_ONCE, PMIC_RG_BUCK_VPA2_DVS_TRANS_ONCE_ADDR,
	 PMIC_RG_BUCK_VPA2_DVS_TRANS_ONCE_MASK, PMIC_RG_BUCK_VPA2_DVS_TRANS_ONCE_SHIFT},
	{PMIC_RG_BUCK_VPA2_DVS_BW_TD, PMIC_RG_BUCK_VPA2_DVS_BW_TD_ADDR,
	 PMIC_RG_BUCK_VPA2_DVS_BW_TD_MASK, PMIC_RG_BUCK_VPA2_DVS_BW_TD_SHIFT},
	{PMIC_RG_BUCK_VPA2_DVS_BW_CTRL, PMIC_RG_BUCK_VPA2_DVS_BW_CTRL_ADDR,
	 PMIC_RG_BUCK_VPA2_DVS_BW_CTRL_MASK, PMIC_RG_BUCK_VPA2_DVS_BW_CTRL_SHIFT},
	{PMIC_RG_BUCK_VPA2_DVS_BW_ONCE, PMIC_RG_BUCK_VPA2_DVS_BW_ONCE_ADDR,
	 PMIC_RG_BUCK_VPA2_DVS_BW_ONCE_MASK, PMIC_RG_BUCK_VPA2_DVS_BW_ONCE_SHIFT},
	{PMIC_RG_BUCK_VPA2_OC_DEG_EN, PMIC_RG_BUCK_VPA2_OC_DEG_EN_ADDR,
	 PMIC_RG_BUCK_VPA2_OC_DEG_EN_MASK, PMIC_RG_BUCK_VPA2_OC_DEG_EN_SHIFT},
	{PMIC_RG_BUCK_VPA2_OC_WND, PMIC_RG_BUCK_VPA2_OC_WND_ADDR, PMIC_RG_BUCK_VPA2_OC_WND_MASK,
	 PMIC_RG_BUCK_VPA2_OC_WND_SHIFT},
	{PMIC_RG_BUCK_VPA2_OC_THD, PMIC_RG_BUCK_VPA2_OC_THD_ADDR, PMIC_RG_BUCK_VPA2_OC_THD_MASK,
	 PMIC_RG_BUCK_VPA2_OC_THD_SHIFT},
	{PMIC_DA_NI_VPA2_VOSEL, PMIC_DA_NI_VPA2_VOSEL_ADDR, PMIC_DA_NI_VPA2_VOSEL_MASK,
	 PMIC_DA_NI_VPA2_VOSEL_SHIFT},
	{PMIC_DA_NI_VPA2_VOSEL_GRAY, PMIC_DA_NI_VPA2_VOSEL_GRAY_ADDR,
	 PMIC_DA_NI_VPA2_VOSEL_GRAY_MASK, PMIC_DA_NI_VPA2_VOSEL_GRAY_SHIFT},
	{PMIC_DA_QI_VPA2_EN, PMIC_DA_QI_VPA2_EN_ADDR, PMIC_DA_QI_VPA2_EN_MASK,
	 PMIC_DA_QI_VPA2_EN_SHIFT},
	{PMIC_DA_VPA2_STB, PMIC_DA_VPA2_STB_ADDR, PMIC_DA_VPA2_STB_MASK, PMIC_DA_VPA2_STB_SHIFT},
	{PMIC_DA_NI_VPA2_DVS_TRANST, PMIC_DA_NI_VPA2_DVS_TRANST_ADDR,
	 PMIC_DA_NI_VPA2_DVS_TRANST_MASK, PMIC_DA_NI_VPA2_DVS_TRANST_SHIFT},
	{PMIC_DA_NI_VPA2_DVS_BW, PMIC_DA_NI_VPA2_DVS_BW_ADDR, PMIC_DA_NI_VPA2_DVS_BW_MASK,
	 PMIC_DA_NI_VPA2_DVS_BW_SHIFT},
	{PMIC_DA_NI_VPA2_DLC, PMIC_DA_NI_VPA2_DLC_ADDR, PMIC_DA_NI_VPA2_DLC_MASK,
	 PMIC_DA_NI_VPA2_DLC_SHIFT},
	{PMIC_RG_BUCK_VPA2_LP_DBG_EN, PMIC_RG_BUCK_VPA2_LP_DBG_EN_ADDR,
	 PMIC_RG_BUCK_VPA2_LP_DBG_EN_MASK, PMIC_RG_BUCK_VPA2_LP_DBG_EN_SHIFT},
	{PMIC_RG_BUCK_VPA2_OC_FLAG_CLR_SEL, PMIC_RG_BUCK_VPA2_OC_FLAG_CLR_SEL_ADDR,
	 PMIC_RG_BUCK_VPA2_OC_FLAG_CLR_SEL_MASK, PMIC_RG_BUCK_VPA2_OC_FLAG_CLR_SEL_SHIFT},
	{PMIC_RG_BUCK_VPA2_OSC_SEL_DIS, PMIC_RG_BUCK_VPA2_OSC_SEL_DIS_ADDR,
	 PMIC_RG_BUCK_VPA2_OSC_SEL_DIS_MASK, PMIC_RG_BUCK_VPA2_OSC_SEL_DIS_SHIFT},
	{PMIC_RG_BUCK_VPA2_CK_SW_MODE, PMIC_RG_BUCK_VPA2_CK_SW_MODE_ADDR,
	 PMIC_RG_BUCK_VPA2_CK_SW_MODE_MASK, PMIC_RG_BUCK_VPA2_CK_SW_MODE_SHIFT},
	{PMIC_RG_BUCK_VPA2_CK_SW_EN, PMIC_RG_BUCK_VPA2_CK_SW_EN_ADDR,
	 PMIC_RG_BUCK_VPA2_CK_SW_EN_MASK, PMIC_RG_BUCK_VPA2_CK_SW_EN_SHIFT},
	{PMIC_RG_BUCK_VPA2_VOSEL_DLC011, PMIC_RG_BUCK_VPA2_VOSEL_DLC011_ADDR,
	 PMIC_RG_BUCK_VPA2_VOSEL_DLC011_MASK, PMIC_RG_BUCK_VPA2_VOSEL_DLC011_SHIFT},
	{PMIC_RG_BUCK_VPA2_VOSEL_DLC111, PMIC_RG_BUCK_VPA2_VOSEL_DLC111_ADDR,
	 PMIC_RG_BUCK_VPA2_VOSEL_DLC111_MASK, PMIC_RG_BUCK_VPA2_VOSEL_DLC111_SHIFT},
	{PMIC_RG_BUCK_VPA2_VOSEL_DLC001, PMIC_RG_BUCK_VPA2_VOSEL_DLC001_ADDR,
	 PMIC_RG_BUCK_VPA2_VOSEL_DLC001_MASK, PMIC_RG_BUCK_VPA2_VOSEL_DLC001_SHIFT},
	{PMIC_RG_BUCK_VPA2_DLC_MAP_EN, PMIC_RG_BUCK_VPA2_DLC_MAP_EN_ADDR,
	 PMIC_RG_BUCK_VPA2_DLC_MAP_EN_MASK, PMIC_RG_BUCK_VPA2_DLC_MAP_EN_SHIFT},
	{PMIC_RG_BUCK_VPA2_DLC, PMIC_RG_BUCK_VPA2_DLC_ADDR, PMIC_RG_BUCK_VPA2_DLC_MASK,
	 PMIC_RG_BUCK_VPA2_DLC_SHIFT},
	{PMIC_RG_BUCK_VPA2_MSFG_EN, PMIC_RG_BUCK_VPA2_MSFG_EN_ADDR, PMIC_RG_BUCK_VPA2_MSFG_EN_MASK,
	 PMIC_RG_BUCK_VPA2_MSFG_EN_SHIFT},
	{PMIC_RG_BUCK_VPA2_MSFG_RDELTA2GO, PMIC_RG_BUCK_VPA2_MSFG_RDELTA2GO_ADDR,
	 PMIC_RG_BUCK_VPA2_MSFG_RDELTA2GO_MASK, PMIC_RG_BUCK_VPA2_MSFG_RDELTA2GO_SHIFT},
	{PMIC_RG_BUCK_VPA2_MSFG_FDELTA2GO, PMIC_RG_BUCK_VPA2_MSFG_FDELTA2GO_ADDR,
	 PMIC_RG_BUCK_VPA2_MSFG_FDELTA2GO_MASK, PMIC_RG_BUCK_VPA2_MSFG_FDELTA2GO_SHIFT},
	{PMIC_RG_BUCK_VPA2_MSFG_RRATE0, PMIC_RG_BUCK_VPA2_MSFG_RRATE0_ADDR,
	 PMIC_RG_BUCK_VPA2_MSFG_RRATE0_MASK, PMIC_RG_BUCK_VPA2_MSFG_RRATE0_SHIFT},
	{PMIC_RG_BUCK_VPA2_MSFG_RRATE1, PMIC_RG_BUCK_VPA2_MSFG_RRATE1_ADDR,
	 PMIC_RG_BUCK_VPA2_MSFG_RRATE1_MASK, PMIC_RG_BUCK_VPA2_MSFG_RRATE1_SHIFT},
	{PMIC_RG_BUCK_VPA2_MSFG_RRATE2, PMIC_RG_BUCK_VPA2_MSFG_RRATE2_ADDR,
	 PMIC_RG_BUCK_VPA2_MSFG_RRATE2_MASK, PMIC_RG_BUCK_VPA2_MSFG_RRATE2_SHIFT},
	{PMIC_RG_BUCK_VPA2_MSFG_RRATE3, PMIC_RG_BUCK_VPA2_MSFG_RRATE3_ADDR,
	 PMIC_RG_BUCK_VPA2_MSFG_RRATE3_MASK, PMIC_RG_BUCK_VPA2_MSFG_RRATE3_SHIFT},
	{PMIC_RG_BUCK_VPA2_MSFG_RRATE4, PMIC_RG_BUCK_VPA2_MSFG_RRATE4_ADDR,
	 PMIC_RG_BUCK_VPA2_MSFG_RRATE4_MASK, PMIC_RG_BUCK_VPA2_MSFG_RRATE4_SHIFT},
	{PMIC_RG_BUCK_VPA2_MSFG_RRATE5, PMIC_RG_BUCK_VPA2_MSFG_RRATE5_ADDR,
	 PMIC_RG_BUCK_VPA2_MSFG_RRATE5_MASK, PMIC_RG_BUCK_VPA2_MSFG_RRATE5_SHIFT},
	{PMIC_RG_BUCK_VPA2_MSFG_RTHD0, PMIC_RG_BUCK_VPA2_MSFG_RTHD0_ADDR,
	 PMIC_RG_BUCK_VPA2_MSFG_RTHD0_MASK, PMIC_RG_BUCK_VPA2_MSFG_RTHD0_SHIFT},
	{PMIC_RG_BUCK_VPA2_MSFG_RTHD1, PMIC_RG_BUCK_VPA2_MSFG_RTHD1_ADDR,
	 PMIC_RG_BUCK_VPA2_MSFG_RTHD1_MASK, PMIC_RG_BUCK_VPA2_MSFG_RTHD1_SHIFT},
	{PMIC_RG_BUCK_VPA2_MSFG_RTHD2, PMIC_RG_BUCK_VPA2_MSFG_RTHD2_ADDR,
	 PMIC_RG_BUCK_VPA2_MSFG_RTHD2_MASK, PMIC_RG_BUCK_VPA2_MSFG_RTHD2_SHIFT},
	{PMIC_RG_BUCK_VPA2_MSFG_RTHD3, PMIC_RG_BUCK_VPA2_MSFG_RTHD3_ADDR,
	 PMIC_RG_BUCK_VPA2_MSFG_RTHD3_MASK, PMIC_RG_BUCK_VPA2_MSFG_RTHD3_SHIFT},
	{PMIC_RG_BUCK_VPA2_MSFG_RTHD4, PMIC_RG_BUCK_VPA2_MSFG_RTHD4_ADDR,
	 PMIC_RG_BUCK_VPA2_MSFG_RTHD4_MASK, PMIC_RG_BUCK_VPA2_MSFG_RTHD4_SHIFT},
	{PMIC_RG_BUCK_VPA2_MSFG_FRATE0, PMIC_RG_BUCK_VPA2_MSFG_FRATE0_ADDR,
	 PMIC_RG_BUCK_VPA2_MSFG_FRATE0_MASK, PMIC_RG_BUCK_VPA2_MSFG_FRATE0_SHIFT},
	{PMIC_RG_BUCK_VPA2_MSFG_FRATE1, PMIC_RG_BUCK_VPA2_MSFG_FRATE1_ADDR,
	 PMIC_RG_BUCK_VPA2_MSFG_FRATE1_MASK, PMIC_RG_BUCK_VPA2_MSFG_FRATE1_SHIFT},
	{PMIC_RG_BUCK_VPA2_MSFG_FRATE2, PMIC_RG_BUCK_VPA2_MSFG_FRATE2_ADDR,
	 PMIC_RG_BUCK_VPA2_MSFG_FRATE2_MASK, PMIC_RG_BUCK_VPA2_MSFG_FRATE2_SHIFT},
	{PMIC_RG_BUCK_VPA2_MSFG_FRATE3, PMIC_RG_BUCK_VPA2_MSFG_FRATE3_ADDR,
	 PMIC_RG_BUCK_VPA2_MSFG_FRATE3_MASK, PMIC_RG_BUCK_VPA2_MSFG_FRATE3_SHIFT},
	{PMIC_RG_BUCK_VPA2_MSFG_FRATE4, PMIC_RG_BUCK_VPA2_MSFG_FRATE4_ADDR,
	 PMIC_RG_BUCK_VPA2_MSFG_FRATE4_MASK, PMIC_RG_BUCK_VPA2_MSFG_FRATE4_SHIFT},
	{PMIC_RG_BUCK_VPA2_MSFG_FRATE5, PMIC_RG_BUCK_VPA2_MSFG_FRATE5_ADDR,
	 PMIC_RG_BUCK_VPA2_MSFG_FRATE5_MASK, PMIC_RG_BUCK_VPA2_MSFG_FRATE5_SHIFT},
	{PMIC_RG_BUCK_VPA2_MSFG_FTHD0, PMIC_RG_BUCK_VPA2_MSFG_FTHD0_ADDR,
	 PMIC_RG_BUCK_VPA2_MSFG_FTHD0_MASK, PMIC_RG_BUCK_VPA2_MSFG_FTHD0_SHIFT},
	{PMIC_RG_BUCK_VPA2_MSFG_FTHD1, PMIC_RG_BUCK_VPA2_MSFG_FTHD1_ADDR,
	 PMIC_RG_BUCK_VPA2_MSFG_FTHD1_MASK, PMIC_RG_BUCK_VPA2_MSFG_FTHD1_SHIFT},
	{PMIC_RG_BUCK_VPA2_MSFG_FTHD2, PMIC_RG_BUCK_VPA2_MSFG_FTHD2_ADDR,
	 PMIC_RG_BUCK_VPA2_MSFG_FTHD2_MASK, PMIC_RG_BUCK_VPA2_MSFG_FTHD2_SHIFT},
	{PMIC_RG_BUCK_VPA2_MSFG_FTHD3, PMIC_RG_BUCK_VPA2_MSFG_FTHD3_ADDR,
	 PMIC_RG_BUCK_VPA2_MSFG_FTHD3_MASK, PMIC_RG_BUCK_VPA2_MSFG_FTHD3_SHIFT},
	{PMIC_RG_BUCK_VPA2_MSFG_FTHD4, PMIC_RG_BUCK_VPA2_MSFG_FTHD4_ADDR,
	 PMIC_RG_BUCK_VPA2_MSFG_FTHD4_MASK, PMIC_RG_BUCK_VPA2_MSFG_FTHD4_SHIFT},
	{PMIC_RG_SMPS_TESTMODE_B, PMIC_RG_SMPS_TESTMODE_B_ADDR, PMIC_RG_SMPS_TESTMODE_B_MASK,
	 PMIC_RG_SMPS_TESTMODE_B_SHIFT},
	{PMIC_RG_VPA1_BURSTH, PMIC_RG_VPA1_BURSTH_ADDR, PMIC_RG_VPA1_BURSTH_MASK,
	 PMIC_RG_VPA1_BURSTH_SHIFT},
	{PMIC_RG_VPA1_BURSTL, PMIC_RG_VPA1_BURSTL_ADDR, PMIC_RG_VPA1_BURSTL_MASK,
	 PMIC_RG_VPA1_BURSTL_SHIFT},
	{PMIC_RG_VPA1_TRIMH, PMIC_RG_VPA1_TRIMH_ADDR, PMIC_RG_VPA1_TRIMH_MASK,
	 PMIC_RG_VPA1_TRIMH_SHIFT},
	{PMIC_RG_VPA1_TRIML, PMIC_RG_VPA1_TRIML_ADDR, PMIC_RG_VPA1_TRIML_MASK,
	 PMIC_RG_VPA1_TRIML_SHIFT},
	{PMIC_RG_VPA1_TRIM_REF, PMIC_RG_VPA1_TRIM_REF_ADDR, PMIC_RG_VPA1_TRIM_REF_MASK,
	 PMIC_RG_VPA1_TRIM_REF_SHIFT},
	{PMIC_RG_VPA2_BURSTH, PMIC_RG_VPA2_BURSTH_ADDR, PMIC_RG_VPA2_BURSTH_MASK,
	 PMIC_RG_VPA2_BURSTH_SHIFT},
	{PMIC_RG_VPA2_BURSTL, PMIC_RG_VPA2_BURSTL_ADDR, PMIC_RG_VPA2_BURSTL_MASK,
	 PMIC_RG_VPA2_BURSTL_SHIFT},
	{PMIC_RG_VPA2_TRIMH, PMIC_RG_VPA2_TRIMH_ADDR, PMIC_RG_VPA2_TRIMH_MASK,
	 PMIC_RG_VPA2_TRIMH_SHIFT},
	{PMIC_RG_VPA2_TRIML, PMIC_RG_VPA2_TRIML_ADDR, PMIC_RG_VPA2_TRIML_MASK,
	 PMIC_RG_VPA2_TRIML_SHIFT},
	{PMIC_RG_VPA2_TRIM_REF, PMIC_RG_VPA2_TRIM_REF_ADDR, PMIC_RG_VPA2_TRIM_REF_MASK,
	 PMIC_RG_VPA2_TRIM_REF_SHIFT},
	{PMIC_RG_VCORE_TRIMH, PMIC_RG_VCORE_TRIMH_ADDR, PMIC_RG_VCORE_TRIMH_MASK,
	 PMIC_RG_VCORE_TRIMH_SHIFT},
	{PMIC_RG_VCORE_TRIML, PMIC_RG_VCORE_TRIML_ADDR, PMIC_RG_VCORE_TRIML_MASK,
	 PMIC_RG_VCORE_TRIML_SHIFT},
	{PMIC_RG_VCORE_VSLEEP, PMIC_RG_VCORE_VSLEEP_ADDR, PMIC_RG_VCORE_VSLEEP_MASK,
	 PMIC_RG_VCORE_VSLEEP_SHIFT},
	{PMIC_RG_VDRAM_TRIMH, PMIC_RG_VDRAM_TRIMH_ADDR, PMIC_RG_VDRAM_TRIMH_MASK,
	 PMIC_RG_VDRAM_TRIMH_SHIFT},
	{PMIC_RG_VDRAM_TRIML, PMIC_RG_VDRAM_TRIML_ADDR, PMIC_RG_VDRAM_TRIML_MASK,
	 PMIC_RG_VDRAM_TRIML_SHIFT},
	{PMIC_RG_VDRAM_VSLEEP, PMIC_RG_VDRAM_VSLEEP_ADDR, PMIC_RG_VDRAM_VSLEEP_MASK,
	 PMIC_RG_VDRAM_VSLEEP_SHIFT},
	{PMIC_RG_VMODEM_TRIMH, PMIC_RG_VMODEM_TRIMH_ADDR, PMIC_RG_VMODEM_TRIMH_MASK,
	 PMIC_RG_VMODEM_TRIMH_SHIFT},
	{PMIC_RG_VMODEM_TRIML, PMIC_RG_VMODEM_TRIML_ADDR, PMIC_RG_VMODEM_TRIML_MASK,
	 PMIC_RG_VMODEM_TRIML_SHIFT},
	{PMIC_RG_VMODEM_VSLEEP, PMIC_RG_VMODEM_VSLEEP_ADDR, PMIC_RG_VMODEM_VSLEEP_MASK,
	 PMIC_RG_VMODEM_VSLEEP_SHIFT},
	{PMIC_RG_VMD1_TRIMH, PMIC_RG_VMD1_TRIMH_ADDR, PMIC_RG_VMD1_TRIMH_MASK,
	 PMIC_RG_VMD1_TRIMH_SHIFT},
	{PMIC_RG_VMD1_TRIML, PMIC_RG_VMD1_TRIML_ADDR, PMIC_RG_VMD1_TRIML_MASK,
	 PMIC_RG_VMD1_TRIML_SHIFT},
	{PMIC_RG_VMD1_VSLEEP, PMIC_RG_VMD1_VSLEEP_ADDR, PMIC_RG_VMD1_VSLEEP_MASK,
	 PMIC_RG_VMD1_VSLEEP_SHIFT},
	{PMIC_RG_VS1_TRIMH, PMIC_RG_VS1_TRIMH_ADDR, PMIC_RG_VS1_TRIMH_MASK,
	 PMIC_RG_VS1_TRIMH_SHIFT},
	{PMIC_RG_VS1_TRIML, PMIC_RG_VS1_TRIML_ADDR, PMIC_RG_VS1_TRIML_MASK,
	 PMIC_RG_VS1_TRIML_SHIFT},
	{PMIC_RG_VS1_VSLEEP, PMIC_RG_VS1_VSLEEP_ADDR, PMIC_RG_VS1_VSLEEP_MASK,
	 PMIC_RG_VS1_VSLEEP_SHIFT},
	{PMIC_RG_VS2_TRIML, PMIC_RG_VS2_TRIML_ADDR, PMIC_RG_VS2_TRIML_MASK,
	 PMIC_RG_VS2_TRIML_SHIFT},
	{PMIC_RG_VS2_VSLEEP, PMIC_RG_VS2_VSLEEP_ADDR, PMIC_RG_VS2_VSLEEP_MASK,
	 PMIC_RG_VS2_VSLEEP_SHIFT},
	{PMIC_RG_VSRAM_DVFS1_TRIMH, PMIC_RG_VSRAM_DVFS1_TRIMH_ADDR, PMIC_RG_VSRAM_DVFS1_TRIMH_MASK,
	 PMIC_RG_VSRAM_DVFS1_TRIMH_SHIFT},
	{PMIC_RG_VSRAM_DVFS2_TRIMH, PMIC_RG_VSRAM_DVFS2_TRIMH_ADDR, PMIC_RG_VSRAM_DVFS2_TRIMH_MASK,
	 PMIC_RG_VSRAM_DVFS2_TRIMH_SHIFT},
	{PMIC_RG_VSRAM_VCORE_TRIMH, PMIC_RG_VSRAM_VCORE_TRIMH_ADDR, PMIC_RG_VSRAM_VCORE_TRIMH_MASK,
	 PMIC_RG_VSRAM_VCORE_TRIMH_SHIFT},
	{PMIC_RG_VSRAM_VMD_TRIMH, PMIC_RG_VSRAM_VMD_TRIMH_ADDR, PMIC_RG_VSRAM_VMD_TRIMH_MASK,
	 PMIC_RG_VSRAM_VMD_TRIMH_SHIFT},
	{PMIC_RG_VSRAM_VGPU_TRIMH, PMIC_RG_VSRAM_VGPU_TRIMH_ADDR, PMIC_RG_VSRAM_VGPU_TRIMH_MASK,
	 PMIC_RG_VSRAM_VGPU_TRIMH_SHIFT},
	{PMIC_RG_VSRAM_DVFS1_TRIML, PMIC_RG_VSRAM_DVFS1_TRIML_ADDR, PMIC_RG_VSRAM_DVFS1_TRIML_MASK,
	 PMIC_RG_VSRAM_DVFS1_TRIML_SHIFT},
	{PMIC_RG_VSRAM_DVFS2_TRIML, PMIC_RG_VSRAM_DVFS2_TRIML_ADDR, PMIC_RG_VSRAM_DVFS2_TRIML_MASK,
	 PMIC_RG_VSRAM_DVFS2_TRIML_SHIFT},
	{PMIC_RG_VSRAM_VCORE_TRIML, PMIC_RG_VSRAM_VCORE_TRIML_ADDR, PMIC_RG_VSRAM_VCORE_TRIML_MASK,
	 PMIC_RG_VSRAM_VCORE_TRIML_SHIFT},
	{PMIC_RG_VSRAM_VMD_TRIML, PMIC_RG_VSRAM_VMD_TRIML_ADDR, PMIC_RG_VSRAM_VMD_TRIML_MASK,
	 PMIC_RG_VSRAM_VMD_TRIML_SHIFT},
	{PMIC_RG_VSRAM_VGPU_TRIML, PMIC_RG_VSRAM_VGPU_TRIML_ADDR, PMIC_RG_VSRAM_VGPU_TRIML_MASK,
	 PMIC_RG_VSRAM_VGPU_TRIML_SHIFT},
	{PMIC_RG_VSRAM_DVFS1_VSLEEP, PMIC_RG_VSRAM_DVFS1_VSLEEP_ADDR,
	 PMIC_RG_VSRAM_DVFS1_VSLEEP_MASK, PMIC_RG_VSRAM_DVFS1_VSLEEP_SHIFT},
	{PMIC_RG_VSRAM_DVFS2_VSLEEP, PMIC_RG_VSRAM_DVFS2_VSLEEP_ADDR,
	 PMIC_RG_VSRAM_DVFS2_VSLEEP_MASK, PMIC_RG_VSRAM_DVFS2_VSLEEP_SHIFT},
	{PMIC_RG_VSRAM_VCORE_VSLEEP, PMIC_RG_VSRAM_VCORE_VSLEEP_ADDR,
	 PMIC_RG_VSRAM_VCORE_VSLEEP_MASK, PMIC_RG_VSRAM_VCORE_VSLEEP_SHIFT},
	{PMIC_RG_VSRAM_VMD_VSLEEP, PMIC_RG_VSRAM_VMD_VSLEEP_ADDR, PMIC_RG_VSRAM_VMD_VSLEEP_MASK,
	 PMIC_RG_VSRAM_VMD_VSLEEP_SHIFT},
	{PMIC_RG_VSRAM_VGPU_VSLEEP, PMIC_RG_VSRAM_VGPU_VSLEEP_ADDR, PMIC_RG_VSRAM_VGPU_VSLEEP_MASK,
	 PMIC_RG_VSRAM_VGPU_VSLEEP_SHIFT},
	{PMIC_RG_VCORE_SLEEP_VOLTAGE, PMIC_RG_VCORE_SLEEP_VOLTAGE_ADDR,
	 PMIC_RG_VCORE_SLEEP_VOLTAGE_MASK, PMIC_RG_VCORE_SLEEP_VOLTAGE_SHIFT},
	{PMIC_RG_VDRAM_SLEEP_VOLTAGE, PMIC_RG_VDRAM_SLEEP_VOLTAGE_ADDR,
	 PMIC_RG_VDRAM_SLEEP_VOLTAGE_MASK, PMIC_RG_VDRAM_SLEEP_VOLTAGE_SHIFT},
	{PMIC_RG_VMODEM_SLEEP_VOLTAGE, PMIC_RG_VMODEM_SLEEP_VOLTAGE_ADDR,
	 PMIC_RG_VMODEM_SLEEP_VOLTAGE_MASK, PMIC_RG_VMODEM_SLEEP_VOLTAGE_SHIFT},
	{PMIC_RG_VMD1_SLEEP_VOLTAGE, PMIC_RG_VMD1_SLEEP_VOLTAGE_ADDR,
	 PMIC_RG_VMD1_SLEEP_VOLTAGE_MASK, PMIC_RG_VMD1_SLEEP_VOLTAGE_SHIFT},
	{PMIC_RG_VS1_SLEEP_VOLTAGE, PMIC_RG_VS1_SLEEP_VOLTAGE_ADDR, PMIC_RG_VS1_SLEEP_VOLTAGE_MASK,
	 PMIC_RG_VS1_SLEEP_VOLTAGE_SHIFT},
	{PMIC_RG_VS2_SLEEP_VOLTAGE, PMIC_RG_VS2_SLEEP_VOLTAGE_ADDR, PMIC_RG_VS2_SLEEP_VOLTAGE_MASK,
	 PMIC_RG_VS2_SLEEP_VOLTAGE_SHIFT},
	{PMIC_RG_VSRAM_DVFS1_SLEEP_VOLTAGE, PMIC_RG_VSRAM_DVFS1_SLEEP_VOLTAGE_ADDR,
	 PMIC_RG_VSRAM_DVFS1_SLEEP_VOLTAGE_MASK, PMIC_RG_VSRAM_DVFS1_SLEEP_VOLTAGE_SHIFT},
	{PMIC_RG_VSRAM_DVFS2_SLEEP_VOLTAGE, PMIC_RG_VSRAM_DVFS2_SLEEP_VOLTAGE_ADDR,
	 PMIC_RG_VSRAM_DVFS2_SLEEP_VOLTAGE_MASK, PMIC_RG_VSRAM_DVFS2_SLEEP_VOLTAGE_SHIFT},
	{PMIC_RG_VSRAM_VCORE_SLEEP_VOLTAGE, PMIC_RG_VSRAM_VCORE_SLEEP_VOLTAGE_ADDR,
	 PMIC_RG_VSRAM_VCORE_SLEEP_VOLTAGE_MASK, PMIC_RG_VSRAM_VCORE_SLEEP_VOLTAGE_SHIFT},
	{PMIC_RG_VSRAM_VMD_SLEEP_VOLTAGE, PMIC_RG_VSRAM_VMD_SLEEP_VOLTAGE_ADDR,
	 PMIC_RG_VSRAM_VMD_SLEEP_VOLTAGE_MASK, PMIC_RG_VSRAM_VMD_SLEEP_VOLTAGE_SHIFT},
	{PMIC_RG_VSRAM_VGPU_SLEEP_VOLTAGE, PMIC_RG_VSRAM_VGPU_SLEEP_VOLTAGE_ADDR,
	 PMIC_RG_VSRAM_VGPU_SLEEP_VOLTAGE_MASK, PMIC_RG_VSRAM_VGPU_SLEEP_VOLTAGE_SHIFT},
	{PMIC_RG_VOUTDET_EN, PMIC_RG_VOUTDET_EN_ADDR, PMIC_RG_VOUTDET_EN_MASK,
	 PMIC_RG_VOUTDET_EN_SHIFT},
	{PMIC_RG_SMPS_IVGD_DET, PMIC_RG_SMPS_IVGD_DET_ADDR, PMIC_RG_SMPS_IVGD_DET_MASK,
	 PMIC_RG_SMPS_IVGD_DET_SHIFT},
	{PMIC_RG_VS2_TRIMH, PMIC_RG_VS2_TRIMH_ADDR, PMIC_RG_VS2_TRIMH_MASK,
	 PMIC_RG_VS2_TRIMH_SHIFT},
	{PMIC_RG_VCORE_MODESET, PMIC_RG_VCORE_MODESET_ADDR, PMIC_RG_VCORE_MODESET_MASK,
	 PMIC_RG_VCORE_MODESET_SHIFT},
	{PMIC_RG_VCORE_NDIS_EN, PMIC_RG_VCORE_NDIS_EN_ADDR, PMIC_RG_VCORE_NDIS_EN_MASK,
	 PMIC_RG_VCORE_NDIS_EN_SHIFT},
	{PMIC_RG_VCORE_VRF18_SSTART_EN, PMIC_RG_VCORE_VRF18_SSTART_EN_ADDR,
	 PMIC_RG_VCORE_VRF18_SSTART_EN_MASK, PMIC_RG_VCORE_VRF18_SSTART_EN_SHIFT},
	{PMIC_RG_VCORE_AUTO_MODE, PMIC_RG_VCORE_AUTO_MODE_ADDR, PMIC_RG_VCORE_AUTO_MODE_MASK,
	 PMIC_RG_VCORE_AUTO_MODE_SHIFT},
	{PMIC_RG_VCORE_RZSEL0, PMIC_RG_VCORE_RZSEL0_ADDR, PMIC_RG_VCORE_RZSEL0_MASK,
	 PMIC_RG_VCORE_RZSEL0_SHIFT},
	{PMIC_RG_VCORE_RZSEL1, PMIC_RG_VCORE_RZSEL1_ADDR, PMIC_RG_VCORE_RZSEL1_MASK,
	 PMIC_RG_VCORE_RZSEL1_SHIFT},
	{PMIC_RG_VCORE_CCSEL0, PMIC_RG_VCORE_CCSEL0_ADDR, PMIC_RG_VCORE_CCSEL0_MASK,
	 PMIC_RG_VCORE_CCSEL0_SHIFT},
	{PMIC_RG_VCORE_CCSEL1, PMIC_RG_VCORE_CCSEL1_ADDR, PMIC_RG_VCORE_CCSEL1_MASK,
	 PMIC_RG_VCORE_CCSEL1_SHIFT},
	{PMIC_RG_VCORE_CSL, PMIC_RG_VCORE_CSL_ADDR, PMIC_RG_VCORE_CSL_MASK,
	 PMIC_RG_VCORE_CSL_SHIFT},
	{PMIC_RG_VCORE_SLP, PMIC_RG_VCORE_SLP_ADDR, PMIC_RG_VCORE_SLP_MASK,
	 PMIC_RG_VCORE_SLP_SHIFT},
	{PMIC_RG_VCORE_ADRC_FEN, PMIC_RG_VCORE_ADRC_FEN_ADDR, PMIC_RG_VCORE_ADRC_FEN_MASK,
	 PMIC_RG_VCORE_ADRC_FEN_SHIFT},
	{PMIC_RG_VCORE_VCCAP_CLAMP_FEN, PMIC_RG_VCORE_VCCAP_CLAMP_FEN_ADDR,
	 PMIC_RG_VCORE_VCCAP_CLAMP_FEN_MASK, PMIC_RG_VCORE_VCCAP_CLAMP_FEN_SHIFT},
	{PMIC_RG_VCORE_VC_CLAMP_FEN, PMIC_RG_VCORE_VC_CLAMP_FEN_ADDR,
	 PMIC_RG_VCORE_VC_CLAMP_FEN_MASK, PMIC_RG_VCORE_VC_CLAMP_FEN_SHIFT},
	{PMIC_RG_VCORE_PREOC_SEL, PMIC_RG_VCORE_PREOC_SEL_ADDR, PMIC_RG_VCORE_PREOC_SEL_MASK,
	 PMIC_RG_VCORE_PREOC_SEL_SHIFT},
	{PMIC_RG_VCORE_PFMOC, PMIC_RG_VCORE_PFMOC_ADDR, PMIC_RG_VCORE_PFMOC_MASK,
	 PMIC_RG_VCORE_PFMOC_SHIFT},
	{PMIC_RG_VCORE_CSR, PMIC_RG_VCORE_CSR_ADDR, PMIC_RG_VCORE_CSR_MASK,
	 PMIC_RG_VCORE_CSR_SHIFT},
	{PMIC_RG_VCORE_ZXOS_TRIM, PMIC_RG_VCORE_ZXOS_TRIM_ADDR, PMIC_RG_VCORE_ZXOS_TRIM_MASK,
	 PMIC_RG_VCORE_ZXOS_TRIM_SHIFT},
	{PMIC_RG_VCORE_CSM_N, PMIC_RG_VCORE_CSM_N_ADDR, PMIC_RG_VCORE_CSM_N_MASK,
	 PMIC_RG_VCORE_CSM_N_SHIFT},
	{PMIC_RG_VCORE_CSM_P, PMIC_RG_VCORE_CSM_P_ADDR, PMIC_RG_VCORE_CSM_P_MASK,
	 PMIC_RG_VCORE_CSM_P_SHIFT},
	{PMIC_RG_VCORE_PFMSR_EH, PMIC_RG_VCORE_PFMSR_EH_ADDR, PMIC_RG_VCORE_PFMSR_EH_MASK,
	 PMIC_RG_VCORE_PFMSR_EH_SHIFT},
	{PMIC_RG_VCORE_NLIM_GATING, PMIC_RG_VCORE_NLIM_GATING_ADDR, PMIC_RG_VCORE_NLIM_GATING_MASK,
	 PMIC_RG_VCORE_NLIM_GATING_SHIFT},
	{PMIC_RG_VCORE_PWRSR_EH, PMIC_RG_VCORE_PWRSR_EH_ADDR, PMIC_RG_VCORE_PWRSR_EH_MASK,
	 PMIC_RG_VCORE_PWRSR_EH_SHIFT},
	{PMIC_RG_VCORE_HS_VTHDET, PMIC_RG_VCORE_HS_VTHDET_ADDR, PMIC_RG_VCORE_HS_VTHDET_MASK,
	 PMIC_RG_VCORE_HS_VTHDET_SHIFT},
	{PMIC_RG_VCORE_PG_GATING, PMIC_RG_VCORE_PG_GATING_ADDR, PMIC_RG_VCORE_PG_GATING_MASK,
	 PMIC_RG_VCORE_PG_GATING_SHIFT},
	{PMIC_RG_VCORE_HS_ONSPEED_EH, PMIC_RG_VCORE_HS_ONSPEED_EH_ADDR,
	 PMIC_RG_VCORE_HS_ONSPEED_EH_MASK, PMIC_RG_VCORE_HS_ONSPEED_EH_SHIFT},
	{PMIC_RG_VCORE_NLIM_TRIMMING, PMIC_RG_VCORE_NLIM_TRIMMING_ADDR,
	 PMIC_RG_VCORE_NLIM_TRIMMING_MASK, PMIC_RG_VCORE_NLIM_TRIMMING_SHIFT},
	{PMIC_RG_VCORE_DLC, PMIC_RG_VCORE_DLC_ADDR, PMIC_RG_VCORE_DLC_MASK,
	 PMIC_RG_VCORE_DLC_SHIFT},
	{PMIC_RG_VCORE_DLC_N, PMIC_RG_VCORE_DLC_N_ADDR, PMIC_RG_VCORE_DLC_N_MASK,
	 PMIC_RG_VCORE_DLC_N_SHIFT},
	{PMIC_RG_VCORE_PFM_RIP, PMIC_RG_VCORE_PFM_RIP_ADDR, PMIC_RG_VCORE_PFM_RIP_MASK,
	 PMIC_RG_VCORE_PFM_RIP_SHIFT},
	{PMIC_RG_VCORE_TRAN_BST, PMIC_RG_VCORE_TRAN_BST_ADDR, PMIC_RG_VCORE_TRAN_BST_MASK,
	 PMIC_RG_VCORE_TRAN_BST_SHIFT},
	{PMIC_RG_VCORE_DTS_ENB, PMIC_RG_VCORE_DTS_ENB_ADDR, PMIC_RG_VCORE_DTS_ENB_MASK,
	 PMIC_RG_VCORE_DTS_ENB_SHIFT},
	{PMIC_RG_VCORE_MIN_OFF, PMIC_RG_VCORE_MIN_OFF_ADDR, PMIC_RG_VCORE_MIN_OFF_MASK,
	 PMIC_RG_VCORE_MIN_OFF_SHIFT},
	{PMIC_RG_VCORE_1P35UP_SEL_EN, PMIC_RG_VCORE_1P35UP_SEL_EN_ADDR,
	 PMIC_RG_VCORE_1P35UP_SEL_EN_MASK, PMIC_RG_VCORE_1P35UP_SEL_EN_SHIFT},
	{PMIC_RG_VCORE_DLC_AUTO_MODE, PMIC_RG_VCORE_DLC_AUTO_MODE_ADDR,
	 PMIC_RG_VCORE_DLC_AUTO_MODE_MASK, PMIC_RG_VCORE_DLC_AUTO_MODE_SHIFT},
	{PMIC_RG_VCORE_DLC_SEL, PMIC_RG_VCORE_DLC_SEL_ADDR, PMIC_RG_VCORE_DLC_SEL_MASK,
	 PMIC_RG_VCORE_DLC_SEL_SHIFT},
	{PMIC_RG_VCORE_SRC_AUTO_MODE, PMIC_RG_VCORE_SRC_AUTO_MODE_ADDR,
	 PMIC_RG_VCORE_SRC_AUTO_MODE_MASK, PMIC_RG_VCORE_SRC_AUTO_MODE_SHIFT},
	{PMIC_RG_VCORE_UGP_SR, PMIC_RG_VCORE_UGP_SR_ADDR, PMIC_RG_VCORE_UGP_SR_MASK,
	 PMIC_RG_VCORE_UGP_SR_SHIFT},
	{PMIC_RG_VCORE_LGN_SR, PMIC_RG_VCORE_LGN_SR_ADDR, PMIC_RG_VCORE_LGN_SR_MASK,
	 PMIC_RG_VCORE_LGN_SR_SHIFT},
	{PMIC_RG_VCORE_UGP_SR_PFM, PMIC_RG_VCORE_UGP_SR_PFM_ADDR, PMIC_RG_VCORE_UGP_SR_PFM_MASK,
	 PMIC_RG_VCORE_UGP_SR_PFM_SHIFT},
	{PMIC_RG_VCORE_LGN_SR_PFM, PMIC_RG_VCORE_LGN_SR_PFM_ADDR, PMIC_RG_VCORE_LGN_SR_PFM_MASK,
	 PMIC_RG_VCORE_LGN_SR_PFM_SHIFT},
	{PMIC_RG_VCORE_UGD_VTHSEL, PMIC_RG_VCORE_UGD_VTHSEL_ADDR, PMIC_RG_VCORE_UGD_VTHSEL_MASK,
	 PMIC_RG_VCORE_UGD_VTHSEL_SHIFT},
	{PMIC_RG_VCORE_FNLX_SNS, PMIC_RG_VCORE_FNLX_SNS_ADDR, PMIC_RG_VCORE_FNLX_SNS_MASK,
	 PMIC_RG_VCORE_FNLX_SNS_SHIFT},
	{PMIC_RG_VCORE_VDIFF_ENLOWIQ, PMIC_RG_VCORE_VDIFF_ENLOWIQ_ADDR,
	 PMIC_RG_VCORE_VDIFF_ENLOWIQ_MASK, PMIC_RG_VCORE_VDIFF_ENLOWIQ_SHIFT},
	{PMIC_RG_VCORE_PFMOC_FWUPOFF, PMIC_RG_VCORE_PFMOC_FWUPOFF_ADDR,
	 PMIC_RG_VCORE_PFMOC_FWUPOFF_MASK, PMIC_RG_VCORE_PFMOC_FWUPOFF_SHIFT},
	{PMIC_RG_VCORE_PWFMOC_FWUPOFF, PMIC_RG_VCORE_PWFMOC_FWUPOFF_ADDR,
	 PMIC_RG_VCORE_PWFMOC_FWUPOFF_MASK, PMIC_RG_VCORE_PWFMOC_FWUPOFF_SHIFT},
	{PMIC_RG_VCORE_CP_FWUPOFF, PMIC_RG_VCORE_CP_FWUPOFF_ADDR, PMIC_RG_VCORE_CP_FWUPOFF_MASK,
	 PMIC_RG_VCORE_CP_FWUPOFF_SHIFT},
	{PMIC_RG_VCORE_ZX_GATING, PMIC_RG_VCORE_ZX_GATING_ADDR, PMIC_RG_VCORE_ZX_GATING_MASK,
	 PMIC_RG_VCORE_ZX_GATING_SHIFT},
	{PMIC_RG_VCORE_RSV, PMIC_RG_VCORE_RSV_ADDR, PMIC_RG_VCORE_RSV_MASK,
	 PMIC_RG_VCORE_RSV_SHIFT},
	{PMIC_RG_VCORE_PREOC_TRIMMING, PMIC_RG_VCORE_PREOC_TRIMMING_ADDR,
	 PMIC_RG_VCORE_PREOC_TRIMMING_MASK, PMIC_RG_VCORE_PREOC_TRIMMING_SHIFT},
	{PMIC_RG_VCORE_AZC_EN, PMIC_RG_VCORE_AZC_EN_ADDR, PMIC_RG_VCORE_AZC_EN_MASK,
	 PMIC_RG_VCORE_AZC_EN_SHIFT},
	{PMIC_RG_VCORE_AZC_DELAY, PMIC_RG_VCORE_AZC_DELAY_ADDR, PMIC_RG_VCORE_AZC_DELAY_MASK,
	 PMIC_RG_VCORE_AZC_DELAY_SHIFT},
	{PMIC_RG_VCORE_AZC_HOLD_ENB, PMIC_RG_VCORE_AZC_HOLD_ENB_ADDR,
	 PMIC_RG_VCORE_AZC_HOLD_ENB_MASK, PMIC_RG_VCORE_AZC_HOLD_ENB_SHIFT},
	{PMIC_RGS_QI_VCORE_OC_STATUS, PMIC_RGS_QI_VCORE_OC_STATUS_ADDR,
	 PMIC_RGS_QI_VCORE_OC_STATUS_MASK, PMIC_RGS_QI_VCORE_OC_STATUS_SHIFT},
	{PMIC_RGS_QI_VCORE_DIG_MON, PMIC_RGS_QI_VCORE_DIG_MON_ADDR, PMIC_RGS_QI_VCORE_DIG_MON_MASK,
	 PMIC_RGS_QI_VCORE_DIG_MON_SHIFT},
	{PMIC_RGS_VCORE_ENPWM_STATUS, PMIC_RGS_VCORE_ENPWM_STATUS_ADDR,
	 PMIC_RGS_VCORE_ENPWM_STATUS_MASK, PMIC_RGS_VCORE_ENPWM_STATUS_SHIFT},
	{PMIC_RGS_QI_VCORE_PREOC, PMIC_RGS_QI_VCORE_PREOC_ADDR, PMIC_RGS_QI_VCORE_PREOC_MASK,
	 PMIC_RGS_QI_VCORE_PREOC_SHIFT},
	{PMIC_RG_VDRAM_MODESET, PMIC_RG_VDRAM_MODESET_ADDR, PMIC_RG_VDRAM_MODESET_MASK,
	 PMIC_RG_VDRAM_MODESET_SHIFT},
	{PMIC_RG_VDRAM_NDIS_EN, PMIC_RG_VDRAM_NDIS_EN_ADDR, PMIC_RG_VDRAM_NDIS_EN_MASK,
	 PMIC_RG_VDRAM_NDIS_EN_SHIFT},
	{PMIC_RG_VDRAM_VRF18_SSTART_EN, PMIC_RG_VDRAM_VRF18_SSTART_EN_ADDR,
	 PMIC_RG_VDRAM_VRF18_SSTART_EN_MASK, PMIC_RG_VDRAM_VRF18_SSTART_EN_SHIFT},
	{PMIC_RG_VDRAM_AUTO_MODE, PMIC_RG_VDRAM_AUTO_MODE_ADDR, PMIC_RG_VDRAM_AUTO_MODE_MASK,
	 PMIC_RG_VDRAM_AUTO_MODE_SHIFT},
	{PMIC_RG_VDRAM_RZSEL0, PMIC_RG_VDRAM_RZSEL0_ADDR, PMIC_RG_VDRAM_RZSEL0_MASK,
	 PMIC_RG_VDRAM_RZSEL0_SHIFT},
	{PMIC_RG_VDRAM_RZSEL1, PMIC_RG_VDRAM_RZSEL1_ADDR, PMIC_RG_VDRAM_RZSEL1_MASK,
	 PMIC_RG_VDRAM_RZSEL1_SHIFT},
	{PMIC_RG_VDRAM_CCSEL0, PMIC_RG_VDRAM_CCSEL0_ADDR, PMIC_RG_VDRAM_CCSEL0_MASK,
	 PMIC_RG_VDRAM_CCSEL0_SHIFT},
	{PMIC_RG_VDRAM_CCSEL1, PMIC_RG_VDRAM_CCSEL1_ADDR, PMIC_RG_VDRAM_CCSEL1_MASK,
	 PMIC_RG_VDRAM_CCSEL1_SHIFT},
	{PMIC_RG_VDRAM_CSL, PMIC_RG_VDRAM_CSL_ADDR, PMIC_RG_VDRAM_CSL_MASK,
	 PMIC_RG_VDRAM_CSL_SHIFT},
	{PMIC_RG_VDRAM_SLP, PMIC_RG_VDRAM_SLP_ADDR, PMIC_RG_VDRAM_SLP_MASK,
	 PMIC_RG_VDRAM_SLP_SHIFT},
	{PMIC_RG_VDRAM_ADRC_FEN, PMIC_RG_VDRAM_ADRC_FEN_ADDR, PMIC_RG_VDRAM_ADRC_FEN_MASK,
	 PMIC_RG_VDRAM_ADRC_FEN_SHIFT},
	{PMIC_RG_VDRAM_VCCAP_CLAMP_FEN, PMIC_RG_VDRAM_VCCAP_CLAMP_FEN_ADDR,
	 PMIC_RG_VDRAM_VCCAP_CLAMP_FEN_MASK, PMIC_RG_VDRAM_VCCAP_CLAMP_FEN_SHIFT},
	{PMIC_RG_VDRAM_VC_CLAMP_FEN, PMIC_RG_VDRAM_VC_CLAMP_FEN_ADDR,
	 PMIC_RG_VDRAM_VC_CLAMP_FEN_MASK, PMIC_RG_VDRAM_VC_CLAMP_FEN_SHIFT},
	{PMIC_RG_VDRAM_PFMOC, PMIC_RG_VDRAM_PFMOC_ADDR, PMIC_RG_VDRAM_PFMOC_MASK,
	 PMIC_RG_VDRAM_PFMOC_SHIFT},
	{PMIC_RG_VDRAM_CSR, PMIC_RG_VDRAM_CSR_ADDR, PMIC_RG_VDRAM_CSR_MASK,
	 PMIC_RG_VDRAM_CSR_SHIFT},
	{PMIC_RG_VDRAM_ZXOS_TRIM, PMIC_RG_VDRAM_ZXOS_TRIM_ADDR, PMIC_RG_VDRAM_ZXOS_TRIM_MASK,
	 PMIC_RG_VDRAM_ZXOS_TRIM_SHIFT},
	{PMIC_RG_VDRAM_PFMSR_EH, PMIC_RG_VDRAM_PFMSR_EH_ADDR, PMIC_RG_VDRAM_PFMSR_EH_MASK,
	 PMIC_RG_VDRAM_PFMSR_EH_SHIFT},
	{PMIC_RG_VDRAM_NLIM_GATING, PMIC_RG_VDRAM_NLIM_GATING_ADDR, PMIC_RG_VDRAM_NLIM_GATING_MASK,
	 PMIC_RG_VDRAM_NLIM_GATING_SHIFT},
	{PMIC_RG_VDRAM_PWRSR_EH, PMIC_RG_VDRAM_PWRSR_EH_ADDR, PMIC_RG_VDRAM_PWRSR_EH_MASK,
	 PMIC_RG_VDRAM_PWRSR_EH_SHIFT},
	{PMIC_RG_VDRAM_HS_VTHDET, PMIC_RG_VDRAM_HS_VTHDET_ADDR, PMIC_RG_VDRAM_HS_VTHDET_MASK,
	 PMIC_RG_VDRAM_HS_VTHDET_SHIFT},
	{PMIC_RG_VDRAM_PG_GATING, PMIC_RG_VDRAM_PG_GATING_ADDR, PMIC_RG_VDRAM_PG_GATING_MASK,
	 PMIC_RG_VDRAM_PG_GATING_SHIFT},
	{PMIC_RG_VDRAM_HS_ONSPEED_EH, PMIC_RG_VDRAM_HS_ONSPEED_EH_ADDR,
	 PMIC_RG_VDRAM_HS_ONSPEED_EH_MASK, PMIC_RG_VDRAM_HS_ONSPEED_EH_SHIFT},
	{PMIC_RG_VDRAM_NLIM_TRIMMING, PMIC_RG_VDRAM_NLIM_TRIMMING_ADDR,
	 PMIC_RG_VDRAM_NLIM_TRIMMING_MASK, PMIC_RG_VDRAM_NLIM_TRIMMING_SHIFT},
	{PMIC_RG_VDRAM_DLC, PMIC_RG_VDRAM_DLC_ADDR, PMIC_RG_VDRAM_DLC_MASK,
	 PMIC_RG_VDRAM_DLC_SHIFT},
	{PMIC_RG_VDRAM_DLC_N, PMIC_RG_VDRAM_DLC_N_ADDR, PMIC_RG_VDRAM_DLC_N_MASK,
	 PMIC_RG_VDRAM_DLC_N_SHIFT},
	{PMIC_RG_VDRAM_PFM_RIP, PMIC_RG_VDRAM_PFM_RIP_ADDR, PMIC_RG_VDRAM_PFM_RIP_MASK,
	 PMIC_RG_VDRAM_PFM_RIP_SHIFT},
	{PMIC_RG_VDRAM_TRAN_BST, PMIC_RG_VDRAM_TRAN_BST_ADDR, PMIC_RG_VDRAM_TRAN_BST_MASK,
	 PMIC_RG_VDRAM_TRAN_BST_SHIFT},
	{PMIC_RG_VDRAM_DTS_ENB, PMIC_RG_VDRAM_DTS_ENB_ADDR, PMIC_RG_VDRAM_DTS_ENB_MASK,
	 PMIC_RG_VDRAM_DTS_ENB_SHIFT},
	{PMIC_RG_VDRAM_MIN_OFF, PMIC_RG_VDRAM_MIN_OFF_ADDR, PMIC_RG_VDRAM_MIN_OFF_MASK,
	 PMIC_RG_VDRAM_MIN_OFF_SHIFT},
	{PMIC_RG_VDRAM_1P35UP_SEL_EN, PMIC_RG_VDRAM_1P35UP_SEL_EN_ADDR,
	 PMIC_RG_VDRAM_1P35UP_SEL_EN_MASK, PMIC_RG_VDRAM_1P35UP_SEL_EN_SHIFT},
	{PMIC_RG_VDRAM_DLC_AUTO_MODE, PMIC_RG_VDRAM_DLC_AUTO_MODE_ADDR,
	 PMIC_RG_VDRAM_DLC_AUTO_MODE_MASK, PMIC_RG_VDRAM_DLC_AUTO_MODE_SHIFT},
	{PMIC_RG_VDRAM_DLC_SEL, PMIC_RG_VDRAM_DLC_SEL_ADDR, PMIC_RG_VDRAM_DLC_SEL_MASK,
	 PMIC_RG_VDRAM_DLC_SEL_SHIFT},
	{PMIC_RG_VDRAM_SRC_AUTO_MODE, PMIC_RG_VDRAM_SRC_AUTO_MODE_ADDR,
	 PMIC_RG_VDRAM_SRC_AUTO_MODE_MASK, PMIC_RG_VDRAM_SRC_AUTO_MODE_SHIFT},
	{PMIC_RG_VDRAM_UGP_SR, PMIC_RG_VDRAM_UGP_SR_ADDR, PMIC_RG_VDRAM_UGP_SR_MASK,
	 PMIC_RG_VDRAM_UGP_SR_SHIFT},
	{PMIC_RG_VDRAM_LGN_SR, PMIC_RG_VDRAM_LGN_SR_ADDR, PMIC_RG_VDRAM_LGN_SR_MASK,
	 PMIC_RG_VDRAM_LGN_SR_SHIFT},
	{PMIC_RG_VDRAM_UGP_SR_PFM, PMIC_RG_VDRAM_UGP_SR_PFM_ADDR, PMIC_RG_VDRAM_UGP_SR_PFM_MASK,
	 PMIC_RG_VDRAM_UGP_SR_PFM_SHIFT},
	{PMIC_RG_VDRAM_LGN_SR_PFM, PMIC_RG_VDRAM_LGN_SR_PFM_ADDR, PMIC_RG_VDRAM_LGN_SR_PFM_MASK,
	 PMIC_RG_VDRAM_LGN_SR_PFM_SHIFT},
	{PMIC_RG_VDRAM_UGD_VTHSEL, PMIC_RG_VDRAM_UGD_VTHSEL_ADDR, PMIC_RG_VDRAM_UGD_VTHSEL_MASK,
	 PMIC_RG_VDRAM_UGD_VTHSEL_SHIFT},
	{PMIC_RG_VDRAM_FNLX_SNS, PMIC_RG_VDRAM_FNLX_SNS_ADDR, PMIC_RG_VDRAM_FNLX_SNS_MASK,
	 PMIC_RG_VDRAM_FNLX_SNS_SHIFT},
	{PMIC_RG_VDRAM_VDIFF_ENLOWIQ, PMIC_RG_VDRAM_VDIFF_ENLOWIQ_ADDR,
	 PMIC_RG_VDRAM_VDIFF_ENLOWIQ_MASK, PMIC_RG_VDRAM_VDIFF_ENLOWIQ_SHIFT},
	{PMIC_RG_VDRAM_PFMOC_FWUPOFF, PMIC_RG_VDRAM_PFMOC_FWUPOFF_ADDR,
	 PMIC_RG_VDRAM_PFMOC_FWUPOFF_MASK, PMIC_RG_VDRAM_PFMOC_FWUPOFF_SHIFT},
	{PMIC_RG_VDRAM_PWFMOC_FWUPOFF, PMIC_RG_VDRAM_PWFMOC_FWUPOFF_ADDR,
	 PMIC_RG_VDRAM_PWFMOC_FWUPOFF_MASK, PMIC_RG_VDRAM_PWFMOC_FWUPOFF_SHIFT},
	{PMIC_RG_VDRAM_CP_FWUPOFF, PMIC_RG_VDRAM_CP_FWUPOFF_ADDR, PMIC_RG_VDRAM_CP_FWUPOFF_MASK,
	 PMIC_RG_VDRAM_CP_FWUPOFF_SHIFT},
	{PMIC_RG_VDRAM_ZX_GATING, PMIC_RG_VDRAM_ZX_GATING_ADDR, PMIC_RG_VDRAM_ZX_GATING_MASK,
	 PMIC_RG_VDRAM_ZX_GATING_SHIFT},
	{PMIC_RG_VDRAM_RSV, PMIC_RG_VDRAM_RSV_ADDR, PMIC_RG_VDRAM_RSV_MASK,
	 PMIC_RG_VDRAM_RSV_SHIFT},
	{PMIC_RG_VDRAM_AZC_EN, PMIC_RG_VDRAM_AZC_EN_ADDR, PMIC_RG_VDRAM_AZC_EN_MASK,
	 PMIC_RG_VDRAM_AZC_EN_SHIFT},
	{PMIC_RG_VDRAM_AZC_DELAY, PMIC_RG_VDRAM_AZC_DELAY_ADDR, PMIC_RG_VDRAM_AZC_DELAY_MASK,
	 PMIC_RG_VDRAM_AZC_DELAY_SHIFT},
	{PMIC_RG_VDRAM_AZC_HOLD_ENB, PMIC_RG_VDRAM_AZC_HOLD_ENB_ADDR,
	 PMIC_RG_VDRAM_AZC_HOLD_ENB_MASK, PMIC_RG_VDRAM_AZC_HOLD_ENB_SHIFT},
	{PMIC_RGS_QI_VDRAM_OC_STATUS, PMIC_RGS_QI_VDRAM_OC_STATUS_ADDR,
	 PMIC_RGS_QI_VDRAM_OC_STATUS_MASK, PMIC_RGS_QI_VDRAM_OC_STATUS_SHIFT},
	{PMIC_RGS_QI_VDRAM_DIG_MON, PMIC_RGS_QI_VDRAM_DIG_MON_ADDR, PMIC_RGS_QI_VDRAM_DIG_MON_MASK,
	 PMIC_RGS_QI_VDRAM_DIG_MON_SHIFT},
	{PMIC_RGS_VDRAM_ENPWM_STATUS, PMIC_RGS_VDRAM_ENPWM_STATUS_ADDR,
	 PMIC_RGS_VDRAM_ENPWM_STATUS_MASK, PMIC_RGS_VDRAM_ENPWM_STATUS_SHIFT},
	{PMIC_RG_VMODEM_MODESET, PMIC_RG_VMODEM_MODESET_ADDR, PMIC_RG_VMODEM_MODESET_MASK,
	 PMIC_RG_VMODEM_MODESET_SHIFT},
	{PMIC_RG_VMODEM_NDIS_EN, PMIC_RG_VMODEM_NDIS_EN_ADDR, PMIC_RG_VMODEM_NDIS_EN_MASK,
	 PMIC_RG_VMODEM_NDIS_EN_SHIFT},
	{PMIC_RG_VMODEM_VRF18_SSTART_EN, PMIC_RG_VMODEM_VRF18_SSTART_EN_ADDR,
	 PMIC_RG_VMODEM_VRF18_SSTART_EN_MASK, PMIC_RG_VMODEM_VRF18_SSTART_EN_SHIFT},
	{PMIC_RG_VMODEM_AUTO_MODE, PMIC_RG_VMODEM_AUTO_MODE_ADDR, PMIC_RG_VMODEM_AUTO_MODE_MASK,
	 PMIC_RG_VMODEM_AUTO_MODE_SHIFT},
	{PMIC_RG_VMODEM_RZSEL0, PMIC_RG_VMODEM_RZSEL0_ADDR, PMIC_RG_VMODEM_RZSEL0_MASK,
	 PMIC_RG_VMODEM_RZSEL0_SHIFT},
	{PMIC_RG_VMODEM_RZSEL1, PMIC_RG_VMODEM_RZSEL1_ADDR, PMIC_RG_VMODEM_RZSEL1_MASK,
	 PMIC_RG_VMODEM_RZSEL1_SHIFT},
	{PMIC_RG_VMODEM_CCSEL0, PMIC_RG_VMODEM_CCSEL0_ADDR, PMIC_RG_VMODEM_CCSEL0_MASK,
	 PMIC_RG_VMODEM_CCSEL0_SHIFT},
	{PMIC_RG_VMODEM_CCSEL1, PMIC_RG_VMODEM_CCSEL1_ADDR, PMIC_RG_VMODEM_CCSEL1_MASK,
	 PMIC_RG_VMODEM_CCSEL1_SHIFT},
	{PMIC_RG_VMODEM_CSL, PMIC_RG_VMODEM_CSL_ADDR, PMIC_RG_VMODEM_CSL_MASK,
	 PMIC_RG_VMODEM_CSL_SHIFT},
	{PMIC_RG_VMODEM_SLP, PMIC_RG_VMODEM_SLP_ADDR, PMIC_RG_VMODEM_SLP_MASK,
	 PMIC_RG_VMODEM_SLP_SHIFT},
	{PMIC_RG_VMODEM_ADRC_FEN, PMIC_RG_VMODEM_ADRC_FEN_ADDR, PMIC_RG_VMODEM_ADRC_FEN_MASK,
	 PMIC_RG_VMODEM_ADRC_FEN_SHIFT},
	{PMIC_RG_VMODEM_VCCAP_CLAMP_FEN, PMIC_RG_VMODEM_VCCAP_CLAMP_FEN_ADDR,
	 PMIC_RG_VMODEM_VCCAP_CLAMP_FEN_MASK, PMIC_RG_VMODEM_VCCAP_CLAMP_FEN_SHIFT},
	{PMIC_RG_VMODEM_VC_CLAMP_FEN, PMIC_RG_VMODEM_VC_CLAMP_FEN_ADDR,
	 PMIC_RG_VMODEM_VC_CLAMP_FEN_MASK, PMIC_RG_VMODEM_VC_CLAMP_FEN_SHIFT},
	{PMIC_RG_VMODEM_PFMOC, PMIC_RG_VMODEM_PFMOC_ADDR, PMIC_RG_VMODEM_PFMOC_MASK,
	 PMIC_RG_VMODEM_PFMOC_SHIFT},
	{PMIC_RG_VMODEM_CSR, PMIC_RG_VMODEM_CSR_ADDR, PMIC_RG_VMODEM_CSR_MASK,
	 PMIC_RG_VMODEM_CSR_SHIFT},
	{PMIC_RG_VMODEM_ZXOS_TRIM, PMIC_RG_VMODEM_ZXOS_TRIM_ADDR, PMIC_RG_VMODEM_ZXOS_TRIM_MASK,
	 PMIC_RG_VMODEM_ZXOS_TRIM_SHIFT},
	{PMIC_RG_VMODEM_PFMSR_EH, PMIC_RG_VMODEM_PFMSR_EH_ADDR, PMIC_RG_VMODEM_PFMSR_EH_MASK,
	 PMIC_RG_VMODEM_PFMSR_EH_SHIFT},
	{PMIC_RG_VMODEM_NLIM_GATING, PMIC_RG_VMODEM_NLIM_GATING_ADDR,
	 PMIC_RG_VMODEM_NLIM_GATING_MASK, PMIC_RG_VMODEM_NLIM_GATING_SHIFT},
	{PMIC_RG_VMODEM_PWRSR_EH, PMIC_RG_VMODEM_PWRSR_EH_ADDR, PMIC_RG_VMODEM_PWRSR_EH_MASK,
	 PMIC_RG_VMODEM_PWRSR_EH_SHIFT},
	{PMIC_RG_VMODEM_HS_VTHDET, PMIC_RG_VMODEM_HS_VTHDET_ADDR, PMIC_RG_VMODEM_HS_VTHDET_MASK,
	 PMIC_RG_VMODEM_HS_VTHDET_SHIFT},
	{PMIC_RG_VMODEM_PG_GATING, PMIC_RG_VMODEM_PG_GATING_ADDR, PMIC_RG_VMODEM_PG_GATING_MASK,
	 PMIC_RG_VMODEM_PG_GATING_SHIFT},
	{PMIC_RG_VMODEM_HS_ONSPEED_EH, PMIC_RG_VMODEM_HS_ONSPEED_EH_ADDR,
	 PMIC_RG_VMODEM_HS_ONSPEED_EH_MASK, PMIC_RG_VMODEM_HS_ONSPEED_EH_SHIFT},
	{PMIC_RG_VMODEM_NLIM_TRIMMING, PMIC_RG_VMODEM_NLIM_TRIMMING_ADDR,
	 PMIC_RG_VMODEM_NLIM_TRIMMING_MASK, PMIC_RG_VMODEM_NLIM_TRIMMING_SHIFT},
	{PMIC_RG_VMODEM_DLC, PMIC_RG_VMODEM_DLC_ADDR, PMIC_RG_VMODEM_DLC_MASK,
	 PMIC_RG_VMODEM_DLC_SHIFT},
	{PMIC_RG_VMODEM_DLC_N, PMIC_RG_VMODEM_DLC_N_ADDR, PMIC_RG_VMODEM_DLC_N_MASK,
	 PMIC_RG_VMODEM_DLC_N_SHIFT},
	{PMIC_RG_VMODEM_PFM_RIP, PMIC_RG_VMODEM_PFM_RIP_ADDR, PMIC_RG_VMODEM_PFM_RIP_MASK,
	 PMIC_RG_VMODEM_PFM_RIP_SHIFT},
	{PMIC_RG_VMODEM_TRAN_BST, PMIC_RG_VMODEM_TRAN_BST_ADDR, PMIC_RG_VMODEM_TRAN_BST_MASK,
	 PMIC_RG_VMODEM_TRAN_BST_SHIFT},
	{PMIC_RG_VMODEM_DTS_ENB, PMIC_RG_VMODEM_DTS_ENB_ADDR, PMIC_RG_VMODEM_DTS_ENB_MASK,
	 PMIC_RG_VMODEM_DTS_ENB_SHIFT},
	{PMIC_RG_VMODEM_MIN_OFF, PMIC_RG_VMODEM_MIN_OFF_ADDR, PMIC_RG_VMODEM_MIN_OFF_MASK,
	 PMIC_RG_VMODEM_MIN_OFF_SHIFT},
	{PMIC_RG_VMODEM_1P35UP_SEL_EN, PMIC_RG_VMODEM_1P35UP_SEL_EN_ADDR,
	 PMIC_RG_VMODEM_1P35UP_SEL_EN_MASK, PMIC_RG_VMODEM_1P35UP_SEL_EN_SHIFT},
	{PMIC_RG_VMODEM_DLC_AUTO_MODE, PMIC_RG_VMODEM_DLC_AUTO_MODE_ADDR,
	 PMIC_RG_VMODEM_DLC_AUTO_MODE_MASK, PMIC_RG_VMODEM_DLC_AUTO_MODE_SHIFT},
	{PMIC_RG_VMODEM_DLC_SEL, PMIC_RG_VMODEM_DLC_SEL_ADDR, PMIC_RG_VMODEM_DLC_SEL_MASK,
	 PMIC_RG_VMODEM_DLC_SEL_SHIFT},
	{PMIC_RG_VMODEM_SRC_AUTO_MODE, PMIC_RG_VMODEM_SRC_AUTO_MODE_ADDR,
	 PMIC_RG_VMODEM_SRC_AUTO_MODE_MASK, PMIC_RG_VMODEM_SRC_AUTO_MODE_SHIFT},
	{PMIC_RG_VMODEM_UGP_SR, PMIC_RG_VMODEM_UGP_SR_ADDR, PMIC_RG_VMODEM_UGP_SR_MASK,
	 PMIC_RG_VMODEM_UGP_SR_SHIFT},
	{PMIC_RG_VMODEM_LGN_SR, PMIC_RG_VMODEM_LGN_SR_ADDR, PMIC_RG_VMODEM_LGN_SR_MASK,
	 PMIC_RG_VMODEM_LGN_SR_SHIFT},
	{PMIC_RG_VMODEM_UGP_SR_PFM, PMIC_RG_VMODEM_UGP_SR_PFM_ADDR, PMIC_RG_VMODEM_UGP_SR_PFM_MASK,
	 PMIC_RG_VMODEM_UGP_SR_PFM_SHIFT},
	{PMIC_RG_VMODEM_LGN_SR_PFM, PMIC_RG_VMODEM_LGN_SR_PFM_ADDR, PMIC_RG_VMODEM_LGN_SR_PFM_MASK,
	 PMIC_RG_VMODEM_LGN_SR_PFM_SHIFT},
	{PMIC_RG_VMODEM_UGD_VTHSEL, PMIC_RG_VMODEM_UGD_VTHSEL_ADDR, PMIC_RG_VMODEM_UGD_VTHSEL_MASK,
	 PMIC_RG_VMODEM_UGD_VTHSEL_SHIFT},
	{PMIC_RG_VMODEM_FNLX_SNS, PMIC_RG_VMODEM_FNLX_SNS_ADDR, PMIC_RG_VMODEM_FNLX_SNS_MASK,
	 PMIC_RG_VMODEM_FNLX_SNS_SHIFT},
	{PMIC_RG_VMODEM_VDIFF_ENLOWIQ, PMIC_RG_VMODEM_VDIFF_ENLOWIQ_ADDR,
	 PMIC_RG_VMODEM_VDIFF_ENLOWIQ_MASK, PMIC_RG_VMODEM_VDIFF_ENLOWIQ_SHIFT},
	{PMIC_RG_VMODEM_PFMOC_FWUPOFF, PMIC_RG_VMODEM_PFMOC_FWUPOFF_ADDR,
	 PMIC_RG_VMODEM_PFMOC_FWUPOFF_MASK, PMIC_RG_VMODEM_PFMOC_FWUPOFF_SHIFT},
	{PMIC_RG_VMODEM_PWFMOC_FWUPOFF, PMIC_RG_VMODEM_PWFMOC_FWUPOFF_ADDR,
	 PMIC_RG_VMODEM_PWFMOC_FWUPOFF_MASK, PMIC_RG_VMODEM_PWFMOC_FWUPOFF_SHIFT},
	{PMIC_RG_VMODEM_CP_FWUPOFF, PMIC_RG_VMODEM_CP_FWUPOFF_ADDR, PMIC_RG_VMODEM_CP_FWUPOFF_MASK,
	 PMIC_RG_VMODEM_CP_FWUPOFF_SHIFT},
	{PMIC_RG_VMODEM_ZX_GATING, PMIC_RG_VMODEM_ZX_GATING_ADDR, PMIC_RG_VMODEM_ZX_GATING_MASK,
	 PMIC_RG_VMODEM_ZX_GATING_SHIFT},
	{PMIC_RG_VMODEM_RSV, PMIC_RG_VMODEM_RSV_ADDR, PMIC_RG_VMODEM_RSV_MASK,
	 PMIC_RG_VMODEM_RSV_SHIFT},
	{PMIC_RG_VMODEM_AZC_EN, PMIC_RG_VMODEM_AZC_EN_ADDR, PMIC_RG_VMODEM_AZC_EN_MASK,
	 PMIC_RG_VMODEM_AZC_EN_SHIFT},
	{PMIC_RG_VMODEM_AZC_DELAY, PMIC_RG_VMODEM_AZC_DELAY_ADDR, PMIC_RG_VMODEM_AZC_DELAY_MASK,
	 PMIC_RG_VMODEM_AZC_DELAY_SHIFT},
	{PMIC_RG_VMODEM_AZC_HOLD_ENB, PMIC_RG_VMODEM_AZC_HOLD_ENB_ADDR,
	 PMIC_RG_VMODEM_AZC_HOLD_ENB_MASK, PMIC_RG_VMODEM_AZC_HOLD_ENB_SHIFT},
	{PMIC_RGS_QI_VMODEM_OC_STATUS, PMIC_RGS_QI_VMODEM_OC_STATUS_ADDR,
	 PMIC_RGS_QI_VMODEM_OC_STATUS_MASK, PMIC_RGS_QI_VMODEM_OC_STATUS_SHIFT},
	{PMIC_RGS_QI_VMODEM_DIG_MON, PMIC_RGS_QI_VMODEM_DIG_MON_ADDR,
	 PMIC_RGS_QI_VMODEM_DIG_MON_MASK, PMIC_RGS_QI_VMODEM_DIG_MON_SHIFT},
	{PMIC_RGS_VMODEM_ENPWM_STATUS, PMIC_RGS_VMODEM_ENPWM_STATUS_ADDR,
	 PMIC_RGS_VMODEM_ENPWM_STATUS_MASK, PMIC_RGS_VMODEM_ENPWM_STATUS_SHIFT},
	{PMIC_RG_VMD1_MODESET, PMIC_RG_VMD1_MODESET_ADDR, PMIC_RG_VMD1_MODESET_MASK,
	 PMIC_RG_VMD1_MODESET_SHIFT},
	{PMIC_RG_VMD1_NDIS_EN, PMIC_RG_VMD1_NDIS_EN_ADDR, PMIC_RG_VMD1_NDIS_EN_MASK,
	 PMIC_RG_VMD1_NDIS_EN_SHIFT},
	{PMIC_RG_VMD1_VRF18_SSTART_EN, PMIC_RG_VMD1_VRF18_SSTART_EN_ADDR,
	 PMIC_RG_VMD1_VRF18_SSTART_EN_MASK, PMIC_RG_VMD1_VRF18_SSTART_EN_SHIFT},
	{PMIC_RG_VMD1_AUTO_MODE, PMIC_RG_VMD1_AUTO_MODE_ADDR, PMIC_RG_VMD1_AUTO_MODE_MASK,
	 PMIC_RG_VMD1_AUTO_MODE_SHIFT},
	{PMIC_RG_VMD1_RZSEL0, PMIC_RG_VMD1_RZSEL0_ADDR, PMIC_RG_VMD1_RZSEL0_MASK,
	 PMIC_RG_VMD1_RZSEL0_SHIFT},
	{PMIC_RG_VMD1_RZSEL1, PMIC_RG_VMD1_RZSEL1_ADDR, PMIC_RG_VMD1_RZSEL1_MASK,
	 PMIC_RG_VMD1_RZSEL1_SHIFT},
	{PMIC_RG_VMD1_CCSEL0, PMIC_RG_VMD1_CCSEL0_ADDR, PMIC_RG_VMD1_CCSEL0_MASK,
	 PMIC_RG_VMD1_CCSEL0_SHIFT},
	{PMIC_RG_VMD1_CCSEL1, PMIC_RG_VMD1_CCSEL1_ADDR, PMIC_RG_VMD1_CCSEL1_MASK,
	 PMIC_RG_VMD1_CCSEL1_SHIFT},
	{PMIC_RG_VMD1_CSL, PMIC_RG_VMD1_CSL_ADDR, PMIC_RG_VMD1_CSL_MASK, PMIC_RG_VMD1_CSL_SHIFT},
	{PMIC_RG_VMD1_SLP, PMIC_RG_VMD1_SLP_ADDR, PMIC_RG_VMD1_SLP_MASK, PMIC_RG_VMD1_SLP_SHIFT},
	{PMIC_RG_VMD1_ADRC_FEN, PMIC_RG_VMD1_ADRC_FEN_ADDR, PMIC_RG_VMD1_ADRC_FEN_MASK,
	 PMIC_RG_VMD1_ADRC_FEN_SHIFT},
	{PMIC_RG_VMD1_VCCAP_CLAMP_FEN, PMIC_RG_VMD1_VCCAP_CLAMP_FEN_ADDR,
	 PMIC_RG_VMD1_VCCAP_CLAMP_FEN_MASK, PMIC_RG_VMD1_VCCAP_CLAMP_FEN_SHIFT},
	{PMIC_RG_VMD1_VC_CLAMP_FEN, PMIC_RG_VMD1_VC_CLAMP_FEN_ADDR, PMIC_RG_VMD1_VC_CLAMP_FEN_MASK,
	 PMIC_RG_VMD1_VC_CLAMP_FEN_SHIFT},
	{PMIC_RG_VMD1_PFMOC, PMIC_RG_VMD1_PFMOC_ADDR, PMIC_RG_VMD1_PFMOC_MASK,
	 PMIC_RG_VMD1_PFMOC_SHIFT},
	{PMIC_RG_VMD1_CSR, PMIC_RG_VMD1_CSR_ADDR, PMIC_RG_VMD1_CSR_MASK, PMIC_RG_VMD1_CSR_SHIFT},
	{PMIC_RG_VMD1_ZXOS_TRIM, PMIC_RG_VMD1_ZXOS_TRIM_ADDR, PMIC_RG_VMD1_ZXOS_TRIM_MASK,
	 PMIC_RG_VMD1_ZXOS_TRIM_SHIFT},
	{PMIC_RG_VMD1_PFMSR_EH, PMIC_RG_VMD1_PFMSR_EH_ADDR, PMIC_RG_VMD1_PFMSR_EH_MASK,
	 PMIC_RG_VMD1_PFMSR_EH_SHIFT},
	{PMIC_RG_VMD1_NLIM_GATING, PMIC_RG_VMD1_NLIM_GATING_ADDR, PMIC_RG_VMD1_NLIM_GATING_MASK,
	 PMIC_RG_VMD1_NLIM_GATING_SHIFT},
	{PMIC_RG_VMD1_PWRSR_EH, PMIC_RG_VMD1_PWRSR_EH_ADDR, PMIC_RG_VMD1_PWRSR_EH_MASK,
	 PMIC_RG_VMD1_PWRSR_EH_SHIFT},
	{PMIC_RG_VMD1_HS_VTHDET, PMIC_RG_VMD1_HS_VTHDET_ADDR, PMIC_RG_VMD1_HS_VTHDET_MASK,
	 PMIC_RG_VMD1_HS_VTHDET_SHIFT},
	{PMIC_RG_VMD1_PG_GATING, PMIC_RG_VMD1_PG_GATING_ADDR, PMIC_RG_VMD1_PG_GATING_MASK,
	 PMIC_RG_VMD1_PG_GATING_SHIFT},
	{PMIC_RG_VMD1_HS_ONSPEED_EH, PMIC_RG_VMD1_HS_ONSPEED_EH_ADDR,
	 PMIC_RG_VMD1_HS_ONSPEED_EH_MASK, PMIC_RG_VMD1_HS_ONSPEED_EH_SHIFT},
	{PMIC_RG_VMD1_NLIM_TRIMMING, PMIC_RG_VMD1_NLIM_TRIMMING_ADDR,
	 PMIC_RG_VMD1_NLIM_TRIMMING_MASK, PMIC_RG_VMD1_NLIM_TRIMMING_SHIFT},
	{PMIC_RG_VMD1_DLC, PMIC_RG_VMD1_DLC_ADDR, PMIC_RG_VMD1_DLC_MASK, PMIC_RG_VMD1_DLC_SHIFT},
	{PMIC_RG_VMD1_DLC_N, PMIC_RG_VMD1_DLC_N_ADDR, PMIC_RG_VMD1_DLC_N_MASK,
	 PMIC_RG_VMD1_DLC_N_SHIFT},
	{PMIC_RG_VMD1_PFM_RIP, PMIC_RG_VMD1_PFM_RIP_ADDR, PMIC_RG_VMD1_PFM_RIP_MASK,
	 PMIC_RG_VMD1_PFM_RIP_SHIFT},
	{PMIC_RG_VMD1_TRAN_BST, PMIC_RG_VMD1_TRAN_BST_ADDR, PMIC_RG_VMD1_TRAN_BST_MASK,
	 PMIC_RG_VMD1_TRAN_BST_SHIFT},
	{PMIC_RG_VMD1_DTS_ENB, PMIC_RG_VMD1_DTS_ENB_ADDR, PMIC_RG_VMD1_DTS_ENB_MASK,
	 PMIC_RG_VMD1_DTS_ENB_SHIFT},
	{PMIC_RG_VMD1_MIN_OFF, PMIC_RG_VMD1_MIN_OFF_ADDR, PMIC_RG_VMD1_MIN_OFF_MASK,
	 PMIC_RG_VMD1_MIN_OFF_SHIFT},
	{PMIC_RG_VMD1_1P35UP_SEL_EN, PMIC_RG_VMD1_1P35UP_SEL_EN_ADDR,
	 PMIC_RG_VMD1_1P35UP_SEL_EN_MASK, PMIC_RG_VMD1_1P35UP_SEL_EN_SHIFT},
	{PMIC_RG_VMD1_DLC_AUTO_MODE, PMIC_RG_VMD1_DLC_AUTO_MODE_ADDR,
	 PMIC_RG_VMD1_DLC_AUTO_MODE_MASK, PMIC_RG_VMD1_DLC_AUTO_MODE_SHIFT},
	{PMIC_RG_VMD1_DLC_SEL, PMIC_RG_VMD1_DLC_SEL_ADDR, PMIC_RG_VMD1_DLC_SEL_MASK,
	 PMIC_RG_VMD1_DLC_SEL_SHIFT},
	{PMIC_RG_VMD1_SRC_AUTO_MODE, PMIC_RG_VMD1_SRC_AUTO_MODE_ADDR,
	 PMIC_RG_VMD1_SRC_AUTO_MODE_MASK, PMIC_RG_VMD1_SRC_AUTO_MODE_SHIFT},
	{PMIC_RG_VMD1_UGP_SR, PMIC_RG_VMD1_UGP_SR_ADDR, PMIC_RG_VMD1_UGP_SR_MASK,
	 PMIC_RG_VMD1_UGP_SR_SHIFT},
	{PMIC_RG_VMD1_LGN_SR, PMIC_RG_VMD1_LGN_SR_ADDR, PMIC_RG_VMD1_LGN_SR_MASK,
	 PMIC_RG_VMD1_LGN_SR_SHIFT},
	{PMIC_RG_VMD1_UGP_SR_PFM, PMIC_RG_VMD1_UGP_SR_PFM_ADDR, PMIC_RG_VMD1_UGP_SR_PFM_MASK,
	 PMIC_RG_VMD1_UGP_SR_PFM_SHIFT},
	{PMIC_RG_VMD1_LGN_SR_PFM, PMIC_RG_VMD1_LGN_SR_PFM_ADDR, PMIC_RG_VMD1_LGN_SR_PFM_MASK,
	 PMIC_RG_VMD1_LGN_SR_PFM_SHIFT},
	{PMIC_RG_VMD1_UGD_VTHSEL, PMIC_RG_VMD1_UGD_VTHSEL_ADDR, PMIC_RG_VMD1_UGD_VTHSEL_MASK,
	 PMIC_RG_VMD1_UGD_VTHSEL_SHIFT},
	{PMIC_RG_VMD1_FNLX_SNS, PMIC_RG_VMD1_FNLX_SNS_ADDR, PMIC_RG_VMD1_FNLX_SNS_MASK,
	 PMIC_RG_VMD1_FNLX_SNS_SHIFT},
	{PMIC_RG_VMD1_VDIFF_ENLOWIQ, PMIC_RG_VMD1_VDIFF_ENLOWIQ_ADDR,
	 PMIC_RG_VMD1_VDIFF_ENLOWIQ_MASK, PMIC_RG_VMD1_VDIFF_ENLOWIQ_SHIFT},
	{PMIC_RG_VMD1_PFMOC_FWUPOFF, PMIC_RG_VMD1_PFMOC_FWUPOFF_ADDR,
	 PMIC_RG_VMD1_PFMOC_FWUPOFF_MASK, PMIC_RG_VMD1_PFMOC_FWUPOFF_SHIFT},
	{PMIC_RG_VMD1_PWFMOC_FWUPOFF, PMIC_RG_VMD1_PWFMOC_FWUPOFF_ADDR,
	 PMIC_RG_VMD1_PWFMOC_FWUPOFF_MASK, PMIC_RG_VMD1_PWFMOC_FWUPOFF_SHIFT},
	{PMIC_RG_VMD1_CP_FWUPOFF, PMIC_RG_VMD1_CP_FWUPOFF_ADDR, PMIC_RG_VMD1_CP_FWUPOFF_MASK,
	 PMIC_RG_VMD1_CP_FWUPOFF_SHIFT},
	{PMIC_RG_VMD1_ZX_GATING, PMIC_RG_VMD1_ZX_GATING_ADDR, PMIC_RG_VMD1_ZX_GATING_MASK,
	 PMIC_RG_VMD1_ZX_GATING_SHIFT},
	{PMIC_RG_VMD1_RSV, PMIC_RG_VMD1_RSV_ADDR, PMIC_RG_VMD1_RSV_MASK, PMIC_RG_VMD1_RSV_SHIFT},
	{PMIC_RG_VMD1_AZC_EN, PMIC_RG_VMD1_AZC_EN_ADDR, PMIC_RG_VMD1_AZC_EN_MASK,
	 PMIC_RG_VMD1_AZC_EN_SHIFT},
	{PMIC_RG_VMD1_AZC_DELAY, PMIC_RG_VMD1_AZC_DELAY_ADDR, PMIC_RG_VMD1_AZC_DELAY_MASK,
	 PMIC_RG_VMD1_AZC_DELAY_SHIFT},
	{PMIC_RG_VMD1_AZC_HOLD_ENB, PMIC_RG_VMD1_AZC_HOLD_ENB_ADDR, PMIC_RG_VMD1_AZC_HOLD_ENB_MASK,
	 PMIC_RG_VMD1_AZC_HOLD_ENB_SHIFT},
	{PMIC_RGS_QI_VMD1_OC_STATUS, PMIC_RGS_QI_VMD1_OC_STATUS_ADDR,
	 PMIC_RGS_QI_VMD1_OC_STATUS_MASK, PMIC_RGS_QI_VMD1_OC_STATUS_SHIFT},
	{PMIC_RGS_QI_VMD1_DIG_MON, PMIC_RGS_QI_VMD1_DIG_MON_ADDR, PMIC_RGS_QI_VMD1_DIG_MON_MASK,
	 PMIC_RGS_QI_VMD1_DIG_MON_SHIFT},
	{PMIC_RGS_VMD1_ENPWM_STATUS, PMIC_RGS_VMD1_ENPWM_STATUS_ADDR,
	 PMIC_RGS_VMD1_ENPWM_STATUS_MASK, PMIC_RGS_VMD1_ENPWM_STATUS_SHIFT},
	{PMIC_RG_VS1_MODESET, PMIC_RG_VS1_MODESET_ADDR, PMIC_RG_VS1_MODESET_MASK,
	 PMIC_RG_VS1_MODESET_SHIFT},
	{PMIC_RG_VS1_NDIS_EN, PMIC_RG_VS1_NDIS_EN_ADDR, PMIC_RG_VS1_NDIS_EN_MASK,
	 PMIC_RG_VS1_NDIS_EN_SHIFT},
	{PMIC_RG_VS1_VRF18_SSTART_EN, PMIC_RG_VS1_VRF18_SSTART_EN_ADDR,
	 PMIC_RG_VS1_VRF18_SSTART_EN_MASK, PMIC_RG_VS1_VRF18_SSTART_EN_SHIFT},
	{PMIC_RG_VS1_AUTO_MODE, PMIC_RG_VS1_AUTO_MODE_ADDR, PMIC_RG_VS1_AUTO_MODE_MASK,
	 PMIC_RG_VS1_AUTO_MODE_SHIFT},
	{PMIC_RG_VS1_RZSEL0, PMIC_RG_VS1_RZSEL0_ADDR, PMIC_RG_VS1_RZSEL0_MASK,
	 PMIC_RG_VS1_RZSEL0_SHIFT},
	{PMIC_RG_VS1_RZSEL1, PMIC_RG_VS1_RZSEL1_ADDR, PMIC_RG_VS1_RZSEL1_MASK,
	 PMIC_RG_VS1_RZSEL1_SHIFT},
	{PMIC_RG_VS1_CCSEL0, PMIC_RG_VS1_CCSEL0_ADDR, PMIC_RG_VS1_CCSEL0_MASK,
	 PMIC_RG_VS1_CCSEL0_SHIFT},
	{PMIC_RG_VS1_CCSEL1, PMIC_RG_VS1_CCSEL1_ADDR, PMIC_RG_VS1_CCSEL1_MASK,
	 PMIC_RG_VS1_CCSEL1_SHIFT},
	{PMIC_RG_VS1_CSL, PMIC_RG_VS1_CSL_ADDR, PMIC_RG_VS1_CSL_MASK, PMIC_RG_VS1_CSL_SHIFT},
	{PMIC_RG_VS1_SLP, PMIC_RG_VS1_SLP_ADDR, PMIC_RG_VS1_SLP_MASK, PMIC_RG_VS1_SLP_SHIFT},
	{PMIC_RG_VS1_ADRC_FEN, PMIC_RG_VS1_ADRC_FEN_ADDR, PMIC_RG_VS1_ADRC_FEN_MASK,
	 PMIC_RG_VS1_ADRC_FEN_SHIFT},
	{PMIC_RG_VS1_VCCAP_CLAMP_FEN, PMIC_RG_VS1_VCCAP_CLAMP_FEN_ADDR,
	 PMIC_RG_VS1_VCCAP_CLAMP_FEN_MASK, PMIC_RG_VS1_VCCAP_CLAMP_FEN_SHIFT},
	{PMIC_RG_VS1_VC_CLAMP_FEN, PMIC_RG_VS1_VC_CLAMP_FEN_ADDR, PMIC_RG_VS1_VC_CLAMP_FEN_MASK,
	 PMIC_RG_VS1_VC_CLAMP_FEN_SHIFT},
	{PMIC_RG_VS1_PFMOC, PMIC_RG_VS1_PFMOC_ADDR, PMIC_RG_VS1_PFMOC_MASK,
	 PMIC_RG_VS1_PFMOC_SHIFT},
	{PMIC_RG_VS1_CSR, PMIC_RG_VS1_CSR_ADDR, PMIC_RG_VS1_CSR_MASK, PMIC_RG_VS1_CSR_SHIFT},
	{PMIC_RG_VS1_ZXOS_TRIM, PMIC_RG_VS1_ZXOS_TRIM_ADDR, PMIC_RG_VS1_ZXOS_TRIM_MASK,
	 PMIC_RG_VS1_ZXOS_TRIM_SHIFT},
	{PMIC_RG_VS1_PFMSR_EH, PMIC_RG_VS1_PFMSR_EH_ADDR, PMIC_RG_VS1_PFMSR_EH_MASK,
	 PMIC_RG_VS1_PFMSR_EH_SHIFT},
	{PMIC_RG_VS1_NLIM_GATING, PMIC_RG_VS1_NLIM_GATING_ADDR, PMIC_RG_VS1_NLIM_GATING_MASK,
	 PMIC_RG_VS1_NLIM_GATING_SHIFT},
	{PMIC_RG_VS1_PWRSR_EH, PMIC_RG_VS1_PWRSR_EH_ADDR, PMIC_RG_VS1_PWRSR_EH_MASK,
	 PMIC_RG_VS1_PWRSR_EH_SHIFT},
	{PMIC_RG_VS1_HS_VTHDET, PMIC_RG_VS1_HS_VTHDET_ADDR, PMIC_RG_VS1_HS_VTHDET_MASK,
	 PMIC_RG_VS1_HS_VTHDET_SHIFT},
	{PMIC_RG_VS1_PG_GATING, PMIC_RG_VS1_PG_GATING_ADDR, PMIC_RG_VS1_PG_GATING_MASK,
	 PMIC_RG_VS1_PG_GATING_SHIFT},
	{PMIC_RG_VS1_HS_ONSPEED_EH, PMIC_RG_VS1_HS_ONSPEED_EH_ADDR, PMIC_RG_VS1_HS_ONSPEED_EH_MASK,
	 PMIC_RG_VS1_HS_ONSPEED_EH_SHIFT},
	{PMIC_RG_VS1_NLIM_TRIMMING, PMIC_RG_VS1_NLIM_TRIMMING_ADDR, PMIC_RG_VS1_NLIM_TRIMMING_MASK,
	 PMIC_RG_VS1_NLIM_TRIMMING_SHIFT},
	{PMIC_RG_VS1_DLC, PMIC_RG_VS1_DLC_ADDR, PMIC_RG_VS1_DLC_MASK, PMIC_RG_VS1_DLC_SHIFT},
	{PMIC_RG_VS1_DLC_N, PMIC_RG_VS1_DLC_N_ADDR, PMIC_RG_VS1_DLC_N_MASK,
	 PMIC_RG_VS1_DLC_N_SHIFT},
	{PMIC_RG_VS1_PFM_RIP, PMIC_RG_VS1_PFM_RIP_ADDR, PMIC_RG_VS1_PFM_RIP_MASK,
	 PMIC_RG_VS1_PFM_RIP_SHIFT},
	{PMIC_RG_VS1_TRAN_BST, PMIC_RG_VS1_TRAN_BST_ADDR, PMIC_RG_VS1_TRAN_BST_MASK,
	 PMIC_RG_VS1_TRAN_BST_SHIFT},
	{PMIC_RG_VS1_DTS_ENB, PMIC_RG_VS1_DTS_ENB_ADDR, PMIC_RG_VS1_DTS_ENB_MASK,
	 PMIC_RG_VS1_DTS_ENB_SHIFT},
	{PMIC_RG_VS1_MIN_OFF, PMIC_RG_VS1_MIN_OFF_ADDR, PMIC_RG_VS1_MIN_OFF_MASK,
	 PMIC_RG_VS1_MIN_OFF_SHIFT},
	{PMIC_RG_VS1_1P35UP_SEL_EN, PMIC_RG_VS1_1P35UP_SEL_EN_ADDR, PMIC_RG_VS1_1P35UP_SEL_EN_MASK,
	 PMIC_RG_VS1_1P35UP_SEL_EN_SHIFT},
	{PMIC_RG_VS1_DLC_AUTO_MODE, PMIC_RG_VS1_DLC_AUTO_MODE_ADDR, PMIC_RG_VS1_DLC_AUTO_MODE_MASK,
	 PMIC_RG_VS1_DLC_AUTO_MODE_SHIFT},
	{PMIC_RG_VS1_DLC_SEL, PMIC_RG_VS1_DLC_SEL_ADDR, PMIC_RG_VS1_DLC_SEL_MASK,
	 PMIC_RG_VS1_DLC_SEL_SHIFT},
	{PMIC_RG_VS1_SRC_AUTO_MODE, PMIC_RG_VS1_SRC_AUTO_MODE_ADDR, PMIC_RG_VS1_SRC_AUTO_MODE_MASK,
	 PMIC_RG_VS1_SRC_AUTO_MODE_SHIFT},
	{PMIC_RG_VS1_UGP_SR, PMIC_RG_VS1_UGP_SR_ADDR, PMIC_RG_VS1_UGP_SR_MASK,
	 PMIC_RG_VS1_UGP_SR_SHIFT},
	{PMIC_RG_VS1_LGN_SR, PMIC_RG_VS1_LGN_SR_ADDR, PMIC_RG_VS1_LGN_SR_MASK,
	 PMIC_RG_VS1_LGN_SR_SHIFT},
	{PMIC_RG_VS1_UGP_SR_PFM, PMIC_RG_VS1_UGP_SR_PFM_ADDR, PMIC_RG_VS1_UGP_SR_PFM_MASK,
	 PMIC_RG_VS1_UGP_SR_PFM_SHIFT},
	{PMIC_RG_VS1_LGN_SR_PFM, PMIC_RG_VS1_LGN_SR_PFM_ADDR, PMIC_RG_VS1_LGN_SR_PFM_MASK,
	 PMIC_RG_VS1_LGN_SR_PFM_SHIFT},
	{PMIC_RG_VS1_UGD_VTHSEL, PMIC_RG_VS1_UGD_VTHSEL_ADDR, PMIC_RG_VS1_UGD_VTHSEL_MASK,
	 PMIC_RG_VS1_UGD_VTHSEL_SHIFT},
	{PMIC_RG_VS1_FNLX_SNS, PMIC_RG_VS1_FNLX_SNS_ADDR, PMIC_RG_VS1_FNLX_SNS_MASK,
	 PMIC_RG_VS1_FNLX_SNS_SHIFT},
	{PMIC_RG_VS1_VDIFF_ENLOWIQ, PMIC_RG_VS1_VDIFF_ENLOWIQ_ADDR, PMIC_RG_VS1_VDIFF_ENLOWIQ_MASK,
	 PMIC_RG_VS1_VDIFF_ENLOWIQ_SHIFT},
	{PMIC_RG_VS1_PFMOC_FWUPOFF, PMIC_RG_VS1_PFMOC_FWUPOFF_ADDR, PMIC_RG_VS1_PFMOC_FWUPOFF_MASK,
	 PMIC_RG_VS1_PFMOC_FWUPOFF_SHIFT},
	{PMIC_RG_VS1_PWFMOC_FWUPOFF, PMIC_RG_VS1_PWFMOC_FWUPOFF_ADDR,
	 PMIC_RG_VS1_PWFMOC_FWUPOFF_MASK, PMIC_RG_VS1_PWFMOC_FWUPOFF_SHIFT},
	{PMIC_RG_VS1_CP_FWUPOFF, PMIC_RG_VS1_CP_FWUPOFF_ADDR, PMIC_RG_VS1_CP_FWUPOFF_MASK,
	 PMIC_RG_VS1_CP_FWUPOFF_SHIFT},
	{PMIC_RG_VS1_ZX_GATING, PMIC_RG_VS1_ZX_GATING_ADDR, PMIC_RG_VS1_ZX_GATING_MASK,
	 PMIC_RG_VS1_ZX_GATING_SHIFT},
	{PMIC_RG_VS1_RSV, PMIC_RG_VS1_RSV_ADDR, PMIC_RG_VS1_RSV_MASK, PMIC_RG_VS1_RSV_SHIFT},
	{PMIC_RG_VS1_AZC_EN, PMIC_RG_VS1_AZC_EN_ADDR, PMIC_RG_VS1_AZC_EN_MASK,
	 PMIC_RG_VS1_AZC_EN_SHIFT},
	{PMIC_RG_VS1_AZC_DELAY, PMIC_RG_VS1_AZC_DELAY_ADDR, PMIC_RG_VS1_AZC_DELAY_MASK,
	 PMIC_RG_VS1_AZC_DELAY_SHIFT},
	{PMIC_RG_VS1_AZC_HOLD_ENB, PMIC_RG_VS1_AZC_HOLD_ENB_ADDR, PMIC_RG_VS1_AZC_HOLD_ENB_MASK,
	 PMIC_RG_VS1_AZC_HOLD_ENB_SHIFT},
	{PMIC_RGS_QI_VS1_OC_STATUS, PMIC_RGS_QI_VS1_OC_STATUS_ADDR, PMIC_RGS_QI_VS1_OC_STATUS_MASK,
	 PMIC_RGS_QI_VS1_OC_STATUS_SHIFT},
	{PMIC_RGS_QI_VS1_DIG_MON, PMIC_RGS_QI_VS1_DIG_MON_ADDR, PMIC_RGS_QI_VS1_DIG_MON_MASK,
	 PMIC_RGS_QI_VS1_DIG_MON_SHIFT},
	{PMIC_RGS_VS1_ENPWM_STATUS, PMIC_RGS_VS1_ENPWM_STATUS_ADDR, PMIC_RGS_VS1_ENPWM_STATUS_MASK,
	 PMIC_RGS_VS1_ENPWM_STATUS_SHIFT},
	{PMIC_RG_VS2_MODESET, PMIC_RG_VS2_MODESET_ADDR, PMIC_RG_VS2_MODESET_MASK,
	 PMIC_RG_VS2_MODESET_SHIFT},
	{PMIC_RG_VS2_NDIS_EN, PMIC_RG_VS2_NDIS_EN_ADDR, PMIC_RG_VS2_NDIS_EN_MASK,
	 PMIC_RG_VS2_NDIS_EN_SHIFT},
	{PMIC_RG_VS2_VRF18_SSTART_EN, PMIC_RG_VS2_VRF18_SSTART_EN_ADDR,
	 PMIC_RG_VS2_VRF18_SSTART_EN_MASK, PMIC_RG_VS2_VRF18_SSTART_EN_SHIFT},
	{PMIC_RG_VS2_AUTO_MODE, PMIC_RG_VS2_AUTO_MODE_ADDR, PMIC_RG_VS2_AUTO_MODE_MASK,
	 PMIC_RG_VS2_AUTO_MODE_SHIFT},
	{PMIC_RG_VS2_RZSEL0, PMIC_RG_VS2_RZSEL0_ADDR, PMIC_RG_VS2_RZSEL0_MASK,
	 PMIC_RG_VS2_RZSEL0_SHIFT},
	{PMIC_RG_VS2_RZSEL1, PMIC_RG_VS2_RZSEL1_ADDR, PMIC_RG_VS2_RZSEL1_MASK,
	 PMIC_RG_VS2_RZSEL1_SHIFT},
	{PMIC_RG_VS2_CCSEL0, PMIC_RG_VS2_CCSEL0_ADDR, PMIC_RG_VS2_CCSEL0_MASK,
	 PMIC_RG_VS2_CCSEL0_SHIFT},
	{PMIC_RG_VS2_CCSEL1, PMIC_RG_VS2_CCSEL1_ADDR, PMIC_RG_VS2_CCSEL1_MASK,
	 PMIC_RG_VS2_CCSEL1_SHIFT},
	{PMIC_RG_VS2_CSL, PMIC_RG_VS2_CSL_ADDR, PMIC_RG_VS2_CSL_MASK, PMIC_RG_VS2_CSL_SHIFT},
	{PMIC_RG_VS2_SLP, PMIC_RG_VS2_SLP_ADDR, PMIC_RG_VS2_SLP_MASK, PMIC_RG_VS2_SLP_SHIFT},
	{PMIC_RG_VS2_ADRC_FEN, PMIC_RG_VS2_ADRC_FEN_ADDR, PMIC_RG_VS2_ADRC_FEN_MASK,
	 PMIC_RG_VS2_ADRC_FEN_SHIFT},
	{PMIC_RG_VS2_VCCAP_CLAMP_FEN, PMIC_RG_VS2_VCCAP_CLAMP_FEN_ADDR,
	 PMIC_RG_VS2_VCCAP_CLAMP_FEN_MASK, PMIC_RG_VS2_VCCAP_CLAMP_FEN_SHIFT},
	{PMIC_RG_VS2_VC_CLAMP_FEN, PMIC_RG_VS2_VC_CLAMP_FEN_ADDR, PMIC_RG_VS2_VC_CLAMP_FEN_MASK,
	 PMIC_RG_VS2_VC_CLAMP_FEN_SHIFT},
	{PMIC_RG_VS2_PFMOC, PMIC_RG_VS2_PFMOC_ADDR, PMIC_RG_VS2_PFMOC_MASK,
	 PMIC_RG_VS2_PFMOC_SHIFT},
	{PMIC_RG_VS2_CSR, PMIC_RG_VS2_CSR_ADDR, PMIC_RG_VS2_CSR_MASK, PMIC_RG_VS2_CSR_SHIFT},
	{PMIC_RG_VS2_ZXOS_TRIM, PMIC_RG_VS2_ZXOS_TRIM_ADDR, PMIC_RG_VS2_ZXOS_TRIM_MASK,
	 PMIC_RG_VS2_ZXOS_TRIM_SHIFT},
	{PMIC_RG_VS2_PFMSR_EH, PMIC_RG_VS2_PFMSR_EH_ADDR, PMIC_RG_VS2_PFMSR_EH_MASK,
	 PMIC_RG_VS2_PFMSR_EH_SHIFT},
	{PMIC_RG_VS2_NLIM_GATING, PMIC_RG_VS2_NLIM_GATING_ADDR, PMIC_RG_VS2_NLIM_GATING_MASK,
	 PMIC_RG_VS2_NLIM_GATING_SHIFT},
	{PMIC_RG_VS2_PWRSR_EH, PMIC_RG_VS2_PWRSR_EH_ADDR, PMIC_RG_VS2_PWRSR_EH_MASK,
	 PMIC_RG_VS2_PWRSR_EH_SHIFT},
	{PMIC_RG_VS2_HS_VTHDET, PMIC_RG_VS2_HS_VTHDET_ADDR, PMIC_RG_VS2_HS_VTHDET_MASK,
	 PMIC_RG_VS2_HS_VTHDET_SHIFT},
	{PMIC_RG_VS2_PG_GATING, PMIC_RG_VS2_PG_GATING_ADDR, PMIC_RG_VS2_PG_GATING_MASK,
	 PMIC_RG_VS2_PG_GATING_SHIFT},
	{PMIC_RG_VS2_HS_ONSPEED_EH, PMIC_RG_VS2_HS_ONSPEED_EH_ADDR, PMIC_RG_VS2_HS_ONSPEED_EH_MASK,
	 PMIC_RG_VS2_HS_ONSPEED_EH_SHIFT},
	{PMIC_RG_VS2_NLIM_TRIMMING, PMIC_RG_VS2_NLIM_TRIMMING_ADDR, PMIC_RG_VS2_NLIM_TRIMMING_MASK,
	 PMIC_RG_VS2_NLIM_TRIMMING_SHIFT},
	{PMIC_RG_VS2_DLC, PMIC_RG_VS2_DLC_ADDR, PMIC_RG_VS2_DLC_MASK, PMIC_RG_VS2_DLC_SHIFT},
	{PMIC_RG_VS2_DLC_N, PMIC_RG_VS2_DLC_N_ADDR, PMIC_RG_VS2_DLC_N_MASK,
	 PMIC_RG_VS2_DLC_N_SHIFT},
	{PMIC_RG_VS2_PFM_RIP, PMIC_RG_VS2_PFM_RIP_ADDR, PMIC_RG_VS2_PFM_RIP_MASK,
	 PMIC_RG_VS2_PFM_RIP_SHIFT},
	{PMIC_RG_VS2_TRAN_BST, PMIC_RG_VS2_TRAN_BST_ADDR, PMIC_RG_VS2_TRAN_BST_MASK,
	 PMIC_RG_VS2_TRAN_BST_SHIFT},
	{PMIC_RG_VS2_DTS_ENB, PMIC_RG_VS2_DTS_ENB_ADDR, PMIC_RG_VS2_DTS_ENB_MASK,
	 PMIC_RG_VS2_DTS_ENB_SHIFT},
	{PMIC_RG_VS2_MIN_OFF, PMIC_RG_VS2_MIN_OFF_ADDR, PMIC_RG_VS2_MIN_OFF_MASK,
	 PMIC_RG_VS2_MIN_OFF_SHIFT},
	{PMIC_RG_VS2_1P35UP_SEL_EN, PMIC_RG_VS2_1P35UP_SEL_EN_ADDR, PMIC_RG_VS2_1P35UP_SEL_EN_MASK,
	 PMIC_RG_VS2_1P35UP_SEL_EN_SHIFT},
	{PMIC_RG_VS2_DLC_AUTO_MODE, PMIC_RG_VS2_DLC_AUTO_MODE_ADDR, PMIC_RG_VS2_DLC_AUTO_MODE_MASK,
	 PMIC_RG_VS2_DLC_AUTO_MODE_SHIFT},
	{PMIC_RG_VS2_DLC_SEL, PMIC_RG_VS2_DLC_SEL_ADDR, PMIC_RG_VS2_DLC_SEL_MASK,
	 PMIC_RG_VS2_DLC_SEL_SHIFT},
	{PMIC_RG_VS2_SRC_AUTO_MODE, PMIC_RG_VS2_SRC_AUTO_MODE_ADDR, PMIC_RG_VS2_SRC_AUTO_MODE_MASK,
	 PMIC_RG_VS2_SRC_AUTO_MODE_SHIFT},
	{PMIC_RG_VS2_UGP_SR, PMIC_RG_VS2_UGP_SR_ADDR, PMIC_RG_VS2_UGP_SR_MASK,
	 PMIC_RG_VS2_UGP_SR_SHIFT},
	{PMIC_RG_VS2_LGN_SR, PMIC_RG_VS2_LGN_SR_ADDR, PMIC_RG_VS2_LGN_SR_MASK,
	 PMIC_RG_VS2_LGN_SR_SHIFT},
	{PMIC_RG_VS2_UGP_SR_PFM, PMIC_RG_VS2_UGP_SR_PFM_ADDR, PMIC_RG_VS2_UGP_SR_PFM_MASK,
	 PMIC_RG_VS2_UGP_SR_PFM_SHIFT},
	{PMIC_RG_VS2_LGN_SR_PFM, PMIC_RG_VS2_LGN_SR_PFM_ADDR, PMIC_RG_VS2_LGN_SR_PFM_MASK,
	 PMIC_RG_VS2_LGN_SR_PFM_SHIFT},
	{PMIC_RG_VS2_UGD_VTHSEL, PMIC_RG_VS2_UGD_VTHSEL_ADDR, PMIC_RG_VS2_UGD_VTHSEL_MASK,
	 PMIC_RG_VS2_UGD_VTHSEL_SHIFT},
	{PMIC_RG_VS2_FNLX_SNS, PMIC_RG_VS2_FNLX_SNS_ADDR, PMIC_RG_VS2_FNLX_SNS_MASK,
	 PMIC_RG_VS2_FNLX_SNS_SHIFT},
	{PMIC_RG_VS2_VDIFF_ENLOWIQ, PMIC_RG_VS2_VDIFF_ENLOWIQ_ADDR, PMIC_RG_VS2_VDIFF_ENLOWIQ_MASK,
	 PMIC_RG_VS2_VDIFF_ENLOWIQ_SHIFT},
	{PMIC_RG_VS2_PFMOC_FWUPOFF, PMIC_RG_VS2_PFMOC_FWUPOFF_ADDR, PMIC_RG_VS2_PFMOC_FWUPOFF_MASK,
	 PMIC_RG_VS2_PFMOC_FWUPOFF_SHIFT},
	{PMIC_RG_VS2_PWFMOC_FWUPOFF, PMIC_RG_VS2_PWFMOC_FWUPOFF_ADDR,
	 PMIC_RG_VS2_PWFMOC_FWUPOFF_MASK, PMIC_RG_VS2_PWFMOC_FWUPOFF_SHIFT},
	{PMIC_RG_VS2_CP_FWUPOFF, PMIC_RG_VS2_CP_FWUPOFF_ADDR, PMIC_RG_VS2_CP_FWUPOFF_MASK,
	 PMIC_RG_VS2_CP_FWUPOFF_SHIFT},
	{PMIC_RG_VS2_ZX_GATING, PMIC_RG_VS2_ZX_GATING_ADDR, PMIC_RG_VS2_ZX_GATING_MASK,
	 PMIC_RG_VS2_ZX_GATING_SHIFT},
	{PMIC_RG_VS2_RSV, PMIC_RG_VS2_RSV_ADDR, PMIC_RG_VS2_RSV_MASK, PMIC_RG_VS2_RSV_SHIFT},
	{PMIC_RG_VS2_AZC_EN, PMIC_RG_VS2_AZC_EN_ADDR, PMIC_RG_VS2_AZC_EN_MASK,
	 PMIC_RG_VS2_AZC_EN_SHIFT},
	{PMIC_RG_VS2_AZC_DELAY, PMIC_RG_VS2_AZC_DELAY_ADDR, PMIC_RG_VS2_AZC_DELAY_MASK,
	 PMIC_RG_VS2_AZC_DELAY_SHIFT},
	{PMIC_RG_VS2_AZC_HOLD_ENB, PMIC_RG_VS2_AZC_HOLD_ENB_ADDR, PMIC_RG_VS2_AZC_HOLD_ENB_MASK,
	 PMIC_RG_VS2_AZC_HOLD_ENB_SHIFT},
	{PMIC_RGS_QI_VS2_OC_STATUS, PMIC_RGS_QI_VS2_OC_STATUS_ADDR, PMIC_RGS_QI_VS2_OC_STATUS_MASK,
	 PMIC_RGS_QI_VS2_OC_STATUS_SHIFT},
	{PMIC_RGS_QI_VS2_DIG_MON, PMIC_RGS_QI_VS2_DIG_MON_ADDR, PMIC_RGS_QI_VS2_DIG_MON_MASK,
	 PMIC_RGS_QI_VS2_DIG_MON_SHIFT},
	{PMIC_RGS_VS2_ENPWM_STATUS, PMIC_RGS_VS2_ENPWM_STATUS_ADDR, PMIC_RGS_VS2_ENPWM_STATUS_MASK,
	 PMIC_RGS_VS2_ENPWM_STATUS_SHIFT},
	{PMIC_RG_VPA1_NDIS_EN, PMIC_RG_VPA1_NDIS_EN_ADDR, PMIC_RG_VPA1_NDIS_EN_MASK,
	 PMIC_RG_VPA1_NDIS_EN_SHIFT},
	{PMIC_RG_VPA1_MODESET, PMIC_RG_VPA1_MODESET_ADDR, PMIC_RG_VPA1_MODESET_MASK,
	 PMIC_RG_VPA1_MODESET_SHIFT},
	{PMIC_RG_VPA1_CC, PMIC_RG_VPA1_CC_ADDR, PMIC_RG_VPA1_CC_MASK, PMIC_RG_VPA1_CC_SHIFT},
	{PMIC_RG_VPA1_CSR, PMIC_RG_VPA1_CSR_ADDR, PMIC_RG_VPA1_CSR_MASK, PMIC_RG_VPA1_CSR_SHIFT},
	{PMIC_RG_VPA1_CSMIR, PMIC_RG_VPA1_CSMIR_ADDR, PMIC_RG_VPA1_CSMIR_MASK,
	 PMIC_RG_VPA1_CSMIR_SHIFT},
	{PMIC_RG_VPA1_CSL, PMIC_RG_VPA1_CSL_ADDR, PMIC_RG_VPA1_CSL_MASK, PMIC_RG_VPA1_CSL_SHIFT},
	{PMIC_RG_VPA1_SLP, PMIC_RG_VPA1_SLP_ADDR, PMIC_RG_VPA1_SLP_MASK, PMIC_RG_VPA1_SLP_SHIFT},
	{PMIC_RG_VPA1_AZC_EN, PMIC_RG_VPA1_AZC_EN_ADDR, PMIC_RG_VPA1_AZC_EN_MASK,
	 PMIC_RG_VPA1_AZC_EN_SHIFT},
	{PMIC_RG_VPA1_CP_FWUPOFF, PMIC_RG_VPA1_CP_FWUPOFF_ADDR, PMIC_RG_VPA1_CP_FWUPOFF_MASK,
	 PMIC_RG_VPA1_CP_FWUPOFF_SHIFT},
	{PMIC_RG_VPA1_AZC_DELAY, PMIC_RG_VPA1_AZC_DELAY_ADDR, PMIC_RG_VPA1_AZC_DELAY_MASK,
	 PMIC_RG_VPA1_AZC_DELAY_SHIFT},
	{PMIC_RG_VPA1_RZSEL, PMIC_RG_VPA1_RZSEL_ADDR, PMIC_RG_VPA1_RZSEL_MASK,
	 PMIC_RG_VPA1_RZSEL_SHIFT},
	{PMIC_RG_VPA1_ZXREF, PMIC_RG_VPA1_ZXREF_ADDR, PMIC_RG_VPA1_ZXREF_MASK,
	 PMIC_RG_VPA1_ZXREF_SHIFT},
	{PMIC_RG_VPA1_NLIM_SEL, PMIC_RG_VPA1_NLIM_SEL_ADDR, PMIC_RG_VPA1_NLIM_SEL_MASK,
	 PMIC_RG_VPA1_NLIM_SEL_SHIFT},
	{PMIC_RG_VPA1_HZP, PMIC_RG_VPA1_HZP_ADDR, PMIC_RG_VPA1_HZP_MASK, PMIC_RG_VPA1_HZP_SHIFT},
	{PMIC_RG_VPA1_BWEX_GAT, PMIC_RG_VPA1_BWEX_GAT_ADDR, PMIC_RG_VPA1_BWEX_GAT_MASK,
	 PMIC_RG_VPA1_BWEX_GAT_SHIFT},
	{PMIC_RG_VPA1_SLEW, PMIC_RG_VPA1_SLEW_ADDR, PMIC_RG_VPA1_SLEW_MASK,
	 PMIC_RG_VPA1_SLEW_SHIFT},
	{PMIC_RG_VPA1_SLEW_NMOS, PMIC_RG_VPA1_SLEW_NMOS_ADDR, PMIC_RG_VPA1_SLEW_NMOS_MASK,
	 PMIC_RG_VPA1_SLEW_NMOS_SHIFT},
	{PMIC_RG_VPA1_MIN_ON, PMIC_RG_VPA1_MIN_ON_ADDR, PMIC_RG_VPA1_MIN_ON_MASK,
	 PMIC_RG_VPA1_MIN_ON_SHIFT},
	{PMIC_RG_VPA1_VBAT_DEL, PMIC_RG_VPA1_VBAT_DEL_ADDR, PMIC_RG_VPA1_VBAT_DEL_MASK,
	 PMIC_RG_VPA1_VBAT_DEL_SHIFT},
	{PMIC_RGS_VPA1_AZC_VOS_SEL, PMIC_RGS_VPA1_AZC_VOS_SEL_ADDR, PMIC_RGS_VPA1_AZC_VOS_SEL_MASK,
	 PMIC_RGS_VPA1_AZC_VOS_SEL_SHIFT},
	{PMIC_RG_VPA1_MIN_PK, PMIC_RG_VPA1_MIN_PK_ADDR, PMIC_RG_VPA1_MIN_PK_MASK,
	 PMIC_RG_VPA1_MIN_PK_SHIFT},
	{PMIC_RG_VPA1_RSV1, PMIC_RG_VPA1_RSV1_ADDR, PMIC_RG_VPA1_RSV1_MASK,
	 PMIC_RG_VPA1_RSV1_SHIFT},
	{PMIC_RG_VPA1_RSV2, PMIC_RG_VPA1_RSV2_ADDR, PMIC_RG_VPA1_RSV2_MASK,
	 PMIC_RG_VPA1_RSV2_SHIFT},
	{PMIC_RGS_QI_VPA1_OC_STATUS, PMIC_RGS_QI_VPA1_OC_STATUS_ADDR,
	 PMIC_RGS_QI_VPA1_OC_STATUS_MASK, PMIC_RGS_QI_VPA1_OC_STATUS_SHIFT},
	{PMIC_RGS_NI_VPA1_AZC_ZX, PMIC_RGS_NI_VPA1_AZC_ZX_ADDR, PMIC_RGS_NI_VPA1_AZC_ZX_MASK,
	 PMIC_RGS_NI_VPA1_AZC_ZX_SHIFT},
	{PMIC_RG_VPA2_NDIS_EN, PMIC_RG_VPA2_NDIS_EN_ADDR, PMIC_RG_VPA2_NDIS_EN_MASK,
	 PMIC_RG_VPA2_NDIS_EN_SHIFT},
	{PMIC_RG_VPA2_MODESET, PMIC_RG_VPA2_MODESET_ADDR, PMIC_RG_VPA2_MODESET_MASK,
	 PMIC_RG_VPA2_MODESET_SHIFT},
	{PMIC_RG_VPA2_CC, PMIC_RG_VPA2_CC_ADDR, PMIC_RG_VPA2_CC_MASK, PMIC_RG_VPA2_CC_SHIFT},
	{PMIC_RG_VPA2_CSR, PMIC_RG_VPA2_CSR_ADDR, PMIC_RG_VPA2_CSR_MASK, PMIC_RG_VPA2_CSR_SHIFT},
	{PMIC_RG_VPA2_CSMIR, PMIC_RG_VPA2_CSMIR_ADDR, PMIC_RG_VPA2_CSMIR_MASK,
	 PMIC_RG_VPA2_CSMIR_SHIFT},
	{PMIC_RG_VPA2_CSL, PMIC_RG_VPA2_CSL_ADDR, PMIC_RG_VPA2_CSL_MASK, PMIC_RG_VPA2_CSL_SHIFT},
	{PMIC_RG_VPA2_SLP, PMIC_RG_VPA2_SLP_ADDR, PMIC_RG_VPA2_SLP_MASK, PMIC_RG_VPA2_SLP_SHIFT},
	{PMIC_RG_VPA2_AZC_EN, PMIC_RG_VPA2_AZC_EN_ADDR, PMIC_RG_VPA2_AZC_EN_MASK,
	 PMIC_RG_VPA2_AZC_EN_SHIFT},
	{PMIC_RG_VPA2_CP_FWUPOFF, PMIC_RG_VPA2_CP_FWUPOFF_ADDR, PMIC_RG_VPA2_CP_FWUPOFF_MASK,
	 PMIC_RG_VPA2_CP_FWUPOFF_SHIFT},
	{PMIC_RG_VPA2_AZC_DELAY, PMIC_RG_VPA2_AZC_DELAY_ADDR, PMIC_RG_VPA2_AZC_DELAY_MASK,
	 PMIC_RG_VPA2_AZC_DELAY_SHIFT},
	{PMIC_RG_VPA2_RZSEL, PMIC_RG_VPA2_RZSEL_ADDR, PMIC_RG_VPA2_RZSEL_MASK,
	 PMIC_RG_VPA2_RZSEL_SHIFT},
	{PMIC_RG_VPA2_ZXREF, PMIC_RG_VPA2_ZXREF_ADDR, PMIC_RG_VPA2_ZXREF_MASK,
	 PMIC_RG_VPA2_ZXREF_SHIFT},
	{PMIC_RG_VPA2_NLIM_SEL, PMIC_RG_VPA2_NLIM_SEL_ADDR, PMIC_RG_VPA2_NLIM_SEL_MASK,
	 PMIC_RG_VPA2_NLIM_SEL_SHIFT},
	{PMIC_RG_VPA2_HZP, PMIC_RG_VPA2_HZP_ADDR, PMIC_RG_VPA2_HZP_MASK, PMIC_RG_VPA2_HZP_SHIFT},
	{PMIC_RG_VPA2_BWEX_GAT, PMIC_RG_VPA2_BWEX_GAT_ADDR, PMIC_RG_VPA2_BWEX_GAT_MASK,
	 PMIC_RG_VPA2_BWEX_GAT_SHIFT},
	{PMIC_RG_VPA2_SLEW, PMIC_RG_VPA2_SLEW_ADDR, PMIC_RG_VPA2_SLEW_MASK,
	 PMIC_RG_VPA2_SLEW_SHIFT},
	{PMIC_RG_VPA2_SLEW_NMOS, PMIC_RG_VPA2_SLEW_NMOS_ADDR, PMIC_RG_VPA2_SLEW_NMOS_MASK,
	 PMIC_RG_VPA2_SLEW_NMOS_SHIFT},
	{PMIC_RG_VPA2_MIN_ON, PMIC_RG_VPA2_MIN_ON_ADDR, PMIC_RG_VPA2_MIN_ON_MASK,
	 PMIC_RG_VPA2_MIN_ON_SHIFT},
	{PMIC_RG_VPA2_VBAT_DEL, PMIC_RG_VPA2_VBAT_DEL_ADDR, PMIC_RG_VPA2_VBAT_DEL_MASK,
	 PMIC_RG_VPA2_VBAT_DEL_SHIFT},
	{PMIC_RGS_VPA2_AZC_VOS_SEL, PMIC_RGS_VPA2_AZC_VOS_SEL_ADDR, PMIC_RGS_VPA2_AZC_VOS_SEL_MASK,
	 PMIC_RGS_VPA2_AZC_VOS_SEL_SHIFT},
	{PMIC_RG_VPA2_MIN_PK, PMIC_RG_VPA2_MIN_PK_ADDR, PMIC_RG_VPA2_MIN_PK_MASK,
	 PMIC_RG_VPA2_MIN_PK_SHIFT},
	{PMIC_RG_VPA2_RSV1, PMIC_RG_VPA2_RSV1_ADDR, PMIC_RG_VPA2_RSV1_MASK,
	 PMIC_RG_VPA2_RSV1_SHIFT},
	{PMIC_RG_VPA2_RSV2, PMIC_RG_VPA2_RSV2_ADDR, PMIC_RG_VPA2_RSV2_MASK,
	 PMIC_RG_VPA2_RSV2_SHIFT},
	{PMIC_RGS_QI_VPA2_OC_STATUS, PMIC_RGS_QI_VPA2_OC_STATUS_ADDR,
	 PMIC_RGS_QI_VPA2_OC_STATUS_MASK, PMIC_RGS_QI_VPA2_OC_STATUS_SHIFT},
	{PMIC_RGS_NI_VPA2_AZC_ZX, PMIC_RGS_NI_VPA2_AZC_ZX_ADDR, PMIC_RGS_NI_VPA2_AZC_ZX_MASK,
	 PMIC_RGS_NI_VPA2_AZC_ZX_SHIFT},
	{PMIC_WDTDBG_CLR, PMIC_WDTDBG_CLR_ADDR, PMIC_WDTDBG_CLR_MASK, PMIC_WDTDBG_CLR_SHIFT},
	{PMIC_WDTDBG_CON0_RSV0, PMIC_WDTDBG_CON0_RSV0_ADDR, PMIC_WDTDBG_CON0_RSV0_MASK,
	 PMIC_WDTDBG_CON0_RSV0_SHIFT},
	{PMIC_VCORE_VOSEL_WDTDBG, PMIC_VCORE_VOSEL_WDTDBG_ADDR, PMIC_VCORE_VOSEL_WDTDBG_MASK,
	 PMIC_VCORE_VOSEL_WDTDBG_SHIFT},
	{PMIC_VDRAM_VOSEL_WDTDBG, PMIC_VDRAM_VOSEL_WDTDBG_ADDR, PMIC_VDRAM_VOSEL_WDTDBG_MASK,
	 PMIC_VDRAM_VOSEL_WDTDBG_SHIFT},
	{PMIC_VMD1_VOSEL_WDTDBG, PMIC_VMD1_VOSEL_WDTDBG_ADDR, PMIC_VMD1_VOSEL_WDTDBG_MASK,
	 PMIC_VMD1_VOSEL_WDTDBG_SHIFT},
	{PMIC_VMODEM_VOSEL_WDTDBG, PMIC_VMODEM_VOSEL_WDTDBG_ADDR, PMIC_VMODEM_VOSEL_WDTDBG_MASK,
	 PMIC_VMODEM_VOSEL_WDTDBG_SHIFT},
	{PMIC_VS1_VOSEL_WDTDBG, PMIC_VS1_VOSEL_WDTDBG_ADDR, PMIC_VS1_VOSEL_WDTDBG_MASK,
	 PMIC_VS1_VOSEL_WDTDBG_SHIFT},
	{PMIC_VS2_VOSEL_WDTDBG, PMIC_VS2_VOSEL_WDTDBG_ADDR, PMIC_VS2_VOSEL_WDTDBG_MASK,
	 PMIC_VS2_VOSEL_WDTDBG_SHIFT},
	{PMIC_VPA1_VOSEL_WDTDBG, PMIC_VPA1_VOSEL_WDTDBG_ADDR, PMIC_VPA1_VOSEL_WDTDBG_MASK,
	 PMIC_VPA1_VOSEL_WDTDBG_SHIFT},
	{PMIC_VPA2_VOSEL_WDTDBG, PMIC_VPA2_VOSEL_WDTDBG_ADDR, PMIC_VPA2_VOSEL_WDTDBG_MASK,
	 PMIC_VPA2_VOSEL_WDTDBG_SHIFT},
	{PMIC_VSRAM_DVFS1_VOSEL_WDTDBG, PMIC_VSRAM_DVFS1_VOSEL_WDTDBG_ADDR,
	 PMIC_VSRAM_DVFS1_VOSEL_WDTDBG_MASK, PMIC_VSRAM_DVFS1_VOSEL_WDTDBG_SHIFT},
	{PMIC_VSRAM_DVFS2_VOSEL_WDTDBG, PMIC_VSRAM_DVFS2_VOSEL_WDTDBG_ADDR,
	 PMIC_VSRAM_DVFS2_VOSEL_WDTDBG_MASK, PMIC_VSRAM_DVFS2_VOSEL_WDTDBG_SHIFT},
	{PMIC_VSRAM_GPU_VOSEL_WDTDBG, PMIC_VSRAM_GPU_VOSEL_WDTDBG_ADDR,
	 PMIC_VSRAM_GPU_VOSEL_WDTDBG_MASK, PMIC_VSRAM_GPU_VOSEL_WDTDBG_SHIFT},
	{PMIC_VSRAM_MD_VOSEL_WDTDBG, PMIC_VSRAM_MD_VOSEL_WDTDBG_ADDR,
	 PMIC_VSRAM_MD_VOSEL_WDTDBG_MASK, PMIC_VSRAM_MD_VOSEL_WDTDBG_SHIFT},
	{PMIC_VSRAM_CORE_VOSEL_WDTDBG, PMIC_VSRAM_CORE_VOSEL_WDTDBG_ADDR,
	 PMIC_VSRAM_CORE_VOSEL_WDTDBG_MASK, PMIC_VSRAM_CORE_VOSEL_WDTDBG_SHIFT},
	{PMIC_RG_A_TRIM_EN, PMIC_RG_A_TRIM_EN_ADDR, PMIC_RG_A_TRIM_EN_MASK,
	 PMIC_RG_A_TRIM_EN_SHIFT},
	{PMIC_RG_A_TRIM_SEL, PMIC_RG_A_TRIM_SEL_ADDR, PMIC_RG_A_TRIM_SEL_MASK,
	 PMIC_RG_A_TRIM_SEL_SHIFT},
	{PMIC_RG_A_ISINKS_RSV, PMIC_RG_A_ISINKS_RSV_ADDR, PMIC_RG_A_ISINKS_RSV_MASK,
	 PMIC_RG_A_ISINKS_RSV_SHIFT},
	{PMIC_RG_B_TRIM_EN, PMIC_RG_B_TRIM_EN_ADDR, PMIC_RG_B_TRIM_EN_MASK,
	 PMIC_RG_B_TRIM_EN_SHIFT},
	{PMIC_RG_B_TRIM_SEL, PMIC_RG_B_TRIM_SEL_ADDR, PMIC_RG_B_TRIM_SEL_MASK,
	 PMIC_RG_B_TRIM_SEL_SHIFT},
	{PMIC_RG_B_ISINKS_RSV, PMIC_RG_B_ISINKS_RSV_ADDR, PMIC_RG_B_ISINKS_RSV_MASK,
	 PMIC_RG_B_ISINKS_RSV_SHIFT},
	{PMIC_ISINK_DIM0_FSEL, PMIC_ISINK_DIM0_FSEL_ADDR, PMIC_ISINK_DIM0_FSEL_MASK,
	 PMIC_ISINK_DIM0_FSEL_SHIFT},
	{PMIC_ISINK0_RSV1, PMIC_ISINK0_RSV1_ADDR, PMIC_ISINK0_RSV1_MASK, PMIC_ISINK0_RSV1_SHIFT},
	{PMIC_ISINK_DIM0_DUTY, PMIC_ISINK_DIM0_DUTY_ADDR, PMIC_ISINK_DIM0_DUTY_MASK,
	 PMIC_ISINK_DIM0_DUTY_SHIFT},
	{PMIC_ISINK_CH0_STEP, PMIC_ISINK_CH0_STEP_ADDR, PMIC_ISINK_CH0_STEP_MASK,
	 PMIC_ISINK_CH0_STEP_SHIFT},
	{PMIC_ISINK_BREATH0_TF2_SEL, PMIC_ISINK_BREATH0_TF2_SEL_ADDR,
	 PMIC_ISINK_BREATH0_TF2_SEL_MASK, PMIC_ISINK_BREATH0_TF2_SEL_SHIFT},
	{PMIC_ISINK_BREATH0_TF1_SEL, PMIC_ISINK_BREATH0_TF1_SEL_ADDR,
	 PMIC_ISINK_BREATH0_TF1_SEL_MASK, PMIC_ISINK_BREATH0_TF1_SEL_SHIFT},
	{PMIC_ISINK_BREATH0_TR2_SEL, PMIC_ISINK_BREATH0_TR2_SEL_ADDR,
	 PMIC_ISINK_BREATH0_TR2_SEL_MASK, PMIC_ISINK_BREATH0_TR2_SEL_SHIFT},
	{PMIC_ISINK_BREATH0_TR1_SEL, PMIC_ISINK_BREATH0_TR1_SEL_ADDR,
	 PMIC_ISINK_BREATH0_TR1_SEL_MASK, PMIC_ISINK_BREATH0_TR1_SEL_SHIFT},
	{PMIC_ISINK_BREATH0_TOFF_SEL, PMIC_ISINK_BREATH0_TOFF_SEL_ADDR,
	 PMIC_ISINK_BREATH0_TOFF_SEL_MASK, PMIC_ISINK_BREATH0_TOFF_SEL_SHIFT},
	{PMIC_ISINK_BREATH0_TON_SEL, PMIC_ISINK_BREATH0_TON_SEL_ADDR,
	 PMIC_ISINK_BREATH0_TON_SEL_MASK, PMIC_ISINK_BREATH0_TON_SEL_SHIFT},
	{PMIC_ISINK_DIM1_FSEL, PMIC_ISINK_DIM1_FSEL_ADDR, PMIC_ISINK_DIM1_FSEL_MASK,
	 PMIC_ISINK_DIM1_FSEL_SHIFT},
	{PMIC_ISINK1_RSV1, PMIC_ISINK1_RSV1_ADDR, PMIC_ISINK1_RSV1_MASK, PMIC_ISINK1_RSV1_SHIFT},
	{PMIC_ISINK_DIM1_DUTY, PMIC_ISINK_DIM1_DUTY_ADDR, PMIC_ISINK_DIM1_DUTY_MASK,
	 PMIC_ISINK_DIM1_DUTY_SHIFT},
	{PMIC_ISINK_CH1_STEP, PMIC_ISINK_CH1_STEP_ADDR, PMIC_ISINK_CH1_STEP_MASK,
	 PMIC_ISINK_CH1_STEP_SHIFT},
	{PMIC_ISINK_BREATH1_TF2_SEL, PMIC_ISINK_BREATH1_TF2_SEL_ADDR,
	 PMIC_ISINK_BREATH1_TF2_SEL_MASK, PMIC_ISINK_BREATH1_TF2_SEL_SHIFT},
	{PMIC_ISINK_BREATH1_TF1_SEL, PMIC_ISINK_BREATH1_TF1_SEL_ADDR,
	 PMIC_ISINK_BREATH1_TF1_SEL_MASK, PMIC_ISINK_BREATH1_TF1_SEL_SHIFT},
	{PMIC_ISINK_BREATH1_TR2_SEL, PMIC_ISINK_BREATH1_TR2_SEL_ADDR,
	 PMIC_ISINK_BREATH1_TR2_SEL_MASK, PMIC_ISINK_BREATH1_TR2_SEL_SHIFT},
	{PMIC_ISINK_BREATH1_TR1_SEL, PMIC_ISINK_BREATH1_TR1_SEL_ADDR,
	 PMIC_ISINK_BREATH1_TR1_SEL_MASK, PMIC_ISINK_BREATH1_TR1_SEL_SHIFT},
	{PMIC_ISINK_BREATH1_TOFF_SEL, PMIC_ISINK_BREATH1_TOFF_SEL_ADDR,
	 PMIC_ISINK_BREATH1_TOFF_SEL_MASK, PMIC_ISINK_BREATH1_TOFF_SEL_SHIFT},
	{PMIC_ISINK_BREATH1_TON_SEL, PMIC_ISINK_BREATH1_TON_SEL_ADDR,
	 PMIC_ISINK_BREATH1_TON_SEL_MASK, PMIC_ISINK_BREATH1_TON_SEL_SHIFT},
	{PMIC_ISINK_DIM2_FSEL, PMIC_ISINK_DIM2_FSEL_ADDR, PMIC_ISINK_DIM2_FSEL_MASK,
	 PMIC_ISINK_DIM2_FSEL_SHIFT},
	{PMIC_ISINK2_RSV1, PMIC_ISINK2_RSV1_ADDR, PMIC_ISINK2_RSV1_MASK, PMIC_ISINK2_RSV1_SHIFT},
	{PMIC_ISINK_DIM2_DUTY, PMIC_ISINK_DIM2_DUTY_ADDR, PMIC_ISINK_DIM2_DUTY_MASK,
	 PMIC_ISINK_DIM2_DUTY_SHIFT},
	{PMIC_ISINK_CH2_STEP, PMIC_ISINK_CH2_STEP_ADDR, PMIC_ISINK_CH2_STEP_MASK,
	 PMIC_ISINK_CH2_STEP_SHIFT},
	{PMIC_ISINK_BREATH2_TF2_SEL, PMIC_ISINK_BREATH2_TF2_SEL_ADDR,
	 PMIC_ISINK_BREATH2_TF2_SEL_MASK, PMIC_ISINK_BREATH2_TF2_SEL_SHIFT},
	{PMIC_ISINK_BREATH2_TF1_SEL, PMIC_ISINK_BREATH2_TF1_SEL_ADDR,
	 PMIC_ISINK_BREATH2_TF1_SEL_MASK, PMIC_ISINK_BREATH2_TF1_SEL_SHIFT},
	{PMIC_ISINK_BREATH2_TR2_SEL, PMIC_ISINK_BREATH2_TR2_SEL_ADDR,
	 PMIC_ISINK_BREATH2_TR2_SEL_MASK, PMIC_ISINK_BREATH2_TR2_SEL_SHIFT},
	{PMIC_ISINK_BREATH2_TR1_SEL, PMIC_ISINK_BREATH2_TR1_SEL_ADDR,
	 PMIC_ISINK_BREATH2_TR1_SEL_MASK, PMIC_ISINK_BREATH2_TR1_SEL_SHIFT},
	{PMIC_ISINK_BREATH2_TOFF_SEL, PMIC_ISINK_BREATH2_TOFF_SEL_ADDR,
	 PMIC_ISINK_BREATH2_TOFF_SEL_MASK, PMIC_ISINK_BREATH2_TOFF_SEL_SHIFT},
	{PMIC_ISINK_BREATH2_TON_SEL, PMIC_ISINK_BREATH2_TON_SEL_ADDR,
	 PMIC_ISINK_BREATH2_TON_SEL_MASK, PMIC_ISINK_BREATH2_TON_SEL_SHIFT},
	{PMIC_ISINK_DIM3_FSEL, PMIC_ISINK_DIM3_FSEL_ADDR, PMIC_ISINK_DIM3_FSEL_MASK,
	 PMIC_ISINK_DIM3_FSEL_SHIFT},
	{PMIC_ISINK3_RSV1, PMIC_ISINK3_RSV1_ADDR, PMIC_ISINK3_RSV1_MASK, PMIC_ISINK3_RSV1_SHIFT},
	{PMIC_ISINK_DIM3_DUTY, PMIC_ISINK_DIM3_DUTY_ADDR, PMIC_ISINK_DIM3_DUTY_MASK,
	 PMIC_ISINK_DIM3_DUTY_SHIFT},
	{PMIC_ISINK_CH3_STEP, PMIC_ISINK_CH3_STEP_ADDR, PMIC_ISINK_CH3_STEP_MASK,
	 PMIC_ISINK_CH3_STEP_SHIFT},
	{PMIC_ISINK_BREATH3_TF2_SEL, PMIC_ISINK_BREATH3_TF2_SEL_ADDR,
	 PMIC_ISINK_BREATH3_TF2_SEL_MASK, PMIC_ISINK_BREATH3_TF2_SEL_SHIFT},
	{PMIC_ISINK_BREATH3_TF1_SEL, PMIC_ISINK_BREATH3_TF1_SEL_ADDR,
	 PMIC_ISINK_BREATH3_TF1_SEL_MASK, PMIC_ISINK_BREATH3_TF1_SEL_SHIFT},
	{PMIC_ISINK_BREATH3_TR2_SEL, PMIC_ISINK_BREATH3_TR2_SEL_ADDR,
	 PMIC_ISINK_BREATH3_TR2_SEL_MASK, PMIC_ISINK_BREATH3_TR2_SEL_SHIFT},
	{PMIC_ISINK_BREATH3_TR1_SEL, PMIC_ISINK_BREATH3_TR1_SEL_ADDR,
	 PMIC_ISINK_BREATH3_TR1_SEL_MASK, PMIC_ISINK_BREATH3_TR1_SEL_SHIFT},
	{PMIC_ISINK_BREATH3_TOFF_SEL, PMIC_ISINK_BREATH3_TOFF_SEL_ADDR,
	 PMIC_ISINK_BREATH3_TOFF_SEL_MASK, PMIC_ISINK_BREATH3_TOFF_SEL_SHIFT},
	{PMIC_ISINK_BREATH3_TON_SEL, PMIC_ISINK_BREATH3_TON_SEL_ADDR,
	 PMIC_ISINK_BREATH3_TON_SEL_MASK, PMIC_ISINK_BREATH3_TON_SEL_SHIFT},
	{PMIC_AD_NI_B_ISINK1_STATUS, PMIC_AD_NI_B_ISINK1_STATUS_ADDR,
	 PMIC_AD_NI_B_ISINK1_STATUS_MASK, PMIC_AD_NI_B_ISINK1_STATUS_SHIFT},
	{PMIC_AD_NI_B_ISINK0_STATUS, PMIC_AD_NI_B_ISINK0_STATUS_ADDR,
	 PMIC_AD_NI_B_ISINK0_STATUS_MASK, PMIC_AD_NI_B_ISINK0_STATUS_SHIFT},
	{PMIC_AD_NI_A_ISINK1_STATUS, PMIC_AD_NI_A_ISINK1_STATUS_ADDR,
	 PMIC_AD_NI_A_ISINK1_STATUS_MASK, PMIC_AD_NI_A_ISINK1_STATUS_SHIFT},
	{PMIC_AD_NI_A_ISINK0_STATUS, PMIC_AD_NI_A_ISINK0_STATUS_ADDR,
	 PMIC_AD_NI_A_ISINK0_STATUS_MASK, PMIC_AD_NI_A_ISINK0_STATUS_SHIFT},
	{PMIC_ISINK_PHASE0_DLY_EN, PMIC_ISINK_PHASE0_DLY_EN_ADDR, PMIC_ISINK_PHASE0_DLY_EN_MASK,
	 PMIC_ISINK_PHASE0_DLY_EN_SHIFT},
	{PMIC_ISINK_PHASE1_DLY_EN, PMIC_ISINK_PHASE1_DLY_EN_ADDR, PMIC_ISINK_PHASE1_DLY_EN_MASK,
	 PMIC_ISINK_PHASE1_DLY_EN_SHIFT},
	{PMIC_ISINK_PHASE2_DLY_EN, PMIC_ISINK_PHASE2_DLY_EN_ADDR, PMIC_ISINK_PHASE2_DLY_EN_MASK,
	 PMIC_ISINK_PHASE2_DLY_EN_SHIFT},
	{PMIC_ISINK_PHASE3_DLY_EN, PMIC_ISINK_PHASE3_DLY_EN_ADDR, PMIC_ISINK_PHASE3_DLY_EN_MASK,
	 PMIC_ISINK_PHASE3_DLY_EN_SHIFT},
	{PMIC_ISINK_PHASE_DLY_TC, PMIC_ISINK_PHASE_DLY_TC_ADDR, PMIC_ISINK_PHASE_DLY_TC_MASK,
	 PMIC_ISINK_PHASE_DLY_TC_SHIFT},
	{PMIC_ISINK_CHOP0_SW, PMIC_ISINK_CHOP0_SW_ADDR, PMIC_ISINK_CHOP0_SW_MASK,
	 PMIC_ISINK_CHOP0_SW_SHIFT},
	{PMIC_ISINK_CHOP1_SW, PMIC_ISINK_CHOP1_SW_ADDR, PMIC_ISINK_CHOP1_SW_MASK,
	 PMIC_ISINK_CHOP1_SW_SHIFT},
	{PMIC_ISINK_CHOP2_SW, PMIC_ISINK_CHOP2_SW_ADDR, PMIC_ISINK_CHOP2_SW_MASK,
	 PMIC_ISINK_CHOP2_SW_SHIFT},
	{PMIC_ISINK_CHOP3_SW, PMIC_ISINK_CHOP3_SW_ADDR, PMIC_ISINK_CHOP3_SW_MASK,
	 PMIC_ISINK_CHOP3_SW_SHIFT},
	{PMIC_ISINK_SFSTR3_EN, PMIC_ISINK_SFSTR3_EN_ADDR, PMIC_ISINK_SFSTR3_EN_MASK,
	 PMIC_ISINK_SFSTR3_EN_SHIFT},
	{PMIC_ISINK_SFSTR3_TC, PMIC_ISINK_SFSTR3_TC_ADDR, PMIC_ISINK_SFSTR3_TC_MASK,
	 PMIC_ISINK_SFSTR3_TC_SHIFT},
	{PMIC_ISINK_SFSTR2_EN, PMIC_ISINK_SFSTR2_EN_ADDR, PMIC_ISINK_SFSTR2_EN_MASK,
	 PMIC_ISINK_SFSTR2_EN_SHIFT},
	{PMIC_ISINK_SFSTR2_TC, PMIC_ISINK_SFSTR2_TC_ADDR, PMIC_ISINK_SFSTR2_TC_MASK,
	 PMIC_ISINK_SFSTR2_TC_SHIFT},
	{PMIC_ISINK_SFSTR1_EN, PMIC_ISINK_SFSTR1_EN_ADDR, PMIC_ISINK_SFSTR1_EN_MASK,
	 PMIC_ISINK_SFSTR1_EN_SHIFT},
	{PMIC_ISINK_SFSTR1_TC, PMIC_ISINK_SFSTR1_TC_ADDR, PMIC_ISINK_SFSTR1_TC_MASK,
	 PMIC_ISINK_SFSTR1_TC_SHIFT},
	{PMIC_ISINK_SFSTR0_EN, PMIC_ISINK_SFSTR0_EN_ADDR, PMIC_ISINK_SFSTR0_EN_MASK,
	 PMIC_ISINK_SFSTR0_EN_SHIFT},
	{PMIC_ISINK_SFSTR0_TC, PMIC_ISINK_SFSTR0_TC_ADDR, PMIC_ISINK_SFSTR0_TC_MASK,
	 PMIC_ISINK_SFSTR0_TC_SHIFT},
	{PMIC_ISINK_CH0_EN, PMIC_ISINK_CH0_EN_ADDR, PMIC_ISINK_CH0_EN_MASK,
	 PMIC_ISINK_CH0_EN_SHIFT},
	{PMIC_ISINK_CH1_EN, PMIC_ISINK_CH1_EN_ADDR, PMIC_ISINK_CH1_EN_MASK,
	 PMIC_ISINK_CH1_EN_SHIFT},
	{PMIC_ISINK_CH2_EN, PMIC_ISINK_CH2_EN_ADDR, PMIC_ISINK_CH2_EN_MASK,
	 PMIC_ISINK_CH2_EN_SHIFT},
	{PMIC_ISINK_CH3_EN, PMIC_ISINK_CH3_EN_ADDR, PMIC_ISINK_CH3_EN_MASK,
	 PMIC_ISINK_CH3_EN_SHIFT},
	{PMIC_ISINK_CHOP0_EN, PMIC_ISINK_CHOP0_EN_ADDR, PMIC_ISINK_CHOP0_EN_MASK,
	 PMIC_ISINK_CHOP0_EN_SHIFT},
	{PMIC_ISINK_CHOP1_EN, PMIC_ISINK_CHOP1_EN_ADDR, PMIC_ISINK_CHOP1_EN_MASK,
	 PMIC_ISINK_CHOP1_EN_SHIFT},
	{PMIC_ISINK_CHOP2_EN, PMIC_ISINK_CHOP2_EN_ADDR, PMIC_ISINK_CHOP2_EN_MASK,
	 PMIC_ISINK_CHOP2_EN_SHIFT},
	{PMIC_ISINK_CHOP3_EN, PMIC_ISINK_CHOP3_EN_ADDR, PMIC_ISINK_CHOP3_EN_MASK,
	 PMIC_ISINK_CHOP3_EN_SHIFT},
	{PMIC_ISINK_CH0_BIAS_EN, PMIC_ISINK_CH0_BIAS_EN_ADDR, PMIC_ISINK_CH0_BIAS_EN_MASK,
	 PMIC_ISINK_CH0_BIAS_EN_SHIFT},
	{PMIC_ISINK_CH1_BIAS_EN, PMIC_ISINK_CH1_BIAS_EN_ADDR, PMIC_ISINK_CH1_BIAS_EN_MASK,
	 PMIC_ISINK_CH1_BIAS_EN_SHIFT},
	{PMIC_ISINK_CH2_BIAS_EN, PMIC_ISINK_CH2_BIAS_EN_ADDR, PMIC_ISINK_CH2_BIAS_EN_MASK,
	 PMIC_ISINK_CH2_BIAS_EN_SHIFT},
	{PMIC_ISINK_CH3_BIAS_EN, PMIC_ISINK_CH3_BIAS_EN_ADDR, PMIC_ISINK_CH3_BIAS_EN_MASK,
	 PMIC_ISINK_CH3_BIAS_EN_SHIFT},
	{PMIC_ISINK_RSV, PMIC_ISINK_RSV_ADDR, PMIC_ISINK_RSV_MASK, PMIC_ISINK_RSV_SHIFT},
	{PMIC_ISINK_CH3_MODE, PMIC_ISINK_CH3_MODE_ADDR, PMIC_ISINK_CH3_MODE_MASK,
	 PMIC_ISINK_CH3_MODE_SHIFT},
	{PMIC_ISINK_CH2_MODE, PMIC_ISINK_CH2_MODE_ADDR, PMIC_ISINK_CH2_MODE_MASK,
	 PMIC_ISINK_CH2_MODE_SHIFT},
	{PMIC_ISINK_CH1_MODE, PMIC_ISINK_CH1_MODE_ADDR, PMIC_ISINK_CH1_MODE_MASK,
	 PMIC_ISINK_CH1_MODE_SHIFT},
	{PMIC_ISINK_CH0_MODE, PMIC_ISINK_CH0_MODE_ADDR, PMIC_ISINK_CH0_MODE_MASK,
	 PMIC_ISINK_CH0_MODE_SHIFT},
	{PMIC_DA_QI_A_ISINKS_CH0_STEP, PMIC_DA_QI_A_ISINKS_CH0_STEP_ADDR,
	 PMIC_DA_QI_A_ISINKS_CH0_STEP_MASK, PMIC_DA_QI_A_ISINKS_CH0_STEP_SHIFT},
	{PMIC_DA_QI_A_ISINKS_CH1_STEP, PMIC_DA_QI_A_ISINKS_CH1_STEP_ADDR,
	 PMIC_DA_QI_A_ISINKS_CH1_STEP_MASK, PMIC_DA_QI_A_ISINKS_CH1_STEP_SHIFT},
	{PMIC_DA_QI_B_ISINKS_CH0_STEP, PMIC_DA_QI_B_ISINKS_CH0_STEP_ADDR,
	 PMIC_DA_QI_B_ISINKS_CH0_STEP_MASK, PMIC_DA_QI_B_ISINKS_CH0_STEP_SHIFT},
	{PMIC_DA_QI_B_ISINKS_CH1_STEP, PMIC_DA_QI_B_ISINKS_CH1_STEP_ADDR,
	 PMIC_DA_QI_B_ISINKS_CH1_STEP_MASK, PMIC_DA_QI_B_ISINKS_CH1_STEP_SHIFT},
	{PMIC_ISINK4_RSV1, PMIC_ISINK4_RSV1_ADDR, PMIC_ISINK4_RSV1_MASK, PMIC_ISINK4_RSV1_SHIFT},
	{PMIC_ISINK4_RSV0, PMIC_ISINK4_RSV0_ADDR, PMIC_ISINK4_RSV0_MASK, PMIC_ISINK4_RSV0_SHIFT},
	{PMIC_ISINK_CH4_STEP, PMIC_ISINK_CH4_STEP_ADDR, PMIC_ISINK_CH4_STEP_MASK,
	 PMIC_ISINK_CH4_STEP_SHIFT},
	{PMIC_ISINK5_RSV1, PMIC_ISINK5_RSV1_ADDR, PMIC_ISINK5_RSV1_MASK, PMIC_ISINK5_RSV1_SHIFT},
	{PMIC_ISINK5_RSV0, PMIC_ISINK5_RSV0_ADDR, PMIC_ISINK5_RSV0_MASK, PMIC_ISINK5_RSV0_SHIFT},
	{PMIC_ISINK_CH5_STEP, PMIC_ISINK_CH5_STEP_ADDR, PMIC_ISINK_CH5_STEP_MASK,
	 PMIC_ISINK_CH5_STEP_SHIFT},
	{PMIC_ISINK6_RSV1, PMIC_ISINK6_RSV1_ADDR, PMIC_ISINK6_RSV1_MASK, PMIC_ISINK6_RSV1_SHIFT},
	{PMIC_ISINK6_RSV0, PMIC_ISINK6_RSV0_ADDR, PMIC_ISINK6_RSV0_MASK, PMIC_ISINK6_RSV0_SHIFT},
	{PMIC_ISINK_CH6_STEP, PMIC_ISINK_CH6_STEP_ADDR, PMIC_ISINK_CH6_STEP_MASK,
	 PMIC_ISINK_CH6_STEP_SHIFT},
	{PMIC_ISINK7_RSV1, PMIC_ISINK7_RSV1_ADDR, PMIC_ISINK7_RSV1_MASK, PMIC_ISINK7_RSV1_SHIFT},
	{PMIC_ISINK7_RSV0, PMIC_ISINK7_RSV0_ADDR, PMIC_ISINK7_RSV0_MASK, PMIC_ISINK7_RSV0_SHIFT},
	{PMIC_ISINK_CH7_STEP, PMIC_ISINK_CH7_STEP_ADDR, PMIC_ISINK_CH7_STEP_MASK,
	 PMIC_ISINK_CH7_STEP_SHIFT},
	{PMIC_AD_NI_B_ISINK3_STATUS, PMIC_AD_NI_B_ISINK3_STATUS_ADDR,
	 PMIC_AD_NI_B_ISINK3_STATUS_MASK, PMIC_AD_NI_B_ISINK3_STATUS_SHIFT},
	{PMIC_AD_NI_B_ISINK2_STATUS, PMIC_AD_NI_B_ISINK2_STATUS_ADDR,
	 PMIC_AD_NI_B_ISINK2_STATUS_MASK, PMIC_AD_NI_B_ISINK2_STATUS_SHIFT},
	{PMIC_AD_NI_A_ISINK3_STATUS, PMIC_AD_NI_A_ISINK3_STATUS_ADDR,
	 PMIC_AD_NI_A_ISINK3_STATUS_MASK, PMIC_AD_NI_A_ISINK3_STATUS_SHIFT},
	{PMIC_AD_NI_A_ISINK2_STATUS, PMIC_AD_NI_A_ISINK2_STATUS_ADDR,
	 PMIC_AD_NI_A_ISINK2_STATUS_MASK, PMIC_AD_NI_A_ISINK2_STATUS_SHIFT},
	{PMIC_ISINK_CHOP7_SW, PMIC_ISINK_CHOP7_SW_ADDR, PMIC_ISINK_CHOP7_SW_MASK,
	 PMIC_ISINK_CHOP7_SW_SHIFT},
	{PMIC_ISINK_CHOP6_SW, PMIC_ISINK_CHOP6_SW_ADDR, PMIC_ISINK_CHOP6_SW_MASK,
	 PMIC_ISINK_CHOP6_SW_SHIFT},
	{PMIC_ISINK_CHOP5_SW, PMIC_ISINK_CHOP5_SW_ADDR, PMIC_ISINK_CHOP5_SW_MASK,
	 PMIC_ISINK_CHOP5_SW_SHIFT},
	{PMIC_ISINK_CHOP4_SW, PMIC_ISINK_CHOP4_SW_ADDR, PMIC_ISINK_CHOP4_SW_MASK,
	 PMIC_ISINK_CHOP4_SW_SHIFT},
	{PMIC_ISINK_CH7_EN, PMIC_ISINK_CH7_EN_ADDR, PMIC_ISINK_CH7_EN_MASK,
	 PMIC_ISINK_CH7_EN_SHIFT},
	{PMIC_ISINK_CH6_EN, PMIC_ISINK_CH6_EN_ADDR, PMIC_ISINK_CH6_EN_MASK,
	 PMIC_ISINK_CH6_EN_SHIFT},
	{PMIC_ISINK_CH5_EN, PMIC_ISINK_CH5_EN_ADDR, PMIC_ISINK_CH5_EN_MASK,
	 PMIC_ISINK_CH5_EN_SHIFT},
	{PMIC_ISINK_CH4_EN, PMIC_ISINK_CH4_EN_ADDR, PMIC_ISINK_CH4_EN_MASK,
	 PMIC_ISINK_CH4_EN_SHIFT},
	{PMIC_ISINK_CHOP7_EN, PMIC_ISINK_CHOP7_EN_ADDR, PMIC_ISINK_CHOP7_EN_MASK,
	 PMIC_ISINK_CHOP7_EN_SHIFT},
	{PMIC_ISINK_CHOP6_EN, PMIC_ISINK_CHOP6_EN_ADDR, PMIC_ISINK_CHOP6_EN_MASK,
	 PMIC_ISINK_CHOP6_EN_SHIFT},
	{PMIC_ISINK_CHOP5_EN, PMIC_ISINK_CHOP5_EN_ADDR, PMIC_ISINK_CHOP5_EN_MASK,
	 PMIC_ISINK_CHOP5_EN_SHIFT},
	{PMIC_ISINK_CHOP4_EN, PMIC_ISINK_CHOP4_EN_ADDR, PMIC_ISINK_CHOP4_EN_MASK,
	 PMIC_ISINK_CHOP4_EN_SHIFT},
	{PMIC_ISINK_CH7_BIAS_EN, PMIC_ISINK_CH7_BIAS_EN_ADDR, PMIC_ISINK_CH7_BIAS_EN_MASK,
	 PMIC_ISINK_CH7_BIAS_EN_SHIFT},
	{PMIC_ISINK_CH6_BIAS_EN, PMIC_ISINK_CH6_BIAS_EN_ADDR, PMIC_ISINK_CH6_BIAS_EN_MASK,
	 PMIC_ISINK_CH6_BIAS_EN_SHIFT},
	{PMIC_ISINK_CH5_BIAS_EN, PMIC_ISINK_CH5_BIAS_EN_ADDR, PMIC_ISINK_CH5_BIAS_EN_MASK,
	 PMIC_ISINK_CH5_BIAS_EN_SHIFT},
	{PMIC_ISINK_CH4_BIAS_EN, PMIC_ISINK_CH4_BIAS_EN_ADDR, PMIC_ISINK_CH4_BIAS_EN_MASK,
	 PMIC_ISINK_CH4_BIAS_EN_SHIFT},
	{PMIC_RG_VIO28_SW_EN, PMIC_RG_VIO28_SW_EN_ADDR, PMIC_RG_VIO28_SW_EN_MASK,
	 PMIC_RG_VIO28_SW_EN_SHIFT},
	{PMIC_RG_VIO28_SW_LP, PMIC_RG_VIO28_SW_LP_ADDR, PMIC_RG_VIO28_SW_LP_MASK,
	 PMIC_RG_VIO28_SW_LP_SHIFT},
	{PMIC_RG_VIO28_SW_OP_EN, PMIC_RG_VIO28_SW_OP_EN_ADDR, PMIC_RG_VIO28_SW_OP_EN_MASK,
	 PMIC_RG_VIO28_SW_OP_EN_SHIFT},
	{PMIC_RG_VIO28_HW0_OP_EN, PMIC_RG_VIO28_HW0_OP_EN_ADDR, PMIC_RG_VIO28_HW0_OP_EN_MASK,
	 PMIC_RG_VIO28_HW0_OP_EN_SHIFT},
	{PMIC_RG_VIO28_HW1_OP_EN, PMIC_RG_VIO28_HW1_OP_EN_ADDR, PMIC_RG_VIO28_HW1_OP_EN_MASK,
	 PMIC_RG_VIO28_HW1_OP_EN_SHIFT},
	{PMIC_RG_VIO28_HW2_OP_EN, PMIC_RG_VIO28_HW2_OP_EN_ADDR, PMIC_RG_VIO28_HW2_OP_EN_MASK,
	 PMIC_RG_VIO28_HW2_OP_EN_SHIFT},
	{PMIC_RG_VIO28_OP_EN_SET, PMIC_RG_VIO28_OP_EN_SET_ADDR, PMIC_RG_VIO28_OP_EN_SET_MASK,
	 PMIC_RG_VIO28_OP_EN_SET_SHIFT},
	{PMIC_RG_VIO28_OP_EN_CLR, PMIC_RG_VIO28_OP_EN_CLR_ADDR, PMIC_RG_VIO28_OP_EN_CLR_MASK,
	 PMIC_RG_VIO28_OP_EN_CLR_SHIFT},
	{PMIC_RG_VIO28_HW0_OP_CFG, PMIC_RG_VIO28_HW0_OP_CFG_ADDR, PMIC_RG_VIO28_HW0_OP_CFG_MASK,
	 PMIC_RG_VIO28_HW0_OP_CFG_SHIFT},
	{PMIC_RG_VIO28_HW1_OP_CFG, PMIC_RG_VIO28_HW1_OP_CFG_ADDR, PMIC_RG_VIO28_HW1_OP_CFG_MASK,
	 PMIC_RG_VIO28_HW1_OP_CFG_SHIFT},
	{PMIC_RG_VIO28_HW2_OP_CFG, PMIC_RG_VIO28_HW2_OP_CFG_ADDR, PMIC_RG_VIO28_HW2_OP_CFG_MASK,
	 PMIC_RG_VIO28_HW2_OP_CFG_SHIFT},
	{PMIC_RG_VIO28_GO_ON_OP, PMIC_RG_VIO28_GO_ON_OP_ADDR, PMIC_RG_VIO28_GO_ON_OP_MASK,
	 PMIC_RG_VIO28_GO_ON_OP_SHIFT},
	{PMIC_RG_VIO28_GO_LP_OP, PMIC_RG_VIO28_GO_LP_OP_ADDR, PMIC_RG_VIO28_GO_LP_OP_MASK,
	 PMIC_RG_VIO28_GO_LP_OP_SHIFT},
	{PMIC_RG_VIO28_OP_CFG_SET, PMIC_RG_VIO28_OP_CFG_SET_ADDR, PMIC_RG_VIO28_OP_CFG_SET_MASK,
	 PMIC_RG_VIO28_OP_CFG_SET_SHIFT},
	{PMIC_RG_VIO28_OP_CFG_CLR, PMIC_RG_VIO28_OP_CFG_CLR_ADDR, PMIC_RG_VIO28_OP_CFG_CLR_MASK,
	 PMIC_RG_VIO28_OP_CFG_CLR_SHIFT},
	{PMIC_DA_QI_VIO28_MODE, PMIC_DA_QI_VIO28_MODE_ADDR, PMIC_DA_QI_VIO28_MODE_MASK,
	 PMIC_DA_QI_VIO28_MODE_SHIFT},
	{PMIC_RG_VIO28_STBTD, PMIC_RG_VIO28_STBTD_ADDR, PMIC_RG_VIO28_STBTD_MASK,
	 PMIC_RG_VIO28_STBTD_SHIFT},
	{PMIC_DA_QI_VIO28_STB, PMIC_DA_QI_VIO28_STB_ADDR, PMIC_DA_QI_VIO28_STB_MASK,
	 PMIC_DA_QI_VIO28_STB_SHIFT},
	{PMIC_DA_QI_VIO28_EN, PMIC_DA_QI_VIO28_EN_ADDR, PMIC_DA_QI_VIO28_EN_MASK,
	 PMIC_DA_QI_VIO28_EN_SHIFT},
	{PMIC_RG_VIO28_OCFB_EN, PMIC_RG_VIO28_OCFB_EN_ADDR, PMIC_RG_VIO28_OCFB_EN_MASK,
	 PMIC_RG_VIO28_OCFB_EN_SHIFT},
	{PMIC_DA_QI_VIO28_OCFB_EN, PMIC_DA_QI_VIO28_OCFB_EN_ADDR, PMIC_DA_QI_VIO28_OCFB_EN_MASK,
	 PMIC_DA_QI_VIO28_OCFB_EN_SHIFT},
	{PMIC_RG_VIO28_DUMMY_LOAD, PMIC_RG_VIO28_DUMMY_LOAD_ADDR, PMIC_RG_VIO28_DUMMY_LOAD_MASK,
	 PMIC_RG_VIO28_DUMMY_LOAD_SHIFT},
	{PMIC_DA_QI_VIO28_DUMMY_LOAD, PMIC_DA_QI_VIO28_DUMMY_LOAD_ADDR,
	 PMIC_DA_QI_VIO28_DUMMY_LOAD_MASK, PMIC_DA_QI_VIO28_DUMMY_LOAD_SHIFT},
	{PMIC_RG_VIO18_SW_EN, PMIC_RG_VIO18_SW_EN_ADDR, PMIC_RG_VIO18_SW_EN_MASK,
	 PMIC_RG_VIO18_SW_EN_SHIFT},
	{PMIC_RG_VIO18_SW_LP, PMIC_RG_VIO18_SW_LP_ADDR, PMIC_RG_VIO18_SW_LP_MASK,
	 PMIC_RG_VIO18_SW_LP_SHIFT},
	{PMIC_RG_VIO18_SW_OP_EN, PMIC_RG_VIO18_SW_OP_EN_ADDR, PMIC_RG_VIO18_SW_OP_EN_MASK,
	 PMIC_RG_VIO18_SW_OP_EN_SHIFT},
	{PMIC_RG_VIO18_HW0_OP_EN, PMIC_RG_VIO18_HW0_OP_EN_ADDR, PMIC_RG_VIO18_HW0_OP_EN_MASK,
	 PMIC_RG_VIO18_HW0_OP_EN_SHIFT},
	{PMIC_RG_VIO18_HW1_OP_EN, PMIC_RG_VIO18_HW1_OP_EN_ADDR, PMIC_RG_VIO18_HW1_OP_EN_MASK,
	 PMIC_RG_VIO18_HW1_OP_EN_SHIFT},
	{PMIC_RG_VIO18_HW2_OP_EN, PMIC_RG_VIO18_HW2_OP_EN_ADDR, PMIC_RG_VIO18_HW2_OP_EN_MASK,
	 PMIC_RG_VIO18_HW2_OP_EN_SHIFT},
	{PMIC_RG_VIO18_OP_EN_SET, PMIC_RG_VIO18_OP_EN_SET_ADDR, PMIC_RG_VIO18_OP_EN_SET_MASK,
	 PMIC_RG_VIO18_OP_EN_SET_SHIFT},
	{PMIC_RG_VIO18_OP_EN_CLR, PMIC_RG_VIO18_OP_EN_CLR_ADDR, PMIC_RG_VIO18_OP_EN_CLR_MASK,
	 PMIC_RG_VIO18_OP_EN_CLR_SHIFT},
	{PMIC_RG_VIO18_HW0_OP_CFG, PMIC_RG_VIO18_HW0_OP_CFG_ADDR, PMIC_RG_VIO18_HW0_OP_CFG_MASK,
	 PMIC_RG_VIO18_HW0_OP_CFG_SHIFT},
	{PMIC_RG_VIO18_HW1_OP_CFG, PMIC_RG_VIO18_HW1_OP_CFG_ADDR, PMIC_RG_VIO18_HW1_OP_CFG_MASK,
	 PMIC_RG_VIO18_HW1_OP_CFG_SHIFT},
	{PMIC_RG_VIO18_HW2_OP_CFG, PMIC_RG_VIO18_HW2_OP_CFG_ADDR, PMIC_RG_VIO18_HW2_OP_CFG_MASK,
	 PMIC_RG_VIO18_HW2_OP_CFG_SHIFT},
	{PMIC_RG_VIO18_GO_ON_OP, PMIC_RG_VIO18_GO_ON_OP_ADDR, PMIC_RG_VIO18_GO_ON_OP_MASK,
	 PMIC_RG_VIO18_GO_ON_OP_SHIFT},
	{PMIC_RG_VIO18_GO_LP_OP, PMIC_RG_VIO18_GO_LP_OP_ADDR, PMIC_RG_VIO18_GO_LP_OP_MASK,
	 PMIC_RG_VIO18_GO_LP_OP_SHIFT},
	{PMIC_RG_VIO18_OP_CFG_SET, PMIC_RG_VIO18_OP_CFG_SET_ADDR, PMIC_RG_VIO18_OP_CFG_SET_MASK,
	 PMIC_RG_VIO18_OP_CFG_SET_SHIFT},
	{PMIC_RG_VIO18_OP_CFG_CLR, PMIC_RG_VIO18_OP_CFG_CLR_ADDR, PMIC_RG_VIO18_OP_CFG_CLR_MASK,
	 PMIC_RG_VIO18_OP_CFG_CLR_SHIFT},
	{PMIC_DA_QI_VIO18_MODE, PMIC_DA_QI_VIO18_MODE_ADDR, PMIC_DA_QI_VIO18_MODE_MASK,
	 PMIC_DA_QI_VIO18_MODE_SHIFT},
	{PMIC_RG_VIO18_STBTD, PMIC_RG_VIO18_STBTD_ADDR, PMIC_RG_VIO18_STBTD_MASK,
	 PMIC_RG_VIO18_STBTD_SHIFT},
	{PMIC_DA_QI_VIO18_STB, PMIC_DA_QI_VIO18_STB_ADDR, PMIC_DA_QI_VIO18_STB_MASK,
	 PMIC_DA_QI_VIO18_STB_SHIFT},
	{PMIC_DA_QI_VIO18_EN, PMIC_DA_QI_VIO18_EN_ADDR, PMIC_DA_QI_VIO18_EN_MASK,
	 PMIC_DA_QI_VIO18_EN_SHIFT},
	{PMIC_RG_VIO18_OCFB_EN, PMIC_RG_VIO18_OCFB_EN_ADDR, PMIC_RG_VIO18_OCFB_EN_MASK,
	 PMIC_RG_VIO18_OCFB_EN_SHIFT},
	{PMIC_DA_QI_VIO18_OCFB_EN, PMIC_DA_QI_VIO18_OCFB_EN_ADDR, PMIC_DA_QI_VIO18_OCFB_EN_MASK,
	 PMIC_DA_QI_VIO18_OCFB_EN_SHIFT},
	{PMIC_RG_VIO18_DUMMY_LOAD, PMIC_RG_VIO18_DUMMY_LOAD_ADDR, PMIC_RG_VIO18_DUMMY_LOAD_MASK,
	 PMIC_RG_VIO18_DUMMY_LOAD_SHIFT},
	{PMIC_DA_QI_VIO18_DUMMY_LOAD, PMIC_DA_QI_VIO18_DUMMY_LOAD_ADDR,
	 PMIC_DA_QI_VIO18_DUMMY_LOAD_MASK, PMIC_DA_QI_VIO18_DUMMY_LOAD_SHIFT},
	{PMIC_RG_VUFS18_SW_EN, PMIC_RG_VUFS18_SW_EN_ADDR, PMIC_RG_VUFS18_SW_EN_MASK,
	 PMIC_RG_VUFS18_SW_EN_SHIFT},
	{PMIC_RG_VUFS18_SW_LP, PMIC_RG_VUFS18_SW_LP_ADDR, PMIC_RG_VUFS18_SW_LP_MASK,
	 PMIC_RG_VUFS18_SW_LP_SHIFT},
	{PMIC_RG_VUFS18_SW_OP_EN, PMIC_RG_VUFS18_SW_OP_EN_ADDR, PMIC_RG_VUFS18_SW_OP_EN_MASK,
	 PMIC_RG_VUFS18_SW_OP_EN_SHIFT},
	{PMIC_RG_VUFS18_HW0_OP_EN, PMIC_RG_VUFS18_HW0_OP_EN_ADDR, PMIC_RG_VUFS18_HW0_OP_EN_MASK,
	 PMIC_RG_VUFS18_HW0_OP_EN_SHIFT},
	{PMIC_RG_VUFS18_HW1_OP_EN, PMIC_RG_VUFS18_HW1_OP_EN_ADDR, PMIC_RG_VUFS18_HW1_OP_EN_MASK,
	 PMIC_RG_VUFS18_HW1_OP_EN_SHIFT},
	{PMIC_RG_VUFS18_HW2_OP_EN, PMIC_RG_VUFS18_HW2_OP_EN_ADDR, PMIC_RG_VUFS18_HW2_OP_EN_MASK,
	 PMIC_RG_VUFS18_HW2_OP_EN_SHIFT},
	{PMIC_RG_VUFS18_OP_EN_SET, PMIC_RG_VUFS18_OP_EN_SET_ADDR, PMIC_RG_VUFS18_OP_EN_SET_MASK,
	 PMIC_RG_VUFS18_OP_EN_SET_SHIFT},
	{PMIC_RG_VUFS18_OP_EN_CLR, PMIC_RG_VUFS18_OP_EN_CLR_ADDR, PMIC_RG_VUFS18_OP_EN_CLR_MASK,
	 PMIC_RG_VUFS18_OP_EN_CLR_SHIFT},
	{PMIC_RG_VUFS18_HW0_OP_CFG, PMIC_RG_VUFS18_HW0_OP_CFG_ADDR, PMIC_RG_VUFS18_HW0_OP_CFG_MASK,
	 PMIC_RG_VUFS18_HW0_OP_CFG_SHIFT},
	{PMIC_RG_VUFS18_HW1_OP_CFG, PMIC_RG_VUFS18_HW1_OP_CFG_ADDR, PMIC_RG_VUFS18_HW1_OP_CFG_MASK,
	 PMIC_RG_VUFS18_HW1_OP_CFG_SHIFT},
	{PMIC_RG_VUFS18_HW2_OP_CFG, PMIC_RG_VUFS18_HW2_OP_CFG_ADDR, PMIC_RG_VUFS18_HW2_OP_CFG_MASK,
	 PMIC_RG_VUFS18_HW2_OP_CFG_SHIFT},
	{PMIC_RG_VUFS18_GO_ON_OP, PMIC_RG_VUFS18_GO_ON_OP_ADDR, PMIC_RG_VUFS18_GO_ON_OP_MASK,
	 PMIC_RG_VUFS18_GO_ON_OP_SHIFT},
	{PMIC_RG_VUFS18_GO_LP_OP, PMIC_RG_VUFS18_GO_LP_OP_ADDR, PMIC_RG_VUFS18_GO_LP_OP_MASK,
	 PMIC_RG_VUFS18_GO_LP_OP_SHIFT},
	{PMIC_RG_VUFS18_OP_CFG_SET, PMIC_RG_VUFS18_OP_CFG_SET_ADDR, PMIC_RG_VUFS18_OP_CFG_SET_MASK,
	 PMIC_RG_VUFS18_OP_CFG_SET_SHIFT},
	{PMIC_RG_VUFS18_OP_CFG_CLR, PMIC_RG_VUFS18_OP_CFG_CLR_ADDR, PMIC_RG_VUFS18_OP_CFG_CLR_MASK,
	 PMIC_RG_VUFS18_OP_CFG_CLR_SHIFT},
	{PMIC_DA_QI_VUFS18_MODE, PMIC_DA_QI_VUFS18_MODE_ADDR, PMIC_DA_QI_VUFS18_MODE_MASK,
	 PMIC_DA_QI_VUFS18_MODE_SHIFT},
	{PMIC_RG_VUFS18_STBTD, PMIC_RG_VUFS18_STBTD_ADDR, PMIC_RG_VUFS18_STBTD_MASK,
	 PMIC_RG_VUFS18_STBTD_SHIFT},
	{PMIC_DA_QI_VUFS18_STB, PMIC_DA_QI_VUFS18_STB_ADDR, PMIC_DA_QI_VUFS18_STB_MASK,
	 PMIC_DA_QI_VUFS18_STB_SHIFT},
	{PMIC_DA_QI_VUFS18_EN, PMIC_DA_QI_VUFS18_EN_ADDR, PMIC_DA_QI_VUFS18_EN_MASK,
	 PMIC_DA_QI_VUFS18_EN_SHIFT},
	{PMIC_RG_VUFS18_OCFB_EN, PMIC_RG_VUFS18_OCFB_EN_ADDR, PMIC_RG_VUFS18_OCFB_EN_MASK,
	 PMIC_RG_VUFS18_OCFB_EN_SHIFT},
	{PMIC_DA_QI_VUFS18_OCFB_EN, PMIC_DA_QI_VUFS18_OCFB_EN_ADDR, PMIC_DA_QI_VUFS18_OCFB_EN_MASK,
	 PMIC_DA_QI_VUFS18_OCFB_EN_SHIFT},
	{PMIC_RG_VUFS18_DUMMY_LOAD, PMIC_RG_VUFS18_DUMMY_LOAD_ADDR, PMIC_RG_VUFS18_DUMMY_LOAD_MASK,
	 PMIC_RG_VUFS18_DUMMY_LOAD_SHIFT},
	{PMIC_DA_QI_VUFS18_DUMMY_LOAD, PMIC_DA_QI_VUFS18_DUMMY_LOAD_ADDR,
	 PMIC_DA_QI_VUFS18_DUMMY_LOAD_MASK, PMIC_DA_QI_VUFS18_DUMMY_LOAD_SHIFT},
	{PMIC_RG_VA10_SW_EN, PMIC_RG_VA10_SW_EN_ADDR, PMIC_RG_VA10_SW_EN_MASK,
	 PMIC_RG_VA10_SW_EN_SHIFT},
	{PMIC_RG_VA10_SW_LP, PMIC_RG_VA10_SW_LP_ADDR, PMIC_RG_VA10_SW_LP_MASK,
	 PMIC_RG_VA10_SW_LP_SHIFT},
	{PMIC_RG_VA10_SW_OP_EN, PMIC_RG_VA10_SW_OP_EN_ADDR, PMIC_RG_VA10_SW_OP_EN_MASK,
	 PMIC_RG_VA10_SW_OP_EN_SHIFT},
	{PMIC_RG_VA10_HW0_OP_EN, PMIC_RG_VA10_HW0_OP_EN_ADDR, PMIC_RG_VA10_HW0_OP_EN_MASK,
	 PMIC_RG_VA10_HW0_OP_EN_SHIFT},
	{PMIC_RG_VA10_HW1_OP_EN, PMIC_RG_VA10_HW1_OP_EN_ADDR, PMIC_RG_VA10_HW1_OP_EN_MASK,
	 PMIC_RG_VA10_HW1_OP_EN_SHIFT},
	{PMIC_RG_VA10_HW2_OP_EN, PMIC_RG_VA10_HW2_OP_EN_ADDR, PMIC_RG_VA10_HW2_OP_EN_MASK,
	 PMIC_RG_VA10_HW2_OP_EN_SHIFT},
	{PMIC_RG_VA10_OP_EN_SET, PMIC_RG_VA10_OP_EN_SET_ADDR, PMIC_RG_VA10_OP_EN_SET_MASK,
	 PMIC_RG_VA10_OP_EN_SET_SHIFT},
	{PMIC_RG_VA10_OP_EN_CLR, PMIC_RG_VA10_OP_EN_CLR_ADDR, PMIC_RG_VA10_OP_EN_CLR_MASK,
	 PMIC_RG_VA10_OP_EN_CLR_SHIFT},
	{PMIC_RG_VA10_HW0_OP_CFG, PMIC_RG_VA10_HW0_OP_CFG_ADDR, PMIC_RG_VA10_HW0_OP_CFG_MASK,
	 PMIC_RG_VA10_HW0_OP_CFG_SHIFT},
	{PMIC_RG_VA10_HW1_OP_CFG, PMIC_RG_VA10_HW1_OP_CFG_ADDR, PMIC_RG_VA10_HW1_OP_CFG_MASK,
	 PMIC_RG_VA10_HW1_OP_CFG_SHIFT},
	{PMIC_RG_VA10_HW2_OP_CFG, PMIC_RG_VA10_HW2_OP_CFG_ADDR, PMIC_RG_VA10_HW2_OP_CFG_MASK,
	 PMIC_RG_VA10_HW2_OP_CFG_SHIFT},
	{PMIC_RG_VA10_GO_ON_OP, PMIC_RG_VA10_GO_ON_OP_ADDR, PMIC_RG_VA10_GO_ON_OP_MASK,
	 PMIC_RG_VA10_GO_ON_OP_SHIFT},
	{PMIC_RG_VA10_GO_LP_OP, PMIC_RG_VA10_GO_LP_OP_ADDR, PMIC_RG_VA10_GO_LP_OP_MASK,
	 PMIC_RG_VA10_GO_LP_OP_SHIFT},
	{PMIC_RG_VA10_OP_CFG_SET, PMIC_RG_VA10_OP_CFG_SET_ADDR, PMIC_RG_VA10_OP_CFG_SET_MASK,
	 PMIC_RG_VA10_OP_CFG_SET_SHIFT},
	{PMIC_RG_VA10_OP_CFG_CLR, PMIC_RG_VA10_OP_CFG_CLR_ADDR, PMIC_RG_VA10_OP_CFG_CLR_MASK,
	 PMIC_RG_VA10_OP_CFG_CLR_SHIFT},
	{PMIC_DA_QI_VA10_MODE, PMIC_DA_QI_VA10_MODE_ADDR, PMIC_DA_QI_VA10_MODE_MASK,
	 PMIC_DA_QI_VA10_MODE_SHIFT},
	{PMIC_RG_VA10_STBTD, PMIC_RG_VA10_STBTD_ADDR, PMIC_RG_VA10_STBTD_MASK,
	 PMIC_RG_VA10_STBTD_SHIFT},
	{PMIC_DA_QI_VA10_STB, PMIC_DA_QI_VA10_STB_ADDR, PMIC_DA_QI_VA10_STB_MASK,
	 PMIC_DA_QI_VA10_STB_SHIFT},
	{PMIC_DA_QI_VA10_EN, PMIC_DA_QI_VA10_EN_ADDR, PMIC_DA_QI_VA10_EN_MASK,
	 PMIC_DA_QI_VA10_EN_SHIFT},
	{PMIC_RG_VA10_WK_TD, PMIC_RG_VA10_WK_TD_ADDR, PMIC_RG_VA10_WK_TD_MASK,
	 PMIC_RG_VA10_WK_TD_SHIFT},
	{PMIC_RG_VA10_SLP_TD, PMIC_RG_VA10_SLP_TD_ADDR, PMIC_RG_VA10_SLP_TD_MASK,
	 PMIC_RG_VA10_SLP_TD_SHIFT},
	{PMIC_RG_VA10_OCFB_EN, PMIC_RG_VA10_OCFB_EN_ADDR, PMIC_RG_VA10_OCFB_EN_MASK,
	 PMIC_RG_VA10_OCFB_EN_SHIFT},
	{PMIC_DA_QI_VA10_OCFB_EN, PMIC_DA_QI_VA10_OCFB_EN_ADDR, PMIC_DA_QI_VA10_OCFB_EN_MASK,
	 PMIC_DA_QI_VA10_OCFB_EN_SHIFT},
	{PMIC_RG_VA10_DUMMY_LOAD, PMIC_RG_VA10_DUMMY_LOAD_ADDR, PMIC_RG_VA10_DUMMY_LOAD_MASK,
	 PMIC_RG_VA10_DUMMY_LOAD_SHIFT},
	{PMIC_DA_QI_VA10_DUMMY_LOAD, PMIC_DA_QI_VA10_DUMMY_LOAD_ADDR,
	 PMIC_DA_QI_VA10_DUMMY_LOAD_MASK, PMIC_DA_QI_VA10_DUMMY_LOAD_SHIFT},
	{PMIC_RG_VA12_SW_EN, PMIC_RG_VA12_SW_EN_ADDR, PMIC_RG_VA12_SW_EN_MASK,
	 PMIC_RG_VA12_SW_EN_SHIFT},
	{PMIC_RG_VA12_SW_LP, PMIC_RG_VA12_SW_LP_ADDR, PMIC_RG_VA12_SW_LP_MASK,
	 PMIC_RG_VA12_SW_LP_SHIFT},
	{PMIC_RG_VA12_SW_OP_EN, PMIC_RG_VA12_SW_OP_EN_ADDR, PMIC_RG_VA12_SW_OP_EN_MASK,
	 PMIC_RG_VA12_SW_OP_EN_SHIFT},
	{PMIC_RG_VA12_HW0_OP_EN, PMIC_RG_VA12_HW0_OP_EN_ADDR, PMIC_RG_VA12_HW0_OP_EN_MASK,
	 PMIC_RG_VA12_HW0_OP_EN_SHIFT},
	{PMIC_RG_VA12_HW1_OP_EN, PMIC_RG_VA12_HW1_OP_EN_ADDR, PMIC_RG_VA12_HW1_OP_EN_MASK,
	 PMIC_RG_VA12_HW1_OP_EN_SHIFT},
	{PMIC_RG_VA12_HW2_OP_EN, PMIC_RG_VA12_HW2_OP_EN_ADDR, PMIC_RG_VA12_HW2_OP_EN_MASK,
	 PMIC_RG_VA12_HW2_OP_EN_SHIFT},
	{PMIC_RG_VA12_OP_EN_SET, PMIC_RG_VA12_OP_EN_SET_ADDR, PMIC_RG_VA12_OP_EN_SET_MASK,
	 PMIC_RG_VA12_OP_EN_SET_SHIFT},
	{PMIC_RG_VA12_OP_EN_CLR, PMIC_RG_VA12_OP_EN_CLR_ADDR, PMIC_RG_VA12_OP_EN_CLR_MASK,
	 PMIC_RG_VA12_OP_EN_CLR_SHIFT},
	{PMIC_RG_VA12_HW0_OP_CFG, PMIC_RG_VA12_HW0_OP_CFG_ADDR, PMIC_RG_VA12_HW0_OP_CFG_MASK,
	 PMIC_RG_VA12_HW0_OP_CFG_SHIFT},
	{PMIC_RG_VA12_HW1_OP_CFG, PMIC_RG_VA12_HW1_OP_CFG_ADDR, PMIC_RG_VA12_HW1_OP_CFG_MASK,
	 PMIC_RG_VA12_HW1_OP_CFG_SHIFT},
	{PMIC_RG_VA12_HW2_OP_CFG, PMIC_RG_VA12_HW2_OP_CFG_ADDR, PMIC_RG_VA12_HW2_OP_CFG_MASK,
	 PMIC_RG_VA12_HW2_OP_CFG_SHIFT},
	{PMIC_RG_VA12_GO_ON_OP, PMIC_RG_VA12_GO_ON_OP_ADDR, PMIC_RG_VA12_GO_ON_OP_MASK,
	 PMIC_RG_VA12_GO_ON_OP_SHIFT},
	{PMIC_RG_VA12_GO_LP_OP, PMIC_RG_VA12_GO_LP_OP_ADDR, PMIC_RG_VA12_GO_LP_OP_MASK,
	 PMIC_RG_VA12_GO_LP_OP_SHIFT},
	{PMIC_RG_VA12_OP_CFG_SET, PMIC_RG_VA12_OP_CFG_SET_ADDR, PMIC_RG_VA12_OP_CFG_SET_MASK,
	 PMIC_RG_VA12_OP_CFG_SET_SHIFT},
	{PMIC_RG_VA12_OP_CFG_CLR, PMIC_RG_VA12_OP_CFG_CLR_ADDR, PMIC_RG_VA12_OP_CFG_CLR_MASK,
	 PMIC_RG_VA12_OP_CFG_CLR_SHIFT},
	{PMIC_DA_QI_VA12_MODE, PMIC_DA_QI_VA12_MODE_ADDR, PMIC_DA_QI_VA12_MODE_MASK,
	 PMIC_DA_QI_VA12_MODE_SHIFT},
	{PMIC_RG_VA12_STBTD, PMIC_RG_VA12_STBTD_ADDR, PMIC_RG_VA12_STBTD_MASK,
	 PMIC_RG_VA12_STBTD_SHIFT},
	{PMIC_DA_QI_VA12_STB, PMIC_DA_QI_VA12_STB_ADDR, PMIC_DA_QI_VA12_STB_MASK,
	 PMIC_DA_QI_VA12_STB_SHIFT},
	{PMIC_DA_QI_VA12_EN, PMIC_DA_QI_VA12_EN_ADDR, PMIC_DA_QI_VA12_EN_MASK,
	 PMIC_DA_QI_VA12_EN_SHIFT},
	{PMIC_RG_VA12_OCFB_EN, PMIC_RG_VA12_OCFB_EN_ADDR, PMIC_RG_VA12_OCFB_EN_MASK,
	 PMIC_RG_VA12_OCFB_EN_SHIFT},
	{PMIC_DA_QI_VA12_OCFB_EN, PMIC_DA_QI_VA12_OCFB_EN_ADDR, PMIC_DA_QI_VA12_OCFB_EN_MASK,
	 PMIC_DA_QI_VA12_OCFB_EN_SHIFT},
	{PMIC_RG_VA12_DUMMY_LOAD, PMIC_RG_VA12_DUMMY_LOAD_ADDR, PMIC_RG_VA12_DUMMY_LOAD_MASK,
	 PMIC_RG_VA12_DUMMY_LOAD_SHIFT},
	{PMIC_DA_QI_VA12_DUMMY_LOAD, PMIC_DA_QI_VA12_DUMMY_LOAD_ADDR,
	 PMIC_DA_QI_VA12_DUMMY_LOAD_MASK, PMIC_DA_QI_VA12_DUMMY_LOAD_SHIFT},
	{PMIC_RG_VA18_SW_EN, PMIC_RG_VA18_SW_EN_ADDR, PMIC_RG_VA18_SW_EN_MASK,
	 PMIC_RG_VA18_SW_EN_SHIFT},
	{PMIC_RG_VA18_SW_LP, PMIC_RG_VA18_SW_LP_ADDR, PMIC_RG_VA18_SW_LP_MASK,
	 PMIC_RG_VA18_SW_LP_SHIFT},
	{PMIC_RG_VA18_SW_OP_EN, PMIC_RG_VA18_SW_OP_EN_ADDR, PMIC_RG_VA18_SW_OP_EN_MASK,
	 PMIC_RG_VA18_SW_OP_EN_SHIFT},
	{PMIC_RG_VA18_HW0_OP_EN, PMIC_RG_VA18_HW0_OP_EN_ADDR, PMIC_RG_VA18_HW0_OP_EN_MASK,
	 PMIC_RG_VA18_HW0_OP_EN_SHIFT},
	{PMIC_RG_VA18_HW1_OP_EN, PMIC_RG_VA18_HW1_OP_EN_ADDR, PMIC_RG_VA18_HW1_OP_EN_MASK,
	 PMIC_RG_VA18_HW1_OP_EN_SHIFT},
	{PMIC_RG_VA18_HW2_OP_EN, PMIC_RG_VA18_HW2_OP_EN_ADDR, PMIC_RG_VA18_HW2_OP_EN_MASK,
	 PMIC_RG_VA18_HW2_OP_EN_SHIFT},
	{PMIC_RG_VA18_OP_EN_SET, PMIC_RG_VA18_OP_EN_SET_ADDR, PMIC_RG_VA18_OP_EN_SET_MASK,
	 PMIC_RG_VA18_OP_EN_SET_SHIFT},
	{PMIC_RG_VA18_OP_EN_CLR, PMIC_RG_VA18_OP_EN_CLR_ADDR, PMIC_RG_VA18_OP_EN_CLR_MASK,
	 PMIC_RG_VA18_OP_EN_CLR_SHIFT},
	{PMIC_RG_VA18_HW0_OP_CFG, PMIC_RG_VA18_HW0_OP_CFG_ADDR, PMIC_RG_VA18_HW0_OP_CFG_MASK,
	 PMIC_RG_VA18_HW0_OP_CFG_SHIFT},
	{PMIC_RG_VA18_HW1_OP_CFG, PMIC_RG_VA18_HW1_OP_CFG_ADDR, PMIC_RG_VA18_HW1_OP_CFG_MASK,
	 PMIC_RG_VA18_HW1_OP_CFG_SHIFT},
	{PMIC_RG_VA18_HW2_OP_CFG, PMIC_RG_VA18_HW2_OP_CFG_ADDR, PMIC_RG_VA18_HW2_OP_CFG_MASK,
	 PMIC_RG_VA18_HW2_OP_CFG_SHIFT},
	{PMIC_RG_VA18_GO_ON_OP, PMIC_RG_VA18_GO_ON_OP_ADDR, PMIC_RG_VA18_GO_ON_OP_MASK,
	 PMIC_RG_VA18_GO_ON_OP_SHIFT},
	{PMIC_RG_VA18_GO_LP_OP, PMIC_RG_VA18_GO_LP_OP_ADDR, PMIC_RG_VA18_GO_LP_OP_MASK,
	 PMIC_RG_VA18_GO_LP_OP_SHIFT},
	{PMIC_RG_VA18_OP_CFG_SET, PMIC_RG_VA18_OP_CFG_SET_ADDR, PMIC_RG_VA18_OP_CFG_SET_MASK,
	 PMIC_RG_VA18_OP_CFG_SET_SHIFT},
	{PMIC_RG_VA18_OP_CFG_CLR, PMIC_RG_VA18_OP_CFG_CLR_ADDR, PMIC_RG_VA18_OP_CFG_CLR_MASK,
	 PMIC_RG_VA18_OP_CFG_CLR_SHIFT},
	{PMIC_DA_QI_VA18_MODE, PMIC_DA_QI_VA18_MODE_ADDR, PMIC_DA_QI_VA18_MODE_MASK,
	 PMIC_DA_QI_VA18_MODE_SHIFT},
	{PMIC_RG_VA18_STBTD, PMIC_RG_VA18_STBTD_ADDR, PMIC_RG_VA18_STBTD_MASK,
	 PMIC_RG_VA18_STBTD_SHIFT},
	{PMIC_DA_QI_VA18_STB, PMIC_DA_QI_VA18_STB_ADDR, PMIC_DA_QI_VA18_STB_MASK,
	 PMIC_DA_QI_VA18_STB_SHIFT},
	{PMIC_DA_QI_VA18_EN, PMIC_DA_QI_VA18_EN_ADDR, PMIC_DA_QI_VA18_EN_MASK,
	 PMIC_DA_QI_VA18_EN_SHIFT},
	{PMIC_RG_VA18_AUXADC_PWDB_EN, PMIC_RG_VA18_AUXADC_PWDB_EN_ADDR,
	 PMIC_RG_VA18_AUXADC_PWDB_EN_MASK, PMIC_RG_VA18_AUXADC_PWDB_EN_SHIFT},
	{PMIC_RG_VA18_OCFB_EN, PMIC_RG_VA18_OCFB_EN_ADDR, PMIC_RG_VA18_OCFB_EN_MASK,
	 PMIC_RG_VA18_OCFB_EN_SHIFT},
	{PMIC_DA_QI_VA18_OCFB_EN, PMIC_DA_QI_VA18_OCFB_EN_ADDR, PMIC_DA_QI_VA18_OCFB_EN_MASK,
	 PMIC_DA_QI_VA18_OCFB_EN_SHIFT},
	{PMIC_RG_VUSB33_SW_EN, PMIC_RG_VUSB33_SW_EN_ADDR, PMIC_RG_VUSB33_SW_EN_MASK,
	 PMIC_RG_VUSB33_SW_EN_SHIFT},
	{PMIC_RG_VUSB33_SW_LP, PMIC_RG_VUSB33_SW_LP_ADDR, PMIC_RG_VUSB33_SW_LP_MASK,
	 PMIC_RG_VUSB33_SW_LP_SHIFT},
	{PMIC_RG_VUSB33_SW_OP_EN, PMIC_RG_VUSB33_SW_OP_EN_ADDR, PMIC_RG_VUSB33_SW_OP_EN_MASK,
	 PMIC_RG_VUSB33_SW_OP_EN_SHIFT},
	{PMIC_RG_VUSB33_HW0_OP_EN, PMIC_RG_VUSB33_HW0_OP_EN_ADDR, PMIC_RG_VUSB33_HW0_OP_EN_MASK,
	 PMIC_RG_VUSB33_HW0_OP_EN_SHIFT},
	{PMIC_RG_VUSB33_HW1_OP_EN, PMIC_RG_VUSB33_HW1_OP_EN_ADDR, PMIC_RG_VUSB33_HW1_OP_EN_MASK,
	 PMIC_RG_VUSB33_HW1_OP_EN_SHIFT},
	{PMIC_RG_VUSB33_HW2_OP_EN, PMIC_RG_VUSB33_HW2_OP_EN_ADDR, PMIC_RG_VUSB33_HW2_OP_EN_MASK,
	 PMIC_RG_VUSB33_HW2_OP_EN_SHIFT},
	{PMIC_RG_VUSB33_OP_EN_SET, PMIC_RG_VUSB33_OP_EN_SET_ADDR, PMIC_RG_VUSB33_OP_EN_SET_MASK,
	 PMIC_RG_VUSB33_OP_EN_SET_SHIFT},
	{PMIC_RG_VUSB33_OP_EN_CLR, PMIC_RG_VUSB33_OP_EN_CLR_ADDR, PMIC_RG_VUSB33_OP_EN_CLR_MASK,
	 PMIC_RG_VUSB33_OP_EN_CLR_SHIFT},
	{PMIC_RG_VUSB33_HW0_OP_CFG, PMIC_RG_VUSB33_HW0_OP_CFG_ADDR, PMIC_RG_VUSB33_HW0_OP_CFG_MASK,
	 PMIC_RG_VUSB33_HW0_OP_CFG_SHIFT},
	{PMIC_RG_VUSB33_HW1_OP_CFG, PMIC_RG_VUSB33_HW1_OP_CFG_ADDR, PMIC_RG_VUSB33_HW1_OP_CFG_MASK,
	 PMIC_RG_VUSB33_HW1_OP_CFG_SHIFT},
	{PMIC_RG_VUSB33_HW2_OP_CFG, PMIC_RG_VUSB33_HW2_OP_CFG_ADDR, PMIC_RG_VUSB33_HW2_OP_CFG_MASK,
	 PMIC_RG_VUSB33_HW2_OP_CFG_SHIFT},
	{PMIC_RG_VUSB33_GO_ON_OP, PMIC_RG_VUSB33_GO_ON_OP_ADDR, PMIC_RG_VUSB33_GO_ON_OP_MASK,
	 PMIC_RG_VUSB33_GO_ON_OP_SHIFT},
	{PMIC_RG_VUSB33_GO_LP_OP, PMIC_RG_VUSB33_GO_LP_OP_ADDR, PMIC_RG_VUSB33_GO_LP_OP_MASK,
	 PMIC_RG_VUSB33_GO_LP_OP_SHIFT},
	{PMIC_RG_VUSB33_OP_CFG_SET, PMIC_RG_VUSB33_OP_CFG_SET_ADDR, PMIC_RG_VUSB33_OP_CFG_SET_MASK,
	 PMIC_RG_VUSB33_OP_CFG_SET_SHIFT},
	{PMIC_RG_VUSB33_OP_CFG_CLR, PMIC_RG_VUSB33_OP_CFG_CLR_ADDR, PMIC_RG_VUSB33_OP_CFG_CLR_MASK,
	 PMIC_RG_VUSB33_OP_CFG_CLR_SHIFT},
	{PMIC_DA_QI_VUSB33_MODE, PMIC_DA_QI_VUSB33_MODE_ADDR, PMIC_DA_QI_VUSB33_MODE_MASK,
	 PMIC_DA_QI_VUSB33_MODE_SHIFT},
	{PMIC_RG_VUSB33_STBTD, PMIC_RG_VUSB33_STBTD_ADDR, PMIC_RG_VUSB33_STBTD_MASK,
	 PMIC_RG_VUSB33_STBTD_SHIFT},
	{PMIC_DA_QI_VUSB33_STB, PMIC_DA_QI_VUSB33_STB_ADDR, PMIC_DA_QI_VUSB33_STB_MASK,
	 PMIC_DA_QI_VUSB33_STB_SHIFT},
	{PMIC_DA_QI_VUSB33_EN, PMIC_DA_QI_VUSB33_EN_ADDR, PMIC_DA_QI_VUSB33_EN_MASK,
	 PMIC_DA_QI_VUSB33_EN_SHIFT},
	{PMIC_RG_VUSB33_OCFB_EN, PMIC_RG_VUSB33_OCFB_EN_ADDR, PMIC_RG_VUSB33_OCFB_EN_MASK,
	 PMIC_RG_VUSB33_OCFB_EN_SHIFT},
	{PMIC_DA_QI_VUSB33_OCFB_EN, PMIC_DA_QI_VUSB33_OCFB_EN_ADDR, PMIC_DA_QI_VUSB33_OCFB_EN_MASK,
	 PMIC_DA_QI_VUSB33_OCFB_EN_SHIFT},
	{PMIC_RG_VEMC_SW_EN, PMIC_RG_VEMC_SW_EN_ADDR, PMIC_RG_VEMC_SW_EN_MASK,
	 PMIC_RG_VEMC_SW_EN_SHIFT},
	{PMIC_RG_VEMC_SW_LP, PMIC_RG_VEMC_SW_LP_ADDR, PMIC_RG_VEMC_SW_LP_MASK,
	 PMIC_RG_VEMC_SW_LP_SHIFT},
	{PMIC_RG_VEMC_SW_OP_EN, PMIC_RG_VEMC_SW_OP_EN_ADDR, PMIC_RG_VEMC_SW_OP_EN_MASK,
	 PMIC_RG_VEMC_SW_OP_EN_SHIFT},
	{PMIC_RG_VEMC_HW0_OP_EN, PMIC_RG_VEMC_HW0_OP_EN_ADDR, PMIC_RG_VEMC_HW0_OP_EN_MASK,
	 PMIC_RG_VEMC_HW0_OP_EN_SHIFT},
	{PMIC_RG_VEMC_HW1_OP_EN, PMIC_RG_VEMC_HW1_OP_EN_ADDR, PMIC_RG_VEMC_HW1_OP_EN_MASK,
	 PMIC_RG_VEMC_HW1_OP_EN_SHIFT},
	{PMIC_RG_VEMC_HW2_OP_EN, PMIC_RG_VEMC_HW2_OP_EN_ADDR, PMIC_RG_VEMC_HW2_OP_EN_MASK,
	 PMIC_RG_VEMC_HW2_OP_EN_SHIFT},
	{PMIC_RG_VEMC_OP_EN_SET, PMIC_RG_VEMC_OP_EN_SET_ADDR, PMIC_RG_VEMC_OP_EN_SET_MASK,
	 PMIC_RG_VEMC_OP_EN_SET_SHIFT},
	{PMIC_RG_VEMC_OP_EN_CLR, PMIC_RG_VEMC_OP_EN_CLR_ADDR, PMIC_RG_VEMC_OP_EN_CLR_MASK,
	 PMIC_RG_VEMC_OP_EN_CLR_SHIFT},
	{PMIC_RG_VEMC_HW0_OP_CFG, PMIC_RG_VEMC_HW0_OP_CFG_ADDR, PMIC_RG_VEMC_HW0_OP_CFG_MASK,
	 PMIC_RG_VEMC_HW0_OP_CFG_SHIFT},
	{PMIC_RG_VEMC_HW1_OP_CFG, PMIC_RG_VEMC_HW1_OP_CFG_ADDR, PMIC_RG_VEMC_HW1_OP_CFG_MASK,
	 PMIC_RG_VEMC_HW1_OP_CFG_SHIFT},
	{PMIC_RG_VEMC_HW2_OP_CFG, PMIC_RG_VEMC_HW2_OP_CFG_ADDR, PMIC_RG_VEMC_HW2_OP_CFG_MASK,
	 PMIC_RG_VEMC_HW2_OP_CFG_SHIFT},
	{PMIC_RG_VEMC_GO_ON_OP, PMIC_RG_VEMC_GO_ON_OP_ADDR, PMIC_RG_VEMC_GO_ON_OP_MASK,
	 PMIC_RG_VEMC_GO_ON_OP_SHIFT},
	{PMIC_RG_VEMC_GO_LP_OP, PMIC_RG_VEMC_GO_LP_OP_ADDR, PMIC_RG_VEMC_GO_LP_OP_MASK,
	 PMIC_RG_VEMC_GO_LP_OP_SHIFT},
	{PMIC_RG_VEMC_OP_CFG_SET, PMIC_RG_VEMC_OP_CFG_SET_ADDR, PMIC_RG_VEMC_OP_CFG_SET_MASK,
	 PMIC_RG_VEMC_OP_CFG_SET_SHIFT},
	{PMIC_RG_VEMC_OP_CFG_CLR, PMIC_RG_VEMC_OP_CFG_CLR_ADDR, PMIC_RG_VEMC_OP_CFG_CLR_MASK,
	 PMIC_RG_VEMC_OP_CFG_CLR_SHIFT},
	{PMIC_DA_QI_VEMC_MODE, PMIC_DA_QI_VEMC_MODE_ADDR, PMIC_DA_QI_VEMC_MODE_MASK,
	 PMIC_DA_QI_VEMC_MODE_SHIFT},
	{PMIC_RG_VEMC_STBTD, PMIC_RG_VEMC_STBTD_ADDR, PMIC_RG_VEMC_STBTD_MASK,
	 PMIC_RG_VEMC_STBTD_SHIFT},
	{PMIC_DA_QI_VEMC_STB, PMIC_DA_QI_VEMC_STB_ADDR, PMIC_DA_QI_VEMC_STB_MASK,
	 PMIC_DA_QI_VEMC_STB_SHIFT},
	{PMIC_DA_QI_VEMC_EN, PMIC_DA_QI_VEMC_EN_ADDR, PMIC_DA_QI_VEMC_EN_MASK,
	 PMIC_DA_QI_VEMC_EN_SHIFT},
	{PMIC_RG_VEMC_OCFB_EN, PMIC_RG_VEMC_OCFB_EN_ADDR, PMIC_RG_VEMC_OCFB_EN_MASK,
	 PMIC_RG_VEMC_OCFB_EN_SHIFT},
	{PMIC_DA_QI_VEMC_OCFB_EN, PMIC_DA_QI_VEMC_OCFB_EN_ADDR, PMIC_DA_QI_VEMC_OCFB_EN_MASK,
	 PMIC_DA_QI_VEMC_OCFB_EN_SHIFT},
	{PMIC_RG_VEMC_DUMMY_LOAD, PMIC_RG_VEMC_DUMMY_LOAD_ADDR, PMIC_RG_VEMC_DUMMY_LOAD_MASK,
	 PMIC_RG_VEMC_DUMMY_LOAD_SHIFT},
	{PMIC_DA_QI_VEMC_DUMMY_LOAD, PMIC_DA_QI_VEMC_DUMMY_LOAD_ADDR,
	 PMIC_DA_QI_VEMC_DUMMY_LOAD_MASK, PMIC_DA_QI_VEMC_DUMMY_LOAD_SHIFT},
	{PMIC_RG_VXO22_SW_EN, PMIC_RG_VXO22_SW_EN_ADDR, PMIC_RG_VXO22_SW_EN_MASK,
	 PMIC_RG_VXO22_SW_EN_SHIFT},
	{PMIC_RG_VXO22_SW_LP, PMIC_RG_VXO22_SW_LP_ADDR, PMIC_RG_VXO22_SW_LP_MASK,
	 PMIC_RG_VXO22_SW_LP_SHIFT},
	{PMIC_RG_VXO22_SW_OP_EN, PMIC_RG_VXO22_SW_OP_EN_ADDR, PMIC_RG_VXO22_SW_OP_EN_MASK,
	 PMIC_RG_VXO22_SW_OP_EN_SHIFT},
	{PMIC_RG_VXO22_HW0_OP_EN, PMIC_RG_VXO22_HW0_OP_EN_ADDR, PMIC_RG_VXO22_HW0_OP_EN_MASK,
	 PMIC_RG_VXO22_HW0_OP_EN_SHIFT},
	{PMIC_RG_VXO22_HW1_OP_EN, PMIC_RG_VXO22_HW1_OP_EN_ADDR, PMIC_RG_VXO22_HW1_OP_EN_MASK,
	 PMIC_RG_VXO22_HW1_OP_EN_SHIFT},
	{PMIC_RG_VXO22_HW2_OP_EN, PMIC_RG_VXO22_HW2_OP_EN_ADDR, PMIC_RG_VXO22_HW2_OP_EN_MASK,
	 PMIC_RG_VXO22_HW2_OP_EN_SHIFT},
	{PMIC_RG_VXO22_OP_EN_SET, PMIC_RG_VXO22_OP_EN_SET_ADDR, PMIC_RG_VXO22_OP_EN_SET_MASK,
	 PMIC_RG_VXO22_OP_EN_SET_SHIFT},
	{PMIC_RG_VXO22_OP_EN_CLR, PMIC_RG_VXO22_OP_EN_CLR_ADDR, PMIC_RG_VXO22_OP_EN_CLR_MASK,
	 PMIC_RG_VXO22_OP_EN_CLR_SHIFT},
	{PMIC_RG_VXO22_HW0_OP_CFG, PMIC_RG_VXO22_HW0_OP_CFG_ADDR, PMIC_RG_VXO22_HW0_OP_CFG_MASK,
	 PMIC_RG_VXO22_HW0_OP_CFG_SHIFT},
	{PMIC_RG_VXO22_HW1_OP_CFG, PMIC_RG_VXO22_HW1_OP_CFG_ADDR, PMIC_RG_VXO22_HW1_OP_CFG_MASK,
	 PMIC_RG_VXO22_HW1_OP_CFG_SHIFT},
	{PMIC_RG_VXO22_HW2_OP_CFG, PMIC_RG_VXO22_HW2_OP_CFG_ADDR, PMIC_RG_VXO22_HW2_OP_CFG_MASK,
	 PMIC_RG_VXO22_HW2_OP_CFG_SHIFT},
	{PMIC_RG_VXO22_GO_ON_OP, PMIC_RG_VXO22_GO_ON_OP_ADDR, PMIC_RG_VXO22_GO_ON_OP_MASK,
	 PMIC_RG_VXO22_GO_ON_OP_SHIFT},
	{PMIC_RG_VXO22_GO_LP_OP, PMIC_RG_VXO22_GO_LP_OP_ADDR, PMIC_RG_VXO22_GO_LP_OP_MASK,
	 PMIC_RG_VXO22_GO_LP_OP_SHIFT},
	{PMIC_RG_VXO22_OP_CFG_SET, PMIC_RG_VXO22_OP_CFG_SET_ADDR, PMIC_RG_VXO22_OP_CFG_SET_MASK,
	 PMIC_RG_VXO22_OP_CFG_SET_SHIFT},
	{PMIC_RG_VXO22_OP_CFG_CLR, PMIC_RG_VXO22_OP_CFG_CLR_ADDR, PMIC_RG_VXO22_OP_CFG_CLR_MASK,
	 PMIC_RG_VXO22_OP_CFG_CLR_SHIFT},
	{PMIC_DA_QI_VXO22_MODE, PMIC_DA_QI_VXO22_MODE_ADDR, PMIC_DA_QI_VXO22_MODE_MASK,
	 PMIC_DA_QI_VXO22_MODE_SHIFT},
	{PMIC_RG_VXO22_STBTD, PMIC_RG_VXO22_STBTD_ADDR, PMIC_RG_VXO22_STBTD_MASK,
	 PMIC_RG_VXO22_STBTD_SHIFT},
	{PMIC_DA_QI_VXO22_STB, PMIC_DA_QI_VXO22_STB_ADDR, PMIC_DA_QI_VXO22_STB_MASK,
	 PMIC_DA_QI_VXO22_STB_SHIFT},
	{PMIC_DA_QI_VXO22_EN, PMIC_DA_QI_VXO22_EN_ADDR, PMIC_DA_QI_VXO22_EN_MASK,
	 PMIC_DA_QI_VXO22_EN_SHIFT},
	{PMIC_RG_VXO22_OCFB_EN, PMIC_RG_VXO22_OCFB_EN_ADDR, PMIC_RG_VXO22_OCFB_EN_MASK,
	 PMIC_RG_VXO22_OCFB_EN_SHIFT},
	{PMIC_DA_QI_VXO22_OCFB_EN, PMIC_DA_QI_VXO22_OCFB_EN_ADDR, PMIC_DA_QI_VXO22_OCFB_EN_MASK,
	 PMIC_DA_QI_VXO22_OCFB_EN_SHIFT},
	{PMIC_RG_VEFUSE_SW_EN, PMIC_RG_VEFUSE_SW_EN_ADDR, PMIC_RG_VEFUSE_SW_EN_MASK,
	 PMIC_RG_VEFUSE_SW_EN_SHIFT},
	{PMIC_RG_VEFUSE_SW_LP, PMIC_RG_VEFUSE_SW_LP_ADDR, PMIC_RG_VEFUSE_SW_LP_MASK,
	 PMIC_RG_VEFUSE_SW_LP_SHIFT},
	{PMIC_RG_VEFUSE_SW_OP_EN, PMIC_RG_VEFUSE_SW_OP_EN_ADDR, PMIC_RG_VEFUSE_SW_OP_EN_MASK,
	 PMIC_RG_VEFUSE_SW_OP_EN_SHIFT},
	{PMIC_RG_VEFUSE_HW0_OP_EN, PMIC_RG_VEFUSE_HW0_OP_EN_ADDR, PMIC_RG_VEFUSE_HW0_OP_EN_MASK,
	 PMIC_RG_VEFUSE_HW0_OP_EN_SHIFT},
	{PMIC_RG_VEFUSE_HW1_OP_EN, PMIC_RG_VEFUSE_HW1_OP_EN_ADDR, PMIC_RG_VEFUSE_HW1_OP_EN_MASK,
	 PMIC_RG_VEFUSE_HW1_OP_EN_SHIFT},
	{PMIC_RG_VEFUSE_HW2_OP_EN, PMIC_RG_VEFUSE_HW2_OP_EN_ADDR, PMIC_RG_VEFUSE_HW2_OP_EN_MASK,
	 PMIC_RG_VEFUSE_HW2_OP_EN_SHIFT},
	{PMIC_RG_VEFUSE_OP_EN_SET, PMIC_RG_VEFUSE_OP_EN_SET_ADDR, PMIC_RG_VEFUSE_OP_EN_SET_MASK,
	 PMIC_RG_VEFUSE_OP_EN_SET_SHIFT},
	{PMIC_RG_VEFUSE_OP_EN_CLR, PMIC_RG_VEFUSE_OP_EN_CLR_ADDR, PMIC_RG_VEFUSE_OP_EN_CLR_MASK,
	 PMIC_RG_VEFUSE_OP_EN_CLR_SHIFT},
	{PMIC_RG_VEFUSE_HW0_OP_CFG, PMIC_RG_VEFUSE_HW0_OP_CFG_ADDR, PMIC_RG_VEFUSE_HW0_OP_CFG_MASK,
	 PMIC_RG_VEFUSE_HW0_OP_CFG_SHIFT},
	{PMIC_RG_VEFUSE_HW1_OP_CFG, PMIC_RG_VEFUSE_HW1_OP_CFG_ADDR, PMIC_RG_VEFUSE_HW1_OP_CFG_MASK,
	 PMIC_RG_VEFUSE_HW1_OP_CFG_SHIFT},
	{PMIC_RG_VEFUSE_HW2_OP_CFG, PMIC_RG_VEFUSE_HW2_OP_CFG_ADDR, PMIC_RG_VEFUSE_HW2_OP_CFG_MASK,
	 PMIC_RG_VEFUSE_HW2_OP_CFG_SHIFT},
	{PMIC_RG_VEFUSE_GO_ON_OP, PMIC_RG_VEFUSE_GO_ON_OP_ADDR, PMIC_RG_VEFUSE_GO_ON_OP_MASK,
	 PMIC_RG_VEFUSE_GO_ON_OP_SHIFT},
	{PMIC_RG_VEFUSE_GO_LP_OP, PMIC_RG_VEFUSE_GO_LP_OP_ADDR, PMIC_RG_VEFUSE_GO_LP_OP_MASK,
	 PMIC_RG_VEFUSE_GO_LP_OP_SHIFT},
	{PMIC_RG_VEFUSE_OP_CFG_SET, PMIC_RG_VEFUSE_OP_CFG_SET_ADDR, PMIC_RG_VEFUSE_OP_CFG_SET_MASK,
	 PMIC_RG_VEFUSE_OP_CFG_SET_SHIFT},
	{PMIC_RG_VEFUSE_OP_CFG_CLR, PMIC_RG_VEFUSE_OP_CFG_CLR_ADDR, PMIC_RG_VEFUSE_OP_CFG_CLR_MASK,
	 PMIC_RG_VEFUSE_OP_CFG_CLR_SHIFT},
	{PMIC_DA_QI_VEFUSE_MODE, PMIC_DA_QI_VEFUSE_MODE_ADDR, PMIC_DA_QI_VEFUSE_MODE_MASK,
	 PMIC_DA_QI_VEFUSE_MODE_SHIFT},
	{PMIC_RG_VEFUSE_STBTD, PMIC_RG_VEFUSE_STBTD_ADDR, PMIC_RG_VEFUSE_STBTD_MASK,
	 PMIC_RG_VEFUSE_STBTD_SHIFT},
	{PMIC_DA_QI_VEFUSE_STB, PMIC_DA_QI_VEFUSE_STB_ADDR, PMIC_DA_QI_VEFUSE_STB_MASK,
	 PMIC_DA_QI_VEFUSE_STB_SHIFT},
	{PMIC_DA_QI_VEFUSE_EN, PMIC_DA_QI_VEFUSE_EN_ADDR, PMIC_DA_QI_VEFUSE_EN_MASK,
	 PMIC_DA_QI_VEFUSE_EN_SHIFT},
	{PMIC_RG_VEFUSE_OCFB_EN, PMIC_RG_VEFUSE_OCFB_EN_ADDR, PMIC_RG_VEFUSE_OCFB_EN_MASK,
	 PMIC_RG_VEFUSE_OCFB_EN_SHIFT},
	{PMIC_DA_QI_VEFUSE_OCFB_EN, PMIC_DA_QI_VEFUSE_OCFB_EN_ADDR, PMIC_DA_QI_VEFUSE_OCFB_EN_MASK,
	 PMIC_DA_QI_VEFUSE_OCFB_EN_SHIFT},
	{PMIC_RG_VEFUSE_DUMMY_LOAD, PMIC_RG_VEFUSE_DUMMY_LOAD_ADDR, PMIC_RG_VEFUSE_DUMMY_LOAD_MASK,
	 PMIC_RG_VEFUSE_DUMMY_LOAD_SHIFT},
	{PMIC_DA_QI_VEFUSE_DUMMY_LOAD, PMIC_DA_QI_VEFUSE_DUMMY_LOAD_ADDR,
	 PMIC_DA_QI_VEFUSE_DUMMY_LOAD_MASK, PMIC_DA_QI_VEFUSE_DUMMY_LOAD_SHIFT},
	{PMIC_RG_VSIM1_SW_EN, PMIC_RG_VSIM1_SW_EN_ADDR, PMIC_RG_VSIM1_SW_EN_MASK,
	 PMIC_RG_VSIM1_SW_EN_SHIFT},
	{PMIC_RG_VSIM1_SW_LP, PMIC_RG_VSIM1_SW_LP_ADDR, PMIC_RG_VSIM1_SW_LP_MASK,
	 PMIC_RG_VSIM1_SW_LP_SHIFT},
	{PMIC_RG_VSIM1_SW_OP_EN, PMIC_RG_VSIM1_SW_OP_EN_ADDR, PMIC_RG_VSIM1_SW_OP_EN_MASK,
	 PMIC_RG_VSIM1_SW_OP_EN_SHIFT},
	{PMIC_RG_VSIM1_HW0_OP_EN, PMIC_RG_VSIM1_HW0_OP_EN_ADDR, PMIC_RG_VSIM1_HW0_OP_EN_MASK,
	 PMIC_RG_VSIM1_HW0_OP_EN_SHIFT},
	{PMIC_RG_VSIM1_HW1_OP_EN, PMIC_RG_VSIM1_HW1_OP_EN_ADDR, PMIC_RG_VSIM1_HW1_OP_EN_MASK,
	 PMIC_RG_VSIM1_HW1_OP_EN_SHIFT},
	{PMIC_RG_VSIM1_HW2_OP_EN, PMIC_RG_VSIM1_HW2_OP_EN_ADDR, PMIC_RG_VSIM1_HW2_OP_EN_MASK,
	 PMIC_RG_VSIM1_HW2_OP_EN_SHIFT},
	{PMIC_RG_VSIM1_OP_EN_SET, PMIC_RG_VSIM1_OP_EN_SET_ADDR, PMIC_RG_VSIM1_OP_EN_SET_MASK,
	 PMIC_RG_VSIM1_OP_EN_SET_SHIFT},
	{PMIC_RG_VSIM1_OP_EN_CLR, PMIC_RG_VSIM1_OP_EN_CLR_ADDR, PMIC_RG_VSIM1_OP_EN_CLR_MASK,
	 PMIC_RG_VSIM1_OP_EN_CLR_SHIFT},
	{PMIC_RG_VSIM1_HW0_OP_CFG, PMIC_RG_VSIM1_HW0_OP_CFG_ADDR, PMIC_RG_VSIM1_HW0_OP_CFG_MASK,
	 PMIC_RG_VSIM1_HW0_OP_CFG_SHIFT},
	{PMIC_RG_VSIM1_HW1_OP_CFG, PMIC_RG_VSIM1_HW1_OP_CFG_ADDR, PMIC_RG_VSIM1_HW1_OP_CFG_MASK,
	 PMIC_RG_VSIM1_HW1_OP_CFG_SHIFT},
	{PMIC_RG_VSIM1_HW2_OP_CFG, PMIC_RG_VSIM1_HW2_OP_CFG_ADDR, PMIC_RG_VSIM1_HW2_OP_CFG_MASK,
	 PMIC_RG_VSIM1_HW2_OP_CFG_SHIFT},
	{PMIC_RG_VSIM1_GO_ON_OP, PMIC_RG_VSIM1_GO_ON_OP_ADDR, PMIC_RG_VSIM1_GO_ON_OP_MASK,
	 PMIC_RG_VSIM1_GO_ON_OP_SHIFT},
	{PMIC_RG_VSIM1_GO_LP_OP, PMIC_RG_VSIM1_GO_LP_OP_ADDR, PMIC_RG_VSIM1_GO_LP_OP_MASK,
	 PMIC_RG_VSIM1_GO_LP_OP_SHIFT},
	{PMIC_RG_VSIM1_OP_CFG_SET, PMIC_RG_VSIM1_OP_CFG_SET_ADDR, PMIC_RG_VSIM1_OP_CFG_SET_MASK,
	 PMIC_RG_VSIM1_OP_CFG_SET_SHIFT},
	{PMIC_RG_VSIM1_OP_CFG_CLR, PMIC_RG_VSIM1_OP_CFG_CLR_ADDR, PMIC_RG_VSIM1_OP_CFG_CLR_MASK,
	 PMIC_RG_VSIM1_OP_CFG_CLR_SHIFT},
	{PMIC_DA_QI_VSIM1_MODE, PMIC_DA_QI_VSIM1_MODE_ADDR, PMIC_DA_QI_VSIM1_MODE_MASK,
	 PMIC_DA_QI_VSIM1_MODE_SHIFT},
	{PMIC_RG_VSIM1_STBTD, PMIC_RG_VSIM1_STBTD_ADDR, PMIC_RG_VSIM1_STBTD_MASK,
	 PMIC_RG_VSIM1_STBTD_SHIFT},
	{PMIC_DA_QI_VSIM1_STB, PMIC_DA_QI_VSIM1_STB_ADDR, PMIC_DA_QI_VSIM1_STB_MASK,
	 PMIC_DA_QI_VSIM1_STB_SHIFT},
	{PMIC_DA_QI_VSIM1_EN, PMIC_DA_QI_VSIM1_EN_ADDR, PMIC_DA_QI_VSIM1_EN_MASK,
	 PMIC_DA_QI_VSIM1_EN_SHIFT},
	{PMIC_RG_VSIM1_OCFB_EN, PMIC_RG_VSIM1_OCFB_EN_ADDR, PMIC_RG_VSIM1_OCFB_EN_MASK,
	 PMIC_RG_VSIM1_OCFB_EN_SHIFT},
	{PMIC_DA_QI_VSIM1_OCFB_EN, PMIC_DA_QI_VSIM1_OCFB_EN_ADDR, PMIC_DA_QI_VSIM1_OCFB_EN_MASK,
	 PMIC_DA_QI_VSIM1_OCFB_EN_SHIFT},
	{PMIC_RG_VSIM1_DUMMY_LOAD, PMIC_RG_VSIM1_DUMMY_LOAD_ADDR, PMIC_RG_VSIM1_DUMMY_LOAD_MASK,
	 PMIC_RG_VSIM1_DUMMY_LOAD_SHIFT},
	{PMIC_DA_QI_VSIM1_DUMMY_LOAD, PMIC_DA_QI_VSIM1_DUMMY_LOAD_ADDR,
	 PMIC_DA_QI_VSIM1_DUMMY_LOAD_MASK, PMIC_DA_QI_VSIM1_DUMMY_LOAD_SHIFT},
	{PMIC_RG_VSIM2_SW_EN, PMIC_RG_VSIM2_SW_EN_ADDR, PMIC_RG_VSIM2_SW_EN_MASK,
	 PMIC_RG_VSIM2_SW_EN_SHIFT},
	{PMIC_RG_VSIM2_SW_LP, PMIC_RG_VSIM2_SW_LP_ADDR, PMIC_RG_VSIM2_SW_LP_MASK,
	 PMIC_RG_VSIM2_SW_LP_SHIFT},
	{PMIC_RG_VSIM2_SW_OP_EN, PMIC_RG_VSIM2_SW_OP_EN_ADDR, PMIC_RG_VSIM2_SW_OP_EN_MASK,
	 PMIC_RG_VSIM2_SW_OP_EN_SHIFT},
	{PMIC_RG_VSIM2_HW0_OP_EN, PMIC_RG_VSIM2_HW0_OP_EN_ADDR, PMIC_RG_VSIM2_HW0_OP_EN_MASK,
	 PMIC_RG_VSIM2_HW0_OP_EN_SHIFT},
	{PMIC_RG_VSIM2_HW1_OP_EN, PMIC_RG_VSIM2_HW1_OP_EN_ADDR, PMIC_RG_VSIM2_HW1_OP_EN_MASK,
	 PMIC_RG_VSIM2_HW1_OP_EN_SHIFT},
	{PMIC_RG_VSIM2_HW2_OP_EN, PMIC_RG_VSIM2_HW2_OP_EN_ADDR, PMIC_RG_VSIM2_HW2_OP_EN_MASK,
	 PMIC_RG_VSIM2_HW2_OP_EN_SHIFT},
	{PMIC_RG_VSIM2_OP_EN_SET, PMIC_RG_VSIM2_OP_EN_SET_ADDR, PMIC_RG_VSIM2_OP_EN_SET_MASK,
	 PMIC_RG_VSIM2_OP_EN_SET_SHIFT},
	{PMIC_RG_VSIM2_OP_EN_CLR, PMIC_RG_VSIM2_OP_EN_CLR_ADDR, PMIC_RG_VSIM2_OP_EN_CLR_MASK,
	 PMIC_RG_VSIM2_OP_EN_CLR_SHIFT},
	{PMIC_RG_VSIM2_HW0_OP_CFG, PMIC_RG_VSIM2_HW0_OP_CFG_ADDR, PMIC_RG_VSIM2_HW0_OP_CFG_MASK,
	 PMIC_RG_VSIM2_HW0_OP_CFG_SHIFT},
	{PMIC_RG_VSIM2_HW1_OP_CFG, PMIC_RG_VSIM2_HW1_OP_CFG_ADDR, PMIC_RG_VSIM2_HW1_OP_CFG_MASK,
	 PMIC_RG_VSIM2_HW1_OP_CFG_SHIFT},
	{PMIC_RG_VSIM2_HW2_OP_CFG, PMIC_RG_VSIM2_HW2_OP_CFG_ADDR, PMIC_RG_VSIM2_HW2_OP_CFG_MASK,
	 PMIC_RG_VSIM2_HW2_OP_CFG_SHIFT},
	{PMIC_RG_VSIM2_GO_ON_OP, PMIC_RG_VSIM2_GO_ON_OP_ADDR, PMIC_RG_VSIM2_GO_ON_OP_MASK,
	 PMIC_RG_VSIM2_GO_ON_OP_SHIFT},
	{PMIC_RG_VSIM2_GO_LP_OP, PMIC_RG_VSIM2_GO_LP_OP_ADDR, PMIC_RG_VSIM2_GO_LP_OP_MASK,
	 PMIC_RG_VSIM2_GO_LP_OP_SHIFT},
	{PMIC_RG_VSIM2_OP_CFG_SET, PMIC_RG_VSIM2_OP_CFG_SET_ADDR, PMIC_RG_VSIM2_OP_CFG_SET_MASK,
	 PMIC_RG_VSIM2_OP_CFG_SET_SHIFT},
	{PMIC_RG_VSIM2_OP_CFG_CLR, PMIC_RG_VSIM2_OP_CFG_CLR_ADDR, PMIC_RG_VSIM2_OP_CFG_CLR_MASK,
	 PMIC_RG_VSIM2_OP_CFG_CLR_SHIFT},
	{PMIC_DA_QI_VSIM2_MODE, PMIC_DA_QI_VSIM2_MODE_ADDR, PMIC_DA_QI_VSIM2_MODE_MASK,
	 PMIC_DA_QI_VSIM2_MODE_SHIFT},
	{PMIC_RG_VSIM2_STBTD, PMIC_RG_VSIM2_STBTD_ADDR, PMIC_RG_VSIM2_STBTD_MASK,
	 PMIC_RG_VSIM2_STBTD_SHIFT},
	{PMIC_DA_QI_VSIM2_STB, PMIC_DA_QI_VSIM2_STB_ADDR, PMIC_DA_QI_VSIM2_STB_MASK,
	 PMIC_DA_QI_VSIM2_STB_SHIFT},
	{PMIC_DA_QI_VSIM2_EN, PMIC_DA_QI_VSIM2_EN_ADDR, PMIC_DA_QI_VSIM2_EN_MASK,
	 PMIC_DA_QI_VSIM2_EN_SHIFT},
	{PMIC_RG_VSIM2_OCFB_EN, PMIC_RG_VSIM2_OCFB_EN_ADDR, PMIC_RG_VSIM2_OCFB_EN_MASK,
	 PMIC_RG_VSIM2_OCFB_EN_SHIFT},
	{PMIC_DA_QI_VSIM2_OCFB_EN, PMIC_DA_QI_VSIM2_OCFB_EN_ADDR, PMIC_DA_QI_VSIM2_OCFB_EN_MASK,
	 PMIC_DA_QI_VSIM2_OCFB_EN_SHIFT},
	{PMIC_RG_VSIM2_DUMMY_LOAD, PMIC_RG_VSIM2_DUMMY_LOAD_ADDR, PMIC_RG_VSIM2_DUMMY_LOAD_MASK,
	 PMIC_RG_VSIM2_DUMMY_LOAD_SHIFT},
	{PMIC_DA_QI_VSIM2_DUMMY_LOAD, PMIC_DA_QI_VSIM2_DUMMY_LOAD_ADDR,
	 PMIC_DA_QI_VSIM2_DUMMY_LOAD_MASK, PMIC_DA_QI_VSIM2_DUMMY_LOAD_SHIFT},
	{PMIC_RG_VCAMAF_SW_EN, PMIC_RG_VCAMAF_SW_EN_ADDR, PMIC_RG_VCAMAF_SW_EN_MASK,
	 PMIC_RG_VCAMAF_SW_EN_SHIFT},
	{PMIC_RG_VCAMAF_SW_LP, PMIC_RG_VCAMAF_SW_LP_ADDR, PMIC_RG_VCAMAF_SW_LP_MASK,
	 PMIC_RG_VCAMAF_SW_LP_SHIFT},
	{PMIC_RG_VCAMAF_SW_OP_EN, PMIC_RG_VCAMAF_SW_OP_EN_ADDR, PMIC_RG_VCAMAF_SW_OP_EN_MASK,
	 PMIC_RG_VCAMAF_SW_OP_EN_SHIFT},
	{PMIC_RG_VCAMAF_HW0_OP_EN, PMIC_RG_VCAMAF_HW0_OP_EN_ADDR, PMIC_RG_VCAMAF_HW0_OP_EN_MASK,
	 PMIC_RG_VCAMAF_HW0_OP_EN_SHIFT},
	{PMIC_RG_VCAMAF_HW1_OP_EN, PMIC_RG_VCAMAF_HW1_OP_EN_ADDR, PMIC_RG_VCAMAF_HW1_OP_EN_MASK,
	 PMIC_RG_VCAMAF_HW1_OP_EN_SHIFT},
	{PMIC_RG_VCAMAF_HW2_OP_EN, PMIC_RG_VCAMAF_HW2_OP_EN_ADDR, PMIC_RG_VCAMAF_HW2_OP_EN_MASK,
	 PMIC_RG_VCAMAF_HW2_OP_EN_SHIFT},
	{PMIC_RG_VCAMAF_OP_EN_SET, PMIC_RG_VCAMAF_OP_EN_SET_ADDR, PMIC_RG_VCAMAF_OP_EN_SET_MASK,
	 PMIC_RG_VCAMAF_OP_EN_SET_SHIFT},
	{PMIC_RG_VCAMAF_OP_EN_CLR, PMIC_RG_VCAMAF_OP_EN_CLR_ADDR, PMIC_RG_VCAMAF_OP_EN_CLR_MASK,
	 PMIC_RG_VCAMAF_OP_EN_CLR_SHIFT},
	{PMIC_RG_VCAMAF_HW0_OP_CFG, PMIC_RG_VCAMAF_HW0_OP_CFG_ADDR, PMIC_RG_VCAMAF_HW0_OP_CFG_MASK,
	 PMIC_RG_VCAMAF_HW0_OP_CFG_SHIFT},
	{PMIC_RG_VCAMAF_HW1_OP_CFG, PMIC_RG_VCAMAF_HW1_OP_CFG_ADDR, PMIC_RG_VCAMAF_HW1_OP_CFG_MASK,
	 PMIC_RG_VCAMAF_HW1_OP_CFG_SHIFT},
	{PMIC_RG_VCAMAF_HW2_OP_CFG, PMIC_RG_VCAMAF_HW2_OP_CFG_ADDR, PMIC_RG_VCAMAF_HW2_OP_CFG_MASK,
	 PMIC_RG_VCAMAF_HW2_OP_CFG_SHIFT},
	{PMIC_RG_VCAMAF_GO_ON_OP, PMIC_RG_VCAMAF_GO_ON_OP_ADDR, PMIC_RG_VCAMAF_GO_ON_OP_MASK,
	 PMIC_RG_VCAMAF_GO_ON_OP_SHIFT},
	{PMIC_RG_VCAMAF_GO_LP_OP, PMIC_RG_VCAMAF_GO_LP_OP_ADDR, PMIC_RG_VCAMAF_GO_LP_OP_MASK,
	 PMIC_RG_VCAMAF_GO_LP_OP_SHIFT},
	{PMIC_RG_VCAMAF_OP_CFG_SET, PMIC_RG_VCAMAF_OP_CFG_SET_ADDR, PMIC_RG_VCAMAF_OP_CFG_SET_MASK,
	 PMIC_RG_VCAMAF_OP_CFG_SET_SHIFT},
	{PMIC_RG_VCAMAF_OP_CFG_CLR, PMIC_RG_VCAMAF_OP_CFG_CLR_ADDR, PMIC_RG_VCAMAF_OP_CFG_CLR_MASK,
	 PMIC_RG_VCAMAF_OP_CFG_CLR_SHIFT},
	{PMIC_DA_QI_VCAMAF_MODE, PMIC_DA_QI_VCAMAF_MODE_ADDR, PMIC_DA_QI_VCAMAF_MODE_MASK,
	 PMIC_DA_QI_VCAMAF_MODE_SHIFT},
	{PMIC_RG_VCAMAF_STBTD, PMIC_RG_VCAMAF_STBTD_ADDR, PMIC_RG_VCAMAF_STBTD_MASK,
	 PMIC_RG_VCAMAF_STBTD_SHIFT},
	{PMIC_DA_QI_VCAMAF_STB, PMIC_DA_QI_VCAMAF_STB_ADDR, PMIC_DA_QI_VCAMAF_STB_MASK,
	 PMIC_DA_QI_VCAMAF_STB_SHIFT},
	{PMIC_DA_QI_VCAMAF_EN, PMIC_DA_QI_VCAMAF_EN_ADDR, PMIC_DA_QI_VCAMAF_EN_MASK,
	 PMIC_DA_QI_VCAMAF_EN_SHIFT},
	{PMIC_RG_VCAMAF_OCFB_EN, PMIC_RG_VCAMAF_OCFB_EN_ADDR, PMIC_RG_VCAMAF_OCFB_EN_MASK,
	 PMIC_RG_VCAMAF_OCFB_EN_SHIFT},
	{PMIC_DA_QI_VCAMAF_OCFB_EN, PMIC_DA_QI_VCAMAF_OCFB_EN_ADDR, PMIC_DA_QI_VCAMAF_OCFB_EN_MASK,
	 PMIC_DA_QI_VCAMAF_OCFB_EN_SHIFT},
	{PMIC_RG_VCAMAF_DUMMY_LOAD, PMIC_RG_VCAMAF_DUMMY_LOAD_ADDR, PMIC_RG_VCAMAF_DUMMY_LOAD_MASK,
	 PMIC_RG_VCAMAF_DUMMY_LOAD_SHIFT},
	{PMIC_DA_QI_VCAMAF_DUMMY_LOAD, PMIC_DA_QI_VCAMAF_DUMMY_LOAD_ADDR,
	 PMIC_DA_QI_VCAMAF_DUMMY_LOAD_MASK, PMIC_DA_QI_VCAMAF_DUMMY_LOAD_SHIFT},
	{PMIC_RG_VTOUCH_SW_EN, PMIC_RG_VTOUCH_SW_EN_ADDR, PMIC_RG_VTOUCH_SW_EN_MASK,
	 PMIC_RG_VTOUCH_SW_EN_SHIFT},
	{PMIC_RG_VTOUCH_SW_LP, PMIC_RG_VTOUCH_SW_LP_ADDR, PMIC_RG_VTOUCH_SW_LP_MASK,
	 PMIC_RG_VTOUCH_SW_LP_SHIFT},
	{PMIC_RG_VTOUCH_SW_OP_EN, PMIC_RG_VTOUCH_SW_OP_EN_ADDR, PMIC_RG_VTOUCH_SW_OP_EN_MASK,
	 PMIC_RG_VTOUCH_SW_OP_EN_SHIFT},
	{PMIC_RG_VTOUCH_HW0_OP_EN, PMIC_RG_VTOUCH_HW0_OP_EN_ADDR, PMIC_RG_VTOUCH_HW0_OP_EN_MASK,
	 PMIC_RG_VTOUCH_HW0_OP_EN_SHIFT},
	{PMIC_RG_VTOUCH_HW1_OP_EN, PMIC_RG_VTOUCH_HW1_OP_EN_ADDR, PMIC_RG_VTOUCH_HW1_OP_EN_MASK,
	 PMIC_RG_VTOUCH_HW1_OP_EN_SHIFT},
	{PMIC_RG_VTOUCH_HW2_OP_EN, PMIC_RG_VTOUCH_HW2_OP_EN_ADDR, PMIC_RG_VTOUCH_HW2_OP_EN_MASK,
	 PMIC_RG_VTOUCH_HW2_OP_EN_SHIFT},
	{PMIC_RG_VTOUCH_OP_EN_SET, PMIC_RG_VTOUCH_OP_EN_SET_ADDR, PMIC_RG_VTOUCH_OP_EN_SET_MASK,
	 PMIC_RG_VTOUCH_OP_EN_SET_SHIFT},
	{PMIC_RG_VTOUCH_OP_EN_CLR, PMIC_RG_VTOUCH_OP_EN_CLR_ADDR, PMIC_RG_VTOUCH_OP_EN_CLR_MASK,
	 PMIC_RG_VTOUCH_OP_EN_CLR_SHIFT},
	{PMIC_RG_VTOUCH_HW0_OP_CFG, PMIC_RG_VTOUCH_HW0_OP_CFG_ADDR, PMIC_RG_VTOUCH_HW0_OP_CFG_MASK,
	 PMIC_RG_VTOUCH_HW0_OP_CFG_SHIFT},
	{PMIC_RG_VTOUCH_HW1_OP_CFG, PMIC_RG_VTOUCH_HW1_OP_CFG_ADDR, PMIC_RG_VTOUCH_HW1_OP_CFG_MASK,
	 PMIC_RG_VTOUCH_HW1_OP_CFG_SHIFT},
	{PMIC_RG_VTOUCH_HW2_OP_CFG, PMIC_RG_VTOUCH_HW2_OP_CFG_ADDR, PMIC_RG_VTOUCH_HW2_OP_CFG_MASK,
	 PMIC_RG_VTOUCH_HW2_OP_CFG_SHIFT},
	{PMIC_RG_VTOUCH_GO_ON_OP, PMIC_RG_VTOUCH_GO_ON_OP_ADDR, PMIC_RG_VTOUCH_GO_ON_OP_MASK,
	 PMIC_RG_VTOUCH_GO_ON_OP_SHIFT},
	{PMIC_RG_VTOUCH_GO_LP_OP, PMIC_RG_VTOUCH_GO_LP_OP_ADDR, PMIC_RG_VTOUCH_GO_LP_OP_MASK,
	 PMIC_RG_VTOUCH_GO_LP_OP_SHIFT},
	{PMIC_RG_VTOUCH_OP_CFG_SET, PMIC_RG_VTOUCH_OP_CFG_SET_ADDR, PMIC_RG_VTOUCH_OP_CFG_SET_MASK,
	 PMIC_RG_VTOUCH_OP_CFG_SET_SHIFT},
	{PMIC_RG_VTOUCH_OP_CFG_CLR, PMIC_RG_VTOUCH_OP_CFG_CLR_ADDR, PMIC_RG_VTOUCH_OP_CFG_CLR_MASK,
	 PMIC_RG_VTOUCH_OP_CFG_CLR_SHIFT},
	{PMIC_DA_QI_VTOUCH_MODE, PMIC_DA_QI_VTOUCH_MODE_ADDR, PMIC_DA_QI_VTOUCH_MODE_MASK,
	 PMIC_DA_QI_VTOUCH_MODE_SHIFT},
	{PMIC_RG_VTOUCH_STBTD, PMIC_RG_VTOUCH_STBTD_ADDR, PMIC_RG_VTOUCH_STBTD_MASK,
	 PMIC_RG_VTOUCH_STBTD_SHIFT},
	{PMIC_DA_QI_VTOUCH_STB, PMIC_DA_QI_VTOUCH_STB_ADDR, PMIC_DA_QI_VTOUCH_STB_MASK,
	 PMIC_DA_QI_VTOUCH_STB_SHIFT},
	{PMIC_DA_QI_VTOUCH_EN, PMIC_DA_QI_VTOUCH_EN_ADDR, PMIC_DA_QI_VTOUCH_EN_MASK,
	 PMIC_DA_QI_VTOUCH_EN_SHIFT},
	{PMIC_RG_VTOUCH_OCFB_EN, PMIC_RG_VTOUCH_OCFB_EN_ADDR, PMIC_RG_VTOUCH_OCFB_EN_MASK,
	 PMIC_RG_VTOUCH_OCFB_EN_SHIFT},
	{PMIC_DA_QI_VTOUCH_OCFB_EN, PMIC_DA_QI_VTOUCH_OCFB_EN_ADDR, PMIC_DA_QI_VTOUCH_OCFB_EN_MASK,
	 PMIC_DA_QI_VTOUCH_OCFB_EN_SHIFT},
	{PMIC_RG_VTOUCH_DUMMY_LOAD, PMIC_RG_VTOUCH_DUMMY_LOAD_ADDR, PMIC_RG_VTOUCH_DUMMY_LOAD_MASK,
	 PMIC_RG_VTOUCH_DUMMY_LOAD_SHIFT},
	{PMIC_DA_QI_VTOUCH_DUMMY_LOAD, PMIC_DA_QI_VTOUCH_DUMMY_LOAD_ADDR,
	 PMIC_DA_QI_VTOUCH_DUMMY_LOAD_MASK, PMIC_DA_QI_VTOUCH_DUMMY_LOAD_SHIFT},
	{PMIC_RG_VCAMD1_SW_EN, PMIC_RG_VCAMD1_SW_EN_ADDR, PMIC_RG_VCAMD1_SW_EN_MASK,
	 PMIC_RG_VCAMD1_SW_EN_SHIFT},
	{PMIC_RG_VCAMD1_SW_LP, PMIC_RG_VCAMD1_SW_LP_ADDR, PMIC_RG_VCAMD1_SW_LP_MASK,
	 PMIC_RG_VCAMD1_SW_LP_SHIFT},
	{PMIC_RG_VCAMD1_SW_OP_EN, PMIC_RG_VCAMD1_SW_OP_EN_ADDR, PMIC_RG_VCAMD1_SW_OP_EN_MASK,
	 PMIC_RG_VCAMD1_SW_OP_EN_SHIFT},
	{PMIC_RG_VCAMD1_HW0_OP_EN, PMIC_RG_VCAMD1_HW0_OP_EN_ADDR, PMIC_RG_VCAMD1_HW0_OP_EN_MASK,
	 PMIC_RG_VCAMD1_HW0_OP_EN_SHIFT},
	{PMIC_RG_VCAMD1_HW1_OP_EN, PMIC_RG_VCAMD1_HW1_OP_EN_ADDR, PMIC_RG_VCAMD1_HW1_OP_EN_MASK,
	 PMIC_RG_VCAMD1_HW1_OP_EN_SHIFT},
	{PMIC_RG_VCAMD1_HW2_OP_EN, PMIC_RG_VCAMD1_HW2_OP_EN_ADDR, PMIC_RG_VCAMD1_HW2_OP_EN_MASK,
	 PMIC_RG_VCAMD1_HW2_OP_EN_SHIFT},
	{PMIC_RG_VCAMD1_OP_EN_SET, PMIC_RG_VCAMD1_OP_EN_SET_ADDR, PMIC_RG_VCAMD1_OP_EN_SET_MASK,
	 PMIC_RG_VCAMD1_OP_EN_SET_SHIFT},
	{PMIC_RG_VCAMD1_OP_EN_CLR, PMIC_RG_VCAMD1_OP_EN_CLR_ADDR, PMIC_RG_VCAMD1_OP_EN_CLR_MASK,
	 PMIC_RG_VCAMD1_OP_EN_CLR_SHIFT},
	{PMIC_RG_VCAMD1_HW0_OP_CFG, PMIC_RG_VCAMD1_HW0_OP_CFG_ADDR, PMIC_RG_VCAMD1_HW0_OP_CFG_MASK,
	 PMIC_RG_VCAMD1_HW0_OP_CFG_SHIFT},
	{PMIC_RG_VCAMD1_HW1_OP_CFG, PMIC_RG_VCAMD1_HW1_OP_CFG_ADDR, PMIC_RG_VCAMD1_HW1_OP_CFG_MASK,
	 PMIC_RG_VCAMD1_HW1_OP_CFG_SHIFT},
	{PMIC_RG_VCAMD1_HW2_OP_CFG, PMIC_RG_VCAMD1_HW2_OP_CFG_ADDR, PMIC_RG_VCAMD1_HW2_OP_CFG_MASK,
	 PMIC_RG_VCAMD1_HW2_OP_CFG_SHIFT},
	{PMIC_RG_VCAMD1_GO_ON_OP, PMIC_RG_VCAMD1_GO_ON_OP_ADDR, PMIC_RG_VCAMD1_GO_ON_OP_MASK,
	 PMIC_RG_VCAMD1_GO_ON_OP_SHIFT},
	{PMIC_RG_VCAMD1_GO_LP_OP, PMIC_RG_VCAMD1_GO_LP_OP_ADDR, PMIC_RG_VCAMD1_GO_LP_OP_MASK,
	 PMIC_RG_VCAMD1_GO_LP_OP_SHIFT},
	{PMIC_RG_VCAMD1_OP_CFG_SET, PMIC_RG_VCAMD1_OP_CFG_SET_ADDR, PMIC_RG_VCAMD1_OP_CFG_SET_MASK,
	 PMIC_RG_VCAMD1_OP_CFG_SET_SHIFT},
	{PMIC_RG_VCAMD1_OP_CFG_CLR, PMIC_RG_VCAMD1_OP_CFG_CLR_ADDR, PMIC_RG_VCAMD1_OP_CFG_CLR_MASK,
	 PMIC_RG_VCAMD1_OP_CFG_CLR_SHIFT},
	{PMIC_DA_QI_VCAMD1_MODE, PMIC_DA_QI_VCAMD1_MODE_ADDR, PMIC_DA_QI_VCAMD1_MODE_MASK,
	 PMIC_DA_QI_VCAMD1_MODE_SHIFT},
	{PMIC_RG_VCAMD1_STBTD, PMIC_RG_VCAMD1_STBTD_ADDR, PMIC_RG_VCAMD1_STBTD_MASK,
	 PMIC_RG_VCAMD1_STBTD_SHIFT},
	{PMIC_DA_QI_VCAMD1_STB, PMIC_DA_QI_VCAMD1_STB_ADDR, PMIC_DA_QI_VCAMD1_STB_MASK,
	 PMIC_DA_QI_VCAMD1_STB_SHIFT},
	{PMIC_DA_QI_VCAMD1_EN, PMIC_DA_QI_VCAMD1_EN_ADDR, PMIC_DA_QI_VCAMD1_EN_MASK,
	 PMIC_DA_QI_VCAMD1_EN_SHIFT},
	{PMIC_RG_VCAMD1_OCFB_EN, PMIC_RG_VCAMD1_OCFB_EN_ADDR, PMIC_RG_VCAMD1_OCFB_EN_MASK,
	 PMIC_RG_VCAMD1_OCFB_EN_SHIFT},
	{PMIC_DA_QI_VCAMD1_OCFB_EN, PMIC_DA_QI_VCAMD1_OCFB_EN_ADDR, PMIC_DA_QI_VCAMD1_OCFB_EN_MASK,
	 PMIC_DA_QI_VCAMD1_OCFB_EN_SHIFT},
	{PMIC_RG_VCAMD1_DUMMY_LOAD, PMIC_RG_VCAMD1_DUMMY_LOAD_ADDR, PMIC_RG_VCAMD1_DUMMY_LOAD_MASK,
	 PMIC_RG_VCAMD1_DUMMY_LOAD_SHIFT},
	{PMIC_DA_QI_VCAMD1_DUMMY_LOAD, PMIC_DA_QI_VCAMD1_DUMMY_LOAD_ADDR,
	 PMIC_DA_QI_VCAMD1_DUMMY_LOAD_MASK, PMIC_DA_QI_VCAMD1_DUMMY_LOAD_SHIFT},
	{PMIC_RG_VCAMD2_SW_EN, PMIC_RG_VCAMD2_SW_EN_ADDR, PMIC_RG_VCAMD2_SW_EN_MASK,
	 PMIC_RG_VCAMD2_SW_EN_SHIFT},
	{PMIC_RG_VCAMD2_SW_LP, PMIC_RG_VCAMD2_SW_LP_ADDR, PMIC_RG_VCAMD2_SW_LP_MASK,
	 PMIC_RG_VCAMD2_SW_LP_SHIFT},
	{PMIC_RG_VCAMD2_SW_OP_EN, PMIC_RG_VCAMD2_SW_OP_EN_ADDR, PMIC_RG_VCAMD2_SW_OP_EN_MASK,
	 PMIC_RG_VCAMD2_SW_OP_EN_SHIFT},
	{PMIC_RG_VCAMD2_HW0_OP_EN, PMIC_RG_VCAMD2_HW0_OP_EN_ADDR, PMIC_RG_VCAMD2_HW0_OP_EN_MASK,
	 PMIC_RG_VCAMD2_HW0_OP_EN_SHIFT},
	{PMIC_RG_VCAMD2_HW1_OP_EN, PMIC_RG_VCAMD2_HW1_OP_EN_ADDR, PMIC_RG_VCAMD2_HW1_OP_EN_MASK,
	 PMIC_RG_VCAMD2_HW1_OP_EN_SHIFT},
	{PMIC_RG_VCAMD2_HW2_OP_EN, PMIC_RG_VCAMD2_HW2_OP_EN_ADDR, PMIC_RG_VCAMD2_HW2_OP_EN_MASK,
	 PMIC_RG_VCAMD2_HW2_OP_EN_SHIFT},
	{PMIC_RG_VCAMD2_OP_EN_SET, PMIC_RG_VCAMD2_OP_EN_SET_ADDR, PMIC_RG_VCAMD2_OP_EN_SET_MASK,
	 PMIC_RG_VCAMD2_OP_EN_SET_SHIFT},
	{PMIC_RG_VCAMD2_OP_EN_CLR, PMIC_RG_VCAMD2_OP_EN_CLR_ADDR, PMIC_RG_VCAMD2_OP_EN_CLR_MASK,
	 PMIC_RG_VCAMD2_OP_EN_CLR_SHIFT},
	{PMIC_RG_VCAMD2_HW0_OP_CFG, PMIC_RG_VCAMD2_HW0_OP_CFG_ADDR, PMIC_RG_VCAMD2_HW0_OP_CFG_MASK,
	 PMIC_RG_VCAMD2_HW0_OP_CFG_SHIFT},
	{PMIC_RG_VCAMD2_HW1_OP_CFG, PMIC_RG_VCAMD2_HW1_OP_CFG_ADDR, PMIC_RG_VCAMD2_HW1_OP_CFG_MASK,
	 PMIC_RG_VCAMD2_HW1_OP_CFG_SHIFT},
	{PMIC_RG_VCAMD2_HW2_OP_CFG, PMIC_RG_VCAMD2_HW2_OP_CFG_ADDR, PMIC_RG_VCAMD2_HW2_OP_CFG_MASK,
	 PMIC_RG_VCAMD2_HW2_OP_CFG_SHIFT},
	{PMIC_RG_VCAMD2_GO_ON_OP, PMIC_RG_VCAMD2_GO_ON_OP_ADDR, PMIC_RG_VCAMD2_GO_ON_OP_MASK,
	 PMIC_RG_VCAMD2_GO_ON_OP_SHIFT},
	{PMIC_RG_VCAMD2_GO_LP_OP, PMIC_RG_VCAMD2_GO_LP_OP_ADDR, PMIC_RG_VCAMD2_GO_LP_OP_MASK,
	 PMIC_RG_VCAMD2_GO_LP_OP_SHIFT},
	{PMIC_RG_VCAMD2_OP_CFG_SET, PMIC_RG_VCAMD2_OP_CFG_SET_ADDR, PMIC_RG_VCAMD2_OP_CFG_SET_MASK,
	 PMIC_RG_VCAMD2_OP_CFG_SET_SHIFT},
	{PMIC_RG_VCAMD2_OP_CFG_CLR, PMIC_RG_VCAMD2_OP_CFG_CLR_ADDR, PMIC_RG_VCAMD2_OP_CFG_CLR_MASK,
	 PMIC_RG_VCAMD2_OP_CFG_CLR_SHIFT},
	{PMIC_DA_QI_VCAMD2_MODE, PMIC_DA_QI_VCAMD2_MODE_ADDR, PMIC_DA_QI_VCAMD2_MODE_MASK,
	 PMIC_DA_QI_VCAMD2_MODE_SHIFT},
	{PMIC_RG_VCAMD2_STBTD, PMIC_RG_VCAMD2_STBTD_ADDR, PMIC_RG_VCAMD2_STBTD_MASK,
	 PMIC_RG_VCAMD2_STBTD_SHIFT},
	{PMIC_DA_QI_VCAMD2_STB, PMIC_DA_QI_VCAMD2_STB_ADDR, PMIC_DA_QI_VCAMD2_STB_MASK,
	 PMIC_DA_QI_VCAMD2_STB_SHIFT},
	{PMIC_DA_QI_VCAMD2_EN, PMIC_DA_QI_VCAMD2_EN_ADDR, PMIC_DA_QI_VCAMD2_EN_MASK,
	 PMIC_DA_QI_VCAMD2_EN_SHIFT},
	{PMIC_RG_VCAMD2_OCFB_EN, PMIC_RG_VCAMD2_OCFB_EN_ADDR, PMIC_RG_VCAMD2_OCFB_EN_MASK,
	 PMIC_RG_VCAMD2_OCFB_EN_SHIFT},
	{PMIC_DA_QI_VCAMD2_OCFB_EN, PMIC_DA_QI_VCAMD2_OCFB_EN_ADDR, PMIC_DA_QI_VCAMD2_OCFB_EN_MASK,
	 PMIC_DA_QI_VCAMD2_OCFB_EN_SHIFT},
	{PMIC_RG_VCAMD2_DUMMY_LOAD, PMIC_RG_VCAMD2_DUMMY_LOAD_ADDR, PMIC_RG_VCAMD2_DUMMY_LOAD_MASK,
	 PMIC_RG_VCAMD2_DUMMY_LOAD_SHIFT},
	{PMIC_DA_QI_VCAMD2_DUMMY_LOAD, PMIC_DA_QI_VCAMD2_DUMMY_LOAD_ADDR,
	 PMIC_DA_QI_VCAMD2_DUMMY_LOAD_MASK, PMIC_DA_QI_VCAMD2_DUMMY_LOAD_SHIFT},
	{PMIC_RG_VCAMIO_SW_EN, PMIC_RG_VCAMIO_SW_EN_ADDR, PMIC_RG_VCAMIO_SW_EN_MASK,
	 PMIC_RG_VCAMIO_SW_EN_SHIFT},
	{PMIC_RG_VCAMIO_SW_LP, PMIC_RG_VCAMIO_SW_LP_ADDR, PMIC_RG_VCAMIO_SW_LP_MASK,
	 PMIC_RG_VCAMIO_SW_LP_SHIFT},
	{PMIC_RG_VCAMIO_SW_OP_EN, PMIC_RG_VCAMIO_SW_OP_EN_ADDR, PMIC_RG_VCAMIO_SW_OP_EN_MASK,
	 PMIC_RG_VCAMIO_SW_OP_EN_SHIFT},
	{PMIC_RG_VCAMIO_HW0_OP_EN, PMIC_RG_VCAMIO_HW0_OP_EN_ADDR, PMIC_RG_VCAMIO_HW0_OP_EN_MASK,
	 PMIC_RG_VCAMIO_HW0_OP_EN_SHIFT},
	{PMIC_RG_VCAMIO_HW1_OP_EN, PMIC_RG_VCAMIO_HW1_OP_EN_ADDR, PMIC_RG_VCAMIO_HW1_OP_EN_MASK,
	 PMIC_RG_VCAMIO_HW1_OP_EN_SHIFT},
	{PMIC_RG_VCAMIO_HW2_OP_EN, PMIC_RG_VCAMIO_HW2_OP_EN_ADDR, PMIC_RG_VCAMIO_HW2_OP_EN_MASK,
	 PMIC_RG_VCAMIO_HW2_OP_EN_SHIFT},
	{PMIC_RG_VCAMIO_OP_EN_SET, PMIC_RG_VCAMIO_OP_EN_SET_ADDR, PMIC_RG_VCAMIO_OP_EN_SET_MASK,
	 PMIC_RG_VCAMIO_OP_EN_SET_SHIFT},
	{PMIC_RG_VCAMIO_OP_EN_CLR, PMIC_RG_VCAMIO_OP_EN_CLR_ADDR, PMIC_RG_VCAMIO_OP_EN_CLR_MASK,
	 PMIC_RG_VCAMIO_OP_EN_CLR_SHIFT},
	{PMIC_RG_VCAMIO_HW0_OP_CFG, PMIC_RG_VCAMIO_HW0_OP_CFG_ADDR, PMIC_RG_VCAMIO_HW0_OP_CFG_MASK,
	 PMIC_RG_VCAMIO_HW0_OP_CFG_SHIFT},
	{PMIC_RG_VCAMIO_HW1_OP_CFG, PMIC_RG_VCAMIO_HW1_OP_CFG_ADDR, PMIC_RG_VCAMIO_HW1_OP_CFG_MASK,
	 PMIC_RG_VCAMIO_HW1_OP_CFG_SHIFT},
	{PMIC_RG_VCAMIO_HW2_OP_CFG, PMIC_RG_VCAMIO_HW2_OP_CFG_ADDR, PMIC_RG_VCAMIO_HW2_OP_CFG_MASK,
	 PMIC_RG_VCAMIO_HW2_OP_CFG_SHIFT},
	{PMIC_RG_VCAMIO_GO_ON_OP, PMIC_RG_VCAMIO_GO_ON_OP_ADDR, PMIC_RG_VCAMIO_GO_ON_OP_MASK,
	 PMIC_RG_VCAMIO_GO_ON_OP_SHIFT},
	{PMIC_RG_VCAMIO_GO_LP_OP, PMIC_RG_VCAMIO_GO_LP_OP_ADDR, PMIC_RG_VCAMIO_GO_LP_OP_MASK,
	 PMIC_RG_VCAMIO_GO_LP_OP_SHIFT},
	{PMIC_RG_VCAMIO_OP_CFG_SET, PMIC_RG_VCAMIO_OP_CFG_SET_ADDR, PMIC_RG_VCAMIO_OP_CFG_SET_MASK,
	 PMIC_RG_VCAMIO_OP_CFG_SET_SHIFT},
	{PMIC_RG_VCAMIO_OP_CFG_CLR, PMIC_RG_VCAMIO_OP_CFG_CLR_ADDR, PMIC_RG_VCAMIO_OP_CFG_CLR_MASK,
	 PMIC_RG_VCAMIO_OP_CFG_CLR_SHIFT},
	{PMIC_DA_QI_VCAMIO_MODE, PMIC_DA_QI_VCAMIO_MODE_ADDR, PMIC_DA_QI_VCAMIO_MODE_MASK,
	 PMIC_DA_QI_VCAMIO_MODE_SHIFT},
	{PMIC_RG_VCAMIO_STBTD, PMIC_RG_VCAMIO_STBTD_ADDR, PMIC_RG_VCAMIO_STBTD_MASK,
	 PMIC_RG_VCAMIO_STBTD_SHIFT},
	{PMIC_DA_QI_VCAMIO_STB, PMIC_DA_QI_VCAMIO_STB_ADDR, PMIC_DA_QI_VCAMIO_STB_MASK,
	 PMIC_DA_QI_VCAMIO_STB_SHIFT},
	{PMIC_DA_QI_VCAMIO_EN, PMIC_DA_QI_VCAMIO_EN_ADDR, PMIC_DA_QI_VCAMIO_EN_MASK,
	 PMIC_DA_QI_VCAMIO_EN_SHIFT},
	{PMIC_RG_VCAMIO_OCFB_EN, PMIC_RG_VCAMIO_OCFB_EN_ADDR, PMIC_RG_VCAMIO_OCFB_EN_MASK,
	 PMIC_RG_VCAMIO_OCFB_EN_SHIFT},
	{PMIC_DA_QI_VCAMIO_OCFB_EN, PMIC_DA_QI_VCAMIO_OCFB_EN_ADDR, PMIC_DA_QI_VCAMIO_OCFB_EN_MASK,
	 PMIC_DA_QI_VCAMIO_OCFB_EN_SHIFT},
	{PMIC_RG_VCAMIO_DUMMY_LOAD, PMIC_RG_VCAMIO_DUMMY_LOAD_ADDR, PMIC_RG_VCAMIO_DUMMY_LOAD_MASK,
	 PMIC_RG_VCAMIO_DUMMY_LOAD_SHIFT},
	{PMIC_DA_QI_VCAMIO_DUMMY_LOAD, PMIC_DA_QI_VCAMIO_DUMMY_LOAD_ADDR,
	 PMIC_DA_QI_VCAMIO_DUMMY_LOAD_MASK, PMIC_DA_QI_VCAMIO_DUMMY_LOAD_SHIFT},
	{PMIC_RG_VMIPI_SW_EN, PMIC_RG_VMIPI_SW_EN_ADDR, PMIC_RG_VMIPI_SW_EN_MASK,
	 PMIC_RG_VMIPI_SW_EN_SHIFT},
	{PMIC_RG_VMIPI_SW_LP, PMIC_RG_VMIPI_SW_LP_ADDR, PMIC_RG_VMIPI_SW_LP_MASK,
	 PMIC_RG_VMIPI_SW_LP_SHIFT},
	{PMIC_RG_VMIPI_SW_OP_EN, PMIC_RG_VMIPI_SW_OP_EN_ADDR, PMIC_RG_VMIPI_SW_OP_EN_MASK,
	 PMIC_RG_VMIPI_SW_OP_EN_SHIFT},
	{PMIC_RG_VMIPI_HW0_OP_EN, PMIC_RG_VMIPI_HW0_OP_EN_ADDR, PMIC_RG_VMIPI_HW0_OP_EN_MASK,
	 PMIC_RG_VMIPI_HW0_OP_EN_SHIFT},
	{PMIC_RG_VMIPI_HW1_OP_EN, PMIC_RG_VMIPI_HW1_OP_EN_ADDR, PMIC_RG_VMIPI_HW1_OP_EN_MASK,
	 PMIC_RG_VMIPI_HW1_OP_EN_SHIFT},
	{PMIC_RG_VMIPI_HW2_OP_EN, PMIC_RG_VMIPI_HW2_OP_EN_ADDR, PMIC_RG_VMIPI_HW2_OP_EN_MASK,
	 PMIC_RG_VMIPI_HW2_OP_EN_SHIFT},
	{PMIC_RG_VMIPI_OP_EN_SET, PMIC_RG_VMIPI_OP_EN_SET_ADDR, PMIC_RG_VMIPI_OP_EN_SET_MASK,
	 PMIC_RG_VMIPI_OP_EN_SET_SHIFT},
	{PMIC_RG_VMIPI_OP_EN_CLR, PMIC_RG_VMIPI_OP_EN_CLR_ADDR, PMIC_RG_VMIPI_OP_EN_CLR_MASK,
	 PMIC_RG_VMIPI_OP_EN_CLR_SHIFT},
	{PMIC_RG_VMIPI_HW0_OP_CFG, PMIC_RG_VMIPI_HW0_OP_CFG_ADDR, PMIC_RG_VMIPI_HW0_OP_CFG_MASK,
	 PMIC_RG_VMIPI_HW0_OP_CFG_SHIFT},
	{PMIC_RG_VMIPI_HW1_OP_CFG, PMIC_RG_VMIPI_HW1_OP_CFG_ADDR, PMIC_RG_VMIPI_HW1_OP_CFG_MASK,
	 PMIC_RG_VMIPI_HW1_OP_CFG_SHIFT},
	{PMIC_RG_VMIPI_HW2_OP_CFG, PMIC_RG_VMIPI_HW2_OP_CFG_ADDR, PMIC_RG_VMIPI_HW2_OP_CFG_MASK,
	 PMIC_RG_VMIPI_HW2_OP_CFG_SHIFT},
	{PMIC_RG_VMIPI_GO_ON_OP, PMIC_RG_VMIPI_GO_ON_OP_ADDR, PMIC_RG_VMIPI_GO_ON_OP_MASK,
	 PMIC_RG_VMIPI_GO_ON_OP_SHIFT},
	{PMIC_RG_VMIPI_GO_LP_OP, PMIC_RG_VMIPI_GO_LP_OP_ADDR, PMIC_RG_VMIPI_GO_LP_OP_MASK,
	 PMIC_RG_VMIPI_GO_LP_OP_SHIFT},
	{PMIC_RG_VMIPI_OP_CFG_SET, PMIC_RG_VMIPI_OP_CFG_SET_ADDR, PMIC_RG_VMIPI_OP_CFG_SET_MASK,
	 PMIC_RG_VMIPI_OP_CFG_SET_SHIFT},
	{PMIC_RG_VMIPI_OP_CFG_CLR, PMIC_RG_VMIPI_OP_CFG_CLR_ADDR, PMIC_RG_VMIPI_OP_CFG_CLR_MASK,
	 PMIC_RG_VMIPI_OP_CFG_CLR_SHIFT},
	{PMIC_DA_QI_VMIPI_MODE, PMIC_DA_QI_VMIPI_MODE_ADDR, PMIC_DA_QI_VMIPI_MODE_MASK,
	 PMIC_DA_QI_VMIPI_MODE_SHIFT},
	{PMIC_RG_VMIPI_STBTD, PMIC_RG_VMIPI_STBTD_ADDR, PMIC_RG_VMIPI_STBTD_MASK,
	 PMIC_RG_VMIPI_STBTD_SHIFT},
	{PMIC_DA_QI_VMIPI_STB, PMIC_DA_QI_VMIPI_STB_ADDR, PMIC_DA_QI_VMIPI_STB_MASK,
	 PMIC_DA_QI_VMIPI_STB_SHIFT},
	{PMIC_DA_QI_VMIPI_EN, PMIC_DA_QI_VMIPI_EN_ADDR, PMIC_DA_QI_VMIPI_EN_MASK,
	 PMIC_DA_QI_VMIPI_EN_SHIFT},
	{PMIC_RG_VMIPI_OCFB_EN, PMIC_RG_VMIPI_OCFB_EN_ADDR, PMIC_RG_VMIPI_OCFB_EN_MASK,
	 PMIC_RG_VMIPI_OCFB_EN_SHIFT},
	{PMIC_DA_QI_VMIPI_OCFB_EN, PMIC_DA_QI_VMIPI_OCFB_EN_ADDR, PMIC_DA_QI_VMIPI_OCFB_EN_MASK,
	 PMIC_DA_QI_VMIPI_OCFB_EN_SHIFT},
	{PMIC_RG_VMIPI_DUMMY_LOAD, PMIC_RG_VMIPI_DUMMY_LOAD_ADDR, PMIC_RG_VMIPI_DUMMY_LOAD_MASK,
	 PMIC_RG_VMIPI_DUMMY_LOAD_SHIFT},
	{PMIC_DA_QI_VMIPI_DUMMY_LOAD, PMIC_DA_QI_VMIPI_DUMMY_LOAD_ADDR,
	 PMIC_DA_QI_VMIPI_DUMMY_LOAD_MASK, PMIC_DA_QI_VMIPI_DUMMY_LOAD_SHIFT},
	{PMIC_RG_VGP3_SW_EN, PMIC_RG_VGP3_SW_EN_ADDR, PMIC_RG_VGP3_SW_EN_MASK,
	 PMIC_RG_VGP3_SW_EN_SHIFT},
	{PMIC_RG_VGP3_SW_LP, PMIC_RG_VGP3_SW_LP_ADDR, PMIC_RG_VGP3_SW_LP_MASK,
	 PMIC_RG_VGP3_SW_LP_SHIFT},
	{PMIC_RG_VGP3_SW_OP_EN, PMIC_RG_VGP3_SW_OP_EN_ADDR, PMIC_RG_VGP3_SW_OP_EN_MASK,
	 PMIC_RG_VGP3_SW_OP_EN_SHIFT},
	{PMIC_RG_VGP3_HW0_OP_EN, PMIC_RG_VGP3_HW0_OP_EN_ADDR, PMIC_RG_VGP3_HW0_OP_EN_MASK,
	 PMIC_RG_VGP3_HW0_OP_EN_SHIFT},
	{PMIC_RG_VGP3_HW1_OP_EN, PMIC_RG_VGP3_HW1_OP_EN_ADDR, PMIC_RG_VGP3_HW1_OP_EN_MASK,
	 PMIC_RG_VGP3_HW1_OP_EN_SHIFT},
	{PMIC_RG_VGP3_HW2_OP_EN, PMIC_RG_VGP3_HW2_OP_EN_ADDR, PMIC_RG_VGP3_HW2_OP_EN_MASK,
	 PMIC_RG_VGP3_HW2_OP_EN_SHIFT},
	{PMIC_RG_VGP3_OP_EN_SET, PMIC_RG_VGP3_OP_EN_SET_ADDR, PMIC_RG_VGP3_OP_EN_SET_MASK,
	 PMIC_RG_VGP3_OP_EN_SET_SHIFT},
	{PMIC_RG_VGP3_OP_EN_CLR, PMIC_RG_VGP3_OP_EN_CLR_ADDR, PMIC_RG_VGP3_OP_EN_CLR_MASK,
	 PMIC_RG_VGP3_OP_EN_CLR_SHIFT},
	{PMIC_RG_VGP3_HW0_OP_CFG, PMIC_RG_VGP3_HW0_OP_CFG_ADDR, PMIC_RG_VGP3_HW0_OP_CFG_MASK,
	 PMIC_RG_VGP3_HW0_OP_CFG_SHIFT},
	{PMIC_RG_VGP3_HW1_OP_CFG, PMIC_RG_VGP3_HW1_OP_CFG_ADDR, PMIC_RG_VGP3_HW1_OP_CFG_MASK,
	 PMIC_RG_VGP3_HW1_OP_CFG_SHIFT},
	{PMIC_RG_VGP3_HW2_OP_CFG, PMIC_RG_VGP3_HW2_OP_CFG_ADDR, PMIC_RG_VGP3_HW2_OP_CFG_MASK,
	 PMIC_RG_VGP3_HW2_OP_CFG_SHIFT},
	{PMIC_RG_VGP3_GO_ON_OP, PMIC_RG_VGP3_GO_ON_OP_ADDR, PMIC_RG_VGP3_GO_ON_OP_MASK,
	 PMIC_RG_VGP3_GO_ON_OP_SHIFT},
	{PMIC_RG_VGP3_GO_LP_OP, PMIC_RG_VGP3_GO_LP_OP_ADDR, PMIC_RG_VGP3_GO_LP_OP_MASK,
	 PMIC_RG_VGP3_GO_LP_OP_SHIFT},
	{PMIC_RG_VGP3_OP_CFG_SET, PMIC_RG_VGP3_OP_CFG_SET_ADDR, PMIC_RG_VGP3_OP_CFG_SET_MASK,
	 PMIC_RG_VGP3_OP_CFG_SET_SHIFT},
	{PMIC_RG_VGP3_OP_CFG_CLR, PMIC_RG_VGP3_OP_CFG_CLR_ADDR, PMIC_RG_VGP3_OP_CFG_CLR_MASK,
	 PMIC_RG_VGP3_OP_CFG_CLR_SHIFT},
	{PMIC_DA_QI_VGP3_MODE, PMIC_DA_QI_VGP3_MODE_ADDR, PMIC_DA_QI_VGP3_MODE_MASK,
	 PMIC_DA_QI_VGP3_MODE_SHIFT},
	{PMIC_RG_VGP3_STBTD, PMIC_RG_VGP3_STBTD_ADDR, PMIC_RG_VGP3_STBTD_MASK,
	 PMIC_RG_VGP3_STBTD_SHIFT},
	{PMIC_DA_QI_VGP3_STB, PMIC_DA_QI_VGP3_STB_ADDR, PMIC_DA_QI_VGP3_STB_MASK,
	 PMIC_DA_QI_VGP3_STB_SHIFT},
	{PMIC_DA_QI_VGP3_EN, PMIC_DA_QI_VGP3_EN_ADDR, PMIC_DA_QI_VGP3_EN_MASK,
	 PMIC_DA_QI_VGP3_EN_SHIFT},
	{PMIC_RG_VGP3_OCFB_EN, PMIC_RG_VGP3_OCFB_EN_ADDR, PMIC_RG_VGP3_OCFB_EN_MASK,
	 PMIC_RG_VGP3_OCFB_EN_SHIFT},
	{PMIC_DA_QI_VGP3_OCFB_EN, PMIC_DA_QI_VGP3_OCFB_EN_ADDR, PMIC_DA_QI_VGP3_OCFB_EN_MASK,
	 PMIC_DA_QI_VGP3_OCFB_EN_SHIFT},
	{PMIC_RG_VGP3_DUMMY_LOAD, PMIC_RG_VGP3_DUMMY_LOAD_ADDR, PMIC_RG_VGP3_DUMMY_LOAD_MASK,
	 PMIC_RG_VGP3_DUMMY_LOAD_SHIFT},
	{PMIC_DA_QI_VGP3_DUMMY_LOAD, PMIC_DA_QI_VGP3_DUMMY_LOAD_ADDR,
	 PMIC_DA_QI_VGP3_DUMMY_LOAD_MASK, PMIC_DA_QI_VGP3_DUMMY_LOAD_SHIFT},
	{PMIC_RG_VCN33_SW_EN_BT, PMIC_RG_VCN33_SW_EN_BT_ADDR, PMIC_RG_VCN33_SW_EN_BT_MASK,
	 PMIC_RG_VCN33_SW_EN_BT_SHIFT},
	{PMIC_RG_VCN33_SW_LP_BT, PMIC_RG_VCN33_SW_LP_BT_ADDR, PMIC_RG_VCN33_SW_LP_BT_MASK,
	 PMIC_RG_VCN33_SW_LP_BT_SHIFT},
	{PMIC_RG_VCN33_SW_OP_EN_BT, PMIC_RG_VCN33_SW_OP_EN_BT_ADDR, PMIC_RG_VCN33_SW_OP_EN_BT_MASK,
	 PMIC_RG_VCN33_SW_OP_EN_BT_SHIFT},
	{PMIC_RG_VCN33_HW0_OP_EN_BT, PMIC_RG_VCN33_HW0_OP_EN_BT_ADDR,
	 PMIC_RG_VCN33_HW0_OP_EN_BT_MASK, PMIC_RG_VCN33_HW0_OP_EN_BT_SHIFT},
	{PMIC_RG_VCN33_HW1_OP_EN_BT, PMIC_RG_VCN33_HW1_OP_EN_BT_ADDR,
	 PMIC_RG_VCN33_HW1_OP_EN_BT_MASK, PMIC_RG_VCN33_HW1_OP_EN_BT_SHIFT},
	{PMIC_RG_VCN33_HW2_OP_EN_BT, PMIC_RG_VCN33_HW2_OP_EN_BT_ADDR,
	 PMIC_RG_VCN33_HW2_OP_EN_BT_MASK, PMIC_RG_VCN33_HW2_OP_EN_BT_SHIFT},
	{PMIC_RG_VCN33_OP_EN_BT_SET, PMIC_RG_VCN33_OP_EN_BT_SET_ADDR,
	 PMIC_RG_VCN33_OP_EN_BT_SET_MASK, PMIC_RG_VCN33_OP_EN_BT_SET_SHIFT},
	{PMIC_RG_VCN33_OP_EN_BT_CLR, PMIC_RG_VCN33_OP_EN_BT_CLR_ADDR,
	 PMIC_RG_VCN33_OP_EN_BT_CLR_MASK, PMIC_RG_VCN33_OP_EN_BT_CLR_SHIFT},
	{PMIC_RG_VCN33_HW0_OP_CFG_BT, PMIC_RG_VCN33_HW0_OP_CFG_BT_ADDR,
	 PMIC_RG_VCN33_HW0_OP_CFG_BT_MASK, PMIC_RG_VCN33_HW0_OP_CFG_BT_SHIFT},
	{PMIC_RG_VCN33_HW1_OP_CFG_BT, PMIC_RG_VCN33_HW1_OP_CFG_BT_ADDR,
	 PMIC_RG_VCN33_HW1_OP_CFG_BT_MASK, PMIC_RG_VCN33_HW1_OP_CFG_BT_SHIFT},
	{PMIC_RG_VCN33_HW2_OP_CFG_BT, PMIC_RG_VCN33_HW2_OP_CFG_BT_ADDR,
	 PMIC_RG_VCN33_HW2_OP_CFG_BT_MASK, PMIC_RG_VCN33_HW2_OP_CFG_BT_SHIFT},
	{PMIC_RG_VCN33_GO_ON_OP_BT, PMIC_RG_VCN33_GO_ON_OP_BT_ADDR, PMIC_RG_VCN33_GO_ON_OP_BT_MASK,
	 PMIC_RG_VCN33_GO_ON_OP_BT_SHIFT},
	{PMIC_RG_VCN33_GO_LP_OP_BT, PMIC_RG_VCN33_GO_LP_OP_BT_ADDR, PMIC_RG_VCN33_GO_LP_OP_BT_MASK,
	 PMIC_RG_VCN33_GO_LP_OP_BT_SHIFT},
	{PMIC_RG_VCN33_OP_CFG_BT_SET, PMIC_RG_VCN33_OP_CFG_BT_SET_ADDR,
	 PMIC_RG_VCN33_OP_CFG_BT_SET_MASK, PMIC_RG_VCN33_OP_CFG_BT_SET_SHIFT},
	{PMIC_RG_VCN33_OP_CFG_BT_CLR, PMIC_RG_VCN33_OP_CFG_BT_CLR_ADDR,
	 PMIC_RG_VCN33_OP_CFG_BT_CLR_MASK, PMIC_RG_VCN33_OP_CFG_BT_CLR_SHIFT},
	{PMIC_RG_VCN33_SW_EN_WIFI, PMIC_RG_VCN33_SW_EN_WIFI_ADDR, PMIC_RG_VCN33_SW_EN_WIFI_MASK,
	 PMIC_RG_VCN33_SW_EN_WIFI_SHIFT},
	{PMIC_RG_VCN33_SW_LP_WIFI, PMIC_RG_VCN33_SW_LP_WIFI_ADDR, PMIC_RG_VCN33_SW_LP_WIFI_MASK,
	 PMIC_RG_VCN33_SW_LP_WIFI_SHIFT},
	{PMIC_RG_VCN33_SW_OP_EN_WIFI, PMIC_RG_VCN33_SW_OP_EN_WIFI_ADDR,
	 PMIC_RG_VCN33_SW_OP_EN_WIFI_MASK, PMIC_RG_VCN33_SW_OP_EN_WIFI_SHIFT},
	{PMIC_RG_VCN33_HW0_OP_EN_WIFI, PMIC_RG_VCN33_HW0_OP_EN_WIFI_ADDR,
	 PMIC_RG_VCN33_HW0_OP_EN_WIFI_MASK, PMIC_RG_VCN33_HW0_OP_EN_WIFI_SHIFT},
	{PMIC_RG_VCN33_HW1_OP_EN_WIFI, PMIC_RG_VCN33_HW1_OP_EN_WIFI_ADDR,
	 PMIC_RG_VCN33_HW1_OP_EN_WIFI_MASK, PMIC_RG_VCN33_HW1_OP_EN_WIFI_SHIFT},
	{PMIC_RG_VCN33_HW2_OP_EN_WIFI, PMIC_RG_VCN33_HW2_OP_EN_WIFI_ADDR,
	 PMIC_RG_VCN33_HW2_OP_EN_WIFI_MASK, PMIC_RG_VCN33_HW2_OP_EN_WIFI_SHIFT},
	{PMIC_RG_VCN33_OP_EN_WIFI_SET, PMIC_RG_VCN33_OP_EN_WIFI_SET_ADDR,
	 PMIC_RG_VCN33_OP_EN_WIFI_SET_MASK, PMIC_RG_VCN33_OP_EN_WIFI_SET_SHIFT},
	{PMIC_RG_VCN33_OP_EN_WIFI_CLR, PMIC_RG_VCN33_OP_EN_WIFI_CLR_ADDR,
	 PMIC_RG_VCN33_OP_EN_WIFI_CLR_MASK, PMIC_RG_VCN33_OP_EN_WIFI_CLR_SHIFT},
	{PMIC_RG_VCN33_HW0_OP_CFG_WIFI, PMIC_RG_VCN33_HW0_OP_CFG_WIFI_ADDR,
	 PMIC_RG_VCN33_HW0_OP_CFG_WIFI_MASK, PMIC_RG_VCN33_HW0_OP_CFG_WIFI_SHIFT},
	{PMIC_RG_VCN33_HW1_OP_CFG_WIFI, PMIC_RG_VCN33_HW1_OP_CFG_WIFI_ADDR,
	 PMIC_RG_VCN33_HW1_OP_CFG_WIFI_MASK, PMIC_RG_VCN33_HW1_OP_CFG_WIFI_SHIFT},
	{PMIC_RG_VCN33_HW2_OP_CFG_WIFI, PMIC_RG_VCN33_HW2_OP_CFG_WIFI_ADDR,
	 PMIC_RG_VCN33_HW2_OP_CFG_WIFI_MASK, PMIC_RG_VCN33_HW2_OP_CFG_WIFI_SHIFT},
	{PMIC_RG_VCN33_GO_ON_OP_WIFI, PMIC_RG_VCN33_GO_ON_OP_WIFI_ADDR,
	 PMIC_RG_VCN33_GO_ON_OP_WIFI_MASK, PMIC_RG_VCN33_GO_ON_OP_WIFI_SHIFT},
	{PMIC_RG_VCN33_GO_LP_OP_WIFI, PMIC_RG_VCN33_GO_LP_OP_WIFI_ADDR,
	 PMIC_RG_VCN33_GO_LP_OP_WIFI_MASK, PMIC_RG_VCN33_GO_LP_OP_WIFI_SHIFT},
	{PMIC_RG_VCN33_OP_CFG_WIFI_SET, PMIC_RG_VCN33_OP_CFG_WIFI_SET_ADDR,
	 PMIC_RG_VCN33_OP_CFG_WIFI_SET_MASK, PMIC_RG_VCN33_OP_CFG_WIFI_SET_SHIFT},
	{PMIC_RG_VCN33_OP_CFG_WIFI_CLR, PMIC_RG_VCN33_OP_CFG_WIFI_CLR_ADDR,
	 PMIC_RG_VCN33_OP_CFG_WIFI_CLR_MASK, PMIC_RG_VCN33_OP_CFG_WIFI_CLR_SHIFT},
	{PMIC_DA_QI_VCN33_MODE, PMIC_DA_QI_VCN33_MODE_ADDR, PMIC_DA_QI_VCN33_MODE_MASK,
	 PMIC_DA_QI_VCN33_MODE_SHIFT},
	{PMIC_RG_VCN33_STBTD, PMIC_RG_VCN33_STBTD_ADDR, PMIC_RG_VCN33_STBTD_MASK,
	 PMIC_RG_VCN33_STBTD_SHIFT},
	{PMIC_DA_QI_VCN33_STB, PMIC_DA_QI_VCN33_STB_ADDR, PMIC_DA_QI_VCN33_STB_MASK,
	 PMIC_DA_QI_VCN33_STB_SHIFT},
	{PMIC_DA_QI_VCN33_EN, PMIC_DA_QI_VCN33_EN_ADDR, PMIC_DA_QI_VCN33_EN_MASK,
	 PMIC_DA_QI_VCN33_EN_SHIFT},
	{PMIC_RG_VCN33_OCFB_EN, PMIC_RG_VCN33_OCFB_EN_ADDR, PMIC_RG_VCN33_OCFB_EN_MASK,
	 PMIC_RG_VCN33_OCFB_EN_SHIFT},
	{PMIC_DA_QI_VCN33_OCFB_EN, PMIC_DA_QI_VCN33_OCFB_EN_ADDR, PMIC_DA_QI_VCN33_OCFB_EN_MASK,
	 PMIC_DA_QI_VCN33_OCFB_EN_SHIFT},
	{PMIC_RG_VCN33_DUMMY_LOAD, PMIC_RG_VCN33_DUMMY_LOAD_ADDR, PMIC_RG_VCN33_DUMMY_LOAD_MASK,
	 PMIC_RG_VCN33_DUMMY_LOAD_SHIFT},
	{PMIC_DA_QI_VCN33_DUMMY_LOAD, PMIC_DA_QI_VCN33_DUMMY_LOAD_ADDR,
	 PMIC_DA_QI_VCN33_DUMMY_LOAD_MASK, PMIC_DA_QI_VCN33_DUMMY_LOAD_SHIFT},
	{PMIC_RG_VCN18_SW_EN_BT, PMIC_RG_VCN18_SW_EN_BT_ADDR, PMIC_RG_VCN18_SW_EN_BT_MASK,
	 PMIC_RG_VCN18_SW_EN_BT_SHIFT},
	{PMIC_RG_VCN18_SW_LP_BT, PMIC_RG_VCN18_SW_LP_BT_ADDR, PMIC_RG_VCN18_SW_LP_BT_MASK,
	 PMIC_RG_VCN18_SW_LP_BT_SHIFT},
	{PMIC_RG_VCN18_SW_OP_EN_BT, PMIC_RG_VCN18_SW_OP_EN_BT_ADDR, PMIC_RG_VCN18_SW_OP_EN_BT_MASK,
	 PMIC_RG_VCN18_SW_OP_EN_BT_SHIFT},
	{PMIC_RG_VCN18_HW0_OP_EN_BT, PMIC_RG_VCN18_HW0_OP_EN_BT_ADDR,
	 PMIC_RG_VCN18_HW0_OP_EN_BT_MASK, PMIC_RG_VCN18_HW0_OP_EN_BT_SHIFT},
	{PMIC_RG_VCN18_HW1_OP_EN_BT, PMIC_RG_VCN18_HW1_OP_EN_BT_ADDR,
	 PMIC_RG_VCN18_HW1_OP_EN_BT_MASK, PMIC_RG_VCN18_HW1_OP_EN_BT_SHIFT},
	{PMIC_RG_VCN18_HW2_OP_EN_BT, PMIC_RG_VCN18_HW2_OP_EN_BT_ADDR,
	 PMIC_RG_VCN18_HW2_OP_EN_BT_MASK, PMIC_RG_VCN18_HW2_OP_EN_BT_SHIFT},
	{PMIC_RG_VCN18_OP_EN_BT_SET, PMIC_RG_VCN18_OP_EN_BT_SET_ADDR,
	 PMIC_RG_VCN18_OP_EN_BT_SET_MASK, PMIC_RG_VCN18_OP_EN_BT_SET_SHIFT},
	{PMIC_RG_VCN18_OP_EN_BT_CLR, PMIC_RG_VCN18_OP_EN_BT_CLR_ADDR,
	 PMIC_RG_VCN18_OP_EN_BT_CLR_MASK, PMIC_RG_VCN18_OP_EN_BT_CLR_SHIFT},
	{PMIC_RG_VCN18_HW0_OP_CFG_BT, PMIC_RG_VCN18_HW0_OP_CFG_BT_ADDR,
	 PMIC_RG_VCN18_HW0_OP_CFG_BT_MASK, PMIC_RG_VCN18_HW0_OP_CFG_BT_SHIFT},
	{PMIC_RG_VCN18_HW1_OP_CFG_BT, PMIC_RG_VCN18_HW1_OP_CFG_BT_ADDR,
	 PMIC_RG_VCN18_HW1_OP_CFG_BT_MASK, PMIC_RG_VCN18_HW1_OP_CFG_BT_SHIFT},
	{PMIC_RG_VCN18_HW2_OP_CFG_BT, PMIC_RG_VCN18_HW2_OP_CFG_BT_ADDR,
	 PMIC_RG_VCN18_HW2_OP_CFG_BT_MASK, PMIC_RG_VCN18_HW2_OP_CFG_BT_SHIFT},
	{PMIC_RG_VCN18_GO_ON_OP_BT, PMIC_RG_VCN18_GO_ON_OP_BT_ADDR, PMIC_RG_VCN18_GO_ON_OP_BT_MASK,
	 PMIC_RG_VCN18_GO_ON_OP_BT_SHIFT},
	{PMIC_RG_VCN18_GO_LP_OP_BT, PMIC_RG_VCN18_GO_LP_OP_BT_ADDR, PMIC_RG_VCN18_GO_LP_OP_BT_MASK,
	 PMIC_RG_VCN18_GO_LP_OP_BT_SHIFT},
	{PMIC_RG_VCN18_OP_CFG_BT_SET, PMIC_RG_VCN18_OP_CFG_BT_SET_ADDR,
	 PMIC_RG_VCN18_OP_CFG_BT_SET_MASK, PMIC_RG_VCN18_OP_CFG_BT_SET_SHIFT},
	{PMIC_RG_VCN18_OP_CFG_BT_CLR, PMIC_RG_VCN18_OP_CFG_BT_CLR_ADDR,
	 PMIC_RG_VCN18_OP_CFG_BT_CLR_MASK, PMIC_RG_VCN18_OP_CFG_BT_CLR_SHIFT},
	{PMIC_RG_VCN18_SW_EN_WIFI, PMIC_RG_VCN18_SW_EN_WIFI_ADDR, PMIC_RG_VCN18_SW_EN_WIFI_MASK,
	 PMIC_RG_VCN18_SW_EN_WIFI_SHIFT},
	{PMIC_RG_VCN18_SW_LP_WIFI, PMIC_RG_VCN18_SW_LP_WIFI_ADDR, PMIC_RG_VCN18_SW_LP_WIFI_MASK,
	 PMIC_RG_VCN18_SW_LP_WIFI_SHIFT},
	{PMIC_RG_VCN18_SW_OP_EN_WIFI, PMIC_RG_VCN18_SW_OP_EN_WIFI_ADDR,
	 PMIC_RG_VCN18_SW_OP_EN_WIFI_MASK, PMIC_RG_VCN18_SW_OP_EN_WIFI_SHIFT},
	{PMIC_RG_VCN18_HW0_OP_EN_WIFI, PMIC_RG_VCN18_HW0_OP_EN_WIFI_ADDR,
	 PMIC_RG_VCN18_HW0_OP_EN_WIFI_MASK, PMIC_RG_VCN18_HW0_OP_EN_WIFI_SHIFT},
	{PMIC_RG_VCN18_HW1_OP_EN_WIFI, PMIC_RG_VCN18_HW1_OP_EN_WIFI_ADDR,
	 PMIC_RG_VCN18_HW1_OP_EN_WIFI_MASK, PMIC_RG_VCN18_HW1_OP_EN_WIFI_SHIFT},
	{PMIC_RG_VCN18_HW2_OP_EN_WIFI, PMIC_RG_VCN18_HW2_OP_EN_WIFI_ADDR,
	 PMIC_RG_VCN18_HW2_OP_EN_WIFI_MASK, PMIC_RG_VCN18_HW2_OP_EN_WIFI_SHIFT},
	{PMIC_RG_VCN18_OP_EN_WIFI_SET, PMIC_RG_VCN18_OP_EN_WIFI_SET_ADDR,
	 PMIC_RG_VCN18_OP_EN_WIFI_SET_MASK, PMIC_RG_VCN18_OP_EN_WIFI_SET_SHIFT},
	{PMIC_RG_VCN18_OP_EN_WIFI_CLR, PMIC_RG_VCN18_OP_EN_WIFI_CLR_ADDR,
	 PMIC_RG_VCN18_OP_EN_WIFI_CLR_MASK, PMIC_RG_VCN18_OP_EN_WIFI_CLR_SHIFT},
	{PMIC_RG_VCN18_HW0_OP_CFG_WIFI, PMIC_RG_VCN18_HW0_OP_CFG_WIFI_ADDR,
	 PMIC_RG_VCN18_HW0_OP_CFG_WIFI_MASK, PMIC_RG_VCN18_HW0_OP_CFG_WIFI_SHIFT},
	{PMIC_RG_VCN18_HW1_OP_CFG_WIFI, PMIC_RG_VCN18_HW1_OP_CFG_WIFI_ADDR,
	 PMIC_RG_VCN18_HW1_OP_CFG_WIFI_MASK, PMIC_RG_VCN18_HW1_OP_CFG_WIFI_SHIFT},
	{PMIC_RG_VCN18_HW2_OP_CFG_WIFI, PMIC_RG_VCN18_HW2_OP_CFG_WIFI_ADDR,
	 PMIC_RG_VCN18_HW2_OP_CFG_WIFI_MASK, PMIC_RG_VCN18_HW2_OP_CFG_WIFI_SHIFT},
	{PMIC_RG_VCN18_GO_ON_OP_WIFI, PMIC_RG_VCN18_GO_ON_OP_WIFI_ADDR,
	 PMIC_RG_VCN18_GO_ON_OP_WIFI_MASK, PMIC_RG_VCN18_GO_ON_OP_WIFI_SHIFT},
	{PMIC_RG_VCN18_GO_LP_OP_WIFI, PMIC_RG_VCN18_GO_LP_OP_WIFI_ADDR,
	 PMIC_RG_VCN18_GO_LP_OP_WIFI_MASK, PMIC_RG_VCN18_GO_LP_OP_WIFI_SHIFT},
	{PMIC_RG_VCN18_OP_CFG_WIFI_SET, PMIC_RG_VCN18_OP_CFG_WIFI_SET_ADDR,
	 PMIC_RG_VCN18_OP_CFG_WIFI_SET_MASK, PMIC_RG_VCN18_OP_CFG_WIFI_SET_SHIFT},
	{PMIC_RG_VCN18_OP_CFG_WIFI_CLR, PMIC_RG_VCN18_OP_CFG_WIFI_CLR_ADDR,
	 PMIC_RG_VCN18_OP_CFG_WIFI_CLR_MASK, PMIC_RG_VCN18_OP_CFG_WIFI_CLR_SHIFT},
	{PMIC_DA_QI_VCN18_MODE, PMIC_DA_QI_VCN18_MODE_ADDR, PMIC_DA_QI_VCN18_MODE_MASK,
	 PMIC_DA_QI_VCN18_MODE_SHIFT},
	{PMIC_RG_VCN18_STBTD, PMIC_RG_VCN18_STBTD_ADDR, PMIC_RG_VCN18_STBTD_MASK,
	 PMIC_RG_VCN18_STBTD_SHIFT},
	{PMIC_DA_QI_VCN18_STB, PMIC_DA_QI_VCN18_STB_ADDR, PMIC_DA_QI_VCN18_STB_MASK,
	 PMIC_DA_QI_VCN18_STB_SHIFT},
	{PMIC_DA_QI_VCN18_EN, PMIC_DA_QI_VCN18_EN_ADDR, PMIC_DA_QI_VCN18_EN_MASK,
	 PMIC_DA_QI_VCN18_EN_SHIFT},
	{PMIC_RG_VCN18_OCFB_EN, PMIC_RG_VCN18_OCFB_EN_ADDR, PMIC_RG_VCN18_OCFB_EN_MASK,
	 PMIC_RG_VCN18_OCFB_EN_SHIFT},
	{PMIC_DA_QI_VCN18_OCFB_EN, PMIC_DA_QI_VCN18_OCFB_EN_ADDR, PMIC_DA_QI_VCN18_OCFB_EN_MASK,
	 PMIC_DA_QI_VCN18_OCFB_EN_SHIFT},
	{PMIC_RG_VCN18_DUMMY_LOAD, PMIC_RG_VCN18_DUMMY_LOAD_ADDR, PMIC_RG_VCN18_DUMMY_LOAD_MASK,
	 PMIC_RG_VCN18_DUMMY_LOAD_SHIFT},
	{PMIC_DA_QI_VCN18_DUMMY_LOAD, PMIC_DA_QI_VCN18_DUMMY_LOAD_ADDR,
	 PMIC_DA_QI_VCN18_DUMMY_LOAD_MASK, PMIC_DA_QI_VCN18_DUMMY_LOAD_SHIFT},
	{PMIC_RG_VCN28_SW_EN, PMIC_RG_VCN28_SW_EN_ADDR, PMIC_RG_VCN28_SW_EN_MASK,
	 PMIC_RG_VCN28_SW_EN_SHIFT},
	{PMIC_RG_VCN28_SW_LP, PMIC_RG_VCN28_SW_LP_ADDR, PMIC_RG_VCN28_SW_LP_MASK,
	 PMIC_RG_VCN28_SW_LP_SHIFT},
	{PMIC_RG_VCN28_SW_OP_EN, PMIC_RG_VCN28_SW_OP_EN_ADDR, PMIC_RG_VCN28_SW_OP_EN_MASK,
	 PMIC_RG_VCN28_SW_OP_EN_SHIFT},
	{PMIC_RG_VCN28_HW0_OP_EN, PMIC_RG_VCN28_HW0_OP_EN_ADDR, PMIC_RG_VCN28_HW0_OP_EN_MASK,
	 PMIC_RG_VCN28_HW0_OP_EN_SHIFT},
	{PMIC_RG_VCN28_HW1_OP_EN, PMIC_RG_VCN28_HW1_OP_EN_ADDR, PMIC_RG_VCN28_HW1_OP_EN_MASK,
	 PMIC_RG_VCN28_HW1_OP_EN_SHIFT},
	{PMIC_RG_VCN28_HW2_OP_EN, PMIC_RG_VCN28_HW2_OP_EN_ADDR, PMIC_RG_VCN28_HW2_OP_EN_MASK,
	 PMIC_RG_VCN28_HW2_OP_EN_SHIFT},
	{PMIC_RG_VCN28_HW3_OP_EN, PMIC_RG_VCN28_HW3_OP_EN_ADDR, PMIC_RG_VCN28_HW3_OP_EN_MASK,
	 PMIC_RG_VCN28_HW3_OP_EN_SHIFT},
	{PMIC_RG_VCN28_OP_EN_SET, PMIC_RG_VCN28_OP_EN_SET_ADDR, PMIC_RG_VCN28_OP_EN_SET_MASK,
	 PMIC_RG_VCN28_OP_EN_SET_SHIFT},
	{PMIC_RG_VCN28_OP_EN_CLR, PMIC_RG_VCN28_OP_EN_CLR_ADDR, PMIC_RG_VCN28_OP_EN_CLR_MASK,
	 PMIC_RG_VCN28_OP_EN_CLR_SHIFT},
	{PMIC_RG_VCN28_HW0_OP_CFG, PMIC_RG_VCN28_HW0_OP_CFG_ADDR, PMIC_RG_VCN28_HW0_OP_CFG_MASK,
	 PMIC_RG_VCN28_HW0_OP_CFG_SHIFT},
	{PMIC_RG_VCN28_HW1_OP_CFG, PMIC_RG_VCN28_HW1_OP_CFG_ADDR, PMIC_RG_VCN28_HW1_OP_CFG_MASK,
	 PMIC_RG_VCN28_HW1_OP_CFG_SHIFT},
	{PMIC_RG_VCN28_HW2_OP_CFG, PMIC_RG_VCN28_HW2_OP_CFG_ADDR, PMIC_RG_VCN28_HW2_OP_CFG_MASK,
	 PMIC_RG_VCN28_HW2_OP_CFG_SHIFT},
	{PMIC_RG_VCN28_HW3_OP_CFG, PMIC_RG_VCN28_HW3_OP_CFG_ADDR, PMIC_RG_VCN28_HW3_OP_CFG_MASK,
	 PMIC_RG_VCN28_HW3_OP_CFG_SHIFT},
	{PMIC_RG_VCN28_GO_ON_OP, PMIC_RG_VCN28_GO_ON_OP_ADDR, PMIC_RG_VCN28_GO_ON_OP_MASK,
	 PMIC_RG_VCN28_GO_ON_OP_SHIFT},
	{PMIC_RG_VCN28_GO_LP_OP, PMIC_RG_VCN28_GO_LP_OP_ADDR, PMIC_RG_VCN28_GO_LP_OP_MASK,
	 PMIC_RG_VCN28_GO_LP_OP_SHIFT},
	{PMIC_RG_VCN28_OP_CFG_SET, PMIC_RG_VCN28_OP_CFG_SET_ADDR, PMIC_RG_VCN28_OP_CFG_SET_MASK,
	 PMIC_RG_VCN28_OP_CFG_SET_SHIFT},
	{PMIC_RG_VCN28_OP_CFG_CLR, PMIC_RG_VCN28_OP_CFG_CLR_ADDR, PMIC_RG_VCN28_OP_CFG_CLR_MASK,
	 PMIC_RG_VCN28_OP_CFG_CLR_SHIFT},
	{PMIC_DA_QI_VCN28_MODE, PMIC_DA_QI_VCN28_MODE_ADDR, PMIC_DA_QI_VCN28_MODE_MASK,
	 PMIC_DA_QI_VCN28_MODE_SHIFT},
	{PMIC_RG_VCN28_STBTD, PMIC_RG_VCN28_STBTD_ADDR, PMIC_RG_VCN28_STBTD_MASK,
	 PMIC_RG_VCN28_STBTD_SHIFT},
	{PMIC_DA_QI_VCN28_STB, PMIC_DA_QI_VCN28_STB_ADDR, PMIC_DA_QI_VCN28_STB_MASK,
	 PMIC_DA_QI_VCN28_STB_SHIFT},
	{PMIC_DA_QI_VCN28_EN, PMIC_DA_QI_VCN28_EN_ADDR, PMIC_DA_QI_VCN28_EN_MASK,
	 PMIC_DA_QI_VCN28_EN_SHIFT},
	{PMIC_RG_VCN28_OCFB_EN, PMIC_RG_VCN28_OCFB_EN_ADDR, PMIC_RG_VCN28_OCFB_EN_MASK,
	 PMIC_RG_VCN28_OCFB_EN_SHIFT},
	{PMIC_DA_QI_VCN28_OCFB_EN, PMIC_DA_QI_VCN28_OCFB_EN_ADDR, PMIC_DA_QI_VCN28_OCFB_EN_MASK,
	 PMIC_DA_QI_VCN28_OCFB_EN_SHIFT},
	{PMIC_RG_VIBR_SW_EN, PMIC_RG_VIBR_SW_EN_ADDR, PMIC_RG_VIBR_SW_EN_MASK,
	 PMIC_RG_VIBR_SW_EN_SHIFT},
	{PMIC_RG_VIBR_SW_LP, PMIC_RG_VIBR_SW_LP_ADDR, PMIC_RG_VIBR_SW_LP_MASK,
	 PMIC_RG_VIBR_SW_LP_SHIFT},
	{PMIC_RG_VIBR_SW_OP_EN, PMIC_RG_VIBR_SW_OP_EN_ADDR, PMIC_RG_VIBR_SW_OP_EN_MASK,
	 PMIC_RG_VIBR_SW_OP_EN_SHIFT},
	{PMIC_RG_VIBR_HW0_OP_EN, PMIC_RG_VIBR_HW0_OP_EN_ADDR, PMIC_RG_VIBR_HW0_OP_EN_MASK,
	 PMIC_RG_VIBR_HW0_OP_EN_SHIFT},
	{PMIC_RG_VIBR_HW1_OP_EN, PMIC_RG_VIBR_HW1_OP_EN_ADDR, PMIC_RG_VIBR_HW1_OP_EN_MASK,
	 PMIC_RG_VIBR_HW1_OP_EN_SHIFT},
	{PMIC_RG_VIBR_HW2_OP_EN, PMIC_RG_VIBR_HW2_OP_EN_ADDR, PMIC_RG_VIBR_HW2_OP_EN_MASK,
	 PMIC_RG_VIBR_HW2_OP_EN_SHIFT},
	{PMIC_RG_VIBR_OP_EN_SET, PMIC_RG_VIBR_OP_EN_SET_ADDR, PMIC_RG_VIBR_OP_EN_SET_MASK,
	 PMIC_RG_VIBR_OP_EN_SET_SHIFT},
	{PMIC_RG_VIBR_OP_EN_CLR, PMIC_RG_VIBR_OP_EN_CLR_ADDR, PMIC_RG_VIBR_OP_EN_CLR_MASK,
	 PMIC_RG_VIBR_OP_EN_CLR_SHIFT},
	{PMIC_RG_VIBR_HW0_OP_CFG, PMIC_RG_VIBR_HW0_OP_CFG_ADDR, PMIC_RG_VIBR_HW0_OP_CFG_MASK,
	 PMIC_RG_VIBR_HW0_OP_CFG_SHIFT},
	{PMIC_RG_VIBR_HW1_OP_CFG, PMIC_RG_VIBR_HW1_OP_CFG_ADDR, PMIC_RG_VIBR_HW1_OP_CFG_MASK,
	 PMIC_RG_VIBR_HW1_OP_CFG_SHIFT},
	{PMIC_RG_VIBR_HW2_OP_CFG, PMIC_RG_VIBR_HW2_OP_CFG_ADDR, PMIC_RG_VIBR_HW2_OP_CFG_MASK,
	 PMIC_RG_VIBR_HW2_OP_CFG_SHIFT},
	{PMIC_RG_VIBR_GO_ON_OP, PMIC_RG_VIBR_GO_ON_OP_ADDR, PMIC_RG_VIBR_GO_ON_OP_MASK,
	 PMIC_RG_VIBR_GO_ON_OP_SHIFT},
	{PMIC_RG_VIBR_GO_LP_OP, PMIC_RG_VIBR_GO_LP_OP_ADDR, PMIC_RG_VIBR_GO_LP_OP_MASK,
	 PMIC_RG_VIBR_GO_LP_OP_SHIFT},
	{PMIC_RG_VIBR_OP_CFG_SET, PMIC_RG_VIBR_OP_CFG_SET_ADDR, PMIC_RG_VIBR_OP_CFG_SET_MASK,
	 PMIC_RG_VIBR_OP_CFG_SET_SHIFT},
	{PMIC_RG_VIBR_OP_CFG_CLR, PMIC_RG_VIBR_OP_CFG_CLR_ADDR, PMIC_RG_VIBR_OP_CFG_CLR_MASK,
	 PMIC_RG_VIBR_OP_CFG_CLR_SHIFT},
	{PMIC_DA_QI_VIBR_MODE, PMIC_DA_QI_VIBR_MODE_ADDR, PMIC_DA_QI_VIBR_MODE_MASK,
	 PMIC_DA_QI_VIBR_MODE_SHIFT},
	{PMIC_RG_VIBR_STBTD, PMIC_RG_VIBR_STBTD_ADDR, PMIC_RG_VIBR_STBTD_MASK,
	 PMIC_RG_VIBR_STBTD_SHIFT},
	{PMIC_DA_QI_VIBR_STB, PMIC_DA_QI_VIBR_STB_ADDR, PMIC_DA_QI_VIBR_STB_MASK,
	 PMIC_DA_QI_VIBR_STB_SHIFT},
	{PMIC_DA_QI_VIBR_EN, PMIC_DA_QI_VIBR_EN_ADDR, PMIC_DA_QI_VIBR_EN_MASK,
	 PMIC_DA_QI_VIBR_EN_SHIFT},
	{PMIC_RG_VIBR_THER_SDN_EN, PMIC_RG_VIBR_THER_SDN_EN_ADDR, PMIC_RG_VIBR_THER_SDN_EN_MASK,
	 PMIC_RG_VIBR_THER_SDN_EN_SHIFT},
	{PMIC_RG_VIBR_OCFB_EN, PMIC_RG_VIBR_OCFB_EN_ADDR, PMIC_RG_VIBR_OCFB_EN_MASK,
	 PMIC_RG_VIBR_OCFB_EN_SHIFT},
	{PMIC_DA_QI_VIBR_OCFB_EN, PMIC_DA_QI_VIBR_OCFB_EN_ADDR, PMIC_DA_QI_VIBR_OCFB_EN_MASK,
	 PMIC_DA_QI_VIBR_OCFB_EN_SHIFT},
	{PMIC_RG_VBIF28_SW_EN, PMIC_RG_VBIF28_SW_EN_ADDR, PMIC_RG_VBIF28_SW_EN_MASK,
	 PMIC_RG_VBIF28_SW_EN_SHIFT},
	{PMIC_RG_VBIF28_SW_LP, PMIC_RG_VBIF28_SW_LP_ADDR, PMIC_RG_VBIF28_SW_LP_MASK,
	 PMIC_RG_VBIF28_SW_LP_SHIFT},
	{PMIC_RG_VBIF28_SW_OP_EN, PMIC_RG_VBIF28_SW_OP_EN_ADDR, PMIC_RG_VBIF28_SW_OP_EN_MASK,
	 PMIC_RG_VBIF28_SW_OP_EN_SHIFT},
	{PMIC_RG_VBIF28_HW0_OP_EN, PMIC_RG_VBIF28_HW0_OP_EN_ADDR, PMIC_RG_VBIF28_HW0_OP_EN_MASK,
	 PMIC_RG_VBIF28_HW0_OP_EN_SHIFT},
	{PMIC_RG_VBIF28_HW1_OP_EN, PMIC_RG_VBIF28_HW1_OP_EN_ADDR, PMIC_RG_VBIF28_HW1_OP_EN_MASK,
	 PMIC_RG_VBIF28_HW1_OP_EN_SHIFT},
	{PMIC_RG_VBIF28_HW2_OP_EN, PMIC_RG_VBIF28_HW2_OP_EN_ADDR, PMIC_RG_VBIF28_HW2_OP_EN_MASK,
	 PMIC_RG_VBIF28_HW2_OP_EN_SHIFT},
	{PMIC_RG_VBIF28_OP_EN_SET, PMIC_RG_VBIF28_OP_EN_SET_ADDR, PMIC_RG_VBIF28_OP_EN_SET_MASK,
	 PMIC_RG_VBIF28_OP_EN_SET_SHIFT},
	{PMIC_RG_VBIF28_OP_EN_CLR, PMIC_RG_VBIF28_OP_EN_CLR_ADDR, PMIC_RG_VBIF28_OP_EN_CLR_MASK,
	 PMIC_RG_VBIF28_OP_EN_CLR_SHIFT},
	{PMIC_RG_VBIF28_HW0_OP_CFG, PMIC_RG_VBIF28_HW0_OP_CFG_ADDR, PMIC_RG_VBIF28_HW0_OP_CFG_MASK,
	 PMIC_RG_VBIF28_HW0_OP_CFG_SHIFT},
	{PMIC_RG_VBIF28_HW1_OP_CFG, PMIC_RG_VBIF28_HW1_OP_CFG_ADDR, PMIC_RG_VBIF28_HW1_OP_CFG_MASK,
	 PMIC_RG_VBIF28_HW1_OP_CFG_SHIFT},
	{PMIC_RG_VBIF28_HW2_OP_CFG, PMIC_RG_VBIF28_HW2_OP_CFG_ADDR, PMIC_RG_VBIF28_HW2_OP_CFG_MASK,
	 PMIC_RG_VBIF28_HW2_OP_CFG_SHIFT},
	{PMIC_RG_VBIF28_GO_ON_OP, PMIC_RG_VBIF28_GO_ON_OP_ADDR, PMIC_RG_VBIF28_GO_ON_OP_MASK,
	 PMIC_RG_VBIF28_GO_ON_OP_SHIFT},
	{PMIC_RG_VBIF28_GO_LP_OP, PMIC_RG_VBIF28_GO_LP_OP_ADDR, PMIC_RG_VBIF28_GO_LP_OP_MASK,
	 PMIC_RG_VBIF28_GO_LP_OP_SHIFT},
	{PMIC_RG_VBIF28_OP_CFG_SET, PMIC_RG_VBIF28_OP_CFG_SET_ADDR, PMIC_RG_VBIF28_OP_CFG_SET_MASK,
	 PMIC_RG_VBIF28_OP_CFG_SET_SHIFT},
	{PMIC_RG_VBIF28_OP_CFG_CLR, PMIC_RG_VBIF28_OP_CFG_CLR_ADDR, PMIC_RG_VBIF28_OP_CFG_CLR_MASK,
	 PMIC_RG_VBIF28_OP_CFG_CLR_SHIFT},
	{PMIC_DA_QI_VBIF28_MODE, PMIC_DA_QI_VBIF28_MODE_ADDR, PMIC_DA_QI_VBIF28_MODE_MASK,
	 PMIC_DA_QI_VBIF28_MODE_SHIFT},
	{PMIC_RG_VBIF28_STBTD, PMIC_RG_VBIF28_STBTD_ADDR, PMIC_RG_VBIF28_STBTD_MASK,
	 PMIC_RG_VBIF28_STBTD_SHIFT},
	{PMIC_DA_QI_VBIF28_STB, PMIC_DA_QI_VBIF28_STB_ADDR, PMIC_DA_QI_VBIF28_STB_MASK,
	 PMIC_DA_QI_VBIF28_STB_SHIFT},
	{PMIC_DA_QI_VBIF28_EN, PMIC_DA_QI_VBIF28_EN_ADDR, PMIC_DA_QI_VBIF28_EN_MASK,
	 PMIC_DA_QI_VBIF28_EN_SHIFT},
	{PMIC_RG_VBIF28_OCFB_EN, PMIC_RG_VBIF28_OCFB_EN_ADDR, PMIC_RG_VBIF28_OCFB_EN_MASK,
	 PMIC_RG_VBIF28_OCFB_EN_SHIFT},
	{PMIC_DA_QI_VBIF28_OCFB_EN, PMIC_DA_QI_VBIF28_OCFB_EN_ADDR, PMIC_DA_QI_VBIF28_OCFB_EN_MASK,
	 PMIC_DA_QI_VBIF28_OCFB_EN_SHIFT},
	{PMIC_RG_VFE28_SW_EN, PMIC_RG_VFE28_SW_EN_ADDR, PMIC_RG_VFE28_SW_EN_MASK,
	 PMIC_RG_VFE28_SW_EN_SHIFT},
	{PMIC_RG_VFE28_SW_LP, PMIC_RG_VFE28_SW_LP_ADDR, PMIC_RG_VFE28_SW_LP_MASK,
	 PMIC_RG_VFE28_SW_LP_SHIFT},
	{PMIC_RG_VFE28_SW_OP_EN, PMIC_RG_VFE28_SW_OP_EN_ADDR, PMIC_RG_VFE28_SW_OP_EN_MASK,
	 PMIC_RG_VFE28_SW_OP_EN_SHIFT},
	{PMIC_RG_VFE28_HW0_OP_EN, PMIC_RG_VFE28_HW0_OP_EN_ADDR, PMIC_RG_VFE28_HW0_OP_EN_MASK,
	 PMIC_RG_VFE28_HW0_OP_EN_SHIFT},
	{PMIC_RG_VFE28_HW1_OP_EN, PMIC_RG_VFE28_HW1_OP_EN_ADDR, PMIC_RG_VFE28_HW1_OP_EN_MASK,
	 PMIC_RG_VFE28_HW1_OP_EN_SHIFT},
	{PMIC_RG_VFE28_HW2_OP_EN, PMIC_RG_VFE28_HW2_OP_EN_ADDR, PMIC_RG_VFE28_HW2_OP_EN_MASK,
	 PMIC_RG_VFE28_HW2_OP_EN_SHIFT},
	{PMIC_RG_VFE28_OP_EN_SET, PMIC_RG_VFE28_OP_EN_SET_ADDR, PMIC_RG_VFE28_OP_EN_SET_MASK,
	 PMIC_RG_VFE28_OP_EN_SET_SHIFT},
	{PMIC_RG_VFE28_OP_EN_CLR, PMIC_RG_VFE28_OP_EN_CLR_ADDR, PMIC_RG_VFE28_OP_EN_CLR_MASK,
	 PMIC_RG_VFE28_OP_EN_CLR_SHIFT},
	{PMIC_RG_VFE28_HW0_OP_CFG, PMIC_RG_VFE28_HW0_OP_CFG_ADDR, PMIC_RG_VFE28_HW0_OP_CFG_MASK,
	 PMIC_RG_VFE28_HW0_OP_CFG_SHIFT},
	{PMIC_RG_VFE28_HW1_OP_CFG, PMIC_RG_VFE28_HW1_OP_CFG_ADDR, PMIC_RG_VFE28_HW1_OP_CFG_MASK,
	 PMIC_RG_VFE28_HW1_OP_CFG_SHIFT},
	{PMIC_RG_VFE28_HW2_OP_CFG, PMIC_RG_VFE28_HW2_OP_CFG_ADDR, PMIC_RG_VFE28_HW2_OP_CFG_MASK,
	 PMIC_RG_VFE28_HW2_OP_CFG_SHIFT},
	{PMIC_RG_VFE28_GO_ON_OP, PMIC_RG_VFE28_GO_ON_OP_ADDR, PMIC_RG_VFE28_GO_ON_OP_MASK,
	 PMIC_RG_VFE28_GO_ON_OP_SHIFT},
	{PMIC_RG_VFE28_GO_LP_OP, PMIC_RG_VFE28_GO_LP_OP_ADDR, PMIC_RG_VFE28_GO_LP_OP_MASK,
	 PMIC_RG_VFE28_GO_LP_OP_SHIFT},
	{PMIC_RG_VFE28_OP_CFG_SET, PMIC_RG_VFE28_OP_CFG_SET_ADDR, PMIC_RG_VFE28_OP_CFG_SET_MASK,
	 PMIC_RG_VFE28_OP_CFG_SET_SHIFT},
	{PMIC_RG_VFE28_OP_CFG_CLR, PMIC_RG_VFE28_OP_CFG_CLR_ADDR, PMIC_RG_VFE28_OP_CFG_CLR_MASK,
	 PMIC_RG_VFE28_OP_CFG_CLR_SHIFT},
	{PMIC_DA_QI_VFE28_MODE, PMIC_DA_QI_VFE28_MODE_ADDR, PMIC_DA_QI_VFE28_MODE_MASK,
	 PMIC_DA_QI_VFE28_MODE_SHIFT},
	{PMIC_RG_VFE28_STBTD, PMIC_RG_VFE28_STBTD_ADDR, PMIC_RG_VFE28_STBTD_MASK,
	 PMIC_RG_VFE28_STBTD_SHIFT},
	{PMIC_DA_QI_VFE28_STB, PMIC_DA_QI_VFE28_STB_ADDR, PMIC_DA_QI_VFE28_STB_MASK,
	 PMIC_DA_QI_VFE28_STB_SHIFT},
	{PMIC_DA_QI_VFE28_EN, PMIC_DA_QI_VFE28_EN_ADDR, PMIC_DA_QI_VFE28_EN_MASK,
	 PMIC_DA_QI_VFE28_EN_SHIFT},
	{PMIC_RG_VFE28_OCFB_EN, PMIC_RG_VFE28_OCFB_EN_ADDR, PMIC_RG_VFE28_OCFB_EN_MASK,
	 PMIC_RG_VFE28_OCFB_EN_SHIFT},
	{PMIC_DA_QI_VFE28_OCFB_EN, PMIC_DA_QI_VFE28_OCFB_EN_ADDR, PMIC_DA_QI_VFE28_OCFB_EN_MASK,
	 PMIC_DA_QI_VFE28_OCFB_EN_SHIFT},
	{PMIC_RG_VMCH_SW_EN, PMIC_RG_VMCH_SW_EN_ADDR, PMIC_RG_VMCH_SW_EN_MASK,
	 PMIC_RG_VMCH_SW_EN_SHIFT},
	{PMIC_RG_VMCH_SW_LP, PMIC_RG_VMCH_SW_LP_ADDR, PMIC_RG_VMCH_SW_LP_MASK,
	 PMIC_RG_VMCH_SW_LP_SHIFT},
	{PMIC_RG_VMCH_SW_OP_EN, PMIC_RG_VMCH_SW_OP_EN_ADDR, PMIC_RG_VMCH_SW_OP_EN_MASK,
	 PMIC_RG_VMCH_SW_OP_EN_SHIFT},
	{PMIC_RG_VMCH_HW0_OP_EN, PMIC_RG_VMCH_HW0_OP_EN_ADDR, PMIC_RG_VMCH_HW0_OP_EN_MASK,
	 PMIC_RG_VMCH_HW0_OP_EN_SHIFT},
	{PMIC_RG_VMCH_HW1_OP_EN, PMIC_RG_VMCH_HW1_OP_EN_ADDR, PMIC_RG_VMCH_HW1_OP_EN_MASK,
	 PMIC_RG_VMCH_HW1_OP_EN_SHIFT},
	{PMIC_RG_VMCH_HW2_OP_EN, PMIC_RG_VMCH_HW2_OP_EN_ADDR, PMIC_RG_VMCH_HW2_OP_EN_MASK,
	 PMIC_RG_VMCH_HW2_OP_EN_SHIFT},
	{PMIC_RG_VMCH_OP_EN_SET, PMIC_RG_VMCH_OP_EN_SET_ADDR, PMIC_RG_VMCH_OP_EN_SET_MASK,
	 PMIC_RG_VMCH_OP_EN_SET_SHIFT},
	{PMIC_RG_VMCH_OP_EN_CLR, PMIC_RG_VMCH_OP_EN_CLR_ADDR, PMIC_RG_VMCH_OP_EN_CLR_MASK,
	 PMIC_RG_VMCH_OP_EN_CLR_SHIFT},
	{PMIC_RG_VMCH_HW0_OP_CFG, PMIC_RG_VMCH_HW0_OP_CFG_ADDR, PMIC_RG_VMCH_HW0_OP_CFG_MASK,
	 PMIC_RG_VMCH_HW0_OP_CFG_SHIFT},
	{PMIC_RG_VMCH_HW1_OP_CFG, PMIC_RG_VMCH_HW1_OP_CFG_ADDR, PMIC_RG_VMCH_HW1_OP_CFG_MASK,
	 PMIC_RG_VMCH_HW1_OP_CFG_SHIFT},
	{PMIC_RG_VMCH_HW2_OP_CFG, PMIC_RG_VMCH_HW2_OP_CFG_ADDR, PMIC_RG_VMCH_HW2_OP_CFG_MASK,
	 PMIC_RG_VMCH_HW2_OP_CFG_SHIFT},
	{PMIC_RG_VMCH_GO_ON_OP, PMIC_RG_VMCH_GO_ON_OP_ADDR, PMIC_RG_VMCH_GO_ON_OP_MASK,
	 PMIC_RG_VMCH_GO_ON_OP_SHIFT},
	{PMIC_RG_VMCH_GO_LP_OP, PMIC_RG_VMCH_GO_LP_OP_ADDR, PMIC_RG_VMCH_GO_LP_OP_MASK,
	 PMIC_RG_VMCH_GO_LP_OP_SHIFT},
	{PMIC_RG_VMCH_OP_CFG_SET, PMIC_RG_VMCH_OP_CFG_SET_ADDR, PMIC_RG_VMCH_OP_CFG_SET_MASK,
	 PMIC_RG_VMCH_OP_CFG_SET_SHIFT},
	{PMIC_RG_VMCH_OP_CFG_CLR, PMIC_RG_VMCH_OP_CFG_CLR_ADDR, PMIC_RG_VMCH_OP_CFG_CLR_MASK,
	 PMIC_RG_VMCH_OP_CFG_CLR_SHIFT},
	{PMIC_DA_QI_VMCH_MODE, PMIC_DA_QI_VMCH_MODE_ADDR, PMIC_DA_QI_VMCH_MODE_MASK,
	 PMIC_DA_QI_VMCH_MODE_SHIFT},
	{PMIC_RG_VMCH_STBTD, PMIC_RG_VMCH_STBTD_ADDR, PMIC_RG_VMCH_STBTD_MASK,
	 PMIC_RG_VMCH_STBTD_SHIFT},
	{PMIC_DA_QI_VMCH_STB, PMIC_DA_QI_VMCH_STB_ADDR, PMIC_DA_QI_VMCH_STB_MASK,
	 PMIC_DA_QI_VMCH_STB_SHIFT},
	{PMIC_DA_QI_VMCH_EN, PMIC_DA_QI_VMCH_EN_ADDR, PMIC_DA_QI_VMCH_EN_MASK,
	 PMIC_DA_QI_VMCH_EN_SHIFT},
	{PMIC_RG_VMCH_OCFB_EN, PMIC_RG_VMCH_OCFB_EN_ADDR, PMIC_RG_VMCH_OCFB_EN_MASK,
	 PMIC_RG_VMCH_OCFB_EN_SHIFT},
	{PMIC_DA_QI_VMCH_OCFB_EN, PMIC_DA_QI_VMCH_OCFB_EN_ADDR, PMIC_DA_QI_VMCH_OCFB_EN_MASK,
	 PMIC_DA_QI_VMCH_OCFB_EN_SHIFT},
	{PMIC_RG_VMCH_DUMMY_LOAD, PMIC_RG_VMCH_DUMMY_LOAD_ADDR, PMIC_RG_VMCH_DUMMY_LOAD_MASK,
	 PMIC_RG_VMCH_DUMMY_LOAD_SHIFT},
	{PMIC_DA_QI_VMCH_DUMMY_LOAD, PMIC_DA_QI_VMCH_DUMMY_LOAD_ADDR,
	 PMIC_DA_QI_VMCH_DUMMY_LOAD_MASK, PMIC_DA_QI_VMCH_DUMMY_LOAD_SHIFT},
	{PMIC_RG_VMC_SW_EN, PMIC_RG_VMC_SW_EN_ADDR, PMIC_RG_VMC_SW_EN_MASK,
	 PMIC_RG_VMC_SW_EN_SHIFT},
	{PMIC_RG_VMC_SW_LP, PMIC_RG_VMC_SW_LP_ADDR, PMIC_RG_VMC_SW_LP_MASK,
	 PMIC_RG_VMC_SW_LP_SHIFT},
	{PMIC_RG_VMC_SW_OP_EN, PMIC_RG_VMC_SW_OP_EN_ADDR, PMIC_RG_VMC_SW_OP_EN_MASK,
	 PMIC_RG_VMC_SW_OP_EN_SHIFT},
	{PMIC_RG_VMC_HW0_OP_EN, PMIC_RG_VMC_HW0_OP_EN_ADDR, PMIC_RG_VMC_HW0_OP_EN_MASK,
	 PMIC_RG_VMC_HW0_OP_EN_SHIFT},
	{PMIC_RG_VMC_HW1_OP_EN, PMIC_RG_VMC_HW1_OP_EN_ADDR, PMIC_RG_VMC_HW1_OP_EN_MASK,
	 PMIC_RG_VMC_HW1_OP_EN_SHIFT},
	{PMIC_RG_VMC_HW2_OP_EN, PMIC_RG_VMC_HW2_OP_EN_ADDR, PMIC_RG_VMC_HW2_OP_EN_MASK,
	 PMIC_RG_VMC_HW2_OP_EN_SHIFT},
	{PMIC_RG_VMC_OP_EN_SET, PMIC_RG_VMC_OP_EN_SET_ADDR, PMIC_RG_VMC_OP_EN_SET_MASK,
	 PMIC_RG_VMC_OP_EN_SET_SHIFT},
	{PMIC_RG_VMC_OP_EN_CLR, PMIC_RG_VMC_OP_EN_CLR_ADDR, PMIC_RG_VMC_OP_EN_CLR_MASK,
	 PMIC_RG_VMC_OP_EN_CLR_SHIFT},
	{PMIC_RG_VMC_HW0_OP_CFG, PMIC_RG_VMC_HW0_OP_CFG_ADDR, PMIC_RG_VMC_HW0_OP_CFG_MASK,
	 PMIC_RG_VMC_HW0_OP_CFG_SHIFT},
	{PMIC_RG_VMC_HW1_OP_CFG, PMIC_RG_VMC_HW1_OP_CFG_ADDR, PMIC_RG_VMC_HW1_OP_CFG_MASK,
	 PMIC_RG_VMC_HW1_OP_CFG_SHIFT},
	{PMIC_RG_VMC_HW2_OP_CFG, PMIC_RG_VMC_HW2_OP_CFG_ADDR, PMIC_RG_VMC_HW2_OP_CFG_MASK,
	 PMIC_RG_VMC_HW2_OP_CFG_SHIFT},
	{PMIC_RG_VMC_GO_ON_OP, PMIC_RG_VMC_GO_ON_OP_ADDR, PMIC_RG_VMC_GO_ON_OP_MASK,
	 PMIC_RG_VMC_GO_ON_OP_SHIFT},
	{PMIC_RG_VMC_GO_LP_OP, PMIC_RG_VMC_GO_LP_OP_ADDR, PMIC_RG_VMC_GO_LP_OP_MASK,
	 PMIC_RG_VMC_GO_LP_OP_SHIFT},
	{PMIC_RG_VMC_OP_CFG_SET, PMIC_RG_VMC_OP_CFG_SET_ADDR, PMIC_RG_VMC_OP_CFG_SET_MASK,
	 PMIC_RG_VMC_OP_CFG_SET_SHIFT},
	{PMIC_RG_VMC_OP_CFG_CLR, PMIC_RG_VMC_OP_CFG_CLR_ADDR, PMIC_RG_VMC_OP_CFG_CLR_MASK,
	 PMIC_RG_VMC_OP_CFG_CLR_SHIFT},
	{PMIC_DA_QI_VMC_MODE, PMIC_DA_QI_VMC_MODE_ADDR, PMIC_DA_QI_VMC_MODE_MASK,
	 PMIC_DA_QI_VMC_MODE_SHIFT},
	{PMIC_RG_VMC_STBTD, PMIC_RG_VMC_STBTD_ADDR, PMIC_RG_VMC_STBTD_MASK,
	 PMIC_RG_VMC_STBTD_SHIFT},
	{PMIC_DA_QI_VMC_STB, PMIC_DA_QI_VMC_STB_ADDR, PMIC_DA_QI_VMC_STB_MASK,
	 PMIC_DA_QI_VMC_STB_SHIFT},
	{PMIC_DA_QI_VMC_EN, PMIC_DA_QI_VMC_EN_ADDR, PMIC_DA_QI_VMC_EN_MASK,
	 PMIC_DA_QI_VMC_EN_SHIFT},
	{PMIC_RG_VMC_OCFB_EN, PMIC_RG_VMC_OCFB_EN_ADDR, PMIC_RG_VMC_OCFB_EN_MASK,
	 PMIC_RG_VMC_OCFB_EN_SHIFT},
	{PMIC_DA_QI_VMC_OCFB_EN, PMIC_DA_QI_VMC_OCFB_EN_ADDR, PMIC_DA_QI_VMC_OCFB_EN_MASK,
	 PMIC_DA_QI_VMC_OCFB_EN_SHIFT},
	{PMIC_RG_VMC_DUMMY_LOAD, PMIC_RG_VMC_DUMMY_LOAD_ADDR, PMIC_RG_VMC_DUMMY_LOAD_MASK,
	 PMIC_RG_VMC_DUMMY_LOAD_SHIFT},
	{PMIC_DA_QI_VMC_DUMMY_LOAD, PMIC_DA_QI_VMC_DUMMY_LOAD_ADDR, PMIC_DA_QI_VMC_DUMMY_LOAD_MASK,
	 PMIC_DA_QI_VMC_DUMMY_LOAD_SHIFT},
	{PMIC_RG_VRF18_1_SW_EN, PMIC_RG_VRF18_1_SW_EN_ADDR, PMIC_RG_VRF18_1_SW_EN_MASK,
	 PMIC_RG_VRF18_1_SW_EN_SHIFT},
	{PMIC_RG_VRF18_1_SW_LP, PMIC_RG_VRF18_1_SW_LP_ADDR, PMIC_RG_VRF18_1_SW_LP_MASK,
	 PMIC_RG_VRF18_1_SW_LP_SHIFT},
	{PMIC_RG_VRF18_1_SW_OP_EN, PMIC_RG_VRF18_1_SW_OP_EN_ADDR, PMIC_RG_VRF18_1_SW_OP_EN_MASK,
	 PMIC_RG_VRF18_1_SW_OP_EN_SHIFT},
	{PMIC_RG_VRF18_1_HW0_OP_EN, PMIC_RG_VRF18_1_HW0_OP_EN_ADDR, PMIC_RG_VRF18_1_HW0_OP_EN_MASK,
	 PMIC_RG_VRF18_1_HW0_OP_EN_SHIFT},
	{PMIC_RG_VRF18_1_HW1_OP_EN, PMIC_RG_VRF18_1_HW1_OP_EN_ADDR, PMIC_RG_VRF18_1_HW1_OP_EN_MASK,
	 PMIC_RG_VRF18_1_HW1_OP_EN_SHIFT},
	{PMIC_RG_VRF18_1_HW2_OP_EN, PMIC_RG_VRF18_1_HW2_OP_EN_ADDR, PMIC_RG_VRF18_1_HW2_OP_EN_MASK,
	 PMIC_RG_VRF18_1_HW2_OP_EN_SHIFT},
	{PMIC_RG_VRF18_1_OP_EN_SET, PMIC_RG_VRF18_1_OP_EN_SET_ADDR, PMIC_RG_VRF18_1_OP_EN_SET_MASK,
	 PMIC_RG_VRF18_1_OP_EN_SET_SHIFT},
	{PMIC_RG_VRF18_1_OP_EN_CLR, PMIC_RG_VRF18_1_OP_EN_CLR_ADDR, PMIC_RG_VRF18_1_OP_EN_CLR_MASK,
	 PMIC_RG_VRF18_1_OP_EN_CLR_SHIFT},
	{PMIC_RG_VRF18_1_HW0_OP_CFG, PMIC_RG_VRF18_1_HW0_OP_CFG_ADDR,
	 PMIC_RG_VRF18_1_HW0_OP_CFG_MASK, PMIC_RG_VRF18_1_HW0_OP_CFG_SHIFT},
	{PMIC_RG_VRF18_1_HW1_OP_CFG, PMIC_RG_VRF18_1_HW1_OP_CFG_ADDR,
	 PMIC_RG_VRF18_1_HW1_OP_CFG_MASK, PMIC_RG_VRF18_1_HW1_OP_CFG_SHIFT},
	{PMIC_RG_VRF18_1_HW2_OP_CFG, PMIC_RG_VRF18_1_HW2_OP_CFG_ADDR,
	 PMIC_RG_VRF18_1_HW2_OP_CFG_MASK, PMIC_RG_VRF18_1_HW2_OP_CFG_SHIFT},
	{PMIC_RG_VRF18_1_GO_ON_OP, PMIC_RG_VRF18_1_GO_ON_OP_ADDR, PMIC_RG_VRF18_1_GO_ON_OP_MASK,
	 PMIC_RG_VRF18_1_GO_ON_OP_SHIFT},
	{PMIC_RG_VRF18_1_GO_LP_OP, PMIC_RG_VRF18_1_GO_LP_OP_ADDR, PMIC_RG_VRF18_1_GO_LP_OP_MASK,
	 PMIC_RG_VRF18_1_GO_LP_OP_SHIFT},
	{PMIC_RG_VRF18_1_OP_CFG_SET, PMIC_RG_VRF18_1_OP_CFG_SET_ADDR,
	 PMIC_RG_VRF18_1_OP_CFG_SET_MASK, PMIC_RG_VRF18_1_OP_CFG_SET_SHIFT},
	{PMIC_RG_VRF18_1_OP_CFG_CLR, PMIC_RG_VRF18_1_OP_CFG_CLR_ADDR,
	 PMIC_RG_VRF18_1_OP_CFG_CLR_MASK, PMIC_RG_VRF18_1_OP_CFG_CLR_SHIFT},
	{PMIC_DA_QI_VRF18_1_MODE, PMIC_DA_QI_VRF18_1_MODE_ADDR, PMIC_DA_QI_VRF18_1_MODE_MASK,
	 PMIC_DA_QI_VRF18_1_MODE_SHIFT},
	{PMIC_RG_VRF18_1_STBTD, PMIC_RG_VRF18_1_STBTD_ADDR, PMIC_RG_VRF18_1_STBTD_MASK,
	 PMIC_RG_VRF18_1_STBTD_SHIFT},
	{PMIC_DA_QI_VRF18_1_STB, PMIC_DA_QI_VRF18_1_STB_ADDR, PMIC_DA_QI_VRF18_1_STB_MASK,
	 PMIC_DA_QI_VRF18_1_STB_SHIFT},
	{PMIC_DA_QI_VRF18_1_EN, PMIC_DA_QI_VRF18_1_EN_ADDR, PMIC_DA_QI_VRF18_1_EN_MASK,
	 PMIC_DA_QI_VRF18_1_EN_SHIFT},
	{PMIC_RG_VRF18_1_OCFB_EN, PMIC_RG_VRF18_1_OCFB_EN_ADDR, PMIC_RG_VRF18_1_OCFB_EN_MASK,
	 PMIC_RG_VRF18_1_OCFB_EN_SHIFT},
	{PMIC_DA_QI_VRF18_1_OCFB_EN, PMIC_DA_QI_VRF18_1_OCFB_EN_ADDR,
	 PMIC_DA_QI_VRF18_1_OCFB_EN_MASK, PMIC_DA_QI_VRF18_1_OCFB_EN_SHIFT},
	{PMIC_RG_VRF18_1_DUMMY_LOAD, PMIC_RG_VRF18_1_DUMMY_LOAD_ADDR,
	 PMIC_RG_VRF18_1_DUMMY_LOAD_MASK, PMIC_RG_VRF18_1_DUMMY_LOAD_SHIFT},
	{PMIC_DA_QI_VRF18_1_DUMMY_LOAD, PMIC_DA_QI_VRF18_1_DUMMY_LOAD_ADDR,
	 PMIC_DA_QI_VRF18_1_DUMMY_LOAD_MASK, PMIC_DA_QI_VRF18_1_DUMMY_LOAD_SHIFT},
	{PMIC_RG_VRF18_2_SW_EN, PMIC_RG_VRF18_2_SW_EN_ADDR, PMIC_RG_VRF18_2_SW_EN_MASK,
	 PMIC_RG_VRF18_2_SW_EN_SHIFT},
	{PMIC_RG_VRF18_2_SW_LP, PMIC_RG_VRF18_2_SW_LP_ADDR, PMIC_RG_VRF18_2_SW_LP_MASK,
	 PMIC_RG_VRF18_2_SW_LP_SHIFT},
	{PMIC_RG_VRF18_2_SW_OP_EN, PMIC_RG_VRF18_2_SW_OP_EN_ADDR, PMIC_RG_VRF18_2_SW_OP_EN_MASK,
	 PMIC_RG_VRF18_2_SW_OP_EN_SHIFT},
	{PMIC_RG_VRF18_2_HW0_OP_EN, PMIC_RG_VRF18_2_HW0_OP_EN_ADDR, PMIC_RG_VRF18_2_HW0_OP_EN_MASK,
	 PMIC_RG_VRF18_2_HW0_OP_EN_SHIFT},
	{PMIC_RG_VRF18_2_HW1_OP_EN, PMIC_RG_VRF18_2_HW1_OP_EN_ADDR, PMIC_RG_VRF18_2_HW1_OP_EN_MASK,
	 PMIC_RG_VRF18_2_HW1_OP_EN_SHIFT},
	{PMIC_RG_VRF18_2_HW2_OP_EN, PMIC_RG_VRF18_2_HW2_OP_EN_ADDR, PMIC_RG_VRF18_2_HW2_OP_EN_MASK,
	 PMIC_RG_VRF18_2_HW2_OP_EN_SHIFT},
	{PMIC_RG_VRF18_2_OP_EN_SET, PMIC_RG_VRF18_2_OP_EN_SET_ADDR, PMIC_RG_VRF18_2_OP_EN_SET_MASK,
	 PMIC_RG_VRF18_2_OP_EN_SET_SHIFT},
	{PMIC_RG_VRF18_2_OP_EN_CLR, PMIC_RG_VRF18_2_OP_EN_CLR_ADDR, PMIC_RG_VRF18_2_OP_EN_CLR_MASK,
	 PMIC_RG_VRF18_2_OP_EN_CLR_SHIFT},
	{PMIC_RG_VRF18_2_HW0_OP_CFG, PMIC_RG_VRF18_2_HW0_OP_CFG_ADDR,
	 PMIC_RG_VRF18_2_HW0_OP_CFG_MASK, PMIC_RG_VRF18_2_HW0_OP_CFG_SHIFT},
	{PMIC_RG_VRF18_2_HW1_OP_CFG, PMIC_RG_VRF18_2_HW1_OP_CFG_ADDR,
	 PMIC_RG_VRF18_2_HW1_OP_CFG_MASK, PMIC_RG_VRF18_2_HW1_OP_CFG_SHIFT},
	{PMIC_RG_VRF18_2_HW2_OP_CFG, PMIC_RG_VRF18_2_HW2_OP_CFG_ADDR,
	 PMIC_RG_VRF18_2_HW2_OP_CFG_MASK, PMIC_RG_VRF18_2_HW2_OP_CFG_SHIFT},
	{PMIC_RG_VRF18_2_GO_ON_OP, PMIC_RG_VRF18_2_GO_ON_OP_ADDR, PMIC_RG_VRF18_2_GO_ON_OP_MASK,
	 PMIC_RG_VRF18_2_GO_ON_OP_SHIFT},
	{PMIC_RG_VRF18_2_GO_LP_OP, PMIC_RG_VRF18_2_GO_LP_OP_ADDR, PMIC_RG_VRF18_2_GO_LP_OP_MASK,
	 PMIC_RG_VRF18_2_GO_LP_OP_SHIFT},
	{PMIC_RG_VRF18_2_OP_CFG_SET, PMIC_RG_VRF18_2_OP_CFG_SET_ADDR,
	 PMIC_RG_VRF18_2_OP_CFG_SET_MASK, PMIC_RG_VRF18_2_OP_CFG_SET_SHIFT},
	{PMIC_RG_VRF18_2_OP_CFG_CLR, PMIC_RG_VRF18_2_OP_CFG_CLR_ADDR,
	 PMIC_RG_VRF18_2_OP_CFG_CLR_MASK, PMIC_RG_VRF18_2_OP_CFG_CLR_SHIFT},
	{PMIC_DA_QI_VRF18_2_MODE, PMIC_DA_QI_VRF18_2_MODE_ADDR, PMIC_DA_QI_VRF18_2_MODE_MASK,
	 PMIC_DA_QI_VRF18_2_MODE_SHIFT},
	{PMIC_RG_VRF18_2_STBTD, PMIC_RG_VRF18_2_STBTD_ADDR, PMIC_RG_VRF18_2_STBTD_MASK,
	 PMIC_RG_VRF18_2_STBTD_SHIFT},
	{PMIC_DA_QI_VRF18_2_STB, PMIC_DA_QI_VRF18_2_STB_ADDR, PMIC_DA_QI_VRF18_2_STB_MASK,
	 PMIC_DA_QI_VRF18_2_STB_SHIFT},
	{PMIC_DA_QI_VRF18_2_EN, PMIC_DA_QI_VRF18_2_EN_ADDR, PMIC_DA_QI_VRF18_2_EN_MASK,
	 PMIC_DA_QI_VRF18_2_EN_SHIFT},
	{PMIC_RG_VRF18_2_OCFB_EN, PMIC_RG_VRF18_2_OCFB_EN_ADDR, PMIC_RG_VRF18_2_OCFB_EN_MASK,
	 PMIC_RG_VRF18_2_OCFB_EN_SHIFT},
	{PMIC_DA_QI_VRF18_2_OCFB_EN, PMIC_DA_QI_VRF18_2_OCFB_EN_ADDR,
	 PMIC_DA_QI_VRF18_2_OCFB_EN_MASK, PMIC_DA_QI_VRF18_2_OCFB_EN_SHIFT},
	{PMIC_RG_VRF18_2_DUMMY_LOAD, PMIC_RG_VRF18_2_DUMMY_LOAD_ADDR,
	 PMIC_RG_VRF18_2_DUMMY_LOAD_MASK, PMIC_RG_VRF18_2_DUMMY_LOAD_SHIFT},
	{PMIC_DA_QI_VRF18_2_DUMMY_LOAD, PMIC_DA_QI_VRF18_2_DUMMY_LOAD_ADDR,
	 PMIC_DA_QI_VRF18_2_DUMMY_LOAD_MASK, PMIC_DA_QI_VRF18_2_DUMMY_LOAD_SHIFT},
	{PMIC_RG_VRF12_SW_EN, PMIC_RG_VRF12_SW_EN_ADDR, PMIC_RG_VRF12_SW_EN_MASK,
	 PMIC_RG_VRF12_SW_EN_SHIFT},
	{PMIC_RG_VRF12_SW_LP, PMIC_RG_VRF12_SW_LP_ADDR, PMIC_RG_VRF12_SW_LP_MASK,
	 PMIC_RG_VRF12_SW_LP_SHIFT},
	{PMIC_RG_VRF12_SW_OP_EN, PMIC_RG_VRF12_SW_OP_EN_ADDR, PMIC_RG_VRF12_SW_OP_EN_MASK,
	 PMIC_RG_VRF12_SW_OP_EN_SHIFT},
	{PMIC_RG_VRF12_HW0_OP_EN, PMIC_RG_VRF12_HW0_OP_EN_ADDR, PMIC_RG_VRF12_HW0_OP_EN_MASK,
	 PMIC_RG_VRF12_HW0_OP_EN_SHIFT},
	{PMIC_RG_VRF12_HW1_OP_EN, PMIC_RG_VRF12_HW1_OP_EN_ADDR, PMIC_RG_VRF12_HW1_OP_EN_MASK,
	 PMIC_RG_VRF12_HW1_OP_EN_SHIFT},
	{PMIC_RG_VRF12_HW2_OP_EN, PMIC_RG_VRF12_HW2_OP_EN_ADDR, PMIC_RG_VRF12_HW2_OP_EN_MASK,
	 PMIC_RG_VRF12_HW2_OP_EN_SHIFT},
	{PMIC_RG_VRF12_OP_EN_SET, PMIC_RG_VRF12_OP_EN_SET_ADDR, PMIC_RG_VRF12_OP_EN_SET_MASK,
	 PMIC_RG_VRF12_OP_EN_SET_SHIFT},
	{PMIC_RG_VRF12_OP_EN_CLR, PMIC_RG_VRF12_OP_EN_CLR_ADDR, PMIC_RG_VRF12_OP_EN_CLR_MASK,
	 PMIC_RG_VRF12_OP_EN_CLR_SHIFT},
	{PMIC_RG_VRF12_HW0_OP_CFG, PMIC_RG_VRF12_HW0_OP_CFG_ADDR, PMIC_RG_VRF12_HW0_OP_CFG_MASK,
	 PMIC_RG_VRF12_HW0_OP_CFG_SHIFT},
	{PMIC_RG_VRF12_HW1_OP_CFG, PMIC_RG_VRF12_HW1_OP_CFG_ADDR, PMIC_RG_VRF12_HW1_OP_CFG_MASK,
	 PMIC_RG_VRF12_HW1_OP_CFG_SHIFT},
	{PMIC_RG_VRF12_HW2_OP_CFG, PMIC_RG_VRF12_HW2_OP_CFG_ADDR, PMIC_RG_VRF12_HW2_OP_CFG_MASK,
	 PMIC_RG_VRF12_HW2_OP_CFG_SHIFT},
	{PMIC_RG_VRF12_GO_ON_OP, PMIC_RG_VRF12_GO_ON_OP_ADDR, PMIC_RG_VRF12_GO_ON_OP_MASK,
	 PMIC_RG_VRF12_GO_ON_OP_SHIFT},
	{PMIC_RG_VRF12_GO_LP_OP, PMIC_RG_VRF12_GO_LP_OP_ADDR, PMIC_RG_VRF12_GO_LP_OP_MASK,
	 PMIC_RG_VRF12_GO_LP_OP_SHIFT},
	{PMIC_RG_VRF12_OP_CFG_SET, PMIC_RG_VRF12_OP_CFG_SET_ADDR, PMIC_RG_VRF12_OP_CFG_SET_MASK,
	 PMIC_RG_VRF12_OP_CFG_SET_SHIFT},
	{PMIC_RG_VRF12_OP_CFG_CLR, PMIC_RG_VRF12_OP_CFG_CLR_ADDR, PMIC_RG_VRF12_OP_CFG_CLR_MASK,
	 PMIC_RG_VRF12_OP_CFG_CLR_SHIFT},
	{PMIC_DA_QI_VRF12_MODE, PMIC_DA_QI_VRF12_MODE_ADDR, PMIC_DA_QI_VRF12_MODE_MASK,
	 PMIC_DA_QI_VRF12_MODE_SHIFT},
	{PMIC_RG_VRF12_STBTD, PMIC_RG_VRF12_STBTD_ADDR, PMIC_RG_VRF12_STBTD_MASK,
	 PMIC_RG_VRF12_STBTD_SHIFT},
	{PMIC_DA_QI_VRF12_STB, PMIC_DA_QI_VRF12_STB_ADDR, PMIC_DA_QI_VRF12_STB_MASK,
	 PMIC_DA_QI_VRF12_STB_SHIFT},
	{PMIC_DA_QI_VRF12_EN, PMIC_DA_QI_VRF12_EN_ADDR, PMIC_DA_QI_VRF12_EN_MASK,
	 PMIC_DA_QI_VRF12_EN_SHIFT},
	{PMIC_RG_VRF12_OCFB_EN, PMIC_RG_VRF12_OCFB_EN_ADDR, PMIC_RG_VRF12_OCFB_EN_MASK,
	 PMIC_RG_VRF12_OCFB_EN_SHIFT},
	{PMIC_DA_QI_VRF12_OCFB_EN, PMIC_DA_QI_VRF12_OCFB_EN_ADDR, PMIC_DA_QI_VRF12_OCFB_EN_MASK,
	 PMIC_DA_QI_VRF12_OCFB_EN_SHIFT},
	{PMIC_RG_VRF12_DUMMY_LOAD, PMIC_RG_VRF12_DUMMY_LOAD_ADDR, PMIC_RG_VRF12_DUMMY_LOAD_MASK,
	 PMIC_RG_VRF12_DUMMY_LOAD_SHIFT},
	{PMIC_DA_QI_VRF12_DUMMY_LOAD, PMIC_DA_QI_VRF12_DUMMY_LOAD_ADDR,
	 PMIC_DA_QI_VRF12_DUMMY_LOAD_MASK, PMIC_DA_QI_VRF12_DUMMY_LOAD_SHIFT},
	{PMIC_RG_VCAMA1_SW_EN, PMIC_RG_VCAMA1_SW_EN_ADDR, PMIC_RG_VCAMA1_SW_EN_MASK,
	 PMIC_RG_VCAMA1_SW_EN_SHIFT},
	{PMIC_RG_VCAMA1_SW_LP, PMIC_RG_VCAMA1_SW_LP_ADDR, PMIC_RG_VCAMA1_SW_LP_MASK,
	 PMIC_RG_VCAMA1_SW_LP_SHIFT},
	{PMIC_RG_VCAMA1_SW_OP_EN, PMIC_RG_VCAMA1_SW_OP_EN_ADDR, PMIC_RG_VCAMA1_SW_OP_EN_MASK,
	 PMIC_RG_VCAMA1_SW_OP_EN_SHIFT},
	{PMIC_RG_VCAMA1_GO_ON_OP, PMIC_RG_VCAMA1_GO_ON_OP_ADDR, PMIC_RG_VCAMA1_GO_ON_OP_MASK,
	 PMIC_RG_VCAMA1_GO_ON_OP_SHIFT},
	{PMIC_RG_VCAMA1_GO_LP_OP, PMIC_RG_VCAMA1_GO_LP_OP_ADDR, PMIC_RG_VCAMA1_GO_LP_OP_MASK,
	 PMIC_RG_VCAMA1_GO_LP_OP_SHIFT},
	{PMIC_DA_QI_VCAMA1_MODE, PMIC_DA_QI_VCAMA1_MODE_ADDR, PMIC_DA_QI_VCAMA1_MODE_MASK,
	 PMIC_DA_QI_VCAMA1_MODE_SHIFT},
	{PMIC_RG_VCAMA1_STBTD, PMIC_RG_VCAMA1_STBTD_ADDR, PMIC_RG_VCAMA1_STBTD_MASK,
	 PMIC_RG_VCAMA1_STBTD_SHIFT},
	{PMIC_DA_QI_VCAMA1_STB, PMIC_DA_QI_VCAMA1_STB_ADDR, PMIC_DA_QI_VCAMA1_STB_MASK,
	 PMIC_DA_QI_VCAMA1_STB_SHIFT},
	{PMIC_DA_QI_VCAMA1_EN, PMIC_DA_QI_VCAMA1_EN_ADDR, PMIC_DA_QI_VCAMA1_EN_MASK,
	 PMIC_DA_QI_VCAMA1_EN_SHIFT},
	{PMIC_RG_VCAMA1_OCFB_EN, PMIC_RG_VCAMA1_OCFB_EN_ADDR, PMIC_RG_VCAMA1_OCFB_EN_MASK,
	 PMIC_RG_VCAMA1_OCFB_EN_SHIFT},
	{PMIC_DA_QI_VCAMA1_OCFB_EN, PMIC_DA_QI_VCAMA1_OCFB_EN_ADDR, PMIC_DA_QI_VCAMA1_OCFB_EN_MASK,
	 PMIC_DA_QI_VCAMA1_OCFB_EN_SHIFT},
	{PMIC_RG_VCAMA2_SW_EN, PMIC_RG_VCAMA2_SW_EN_ADDR, PMIC_RG_VCAMA2_SW_EN_MASK,
	 PMIC_RG_VCAMA2_SW_EN_SHIFT},
	{PMIC_RG_VCAMA2_SW_LP, PMIC_RG_VCAMA2_SW_LP_ADDR, PMIC_RG_VCAMA2_SW_LP_MASK,
	 PMIC_RG_VCAMA2_SW_LP_SHIFT},
	{PMIC_RG_VCAMA2_SW_OP_EN, PMIC_RG_VCAMA2_SW_OP_EN_ADDR, PMIC_RG_VCAMA2_SW_OP_EN_MASK,
	 PMIC_RG_VCAMA2_SW_OP_EN_SHIFT},
	{PMIC_RG_VCAMA2_GO_ON_OP, PMIC_RG_VCAMA2_GO_ON_OP_ADDR, PMIC_RG_VCAMA2_GO_ON_OP_MASK,
	 PMIC_RG_VCAMA2_GO_ON_OP_SHIFT},
	{PMIC_RG_VCAMA2_GO_LP_OP, PMIC_RG_VCAMA2_GO_LP_OP_ADDR, PMIC_RG_VCAMA2_GO_LP_OP_MASK,
	 PMIC_RG_VCAMA2_GO_LP_OP_SHIFT},
	{PMIC_DA_QI_VCAMA2_MODE, PMIC_DA_QI_VCAMA2_MODE_ADDR, PMIC_DA_QI_VCAMA2_MODE_MASK,
	 PMIC_DA_QI_VCAMA2_MODE_SHIFT},
	{PMIC_RG_VCAMA2_STBTD, PMIC_RG_VCAMA2_STBTD_ADDR, PMIC_RG_VCAMA2_STBTD_MASK,
	 PMIC_RG_VCAMA2_STBTD_SHIFT},
	{PMIC_DA_QI_VCAMA2_STB, PMIC_DA_QI_VCAMA2_STB_ADDR, PMIC_DA_QI_VCAMA2_STB_MASK,
	 PMIC_DA_QI_VCAMA2_STB_SHIFT},
	{PMIC_DA_QI_VCAMA2_EN, PMIC_DA_QI_VCAMA2_EN_ADDR, PMIC_DA_QI_VCAMA2_EN_MASK,
	 PMIC_DA_QI_VCAMA2_EN_SHIFT},
	{PMIC_RG_VCAMA2_OCFB_EN, PMIC_RG_VCAMA2_OCFB_EN_ADDR, PMIC_RG_VCAMA2_OCFB_EN_MASK,
	 PMIC_RG_VCAMA2_OCFB_EN_SHIFT},
	{PMIC_DA_QI_VCAMA2_OCFB_EN, PMIC_DA_QI_VCAMA2_OCFB_EN_ADDR, PMIC_DA_QI_VCAMA2_OCFB_EN_MASK,
	 PMIC_DA_QI_VCAMA2_OCFB_EN_SHIFT},
	{PMIC_RG_LDO_MANUAL_OFS_SEL, PMIC_RG_LDO_MANUAL_OFS_SEL_ADDR,
	 PMIC_RG_LDO_MANUAL_OFS_SEL_MASK, PMIC_RG_LDO_MANUAL_OFS_SEL_SHIFT},
	{PMIC_RG_LDO_MANUAL_OFS_EN, PMIC_RG_LDO_MANUAL_OFS_EN_ADDR, PMIC_RG_LDO_MANUAL_OFS_EN_MASK,
	 PMIC_RG_LDO_MANUAL_OFS_EN_SHIFT},
	{PMIC_RG_LDO_CALI_RSV, PMIC_RG_LDO_CALI_RSV_ADDR, PMIC_RG_LDO_CALI_RSV_MASK,
	 PMIC_RG_LDO_CALI_RSV_SHIFT},
	{PMIC_RG_LDO_CALI_MODE_SEL, PMIC_RG_LDO_CALI_MODE_SEL_ADDR, PMIC_RG_LDO_CALI_MODE_SEL_MASK,
	 PMIC_RG_LDO_CALI_MODE_SEL_SHIFT},
	{PMIC_RG_LDO_MANUAL_MODE, PMIC_RG_LDO_MANUAL_MODE_ADDR, PMIC_RG_LDO_MANUAL_MODE_MASK,
	 PMIC_RG_LDO_MANUAL_MODE_SHIFT},
	{PMIC_RG_LDO_AUTO_START, PMIC_RG_LDO_AUTO_START_ADDR, PMIC_RG_LDO_AUTO_START_MASK,
	 PMIC_RG_LDO_AUTO_START_SHIFT},
	{PMIC_RG_LDO_CALI_CLR, PMIC_RG_LDO_CALI_CLR_ADDR, PMIC_RG_LDO_CALI_CLR_MASK,
	 PMIC_RG_LDO_CALI_CLR_SHIFT},
	{PMIC_LDO_CALI_INC_COUNT, PMIC_LDO_CALI_INC_COUNT_ADDR, PMIC_LDO_CALI_INC_COUNT_MASK,
	 PMIC_LDO_CALI_INC_COUNT_SHIFT},
	{PMIC_LDO_K_START, PMIC_LDO_K_START_ADDR, PMIC_LDO_K_START_MASK, PMIC_LDO_K_START_SHIFT},
	{PMIC_LDO_CALI_DONE, PMIC_LDO_CALI_DONE_ADDR, PMIC_LDO_CALI_DONE_MASK,
	 PMIC_LDO_CALI_DONE_SHIFT},
	{PMIC_LDO_CALI_CNT, PMIC_LDO_CALI_CNT_ADDR, PMIC_LDO_CALI_CNT_MASK,
	 PMIC_LDO_CALI_CNT_SHIFT},
	{PMIC_LDO_CALI_EFUSE_EN, PMIC_LDO_CALI_EFUSE_EN_ADDR, PMIC_LDO_CALI_EFUSE_EN_MASK,
	 PMIC_LDO_CALI_EFUSE_EN_SHIFT},
	{PMIC_DA_QI_VIO18_OFS_CAL_EN, PMIC_DA_QI_VIO18_OFS_CAL_EN_ADDR,
	 PMIC_DA_QI_VIO18_OFS_CAL_EN_MASK, PMIC_DA_QI_VIO18_OFS_CAL_EN_SHIFT},
	{PMIC_AD_QI_VIO18_CAL_INDI, PMIC_AD_QI_VIO18_CAL_INDI_ADDR, PMIC_AD_QI_VIO18_CAL_INDI_MASK,
	 PMIC_AD_QI_VIO18_CAL_INDI_SHIFT},
	{PMIC_DA_QI_VIO18_OFS_SEL, PMIC_DA_QI_VIO18_OFS_SEL_ADDR, PMIC_DA_QI_VIO18_OFS_SEL_MASK,
	 PMIC_DA_QI_VIO18_OFS_SEL_SHIFT},
	{PMIC_LDO_DEGTD_SEL, PMIC_LDO_DEGTD_SEL_ADDR, PMIC_LDO_DEGTD_SEL_MASK,
	 PMIC_LDO_DEGTD_SEL_SHIFT},
	{PMIC_RG_VRTC_EN, PMIC_RG_VRTC_EN_ADDR, PMIC_RG_VRTC_EN_MASK, PMIC_RG_VRTC_EN_SHIFT},
	{PMIC_DA_QI_VRTC_EN, PMIC_DA_QI_VRTC_EN_ADDR, PMIC_DA_QI_VRTC_EN_MASK,
	 PMIC_DA_QI_VRTC_EN_SHIFT},
	{PMIC_RG_LDO_RSV1, PMIC_RG_LDO_RSV1_ADDR, PMIC_RG_LDO_RSV1_MASK, PMIC_RG_LDO_RSV1_SHIFT},
	{PMIC_RG_LDO_RSV0, PMIC_RG_LDO_RSV0_ADDR, PMIC_RG_LDO_RSV0_MASK, PMIC_RG_LDO_RSV0_SHIFT},
	{PMIC_RG_LDO_RSV2, PMIC_RG_LDO_RSV2_ADDR, PMIC_RG_LDO_RSV2_MASK, PMIC_RG_LDO_RSV2_SHIFT},
	{PMIC_RG_VSRAM_DVFS1_SW_EN, PMIC_RG_VSRAM_DVFS1_SW_EN_ADDR, PMIC_RG_VSRAM_DVFS1_SW_EN_MASK,
	 PMIC_RG_VSRAM_DVFS1_SW_EN_SHIFT},
	{PMIC_RG_VSRAM_DVFS1_SW_LP, PMIC_RG_VSRAM_DVFS1_SW_LP_ADDR, PMIC_RG_VSRAM_DVFS1_SW_LP_MASK,
	 PMIC_RG_VSRAM_DVFS1_SW_LP_SHIFT},
	{PMIC_RG_VSRAM_DVFS1_VOSEL, PMIC_RG_VSRAM_DVFS1_VOSEL_ADDR, PMIC_RG_VSRAM_DVFS1_VOSEL_MASK,
	 PMIC_RG_VSRAM_DVFS1_VOSEL_SHIFT},
	{PMIC_RG_VSRAM_DVFS1_VOSEL_SLEEP, PMIC_RG_VSRAM_DVFS1_VOSEL_SLEEP_ADDR,
	 PMIC_RG_VSRAM_DVFS1_VOSEL_SLEEP_MASK, PMIC_RG_VSRAM_DVFS1_VOSEL_SLEEP_SHIFT},
	{PMIC_RG_VSRAM_DVFS1_SFCHG_FRATE, PMIC_RG_VSRAM_DVFS1_SFCHG_FRATE_ADDR,
	 PMIC_RG_VSRAM_DVFS1_SFCHG_FRATE_MASK, PMIC_RG_VSRAM_DVFS1_SFCHG_FRATE_SHIFT},
	{PMIC_RG_VSRAM_DVFS1_SFCHG_FEN, PMIC_RG_VSRAM_DVFS1_SFCHG_FEN_ADDR,
	 PMIC_RG_VSRAM_DVFS1_SFCHG_FEN_MASK, PMIC_RG_VSRAM_DVFS1_SFCHG_FEN_SHIFT},
	{PMIC_RG_VSRAM_DVFS1_SFCHG_RRATE, PMIC_RG_VSRAM_DVFS1_SFCHG_RRATE_ADDR,
	 PMIC_RG_VSRAM_DVFS1_SFCHG_RRATE_MASK, PMIC_RG_VSRAM_DVFS1_SFCHG_RRATE_SHIFT},
	{PMIC_RG_VSRAM_DVFS1_SFCHG_REN, PMIC_RG_VSRAM_DVFS1_SFCHG_REN_ADDR,
	 PMIC_RG_VSRAM_DVFS1_SFCHG_REN_MASK, PMIC_RG_VSRAM_DVFS1_SFCHG_REN_SHIFT},
	{PMIC_RG_VSRAM_DVFS1_DVS_TRANS_TD, PMIC_RG_VSRAM_DVFS1_DVS_TRANS_TD_ADDR,
	 PMIC_RG_VSRAM_DVFS1_DVS_TRANS_TD_MASK, PMIC_RG_VSRAM_DVFS1_DVS_TRANS_TD_SHIFT},
	{PMIC_RG_VSRAM_DVFS1_DVS_TRANS_CTRL, PMIC_RG_VSRAM_DVFS1_DVS_TRANS_CTRL_ADDR,
	 PMIC_RG_VSRAM_DVFS1_DVS_TRANS_CTRL_MASK, PMIC_RG_VSRAM_DVFS1_DVS_TRANS_CTRL_SHIFT},
	{PMIC_RG_VSRAM_DVFS1_DVS_TRANS_ONCE, PMIC_RG_VSRAM_DVFS1_DVS_TRANS_ONCE_ADDR,
	 PMIC_RG_VSRAM_DVFS1_DVS_TRANS_ONCE_MASK, PMIC_RG_VSRAM_DVFS1_DVS_TRANS_ONCE_SHIFT},
	{PMIC_RG_VSRAM_DVFS1_SW_OP_EN, PMIC_RG_VSRAM_DVFS1_SW_OP_EN_ADDR,
	 PMIC_RG_VSRAM_DVFS1_SW_OP_EN_MASK, PMIC_RG_VSRAM_DVFS1_SW_OP_EN_SHIFT},
	{PMIC_RG_VSRAM_DVFS1_HW0_OP_EN, PMIC_RG_VSRAM_DVFS1_HW0_OP_EN_ADDR,
	 PMIC_RG_VSRAM_DVFS1_HW0_OP_EN_MASK, PMIC_RG_VSRAM_DVFS1_HW0_OP_EN_SHIFT},
	{PMIC_RG_VSRAM_DVFS1_HW1_OP_EN, PMIC_RG_VSRAM_DVFS1_HW1_OP_EN_ADDR,
	 PMIC_RG_VSRAM_DVFS1_HW1_OP_EN_MASK, PMIC_RG_VSRAM_DVFS1_HW1_OP_EN_SHIFT},
	{PMIC_RG_VSRAM_DVFS1_HW2_OP_EN, PMIC_RG_VSRAM_DVFS1_HW2_OP_EN_ADDR,
	 PMIC_RG_VSRAM_DVFS1_HW2_OP_EN_MASK, PMIC_RG_VSRAM_DVFS1_HW2_OP_EN_SHIFT},
	{PMIC_RG_VSRAM_DVFS1_OP_EN_SET, PMIC_RG_VSRAM_DVFS1_OP_EN_SET_ADDR,
	 PMIC_RG_VSRAM_DVFS1_OP_EN_SET_MASK, PMIC_RG_VSRAM_DVFS1_OP_EN_SET_SHIFT},
	{PMIC_RG_VSRAM_DVFS1_OP_EN_CLR, PMIC_RG_VSRAM_DVFS1_OP_EN_CLR_ADDR,
	 PMIC_RG_VSRAM_DVFS1_OP_EN_CLR_MASK, PMIC_RG_VSRAM_DVFS1_OP_EN_CLR_SHIFT},
	{PMIC_RG_VSRAM_DVFS1_HW0_OP_CFG, PMIC_RG_VSRAM_DVFS1_HW0_OP_CFG_ADDR,
	 PMIC_RG_VSRAM_DVFS1_HW0_OP_CFG_MASK, PMIC_RG_VSRAM_DVFS1_HW0_OP_CFG_SHIFT},
	{PMIC_RG_VSRAM_DVFS1_HW1_OP_CFG, PMIC_RG_VSRAM_DVFS1_HW1_OP_CFG_ADDR,
	 PMIC_RG_VSRAM_DVFS1_HW1_OP_CFG_MASK, PMIC_RG_VSRAM_DVFS1_HW1_OP_CFG_SHIFT},
	{PMIC_RG_VSRAM_DVFS1_HW2_OP_CFG, PMIC_RG_VSRAM_DVFS1_HW2_OP_CFG_ADDR,
	 PMIC_RG_VSRAM_DVFS1_HW2_OP_CFG_MASK, PMIC_RG_VSRAM_DVFS1_HW2_OP_CFG_SHIFT},
	{PMIC_RG_VSRAM_DVFS1_GO_ON_OP, PMIC_RG_VSRAM_DVFS1_GO_ON_OP_ADDR,
	 PMIC_RG_VSRAM_DVFS1_GO_ON_OP_MASK, PMIC_RG_VSRAM_DVFS1_GO_ON_OP_SHIFT},
	{PMIC_RG_VSRAM_DVFS1_GO_LP_OP, PMIC_RG_VSRAM_DVFS1_GO_LP_OP_ADDR,
	 PMIC_RG_VSRAM_DVFS1_GO_LP_OP_MASK, PMIC_RG_VSRAM_DVFS1_GO_LP_OP_SHIFT},
	{PMIC_RG_VSRAM_DVFS1_OP_CFG_SET, PMIC_RG_VSRAM_DVFS1_OP_CFG_SET_ADDR,
	 PMIC_RG_VSRAM_DVFS1_OP_CFG_SET_MASK, PMIC_RG_VSRAM_DVFS1_OP_CFG_SET_SHIFT},
	{PMIC_RG_VSRAM_DVFS1_OP_CFG_CLR, PMIC_RG_VSRAM_DVFS1_OP_CFG_CLR_ADDR,
	 PMIC_RG_VSRAM_DVFS1_OP_CFG_CLR_MASK, PMIC_RG_VSRAM_DVFS1_OP_CFG_CLR_SHIFT},
	{PMIC_DA_QI_VSRAM_DVFS1_MODE, PMIC_DA_QI_VSRAM_DVFS1_MODE_ADDR,
	 PMIC_DA_QI_VSRAM_DVFS1_MODE_MASK, PMIC_DA_QI_VSRAM_DVFS1_MODE_SHIFT},
	{PMIC_RG_VSRAM_DVFS1_STBTD, PMIC_RG_VSRAM_DVFS1_STBTD_ADDR, PMIC_RG_VSRAM_DVFS1_STBTD_MASK,
	 PMIC_RG_VSRAM_DVFS1_STBTD_SHIFT},
	{PMIC_RG_VSRAM_DVFS1_OCFB_EN, PMIC_RG_VSRAM_DVFS1_OCFB_EN_ADDR,
	 PMIC_RG_VSRAM_DVFS1_OCFB_EN_MASK, PMIC_RG_VSRAM_DVFS1_OCFB_EN_SHIFT},
	{PMIC_DA_QI_VSRAM_DVFS1_OCFB_EN, PMIC_DA_QI_VSRAM_DVFS1_OCFB_EN_ADDR,
	 PMIC_DA_QI_VSRAM_DVFS1_OCFB_EN_MASK, PMIC_DA_QI_VSRAM_DVFS1_OCFB_EN_SHIFT},
	{PMIC_RG_VSRAM_DVFS1_DUMMY_LOAD, PMIC_RG_VSRAM_DVFS1_DUMMY_LOAD_ADDR,
	 PMIC_RG_VSRAM_DVFS1_DUMMY_LOAD_MASK, PMIC_RG_VSRAM_DVFS1_DUMMY_LOAD_SHIFT},
	{PMIC_DA_QI_VSRAM_DVFS1_DUMMY_LOAD, PMIC_DA_QI_VSRAM_DVFS1_DUMMY_LOAD_ADDR,
	 PMIC_DA_QI_VSRAM_DVFS1_DUMMY_LOAD_MASK, PMIC_DA_QI_VSRAM_DVFS1_DUMMY_LOAD_SHIFT},
	{PMIC_DA_NI_VSRAM_DVFS1_VOSEL_GRAY, PMIC_DA_NI_VSRAM_DVFS1_VOSEL_GRAY_ADDR,
	 PMIC_DA_NI_VSRAM_DVFS1_VOSEL_GRAY_MASK, PMIC_DA_NI_VSRAM_DVFS1_VOSEL_GRAY_SHIFT},
	{PMIC_DA_NI_VSRAM_DVFS1_VOSEL, PMIC_DA_NI_VSRAM_DVFS1_VOSEL_ADDR,
	 PMIC_DA_NI_VSRAM_DVFS1_VOSEL_MASK, PMIC_DA_NI_VSRAM_DVFS1_VOSEL_SHIFT},
	{PMIC_DA_QI_VSRAM_DVFS1_EN, PMIC_DA_QI_VSRAM_DVFS1_EN_ADDR, PMIC_DA_QI_VSRAM_DVFS1_EN_MASK,
	 PMIC_DA_QI_VSRAM_DVFS1_EN_SHIFT},
	{PMIC_DA_QI_VSRAM_DVFS1_STB, PMIC_DA_QI_VSRAM_DVFS1_STB_ADDR,
	 PMIC_DA_QI_VSRAM_DVFS1_STB_MASK, PMIC_DA_QI_VSRAM_DVFS1_STB_SHIFT},
	{PMIC_DA_NI_VSRAM_DVFS1_VSLEEP_SEL, PMIC_DA_NI_VSRAM_DVFS1_VSLEEP_SEL_ADDR,
	 PMIC_DA_NI_VSRAM_DVFS1_VSLEEP_SEL_MASK, PMIC_DA_NI_VSRAM_DVFS1_VSLEEP_SEL_SHIFT},
	{PMIC_DA_NI_VSRAM_DVFS1_R2R_PDN, PMIC_DA_NI_VSRAM_DVFS1_R2R_PDN_ADDR,
	 PMIC_DA_NI_VSRAM_DVFS1_R2R_PDN_MASK, PMIC_DA_NI_VSRAM_DVFS1_R2R_PDN_SHIFT},
	{PMIC_DA_NI_VSRAM_DVFS1_TRACK_NDIS_EN, PMIC_DA_NI_VSRAM_DVFS1_TRACK_NDIS_EN_ADDR,
	 PMIC_DA_NI_VSRAM_DVFS1_TRACK_NDIS_EN_MASK, PMIC_DA_NI_VSRAM_DVFS1_TRACK_NDIS_EN_SHIFT},
	{PMIC_RG_VSRAM_DVFS2_SW_EN, PMIC_RG_VSRAM_DVFS2_SW_EN_ADDR, PMIC_RG_VSRAM_DVFS2_SW_EN_MASK,
	 PMIC_RG_VSRAM_DVFS2_SW_EN_SHIFT},
	{PMIC_RG_VSRAM_DVFS2_SW_LP, PMIC_RG_VSRAM_DVFS2_SW_LP_ADDR, PMIC_RG_VSRAM_DVFS2_SW_LP_MASK,
	 PMIC_RG_VSRAM_DVFS2_SW_LP_SHIFT},
	{PMIC_RG_VSRAM_DVFS2_VOSEL, PMIC_RG_VSRAM_DVFS2_VOSEL_ADDR, PMIC_RG_VSRAM_DVFS2_VOSEL_MASK,
	 PMIC_RG_VSRAM_DVFS2_VOSEL_SHIFT},
	{PMIC_RG_VSRAM_DVFS2_VOSEL_SLEEP, PMIC_RG_VSRAM_DVFS2_VOSEL_SLEEP_ADDR,
	 PMIC_RG_VSRAM_DVFS2_VOSEL_SLEEP_MASK, PMIC_RG_VSRAM_DVFS2_VOSEL_SLEEP_SHIFT},
	{PMIC_RG_VSRAM_DVFS2_SFCHG_FRATE, PMIC_RG_VSRAM_DVFS2_SFCHG_FRATE_ADDR,
	 PMIC_RG_VSRAM_DVFS2_SFCHG_FRATE_MASK, PMIC_RG_VSRAM_DVFS2_SFCHG_FRATE_SHIFT},
	{PMIC_RG_VSRAM_DVFS2_SFCHG_FEN, PMIC_RG_VSRAM_DVFS2_SFCHG_FEN_ADDR,
	 PMIC_RG_VSRAM_DVFS2_SFCHG_FEN_MASK, PMIC_RG_VSRAM_DVFS2_SFCHG_FEN_SHIFT},
	{PMIC_RG_VSRAM_DVFS2_SFCHG_RRATE, PMIC_RG_VSRAM_DVFS2_SFCHG_RRATE_ADDR,
	 PMIC_RG_VSRAM_DVFS2_SFCHG_RRATE_MASK, PMIC_RG_VSRAM_DVFS2_SFCHG_RRATE_SHIFT},
	{PMIC_RG_VSRAM_DVFS2_SFCHG_REN, PMIC_RG_VSRAM_DVFS2_SFCHG_REN_ADDR,
	 PMIC_RG_VSRAM_DVFS2_SFCHG_REN_MASK, PMIC_RG_VSRAM_DVFS2_SFCHG_REN_SHIFT},
	{PMIC_RG_VSRAM_DVFS2_DVS_TRANS_TD, PMIC_RG_VSRAM_DVFS2_DVS_TRANS_TD_ADDR,
	 PMIC_RG_VSRAM_DVFS2_DVS_TRANS_TD_MASK, PMIC_RG_VSRAM_DVFS2_DVS_TRANS_TD_SHIFT},
	{PMIC_RG_VSRAM_DVFS2_DVS_TRANS_CTRL, PMIC_RG_VSRAM_DVFS2_DVS_TRANS_CTRL_ADDR,
	 PMIC_RG_VSRAM_DVFS2_DVS_TRANS_CTRL_MASK, PMIC_RG_VSRAM_DVFS2_DVS_TRANS_CTRL_SHIFT},
	{PMIC_RG_VSRAM_DVFS2_DVS_TRANS_ONCE, PMIC_RG_VSRAM_DVFS2_DVS_TRANS_ONCE_ADDR,
	 PMIC_RG_VSRAM_DVFS2_DVS_TRANS_ONCE_MASK, PMIC_RG_VSRAM_DVFS2_DVS_TRANS_ONCE_SHIFT},
	{PMIC_RG_VSRAM_DVFS2_SW_OP_EN, PMIC_RG_VSRAM_DVFS2_SW_OP_EN_ADDR,
	 PMIC_RG_VSRAM_DVFS2_SW_OP_EN_MASK, PMIC_RG_VSRAM_DVFS2_SW_OP_EN_SHIFT},
	{PMIC_RG_VSRAM_DVFS2_HW0_OP_EN, PMIC_RG_VSRAM_DVFS2_HW0_OP_EN_ADDR,
	 PMIC_RG_VSRAM_DVFS2_HW0_OP_EN_MASK, PMIC_RG_VSRAM_DVFS2_HW0_OP_EN_SHIFT},
	{PMIC_RG_VSRAM_DVFS2_HW1_OP_EN, PMIC_RG_VSRAM_DVFS2_HW1_OP_EN_ADDR,
	 PMIC_RG_VSRAM_DVFS2_HW1_OP_EN_MASK, PMIC_RG_VSRAM_DVFS2_HW1_OP_EN_SHIFT},
	{PMIC_RG_VSRAM_DVFS2_HW2_OP_EN, PMIC_RG_VSRAM_DVFS2_HW2_OP_EN_ADDR,
	 PMIC_RG_VSRAM_DVFS2_HW2_OP_EN_MASK, PMIC_RG_VSRAM_DVFS2_HW2_OP_EN_SHIFT},
	{PMIC_RG_VSRAM_DVFS2_OP_EN_SET, PMIC_RG_VSRAM_DVFS2_OP_EN_SET_ADDR,
	 PMIC_RG_VSRAM_DVFS2_OP_EN_SET_MASK, PMIC_RG_VSRAM_DVFS2_OP_EN_SET_SHIFT},
	{PMIC_RG_VSRAM_DVFS2_OP_EN_CLR, PMIC_RG_VSRAM_DVFS2_OP_EN_CLR_ADDR,
	 PMIC_RG_VSRAM_DVFS2_OP_EN_CLR_MASK, PMIC_RG_VSRAM_DVFS2_OP_EN_CLR_SHIFT},
	{PMIC_RG_VSRAM_DVFS2_HW0_OP_CFG, PMIC_RG_VSRAM_DVFS2_HW0_OP_CFG_ADDR,
	 PMIC_RG_VSRAM_DVFS2_HW0_OP_CFG_MASK, PMIC_RG_VSRAM_DVFS2_HW0_OP_CFG_SHIFT},
	{PMIC_RG_VSRAM_DVFS2_HW1_OP_CFG, PMIC_RG_VSRAM_DVFS2_HW1_OP_CFG_ADDR,
	 PMIC_RG_VSRAM_DVFS2_HW1_OP_CFG_MASK, PMIC_RG_VSRAM_DVFS2_HW1_OP_CFG_SHIFT},
	{PMIC_RG_VSRAM_DVFS2_HW2_OP_CFG, PMIC_RG_VSRAM_DVFS2_HW2_OP_CFG_ADDR,
	 PMIC_RG_VSRAM_DVFS2_HW2_OP_CFG_MASK, PMIC_RG_VSRAM_DVFS2_HW2_OP_CFG_SHIFT},
	{PMIC_RG_VSRAM_DVFS2_GO_ON_OP, PMIC_RG_VSRAM_DVFS2_GO_ON_OP_ADDR,
	 PMIC_RG_VSRAM_DVFS2_GO_ON_OP_MASK, PMIC_RG_VSRAM_DVFS2_GO_ON_OP_SHIFT},
	{PMIC_RG_VSRAM_DVFS2_GO_LP_OP, PMIC_RG_VSRAM_DVFS2_GO_LP_OP_ADDR,
	 PMIC_RG_VSRAM_DVFS2_GO_LP_OP_MASK, PMIC_RG_VSRAM_DVFS2_GO_LP_OP_SHIFT},
	{PMIC_RG_VSRAM_DVFS2_OP_CFG_SET, PMIC_RG_VSRAM_DVFS2_OP_CFG_SET_ADDR,
	 PMIC_RG_VSRAM_DVFS2_OP_CFG_SET_MASK, PMIC_RG_VSRAM_DVFS2_OP_CFG_SET_SHIFT},
	{PMIC_RG_VSRAM_DVFS2_OP_CFG_CLR, PMIC_RG_VSRAM_DVFS2_OP_CFG_CLR_ADDR,
	 PMIC_RG_VSRAM_DVFS2_OP_CFG_CLR_MASK, PMIC_RG_VSRAM_DVFS2_OP_CFG_CLR_SHIFT},
	{PMIC_DA_QI_VSRAM_DVFS2_MODE, PMIC_DA_QI_VSRAM_DVFS2_MODE_ADDR,
	 PMIC_DA_QI_VSRAM_DVFS2_MODE_MASK, PMIC_DA_QI_VSRAM_DVFS2_MODE_SHIFT},
	{PMIC_RG_VSRAM_DVFS2_STBTD, PMIC_RG_VSRAM_DVFS2_STBTD_ADDR, PMIC_RG_VSRAM_DVFS2_STBTD_MASK,
	 PMIC_RG_VSRAM_DVFS2_STBTD_SHIFT},
	{PMIC_RG_VSRAM_DVFS2_OCFB_EN, PMIC_RG_VSRAM_DVFS2_OCFB_EN_ADDR,
	 PMIC_RG_VSRAM_DVFS2_OCFB_EN_MASK, PMIC_RG_VSRAM_DVFS2_OCFB_EN_SHIFT},
	{PMIC_DA_QI_VSRAM_DVFS2_OCFB_EN, PMIC_DA_QI_VSRAM_DVFS2_OCFB_EN_ADDR,
	 PMIC_DA_QI_VSRAM_DVFS2_OCFB_EN_MASK, PMIC_DA_QI_VSRAM_DVFS2_OCFB_EN_SHIFT},
	{PMIC_RG_VSRAM_DVFS2_DUMMY_LOAD, PMIC_RG_VSRAM_DVFS2_DUMMY_LOAD_ADDR,
	 PMIC_RG_VSRAM_DVFS2_DUMMY_LOAD_MASK, PMIC_RG_VSRAM_DVFS2_DUMMY_LOAD_SHIFT},
	{PMIC_DA_QI_VSRAM_DVFS2_DUMMY_LOAD, PMIC_DA_QI_VSRAM_DVFS2_DUMMY_LOAD_ADDR,
	 PMIC_DA_QI_VSRAM_DVFS2_DUMMY_LOAD_MASK, PMIC_DA_QI_VSRAM_DVFS2_DUMMY_LOAD_SHIFT},
	{PMIC_DA_NI_VSRAM_DVFS2_VOSEL_GRAY, PMIC_DA_NI_VSRAM_DVFS2_VOSEL_GRAY_ADDR,
	 PMIC_DA_NI_VSRAM_DVFS2_VOSEL_GRAY_MASK, PMIC_DA_NI_VSRAM_DVFS2_VOSEL_GRAY_SHIFT},
	{PMIC_DA_NI_VSRAM_DVFS2_VOSEL, PMIC_DA_NI_VSRAM_DVFS2_VOSEL_ADDR,
	 PMIC_DA_NI_VSRAM_DVFS2_VOSEL_MASK, PMIC_DA_NI_VSRAM_DVFS2_VOSEL_SHIFT},
	{PMIC_DA_QI_VSRAM_DVFS2_EN, PMIC_DA_QI_VSRAM_DVFS2_EN_ADDR, PMIC_DA_QI_VSRAM_DVFS2_EN_MASK,
	 PMIC_DA_QI_VSRAM_DVFS2_EN_SHIFT},
	{PMIC_DA_QI_VSRAM_DVFS2_STB, PMIC_DA_QI_VSRAM_DVFS2_STB_ADDR,
	 PMIC_DA_QI_VSRAM_DVFS2_STB_MASK, PMIC_DA_QI_VSRAM_DVFS2_STB_SHIFT},
	{PMIC_DA_NI_VSRAM_DVFS2_VSLEEP_SEL, PMIC_DA_NI_VSRAM_DVFS2_VSLEEP_SEL_ADDR,
	 PMIC_DA_NI_VSRAM_DVFS2_VSLEEP_SEL_MASK, PMIC_DA_NI_VSRAM_DVFS2_VSLEEP_SEL_SHIFT},
	{PMIC_DA_NI_VSRAM_DVFS2_R2R_PDN, PMIC_DA_NI_VSRAM_DVFS2_R2R_PDN_ADDR,
	 PMIC_DA_NI_VSRAM_DVFS2_R2R_PDN_MASK, PMIC_DA_NI_VSRAM_DVFS2_R2R_PDN_SHIFT},
	{PMIC_DA_NI_VSRAM_DVFS2_TRACK_NDIS_EN, PMIC_DA_NI_VSRAM_DVFS2_TRACK_NDIS_EN_ADDR,
	 PMIC_DA_NI_VSRAM_DVFS2_TRACK_NDIS_EN_MASK, PMIC_DA_NI_VSRAM_DVFS2_TRACK_NDIS_EN_SHIFT},
	{PMIC_RG_VSRAM_VCORE_SW_EN, PMIC_RG_VSRAM_VCORE_SW_EN_ADDR, PMIC_RG_VSRAM_VCORE_SW_EN_MASK,
	 PMIC_RG_VSRAM_VCORE_SW_EN_SHIFT},
	{PMIC_RG_VSRAM_VCORE_SW_LP, PMIC_RG_VSRAM_VCORE_SW_LP_ADDR, PMIC_RG_VSRAM_VCORE_SW_LP_MASK,
	 PMIC_RG_VSRAM_VCORE_SW_LP_SHIFT},
	{PMIC_RG_VSRAM_VCORE_VOSEL, PMIC_RG_VSRAM_VCORE_VOSEL_ADDR, PMIC_RG_VSRAM_VCORE_VOSEL_MASK,
	 PMIC_RG_VSRAM_VCORE_VOSEL_SHIFT},
	{PMIC_RG_VSRAM_VCORE_VOSEL_SLEEP, PMIC_RG_VSRAM_VCORE_VOSEL_SLEEP_ADDR,
	 PMIC_RG_VSRAM_VCORE_VOSEL_SLEEP_MASK, PMIC_RG_VSRAM_VCORE_VOSEL_SLEEP_SHIFT},
	{PMIC_RG_VSRAM_VCORE_SFCHG_FRATE, PMIC_RG_VSRAM_VCORE_SFCHG_FRATE_ADDR,
	 PMIC_RG_VSRAM_VCORE_SFCHG_FRATE_MASK, PMIC_RG_VSRAM_VCORE_SFCHG_FRATE_SHIFT},
	{PMIC_RG_VSRAM_VCORE_SFCHG_FEN, PMIC_RG_VSRAM_VCORE_SFCHG_FEN_ADDR,
	 PMIC_RG_VSRAM_VCORE_SFCHG_FEN_MASK, PMIC_RG_VSRAM_VCORE_SFCHG_FEN_SHIFT},
	{PMIC_RG_VSRAM_VCORE_SFCHG_RRATE, PMIC_RG_VSRAM_VCORE_SFCHG_RRATE_ADDR,
	 PMIC_RG_VSRAM_VCORE_SFCHG_RRATE_MASK, PMIC_RG_VSRAM_VCORE_SFCHG_RRATE_SHIFT},
	{PMIC_RG_VSRAM_VCORE_SFCHG_REN, PMIC_RG_VSRAM_VCORE_SFCHG_REN_ADDR,
	 PMIC_RG_VSRAM_VCORE_SFCHG_REN_MASK, PMIC_RG_VSRAM_VCORE_SFCHG_REN_SHIFT},
	{PMIC_RG_VSRAM_VCORE_DVS_TRANS_TD, PMIC_RG_VSRAM_VCORE_DVS_TRANS_TD_ADDR,
	 PMIC_RG_VSRAM_VCORE_DVS_TRANS_TD_MASK, PMIC_RG_VSRAM_VCORE_DVS_TRANS_TD_SHIFT},
	{PMIC_RG_VSRAM_VCORE_DVS_TRANS_CTRL, PMIC_RG_VSRAM_VCORE_DVS_TRANS_CTRL_ADDR,
	 PMIC_RG_VSRAM_VCORE_DVS_TRANS_CTRL_MASK, PMIC_RG_VSRAM_VCORE_DVS_TRANS_CTRL_SHIFT},
	{PMIC_RG_VSRAM_VCORE_DVS_TRANS_ONCE, PMIC_RG_VSRAM_VCORE_DVS_TRANS_ONCE_ADDR,
	 PMIC_RG_VSRAM_VCORE_DVS_TRANS_ONCE_MASK, PMIC_RG_VSRAM_VCORE_DVS_TRANS_ONCE_SHIFT},
	{PMIC_RG_VSRAM_VCORE_SW_OP_EN, PMIC_RG_VSRAM_VCORE_SW_OP_EN_ADDR,
	 PMIC_RG_VSRAM_VCORE_SW_OP_EN_MASK, PMIC_RG_VSRAM_VCORE_SW_OP_EN_SHIFT},
	{PMIC_RG_VSRAM_VCORE_HW0_OP_EN, PMIC_RG_VSRAM_VCORE_HW0_OP_EN_ADDR,
	 PMIC_RG_VSRAM_VCORE_HW0_OP_EN_MASK, PMIC_RG_VSRAM_VCORE_HW0_OP_EN_SHIFT},
	{PMIC_RG_VSRAM_VCORE_HW1_OP_EN, PMIC_RG_VSRAM_VCORE_HW1_OP_EN_ADDR,
	 PMIC_RG_VSRAM_VCORE_HW1_OP_EN_MASK, PMIC_RG_VSRAM_VCORE_HW1_OP_EN_SHIFT},
	{PMIC_RG_VSRAM_VCORE_HW2_OP_EN, PMIC_RG_VSRAM_VCORE_HW2_OP_EN_ADDR,
	 PMIC_RG_VSRAM_VCORE_HW2_OP_EN_MASK, PMIC_RG_VSRAM_VCORE_HW2_OP_EN_SHIFT},
	{PMIC_RG_VSRAM_VCORE_OP_EN_SET, PMIC_RG_VSRAM_VCORE_OP_EN_SET_ADDR,
	 PMIC_RG_VSRAM_VCORE_OP_EN_SET_MASK, PMIC_RG_VSRAM_VCORE_OP_EN_SET_SHIFT},
	{PMIC_RG_VSRAM_VCORE_OP_EN_CLR, PMIC_RG_VSRAM_VCORE_OP_EN_CLR_ADDR,
	 PMIC_RG_VSRAM_VCORE_OP_EN_CLR_MASK, PMIC_RG_VSRAM_VCORE_OP_EN_CLR_SHIFT},
	{PMIC_RG_VSRAM_VCORE_HW0_OP_CFG, PMIC_RG_VSRAM_VCORE_HW0_OP_CFG_ADDR,
	 PMIC_RG_VSRAM_VCORE_HW0_OP_CFG_MASK, PMIC_RG_VSRAM_VCORE_HW0_OP_CFG_SHIFT},
	{PMIC_RG_VSRAM_VCORE_HW1_OP_CFG, PMIC_RG_VSRAM_VCORE_HW1_OP_CFG_ADDR,
	 PMIC_RG_VSRAM_VCORE_HW1_OP_CFG_MASK, PMIC_RG_VSRAM_VCORE_HW1_OP_CFG_SHIFT},
	{PMIC_RG_VSRAM_VCORE_HW2_OP_CFG, PMIC_RG_VSRAM_VCORE_HW2_OP_CFG_ADDR,
	 PMIC_RG_VSRAM_VCORE_HW2_OP_CFG_MASK, PMIC_RG_VSRAM_VCORE_HW2_OP_CFG_SHIFT},
	{PMIC_RG_VSRAM_VCORE_GO_ON_OP, PMIC_RG_VSRAM_VCORE_GO_ON_OP_ADDR,
	 PMIC_RG_VSRAM_VCORE_GO_ON_OP_MASK, PMIC_RG_VSRAM_VCORE_GO_ON_OP_SHIFT},
	{PMIC_RG_VSRAM_VCORE_GO_LP_OP, PMIC_RG_VSRAM_VCORE_GO_LP_OP_ADDR,
	 PMIC_RG_VSRAM_VCORE_GO_LP_OP_MASK, PMIC_RG_VSRAM_VCORE_GO_LP_OP_SHIFT},
	{PMIC_RG_VSRAM_VCORE_OP_CFG_SET, PMIC_RG_VSRAM_VCORE_OP_CFG_SET_ADDR,
	 PMIC_RG_VSRAM_VCORE_OP_CFG_SET_MASK, PMIC_RG_VSRAM_VCORE_OP_CFG_SET_SHIFT},
	{PMIC_RG_VSRAM_VCORE_OP_CFG_CLR, PMIC_RG_VSRAM_VCORE_OP_CFG_CLR_ADDR,
	 PMIC_RG_VSRAM_VCORE_OP_CFG_CLR_MASK, PMIC_RG_VSRAM_VCORE_OP_CFG_CLR_SHIFT},
	{PMIC_DA_QI_VSRAM_VCORE_MODE, PMIC_DA_QI_VSRAM_VCORE_MODE_ADDR,
	 PMIC_DA_QI_VSRAM_VCORE_MODE_MASK, PMIC_DA_QI_VSRAM_VCORE_MODE_SHIFT},
	{PMIC_RG_VSRAM_VCORE_STBTD, PMIC_RG_VSRAM_VCORE_STBTD_ADDR, PMIC_RG_VSRAM_VCORE_STBTD_MASK,
	 PMIC_RG_VSRAM_VCORE_STBTD_SHIFT},
	{PMIC_RG_VSRAM_VCORE_OCFB_EN, PMIC_RG_VSRAM_VCORE_OCFB_EN_ADDR,
	 PMIC_RG_VSRAM_VCORE_OCFB_EN_MASK, PMIC_RG_VSRAM_VCORE_OCFB_EN_SHIFT},
	{PMIC_DA_QI_VSRAM_VCORE_OCFB_EN, PMIC_DA_QI_VSRAM_VCORE_OCFB_EN_ADDR,
	 PMIC_DA_QI_VSRAM_VCORE_OCFB_EN_MASK, PMIC_DA_QI_VSRAM_VCORE_OCFB_EN_SHIFT},
	{PMIC_RG_VSRAM_VCORE_DUMMY_LOAD, PMIC_RG_VSRAM_VCORE_DUMMY_LOAD_ADDR,
	 PMIC_RG_VSRAM_VCORE_DUMMY_LOAD_MASK, PMIC_RG_VSRAM_VCORE_DUMMY_LOAD_SHIFT},
	{PMIC_DA_QI_VSRAM_VCORE_DUMMY_LOAD, PMIC_DA_QI_VSRAM_VCORE_DUMMY_LOAD_ADDR,
	 PMIC_DA_QI_VSRAM_VCORE_DUMMY_LOAD_MASK, PMIC_DA_QI_VSRAM_VCORE_DUMMY_LOAD_SHIFT},
	{PMIC_DA_NI_VSRAM_VCORE_VOSEL_GRAY, PMIC_DA_NI_VSRAM_VCORE_VOSEL_GRAY_ADDR,
	 PMIC_DA_NI_VSRAM_VCORE_VOSEL_GRAY_MASK, PMIC_DA_NI_VSRAM_VCORE_VOSEL_GRAY_SHIFT},
	{PMIC_DA_NI_VSRAM_VCORE_VOSEL, PMIC_DA_NI_VSRAM_VCORE_VOSEL_ADDR,
	 PMIC_DA_NI_VSRAM_VCORE_VOSEL_MASK, PMIC_DA_NI_VSRAM_VCORE_VOSEL_SHIFT},
	{PMIC_DA_QI_VSRAM_VCORE_EN, PMIC_DA_QI_VSRAM_VCORE_EN_ADDR, PMIC_DA_QI_VSRAM_VCORE_EN_MASK,
	 PMIC_DA_QI_VSRAM_VCORE_EN_SHIFT},
	{PMIC_DA_QI_VSRAM_VCORE_STB, PMIC_DA_QI_VSRAM_VCORE_STB_ADDR,
	 PMIC_DA_QI_VSRAM_VCORE_STB_MASK, PMIC_DA_QI_VSRAM_VCORE_STB_SHIFT},
	{PMIC_DA_NI_VSRAM_VCORE_VSLEEP_SEL, PMIC_DA_NI_VSRAM_VCORE_VSLEEP_SEL_ADDR,
	 PMIC_DA_NI_VSRAM_VCORE_VSLEEP_SEL_MASK, PMIC_DA_NI_VSRAM_VCORE_VSLEEP_SEL_SHIFT},
	{PMIC_DA_NI_VSRAM_VCORE_R2R_PDN, PMIC_DA_NI_VSRAM_VCORE_R2R_PDN_ADDR,
	 PMIC_DA_NI_VSRAM_VCORE_R2R_PDN_MASK, PMIC_DA_NI_VSRAM_VCORE_R2R_PDN_SHIFT},
	{PMIC_DA_NI_VSRAM_VCORE_TRACK_NDIS_EN, PMIC_DA_NI_VSRAM_VCORE_TRACK_NDIS_EN_ADDR,
	 PMIC_DA_NI_VSRAM_VCORE_TRACK_NDIS_EN_MASK, PMIC_DA_NI_VSRAM_VCORE_TRACK_NDIS_EN_SHIFT},
	{PMIC_RG_VSRAM_VGPU_SW_EN, PMIC_RG_VSRAM_VGPU_SW_EN_ADDR, PMIC_RG_VSRAM_VGPU_SW_EN_MASK,
	 PMIC_RG_VSRAM_VGPU_SW_EN_SHIFT},
	{PMIC_RG_VSRAM_VGPU_SW_LP, PMIC_RG_VSRAM_VGPU_SW_LP_ADDR, PMIC_RG_VSRAM_VGPU_SW_LP_MASK,
	 PMIC_RG_VSRAM_VGPU_SW_LP_SHIFT},
	{PMIC_RG_VSRAM_VGPU_VOSEL, PMIC_RG_VSRAM_VGPU_VOSEL_ADDR, PMIC_RG_VSRAM_VGPU_VOSEL_MASK,
	 PMIC_RG_VSRAM_VGPU_VOSEL_SHIFT},
	{PMIC_RG_VSRAM_VGPU_VOSEL_SLEEP, PMIC_RG_VSRAM_VGPU_VOSEL_SLEEP_ADDR,
	 PMIC_RG_VSRAM_VGPU_VOSEL_SLEEP_MASK, PMIC_RG_VSRAM_VGPU_VOSEL_SLEEP_SHIFT},
	{PMIC_RG_VSRAM_VGPU_SFCHG_FRATE, PMIC_RG_VSRAM_VGPU_SFCHG_FRATE_ADDR,
	 PMIC_RG_VSRAM_VGPU_SFCHG_FRATE_MASK, PMIC_RG_VSRAM_VGPU_SFCHG_FRATE_SHIFT},
	{PMIC_RG_VSRAM_VGPU_SFCHG_FEN, PMIC_RG_VSRAM_VGPU_SFCHG_FEN_ADDR,
	 PMIC_RG_VSRAM_VGPU_SFCHG_FEN_MASK, PMIC_RG_VSRAM_VGPU_SFCHG_FEN_SHIFT},
	{PMIC_RG_VSRAM_VGPU_SFCHG_RRATE, PMIC_RG_VSRAM_VGPU_SFCHG_RRATE_ADDR,
	 PMIC_RG_VSRAM_VGPU_SFCHG_RRATE_MASK, PMIC_RG_VSRAM_VGPU_SFCHG_RRATE_SHIFT},
	{PMIC_RG_VSRAM_VGPU_SFCHG_REN, PMIC_RG_VSRAM_VGPU_SFCHG_REN_ADDR,
	 PMIC_RG_VSRAM_VGPU_SFCHG_REN_MASK, PMIC_RG_VSRAM_VGPU_SFCHG_REN_SHIFT},
	{PMIC_RG_VSRAM_VGPU_DVS_TRANS_TD, PMIC_RG_VSRAM_VGPU_DVS_TRANS_TD_ADDR,
	 PMIC_RG_VSRAM_VGPU_DVS_TRANS_TD_MASK, PMIC_RG_VSRAM_VGPU_DVS_TRANS_TD_SHIFT},
	{PMIC_RG_VSRAM_VGPU_DVS_TRANS_CTRL, PMIC_RG_VSRAM_VGPU_DVS_TRANS_CTRL_ADDR,
	 PMIC_RG_VSRAM_VGPU_DVS_TRANS_CTRL_MASK, PMIC_RG_VSRAM_VGPU_DVS_TRANS_CTRL_SHIFT},
	{PMIC_RG_VSRAM_VGPU_DVS_TRANS_ONCE, PMIC_RG_VSRAM_VGPU_DVS_TRANS_ONCE_ADDR,
	 PMIC_RG_VSRAM_VGPU_DVS_TRANS_ONCE_MASK, PMIC_RG_VSRAM_VGPU_DVS_TRANS_ONCE_SHIFT},
	{PMIC_RG_VSRAM_VGPU_SW_OP_EN, PMIC_RG_VSRAM_VGPU_SW_OP_EN_ADDR,
	 PMIC_RG_VSRAM_VGPU_SW_OP_EN_MASK, PMIC_RG_VSRAM_VGPU_SW_OP_EN_SHIFT},
	{PMIC_RG_VSRAM_VGPU_HW0_OP_EN, PMIC_RG_VSRAM_VGPU_HW0_OP_EN_ADDR,
	 PMIC_RG_VSRAM_VGPU_HW0_OP_EN_MASK, PMIC_RG_VSRAM_VGPU_HW0_OP_EN_SHIFT},
	{PMIC_RG_VSRAM_VGPU_HW1_OP_EN, PMIC_RG_VSRAM_VGPU_HW1_OP_EN_ADDR,
	 PMIC_RG_VSRAM_VGPU_HW1_OP_EN_MASK, PMIC_RG_VSRAM_VGPU_HW1_OP_EN_SHIFT},
	{PMIC_RG_VSRAM_VGPU_HW2_OP_EN, PMIC_RG_VSRAM_VGPU_HW2_OP_EN_ADDR,
	 PMIC_RG_VSRAM_VGPU_HW2_OP_EN_MASK, PMIC_RG_VSRAM_VGPU_HW2_OP_EN_SHIFT},
	{PMIC_RG_VSRAM_VGPU_OP_EN_SET, PMIC_RG_VSRAM_VGPU_OP_EN_SET_ADDR,
	 PMIC_RG_VSRAM_VGPU_OP_EN_SET_MASK, PMIC_RG_VSRAM_VGPU_OP_EN_SET_SHIFT},
	{PMIC_RG_VSRAM_VGPU_OP_EN_CLR, PMIC_RG_VSRAM_VGPU_OP_EN_CLR_ADDR,
	 PMIC_RG_VSRAM_VGPU_OP_EN_CLR_MASK, PMIC_RG_VSRAM_VGPU_OP_EN_CLR_SHIFT},
	{PMIC_RG_VSRAM_VGPU_HW0_OP_CFG, PMIC_RG_VSRAM_VGPU_HW0_OP_CFG_ADDR,
	 PMIC_RG_VSRAM_VGPU_HW0_OP_CFG_MASK, PMIC_RG_VSRAM_VGPU_HW0_OP_CFG_SHIFT},
	{PMIC_RG_VSRAM_VGPU_HW1_OP_CFG, PMIC_RG_VSRAM_VGPU_HW1_OP_CFG_ADDR,
	 PMIC_RG_VSRAM_VGPU_HW1_OP_CFG_MASK, PMIC_RG_VSRAM_VGPU_HW1_OP_CFG_SHIFT},
	{PMIC_RG_VSRAM_VGPU_HW2_OP_CFG, PMIC_RG_VSRAM_VGPU_HW2_OP_CFG_ADDR,
	 PMIC_RG_VSRAM_VGPU_HW2_OP_CFG_MASK, PMIC_RG_VSRAM_VGPU_HW2_OP_CFG_SHIFT},
	{PMIC_RG_VSRAM_VGPU_GO_ON_OP, PMIC_RG_VSRAM_VGPU_GO_ON_OP_ADDR,
	 PMIC_RG_VSRAM_VGPU_GO_ON_OP_MASK, PMIC_RG_VSRAM_VGPU_GO_ON_OP_SHIFT},
	{PMIC_RG_VSRAM_VGPU_GO_LP_OP, PMIC_RG_VSRAM_VGPU_GO_LP_OP_ADDR,
	 PMIC_RG_VSRAM_VGPU_GO_LP_OP_MASK, PMIC_RG_VSRAM_VGPU_GO_LP_OP_SHIFT},
	{PMIC_RG_VSRAM_VGPU_OP_CFG_SET, PMIC_RG_VSRAM_VGPU_OP_CFG_SET_ADDR,
	 PMIC_RG_VSRAM_VGPU_OP_CFG_SET_MASK, PMIC_RG_VSRAM_VGPU_OP_CFG_SET_SHIFT},
	{PMIC_RG_VSRAM_VGPU_OP_CFG_CLR, PMIC_RG_VSRAM_VGPU_OP_CFG_CLR_ADDR,
	 PMIC_RG_VSRAM_VGPU_OP_CFG_CLR_MASK, PMIC_RG_VSRAM_VGPU_OP_CFG_CLR_SHIFT},
	{PMIC_DA_QI_VSRAM_VGPU_MODE, PMIC_DA_QI_VSRAM_VGPU_MODE_ADDR,
	 PMIC_DA_QI_VSRAM_VGPU_MODE_MASK, PMIC_DA_QI_VSRAM_VGPU_MODE_SHIFT},
	{PMIC_RG_VSRAM_VGPU_STBTD, PMIC_RG_VSRAM_VGPU_STBTD_ADDR, PMIC_RG_VSRAM_VGPU_STBTD_MASK,
	 PMIC_RG_VSRAM_VGPU_STBTD_SHIFT},
	{PMIC_RG_VSRAM_VGPU_OCFB_EN, PMIC_RG_VSRAM_VGPU_OCFB_EN_ADDR,
	 PMIC_RG_VSRAM_VGPU_OCFB_EN_MASK, PMIC_RG_VSRAM_VGPU_OCFB_EN_SHIFT},
	{PMIC_DA_QI_VSRAM_VGPU_OCFB_EN, PMIC_DA_QI_VSRAM_VGPU_OCFB_EN_ADDR,
	 PMIC_DA_QI_VSRAM_VGPU_OCFB_EN_MASK, PMIC_DA_QI_VSRAM_VGPU_OCFB_EN_SHIFT},
	{PMIC_RG_VSRAM_VGPU_DUMMY_LOAD, PMIC_RG_VSRAM_VGPU_DUMMY_LOAD_ADDR,
	 PMIC_RG_VSRAM_VGPU_DUMMY_LOAD_MASK, PMIC_RG_VSRAM_VGPU_DUMMY_LOAD_SHIFT},
	{PMIC_DA_QI_VSRAM_VGPU_DUMMY_LOAD, PMIC_DA_QI_VSRAM_VGPU_DUMMY_LOAD_ADDR,
	 PMIC_DA_QI_VSRAM_VGPU_DUMMY_LOAD_MASK, PMIC_DA_QI_VSRAM_VGPU_DUMMY_LOAD_SHIFT},
	{PMIC_DA_NI_VSRAM_VGPU_VOSEL_GRAY, PMIC_DA_NI_VSRAM_VGPU_VOSEL_GRAY_ADDR,
	 PMIC_DA_NI_VSRAM_VGPU_VOSEL_GRAY_MASK, PMIC_DA_NI_VSRAM_VGPU_VOSEL_GRAY_SHIFT},
	{PMIC_DA_NI_VSRAM_VGPU_VOSEL, PMIC_DA_NI_VSRAM_VGPU_VOSEL_ADDR,
	 PMIC_DA_NI_VSRAM_VGPU_VOSEL_MASK, PMIC_DA_NI_VSRAM_VGPU_VOSEL_SHIFT},
	{PMIC_DA_QI_VSRAM_VGPU_EN, PMIC_DA_QI_VSRAM_VGPU_EN_ADDR, PMIC_DA_QI_VSRAM_VGPU_EN_MASK,
	 PMIC_DA_QI_VSRAM_VGPU_EN_SHIFT},
	{PMIC_DA_QI_VSRAM_VGPU_STB, PMIC_DA_QI_VSRAM_VGPU_STB_ADDR, PMIC_DA_QI_VSRAM_VGPU_STB_MASK,
	 PMIC_DA_QI_VSRAM_VGPU_STB_SHIFT},
	{PMIC_DA_NI_VSRAM_VGPU_VSLEEP_SEL, PMIC_DA_NI_VSRAM_VGPU_VSLEEP_SEL_ADDR,
	 PMIC_DA_NI_VSRAM_VGPU_VSLEEP_SEL_MASK, PMIC_DA_NI_VSRAM_VGPU_VSLEEP_SEL_SHIFT},
	{PMIC_DA_NI_VSRAM_VGPU_R2R_PDN, PMIC_DA_NI_VSRAM_VGPU_R2R_PDN_ADDR,
	 PMIC_DA_NI_VSRAM_VGPU_R2R_PDN_MASK, PMIC_DA_NI_VSRAM_VGPU_R2R_PDN_SHIFT},
	{PMIC_DA_NI_VSRAM_VGPU_TRACK_NDIS_EN, PMIC_DA_NI_VSRAM_VGPU_TRACK_NDIS_EN_ADDR,
	 PMIC_DA_NI_VSRAM_VGPU_TRACK_NDIS_EN_MASK, PMIC_DA_NI_VSRAM_VGPU_TRACK_NDIS_EN_SHIFT},
	{PMIC_RG_VSRAM_VMD_SW_EN, PMIC_RG_VSRAM_VMD_SW_EN_ADDR, PMIC_RG_VSRAM_VMD_SW_EN_MASK,
	 PMIC_RG_VSRAM_VMD_SW_EN_SHIFT},
	{PMIC_RG_VSRAM_VMD_SW_LP, PMIC_RG_VSRAM_VMD_SW_LP_ADDR, PMIC_RG_VSRAM_VMD_SW_LP_MASK,
	 PMIC_RG_VSRAM_VMD_SW_LP_SHIFT},
	{PMIC_RG_VSRAM_VMD_VOSEL, PMIC_RG_VSRAM_VMD_VOSEL_ADDR, PMIC_RG_VSRAM_VMD_VOSEL_MASK,
	 PMIC_RG_VSRAM_VMD_VOSEL_SHIFT},
	{PMIC_RG_VSRAM_VMD_VOSEL_SLEEP, PMIC_RG_VSRAM_VMD_VOSEL_SLEEP_ADDR,
	 PMIC_RG_VSRAM_VMD_VOSEL_SLEEP_MASK, PMIC_RG_VSRAM_VMD_VOSEL_SLEEP_SHIFT},
	{PMIC_RG_VSRAM_VMD_SFCHG_FRATE, PMIC_RG_VSRAM_VMD_SFCHG_FRATE_ADDR,
	 PMIC_RG_VSRAM_VMD_SFCHG_FRATE_MASK, PMIC_RG_VSRAM_VMD_SFCHG_FRATE_SHIFT},
	{PMIC_RG_VSRAM_VMD_SFCHG_FEN, PMIC_RG_VSRAM_VMD_SFCHG_FEN_ADDR,
	 PMIC_RG_VSRAM_VMD_SFCHG_FEN_MASK, PMIC_RG_VSRAM_VMD_SFCHG_FEN_SHIFT},
	{PMIC_RG_VSRAM_VMD_SFCHG_RRATE, PMIC_RG_VSRAM_VMD_SFCHG_RRATE_ADDR,
	 PMIC_RG_VSRAM_VMD_SFCHG_RRATE_MASK, PMIC_RG_VSRAM_VMD_SFCHG_RRATE_SHIFT},
	{PMIC_RG_VSRAM_VMD_SFCHG_REN, PMIC_RG_VSRAM_VMD_SFCHG_REN_ADDR,
	 PMIC_RG_VSRAM_VMD_SFCHG_REN_MASK, PMIC_RG_VSRAM_VMD_SFCHG_REN_SHIFT},
	{PMIC_RG_VSRAM_VMD_DVS_TRANS_TD, PMIC_RG_VSRAM_VMD_DVS_TRANS_TD_ADDR,
	 PMIC_RG_VSRAM_VMD_DVS_TRANS_TD_MASK, PMIC_RG_VSRAM_VMD_DVS_TRANS_TD_SHIFT},
	{PMIC_RG_VSRAM_VMD_DVS_TRANS_CTRL, PMIC_RG_VSRAM_VMD_DVS_TRANS_CTRL_ADDR,
	 PMIC_RG_VSRAM_VMD_DVS_TRANS_CTRL_MASK, PMIC_RG_VSRAM_VMD_DVS_TRANS_CTRL_SHIFT},
	{PMIC_RG_VSRAM_VMD_DVS_TRANS_ONCE, PMIC_RG_VSRAM_VMD_DVS_TRANS_ONCE_ADDR,
	 PMIC_RG_VSRAM_VMD_DVS_TRANS_ONCE_MASK, PMIC_RG_VSRAM_VMD_DVS_TRANS_ONCE_SHIFT},
	{PMIC_RG_VSRAM_VMD_SW_OP_EN, PMIC_RG_VSRAM_VMD_SW_OP_EN_ADDR,
	 PMIC_RG_VSRAM_VMD_SW_OP_EN_MASK, PMIC_RG_VSRAM_VMD_SW_OP_EN_SHIFT},
	{PMIC_RG_VSRAM_VMD_HW0_OP_EN, PMIC_RG_VSRAM_VMD_HW0_OP_EN_ADDR,
	 PMIC_RG_VSRAM_VMD_HW0_OP_EN_MASK, PMIC_RG_VSRAM_VMD_HW0_OP_EN_SHIFT},
	{PMIC_RG_VSRAM_VMD_HW1_OP_EN, PMIC_RG_VSRAM_VMD_HW1_OP_EN_ADDR,
	 PMIC_RG_VSRAM_VMD_HW1_OP_EN_MASK, PMIC_RG_VSRAM_VMD_HW1_OP_EN_SHIFT},
	{PMIC_RG_VSRAM_VMD_HW2_OP_EN, PMIC_RG_VSRAM_VMD_HW2_OP_EN_ADDR,
	 PMIC_RG_VSRAM_VMD_HW2_OP_EN_MASK, PMIC_RG_VSRAM_VMD_HW2_OP_EN_SHIFT},
	{PMIC_RG_VSRAM_VMD_OP_EN_SET, PMIC_RG_VSRAM_VMD_OP_EN_SET_ADDR,
	 PMIC_RG_VSRAM_VMD_OP_EN_SET_MASK, PMIC_RG_VSRAM_VMD_OP_EN_SET_SHIFT},
	{PMIC_RG_VSRAM_VMD_OP_EN_CLR, PMIC_RG_VSRAM_VMD_OP_EN_CLR_ADDR,
	 PMIC_RG_VSRAM_VMD_OP_EN_CLR_MASK, PMIC_RG_VSRAM_VMD_OP_EN_CLR_SHIFT},
	{PMIC_RG_VSRAM_VMD_HW0_OP_CFG, PMIC_RG_VSRAM_VMD_HW0_OP_CFG_ADDR,
	 PMIC_RG_VSRAM_VMD_HW0_OP_CFG_MASK, PMIC_RG_VSRAM_VMD_HW0_OP_CFG_SHIFT},
	{PMIC_RG_VSRAM_VMD_HW1_OP_CFG, PMIC_RG_VSRAM_VMD_HW1_OP_CFG_ADDR,
	 PMIC_RG_VSRAM_VMD_HW1_OP_CFG_MASK, PMIC_RG_VSRAM_VMD_HW1_OP_CFG_SHIFT},
	{PMIC_RG_VSRAM_VMD_HW2_OP_CFG, PMIC_RG_VSRAM_VMD_HW2_OP_CFG_ADDR,
	 PMIC_RG_VSRAM_VMD_HW2_OP_CFG_MASK, PMIC_RG_VSRAM_VMD_HW2_OP_CFG_SHIFT},
	{PMIC_RG_VSRAM_VMD_GO_ON_OP, PMIC_RG_VSRAM_VMD_GO_ON_OP_ADDR,
	 PMIC_RG_VSRAM_VMD_GO_ON_OP_MASK, PMIC_RG_VSRAM_VMD_GO_ON_OP_SHIFT},
	{PMIC_RG_VSRAM_VMD_GO_LP_OP, PMIC_RG_VSRAM_VMD_GO_LP_OP_ADDR,
	 PMIC_RG_VSRAM_VMD_GO_LP_OP_MASK, PMIC_RG_VSRAM_VMD_GO_LP_OP_SHIFT},
	{PMIC_RG_VSRAM_VMD_OP_CFG_SET, PMIC_RG_VSRAM_VMD_OP_CFG_SET_ADDR,
	 PMIC_RG_VSRAM_VMD_OP_CFG_SET_MASK, PMIC_RG_VSRAM_VMD_OP_CFG_SET_SHIFT},
	{PMIC_RG_VSRAM_VMD_OP_CFG_CLR, PMIC_RG_VSRAM_VMD_OP_CFG_CLR_ADDR,
	 PMIC_RG_VSRAM_VMD_OP_CFG_CLR_MASK, PMIC_RG_VSRAM_VMD_OP_CFG_CLR_SHIFT},
	{PMIC_DA_QI_VSRAM_VMD_MODE, PMIC_DA_QI_VSRAM_VMD_MODE_ADDR, PMIC_DA_QI_VSRAM_VMD_MODE_MASK,
	 PMIC_DA_QI_VSRAM_VMD_MODE_SHIFT},
	{PMIC_RG_VSRAM_VMD_STBTD, PMIC_RG_VSRAM_VMD_STBTD_ADDR, PMIC_RG_VSRAM_VMD_STBTD_MASK,
	 PMIC_RG_VSRAM_VMD_STBTD_SHIFT},
	{PMIC_RG_VSRAM_VMD_OCFB_EN, PMIC_RG_VSRAM_VMD_OCFB_EN_ADDR, PMIC_RG_VSRAM_VMD_OCFB_EN_MASK,
	 PMIC_RG_VSRAM_VMD_OCFB_EN_SHIFT},
	{PMIC_DA_QI_VSRAM_VMD_OCFB_EN, PMIC_DA_QI_VSRAM_VMD_OCFB_EN_ADDR,
	 PMIC_DA_QI_VSRAM_VMD_OCFB_EN_MASK, PMIC_DA_QI_VSRAM_VMD_OCFB_EN_SHIFT},
	{PMIC_RG_VSRAM_VMD_DUMMY_LOAD, PMIC_RG_VSRAM_VMD_DUMMY_LOAD_ADDR,
	 PMIC_RG_VSRAM_VMD_DUMMY_LOAD_MASK, PMIC_RG_VSRAM_VMD_DUMMY_LOAD_SHIFT},
	{PMIC_DA_QI_VSRAM_VMD_DUMMY_LOAD, PMIC_DA_QI_VSRAM_VMD_DUMMY_LOAD_ADDR,
	 PMIC_DA_QI_VSRAM_VMD_DUMMY_LOAD_MASK, PMIC_DA_QI_VSRAM_VMD_DUMMY_LOAD_SHIFT},
	{PMIC_DA_NI_VSRAM_VMD_VOSEL_GRAY, PMIC_DA_NI_VSRAM_VMD_VOSEL_GRAY_ADDR,
	 PMIC_DA_NI_VSRAM_VMD_VOSEL_GRAY_MASK, PMIC_DA_NI_VSRAM_VMD_VOSEL_GRAY_SHIFT},
	{PMIC_DA_NI_VSRAM_VMD_VOSEL, PMIC_DA_NI_VSRAM_VMD_VOSEL_ADDR,
	 PMIC_DA_NI_VSRAM_VMD_VOSEL_MASK, PMIC_DA_NI_VSRAM_VMD_VOSEL_SHIFT},
	{PMIC_DA_QI_VSRAM_VMD_EN, PMIC_DA_QI_VSRAM_VMD_EN_ADDR, PMIC_DA_QI_VSRAM_VMD_EN_MASK,
	 PMIC_DA_QI_VSRAM_VMD_EN_SHIFT},
	{PMIC_DA_QI_VSRAM_VMD_STB, PMIC_DA_QI_VSRAM_VMD_STB_ADDR, PMIC_DA_QI_VSRAM_VMD_STB_MASK,
	 PMIC_DA_QI_VSRAM_VMD_STB_SHIFT},
	{PMIC_DA_NI_VSRAM_VMD_VSLEEP_SEL, PMIC_DA_NI_VSRAM_VMD_VSLEEP_SEL_ADDR,
	 PMIC_DA_NI_VSRAM_VMD_VSLEEP_SEL_MASK, PMIC_DA_NI_VSRAM_VMD_VSLEEP_SEL_SHIFT},
	{PMIC_DA_NI_VSRAM_VMD_R2R_PDN, PMIC_DA_NI_VSRAM_VMD_R2R_PDN_ADDR,
	 PMIC_DA_NI_VSRAM_VMD_R2R_PDN_MASK, PMIC_DA_NI_VSRAM_VMD_R2R_PDN_SHIFT},
	{PMIC_DA_NI_VSRAM_VMD_TRACK_NDIS_EN, PMIC_DA_NI_VSRAM_VMD_TRACK_NDIS_EN_ADDR,
	 PMIC_DA_NI_VSRAM_VMD_TRACK_NDIS_EN_MASK, PMIC_DA_NI_VSRAM_VMD_TRACK_NDIS_EN_SHIFT},
	{PMIC_RG_VSRAM_VCORE_TRACK_SLEEP_CTRL, PMIC_RG_VSRAM_VCORE_TRACK_SLEEP_CTRL_ADDR,
	 PMIC_RG_VSRAM_VCORE_TRACK_SLEEP_CTRL_MASK, PMIC_RG_VSRAM_VCORE_TRACK_SLEEP_CTRL_SHIFT},
	{PMIC_RG_VSRAM_VCORE_TRACK_ON_CTRL, PMIC_RG_VSRAM_VCORE_TRACK_ON_CTRL_ADDR,
	 PMIC_RG_VSRAM_VCORE_TRACK_ON_CTRL_MASK, PMIC_RG_VSRAM_VCORE_TRACK_ON_CTRL_SHIFT},
	{PMIC_RG_VSRAM_VCORE_TRACK_VBUCK_ON_CTRL, PMIC_RG_VSRAM_VCORE_TRACK_VBUCK_ON_CTRL_ADDR,
	 PMIC_RG_VSRAM_VCORE_TRACK_VBUCK_ON_CTRL_MASK,
	 PMIC_RG_VSRAM_VCORE_TRACK_VBUCK_ON_CTRL_SHIFT},
	{PMIC_RG_VSRAM_VCORE_VOSEL_DELTA, PMIC_RG_VSRAM_VCORE_VOSEL_DELTA_ADDR,
	 PMIC_RG_VSRAM_VCORE_VOSEL_DELTA_MASK, PMIC_RG_VSRAM_VCORE_VOSEL_DELTA_SHIFT},
	{PMIC_RG_VSRAM_VCORE_VOSEL_OFFSET, PMIC_RG_VSRAM_VCORE_VOSEL_OFFSET_ADDR,
	 PMIC_RG_VSRAM_VCORE_VOSEL_OFFSET_MASK, PMIC_RG_VSRAM_VCORE_VOSEL_OFFSET_SHIFT},
	{PMIC_RG_VSRAM_VCORE_VOSEL_ON_LB, PMIC_RG_VSRAM_VCORE_VOSEL_ON_LB_ADDR,
	 PMIC_RG_VSRAM_VCORE_VOSEL_ON_LB_MASK, PMIC_RG_VSRAM_VCORE_VOSEL_ON_LB_SHIFT},
	{PMIC_RG_VSRAM_VCORE_VOSEL_ON_HB, PMIC_RG_VSRAM_VCORE_VOSEL_ON_HB_ADDR,
	 PMIC_RG_VSRAM_VCORE_VOSEL_ON_HB_MASK, PMIC_RG_VSRAM_VCORE_VOSEL_ON_HB_SHIFT},
	{PMIC_RG_VSRAM_VCORE_VOSEL_SLEEP_LB, PMIC_RG_VSRAM_VCORE_VOSEL_SLEEP_LB_ADDR,
	 PMIC_RG_VSRAM_VCORE_VOSEL_SLEEP_LB_MASK, PMIC_RG_VSRAM_VCORE_VOSEL_SLEEP_LB_SHIFT},
	{PMIC_RG_DCM_MODE, PMIC_RG_DCM_MODE_ADDR, PMIC_RG_DCM_MODE_MASK, PMIC_RG_DCM_MODE_SHIFT},
	{PMIC_RG_VIO28_CK_SW_MODE, PMIC_RG_VIO28_CK_SW_MODE_ADDR, PMIC_RG_VIO28_CK_SW_MODE_MASK,
	 PMIC_RG_VIO28_CK_SW_MODE_SHIFT},
	{PMIC_RG_VIO28_CK_SW_EN, PMIC_RG_VIO28_CK_SW_EN_ADDR, PMIC_RG_VIO28_CK_SW_EN_MASK,
	 PMIC_RG_VIO28_CK_SW_EN_SHIFT},
	{PMIC_RG_VIO28_OSC_SEL_DIS, PMIC_RG_VIO28_OSC_SEL_DIS_ADDR, PMIC_RG_VIO28_OSC_SEL_DIS_MASK,
	 PMIC_RG_VIO28_OSC_SEL_DIS_SHIFT},
	{PMIC_RG_VIO18_CK_SW_MODE, PMIC_RG_VIO18_CK_SW_MODE_ADDR, PMIC_RG_VIO18_CK_SW_MODE_MASK,
	 PMIC_RG_VIO18_CK_SW_MODE_SHIFT},
	{PMIC_RG_VIO18_CK_SW_EN, PMIC_RG_VIO18_CK_SW_EN_ADDR, PMIC_RG_VIO18_CK_SW_EN_MASK,
	 PMIC_RG_VIO18_CK_SW_EN_SHIFT},
	{PMIC_RG_VIO18_OSC_SEL_DIS, PMIC_RG_VIO18_OSC_SEL_DIS_ADDR, PMIC_RG_VIO18_OSC_SEL_DIS_MASK,
	 PMIC_RG_VIO18_OSC_SEL_DIS_SHIFT},
	{PMIC_RG_VUFS18_CK_SW_MODE, PMIC_RG_VUFS18_CK_SW_MODE_ADDR, PMIC_RG_VUFS18_CK_SW_MODE_MASK,
	 PMIC_RG_VUFS18_CK_SW_MODE_SHIFT},
	{PMIC_RG_VUFS18_CK_SW_EN, PMIC_RG_VUFS18_CK_SW_EN_ADDR, PMIC_RG_VUFS18_CK_SW_EN_MASK,
	 PMIC_RG_VUFS18_CK_SW_EN_SHIFT},
	{PMIC_RG_VUFS18_OSC_SEL_DIS, PMIC_RG_VUFS18_OSC_SEL_DIS_ADDR,
	 PMIC_RG_VUFS18_OSC_SEL_DIS_MASK, PMIC_RG_VUFS18_OSC_SEL_DIS_SHIFT},
	{PMIC_RG_VA10_CK_SW_MODE, PMIC_RG_VA10_CK_SW_MODE_ADDR, PMIC_RG_VA10_CK_SW_MODE_MASK,
	 PMIC_RG_VA10_CK_SW_MODE_SHIFT},
	{PMIC_RG_VA10_CK_SW_EN, PMIC_RG_VA10_CK_SW_EN_ADDR, PMIC_RG_VA10_CK_SW_EN_MASK,
	 PMIC_RG_VA10_CK_SW_EN_SHIFT},
	{PMIC_RG_VA10_OSC_SEL_DIS, PMIC_RG_VA10_OSC_SEL_DIS_ADDR, PMIC_RG_VA10_OSC_SEL_DIS_MASK,
	 PMIC_RG_VA10_OSC_SEL_DIS_SHIFT},
	{PMIC_RG_VA12_CK_SW_MODE, PMIC_RG_VA12_CK_SW_MODE_ADDR, PMIC_RG_VA12_CK_SW_MODE_MASK,
	 PMIC_RG_VA12_CK_SW_MODE_SHIFT},
	{PMIC_RG_VA12_CK_SW_EN, PMIC_RG_VA12_CK_SW_EN_ADDR, PMIC_RG_VA12_CK_SW_EN_MASK,
	 PMIC_RG_VA12_CK_SW_EN_SHIFT},
	{PMIC_RG_VA12_OSC_SEL_DIS, PMIC_RG_VA12_OSC_SEL_DIS_ADDR, PMIC_RG_VA12_OSC_SEL_DIS_MASK,
	 PMIC_RG_VA12_OSC_SEL_DIS_SHIFT},
	{PMIC_RG_VSRAM_DVFS1_CK_SW_MODE, PMIC_RG_VSRAM_DVFS1_CK_SW_MODE_ADDR,
	 PMIC_RG_VSRAM_DVFS1_CK_SW_MODE_MASK, PMIC_RG_VSRAM_DVFS1_CK_SW_MODE_SHIFT},
	{PMIC_RG_VSRAM_DVFS1_CK_SW_EN, PMIC_RG_VSRAM_DVFS1_CK_SW_EN_ADDR,
	 PMIC_RG_VSRAM_DVFS1_CK_SW_EN_MASK, PMIC_RG_VSRAM_DVFS1_CK_SW_EN_SHIFT},
	{PMIC_RG_VSRAM_DVFS1_OSC_SEL_DIS, PMIC_RG_VSRAM_DVFS1_OSC_SEL_DIS_ADDR,
	 PMIC_RG_VSRAM_DVFS1_OSC_SEL_DIS_MASK, PMIC_RG_VSRAM_DVFS1_OSC_SEL_DIS_SHIFT},
	{PMIC_RG_VSRAM_DVFS2_CK_SW_MODE, PMIC_RG_VSRAM_DVFS2_CK_SW_MODE_ADDR,
	 PMIC_RG_VSRAM_DVFS2_CK_SW_MODE_MASK, PMIC_RG_VSRAM_DVFS2_CK_SW_MODE_SHIFT},
	{PMIC_RG_VSRAM_DVFS2_CK_SW_EN, PMIC_RG_VSRAM_DVFS2_CK_SW_EN_ADDR,
	 PMIC_RG_VSRAM_DVFS2_CK_SW_EN_MASK, PMIC_RG_VSRAM_DVFS2_CK_SW_EN_SHIFT},
	{PMIC_RG_VSRAM_DVFS2_OSC_SEL_DIS, PMIC_RG_VSRAM_DVFS2_OSC_SEL_DIS_ADDR,
	 PMIC_RG_VSRAM_DVFS2_OSC_SEL_DIS_MASK, PMIC_RG_VSRAM_DVFS2_OSC_SEL_DIS_SHIFT},
	{PMIC_RG_VSRAM_VCORE_CK_SW_MODE, PMIC_RG_VSRAM_VCORE_CK_SW_MODE_ADDR,
	 PMIC_RG_VSRAM_VCORE_CK_SW_MODE_MASK, PMIC_RG_VSRAM_VCORE_CK_SW_MODE_SHIFT},
	{PMIC_RG_VSRAM_VCORE_CK_SW_EN, PMIC_RG_VSRAM_VCORE_CK_SW_EN_ADDR,
	 PMIC_RG_VSRAM_VCORE_CK_SW_EN_MASK, PMIC_RG_VSRAM_VCORE_CK_SW_EN_SHIFT},
	{PMIC_RG_VSRAM_VCORE_OSC_SEL_DIS, PMIC_RG_VSRAM_VCORE_OSC_SEL_DIS_ADDR,
	 PMIC_RG_VSRAM_VCORE_OSC_SEL_DIS_MASK, PMIC_RG_VSRAM_VCORE_OSC_SEL_DIS_SHIFT},
	{PMIC_RG_VSRAM_VGPU_CK_SW_MODE, PMIC_RG_VSRAM_VGPU_CK_SW_MODE_ADDR,
	 PMIC_RG_VSRAM_VGPU_CK_SW_MODE_MASK, PMIC_RG_VSRAM_VGPU_CK_SW_MODE_SHIFT},
	{PMIC_RG_VSRAM_VGPU_CK_SW_EN, PMIC_RG_VSRAM_VGPU_CK_SW_EN_ADDR,
	 PMIC_RG_VSRAM_VGPU_CK_SW_EN_MASK, PMIC_RG_VSRAM_VGPU_CK_SW_EN_SHIFT},
	{PMIC_RG_VSRAM_VGPU_OSC_SEL_DIS, PMIC_RG_VSRAM_VGPU_OSC_SEL_DIS_ADDR,
	 PMIC_RG_VSRAM_VGPU_OSC_SEL_DIS_MASK, PMIC_RG_VSRAM_VGPU_OSC_SEL_DIS_SHIFT},
	{PMIC_RG_VSRAM_VMD_CK_SW_MODE, PMIC_RG_VSRAM_VMD_CK_SW_MODE_ADDR,
	 PMIC_RG_VSRAM_VMD_CK_SW_MODE_MASK, PMIC_RG_VSRAM_VMD_CK_SW_MODE_SHIFT},
	{PMIC_RG_VSRAM_VMD_CK_SW_EN, PMIC_RG_VSRAM_VMD_CK_SW_EN_ADDR,
	 PMIC_RG_VSRAM_VMD_CK_SW_EN_MASK, PMIC_RG_VSRAM_VMD_CK_SW_EN_SHIFT},
	{PMIC_RG_VSRAM_VMD_OSC_SEL_DIS, PMIC_RG_VSRAM_VMD_OSC_SEL_DIS_ADDR,
	 PMIC_RG_VSRAM_VMD_OSC_SEL_DIS_MASK, PMIC_RG_VSRAM_VMD_OSC_SEL_DIS_SHIFT},
	{PMIC_RG_VA18_CK_SW_MODE, PMIC_RG_VA18_CK_SW_MODE_ADDR, PMIC_RG_VA18_CK_SW_MODE_MASK,
	 PMIC_RG_VA18_CK_SW_MODE_SHIFT},
	{PMIC_RG_VA18_CK_SW_EN, PMIC_RG_VA18_CK_SW_EN_ADDR, PMIC_RG_VA18_CK_SW_EN_MASK,
	 PMIC_RG_VA18_CK_SW_EN_SHIFT},
	{PMIC_RG_VA18_OSC_SEL_DIS, PMIC_RG_VA18_OSC_SEL_DIS_ADDR, PMIC_RG_VA18_OSC_SEL_DIS_MASK,
	 PMIC_RG_VA18_OSC_SEL_DIS_SHIFT},
	{PMIC_RG_VUSB33_CK_SW_MODE, PMIC_RG_VUSB33_CK_SW_MODE_ADDR, PMIC_RG_VUSB33_CK_SW_MODE_MASK,
	 PMIC_RG_VUSB33_CK_SW_MODE_SHIFT},
	{PMIC_RG_VUSB33_CK_SW_EN, PMIC_RG_VUSB33_CK_SW_EN_ADDR, PMIC_RG_VUSB33_CK_SW_EN_MASK,
	 PMIC_RG_VUSB33_CK_SW_EN_SHIFT},
	{PMIC_RG_VUSB33_OSC_SEL_DIS, PMIC_RG_VUSB33_OSC_SEL_DIS_ADDR,
	 PMIC_RG_VUSB33_OSC_SEL_DIS_MASK, PMIC_RG_VUSB33_OSC_SEL_DIS_SHIFT},
	{PMIC_RG_VEMC_CK_SW_MODE, PMIC_RG_VEMC_CK_SW_MODE_ADDR, PMIC_RG_VEMC_CK_SW_MODE_MASK,
	 PMIC_RG_VEMC_CK_SW_MODE_SHIFT},
	{PMIC_RG_VEMC_CK_SW_EN, PMIC_RG_VEMC_CK_SW_EN_ADDR, PMIC_RG_VEMC_CK_SW_EN_MASK,
	 PMIC_RG_VEMC_CK_SW_EN_SHIFT},
	{PMIC_RG_VEMC_OSC_SEL_DIS, PMIC_RG_VEMC_OSC_SEL_DIS_ADDR, PMIC_RG_VEMC_OSC_SEL_DIS_MASK,
	 PMIC_RG_VEMC_OSC_SEL_DIS_SHIFT},
	{PMIC_RG_VXO22_CK_SW_MODE, PMIC_RG_VXO22_CK_SW_MODE_ADDR, PMIC_RG_VXO22_CK_SW_MODE_MASK,
	 PMIC_RG_VXO22_CK_SW_MODE_SHIFT},
	{PMIC_RG_VXO22_CK_SW_EN, PMIC_RG_VXO22_CK_SW_EN_ADDR, PMIC_RG_VXO22_CK_SW_EN_MASK,
	 PMIC_RG_VXO22_CK_SW_EN_SHIFT},
	{PMIC_RG_VXO22_OSC_SEL_DIS, PMIC_RG_VXO22_OSC_SEL_DIS_ADDR, PMIC_RG_VXO22_OSC_SEL_DIS_MASK,
	 PMIC_RG_VXO22_OSC_SEL_DIS_SHIFT},
	{PMIC_RG_VEFUSE_CK_SW_MODE, PMIC_RG_VEFUSE_CK_SW_MODE_ADDR, PMIC_RG_VEFUSE_CK_SW_MODE_MASK,
	 PMIC_RG_VEFUSE_CK_SW_MODE_SHIFT},
	{PMIC_RG_VEFUSE_CK_SW_EN, PMIC_RG_VEFUSE_CK_SW_EN_ADDR, PMIC_RG_VEFUSE_CK_SW_EN_MASK,
	 PMIC_RG_VEFUSE_CK_SW_EN_SHIFT},
	{PMIC_RG_VEFUSE_OSC_SEL_DIS, PMIC_RG_VEFUSE_OSC_SEL_DIS_ADDR,
	 PMIC_RG_VEFUSE_OSC_SEL_DIS_MASK, PMIC_RG_VEFUSE_OSC_SEL_DIS_SHIFT},
	{PMIC_RG_VSIM1_CK_SW_MODE, PMIC_RG_VSIM1_CK_SW_MODE_ADDR, PMIC_RG_VSIM1_CK_SW_MODE_MASK,
	 PMIC_RG_VSIM1_CK_SW_MODE_SHIFT},
	{PMIC_RG_VSIM1_CK_SW_EN, PMIC_RG_VSIM1_CK_SW_EN_ADDR, PMIC_RG_VSIM1_CK_SW_EN_MASK,
	 PMIC_RG_VSIM1_CK_SW_EN_SHIFT},
	{PMIC_RG_VSIM1_OSC_SEL_DIS, PMIC_RG_VSIM1_OSC_SEL_DIS_ADDR, PMIC_RG_VSIM1_OSC_SEL_DIS_MASK,
	 PMIC_RG_VSIM1_OSC_SEL_DIS_SHIFT},
	{PMIC_RG_VSIM2_CK_SW_MODE, PMIC_RG_VSIM2_CK_SW_MODE_ADDR, PMIC_RG_VSIM2_CK_SW_MODE_MASK,
	 PMIC_RG_VSIM2_CK_SW_MODE_SHIFT},
	{PMIC_RG_VSIM2_CK_SW_EN, PMIC_RG_VSIM2_CK_SW_EN_ADDR, PMIC_RG_VSIM2_CK_SW_EN_MASK,
	 PMIC_RG_VSIM2_CK_SW_EN_SHIFT},
	{PMIC_RG_VSIM2_OSC_SEL_DIS, PMIC_RG_VSIM2_OSC_SEL_DIS_ADDR, PMIC_RG_VSIM2_OSC_SEL_DIS_MASK,
	 PMIC_RG_VSIM2_OSC_SEL_DIS_SHIFT},
	{PMIC_RG_VCAMAF_CK_SW_MODE, PMIC_RG_VCAMAF_CK_SW_MODE_ADDR, PMIC_RG_VCAMAF_CK_SW_MODE_MASK,
	 PMIC_RG_VCAMAF_CK_SW_MODE_SHIFT},
	{PMIC_RG_VCAMAF_CK_SW_EN, PMIC_RG_VCAMAF_CK_SW_EN_ADDR, PMIC_RG_VCAMAF_CK_SW_EN_MASK,
	 PMIC_RG_VCAMAF_CK_SW_EN_SHIFT},
	{PMIC_RG_VCAMAF_OSC_SEL_DIS, PMIC_RG_VCAMAF_OSC_SEL_DIS_ADDR,
	 PMIC_RG_VCAMAF_OSC_SEL_DIS_MASK, PMIC_RG_VCAMAF_OSC_SEL_DIS_SHIFT},
	{PMIC_RG_VTOUCH_CK_SW_MODE, PMIC_RG_VTOUCH_CK_SW_MODE_ADDR, PMIC_RG_VTOUCH_CK_SW_MODE_MASK,
	 PMIC_RG_VTOUCH_CK_SW_MODE_SHIFT},
	{PMIC_RG_VTOUCH_CK_SW_EN, PMIC_RG_VTOUCH_CK_SW_EN_ADDR, PMIC_RG_VTOUCH_CK_SW_EN_MASK,
	 PMIC_RG_VTOUCH_CK_SW_EN_SHIFT},
	{PMIC_RG_VTOUCH_OSC_SEL_DIS, PMIC_RG_VTOUCH_OSC_SEL_DIS_ADDR,
	 PMIC_RG_VTOUCH_OSC_SEL_DIS_MASK, PMIC_RG_VTOUCH_OSC_SEL_DIS_SHIFT},
	{PMIC_RG_VCAMD1_CK_SW_MODE, PMIC_RG_VCAMD1_CK_SW_MODE_ADDR, PMIC_RG_VCAMD1_CK_SW_MODE_MASK,
	 PMIC_RG_VCAMD1_CK_SW_MODE_SHIFT},
	{PMIC_RG_VCAMD1_CK_SW_EN, PMIC_RG_VCAMD1_CK_SW_EN_ADDR, PMIC_RG_VCAMD1_CK_SW_EN_MASK,
	 PMIC_RG_VCAMD1_CK_SW_EN_SHIFT},
	{PMIC_RG_VCAMD1_OSC_SEL_DIS, PMIC_RG_VCAMD1_OSC_SEL_DIS_ADDR,
	 PMIC_RG_VCAMD1_OSC_SEL_DIS_MASK, PMIC_RG_VCAMD1_OSC_SEL_DIS_SHIFT},
	{PMIC_RG_VCAMD2_CK_SW_MODE, PMIC_RG_VCAMD2_CK_SW_MODE_ADDR, PMIC_RG_VCAMD2_CK_SW_MODE_MASK,
	 PMIC_RG_VCAMD2_CK_SW_MODE_SHIFT},
	{PMIC_RG_VCAMD2_CK_SW_EN, PMIC_RG_VCAMD2_CK_SW_EN_ADDR, PMIC_RG_VCAMD2_CK_SW_EN_MASK,
	 PMIC_RG_VCAMD2_CK_SW_EN_SHIFT},
	{PMIC_RG_VCAMD2_OSC_SEL_DIS, PMIC_RG_VCAMD2_OSC_SEL_DIS_ADDR,
	 PMIC_RG_VCAMD2_OSC_SEL_DIS_MASK, PMIC_RG_VCAMD2_OSC_SEL_DIS_SHIFT},
	{PMIC_RG_VCAMIO_CK_SW_MODE, PMIC_RG_VCAMIO_CK_SW_MODE_ADDR, PMIC_RG_VCAMIO_CK_SW_MODE_MASK,
	 PMIC_RG_VCAMIO_CK_SW_MODE_SHIFT},
	{PMIC_RG_VCAMIO_CK_SW_EN, PMIC_RG_VCAMIO_CK_SW_EN_ADDR, PMIC_RG_VCAMIO_CK_SW_EN_MASK,
	 PMIC_RG_VCAMIO_CK_SW_EN_SHIFT},
	{PMIC_RG_VCAMIO_OSC_SEL_DIS, PMIC_RG_VCAMIO_OSC_SEL_DIS_ADDR,
	 PMIC_RG_VCAMIO_OSC_SEL_DIS_MASK, PMIC_RG_VCAMIO_OSC_SEL_DIS_SHIFT},
	{PMIC_RG_VMIPI_CK_SW_MODE, PMIC_RG_VMIPI_CK_SW_MODE_ADDR, PMIC_RG_VMIPI_CK_SW_MODE_MASK,
	 PMIC_RG_VMIPI_CK_SW_MODE_SHIFT},
	{PMIC_RG_VMIPI_CK_SW_EN, PMIC_RG_VMIPI_CK_SW_EN_ADDR, PMIC_RG_VMIPI_CK_SW_EN_MASK,
	 PMIC_RG_VMIPI_CK_SW_EN_SHIFT},
	{PMIC_RG_VMIPI_OSC_SEL_DIS, PMIC_RG_VMIPI_OSC_SEL_DIS_ADDR, PMIC_RG_VMIPI_OSC_SEL_DIS_MASK,
	 PMIC_RG_VMIPI_OSC_SEL_DIS_SHIFT},
	{PMIC_RG_VGP3_CK_SW_MODE, PMIC_RG_VGP3_CK_SW_MODE_ADDR, PMIC_RG_VGP3_CK_SW_MODE_MASK,
	 PMIC_RG_VGP3_CK_SW_MODE_SHIFT},
	{PMIC_RG_VGP3_CK_SW_EN, PMIC_RG_VGP3_CK_SW_EN_ADDR, PMIC_RG_VGP3_CK_SW_EN_MASK,
	 PMIC_RG_VGP3_CK_SW_EN_SHIFT},
	{PMIC_RG_VGP3_OSC_SEL_DIS, PMIC_RG_VGP3_OSC_SEL_DIS_ADDR, PMIC_RG_VGP3_OSC_SEL_DIS_MASK,
	 PMIC_RG_VGP3_OSC_SEL_DIS_SHIFT},
	{PMIC_RG_VCN33_CK_SW_MODE, PMIC_RG_VCN33_CK_SW_MODE_ADDR, PMIC_RG_VCN33_CK_SW_MODE_MASK,
	 PMIC_RG_VCN33_CK_SW_MODE_SHIFT},
	{PMIC_RG_VCN33_CK_SW_EN, PMIC_RG_VCN33_CK_SW_EN_ADDR, PMIC_RG_VCN33_CK_SW_EN_MASK,
	 PMIC_RG_VCN33_CK_SW_EN_SHIFT},
	{PMIC_RG_VCN33_OSC_SEL_DIS, PMIC_RG_VCN33_OSC_SEL_DIS_ADDR, PMIC_RG_VCN33_OSC_SEL_DIS_MASK,
	 PMIC_RG_VCN33_OSC_SEL_DIS_SHIFT},
	{PMIC_RG_VCN18_CK_SW_MODE, PMIC_RG_VCN18_CK_SW_MODE_ADDR, PMIC_RG_VCN18_CK_SW_MODE_MASK,
	 PMIC_RG_VCN18_CK_SW_MODE_SHIFT},
	{PMIC_RG_VCN18_CK_SW_EN, PMIC_RG_VCN18_CK_SW_EN_ADDR, PMIC_RG_VCN18_CK_SW_EN_MASK,
	 PMIC_RG_VCN18_CK_SW_EN_SHIFT},
	{PMIC_RG_VCN18_OSC_SEL_DIS, PMIC_RG_VCN18_OSC_SEL_DIS_ADDR, PMIC_RG_VCN18_OSC_SEL_DIS_MASK,
	 PMIC_RG_VCN18_OSC_SEL_DIS_SHIFT},
	{PMIC_RG_VCN28_CK_SW_MODE, PMIC_RG_VCN28_CK_SW_MODE_ADDR, PMIC_RG_VCN28_CK_SW_MODE_MASK,
	 PMIC_RG_VCN28_CK_SW_MODE_SHIFT},
	{PMIC_RG_VCN28_CK_SW_EN, PMIC_RG_VCN28_CK_SW_EN_ADDR, PMIC_RG_VCN28_CK_SW_EN_MASK,
	 PMIC_RG_VCN28_CK_SW_EN_SHIFT},
	{PMIC_RG_VCN28_OSC_SEL_DIS, PMIC_RG_VCN28_OSC_SEL_DIS_ADDR, PMIC_RG_VCN28_OSC_SEL_DIS_MASK,
	 PMIC_RG_VCN28_OSC_SEL_DIS_SHIFT},
	{PMIC_RG_VIBR_CK_SW_MODE, PMIC_RG_VIBR_CK_SW_MODE_ADDR, PMIC_RG_VIBR_CK_SW_MODE_MASK,
	 PMIC_RG_VIBR_CK_SW_MODE_SHIFT},
	{PMIC_RG_VIBR_CK_SW_EN, PMIC_RG_VIBR_CK_SW_EN_ADDR, PMIC_RG_VIBR_CK_SW_EN_MASK,
	 PMIC_RG_VIBR_CK_SW_EN_SHIFT},
	{PMIC_RG_VIBR_OSC_SEL_DIS, PMIC_RG_VIBR_OSC_SEL_DIS_ADDR, PMIC_RG_VIBR_OSC_SEL_DIS_MASK,
	 PMIC_RG_VIBR_OSC_SEL_DIS_SHIFT},
	{PMIC_RG_VBIF28_CK_SW_MODE, PMIC_RG_VBIF28_CK_SW_MODE_ADDR, PMIC_RG_VBIF28_CK_SW_MODE_MASK,
	 PMIC_RG_VBIF28_CK_SW_MODE_SHIFT},
	{PMIC_RG_VBIF28_CK_SW_EN, PMIC_RG_VBIF28_CK_SW_EN_ADDR, PMIC_RG_VBIF28_CK_SW_EN_MASK,
	 PMIC_RG_VBIF28_CK_SW_EN_SHIFT},
	{PMIC_RG_VBIF28_OSC_SEL_DIS, PMIC_RG_VBIF28_OSC_SEL_DIS_ADDR,
	 PMIC_RG_VBIF28_OSC_SEL_DIS_MASK, PMIC_RG_VBIF28_OSC_SEL_DIS_SHIFT},
	{PMIC_RG_VFE28_CK_SW_MODE, PMIC_RG_VFE28_CK_SW_MODE_ADDR, PMIC_RG_VFE28_CK_SW_MODE_MASK,
	 PMIC_RG_VFE28_CK_SW_MODE_SHIFT},
	{PMIC_RG_VFE28_CK_SW_EN, PMIC_RG_VFE28_CK_SW_EN_ADDR, PMIC_RG_VFE28_CK_SW_EN_MASK,
	 PMIC_RG_VFE28_CK_SW_EN_SHIFT},
	{PMIC_RG_VFE28_OSC_SEL_DIS, PMIC_RG_VFE28_OSC_SEL_DIS_ADDR, PMIC_RG_VFE28_OSC_SEL_DIS_MASK,
	 PMIC_RG_VFE28_OSC_SEL_DIS_SHIFT},
	{PMIC_RG_VMCH_CK_SW_MODE, PMIC_RG_VMCH_CK_SW_MODE_ADDR, PMIC_RG_VMCH_CK_SW_MODE_MASK,
	 PMIC_RG_VMCH_CK_SW_MODE_SHIFT},
	{PMIC_RG_VMCH_CK_SW_EN, PMIC_RG_VMCH_CK_SW_EN_ADDR, PMIC_RG_VMCH_CK_SW_EN_MASK,
	 PMIC_RG_VMCH_CK_SW_EN_SHIFT},
	{PMIC_RG_VMCH_OSC_SEL_DIS, PMIC_RG_VMCH_OSC_SEL_DIS_ADDR, PMIC_RG_VMCH_OSC_SEL_DIS_MASK,
	 PMIC_RG_VMCH_OSC_SEL_DIS_SHIFT},
	{PMIC_RG_VMC_CK_SW_MODE, PMIC_RG_VMC_CK_SW_MODE_ADDR, PMIC_RG_VMC_CK_SW_MODE_MASK,
	 PMIC_RG_VMC_CK_SW_MODE_SHIFT},
	{PMIC_RG_VMC_CK_SW_EN, PMIC_RG_VMC_CK_SW_EN_ADDR, PMIC_RG_VMC_CK_SW_EN_MASK,
	 PMIC_RG_VMC_CK_SW_EN_SHIFT},
	{PMIC_RG_VMC_OSC_SEL_DIS, PMIC_RG_VMC_OSC_SEL_DIS_ADDR, PMIC_RG_VMC_OSC_SEL_DIS_MASK,
	 PMIC_RG_VMC_OSC_SEL_DIS_SHIFT},
	{PMIC_RG_VRF18_1_CK_SW_MODE, PMIC_RG_VRF18_1_CK_SW_MODE_ADDR,
	 PMIC_RG_VRF18_1_CK_SW_MODE_MASK, PMIC_RG_VRF18_1_CK_SW_MODE_SHIFT},
	{PMIC_RG_VRF18_1_CK_SW_EN, PMIC_RG_VRF18_1_CK_SW_EN_ADDR, PMIC_RG_VRF18_1_CK_SW_EN_MASK,
	 PMIC_RG_VRF18_1_CK_SW_EN_SHIFT},
	{PMIC_RG_VRF18_1_OSC_SEL_DIS, PMIC_RG_VRF18_1_OSC_SEL_DIS_ADDR,
	 PMIC_RG_VRF18_1_OSC_SEL_DIS_MASK, PMIC_RG_VRF18_1_OSC_SEL_DIS_SHIFT},
	{PMIC_RG_VRF18_2_CK_SW_MODE, PMIC_RG_VRF18_2_CK_SW_MODE_ADDR,
	 PMIC_RG_VRF18_2_CK_SW_MODE_MASK, PMIC_RG_VRF18_2_CK_SW_MODE_SHIFT},
	{PMIC_RG_VRF18_2_CK_SW_EN, PMIC_RG_VRF18_2_CK_SW_EN_ADDR, PMIC_RG_VRF18_2_CK_SW_EN_MASK,
	 PMIC_RG_VRF18_2_CK_SW_EN_SHIFT},
	{PMIC_RG_VRF18_2_OSC_SEL_DIS, PMIC_RG_VRF18_2_OSC_SEL_DIS_ADDR,
	 PMIC_RG_VRF18_2_OSC_SEL_DIS_MASK, PMIC_RG_VRF18_2_OSC_SEL_DIS_SHIFT},
	{PMIC_RG_VRF12_CK_SW_MODE, PMIC_RG_VRF12_CK_SW_MODE_ADDR, PMIC_RG_VRF12_CK_SW_MODE_MASK,
	 PMIC_RG_VRF12_CK_SW_MODE_SHIFT},
	{PMIC_RG_VRF12_CK_SW_EN, PMIC_RG_VRF12_CK_SW_EN_ADDR, PMIC_RG_VRF12_CK_SW_EN_MASK,
	 PMIC_RG_VRF12_CK_SW_EN_SHIFT},
	{PMIC_RG_VRF12_OSC_SEL_DIS, PMIC_RG_VRF12_OSC_SEL_DIS_ADDR, PMIC_RG_VRF12_OSC_SEL_DIS_MASK,
	 PMIC_RG_VRF12_OSC_SEL_DIS_SHIFT},
	{PMIC_RG_VCAMA1_CK_SW_MODE, PMIC_RG_VCAMA1_CK_SW_MODE_ADDR, PMIC_RG_VCAMA1_CK_SW_MODE_MASK,
	 PMIC_RG_VCAMA1_CK_SW_MODE_SHIFT},
	{PMIC_RG_VCAMA1_CK_SW_EN, PMIC_RG_VCAMA1_CK_SW_EN_ADDR, PMIC_RG_VCAMA1_CK_SW_EN_MASK,
	 PMIC_RG_VCAMA1_CK_SW_EN_SHIFT},
	{PMIC_RG_VCAMA1_OSC_SEL_DIS, PMIC_RG_VCAMA1_OSC_SEL_DIS_ADDR,
	 PMIC_RG_VCAMA1_OSC_SEL_DIS_MASK, PMIC_RG_VCAMA1_OSC_SEL_DIS_SHIFT},
	{PMIC_RG_VCAMA2_CK_SW_MODE, PMIC_RG_VCAMA2_CK_SW_MODE_ADDR, PMIC_RG_VCAMA2_CK_SW_MODE_MASK,
	 PMIC_RG_VCAMA2_CK_SW_MODE_SHIFT},
	{PMIC_RG_VCAMA2_CK_SW_EN, PMIC_RG_VCAMA2_CK_SW_EN_ADDR, PMIC_RG_VCAMA2_CK_SW_EN_MASK,
	 PMIC_RG_VCAMA2_CK_SW_EN_SHIFT},
	{PMIC_RG_VCAMA2_OSC_SEL_DIS, PMIC_RG_VCAMA2_OSC_SEL_DIS_ADDR,
	 PMIC_RG_VCAMA2_OSC_SEL_DIS_MASK, PMIC_RG_VCAMA2_OSC_SEL_DIS_SHIFT},
	{PMIC_RG_VSRAM_DVFS1_SP_SW_VOSEL_EN, PMIC_RG_VSRAM_DVFS1_SP_SW_VOSEL_EN_ADDR,
	 PMIC_RG_VSRAM_DVFS1_SP_SW_VOSEL_EN_MASK, PMIC_RG_VSRAM_DVFS1_SP_SW_VOSEL_EN_SHIFT},
	{PMIC_RG_VSRAM_DVFS1_SP_SW_VOSEL, PMIC_RG_VSRAM_DVFS1_SP_SW_VOSEL_ADDR,
	 PMIC_RG_VSRAM_DVFS1_SP_SW_VOSEL_MASK, PMIC_RG_VSRAM_DVFS1_SP_SW_VOSEL_SHIFT},
	{PMIC_RG_VSRAM_DVFS2_SP_SW_VOSEL_EN, PMIC_RG_VSRAM_DVFS2_SP_SW_VOSEL_EN_ADDR,
	 PMIC_RG_VSRAM_DVFS2_SP_SW_VOSEL_EN_MASK, PMIC_RG_VSRAM_DVFS2_SP_SW_VOSEL_EN_SHIFT},
	{PMIC_RG_VSRAM_DVFS2_SP_SW_VOSEL, PMIC_RG_VSRAM_DVFS2_SP_SW_VOSEL_ADDR,
	 PMIC_RG_VSRAM_DVFS2_SP_SW_VOSEL_MASK, PMIC_RG_VSRAM_DVFS2_SP_SW_VOSEL_SHIFT},
	{PMIC_RG_VSRAM_VGPU_SP_SW_VOSEL_EN, PMIC_RG_VSRAM_VGPU_SP_SW_VOSEL_EN_ADDR,
	 PMIC_RG_VSRAM_VGPU_SP_SW_VOSEL_EN_MASK, PMIC_RG_VSRAM_VGPU_SP_SW_VOSEL_EN_SHIFT},
	{PMIC_RG_VSRAM_VGPU_SP_SW_VOSEL, PMIC_RG_VSRAM_VGPU_SP_SW_VOSEL_ADDR,
	 PMIC_RG_VSRAM_VGPU_SP_SW_VOSEL_MASK, PMIC_RG_VSRAM_VGPU_SP_SW_VOSEL_SHIFT},
	{PMIC_RG_VSRAM_VMD_SP_SW_VOSEL_EN, PMIC_RG_VSRAM_VMD_SP_SW_VOSEL_EN_ADDR,
	 PMIC_RG_VSRAM_VMD_SP_SW_VOSEL_EN_MASK, PMIC_RG_VSRAM_VMD_SP_SW_VOSEL_EN_SHIFT},
	{PMIC_RG_VSRAM_VMD_SP_SW_VOSEL, PMIC_RG_VSRAM_VMD_SP_SW_VOSEL_ADDR,
	 PMIC_RG_VSRAM_VMD_SP_SW_VOSEL_MASK, PMIC_RG_VSRAM_VMD_SP_SW_VOSEL_SHIFT},
	{PMIC_RG_VSRAM_VCORE_SP_SW_VOSEL_EN, PMIC_RG_VSRAM_VCORE_SP_SW_VOSEL_EN_ADDR,
	 PMIC_RG_VSRAM_VCORE_SP_SW_VOSEL_EN_MASK, PMIC_RG_VSRAM_VCORE_SP_SW_VOSEL_EN_SHIFT},
	{PMIC_RG_VSRAM_VCORE_SP_SW_VOSEL, PMIC_RG_VSRAM_VCORE_SP_SW_VOSEL_ADDR,
	 PMIC_RG_VSRAM_VCORE_SP_SW_VOSEL_MASK, PMIC_RG_VSRAM_VCORE_SP_SW_VOSEL_SHIFT},
	{PMIC_RG_VSRAM_VCORE_SSHUB_ON, PMIC_RG_VSRAM_VCORE_SSHUB_ON_ADDR,
	 PMIC_RG_VSRAM_VCORE_SSHUB_ON_MASK, PMIC_RG_VSRAM_VCORE_SSHUB_ON_SHIFT},
	{PMIC_RG_VSRAM_VCORE_SSHUB_MODE, PMIC_RG_VSRAM_VCORE_SSHUB_MODE_ADDR,
	 PMIC_RG_VSRAM_VCORE_SSHUB_MODE_MASK, PMIC_RG_VSRAM_VCORE_SSHUB_MODE_SHIFT},
	{PMIC_RG_VSRAM_VCORE_SSHUB_VOSEL, PMIC_RG_VSRAM_VCORE_SSHUB_VOSEL_ADDR,
	 PMIC_RG_VSRAM_VCORE_SSHUB_VOSEL_MASK, PMIC_RG_VSRAM_VCORE_SSHUB_VOSEL_SHIFT},
	{PMIC_RG_ALDO_1_RESEV, PMIC_RG_ALDO_1_RESEV_ADDR, PMIC_RG_ALDO_1_RESEV_MASK,
	 PMIC_RG_ALDO_1_RESEV_SHIFT},
	{PMIC_RG_VFE28_CAL, PMIC_RG_VFE28_CAL_ADDR, PMIC_RG_VFE28_CAL_MASK,
	 PMIC_RG_VFE28_CAL_SHIFT},
	{PMIC_RG_VFE28_VOSEL, PMIC_RG_VFE28_VOSEL_ADDR, PMIC_RG_VFE28_VOSEL_MASK,
	 PMIC_RG_VFE28_VOSEL_SHIFT},
	{PMIC_RG_VFE28_NDIS_EN, PMIC_RG_VFE28_NDIS_EN_ADDR, PMIC_RG_VFE28_NDIS_EN_MASK,
	 PMIC_RG_VFE28_NDIS_EN_SHIFT},
	{PMIC_RG_VBIF28_CAL, PMIC_RG_VBIF28_CAL_ADDR, PMIC_RG_VBIF28_CAL_MASK,
	 PMIC_RG_VBIF28_CAL_SHIFT},
	{PMIC_RG_VBIF28_VOSEL, PMIC_RG_VBIF28_VOSEL_ADDR, PMIC_RG_VBIF28_VOSEL_MASK,
	 PMIC_RG_VBIF28_VOSEL_SHIFT},
	{PMIC_RG_VBIF28_NDIS_EN, PMIC_RG_VBIF28_NDIS_EN_ADDR, PMIC_RG_VBIF28_NDIS_EN_MASK,
	 PMIC_RG_VBIF28_NDIS_EN_SHIFT},
	{PMIC_RG_VCN28_CAL, PMIC_RG_VCN28_CAL_ADDR, PMIC_RG_VCN28_CAL_MASK,
	 PMIC_RG_VCN28_CAL_SHIFT},
	{PMIC_RG_VCN28_VOSEL, PMIC_RG_VCN28_VOSEL_ADDR, PMIC_RG_VCN28_VOSEL_MASK,
	 PMIC_RG_VCN28_VOSEL_SHIFT},
	{PMIC_RG_VCN28_NDIS_EN, PMIC_RG_VCN28_NDIS_EN_ADDR, PMIC_RG_VCN28_NDIS_EN_MASK,
	 PMIC_RG_VCN28_NDIS_EN_SHIFT},
	{PMIC_RG_VXO22_CAL, PMIC_RG_VXO22_CAL_ADDR, PMIC_RG_VXO22_CAL_MASK,
	 PMIC_RG_VXO22_CAL_SHIFT},
	{PMIC_RG_VXO22_VOSEL, PMIC_RG_VXO22_VOSEL_ADDR, PMIC_RG_VXO22_VOSEL_MASK,
	 PMIC_RG_VXO22_VOSEL_SHIFT},
	{PMIC_RG_VXO22_NDIS_EN, PMIC_RG_VXO22_NDIS_EN_ADDR, PMIC_RG_VXO22_NDIS_EN_MASK,
	 PMIC_RG_VXO22_NDIS_EN_SHIFT},
	{PMIC_RG_VA18_CAL, PMIC_RG_VA18_CAL_ADDR, PMIC_RG_VA18_CAL_MASK, PMIC_RG_VA18_CAL_SHIFT},
	{PMIC_RG_VA18_VOSEL, PMIC_RG_VA18_VOSEL_ADDR, PMIC_RG_VA18_VOSEL_MASK,
	 PMIC_RG_VA18_VOSEL_SHIFT},
	{PMIC_RG_VA18_NDIS_EN, PMIC_RG_VA18_NDIS_EN_ADDR, PMIC_RG_VA18_NDIS_EN_MASK,
	 PMIC_RG_VA18_NDIS_EN_SHIFT},
	{PMIC_RG_ALDO_2_RESEV, PMIC_RG_ALDO_2_RESEV_ADDR, PMIC_RG_ALDO_2_RESEV_MASK,
	 PMIC_RG_ALDO_2_RESEV_SHIFT},
	{PMIC_RG_VCAMAF_CAL, PMIC_RG_VCAMAF_CAL_ADDR, PMIC_RG_VCAMAF_CAL_MASK,
	 PMIC_RG_VCAMAF_CAL_SHIFT},
	{PMIC_RG_VCAMAF_NDIS_EN, PMIC_RG_VCAMAF_NDIS_EN_ADDR, PMIC_RG_VCAMAF_NDIS_EN_MASK,
	 PMIC_RG_VCAMAF_NDIS_EN_SHIFT},
	{PMIC_RG_VCAMAF_OC_TRIM, PMIC_RG_VCAMAF_OC_TRIM_ADDR, PMIC_RG_VCAMAF_OC_TRIM_MASK,
	 PMIC_RG_VCAMAF_OC_TRIM_SHIFT},
	{PMIC_RG_VCAMAF_N2LP_EHC_V18, PMIC_RG_VCAMAF_N2LP_EHC_V18_ADDR,
	 PMIC_RG_VCAMAF_N2LP_EHC_V18_MASK, PMIC_RG_VCAMAF_N2LP_EHC_V18_SHIFT},
	{PMIC_RG_VCAMA1_CAL, PMIC_RG_VCAMA1_CAL_ADDR, PMIC_RG_VCAMA1_CAL_MASK,
	 PMIC_RG_VCAMA1_CAL_SHIFT},
	{PMIC_RG_VCAMA1_VOSEL, PMIC_RG_VCAMA1_VOSEL_ADDR, PMIC_RG_VCAMA1_VOSEL_MASK,
	 PMIC_RG_VCAMA1_VOSEL_SHIFT},
	{PMIC_RG_VCAMA1_NDIS_EN, PMIC_RG_VCAMA1_NDIS_EN_ADDR, PMIC_RG_VCAMA1_NDIS_EN_MASK,
	 PMIC_RG_VCAMA1_NDIS_EN_SHIFT},
	{PMIC_RG_VCAMA1_FBSEL, PMIC_RG_VCAMA1_FBSEL_ADDR, PMIC_RG_VCAMA1_FBSEL_MASK,
	 PMIC_RG_VCAMA1_FBSEL_SHIFT},
	{PMIC_RG_VCAMA2_CAL, PMIC_RG_VCAMA2_CAL_ADDR, PMIC_RG_VCAMA2_CAL_MASK,
	 PMIC_RG_VCAMA2_CAL_SHIFT},
	{PMIC_RG_VCAMA2_VOSEL, PMIC_RG_VCAMA2_VOSEL_ADDR, PMIC_RG_VCAMA2_VOSEL_MASK,
	 PMIC_RG_VCAMA2_VOSEL_SHIFT},
	{PMIC_RG_VCAMA2_NDIS_EN, PMIC_RG_VCAMA2_NDIS_EN_ADDR, PMIC_RG_VCAMA2_NDIS_EN_MASK,
	 PMIC_RG_VCAMA2_NDIS_EN_SHIFT},
	{PMIC_RG_VCAMA2_FBSEL, PMIC_RG_VCAMA2_FBSEL_ADDR, PMIC_RG_VCAMA2_FBSEL_MASK,
	 PMIC_RG_VCAMA2_FBSEL_SHIFT},
	{PMIC_RG_DLDO_RESEV, PMIC_RG_DLDO_RESEV_ADDR, PMIC_RG_DLDO_RESEV_MASK,
	 PMIC_RG_DLDO_RESEV_SHIFT},
	{PMIC_RG_VMC_CAL, PMIC_RG_VMC_CAL_ADDR, PMIC_RG_VMC_CAL_MASK, PMIC_RG_VMC_CAL_SHIFT},
	{PMIC_RG_VMC_VOSEL, PMIC_RG_VMC_VOSEL_ADDR, PMIC_RG_VMC_VOSEL_MASK,
	 PMIC_RG_VMC_VOSEL_SHIFT},
	{PMIC_RG_VMC_NDIS_EN, PMIC_RG_VMC_NDIS_EN_ADDR, PMIC_RG_VMC_NDIS_EN_MASK,
	 PMIC_RG_VMC_NDIS_EN_SHIFT},
	{PMIC_RG_VMC_OC_TRIM, PMIC_RG_VMC_OC_TRIM_ADDR, PMIC_RG_VMC_OC_TRIM_MASK,
	 PMIC_RG_VMC_OC_TRIM_SHIFT},
	{PMIC_RG_VMC_N2LP_EHC_V18, PMIC_RG_VMC_N2LP_EHC_V18_ADDR, PMIC_RG_VMC_N2LP_EHC_V18_MASK,
	 PMIC_RG_VMC_N2LP_EHC_V18_SHIFT},
	{PMIC_RG_VCN33_CAL, PMIC_RG_VCN33_CAL_ADDR, PMIC_RG_VCN33_CAL_MASK,
	 PMIC_RG_VCN33_CAL_SHIFT},
	{PMIC_RG_VCN33_VOSEL, PMIC_RG_VCN33_VOSEL_ADDR, PMIC_RG_VCN33_VOSEL_MASK,
	 PMIC_RG_VCN33_VOSEL_SHIFT},
	{PMIC_RG_VCN33_NDIS_EN, PMIC_RG_VCN33_NDIS_EN_ADDR, PMIC_RG_VCN33_NDIS_EN_MASK,
	 PMIC_RG_VCN33_NDIS_EN_SHIFT},
	{PMIC_RG_VCN33_N2LP_EHC_V18, PMIC_RG_VCN33_N2LP_EHC_V18_ADDR,
	 PMIC_RG_VCN33_N2LP_EHC_V18_MASK, PMIC_RG_VCN33_N2LP_EHC_V18_SHIFT},
	{PMIC_RG_VSIM1_CAL, PMIC_RG_VSIM1_CAL_ADDR, PMIC_RG_VSIM1_CAL_MASK,
	 PMIC_RG_VSIM1_CAL_SHIFT},
	{PMIC_RG_VSIM1_VOSEL, PMIC_RG_VSIM1_VOSEL_ADDR, PMIC_RG_VSIM1_VOSEL_MASK,
	 PMIC_RG_VSIM1_VOSEL_SHIFT},
	{PMIC_RG_VSIM1_NDIS_EN, PMIC_RG_VSIM1_NDIS_EN_ADDR, PMIC_RG_VSIM1_NDIS_EN_MASK,
	 PMIC_RG_VSIM1_NDIS_EN_SHIFT},
	{PMIC_RG_VSIM1_STB_CAL, PMIC_RG_VSIM1_STB_CAL_ADDR, PMIC_RG_VSIM1_STB_CAL_MASK,
	 PMIC_RG_VSIM1_STB_CAL_SHIFT},
	{PMIC_RG_VSIM1_STB_SEL, PMIC_RG_VSIM1_STB_SEL_ADDR, PMIC_RG_VSIM1_STB_SEL_MASK,
	 PMIC_RG_VSIM1_STB_SEL_SHIFT},
	{PMIC_RG_VSIM1_OC_TRIM, PMIC_RG_VSIM1_OC_TRIM_ADDR, PMIC_RG_VSIM1_OC_TRIM_MASK,
	 PMIC_RG_VSIM1_OC_TRIM_SHIFT},
	{PMIC_RG_VSIM1_NDIS_EN_INT, PMIC_RG_VSIM1_NDIS_EN_INT_ADDR, PMIC_RG_VSIM1_NDIS_EN_INT_MASK,
	 PMIC_RG_VSIM1_NDIS_EN_INT_SHIFT},
	{PMIC_RG_VSIM1_LP_HIGH_SPEED_ENB, PMIC_RG_VSIM1_LP_HIGH_SPEED_ENB_ADDR,
	 PMIC_RG_VSIM1_LP_HIGH_SPEED_ENB_MASK, PMIC_RG_VSIM1_LP_HIGH_SPEED_ENB_SHIFT},
	{PMIC_RG_VEMC_CAL, PMIC_RG_VEMC_CAL_ADDR, PMIC_RG_VEMC_CAL_MASK, PMIC_RG_VEMC_CAL_SHIFT},
	{PMIC_RG_VEMC_VOSEL, PMIC_RG_VEMC_VOSEL_ADDR, PMIC_RG_VEMC_VOSEL_MASK,
	 PMIC_RG_VEMC_VOSEL_SHIFT},
	{PMIC_RG_VEMC_NDIS_EN, PMIC_RG_VEMC_NDIS_EN_ADDR, PMIC_RG_VEMC_NDIS_EN_MASK,
	 PMIC_RG_VEMC_NDIS_EN_SHIFT},
	{PMIC_RG_VEMC_OC_TRIM, PMIC_RG_VEMC_OC_TRIM_ADDR, PMIC_RG_VEMC_OC_TRIM_MASK,
	 PMIC_RG_VEMC_OC_TRIM_SHIFT},
	{PMIC_RG_VEMC_N2LP_EHC_V18, PMIC_RG_VEMC_N2LP_EHC_V18_ADDR, PMIC_RG_VEMC_N2LP_EHC_V18_MASK,
	 PMIC_RG_VEMC_N2LP_EHC_V18_SHIFT},
	{PMIC_RG_VUSB33_CAL, PMIC_RG_VUSB33_CAL_ADDR, PMIC_RG_VUSB33_CAL_MASK,
	 PMIC_RG_VUSB33_CAL_SHIFT},
	{PMIC_RG_VUSB33_NDIS_EN, PMIC_RG_VUSB33_NDIS_EN_ADDR, PMIC_RG_VUSB33_NDIS_EN_MASK,
	 PMIC_RG_VUSB33_NDIS_EN_SHIFT},
	{PMIC_RG_VUSB33_OC_TRIM, PMIC_RG_VUSB33_OC_TRIM_ADDR, PMIC_RG_VUSB33_OC_TRIM_MASK,
	 PMIC_RG_VUSB33_OC_TRIM_SHIFT},
	{PMIC_RG_VIBR_CAL, PMIC_RG_VIBR_CAL_ADDR, PMIC_RG_VIBR_CAL_MASK, PMIC_RG_VIBR_CAL_SHIFT},
	{PMIC_RG_VIBR_VOSEL, PMIC_RG_VIBR_VOSEL_ADDR, PMIC_RG_VIBR_VOSEL_MASK,
	 PMIC_RG_VIBR_VOSEL_SHIFT},
	{PMIC_RG_VIBR_NDIS_EN, PMIC_RG_VIBR_NDIS_EN_ADDR, PMIC_RG_VIBR_NDIS_EN_MASK,
	 PMIC_RG_VIBR_NDIS_EN_SHIFT},
	{PMIC_RG_VIBR_LP_HIGH_SPEED_ENB, PMIC_RG_VIBR_LP_HIGH_SPEED_ENB_ADDR,
	 PMIC_RG_VIBR_LP_HIGH_SPEED_ENB_MASK, PMIC_RG_VIBR_LP_HIGH_SPEED_ENB_SHIFT},
	{PMIC_RG_VTOUCH_CAL, PMIC_RG_VTOUCH_CAL_ADDR, PMIC_RG_VTOUCH_CAL_MASK,
	 PMIC_RG_VTOUCH_CAL_SHIFT},
	{PMIC_RG_VTOUCH_NDIS_EN, PMIC_RG_VTOUCH_NDIS_EN_ADDR, PMIC_RG_VTOUCH_NDIS_EN_MASK,
	 PMIC_RG_VTOUCH_NDIS_EN_SHIFT},
	{PMIC_RG_VTOUCH_OC_TRIM, PMIC_RG_VTOUCH_OC_TRIM_ADDR, PMIC_RG_VTOUCH_OC_TRIM_MASK,
	 PMIC_RG_VTOUCH_OC_TRIM_SHIFT},
	{PMIC_RG_VTOUCH_N2LP_EHC_V18, PMIC_RG_VTOUCH_N2LP_EHC_V18_ADDR,
	 PMIC_RG_VTOUCH_N2LP_EHC_V18_MASK, PMIC_RG_VTOUCH_N2LP_EHC_V18_SHIFT},
	{PMIC_RG_VEFUSE_CAL, PMIC_RG_VEFUSE_CAL_ADDR, PMIC_RG_VEFUSE_CAL_MASK,
	 PMIC_RG_VEFUSE_CAL_SHIFT},
	{PMIC_RG_VEFUSE_VOSEL, PMIC_RG_VEFUSE_VOSEL_ADDR, PMIC_RG_VEFUSE_VOSEL_MASK,
	 PMIC_RG_VEFUSE_VOSEL_SHIFT},
	{PMIC_RG_VEFUSE_NDIS_EN, PMIC_RG_VEFUSE_NDIS_EN_ADDR, PMIC_RG_VEFUSE_NDIS_EN_MASK,
	 PMIC_RG_VEFUSE_NDIS_EN_SHIFT},
	{PMIC_RG_VEFUSE_STB_CAL, PMIC_RG_VEFUSE_STB_CAL_ADDR, PMIC_RG_VEFUSE_STB_CAL_MASK,
	 PMIC_RG_VEFUSE_STB_CAL_SHIFT},
	{PMIC_RG_VEFUSE_STB_SEL, PMIC_RG_VEFUSE_STB_SEL_ADDR, PMIC_RG_VEFUSE_STB_SEL_MASK,
	 PMIC_RG_VEFUSE_STB_SEL_SHIFT},
	{PMIC_RG_VEFUSE_OC_TRIM, PMIC_RG_VEFUSE_OC_TRIM_ADDR, PMIC_RG_VEFUSE_OC_TRIM_MASK,
	 PMIC_RG_VEFUSE_OC_TRIM_SHIFT},
	{PMIC_RG_VEFUSE_NDIS_EN_INT, PMIC_RG_VEFUSE_NDIS_EN_INT_ADDR,
	 PMIC_RG_VEFUSE_NDIS_EN_INT_MASK, PMIC_RG_VEFUSE_NDIS_EN_INT_SHIFT},
	{PMIC_RG_VEFUSE_LP_HIGH_SPEED_ENB, PMIC_RG_VEFUSE_LP_HIGH_SPEED_ENB_ADDR,
	 PMIC_RG_VEFUSE_LP_HIGH_SPEED_ENB_MASK, PMIC_RG_VEFUSE_LP_HIGH_SPEED_ENB_SHIFT},
	{PMIC_RG_VIO28_CAL, PMIC_RG_VIO28_CAL_ADDR, PMIC_RG_VIO28_CAL_MASK,
	 PMIC_RG_VIO28_CAL_SHIFT},
	{PMIC_RG_VIO28_VOSEL, PMIC_RG_VIO28_VOSEL_ADDR, PMIC_RG_VIO28_VOSEL_MASK,
	 PMIC_RG_VIO28_VOSEL_SHIFT},
	{PMIC_RG_VIO28_NDIS_EN, PMIC_RG_VIO28_NDIS_EN_ADDR, PMIC_RG_VIO28_NDIS_EN_MASK,
	 PMIC_RG_VIO28_NDIS_EN_SHIFT},
	{PMIC_RG_VIO28_LP_HIGH_SPEED_ENB, PMIC_RG_VIO28_LP_HIGH_SPEED_ENB_ADDR,
	 PMIC_RG_VIO28_LP_HIGH_SPEED_ENB_MASK, PMIC_RG_VIO28_LP_HIGH_SPEED_ENB_SHIFT},
	{PMIC_RG_VMCH_CAL, PMIC_RG_VMCH_CAL_ADDR, PMIC_RG_VMCH_CAL_MASK, PMIC_RG_VMCH_CAL_SHIFT},
	{PMIC_RG_VMCH_VOSEL, PMIC_RG_VMCH_VOSEL_ADDR, PMIC_RG_VMCH_VOSEL_MASK,
	 PMIC_RG_VMCH_VOSEL_SHIFT},
	{PMIC_RG_VMCH_NDIS_EN, PMIC_RG_VMCH_NDIS_EN_ADDR, PMIC_RG_VMCH_NDIS_EN_MASK,
	 PMIC_RG_VMCH_NDIS_EN_SHIFT},
	{PMIC_RG_VMCH_OC_TRIM, PMIC_RG_VMCH_OC_TRIM_ADDR, PMIC_RG_VMCH_OC_TRIM_MASK,
	 PMIC_RG_VMCH_OC_TRIM_SHIFT},
	{PMIC_RG_VMCH_N2LP_EHC_V18, PMIC_RG_VMCH_N2LP_EHC_V18_ADDR, PMIC_RG_VMCH_N2LP_EHC_V18_MASK,
	 PMIC_RG_VMCH_N2LP_EHC_V18_SHIFT},
	{PMIC_RG_VSIM2_CAL, PMIC_RG_VSIM2_CAL_ADDR, PMIC_RG_VSIM2_CAL_MASK,
	 PMIC_RG_VSIM2_CAL_SHIFT},
	{PMIC_RG_VSIM2_VOSEL, PMIC_RG_VSIM2_VOSEL_ADDR, PMIC_RG_VSIM2_VOSEL_MASK,
	 PMIC_RG_VSIM2_VOSEL_SHIFT},
	{PMIC_RG_VSIM2_NDIS_EN, PMIC_RG_VSIM2_NDIS_EN_ADDR, PMIC_RG_VSIM2_NDIS_EN_MASK,
	 PMIC_RG_VSIM2_NDIS_EN_SHIFT},
	{PMIC_RG_VSIM2_STB_CAL, PMIC_RG_VSIM2_STB_CAL_ADDR, PMIC_RG_VSIM2_STB_CAL_MASK,
	 PMIC_RG_VSIM2_STB_CAL_SHIFT},
	{PMIC_RG_VSIM2_STB_SEL, PMIC_RG_VSIM2_STB_SEL_ADDR, PMIC_RG_VSIM2_STB_SEL_MASK,
	 PMIC_RG_VSIM2_STB_SEL_SHIFT},
	{PMIC_RG_VSIM2_OC_TRIM, PMIC_RG_VSIM2_OC_TRIM_ADDR, PMIC_RG_VSIM2_OC_TRIM_MASK,
	 PMIC_RG_VSIM2_OC_TRIM_SHIFT},
	{PMIC_RG_VSIM2_NDIS_EN_INT, PMIC_RG_VSIM2_NDIS_EN_INT_ADDR, PMIC_RG_VSIM2_NDIS_EN_INT_MASK,
	 PMIC_RG_VSIM2_NDIS_EN_INT_SHIFT},
	{PMIC_RG_VSIM2_LP_HIGH_SPEED_ENB, PMIC_RG_VSIM2_LP_HIGH_SPEED_ENB_ADDR,
	 PMIC_RG_VSIM2_LP_HIGH_SPEED_ENB_MASK, PMIC_RG_VSIM2_LP_HIGH_SPEED_ENB_SHIFT},
	{PMIC_RG_SLDO14_RESEV, PMIC_RG_SLDO14_RESEV_ADDR, PMIC_RG_SLDO14_RESEV_MASK,
	 PMIC_RG_SLDO14_RESEV_SHIFT},
	{PMIC_RG_VSRAM_DVFS1_NDIS_EN, PMIC_RG_VSRAM_DVFS1_NDIS_EN_ADDR,
	 PMIC_RG_VSRAM_DVFS1_NDIS_EN_MASK, PMIC_RG_VSRAM_DVFS1_NDIS_EN_SHIFT},
	{PMIC_RG_VSRAM_DVFS1_NDIS_PLCUR, PMIC_RG_VSRAM_DVFS1_NDIS_PLCUR_ADDR,
	 PMIC_RG_VSRAM_DVFS1_NDIS_PLCUR_MASK, PMIC_RG_VSRAM_DVFS1_NDIS_PLCUR_SHIFT},
	{PMIC_RG_VSRAM_DVFS1_STB_SEL, PMIC_RG_VSRAM_DVFS1_STB_SEL_ADDR,
	 PMIC_RG_VSRAM_DVFS1_STB_SEL_MASK, PMIC_RG_VSRAM_DVFS1_STB_SEL_SHIFT},
	{PMIC_RG_VSRAM_DVFS1_PLCUR_EN, PMIC_RG_VSRAM_DVFS1_PLCUR_EN_ADDR,
	 PMIC_RG_VSRAM_DVFS1_PLCUR_EN_MASK, PMIC_RG_VSRAM_DVFS1_PLCUR_EN_SHIFT},
	{PMIC_RG_VSRAM_DVFS2_NDIS_EN, PMIC_RG_VSRAM_DVFS2_NDIS_EN_ADDR,
	 PMIC_RG_VSRAM_DVFS2_NDIS_EN_MASK, PMIC_RG_VSRAM_DVFS2_NDIS_EN_SHIFT},
	{PMIC_RG_VSRAM_DVFS2_NDIS_PLCUR, PMIC_RG_VSRAM_DVFS2_NDIS_PLCUR_ADDR,
	 PMIC_RG_VSRAM_DVFS2_NDIS_PLCUR_MASK, PMIC_RG_VSRAM_DVFS2_NDIS_PLCUR_SHIFT},
	{PMIC_RG_VSRAM_DVFS2_STB_SEL, PMIC_RG_VSRAM_DVFS2_STB_SEL_ADDR,
	 PMIC_RG_VSRAM_DVFS2_STB_SEL_MASK, PMIC_RG_VSRAM_DVFS2_STB_SEL_SHIFT},
	{PMIC_RG_VSRAM_DVFS2_PLCUR_EN, PMIC_RG_VSRAM_DVFS2_PLCUR_EN_ADDR,
	 PMIC_RG_VSRAM_DVFS2_PLCUR_EN_MASK, PMIC_RG_VSRAM_DVFS2_PLCUR_EN_SHIFT},
	{PMIC_RG_VSRAM_VCORE_NDIS_EN, PMIC_RG_VSRAM_VCORE_NDIS_EN_ADDR,
	 PMIC_RG_VSRAM_VCORE_NDIS_EN_MASK, PMIC_RG_VSRAM_VCORE_NDIS_EN_SHIFT},
	{PMIC_RG_VSRAM_VCORE_NDIS_PLCUR, PMIC_RG_VSRAM_VCORE_NDIS_PLCUR_ADDR,
	 PMIC_RG_VSRAM_VCORE_NDIS_PLCUR_MASK, PMIC_RG_VSRAM_VCORE_NDIS_PLCUR_SHIFT},
	{PMIC_RG_VSRAM_VCORE_STB_SEL, PMIC_RG_VSRAM_VCORE_STB_SEL_ADDR,
	 PMIC_RG_VSRAM_VCORE_STB_SEL_MASK, PMIC_RG_VSRAM_VCORE_STB_SEL_SHIFT},
	{PMIC_RG_VSRAM_VCORE_PLCUR_EN, PMIC_RG_VSRAM_VCORE_PLCUR_EN_ADDR,
	 PMIC_RG_VSRAM_VCORE_PLCUR_EN_MASK, PMIC_RG_VSRAM_VCORE_PLCUR_EN_SHIFT},
	{PMIC_RG_VSRAM_VGPU_NDIS_EN, PMIC_RG_VSRAM_VGPU_NDIS_EN_ADDR,
	 PMIC_RG_VSRAM_VGPU_NDIS_EN_MASK, PMIC_RG_VSRAM_VGPU_NDIS_EN_SHIFT},
	{PMIC_RG_VSRAM_VGPU_NDIS_PLCUR, PMIC_RG_VSRAM_VGPU_NDIS_PLCUR_ADDR,
	 PMIC_RG_VSRAM_VGPU_NDIS_PLCUR_MASK, PMIC_RG_VSRAM_VGPU_NDIS_PLCUR_SHIFT},
	{PMIC_RG_VSRAM_VGPU_STB_SEL, PMIC_RG_VSRAM_VGPU_STB_SEL_ADDR,
	 PMIC_RG_VSRAM_VGPU_STB_SEL_MASK, PMIC_RG_VSRAM_VGPU_STB_SEL_SHIFT},
	{PMIC_RG_VSRAM_VGPU_PLCUR_EN, PMIC_RG_VSRAM_VGPU_PLCUR_EN_ADDR,
	 PMIC_RG_VSRAM_VGPU_PLCUR_EN_MASK, PMIC_RG_VSRAM_VGPU_PLCUR_EN_SHIFT},
	{PMIC_RG_VSRAM_VMD_NDIS_EN, PMIC_RG_VSRAM_VMD_NDIS_EN_ADDR, PMIC_RG_VSRAM_VMD_NDIS_EN_MASK,
	 PMIC_RG_VSRAM_VMD_NDIS_EN_SHIFT},
	{PMIC_RG_VSRAM_VMD_NDIS_PLCUR, PMIC_RG_VSRAM_VMD_NDIS_PLCUR_ADDR,
	 PMIC_RG_VSRAM_VMD_NDIS_PLCUR_MASK, PMIC_RG_VSRAM_VMD_NDIS_PLCUR_SHIFT},
	{PMIC_RG_VSRAM_VMD_STB_SEL, PMIC_RG_VSRAM_VMD_STB_SEL_ADDR, PMIC_RG_VSRAM_VMD_STB_SEL_MASK,
	 PMIC_RG_VSRAM_VMD_STB_SEL_SHIFT},
	{PMIC_RG_VSRAM_VMD_PLCUR_EN, PMIC_RG_VSRAM_VMD_PLCUR_EN_ADDR,
	 PMIC_RG_VSRAM_VMD_PLCUR_EN_MASK, PMIC_RG_VSRAM_VMD_PLCUR_EN_SHIFT},
	{PMIC_RG_VA12_CAL, PMIC_RG_VA12_CAL_ADDR, PMIC_RG_VA12_CAL_MASK, PMIC_RG_VA12_CAL_SHIFT},
	{PMIC_RG_VA12_VOSEL, PMIC_RG_VA12_VOSEL_ADDR, PMIC_RG_VA12_VOSEL_MASK,
	 PMIC_RG_VA12_VOSEL_SHIFT},
	{PMIC_RG_VA12_NDIS_EN, PMIC_RG_VA12_NDIS_EN_ADDR, PMIC_RG_VA12_NDIS_EN_MASK,
	 PMIC_RG_VA12_NDIS_EN_SHIFT},
	{PMIC_RG_VA12_STB_SEL, PMIC_RG_VA12_STB_SEL_ADDR, PMIC_RG_VA12_STB_SEL_MASK,
	 PMIC_RG_VA12_STB_SEL_SHIFT},
	{PMIC_RG_VCAMD1_CAL, PMIC_RG_VCAMD1_CAL_ADDR, PMIC_RG_VCAMD1_CAL_MASK,
	 PMIC_RG_VCAMD1_CAL_SHIFT},
	{PMIC_RG_VCAMD1_VOSEL, PMIC_RG_VCAMD1_VOSEL_ADDR, PMIC_RG_VCAMD1_VOSEL_MASK,
	 PMIC_RG_VCAMD1_VOSEL_SHIFT},
	{PMIC_RG_VCAMD1_NDIS_EN, PMIC_RG_VCAMD1_NDIS_EN_ADDR, PMIC_RG_VCAMD1_NDIS_EN_MASK,
	 PMIC_RG_VCAMD1_NDIS_EN_SHIFT},
	{PMIC_RG_VCAMD1_RSV, PMIC_RG_VCAMD1_RSV_ADDR, PMIC_RG_VCAMD1_RSV_MASK,
	 PMIC_RG_VCAMD1_RSV_SHIFT},
	{PMIC_RG_VCAMD1_OC_TRIM, PMIC_RG_VCAMD1_OC_TRIM_ADDR, PMIC_RG_VCAMD1_OC_TRIM_MASK,
	 PMIC_RG_VCAMD1_OC_TRIM_SHIFT},
	{PMIC_RG_VA10_CAL, PMIC_RG_VA10_CAL_ADDR, PMIC_RG_VA10_CAL_MASK, PMIC_RG_VA10_CAL_SHIFT},
	{PMIC_RG_VA10_VOSEL, PMIC_RG_VA10_VOSEL_ADDR, PMIC_RG_VA10_VOSEL_MASK,
	 PMIC_RG_VA10_VOSEL_SHIFT},
	{PMIC_RG_VA10_NDIS_EN, PMIC_RG_VA10_NDIS_EN_ADDR, PMIC_RG_VA10_NDIS_EN_MASK,
	 PMIC_RG_VA10_NDIS_EN_SHIFT},
	{PMIC_RG_VA10_STB_SEL, PMIC_RG_VA10_STB_SEL_ADDR, PMIC_RG_VA10_STB_SEL_MASK,
	 PMIC_RG_VA10_STB_SEL_SHIFT},
	{PMIC_RG_VA10_LP_VSEL, PMIC_RG_VA10_LP_VSEL_ADDR, PMIC_RG_VA10_LP_VSEL_MASK,
	 PMIC_RG_VA10_LP_VSEL_SHIFT},
	{PMIC_RG_VA10_LP_VSEL_OPTION, PMIC_RG_VA10_LP_VSEL_OPTION_ADDR,
	 PMIC_RG_VA10_LP_VSEL_OPTION_MASK, PMIC_RG_VA10_LP_VSEL_OPTION_SHIFT},
	{PMIC_RG_VRF12_CAL, PMIC_RG_VRF12_CAL_ADDR, PMIC_RG_VRF12_CAL_MASK,
	 PMIC_RG_VRF12_CAL_SHIFT},
	{PMIC_RG_VRF12_VOSEL, PMIC_RG_VRF12_VOSEL_ADDR, PMIC_RG_VRF12_VOSEL_MASK,
	 PMIC_RG_VRF12_VOSEL_SHIFT},
	{PMIC_RG_VRF12_NDIS_EN, PMIC_RG_VRF12_NDIS_EN_ADDR, PMIC_RG_VRF12_NDIS_EN_MASK,
	 PMIC_RG_VRF12_NDIS_EN_SHIFT},
	{PMIC_RG_VRF12_FT_DNMC_EN, PMIC_RG_VRF12_FT_DNMC_EN_ADDR, PMIC_RG_VRF12_FT_DNMC_EN_MASK,
	 PMIC_RG_VRF12_FT_DNMC_EN_SHIFT},
	{PMIC_RG_VCAMD2_CAL, PMIC_RG_VCAMD2_CAL_ADDR, PMIC_RG_VCAMD2_CAL_MASK,
	 PMIC_RG_VCAMD2_CAL_SHIFT},
	{PMIC_RG_VCAMD2_VOSEL, PMIC_RG_VCAMD2_VOSEL_ADDR, PMIC_RG_VCAMD2_VOSEL_MASK,
	 PMIC_RG_VCAMD2_VOSEL_SHIFT},
	{PMIC_RG_VCAMD2_NDIS_EN, PMIC_RG_VCAMD2_NDIS_EN_ADDR, PMIC_RG_VCAMD2_NDIS_EN_MASK,
	 PMIC_RG_VCAMD2_NDIS_EN_SHIFT},
	{PMIC_RG_VCAMD2_RSV, PMIC_RG_VCAMD2_RSV_ADDR, PMIC_RG_VCAMD2_RSV_MASK,
	 PMIC_RG_VCAMD2_RSV_SHIFT},
	{PMIC_RG_VCAMD2_OC_TRIM, PMIC_RG_VCAMD2_OC_TRIM_ADDR, PMIC_RG_VCAMD2_OC_TRIM_MASK,
	 PMIC_RG_VCAMD2_OC_TRIM_SHIFT},
	{PMIC_RG_SLDO20_RESEV, PMIC_RG_SLDO20_RESEV_ADDR, PMIC_RG_SLDO20_RESEV_MASK,
	 PMIC_RG_SLDO20_RESEV_SHIFT},
	{PMIC_RG_VRF18_1_CAL, PMIC_RG_VRF18_1_CAL_ADDR, PMIC_RG_VRF18_1_CAL_MASK,
	 PMIC_RG_VRF18_1_CAL_SHIFT},
	{PMIC_RG_VRF18_1_VOSEL, PMIC_RG_VRF18_1_VOSEL_ADDR, PMIC_RG_VRF18_1_VOSEL_MASK,
	 PMIC_RG_VRF18_1_VOSEL_SHIFT},
	{PMIC_RG_VRF18_1_NDIS_EN, PMIC_RG_VRF18_1_NDIS_EN_ADDR, PMIC_RG_VRF18_1_NDIS_EN_MASK,
	 PMIC_RG_VRF18_1_NDIS_EN_SHIFT},
	{PMIC_RG_VRF18_1_FT_DNMC_EN, PMIC_RG_VRF18_1_FT_DNMC_EN_ADDR,
	 PMIC_RG_VRF18_1_FT_DNMC_EN_MASK, PMIC_RG_VRF18_1_FT_DNMC_EN_SHIFT},
	{PMIC_RG_VRF18_2_CAL, PMIC_RG_VRF18_2_CAL_ADDR, PMIC_RG_VRF18_2_CAL_MASK,
	 PMIC_RG_VRF18_2_CAL_SHIFT},
	{PMIC_RG_VRF18_2_VOSEL, PMIC_RG_VRF18_2_VOSEL_ADDR, PMIC_RG_VRF18_2_VOSEL_MASK,
	 PMIC_RG_VRF18_2_VOSEL_SHIFT},
	{PMIC_RG_VRF18_2_NDIS_EN, PMIC_RG_VRF18_2_NDIS_EN_ADDR, PMIC_RG_VRF18_2_NDIS_EN_MASK,
	 PMIC_RG_VRF18_2_NDIS_EN_SHIFT},
	{PMIC_RG_VRF18_2_FT_DNMC_EN, PMIC_RG_VRF18_2_FT_DNMC_EN_ADDR,
	 PMIC_RG_VRF18_2_FT_DNMC_EN_MASK, PMIC_RG_VRF18_2_FT_DNMC_EN_SHIFT},
	{PMIC_RG_VCN18_CAL, PMIC_RG_VCN18_CAL_ADDR, PMIC_RG_VCN18_CAL_MASK,
	 PMIC_RG_VCN18_CAL_SHIFT},
	{PMIC_RG_VCN18_VOSEL, PMIC_RG_VCN18_VOSEL_ADDR, PMIC_RG_VCN18_VOSEL_MASK,
	 PMIC_RG_VCN18_VOSEL_SHIFT},
	{PMIC_RG_VCN18_NDIS_EN, PMIC_RG_VCN18_NDIS_EN_ADDR, PMIC_RG_VCN18_NDIS_EN_MASK,
	 PMIC_RG_VCN18_NDIS_EN_SHIFT},
	{PMIC_RG_VCN18_STB_SEL, PMIC_RG_VCN18_STB_SEL_ADDR, PMIC_RG_VCN18_STB_SEL_MASK,
	 PMIC_RG_VCN18_STB_SEL_SHIFT},
	{PMIC_RG_VMIPI_CAL, PMIC_RG_VMIPI_CAL_ADDR, PMIC_RG_VMIPI_CAL_MASK,
	 PMIC_RG_VMIPI_CAL_SHIFT},
	{PMIC_RG_VMIPI_VOSEL, PMIC_RG_VMIPI_VOSEL_ADDR, PMIC_RG_VMIPI_VOSEL_MASK,
	 PMIC_RG_VMIPI_VOSEL_SHIFT},
	{PMIC_RG_VMIPI_NDIS_EN, PMIC_RG_VMIPI_NDIS_EN_ADDR, PMIC_RG_VMIPI_NDIS_EN_MASK,
	 PMIC_RG_VMIPI_NDIS_EN_SHIFT},
	{PMIC_RG_VMIPI_STB_SEL, PMIC_RG_VMIPI_STB_SEL_ADDR, PMIC_RG_VMIPI_STB_SEL_MASK,
	 PMIC_RG_VMIPI_STB_SEL_SHIFT},
	{PMIC_RG_VIO18_CAL, PMIC_RG_VIO18_CAL_ADDR, PMIC_RG_VIO18_CAL_MASK,
	 PMIC_RG_VIO18_CAL_SHIFT},
	{PMIC_RG_VIO18_NDIS_EN, PMIC_RG_VIO18_NDIS_EN_ADDR, PMIC_RG_VIO18_NDIS_EN_MASK,
	 PMIC_RG_VIO18_NDIS_EN_SHIFT},
	{PMIC_RG_VIO18_FT_DNMC_EN, PMIC_RG_VIO18_FT_DNMC_EN_ADDR, PMIC_RG_VIO18_FT_DNMC_EN_MASK,
	 PMIC_RG_VIO18_FT_DNMC_EN_SHIFT},
	{PMIC_RG_VCAMIO_CAL, PMIC_RG_VCAMIO_CAL_ADDR, PMIC_RG_VCAMIO_CAL_MASK,
	 PMIC_RG_VCAMIO_CAL_SHIFT},
	{PMIC_RG_VCAMIO_VOSEL, PMIC_RG_VCAMIO_VOSEL_ADDR, PMIC_RG_VCAMIO_VOSEL_MASK,
	 PMIC_RG_VCAMIO_VOSEL_SHIFT},
	{PMIC_RG_VCAMIO_NDIS_EN, PMIC_RG_VCAMIO_NDIS_EN_ADDR, PMIC_RG_VCAMIO_NDIS_EN_MASK,
	 PMIC_RG_VCAMIO_NDIS_EN_SHIFT},
	{PMIC_RG_VCAMIO_STB_SEL, PMIC_RG_VCAMIO_STB_SEL_ADDR, PMIC_RG_VCAMIO_STB_SEL_MASK,
	 PMIC_RG_VCAMIO_STB_SEL_SHIFT},
	{PMIC_RG_VUFS18_CAL, PMIC_RG_VUFS18_CAL_ADDR, PMIC_RG_VUFS18_CAL_MASK,
	 PMIC_RG_VUFS18_CAL_SHIFT},
	{PMIC_RG_VUFS18_NDIS_EN, PMIC_RG_VUFS18_NDIS_EN_ADDR, PMIC_RG_VUFS18_NDIS_EN_MASK,
	 PMIC_RG_VUFS18_NDIS_EN_SHIFT},
	{PMIC_RG_VUFS18_FT_DNMC_EN, PMIC_RG_VUFS18_FT_DNMC_EN_ADDR, PMIC_RG_VUFS18_FT_DNMC_EN_MASK,
	 PMIC_RG_VUFS18_FT_DNMC_EN_SHIFT},
	{PMIC_RG_VGP3_CAL, PMIC_RG_VGP3_CAL_ADDR, PMIC_RG_VGP3_CAL_MASK, PMIC_RG_VGP3_CAL_SHIFT},
	{PMIC_RG_VGP3_VOSEL, PMIC_RG_VGP3_VOSEL_ADDR, PMIC_RG_VGP3_VOSEL_MASK,
	 PMIC_RG_VGP3_VOSEL_SHIFT},
	{PMIC_RG_VGP3_NDIS_EN, PMIC_RG_VGP3_NDIS_EN_ADDR, PMIC_RG_VGP3_NDIS_EN_MASK,
	 PMIC_RG_VGP3_NDIS_EN_SHIFT},
	{PMIC_RG_VGP3_FT_DNMC_EN, PMIC_RG_VGP3_FT_DNMC_EN_ADDR, PMIC_RG_VGP3_FT_DNMC_EN_MASK,
	 PMIC_RG_VGP3_FT_DNMC_EN_SHIFT},
	{PMIC_BIF_COMMAND_0, PMIC_BIF_COMMAND_0_ADDR, PMIC_BIF_COMMAND_0_MASK,
	 PMIC_BIF_COMMAND_0_SHIFT},
	{PMIC_BIF_COMMAND_1, PMIC_BIF_COMMAND_1_ADDR, PMIC_BIF_COMMAND_1_MASK,
	 PMIC_BIF_COMMAND_1_SHIFT},
	{PMIC_BIF_COMMAND_2, PMIC_BIF_COMMAND_2_ADDR, PMIC_BIF_COMMAND_2_MASK,
	 PMIC_BIF_COMMAND_2_SHIFT},
	{PMIC_BIF_COMMAND_3, PMIC_BIF_COMMAND_3_ADDR, PMIC_BIF_COMMAND_3_MASK,
	 PMIC_BIF_COMMAND_3_SHIFT},
	{PMIC_BIF_COMMAND_4, PMIC_BIF_COMMAND_4_ADDR, PMIC_BIF_COMMAND_4_MASK,
	 PMIC_BIF_COMMAND_4_SHIFT},
	{PMIC_BIF_COMMAND_5, PMIC_BIF_COMMAND_5_ADDR, PMIC_BIF_COMMAND_5_MASK,
	 PMIC_BIF_COMMAND_5_SHIFT},
	{PMIC_BIF_COMMAND_6, PMIC_BIF_COMMAND_6_ADDR, PMIC_BIF_COMMAND_6_MASK,
	 PMIC_BIF_COMMAND_6_SHIFT},
	{PMIC_BIF_COMMAND_7, PMIC_BIF_COMMAND_7_ADDR, PMIC_BIF_COMMAND_7_MASK,
	 PMIC_BIF_COMMAND_7_SHIFT},
	{PMIC_BIF_COMMAND_8, PMIC_BIF_COMMAND_8_ADDR, PMIC_BIF_COMMAND_8_MASK,
	 PMIC_BIF_COMMAND_8_SHIFT},
	{PMIC_BIF_COMMAND_9, PMIC_BIF_COMMAND_9_ADDR, PMIC_BIF_COMMAND_9_MASK,
	 PMIC_BIF_COMMAND_9_SHIFT},
	{PMIC_BIF_COMMAND_10, PMIC_BIF_COMMAND_10_ADDR, PMIC_BIF_COMMAND_10_MASK,
	 PMIC_BIF_COMMAND_10_SHIFT},
	{PMIC_BIF_COMMAND_11, PMIC_BIF_COMMAND_11_ADDR, PMIC_BIF_COMMAND_11_MASK,
	 PMIC_BIF_COMMAND_11_SHIFT},
	{PMIC_BIF_COMMAND_12, PMIC_BIF_COMMAND_12_ADDR, PMIC_BIF_COMMAND_12_MASK,
	 PMIC_BIF_COMMAND_12_SHIFT},
	{PMIC_BIF_COMMAND_13, PMIC_BIF_COMMAND_13_ADDR, PMIC_BIF_COMMAND_13_MASK,
	 PMIC_BIF_COMMAND_13_SHIFT},
	{PMIC_BIF_COMMAND_14, PMIC_BIF_COMMAND_14_ADDR, PMIC_BIF_COMMAND_14_MASK,
	 PMIC_BIF_COMMAND_14_SHIFT},
	{PMIC_BIF_RSV, PMIC_BIF_RSV_ADDR, PMIC_BIF_RSV_MASK, PMIC_BIF_RSV_SHIFT},
	{PMIC_BIF_COMMAND_TYPE, PMIC_BIF_COMMAND_TYPE_ADDR, PMIC_BIF_COMMAND_TYPE_MASK,
	 PMIC_BIF_COMMAND_TYPE_SHIFT},
	{PMIC_BIF_TRASFER_NUM, PMIC_BIF_TRASFER_NUM_ADDR, PMIC_BIF_TRASFER_NUM_MASK,
	 PMIC_BIF_TRASFER_NUM_SHIFT},
	{PMIC_BIF_LOGIC_0_SET, PMIC_BIF_LOGIC_0_SET_ADDR, PMIC_BIF_LOGIC_0_SET_MASK,
	 PMIC_BIF_LOGIC_0_SET_SHIFT},
	{PMIC_BIF_LOGIC_1_SET, PMIC_BIF_LOGIC_1_SET_ADDR, PMIC_BIF_LOGIC_1_SET_MASK,
	 PMIC_BIF_LOGIC_1_SET_SHIFT},
	{PMIC_BIF_STOP_SET, PMIC_BIF_STOP_SET_ADDR, PMIC_BIF_STOP_SET_MASK,
	 PMIC_BIF_STOP_SET_SHIFT},
	{PMIC_BIF_DEBOUNCE_EN, PMIC_BIF_DEBOUNCE_EN_ADDR, PMIC_BIF_DEBOUNCE_EN_MASK,
	 PMIC_BIF_DEBOUNCE_EN_SHIFT},
	{PMIC_BIF_READ_EXPECT_NUM, PMIC_BIF_READ_EXPECT_NUM_ADDR, PMIC_BIF_READ_EXPECT_NUM_MASK,
	 PMIC_BIF_READ_EXPECT_NUM_SHIFT},
	{PMIC_BIF_TRASACT_TRIGGER, PMIC_BIF_TRASACT_TRIGGER_ADDR, PMIC_BIF_TRASACT_TRIGGER_MASK,
	 PMIC_BIF_TRASACT_TRIGGER_SHIFT},
	{PMIC_BIF_DATA_NUM, PMIC_BIF_DATA_NUM_ADDR, PMIC_BIF_DATA_NUM_MASK,
	 PMIC_BIF_DATA_NUM_SHIFT},
	{PMIC_BIF_RESPONSE, PMIC_BIF_RESPONSE_ADDR, PMIC_BIF_RESPONSE_MASK,
	 PMIC_BIF_RESPONSE_SHIFT},
	{PMIC_BIF_DATA_0, PMIC_BIF_DATA_0_ADDR, PMIC_BIF_DATA_0_MASK, PMIC_BIF_DATA_0_SHIFT},
	{PMIC_BIF_ACK_0, PMIC_BIF_ACK_0_ADDR, PMIC_BIF_ACK_0_MASK, PMIC_BIF_ACK_0_SHIFT},
	{PMIC_BIF_ERROR_0, PMIC_BIF_ERROR_0_ADDR, PMIC_BIF_ERROR_0_MASK, PMIC_BIF_ERROR_0_SHIFT},
	{PMIC_BIF_DATA_1, PMIC_BIF_DATA_1_ADDR, PMIC_BIF_DATA_1_MASK, PMIC_BIF_DATA_1_SHIFT},
	{PMIC_BIF_ACK_1, PMIC_BIF_ACK_1_ADDR, PMIC_BIF_ACK_1_MASK, PMIC_BIF_ACK_1_SHIFT},
	{PMIC_BIF_ERROR_1, PMIC_BIF_ERROR_1_ADDR, PMIC_BIF_ERROR_1_MASK, PMIC_BIF_ERROR_1_SHIFT},
	{PMIC_BIF_DATA_2, PMIC_BIF_DATA_2_ADDR, PMIC_BIF_DATA_2_MASK, PMIC_BIF_DATA_2_SHIFT},
	{PMIC_BIF_ACK_2, PMIC_BIF_ACK_2_ADDR, PMIC_BIF_ACK_2_MASK, PMIC_BIF_ACK_2_SHIFT},
	{PMIC_BIF_ERROR_2, PMIC_BIF_ERROR_2_ADDR, PMIC_BIF_ERROR_2_MASK, PMIC_BIF_ERROR_2_SHIFT},
	{PMIC_BIF_DATA_3, PMIC_BIF_DATA_3_ADDR, PMIC_BIF_DATA_3_MASK, PMIC_BIF_DATA_3_SHIFT},
	{PMIC_BIF_ACK_3, PMIC_BIF_ACK_3_ADDR, PMIC_BIF_ACK_3_MASK, PMIC_BIF_ACK_3_SHIFT},
	{PMIC_BIF_ERROR_3, PMIC_BIF_ERROR_3_ADDR, PMIC_BIF_ERROR_3_MASK, PMIC_BIF_ERROR_3_SHIFT},
	{PMIC_BIF_DATA_4, PMIC_BIF_DATA_4_ADDR, PMIC_BIF_DATA_4_MASK, PMIC_BIF_DATA_4_SHIFT},
	{PMIC_BIF_ACK_4, PMIC_BIF_ACK_4_ADDR, PMIC_BIF_ACK_4_MASK, PMIC_BIF_ACK_4_SHIFT},
	{PMIC_BIF_ERROR_4, PMIC_BIF_ERROR_4_ADDR, PMIC_BIF_ERROR_4_MASK, PMIC_BIF_ERROR_4_SHIFT},
	{PMIC_BIF_DATA_5, PMIC_BIF_DATA_5_ADDR, PMIC_BIF_DATA_5_MASK, PMIC_BIF_DATA_5_SHIFT},
	{PMIC_BIF_ACK_5, PMIC_BIF_ACK_5_ADDR, PMIC_BIF_ACK_5_MASK, PMIC_BIF_ACK_5_SHIFT},
	{PMIC_BIF_ERROR_5, PMIC_BIF_ERROR_5_ADDR, PMIC_BIF_ERROR_5_MASK, PMIC_BIF_ERROR_5_SHIFT},
	{PMIC_BIF_DATA_6, PMIC_BIF_DATA_6_ADDR, PMIC_BIF_DATA_6_MASK, PMIC_BIF_DATA_6_SHIFT},
	{PMIC_BIF_ACK_6, PMIC_BIF_ACK_6_ADDR, PMIC_BIF_ACK_6_MASK, PMIC_BIF_ACK_6_SHIFT},
	{PMIC_BIF_ERROR_6, PMIC_BIF_ERROR_6_ADDR, PMIC_BIF_ERROR_6_MASK, PMIC_BIF_ERROR_6_SHIFT},
	{PMIC_BIF_DATA_7, PMIC_BIF_DATA_7_ADDR, PMIC_BIF_DATA_7_MASK, PMIC_BIF_DATA_7_SHIFT},
	{PMIC_BIF_ACK_7, PMIC_BIF_ACK_7_ADDR, PMIC_BIF_ACK_7_MASK, PMIC_BIF_ACK_7_SHIFT},
	{PMIC_BIF_ERROR_7, PMIC_BIF_ERROR_7_ADDR, PMIC_BIF_ERROR_7_MASK, PMIC_BIF_ERROR_7_SHIFT},
	{PMIC_BIF_DATA_8, PMIC_BIF_DATA_8_ADDR, PMIC_BIF_DATA_8_MASK, PMIC_BIF_DATA_8_SHIFT},
	{PMIC_BIF_ACK_8, PMIC_BIF_ACK_8_ADDR, PMIC_BIF_ACK_8_MASK, PMIC_BIF_ACK_8_SHIFT},
	{PMIC_BIF_ERROR_8, PMIC_BIF_ERROR_8_ADDR, PMIC_BIF_ERROR_8_MASK, PMIC_BIF_ERROR_8_SHIFT},
	{PMIC_BIF_DATA_9, PMIC_BIF_DATA_9_ADDR, PMIC_BIF_DATA_9_MASK, PMIC_BIF_DATA_9_SHIFT},
	{PMIC_BIF_ACK_9, PMIC_BIF_ACK_9_ADDR, PMIC_BIF_ACK_9_MASK, PMIC_BIF_ACK_9_SHIFT},
	{PMIC_BIF_ERROR_9, PMIC_BIF_ERROR_9_ADDR, PMIC_BIF_ERROR_9_MASK, PMIC_BIF_ERROR_9_SHIFT},
	{PMIC_BIF_TEST_MODE0, PMIC_BIF_TEST_MODE0_ADDR, PMIC_BIF_TEST_MODE0_MASK,
	 PMIC_BIF_TEST_MODE0_SHIFT},
	{PMIC_BIF_TEST_MODE1, PMIC_BIF_TEST_MODE1_ADDR, PMIC_BIF_TEST_MODE1_MASK,
	 PMIC_BIF_TEST_MODE1_SHIFT},
	{PMIC_BIF_TEST_MODE2, PMIC_BIF_TEST_MODE2_ADDR, PMIC_BIF_TEST_MODE2_MASK,
	 PMIC_BIF_TEST_MODE2_SHIFT},
	{PMIC_BIF_TEST_MODE3, PMIC_BIF_TEST_MODE3_ADDR, PMIC_BIF_TEST_MODE3_MASK,
	 PMIC_BIF_TEST_MODE3_SHIFT},
	{PMIC_BIF_TEST_MODE4, PMIC_BIF_TEST_MODE4_ADDR, PMIC_BIF_TEST_MODE4_MASK,
	 PMIC_BIF_TEST_MODE4_SHIFT},
	{PMIC_BIF_TEST_MODE5, PMIC_BIF_TEST_MODE5_ADDR, PMIC_BIF_TEST_MODE5_MASK,
	 PMIC_BIF_TEST_MODE5_SHIFT},
	{PMIC_BIF_TEST_MODE6, PMIC_BIF_TEST_MODE6_ADDR, PMIC_BIF_TEST_MODE6_MASK,
	 PMIC_BIF_TEST_MODE6_SHIFT},
	{PMIC_BIF_TEST_MODE7, PMIC_BIF_TEST_MODE7_ADDR, PMIC_BIF_TEST_MODE7_MASK,
	 PMIC_BIF_TEST_MODE7_SHIFT},
	{PMIC_BIF_TEST_MODE8, PMIC_BIF_TEST_MODE8_ADDR, PMIC_BIF_TEST_MODE8_MASK,
	 PMIC_BIF_TEST_MODE8_SHIFT},
	{PMIC_BIF_BAT_LOST_SW, PMIC_BIF_BAT_LOST_SW_ADDR, PMIC_BIF_BAT_LOST_SW_MASK,
	 PMIC_BIF_BAT_LOST_SW_SHIFT},
	{PMIC_BIF_RX_DATA_SW, PMIC_BIF_RX_DATA_SW_ADDR, PMIC_BIF_RX_DATA_SW_MASK,
	 PMIC_BIF_RX_DATA_SW_SHIFT},
	{PMIC_BIF_TX_DATA_SW, PMIC_BIF_TX_DATA_SW_ADDR, PMIC_BIF_TX_DATA_SW_MASK,
	 PMIC_BIF_TX_DATA_SW_SHIFT},
	{PMIC_BIF_RX_EN_SW, PMIC_BIF_RX_EN_SW_ADDR, PMIC_BIF_RX_EN_SW_MASK,
	 PMIC_BIF_RX_EN_SW_SHIFT},
	{PMIC_BIF_TX_EN_SW, PMIC_BIF_TX_EN_SW_ADDR, PMIC_BIF_TX_EN_SW_MASK,
	 PMIC_BIF_TX_EN_SW_SHIFT},
	{PMIC_BIF_BACK_NORMAL, PMIC_BIF_BACK_NORMAL_ADDR, PMIC_BIF_BACK_NORMAL_MASK,
	 PMIC_BIF_BACK_NORMAL_SHIFT},
	{PMIC_BIF_IRQ_CLR, PMIC_BIF_IRQ_CLR_ADDR, PMIC_BIF_IRQ_CLR_MASK, PMIC_BIF_IRQ_CLR_SHIFT},
	{PMIC_BIF_IRQ, PMIC_BIF_IRQ_ADDR, PMIC_BIF_IRQ_MASK, PMIC_BIF_IRQ_SHIFT},
	{PMIC_BIF_TIMEOUT, PMIC_BIF_TIMEOUT_ADDR, PMIC_BIF_TIMEOUT_MASK, PMIC_BIF_TIMEOUT_SHIFT},
	{PMIC_BIF_BAT_UNDET, PMIC_BIF_BAT_UNDET_ADDR, PMIC_BIF_BAT_UNDET_MASK,
	 PMIC_BIF_BAT_UNDET_SHIFT},
	{PMIC_BIF_TOTAL_VALID, PMIC_BIF_TOTAL_VALID_ADDR, PMIC_BIF_TOTAL_VALID_MASK,
	 PMIC_BIF_TOTAL_VALID_SHIFT},
	{PMIC_BIF_BUS_STATUS, PMIC_BIF_BUS_STATUS_ADDR, PMIC_BIF_BUS_STATUS_MASK,
	 PMIC_BIF_BUS_STATUS_SHIFT},
	{PMIC_BIF_POWER_UP_COUNT, PMIC_BIF_POWER_UP_COUNT_ADDR, PMIC_BIF_POWER_UP_COUNT_MASK,
	 PMIC_BIF_POWER_UP_COUNT_SHIFT},
	{PMIC_BIF_POWER_UP, PMIC_BIF_POWER_UP_ADDR, PMIC_BIF_POWER_UP_MASK,
	 PMIC_BIF_POWER_UP_SHIFT},
	{PMIC_BIF_RX_ERROR_UNKNOWN, PMIC_BIF_RX_ERROR_UNKNOWN_ADDR, PMIC_BIF_RX_ERROR_UNKNOWN_MASK,
	 PMIC_BIF_RX_ERROR_UNKNOWN_SHIFT},
	{PMIC_BIF_RX_ERROR_INSUFF, PMIC_BIF_RX_ERROR_INSUFF_ADDR, PMIC_BIF_RX_ERROR_INSUFF_MASK,
	 PMIC_BIF_RX_ERROR_INSUFF_SHIFT},
	{PMIC_BIF_RX_ERROR_LOWPHASE, PMIC_BIF_RX_ERROR_LOWPHASE_ADDR,
	 PMIC_BIF_RX_ERROR_LOWPHASE_MASK, PMIC_BIF_RX_ERROR_LOWPHASE_SHIFT},
	{PMIC_BIF_RX_STATE, PMIC_BIF_RX_STATE_ADDR, PMIC_BIF_RX_STATE_MASK,
	 PMIC_BIF_RX_STATE_SHIFT},
	{PMIC_BIF_FLOW_CTL_STATE, PMIC_BIF_FLOW_CTL_STATE_ADDR, PMIC_BIF_FLOW_CTL_STATE_MASK,
	 PMIC_BIF_FLOW_CTL_STATE_SHIFT},
	{PMIC_BIF_TX_STATE, PMIC_BIF_TX_STATE_ADDR, PMIC_BIF_TX_STATE_MASK,
	 PMIC_BIF_TX_STATE_SHIFT},
	{PMIC_AD_QI_BIF_RX_DATA, PMIC_AD_QI_BIF_RX_DATA_ADDR, PMIC_AD_QI_BIF_RX_DATA_MASK,
	 PMIC_AD_QI_BIF_RX_DATA_SHIFT},
	{PMIC_DA_QI_BIF_RX_EN, PMIC_DA_QI_BIF_RX_EN_ADDR, PMIC_DA_QI_BIF_RX_EN_MASK,
	 PMIC_DA_QI_BIF_RX_EN_SHIFT},
	{PMIC_DA_QI_BIF_TX_DATA, PMIC_DA_QI_BIF_TX_DATA_ADDR, PMIC_DA_QI_BIF_TX_DATA_MASK,
	 PMIC_DA_QI_BIF_TX_DATA_SHIFT},
	{PMIC_DA_QI_BIF_TX_EN, PMIC_DA_QI_BIF_TX_EN_ADDR, PMIC_DA_QI_BIF_TX_EN_MASK,
	 PMIC_DA_QI_BIF_TX_EN_SHIFT},
	{PMIC_BIF_TX_DATA_FIANL, PMIC_BIF_TX_DATA_FIANL_ADDR, PMIC_BIF_TX_DATA_FIANL_MASK,
	 PMIC_BIF_TX_DATA_FIANL_SHIFT},
	{PMIC_BIF_RX_DATA_SAMPLING, PMIC_BIF_RX_DATA_SAMPLING_ADDR, PMIC_BIF_RX_DATA_SAMPLING_MASK,
	 PMIC_BIF_RX_DATA_SAMPLING_SHIFT},
	{PMIC_BIF_RX_DATA_RECOVERY, PMIC_BIF_RX_DATA_RECOVERY_ADDR, PMIC_BIF_RX_DATA_RECOVERY_MASK,
	 PMIC_BIF_RX_DATA_RECOVERY_SHIFT},
	{PMIC_RG_BATON_HT_EN, PMIC_RG_BATON_HT_EN_ADDR, PMIC_RG_BATON_HT_EN_MASK,
	 PMIC_RG_BATON_HT_EN_SHIFT},
	{PMIC_RG_BATON_HT_EN_DLY_TIME, PMIC_RG_BATON_HT_EN_DLY_TIME_ADDR,
	 PMIC_RG_BATON_HT_EN_DLY_TIME_MASK, PMIC_RG_BATON_HT_EN_DLY_TIME_SHIFT},
	{PMIC_BIF_TIMEOUT_SET, PMIC_BIF_TIMEOUT_SET_ADDR, PMIC_BIF_TIMEOUT_SET_MASK,
	 PMIC_BIF_TIMEOUT_SET_SHIFT},
	{PMIC_BIF_RX_DEG_WND, PMIC_BIF_RX_DEG_WND_ADDR, PMIC_BIF_RX_DEG_WND_MASK,
	 PMIC_BIF_RX_DEG_WND_SHIFT},
	{PMIC_BIF_RX_DEG_EN, PMIC_BIF_RX_DEG_EN_ADDR, PMIC_BIF_RX_DEG_EN_MASK,
	 PMIC_BIF_RX_DEG_EN_SHIFT},
	{PMIC_BIF_RSV1, PMIC_BIF_RSV1_ADDR, PMIC_BIF_RSV1_MASK, PMIC_BIF_RSV1_SHIFT},
	{PMIC_BIF_RSV0, PMIC_BIF_RSV0_ADDR, PMIC_BIF_RSV0_MASK, PMIC_BIF_RSV0_SHIFT},
	{PMIC_DA_QI_BATON_HT_EN, PMIC_DA_QI_BATON_HT_EN_ADDR, PMIC_DA_QI_BATON_HT_EN_MASK,
	 PMIC_DA_QI_BATON_HT_EN_SHIFT},
	{PMIC_RG_BATON_DEBOUNCE_WND, PMIC_RG_BATON_DEBOUNCE_WND_ADDR,
	 PMIC_RG_BATON_DEBOUNCE_WND_MASK, PMIC_RG_BATON_DEBOUNCE_WND_SHIFT},
	{PMIC_RG_BATON_DEBOUNCE_THD, PMIC_RG_BATON_DEBOUNCE_THD_ADDR,
	 PMIC_RG_BATON_DEBOUNCE_THD_MASK, PMIC_RG_BATON_DEBOUNCE_THD_SHIFT},
	{PMIC_RG_BATON_VBIF_STB, PMIC_RG_BATON_VBIF_STB_ADDR, PMIC_RG_BATON_VBIF_STB_MASK,
	 PMIC_RG_BATON_VBIF_STB_SHIFT},
	{PMIC_RG_BATON_CHRDET_TEST_MODE, PMIC_RG_BATON_CHRDET_TEST_MODE_ADDR,
	 PMIC_RG_BATON_CHRDET_TEST_MODE_MASK, PMIC_RG_BATON_CHRDET_TEST_MODE_SHIFT},
	{PMIC_RG_BATON_UNDET_TEST_MODE, PMIC_RG_BATON_UNDET_TEST_MODE_ADDR,
	 PMIC_RG_BATON_UNDET_TEST_MODE_MASK, PMIC_RG_BATON_UNDET_TEST_MODE_SHIFT},
	{PMIC_RG_BATON_AUXADC_TEST_MODE, PMIC_RG_BATON_AUXADC_TEST_MODE_ADDR,
	 PMIC_RG_BATON_AUXADC_TEST_MODE_MASK, PMIC_RG_BATON_AUXADC_TEST_MODE_SHIFT},
	{PMIC_RG_BATON_FGADC_TEST_MODE, PMIC_RG_BATON_FGADC_TEST_MODE_ADDR,
	 PMIC_RG_BATON_FGADC_TEST_MODE_MASK, PMIC_RG_BATON_FGADC_TEST_MODE_SHIFT},
	{PMIC_RG_BATON_RTC_TEST_MODE, PMIC_RG_BATON_RTC_TEST_MODE_ADDR,
	 PMIC_RG_BATON_RTC_TEST_MODE_MASK, PMIC_RG_BATON_RTC_TEST_MODE_SHIFT},
	{PMIC_RG_BATON_BIF_TEST_MODE, PMIC_RG_BATON_BIF_TEST_MODE_ADDR,
	 PMIC_RG_BATON_BIF_TEST_MODE_MASK, PMIC_RG_BATON_BIF_TEST_MODE_SHIFT},
	{PMIC_RG_BATON_RSV, PMIC_RG_BATON_RSV_ADDR, PMIC_RG_BATON_RSV_MASK,
	 PMIC_RG_BATON_RSV_SHIFT},
	{PMIC_RG_BATON_CHRDET_SW, PMIC_RG_BATON_CHRDET_SW_ADDR, PMIC_RG_BATON_CHRDET_SW_MASK,
	 PMIC_RG_BATON_CHRDET_SW_SHIFT},
	{PMIC_RG_BATON_UNDET_SW, PMIC_RG_BATON_UNDET_SW_ADDR, PMIC_RG_BATON_UNDET_SW_MASK,
	 PMIC_RG_BATON_UNDET_SW_SHIFT},
	{PMIC_RG_BATON_AUXADC_SW, PMIC_RG_BATON_AUXADC_SW_ADDR, PMIC_RG_BATON_AUXADC_SW_MASK,
	 PMIC_RG_BATON_AUXADC_SW_SHIFT},
	{PMIC_RG_BATON_FGADC_SW, PMIC_RG_BATON_FGADC_SW_ADDR, PMIC_RG_BATON_FGADC_SW_MASK,
	 PMIC_RG_BATON_FGADC_SW_SHIFT},
	{PMIC_RG_BATON_RTC_SW, PMIC_RG_BATON_RTC_SW_ADDR, PMIC_RG_BATON_RTC_SW_MASK,
	 PMIC_RG_BATON_RTC_SW_SHIFT},
	{PMIC_RG_BATON_BIF_SW, PMIC_RG_BATON_BIF_SW_ADDR, PMIC_RG_BATON_BIF_SW_MASK,
	 PMIC_RG_BATON_BIF_SW_SHIFT},
	{PMIC_RG_BATON_RSV_SW, PMIC_RG_BATON_RSV_SW_ADDR, PMIC_RG_BATON_RSV_SW_MASK,
	 PMIC_RG_BATON_RSV_SW_SHIFT},
	{PMIC_BATON_STATUS, PMIC_BATON_STATUS_ADDR, PMIC_BATON_STATUS_MASK,
	 PMIC_BATON_STATUS_SHIFT},
	{PMIC_BATON_AUXADC_SET, PMIC_BATON_AUXADC_SET_ADDR, PMIC_BATON_AUXADC_SET_MASK,
	 PMIC_BATON_AUXADC_SET_SHIFT},
	{PMIC_BATON_DEB_VALID, PMIC_BATON_DEB_VALID_ADDR, PMIC_BATON_DEB_VALID_MASK,
	 PMIC_BATON_DEB_VALID_SHIFT},
	{PMIC_BATON_BIF_STATUS, PMIC_BATON_BIF_STATUS_ADDR, PMIC_BATON_BIF_STATUS_MASK,
	 PMIC_BATON_BIF_STATUS_SHIFT},
	{PMIC_BATON_RTC_STATUS, PMIC_BATON_RTC_STATUS_ADDR, PMIC_BATON_RTC_STATUS_MASK,
	 PMIC_BATON_RTC_STATUS_SHIFT},
	{PMIC_BATON_FGADC_STATUS, PMIC_BATON_FGADC_STATUS_ADDR, PMIC_BATON_FGADC_STATUS_MASK,
	 PMIC_BATON_FGADC_STATUS_SHIFT},
	{PMIC_BATON_AUXADC_TRIG, PMIC_BATON_AUXADC_TRIG_ADDR, PMIC_BATON_AUXADC_TRIG_MASK,
	 PMIC_BATON_AUXADC_TRIG_SHIFT},
	{PMIC_BATON_CHRDET_DEB, PMIC_BATON_CHRDET_DEB_ADDR, PMIC_BATON_CHRDET_DEB_MASK,
	 PMIC_BATON_CHRDET_DEB_SHIFT},
	{PMIC_BATON_IVGEN_ENB, PMIC_BATON_IVGEN_ENB_ADDR, PMIC_BATON_IVGEN_ENB_MASK,
	 PMIC_BATON_IVGEN_ENB_SHIFT},
	{PMIC_BATON_VBIF28_STB, PMIC_BATON_VBIF28_STB_ADDR, PMIC_BATON_VBIF28_STB_MASK,
	 PMIC_BATON_VBIF28_STB_SHIFT},
	{PMIC_BATON_VBIF28_EN, PMIC_BATON_VBIF28_EN_ADDR, PMIC_BATON_VBIF28_EN_MASK,
	 PMIC_BATON_VBIF28_EN_SHIFT},
	{PMIC_BATON_RSV_0, PMIC_BATON_RSV_0_ADDR, PMIC_BATON_RSV_0_MASK, PMIC_BATON_RSV_0_SHIFT},
	{PMIC_RG_BIF_BATID_SW_EN, PMIC_RG_BIF_BATID_SW_EN_ADDR, PMIC_RG_BIF_BATID_SW_EN_MASK,
	 PMIC_RG_BIF_BATID_SW_EN_SHIFT},
	{PMIC_RG_OTP_PA, PMIC_RG_OTP_PA_ADDR, PMIC_RG_OTP_PA_MASK, PMIC_RG_OTP_PA_SHIFT},
	{PMIC_RG_OTP_PDIN, PMIC_RG_OTP_PDIN_ADDR, PMIC_RG_OTP_PDIN_MASK, PMIC_RG_OTP_PDIN_SHIFT},
	{PMIC_RG_OTP_PTM, PMIC_RG_OTP_PTM_ADDR, PMIC_RG_OTP_PTM_MASK, PMIC_RG_OTP_PTM_SHIFT},
	{PMIC_RG_OTP_PWE, PMIC_RG_OTP_PWE_ADDR, PMIC_RG_OTP_PWE_MASK, PMIC_RG_OTP_PWE_SHIFT},
	{PMIC_RG_OTP_PPROG, PMIC_RG_OTP_PPROG_ADDR, PMIC_RG_OTP_PPROG_MASK,
	 PMIC_RG_OTP_PPROG_SHIFT},
	{PMIC_RG_OTP_PWE_SRC, PMIC_RG_OTP_PWE_SRC_ADDR, PMIC_RG_OTP_PWE_SRC_MASK,
	 PMIC_RG_OTP_PWE_SRC_SHIFT},
	{PMIC_RG_OTP_PROG_PKEY, PMIC_RG_OTP_PROG_PKEY_ADDR, PMIC_RG_OTP_PROG_PKEY_MASK,
	 PMIC_RG_OTP_PROG_PKEY_SHIFT},
	{PMIC_RG_OTP_RD_PKEY, PMIC_RG_OTP_RD_PKEY_ADDR, PMIC_RG_OTP_RD_PKEY_MASK,
	 PMIC_RG_OTP_RD_PKEY_SHIFT},
	{PMIC_RG_OTP_RD_TRIG, PMIC_RG_OTP_RD_TRIG_ADDR, PMIC_RG_OTP_RD_TRIG_MASK,
	 PMIC_RG_OTP_RD_TRIG_SHIFT},
	{PMIC_RG_RD_RDY_BYPASS, PMIC_RG_RD_RDY_BYPASS_ADDR, PMIC_RG_RD_RDY_BYPASS_MASK,
	 PMIC_RG_RD_RDY_BYPASS_SHIFT},
	{PMIC_RG_SKIP_OTP_OUT, PMIC_RG_SKIP_OTP_OUT_ADDR, PMIC_RG_SKIP_OTP_OUT_MASK,
	 PMIC_RG_SKIP_OTP_OUT_SHIFT},
	{PMIC_RG_OTP_RD_SW, PMIC_RG_OTP_RD_SW_ADDR, PMIC_RG_OTP_RD_SW_MASK,
	 PMIC_RG_OTP_RD_SW_SHIFT},
	{PMIC_RG_OTP_DOUT_SW, PMIC_RG_OTP_DOUT_SW_ADDR, PMIC_RG_OTP_DOUT_SW_MASK,
	 PMIC_RG_OTP_DOUT_SW_SHIFT},
	{PMIC_RG_OTP_RD_BUSY, PMIC_RG_OTP_RD_BUSY_ADDR, PMIC_RG_OTP_RD_BUSY_MASK,
	 PMIC_RG_OTP_RD_BUSY_SHIFT},
	{PMIC_RG_OTP_RD_ACK, PMIC_RG_OTP_RD_ACK_ADDR, PMIC_RG_OTP_RD_ACK_MASK,
	 PMIC_RG_OTP_RD_ACK_SHIFT},
	{PMIC_RG_OTP_PA_SW, PMIC_RG_OTP_PA_SW_ADDR, PMIC_RG_OTP_PA_SW_MASK,
	 PMIC_RG_OTP_PA_SW_SHIFT},
	{PMIC_RG_OTP_DOUT_0_15, PMIC_RG_OTP_DOUT_0_15_ADDR, PMIC_RG_OTP_DOUT_0_15_MASK,
	 PMIC_RG_OTP_DOUT_0_15_SHIFT},
	{PMIC_RG_OTP_DOUT_16_31, PMIC_RG_OTP_DOUT_16_31_ADDR, PMIC_RG_OTP_DOUT_16_31_MASK,
	 PMIC_RG_OTP_DOUT_16_31_SHIFT},
	{PMIC_RG_OTP_DOUT_32_47, PMIC_RG_OTP_DOUT_32_47_ADDR, PMIC_RG_OTP_DOUT_32_47_MASK,
	 PMIC_RG_OTP_DOUT_32_47_SHIFT},
	{PMIC_RG_OTP_DOUT_48_63, PMIC_RG_OTP_DOUT_48_63_ADDR, PMIC_RG_OTP_DOUT_48_63_MASK,
	 PMIC_RG_OTP_DOUT_48_63_SHIFT},
	{PMIC_RG_OTP_DOUT_64_79, PMIC_RG_OTP_DOUT_64_79_ADDR, PMIC_RG_OTP_DOUT_64_79_MASK,
	 PMIC_RG_OTP_DOUT_64_79_SHIFT},
	{PMIC_RG_OTP_DOUT_80_95, PMIC_RG_OTP_DOUT_80_95_ADDR, PMIC_RG_OTP_DOUT_80_95_MASK,
	 PMIC_RG_OTP_DOUT_80_95_SHIFT},
	{PMIC_RG_OTP_DOUT_96_111, PMIC_RG_OTP_DOUT_96_111_ADDR, PMIC_RG_OTP_DOUT_96_111_MASK,
	 PMIC_RG_OTP_DOUT_96_111_SHIFT},
	{PMIC_RG_OTP_DOUT_112_127, PMIC_RG_OTP_DOUT_112_127_ADDR, PMIC_RG_OTP_DOUT_112_127_MASK,
	 PMIC_RG_OTP_DOUT_112_127_SHIFT},
	{PMIC_RG_OTP_DOUT_128_143, PMIC_RG_OTP_DOUT_128_143_ADDR, PMIC_RG_OTP_DOUT_128_143_MASK,
	 PMIC_RG_OTP_DOUT_128_143_SHIFT},
	{PMIC_RG_OTP_DOUT_144_159, PMIC_RG_OTP_DOUT_144_159_ADDR, PMIC_RG_OTP_DOUT_144_159_MASK,
	 PMIC_RG_OTP_DOUT_144_159_SHIFT},
	{PMIC_RG_OTP_DOUT_160_175, PMIC_RG_OTP_DOUT_160_175_ADDR, PMIC_RG_OTP_DOUT_160_175_MASK,
	 PMIC_RG_OTP_DOUT_160_175_SHIFT},
	{PMIC_RG_OTP_DOUT_176_191, PMIC_RG_OTP_DOUT_176_191_ADDR, PMIC_RG_OTP_DOUT_176_191_MASK,
	 PMIC_RG_OTP_DOUT_176_191_SHIFT},
	{PMIC_RG_OTP_DOUT_192_207, PMIC_RG_OTP_DOUT_192_207_ADDR, PMIC_RG_OTP_DOUT_192_207_MASK,
	 PMIC_RG_OTP_DOUT_192_207_SHIFT},
	{PMIC_RG_OTP_DOUT_208_223, PMIC_RG_OTP_DOUT_208_223_ADDR, PMIC_RG_OTP_DOUT_208_223_MASK,
	 PMIC_RG_OTP_DOUT_208_223_SHIFT},
	{PMIC_RG_OTP_DOUT_224_239, PMIC_RG_OTP_DOUT_224_239_ADDR, PMIC_RG_OTP_DOUT_224_239_MASK,
	 PMIC_RG_OTP_DOUT_224_239_SHIFT},
	{PMIC_RG_OTP_DOUT_240_255, PMIC_RG_OTP_DOUT_240_255_ADDR, PMIC_RG_OTP_DOUT_240_255_MASK,
	 PMIC_RG_OTP_DOUT_240_255_SHIFT},
	{PMIC_RG_OTP_DOUT_256_271, PMIC_RG_OTP_DOUT_256_271_ADDR, PMIC_RG_OTP_DOUT_256_271_MASK,
	 PMIC_RG_OTP_DOUT_256_271_SHIFT},
	{PMIC_RG_OTP_DOUT_272_287, PMIC_RG_OTP_DOUT_272_287_ADDR, PMIC_RG_OTP_DOUT_272_287_MASK,
	 PMIC_RG_OTP_DOUT_272_287_SHIFT},
	{PMIC_RG_OTP_DOUT_288_303, PMIC_RG_OTP_DOUT_288_303_ADDR, PMIC_RG_OTP_DOUT_288_303_MASK,
	 PMIC_RG_OTP_DOUT_288_303_SHIFT},
	{PMIC_RG_OTP_DOUT_304_319, PMIC_RG_OTP_DOUT_304_319_ADDR, PMIC_RG_OTP_DOUT_304_319_MASK,
	 PMIC_RG_OTP_DOUT_304_319_SHIFT},
	{PMIC_RG_OTP_DOUT_320_335, PMIC_RG_OTP_DOUT_320_335_ADDR, PMIC_RG_OTP_DOUT_320_335_MASK,
	 PMIC_RG_OTP_DOUT_320_335_SHIFT},
	{PMIC_RG_OTP_DOUT_336_351, PMIC_RG_OTP_DOUT_336_351_ADDR, PMIC_RG_OTP_DOUT_336_351_MASK,
	 PMIC_RG_OTP_DOUT_336_351_SHIFT},
	{PMIC_RG_OTP_DOUT_352_367, PMIC_RG_OTP_DOUT_352_367_ADDR, PMIC_RG_OTP_DOUT_352_367_MASK,
	 PMIC_RG_OTP_DOUT_352_367_SHIFT},
	{PMIC_RG_OTP_DOUT_368_383, PMIC_RG_OTP_DOUT_368_383_ADDR, PMIC_RG_OTP_DOUT_368_383_MASK,
	 PMIC_RG_OTP_DOUT_368_383_SHIFT},
	{PMIC_RG_OTP_DOUT_384_399, PMIC_RG_OTP_DOUT_384_399_ADDR, PMIC_RG_OTP_DOUT_384_399_MASK,
	 PMIC_RG_OTP_DOUT_384_399_SHIFT},
	{PMIC_RG_OTP_DOUT_400_415, PMIC_RG_OTP_DOUT_400_415_ADDR, PMIC_RG_OTP_DOUT_400_415_MASK,
	 PMIC_RG_OTP_DOUT_400_415_SHIFT},
	{PMIC_RG_OTP_DOUT_416_431, PMIC_RG_OTP_DOUT_416_431_ADDR, PMIC_RG_OTP_DOUT_416_431_MASK,
	 PMIC_RG_OTP_DOUT_416_431_SHIFT},
	{PMIC_RG_OTP_DOUT_432_447, PMIC_RG_OTP_DOUT_432_447_ADDR, PMIC_RG_OTP_DOUT_432_447_MASK,
	 PMIC_RG_OTP_DOUT_432_447_SHIFT},
	{PMIC_RG_OTP_DOUT_448_463, PMIC_RG_OTP_DOUT_448_463_ADDR, PMIC_RG_OTP_DOUT_448_463_MASK,
	 PMIC_RG_OTP_DOUT_448_463_SHIFT},
	{PMIC_RG_OTP_DOUT_464_479, PMIC_RG_OTP_DOUT_464_479_ADDR, PMIC_RG_OTP_DOUT_464_479_MASK,
	 PMIC_RG_OTP_DOUT_464_479_SHIFT},
	{PMIC_RG_OTP_DOUT_480_495, PMIC_RG_OTP_DOUT_480_495_ADDR, PMIC_RG_OTP_DOUT_480_495_MASK,
	 PMIC_RG_OTP_DOUT_480_495_SHIFT},
	{PMIC_RG_OTP_DOUT_496_511, PMIC_RG_OTP_DOUT_496_511_ADDR, PMIC_RG_OTP_DOUT_496_511_MASK,
	 PMIC_RG_OTP_DOUT_496_511_SHIFT},
	{PMIC_RG_OTP_DOUT_512_527, PMIC_RG_OTP_DOUT_512_527_ADDR, PMIC_RG_OTP_DOUT_512_527_MASK,
	 PMIC_RG_OTP_DOUT_512_527_SHIFT},
	{PMIC_RG_OTP_DOUT_528_543, PMIC_RG_OTP_DOUT_528_543_ADDR, PMIC_RG_OTP_DOUT_528_543_MASK,
	 PMIC_RG_OTP_DOUT_528_543_SHIFT},
	{PMIC_RG_OTP_DOUT_544_559, PMIC_RG_OTP_DOUT_544_559_ADDR, PMIC_RG_OTP_DOUT_544_559_MASK,
	 PMIC_RG_OTP_DOUT_544_559_SHIFT},
	{PMIC_RG_OTP_DOUT_560_575, PMIC_RG_OTP_DOUT_560_575_ADDR, PMIC_RG_OTP_DOUT_560_575_MASK,
	 PMIC_RG_OTP_DOUT_560_575_SHIFT},
	{PMIC_RG_OTP_DOUT_576_591, PMIC_RG_OTP_DOUT_576_591_ADDR, PMIC_RG_OTP_DOUT_576_591_MASK,
	 PMIC_RG_OTP_DOUT_576_591_SHIFT},
	{PMIC_RG_OTP_DOUT_592_607, PMIC_RG_OTP_DOUT_592_607_ADDR, PMIC_RG_OTP_DOUT_592_607_MASK,
	 PMIC_RG_OTP_DOUT_592_607_SHIFT},
	{PMIC_RG_OTP_DOUT_608_623, PMIC_RG_OTP_DOUT_608_623_ADDR, PMIC_RG_OTP_DOUT_608_623_MASK,
	 PMIC_RG_OTP_DOUT_608_623_SHIFT},
	{PMIC_RG_OTP_DOUT_624_639, PMIC_RG_OTP_DOUT_624_639_ADDR, PMIC_RG_OTP_DOUT_624_639_MASK,
	 PMIC_RG_OTP_DOUT_624_639_SHIFT},
	{PMIC_RG_OTP_DOUT_640_655, PMIC_RG_OTP_DOUT_640_655_ADDR, PMIC_RG_OTP_DOUT_640_655_MASK,
	 PMIC_RG_OTP_DOUT_640_655_SHIFT},
	{PMIC_RG_OTP_DOUT_656_671, PMIC_RG_OTP_DOUT_656_671_ADDR, PMIC_RG_OTP_DOUT_656_671_MASK,
	 PMIC_RG_OTP_DOUT_656_671_SHIFT},
	{PMIC_RG_OTP_DOUT_672_687, PMIC_RG_OTP_DOUT_672_687_ADDR, PMIC_RG_OTP_DOUT_672_687_MASK,
	 PMIC_RG_OTP_DOUT_672_687_SHIFT},
	{PMIC_RG_OTP_DOUT_688_703, PMIC_RG_OTP_DOUT_688_703_ADDR, PMIC_RG_OTP_DOUT_688_703_MASK,
	 PMIC_RG_OTP_DOUT_688_703_SHIFT},
	{PMIC_RG_OTP_DOUT_704_719, PMIC_RG_OTP_DOUT_704_719_ADDR, PMIC_RG_OTP_DOUT_704_719_MASK,
	 PMIC_RG_OTP_DOUT_704_719_SHIFT},
	{PMIC_RG_OTP_DOUT_720_735, PMIC_RG_OTP_DOUT_720_735_ADDR, PMIC_RG_OTP_DOUT_720_735_MASK,
	 PMIC_RG_OTP_DOUT_720_735_SHIFT},
	{PMIC_RG_OTP_DOUT_736_751, PMIC_RG_OTP_DOUT_736_751_ADDR, PMIC_RG_OTP_DOUT_736_751_MASK,
	 PMIC_RG_OTP_DOUT_736_751_SHIFT},
	{PMIC_RG_OTP_DOUT_752_767, PMIC_RG_OTP_DOUT_752_767_ADDR, PMIC_RG_OTP_DOUT_752_767_MASK,
	 PMIC_RG_OTP_DOUT_752_767_SHIFT},
	{PMIC_RG_OTP_DOUT_768_783, PMIC_RG_OTP_DOUT_768_783_ADDR, PMIC_RG_OTP_DOUT_768_783_MASK,
	 PMIC_RG_OTP_DOUT_768_783_SHIFT},
	{PMIC_RG_OTP_DOUT_784_799, PMIC_RG_OTP_DOUT_784_799_ADDR, PMIC_RG_OTP_DOUT_784_799_MASK,
	 PMIC_RG_OTP_DOUT_784_799_SHIFT},
	{PMIC_RG_OTP_DOUT_800_815, PMIC_RG_OTP_DOUT_800_815_ADDR, PMIC_RG_OTP_DOUT_800_815_MASK,
	 PMIC_RG_OTP_DOUT_800_815_SHIFT},
	{PMIC_RG_OTP_DOUT_816_831, PMIC_RG_OTP_DOUT_816_831_ADDR, PMIC_RG_OTP_DOUT_816_831_MASK,
	 PMIC_RG_OTP_DOUT_816_831_SHIFT},
	{PMIC_RG_OTP_DOUT_832_847, PMIC_RG_OTP_DOUT_832_847_ADDR, PMIC_RG_OTP_DOUT_832_847_MASK,
	 PMIC_RG_OTP_DOUT_832_847_SHIFT},
	{PMIC_RG_OTP_DOUT_848_863, PMIC_RG_OTP_DOUT_848_863_ADDR, PMIC_RG_OTP_DOUT_848_863_MASK,
	 PMIC_RG_OTP_DOUT_848_863_SHIFT},
	{PMIC_RG_OTP_DOUT_864_879, PMIC_RG_OTP_DOUT_864_879_ADDR, PMIC_RG_OTP_DOUT_864_879_MASK,
	 PMIC_RG_OTP_DOUT_864_879_SHIFT},
	{PMIC_RG_OTP_DOUT_880_895, PMIC_RG_OTP_DOUT_880_895_ADDR, PMIC_RG_OTP_DOUT_880_895_MASK,
	 PMIC_RG_OTP_DOUT_880_895_SHIFT},
	{PMIC_RG_OTP_DOUT_896_911, PMIC_RG_OTP_DOUT_896_911_ADDR, PMIC_RG_OTP_DOUT_896_911_MASK,
	 PMIC_RG_OTP_DOUT_896_911_SHIFT},
	{PMIC_RG_OTP_DOUT_912_927, PMIC_RG_OTP_DOUT_912_927_ADDR, PMIC_RG_OTP_DOUT_912_927_MASK,
	 PMIC_RG_OTP_DOUT_912_927_SHIFT},
	{PMIC_RG_OTP_DOUT_928_943, PMIC_RG_OTP_DOUT_928_943_ADDR, PMIC_RG_OTP_DOUT_928_943_MASK,
	 PMIC_RG_OTP_DOUT_928_943_SHIFT},
	{PMIC_RG_OTP_DOUT_944_959, PMIC_RG_OTP_DOUT_944_959_ADDR, PMIC_RG_OTP_DOUT_944_959_MASK,
	 PMIC_RG_OTP_DOUT_944_959_SHIFT},
	{PMIC_RG_OTP_DOUT_960_975, PMIC_RG_OTP_DOUT_960_975_ADDR, PMIC_RG_OTP_DOUT_960_975_MASK,
	 PMIC_RG_OTP_DOUT_960_975_SHIFT},
	{PMIC_RG_OTP_DOUT_976_991, PMIC_RG_OTP_DOUT_976_991_ADDR, PMIC_RG_OTP_DOUT_976_991_MASK,
	 PMIC_RG_OTP_DOUT_976_991_SHIFT},
	{PMIC_RG_OTP_DOUT_992_1007, PMIC_RG_OTP_DOUT_992_1007_ADDR, PMIC_RG_OTP_DOUT_992_1007_MASK,
	 PMIC_RG_OTP_DOUT_992_1007_SHIFT},
	{PMIC_RG_OTP_DOUT_1008_1023, PMIC_RG_OTP_DOUT_1008_1023_ADDR,
	 PMIC_RG_OTP_DOUT_1008_1023_MASK, PMIC_RG_OTP_DOUT_1008_1023_SHIFT},
	{PMIC_RG_OTP_VAL_0_15, PMIC_RG_OTP_VAL_0_15_ADDR, PMIC_RG_OTP_VAL_0_15_MASK,
	 PMIC_RG_OTP_VAL_0_15_SHIFT},
	{PMIC_RG_OTP_VAL_16_31, PMIC_RG_OTP_VAL_16_31_ADDR, PMIC_RG_OTP_VAL_16_31_MASK,
	 PMIC_RG_OTP_VAL_16_31_SHIFT},
	{PMIC_RG_OTP_VAL_32_47, PMIC_RG_OTP_VAL_32_47_ADDR, PMIC_RG_OTP_VAL_32_47_MASK,
	 PMIC_RG_OTP_VAL_32_47_SHIFT},
	{PMIC_RG_OTP_VAL_48_63, PMIC_RG_OTP_VAL_48_63_ADDR, PMIC_RG_OTP_VAL_48_63_MASK,
	 PMIC_RG_OTP_VAL_48_63_SHIFT},
	{PMIC_RG_OTP_VAL_64_79, PMIC_RG_OTP_VAL_64_79_ADDR, PMIC_RG_OTP_VAL_64_79_MASK,
	 PMIC_RG_OTP_VAL_64_79_SHIFT},
	{PMIC_RG_OTP_VAL_80_95, PMIC_RG_OTP_VAL_80_95_ADDR, PMIC_RG_OTP_VAL_80_95_MASK,
	 PMIC_RG_OTP_VAL_80_95_SHIFT},
	{PMIC_RG_OTP_VAL_96_111, PMIC_RG_OTP_VAL_96_111_ADDR, PMIC_RG_OTP_VAL_96_111_MASK,
	 PMIC_RG_OTP_VAL_96_111_SHIFT},
	{PMIC_RG_OTP_VAL_112_127, PMIC_RG_OTP_VAL_112_127_ADDR, PMIC_RG_OTP_VAL_112_127_MASK,
	 PMIC_RG_OTP_VAL_112_127_SHIFT},
	{PMIC_RG_OTP_VAL_128_143, PMIC_RG_OTP_VAL_128_143_ADDR, PMIC_RG_OTP_VAL_128_143_MASK,
	 PMIC_RG_OTP_VAL_128_143_SHIFT},
	{PMIC_RG_OTP_VAL_144_159, PMIC_RG_OTP_VAL_144_159_ADDR, PMIC_RG_OTP_VAL_144_159_MASK,
	 PMIC_RG_OTP_VAL_144_159_SHIFT},
	{PMIC_RG_OTP_VAL_160_175, PMIC_RG_OTP_VAL_160_175_ADDR, PMIC_RG_OTP_VAL_160_175_MASK,
	 PMIC_RG_OTP_VAL_160_175_SHIFT},
	{PMIC_RG_OTP_VAL_176_191, PMIC_RG_OTP_VAL_176_191_ADDR, PMIC_RG_OTP_VAL_176_191_MASK,
	 PMIC_RG_OTP_VAL_176_191_SHIFT},
	{PMIC_RG_OTP_VAL_192_207, PMIC_RG_OTP_VAL_192_207_ADDR, PMIC_RG_OTP_VAL_192_207_MASK,
	 PMIC_RG_OTP_VAL_192_207_SHIFT},
	{PMIC_RG_OTP_VAL_208_223, PMIC_RG_OTP_VAL_208_223_ADDR, PMIC_RG_OTP_VAL_208_223_MASK,
	 PMIC_RG_OTP_VAL_208_223_SHIFT},
	{PMIC_RG_OTP_VAL_224_239, PMIC_RG_OTP_VAL_224_239_ADDR, PMIC_RG_OTP_VAL_224_239_MASK,
	 PMIC_RG_OTP_VAL_224_239_SHIFT},
	{PMIC_RG_OTP_VAL_240_255, PMIC_RG_OTP_VAL_240_255_ADDR, PMIC_RG_OTP_VAL_240_255_MASK,
	 PMIC_RG_OTP_VAL_240_255_SHIFT},
	{PMIC_RG_OTP_VAL_256_271, PMIC_RG_OTP_VAL_256_271_ADDR, PMIC_RG_OTP_VAL_256_271_MASK,
	 PMIC_RG_OTP_VAL_256_271_SHIFT},
	{PMIC_RG_OTP_VAL_272_287, PMIC_RG_OTP_VAL_272_287_ADDR, PMIC_RG_OTP_VAL_272_287_MASK,
	 PMIC_RG_OTP_VAL_272_287_SHIFT},
	{PMIC_RG_OTP_VAL_288_303, PMIC_RG_OTP_VAL_288_303_ADDR, PMIC_RG_OTP_VAL_288_303_MASK,
	 PMIC_RG_OTP_VAL_288_303_SHIFT},
	{PMIC_RG_OTP_VAL_304_319, PMIC_RG_OTP_VAL_304_319_ADDR, PMIC_RG_OTP_VAL_304_319_MASK,
	 PMIC_RG_OTP_VAL_304_319_SHIFT},
	{PMIC_RG_OTP_VAL_320_335, PMIC_RG_OTP_VAL_320_335_ADDR, PMIC_RG_OTP_VAL_320_335_MASK,
	 PMIC_RG_OTP_VAL_320_335_SHIFT},
	{PMIC_RG_OTP_VAL_336_351, PMIC_RG_OTP_VAL_336_351_ADDR, PMIC_RG_OTP_VAL_336_351_MASK,
	 PMIC_RG_OTP_VAL_336_351_SHIFT},
	{PMIC_RG_OTP_VAL_352_367, PMIC_RG_OTP_VAL_352_367_ADDR, PMIC_RG_OTP_VAL_352_367_MASK,
	 PMIC_RG_OTP_VAL_352_367_SHIFT},
	{PMIC_RG_OTP_VAL_368_383, PMIC_RG_OTP_VAL_368_383_ADDR, PMIC_RG_OTP_VAL_368_383_MASK,
	 PMIC_RG_OTP_VAL_368_383_SHIFT},
	{PMIC_RG_OTP_VAL_384_399, PMIC_RG_OTP_VAL_384_399_ADDR, PMIC_RG_OTP_VAL_384_399_MASK,
	 PMIC_RG_OTP_VAL_384_399_SHIFT},
	{PMIC_RG_OTP_VAL_400_415, PMIC_RG_OTP_VAL_400_415_ADDR, PMIC_RG_OTP_VAL_400_415_MASK,
	 PMIC_RG_OTP_VAL_400_415_SHIFT},
	{PMIC_RG_OTP_VAL_416_431, PMIC_RG_OTP_VAL_416_431_ADDR, PMIC_RG_OTP_VAL_416_431_MASK,
	 PMIC_RG_OTP_VAL_416_431_SHIFT},
	{PMIC_RG_OTP_VAL_432_447, PMIC_RG_OTP_VAL_432_447_ADDR, PMIC_RG_OTP_VAL_432_447_MASK,
	 PMIC_RG_OTP_VAL_432_447_SHIFT},
	{PMIC_RG_OTP_VAL_448_463, PMIC_RG_OTP_VAL_448_463_ADDR, PMIC_RG_OTP_VAL_448_463_MASK,
	 PMIC_RG_OTP_VAL_448_463_SHIFT},
	{PMIC_RG_OTP_VAL_464_479, PMIC_RG_OTP_VAL_464_479_ADDR, PMIC_RG_OTP_VAL_464_479_MASK,
	 PMIC_RG_OTP_VAL_464_479_SHIFT},
	{PMIC_RG_OTP_VAL_480_495, PMIC_RG_OTP_VAL_480_495_ADDR, PMIC_RG_OTP_VAL_480_495_MASK,
	 PMIC_RG_OTP_VAL_480_495_SHIFT},
	{PMIC_RG_OTP_VAL_496_511, PMIC_RG_OTP_VAL_496_511_ADDR, PMIC_RG_OTP_VAL_496_511_MASK,
	 PMIC_RG_OTP_VAL_496_511_SHIFT},
	{PMIC_RG_OTP_VAL_512_527, PMIC_RG_OTP_VAL_512_527_ADDR, PMIC_RG_OTP_VAL_512_527_MASK,
	 PMIC_RG_OTP_VAL_512_527_SHIFT},
	{PMIC_RG_OTP_VAL_528_543, PMIC_RG_OTP_VAL_528_543_ADDR, PMIC_RG_OTP_VAL_528_543_MASK,
	 PMIC_RG_OTP_VAL_528_543_SHIFT},
	{PMIC_RG_OTP_VAL_544_559, PMIC_RG_OTP_VAL_544_559_ADDR, PMIC_RG_OTP_VAL_544_559_MASK,
	 PMIC_RG_OTP_VAL_544_559_SHIFT},
	{PMIC_RG_OTP_VAL_560_575, PMIC_RG_OTP_VAL_560_575_ADDR, PMIC_RG_OTP_VAL_560_575_MASK,
	 PMIC_RG_OTP_VAL_560_575_SHIFT},
	{PMIC_RG_OTP_VAL_576_591, PMIC_RG_OTP_VAL_576_591_ADDR, PMIC_RG_OTP_VAL_576_591_MASK,
	 PMIC_RG_OTP_VAL_576_591_SHIFT},
	{PMIC_RG_OTP_VAL_592_607, PMIC_RG_OTP_VAL_592_607_ADDR, PMIC_RG_OTP_VAL_592_607_MASK,
	 PMIC_RG_OTP_VAL_592_607_SHIFT},
	{PMIC_RG_OTP_VAL_608_623, PMIC_RG_OTP_VAL_608_623_ADDR, PMIC_RG_OTP_VAL_608_623_MASK,
	 PMIC_RG_OTP_VAL_608_623_SHIFT},
	{PMIC_RG_OTP_VAL_624_639, PMIC_RG_OTP_VAL_624_639_ADDR, PMIC_RG_OTP_VAL_624_639_MASK,
	 PMIC_RG_OTP_VAL_624_639_SHIFT},
	{PMIC_RG_OTP_VAL_640_655, PMIC_RG_OTP_VAL_640_655_ADDR, PMIC_RG_OTP_VAL_640_655_MASK,
	 PMIC_RG_OTP_VAL_640_655_SHIFT},
	{PMIC_RG_OTP_VAL_656_671, PMIC_RG_OTP_VAL_656_671_ADDR, PMIC_RG_OTP_VAL_656_671_MASK,
	 PMIC_RG_OTP_VAL_656_671_SHIFT},
	{PMIC_RG_OTP_VAL_672_687, PMIC_RG_OTP_VAL_672_687_ADDR, PMIC_RG_OTP_VAL_672_687_MASK,
	 PMIC_RG_OTP_VAL_672_687_SHIFT},
	{PMIC_RG_OTP_VAL_688_703, PMIC_RG_OTP_VAL_688_703_ADDR, PMIC_RG_OTP_VAL_688_703_MASK,
	 PMIC_RG_OTP_VAL_688_703_SHIFT},
	{PMIC_RG_OTP_VAL_704_719, PMIC_RG_OTP_VAL_704_719_ADDR, PMIC_RG_OTP_VAL_704_719_MASK,
	 PMIC_RG_OTP_VAL_704_719_SHIFT},
	{PMIC_RG_OTP_VAL_720_735, PMIC_RG_OTP_VAL_720_735_ADDR, PMIC_RG_OTP_VAL_720_735_MASK,
	 PMIC_RG_OTP_VAL_720_735_SHIFT},
	{PMIC_RG_OTP_VAL_736_751, PMIC_RG_OTP_VAL_736_751_ADDR, PMIC_RG_OTP_VAL_736_751_MASK,
	 PMIC_RG_OTP_VAL_736_751_SHIFT},
	{PMIC_RG_OTP_VAL_752_767, PMIC_RG_OTP_VAL_752_767_ADDR, PMIC_RG_OTP_VAL_752_767_MASK,
	 PMIC_RG_OTP_VAL_752_767_SHIFT},
	{PMIC_RG_OTP_VAL_768_783, PMIC_RG_OTP_VAL_768_783_ADDR, PMIC_RG_OTP_VAL_768_783_MASK,
	 PMIC_RG_OTP_VAL_768_783_SHIFT},
	{PMIC_RG_OTP_VAL_784_799, PMIC_RG_OTP_VAL_784_799_ADDR, PMIC_RG_OTP_VAL_784_799_MASK,
	 PMIC_RG_OTP_VAL_784_799_SHIFT},
	{PMIC_RG_OTP_VAL_800_815, PMIC_RG_OTP_VAL_800_815_ADDR, PMIC_RG_OTP_VAL_800_815_MASK,
	 PMIC_RG_OTP_VAL_800_815_SHIFT},
	{PMIC_RG_OTP_VAL_816_831, PMIC_RG_OTP_VAL_816_831_ADDR, PMIC_RG_OTP_VAL_816_831_MASK,
	 PMIC_RG_OTP_VAL_816_831_SHIFT},
	{PMIC_RG_OTP_VAL_832_847, PMIC_RG_OTP_VAL_832_847_ADDR, PMIC_RG_OTP_VAL_832_847_MASK,
	 PMIC_RG_OTP_VAL_832_847_SHIFT},
	{PMIC_RG_OTP_VAL_848_863, PMIC_RG_OTP_VAL_848_863_ADDR, PMIC_RG_OTP_VAL_848_863_MASK,
	 PMIC_RG_OTP_VAL_848_863_SHIFT},
	{PMIC_RG_OTP_VAL_864_879, PMIC_RG_OTP_VAL_864_879_ADDR, PMIC_RG_OTP_VAL_864_879_MASK,
	 PMIC_RG_OTP_VAL_864_879_SHIFT},
	{PMIC_RG_OTP_VAL_880_895, PMIC_RG_OTP_VAL_880_895_ADDR, PMIC_RG_OTP_VAL_880_895_MASK,
	 PMIC_RG_OTP_VAL_880_895_SHIFT},
	{PMIC_RG_OTP_VAL_896_911, PMIC_RG_OTP_VAL_896_911_ADDR, PMIC_RG_OTP_VAL_896_911_MASK,
	 PMIC_RG_OTP_VAL_896_911_SHIFT},
	{PMIC_RG_OTP_VAL_912_927, PMIC_RG_OTP_VAL_912_927_ADDR, PMIC_RG_OTP_VAL_912_927_MASK,
	 PMIC_RG_OTP_VAL_912_927_SHIFT},
	{PMIC_RG_OTP_VAL_928_943, PMIC_RG_OTP_VAL_928_943_ADDR, PMIC_RG_OTP_VAL_928_943_MASK,
	 PMIC_RG_OTP_VAL_928_943_SHIFT},
	{PMIC_RG_OTP_VAL_944_959, PMIC_RG_OTP_VAL_944_959_ADDR, PMIC_RG_OTP_VAL_944_959_MASK,
	 PMIC_RG_OTP_VAL_944_959_SHIFT},
	{PMIC_RG_OTP_VAL_960_975, PMIC_RG_OTP_VAL_960_975_ADDR, PMIC_RG_OTP_VAL_960_975_MASK,
	 PMIC_RG_OTP_VAL_960_975_SHIFT},
	{PMIC_RG_OTP_VAL_976_991, PMIC_RG_OTP_VAL_976_991_ADDR, PMIC_RG_OTP_VAL_976_991_MASK,
	 PMIC_RG_OTP_VAL_976_991_SHIFT},
	{PMIC_RG_OTP_VAL_992_1007, PMIC_RG_OTP_VAL_992_1007_ADDR, PMIC_RG_OTP_VAL_992_1007_MASK,
	 PMIC_RG_OTP_VAL_992_1007_SHIFT},
	{PMIC_RG_OTP_VAL_1008_1023, PMIC_RG_OTP_VAL_1008_1023_ADDR, PMIC_RG_OTP_VAL_1008_1023_MASK,
	 PMIC_RG_OTP_VAL_1008_1023_SHIFT},
	{PMIC_MIX_EOSC32_STP_LPDTB, PMIC_MIX_EOSC32_STP_LPDTB_ADDR, PMIC_MIX_EOSC32_STP_LPDTB_MASK,
	 PMIC_MIX_EOSC32_STP_LPDTB_SHIFT},
	{PMIC_MIX_EOSC32_STP_LPDEN, PMIC_MIX_EOSC32_STP_LPDEN_ADDR, PMIC_MIX_EOSC32_STP_LPDEN_MASK,
	 PMIC_MIX_EOSC32_STP_LPDEN_SHIFT},
	{PMIC_MIX_XOSC32_STP_PWDB, PMIC_MIX_XOSC32_STP_PWDB_ADDR, PMIC_MIX_XOSC32_STP_PWDB_MASK,
	 PMIC_MIX_XOSC32_STP_PWDB_SHIFT},
	{PMIC_MIX_XOSC32_STP_LPDTB, PMIC_MIX_XOSC32_STP_LPDTB_ADDR, PMIC_MIX_XOSC32_STP_LPDTB_MASK,
	 PMIC_MIX_XOSC32_STP_LPDTB_SHIFT},
	{PMIC_MIX_XOSC32_STP_LPDEN, PMIC_MIX_XOSC32_STP_LPDEN_ADDR, PMIC_MIX_XOSC32_STP_LPDEN_MASK,
	 PMIC_MIX_XOSC32_STP_LPDEN_SHIFT},
	{PMIC_MIX_XOSC32_STP_LPDRST, PMIC_MIX_XOSC32_STP_LPDRST_ADDR,
	 PMIC_MIX_XOSC32_STP_LPDRST_MASK, PMIC_MIX_XOSC32_STP_LPDRST_SHIFT},
	{PMIC_MIX_XOSC32_STP_CALI, PMIC_MIX_XOSC32_STP_CALI_ADDR, PMIC_MIX_XOSC32_STP_CALI_MASK,
	 PMIC_MIX_XOSC32_STP_CALI_SHIFT},
	{PMIC_STMP_MODE, PMIC_STMP_MODE_ADDR, PMIC_STMP_MODE_MASK, PMIC_STMP_MODE_SHIFT},
	{PMIC_MIX_EOSC32_STP_CHOP_EN, PMIC_MIX_EOSC32_STP_CHOP_EN_ADDR,
	 PMIC_MIX_EOSC32_STP_CHOP_EN_MASK, PMIC_MIX_EOSC32_STP_CHOP_EN_SHIFT},
	{PMIC_MIX_DCXO_STP_LVSH_EN, PMIC_MIX_DCXO_STP_LVSH_EN_ADDR, PMIC_MIX_DCXO_STP_LVSH_EN_MASK,
	 PMIC_MIX_DCXO_STP_LVSH_EN_SHIFT},
	{PMIC_MIX_PMU_STP_DDLO_VRTC, PMIC_MIX_PMU_STP_DDLO_VRTC_ADDR,
	 PMIC_MIX_PMU_STP_DDLO_VRTC_MASK, PMIC_MIX_PMU_STP_DDLO_VRTC_SHIFT},
	{PMIC_MIX_PMU_STP_DDLO_VRTC_EN, PMIC_MIX_PMU_STP_DDLO_VRTC_EN_ADDR,
	 PMIC_MIX_PMU_STP_DDLO_VRTC_EN_MASK, PMIC_MIX_PMU_STP_DDLO_VRTC_EN_SHIFT},
	{PMIC_MIX_RTC_STP_XOSC32_ENB, PMIC_MIX_RTC_STP_XOSC32_ENB_ADDR,
	 PMIC_MIX_RTC_STP_XOSC32_ENB_MASK, PMIC_MIX_RTC_STP_XOSC32_ENB_SHIFT},
	{PMIC_MIX_DCXO_STP_TEST_DEGLITCH_MODE, PMIC_MIX_DCXO_STP_TEST_DEGLITCH_MODE_ADDR,
	 PMIC_MIX_DCXO_STP_TEST_DEGLITCH_MODE_MASK, PMIC_MIX_DCXO_STP_TEST_DEGLITCH_MODE_SHIFT},
	{PMIC_MIX_EOSC32_STP_RSV, PMIC_MIX_EOSC32_STP_RSV_ADDR, PMIC_MIX_EOSC32_STP_RSV_MASK,
	 PMIC_MIX_EOSC32_STP_RSV_SHIFT},
	{PMIC_MIX_EOSC32_VCT_EN, PMIC_MIX_EOSC32_VCT_EN_ADDR, PMIC_MIX_EOSC32_VCT_EN_MASK,
	 PMIC_MIX_EOSC32_VCT_EN_SHIFT},
	{PMIC_MIX_EOSC32_OPT, PMIC_MIX_EOSC32_OPT_ADDR, PMIC_MIX_EOSC32_OPT_MASK,
	 PMIC_MIX_EOSC32_OPT_SHIFT},
	{PMIC_MIX_DCXO_STP_LVSH_EN_INT, PMIC_MIX_DCXO_STP_LVSH_EN_INT_ADDR,
	 PMIC_MIX_DCXO_STP_LVSH_EN_INT_MASK, PMIC_MIX_DCXO_STP_LVSH_EN_INT_SHIFT},
	{PMIC_MIX_RTC_GPIO_COREDETB, PMIC_MIX_RTC_GPIO_COREDETB_ADDR,
	 PMIC_MIX_RTC_GPIO_COREDETB_MASK, PMIC_MIX_RTC_GPIO_COREDETB_SHIFT},
	{PMIC_MIX_RTC_GPIO_F32KOB, PMIC_MIX_RTC_GPIO_F32KOB_ADDR, PMIC_MIX_RTC_GPIO_F32KOB_MASK,
	 PMIC_MIX_RTC_GPIO_F32KOB_SHIFT},
	{PMIC_MIX_RTC_GPIO_GPO, PMIC_MIX_RTC_GPIO_GPO_ADDR, PMIC_MIX_RTC_GPIO_GPO_MASK,
	 PMIC_MIX_RTC_GPIO_GPO_SHIFT},
	{PMIC_MIX_RTC_GPIO_OE, PMIC_MIX_RTC_GPIO_OE_ADDR, PMIC_MIX_RTC_GPIO_OE_MASK,
	 PMIC_MIX_RTC_GPIO_OE_SHIFT},
	{PMIC_MIX_RTC_STP_DEBUG_OUT, PMIC_MIX_RTC_STP_DEBUG_OUT_ADDR,
	 PMIC_MIX_RTC_STP_DEBUG_OUT_MASK, PMIC_MIX_RTC_STP_DEBUG_OUT_SHIFT},
	{PMIC_MIX_RTC_STP_DEBUG_SEL, PMIC_MIX_RTC_STP_DEBUG_SEL_ADDR,
	 PMIC_MIX_RTC_STP_DEBUG_SEL_MASK, PMIC_MIX_RTC_STP_DEBUG_SEL_SHIFT},
	{PMIC_MIX_RTC_STP_K_EOSC32_EN, PMIC_MIX_RTC_STP_K_EOSC32_EN_ADDR,
	 PMIC_MIX_RTC_STP_K_EOSC32_EN_MASK, PMIC_MIX_RTC_STP_K_EOSC32_EN_SHIFT},
	{PMIC_MIX_RTC_STP_EMBCK_SEL, PMIC_MIX_RTC_STP_EMBCK_SEL_ADDR,
	 PMIC_MIX_RTC_STP_EMBCK_SEL_MASK, PMIC_MIX_RTC_STP_EMBCK_SEL_SHIFT},
	{PMIC_MIX_STP_BBWAKEUP, PMIC_MIX_STP_BBWAKEUP_ADDR, PMIC_MIX_STP_BBWAKEUP_MASK,
	 PMIC_MIX_STP_BBWAKEUP_SHIFT},
	{PMIC_MIX_STP_RTC_DDLO, PMIC_MIX_STP_RTC_DDLO_ADDR, PMIC_MIX_STP_RTC_DDLO_MASK,
	 PMIC_MIX_STP_RTC_DDLO_SHIFT},
	{PMIC_MIX_RTC_XOSC32_ENB, PMIC_MIX_RTC_XOSC32_ENB_ADDR, PMIC_MIX_RTC_XOSC32_ENB_MASK,
	 PMIC_MIX_RTC_XOSC32_ENB_SHIFT},
	{PMIC_MIX_EFUSE_XOSC32_ENB_OPT, PMIC_MIX_EFUSE_XOSC32_ENB_OPT_ADDR,
	 PMIC_MIX_EFUSE_XOSC32_ENB_OPT_MASK, PMIC_MIX_EFUSE_XOSC32_ENB_OPT_SHIFT},
	{PMIC_FG_ON, PMIC_FG_ON_ADDR, PMIC_FG_ON_MASK, PMIC_FG_ON_SHIFT},
	{PMIC_FG_CAL, PMIC_FG_CAL_ADDR, PMIC_FG_CAL_MASK, PMIC_FG_CAL_SHIFT},
	{PMIC_FG_AUTOCALRATE, PMIC_FG_AUTOCALRATE_ADDR, PMIC_FG_AUTOCALRATE_MASK,
	 PMIC_FG_AUTOCALRATE_SHIFT},
	{PMIC_FG_SON_SLP_EN, PMIC_FG_SON_SLP_EN_ADDR, PMIC_FG_SON_SLP_EN_MASK,
	 PMIC_FG_SON_SLP_EN_SHIFT},
	{PMIC_FG_SOFF_SLP_EN, PMIC_FG_SOFF_SLP_EN_ADDR, PMIC_FG_SOFF_SLP_EN_MASK,
	 PMIC_FG_SOFF_SLP_EN_SHIFT},
	{PMIC_FG_ZCV_DET_EN, PMIC_FG_ZCV_DET_EN_ADDR, PMIC_FG_ZCV_DET_EN_MASK,
	 PMIC_FG_ZCV_DET_EN_SHIFT},
	{PMIC_FG_AUXADC_R, PMIC_FG_AUXADC_R_ADDR, PMIC_FG_AUXADC_R_MASK, PMIC_FG_AUXADC_R_SHIFT},
	{PMIC_FG_IAVG_MODE, PMIC_FG_IAVG_MODE_ADDR, PMIC_FG_IAVG_MODE_MASK,
	 PMIC_FG_IAVG_MODE_SHIFT},
	{PMIC_FG_SW_READ_PRE, PMIC_FG_SW_READ_PRE_ADDR, PMIC_FG_SW_READ_PRE_MASK,
	 PMIC_FG_SW_READ_PRE_SHIFT},
	{PMIC_FG_SW_RSTCLR, PMIC_FG_SW_RSTCLR_ADDR, PMIC_FG_SW_RSTCLR_MASK,
	 PMIC_FG_SW_RSTCLR_SHIFT},
	{PMIC_FG_SW_CR, PMIC_FG_SW_CR_ADDR, PMIC_FG_SW_CR_MASK, PMIC_FG_SW_CR_SHIFT},
	{PMIC_FG_SW_CLEAR, PMIC_FG_SW_CLEAR_ADDR, PMIC_FG_SW_CLEAR_MASK, PMIC_FG_SW_CLEAR_SHIFT},
	{PMIC_FG_OFFSET_RST, PMIC_FG_OFFSET_RST_ADDR, PMIC_FG_OFFSET_RST_MASK,
	 PMIC_FG_OFFSET_RST_SHIFT},
	{PMIC_FG_TIME_RST, PMIC_FG_TIME_RST_ADDR, PMIC_FG_TIME_RST_MASK, PMIC_FG_TIME_RST_SHIFT},
	{PMIC_FG_CHARGE_RST, PMIC_FG_CHARGE_RST_ADDR, PMIC_FG_CHARGE_RST_MASK,
	 PMIC_FG_CHARGE_RST_SHIFT},
	{PMIC_FG_N_CHARGE_RST, PMIC_FG_N_CHARGE_RST_ADDR, PMIC_FG_N_CHARGE_RST_MASK,
	 PMIC_FG_N_CHARGE_RST_SHIFT},
	{PMIC_FG_SOFF_RST, PMIC_FG_SOFF_RST_ADDR, PMIC_FG_SOFF_RST_MASK, PMIC_FG_SOFF_RST_SHIFT},
	{PMIC_FG_LATCHDATA_ST, PMIC_FG_LATCHDATA_ST_ADDR, PMIC_FG_LATCHDATA_ST_MASK,
	 PMIC_FG_LATCHDATA_ST_SHIFT},
	{PMIC_EVENT_FG_BAT0_L, PMIC_EVENT_FG_BAT0_L_ADDR, PMIC_EVENT_FG_BAT0_L_MASK,
	 PMIC_EVENT_FG_BAT0_L_SHIFT},
	{PMIC_EVENT_FG_BAT0_H, PMIC_EVENT_FG_BAT0_H_ADDR, PMIC_EVENT_FG_BAT0_H_MASK,
	 PMIC_EVENT_FG_BAT0_H_SHIFT},
	{PMIC_EVENT_FG_BAT1_L, PMIC_EVENT_FG_BAT1_L_ADDR, PMIC_EVENT_FG_BAT1_L_MASK,
	 PMIC_EVENT_FG_BAT1_L_SHIFT},
	{PMIC_EVENT_FG_BAT1_H, PMIC_EVENT_FG_BAT1_H_ADDR, PMIC_EVENT_FG_BAT1_H_MASK,
	 PMIC_EVENT_FG_BAT1_H_SHIFT},
	{PMIC_EVENT_FG_CUR_L, PMIC_EVENT_FG_CUR_L_ADDR, PMIC_EVENT_FG_CUR_L_MASK,
	 PMIC_EVENT_FG_CUR_L_SHIFT},
	{PMIC_EVENT_FG_CUR_H, PMIC_EVENT_FG_CUR_H_ADDR, PMIC_EVENT_FG_CUR_H_MASK,
	 PMIC_EVENT_FG_CUR_H_SHIFT},
	{PMIC_EVENT_FG_IAVG_L, PMIC_EVENT_FG_IAVG_L_ADDR, PMIC_EVENT_FG_IAVG_L_MASK,
	 PMIC_EVENT_FG_IAVG_L_SHIFT},
	{PMIC_EVENT_FG_IAVG_H, PMIC_EVENT_FG_IAVG_H_ADDR, PMIC_EVENT_FG_IAVG_H_MASK,
	 PMIC_EVENT_FG_IAVG_H_SHIFT},
	{PMIC_EVENT_FG_N_CHARGE_L, PMIC_EVENT_FG_N_CHARGE_L_ADDR, PMIC_EVENT_FG_N_CHARGE_L_MASK,
	 PMIC_EVENT_FG_N_CHARGE_L_SHIFT},
	{PMIC_EVENT_FG_TIME_H, PMIC_EVENT_FG_TIME_H_ADDR, PMIC_EVENT_FG_TIME_H_MASK,
	 PMIC_EVENT_FG_TIME_H_SHIFT},
	{PMIC_EVENT_FG_DISCHARGE, PMIC_EVENT_FG_DISCHARGE_ADDR, PMIC_EVENT_FG_DISCHARGE_MASK,
	 PMIC_EVENT_FG_DISCHARGE_SHIFT},
	{PMIC_EVENT_FG_CHARGE, PMIC_EVENT_FG_CHARGE_ADDR, PMIC_EVENT_FG_CHARGE_MASK,
	 PMIC_EVENT_FG_CHARGE_SHIFT},
	{PMIC_EVENT_FG_ZCV, PMIC_EVENT_FG_ZCV_ADDR, PMIC_EVENT_FG_ZCV_MASK,
	 PMIC_EVENT_FG_ZCV_SHIFT},
	{PMIC_FG_OSR1, PMIC_FG_OSR1_ADDR, PMIC_FG_OSR1_MASK, PMIC_FG_OSR1_SHIFT},
	{PMIC_FG_OSR2, PMIC_FG_OSR2_ADDR, PMIC_FG_OSR2_MASK, PMIC_FG_OSR2_SHIFT},
	{PMIC_FG_FIR1BYPASS, PMIC_FG_FIR1BYPASS_ADDR, PMIC_FG_FIR1BYPASS_MASK,
	 PMIC_FG_FIR1BYPASS_SHIFT},
	{PMIC_FG_FIR2BYPASS, PMIC_FG_FIR2BYPASS_ADDR, PMIC_FG_FIR2BYPASS_MASK,
	 PMIC_FG_FIR2BYPASS_SHIFT},
	{PMIC_FG_ADJ_OFFSET_EN, PMIC_FG_ADJ_OFFSET_EN_ADDR, PMIC_FG_ADJ_OFFSET_EN_MASK,
	 PMIC_FG_ADJ_OFFSET_EN_SHIFT},
	{PMIC_FG_ADC_AUTORST, PMIC_FG_ADC_AUTORST_ADDR, PMIC_FG_ADC_AUTORST_MASK,
	 PMIC_FG_ADC_AUTORST_SHIFT},
	{PMIC_FG_ADC_RSTDETECT, PMIC_FG_ADC_RSTDETECT_ADDR, PMIC_FG_ADC_RSTDETECT_MASK,
	 PMIC_FG_ADC_RSTDETECT_SHIFT},
	{PMIC_FG_VA18_AON, PMIC_FG_VA18_AON_ADDR, PMIC_FG_VA18_AON_MASK, PMIC_FG_VA18_AON_SHIFT},
	{PMIC_FG_VA18_AOFF, PMIC_FG_VA18_AOFF_ADDR, PMIC_FG_VA18_AOFF_MASK,
	 PMIC_FG_VA18_AOFF_SHIFT},
	{PMIC_FG_SON_SW, PMIC_FG_SON_SW_ADDR, PMIC_FG_SON_SW_MASK, PMIC_FG_SON_SW_SHIFT},
	{PMIC_FG_SON_SW_MODE, PMIC_FG_SON_SW_MODE_ADDR, PMIC_FG_SON_SW_MODE_MASK,
	 PMIC_FG_SON_SW_MODE_SHIFT},
	{PMIC_FGADC_CON4_RSV, PMIC_FGADC_CON4_RSV_ADDR, PMIC_FGADC_CON4_RSV_MASK,
	 PMIC_FGADC_CON4_RSV_SHIFT},
	{PMIC_FG_RSTB_STATUS, PMIC_FG_RSTB_STATUS_ADDR, PMIC_FG_RSTB_STATUS_MASK,
	 PMIC_FG_RSTB_STATUS_SHIFT},
	{PMIC_FG_R_CURR, PMIC_FG_R_CURR_ADDR, PMIC_FG_R_CURR_MASK, PMIC_FG_R_CURR_SHIFT},
	{PMIC_FG_CURRENT_OUT, PMIC_FG_CURRENT_OUT_ADDR, PMIC_FG_CURRENT_OUT_MASK,
	 PMIC_FG_CURRENT_OUT_SHIFT},
	{PMIC_FG_CUR_LTH, PMIC_FG_CUR_LTH_ADDR, PMIC_FG_CUR_LTH_MASK, PMIC_FG_CUR_LTH_SHIFT},
	{PMIC_FG_CUR_HTH, PMIC_FG_CUR_HTH_ADDR, PMIC_FG_CUR_HTH_MASK, PMIC_FG_CUR_HTH_SHIFT},
	{PMIC_FG_CIC2, PMIC_FG_CIC2_ADDR, PMIC_FG_CIC2_MASK, PMIC_FG_CIC2_SHIFT},
	{PMIC_FG_CAR_31_16, PMIC_FG_CAR_31_16_ADDR, PMIC_FG_CAR_31_16_MASK,
	 PMIC_FG_CAR_31_16_SHIFT},
	{PMIC_FG_CAR_15_00, PMIC_FG_CAR_15_00_ADDR, PMIC_FG_CAR_15_00_MASK,
	 PMIC_FG_CAR_15_00_SHIFT},
	{PMIC_FG_BAT0_LTH_31_16, PMIC_FG_BAT0_LTH_31_16_ADDR, PMIC_FG_BAT0_LTH_31_16_MASK,
	 PMIC_FG_BAT0_LTH_31_16_SHIFT},
	{PMIC_FG_BAT0_LTH_15_14, PMIC_FG_BAT0_LTH_15_14_ADDR, PMIC_FG_BAT0_LTH_15_14_MASK,
	 PMIC_FG_BAT0_LTH_15_14_SHIFT},
	{PMIC_FG_BAT0_HTH_31_16, PMIC_FG_BAT0_HTH_31_16_ADDR, PMIC_FG_BAT0_HTH_31_16_MASK,
	 PMIC_FG_BAT0_HTH_31_16_SHIFT},
	{PMIC_FG_BAT0_HTH_15_14, PMIC_FG_BAT0_HTH_15_14_ADDR, PMIC_FG_BAT0_HTH_15_14_MASK,
	 PMIC_FG_BAT0_HTH_15_14_SHIFT},
	{PMIC_FG_BAT1_LTH_31_16, PMIC_FG_BAT1_LTH_31_16_ADDR, PMIC_FG_BAT1_LTH_31_16_MASK,
	 PMIC_FG_BAT1_LTH_31_16_SHIFT},
	{PMIC_FG_BAT1_LTH_15_14, PMIC_FG_BAT1_LTH_15_14_ADDR, PMIC_FG_BAT1_LTH_15_14_MASK,
	 PMIC_FG_BAT1_LTH_15_14_SHIFT},
	{PMIC_FG_BAT1_HTH_31_16, PMIC_FG_BAT1_HTH_31_16_ADDR, PMIC_FG_BAT1_HTH_31_16_MASK,
	 PMIC_FG_BAT1_HTH_31_16_SHIFT},
	{PMIC_FG_BAT1_HTH_15_14, PMIC_FG_BAT1_HTH_15_14_ADDR, PMIC_FG_BAT1_HTH_15_14_MASK,
	 PMIC_FG_BAT1_HTH_15_14_SHIFT},
	{PMIC_FG_NCAR_31_16, PMIC_FG_NCAR_31_16_ADDR, PMIC_FG_NCAR_31_16_MASK,
	 PMIC_FG_NCAR_31_16_SHIFT},
	{PMIC_FG_NCAR_15_00, PMIC_FG_NCAR_15_00_ADDR, PMIC_FG_NCAR_15_00_MASK,
	 PMIC_FG_NCAR_15_00_SHIFT},
	{PMIC_FG_N_CHARGE_LTH_31_16, PMIC_FG_N_CHARGE_LTH_31_16_ADDR,
	 PMIC_FG_N_CHARGE_LTH_31_16_MASK, PMIC_FG_N_CHARGE_LTH_31_16_SHIFT},
	{PMIC_FG_N_CHARGE_LTH_15_14, PMIC_FG_N_CHARGE_LTH_15_14_ADDR,
	 PMIC_FG_N_CHARGE_LTH_15_14_MASK, PMIC_FG_N_CHARGE_LTH_15_14_SHIFT},
	{PMIC_FG_IAVG_27_16, PMIC_FG_IAVG_27_16_ADDR, PMIC_FG_IAVG_27_16_MASK,
	 PMIC_FG_IAVG_27_16_SHIFT},
	{PMIC_FG_IAVG_VLD, PMIC_FG_IAVG_VLD_ADDR, PMIC_FG_IAVG_VLD_MASK, PMIC_FG_IAVG_VLD_SHIFT},
	{PMIC_FG_IAVG_15_00, PMIC_FG_IAVG_15_00_ADDR, PMIC_FG_IAVG_15_00_MASK,
	 PMIC_FG_IAVG_15_00_SHIFT},
	{PMIC_FG_IAVG_LTH_28_16, PMIC_FG_IAVG_LTH_28_16_ADDR, PMIC_FG_IAVG_LTH_28_16_MASK,
	 PMIC_FG_IAVG_LTH_28_16_SHIFT},
	{PMIC_FG_IAVG_LTH_15_00, PMIC_FG_IAVG_LTH_15_00_ADDR, PMIC_FG_IAVG_LTH_15_00_MASK,
	 PMIC_FG_IAVG_LTH_15_00_SHIFT},
	{PMIC_FG_IAVG_HTH_28_16, PMIC_FG_IAVG_HTH_28_16_ADDR, PMIC_FG_IAVG_HTH_28_16_MASK,
	 PMIC_FG_IAVG_HTH_28_16_SHIFT},
	{PMIC_FG_IAVG_HTH_15_00, PMIC_FG_IAVG_HTH_15_00_ADDR, PMIC_FG_IAVG_HTH_15_00_MASK,
	 PMIC_FG_IAVG_HTH_15_00_SHIFT},
	{PMIC_FG_NTER_29_16, PMIC_FG_NTER_29_16_ADDR, PMIC_FG_NTER_29_16_MASK,
	 PMIC_FG_NTER_29_16_SHIFT},
	{PMIC_FG_NTER_15_00, PMIC_FG_NTER_15_00_ADDR, PMIC_FG_NTER_15_00_MASK,
	 PMIC_FG_NTER_15_00_SHIFT},
	{PMIC_FG_TIME_HTH_29_16, PMIC_FG_TIME_HTH_29_16_ADDR, PMIC_FG_TIME_HTH_29_16_MASK,
	 PMIC_FG_TIME_HTH_29_16_SHIFT},
	{PMIC_FG_TIME_HTH_15_00, PMIC_FG_TIME_HTH_15_00_ADDR, PMIC_FG_TIME_HTH_15_00_MASK,
	 PMIC_FG_TIME_HTH_15_00_SHIFT},
	{PMIC_FG_OFFSET, PMIC_FG_OFFSET_ADDR, PMIC_FG_OFFSET_MASK, PMIC_FG_OFFSET_SHIFT},
	{PMIC_FG_ADJUST_OFFSET_VALUE, PMIC_FG_ADJUST_OFFSET_VALUE_ADDR,
	 PMIC_FG_ADJUST_OFFSET_VALUE_MASK, PMIC_FG_ADJUST_OFFSET_VALUE_SHIFT},
	{PMIC_FG_GAIN, PMIC_FG_GAIN_ADDR, PMIC_FG_GAIN_MASK, PMIC_FG_GAIN_SHIFT},
	{PMIC_FG_SON_SLP_CUR_TH, PMIC_FG_SON_SLP_CUR_TH_ADDR, PMIC_FG_SON_SLP_CUR_TH_MASK,
	 PMIC_FG_SON_SLP_CUR_TH_SHIFT},
	{PMIC_FG_SON_SLP_TIME, PMIC_FG_SON_SLP_TIME_ADDR, PMIC_FG_SON_SLP_TIME_MASK,
	 PMIC_FG_SON_SLP_TIME_SHIFT},
	{PMIC_FG_SON_DET_TIME, PMIC_FG_SON_DET_TIME_ADDR, PMIC_FG_SON_DET_TIME_MASK,
	 PMIC_FG_SON_DET_TIME_SHIFT},
	{PMIC_FG_FP_FTIME, PMIC_FG_FP_FTIME_ADDR, PMIC_FG_FP_FTIME_MASK, PMIC_FG_FP_FTIME_SHIFT},
	{PMIC_FG_SOFF_SLP_CUR_TH, PMIC_FG_SOFF_SLP_CUR_TH_ADDR, PMIC_FG_SOFF_SLP_CUR_TH_MASK,
	 PMIC_FG_SOFF_SLP_CUR_TH_SHIFT},
	{PMIC_FG_SOFF_SLP_TIME, PMIC_FG_SOFF_SLP_TIME_ADDR, PMIC_FG_SOFF_SLP_TIME_MASK,
	 PMIC_FG_SOFF_SLP_TIME_SHIFT},
	{PMIC_FG_SOFF_DET_TIME, PMIC_FG_SOFF_DET_TIME_ADDR, PMIC_FG_SOFF_DET_TIME_MASK,
	 PMIC_FG_SOFF_DET_TIME_SHIFT},
	{PMIC_FG_SOFF_TIME_29_16, PMIC_FG_SOFF_TIME_29_16_ADDR, PMIC_FG_SOFF_TIME_29_16_MASK,
	 PMIC_FG_SOFF_TIME_29_16_SHIFT},
	{PMIC_FG_SOFF, PMIC_FG_SOFF_ADDR, PMIC_FG_SOFF_MASK, PMIC_FG_SOFF_SHIFT},
	{PMIC_FG_SOFF_TIME_15_00, PMIC_FG_SOFF_TIME_15_00_ADDR, PMIC_FG_SOFF_TIME_15_00_MASK,
	 PMIC_FG_SOFF_TIME_15_00_SHIFT},
	{PMIC_FG_PWR_TIME0, PMIC_FG_PWR_TIME0_ADDR, PMIC_FG_PWR_TIME0_MASK,
	 PMIC_FG_PWR_TIME0_SHIFT},
	{PMIC_FG_PWR_TIME1, PMIC_FG_PWR_TIME1_ADDR, PMIC_FG_PWR_TIME1_MASK,
	 PMIC_FG_PWR_TIME1_SHIFT},
	{PMIC_FG_PWR_TIME2, PMIC_FG_PWR_TIME2_ADDR, PMIC_FG_PWR_TIME2_MASK,
	 PMIC_FG_PWR_TIME2_SHIFT},
	{PMIC_FG_ZCV_DET_IV, PMIC_FG_ZCV_DET_IV_ADDR, PMIC_FG_ZCV_DET_IV_MASK,
	 PMIC_FG_ZCV_DET_IV_SHIFT},
	{PMIC_FG_ZCV_CAR_TH_30_16, PMIC_FG_ZCV_CAR_TH_30_16_ADDR, PMIC_FG_ZCV_CAR_TH_30_16_MASK,
	 PMIC_FG_ZCV_CAR_TH_30_16_SHIFT},
	{PMIC_FG_ZCV_CAR_TH_15_00, PMIC_FG_ZCV_CAR_TH_15_00_ADDR, PMIC_FG_ZCV_CAR_TH_15_00_MASK,
	 PMIC_FG_ZCV_CAR_TH_15_00_SHIFT},
	{PMIC_FG_ZCV_CAR_31_16, PMIC_FG_ZCV_CAR_31_16_ADDR, PMIC_FG_ZCV_CAR_31_16_MASK,
	 PMIC_FG_ZCV_CAR_31_16_SHIFT},
	{PMIC_FG_ZCV_CAR_15_00, PMIC_FG_ZCV_CAR_15_00_ADDR, PMIC_FG_ZCV_CAR_15_00_MASK,
	 PMIC_FG_ZCV_CAR_15_00_SHIFT},
	{PMIC_FG_ZCV_CURR, PMIC_FG_ZCV_CURR_ADDR, PMIC_FG_ZCV_CURR_MASK, PMIC_FG_ZCV_CURR_SHIFT},
	{PMIC_DA_FGADC_EN, PMIC_DA_FGADC_EN_ADDR, PMIC_DA_FGADC_EN_MASK, PMIC_DA_FGADC_EN_SHIFT},
	{PMIC_DA_FGCAL_EN, PMIC_DA_FGCAL_EN_ADDR, PMIC_DA_FGCAL_EN_MASK, PMIC_DA_FGCAL_EN_SHIFT},
	{PMIC_DA_FG_RST, PMIC_DA_FG_RST_ADDR, PMIC_DA_FG_RST_MASK, PMIC_DA_FG_RST_SHIFT},
	{PMIC_RG_FGANALOGTEST, PMIC_RG_FGANALOGTEST_ADDR, PMIC_RG_FGANALOGTEST_MASK,
	 PMIC_RG_FGANALOGTEST_SHIFT},
	{PMIC_RG_FGRINTMODE, PMIC_RG_FGRINTMODE_ADDR, PMIC_RG_FGRINTMODE_MASK,
	 PMIC_RG_FGRINTMODE_SHIFT},
	{PMIC_RG_FG_WIDECM_ENB, PMIC_RG_FG_WIDECM_ENB_ADDR, PMIC_RG_FG_WIDECM_ENB_MASK,
	 PMIC_RG_FG_WIDECM_ENB_SHIFT},
	{PMIC_RG_SPARE, PMIC_RG_SPARE_ADDR, PMIC_RG_SPARE_MASK, PMIC_RG_SPARE_SHIFT},
	{PMIC_FG_MODE, PMIC_FG_MODE_ADDR, PMIC_FG_MODE_MASK, PMIC_FG_MODE_SHIFT},
	{PMIC_FG_RST_SW, PMIC_FG_RST_SW_ADDR, PMIC_FG_RST_SW_MASK, PMIC_FG_RST_SW_SHIFT},
	{PMIC_FG_FGCAL_EN_SW, PMIC_FG_FGCAL_EN_SW_ADDR, PMIC_FG_FGCAL_EN_SW_MASK,
	 PMIC_FG_FGCAL_EN_SW_SHIFT},
	{PMIC_FG_FGADC_EN_SW, PMIC_FG_FGADC_EN_SW_ADDR, PMIC_FG_FGADC_EN_SW_MASK,
	 PMIC_FG_FGADC_EN_SW_SHIFT},
	{PMIC_FG_RSV1, PMIC_FG_RSV1_ADDR, PMIC_FG_RSV1_MASK, PMIC_FG_RSV1_SHIFT},
	{PMIC_FG_TEST_MODE0, PMIC_FG_TEST_MODE0_ADDR, PMIC_FG_TEST_MODE0_MASK,
	 PMIC_FG_TEST_MODE0_SHIFT},
	{PMIC_FG_TEST_MODE1, PMIC_FG_TEST_MODE1_ADDR, PMIC_FG_TEST_MODE1_MASK,
	 PMIC_FG_TEST_MODE1_SHIFT},
	{PMIC_FG_SOFF_RD, PMIC_FG_SOFF_RD_ADDR, PMIC_FG_SOFF_RD_MASK, PMIC_FG_SOFF_RD_SHIFT},
	{PMIC_FG_RSV_CON0_RSV, PMIC_FG_RSV_CON0_RSV_ADDR, PMIC_FG_RSV_CON0_RSV_MASK,
	 PMIC_FG_RSV_CON0_RSV_SHIFT},
	{PMIC_FG_RSV_CON1_RSV, PMIC_FG_RSV_CON1_RSV_ADDR, PMIC_FG_RSV_CON1_RSV_MASK,
	 PMIC_FG_RSV_CON1_RSV_SHIFT},
	{PMIC_FG_RSV_CON2_RSV, PMIC_FG_RSV_CON2_RSV_ADDR, PMIC_FG_RSV_CON2_RSV_MASK,
	 PMIC_FG_RSV_CON2_RSV_SHIFT},
	{PMIC_FG_RSV_CON3_RSV, PMIC_FG_RSV_CON3_RSV_ADDR, PMIC_FG_RSV_CON3_RSV_MASK,
	 PMIC_FG_RSV_CON3_RSV_SHIFT},
	{PMIC_SYSTEM_INFO_CON0, PMIC_SYSTEM_INFO_CON0_ADDR, PMIC_SYSTEM_INFO_CON0_MASK,
	 PMIC_SYSTEM_INFO_CON0_SHIFT},
	{PMIC_SYSTEM_INFO_CON1, PMIC_SYSTEM_INFO_CON1_ADDR, PMIC_SYSTEM_INFO_CON1_MASK,
	 PMIC_SYSTEM_INFO_CON1_SHIFT},
	{PMIC_SYSTEM_INFO_CON2, PMIC_SYSTEM_INFO_CON2_ADDR, PMIC_SYSTEM_INFO_CON2_MASK,
	 PMIC_SYSTEM_INFO_CON2_SHIFT},
	{PMIC_SYSTEM_INFO_CON3, PMIC_SYSTEM_INFO_CON3_ADDR, PMIC_SYSTEM_INFO_CON3_MASK,
	 PMIC_SYSTEM_INFO_CON3_SHIFT},
	{PMIC_SYSTEM_INFO_CON4, PMIC_SYSTEM_INFO_CON4_ADDR, PMIC_SYSTEM_INFO_CON4_MASK,
	 PMIC_SYSTEM_INFO_CON4_SHIFT},
	{PMIC_RG_FGADC_GAINERROR_CAL, PMIC_RG_FGADC_GAINERROR_CAL_ADDR,
	 PMIC_RG_FGADC_GAINERROR_CAL_MASK, PMIC_RG_FGADC_GAINERROR_CAL_SHIFT},
	{PMIC_XO_EXTBUF1_MODE, PMIC_XO_EXTBUF1_MODE_ADDR, PMIC_XO_EXTBUF1_MODE_MASK,
	 PMIC_XO_EXTBUF1_MODE_SHIFT},
	{PMIC_XO_EXTBUF1_EN_M, PMIC_XO_EXTBUF1_EN_M_ADDR, PMIC_XO_EXTBUF1_EN_M_MASK,
	 PMIC_XO_EXTBUF1_EN_M_SHIFT},
	{PMIC_XO_EXTBUF2_MODE, PMIC_XO_EXTBUF2_MODE_ADDR, PMIC_XO_EXTBUF2_MODE_MASK,
	 PMIC_XO_EXTBUF2_MODE_SHIFT},
	{PMIC_XO_EXTBUF2_EN_M, PMIC_XO_EXTBUF2_EN_M_ADDR, PMIC_XO_EXTBUF2_EN_M_MASK,
	 PMIC_XO_EXTBUF2_EN_M_SHIFT},
	{PMIC_XO_EXTBUF3_MODE, PMIC_XO_EXTBUF3_MODE_ADDR, PMIC_XO_EXTBUF3_MODE_MASK,
	 PMIC_XO_EXTBUF3_MODE_SHIFT},
	{PMIC_XO_EXTBUF3_EN_M, PMIC_XO_EXTBUF3_EN_M_ADDR, PMIC_XO_EXTBUF3_EN_M_MASK,
	 PMIC_XO_EXTBUF3_EN_M_SHIFT},
	{PMIC_XO_EXTBUF4_MODE, PMIC_XO_EXTBUF4_MODE_ADDR, PMIC_XO_EXTBUF4_MODE_MASK,
	 PMIC_XO_EXTBUF4_MODE_SHIFT},
	{PMIC_XO_EXTBUF4_EN_M, PMIC_XO_EXTBUF4_EN_M_ADDR, PMIC_XO_EXTBUF4_EN_M_MASK,
	 PMIC_XO_EXTBUF4_EN_M_SHIFT},
	{PMIC_XO_BB_LPM_EN, PMIC_XO_BB_LPM_EN_ADDR, PMIC_XO_BB_LPM_EN_MASK,
	 PMIC_XO_BB_LPM_EN_SHIFT},
	{PMIC_XO_ENBB_MAN, PMIC_XO_ENBB_MAN_ADDR, PMIC_XO_ENBB_MAN_MASK, PMIC_XO_ENBB_MAN_SHIFT},
	{PMIC_XO_ENBB_EN_M, PMIC_XO_ENBB_EN_M_ADDR, PMIC_XO_ENBB_EN_M_MASK,
	 PMIC_XO_ENBB_EN_M_SHIFT},
	{PMIC_XO_CLKSEL_MAN, PMIC_XO_CLKSEL_MAN_ADDR, PMIC_XO_CLKSEL_MAN_MASK,
	 PMIC_XO_CLKSEL_MAN_SHIFT},
	{PMIC_DCXO_CW00_SET, PMIC_DCXO_CW00_SET_ADDR, PMIC_DCXO_CW00_SET_MASK,
	 PMIC_DCXO_CW00_SET_SHIFT},
	{PMIC_DCXO_CW00_CLR, PMIC_DCXO_CW00_CLR_ADDR, PMIC_DCXO_CW00_CLR_MASK,
	 PMIC_DCXO_CW00_CLR_SHIFT},
	{PMIC_XO_CLKSEL_EN_M, PMIC_XO_CLKSEL_EN_M_ADDR, PMIC_XO_CLKSEL_EN_M_MASK,
	 PMIC_XO_CLKSEL_EN_M_SHIFT},
	{PMIC_XO_EXTBUF1_CKG_MAN, PMIC_XO_EXTBUF1_CKG_MAN_ADDR, PMIC_XO_EXTBUF1_CKG_MAN_MASK,
	 PMIC_XO_EXTBUF1_CKG_MAN_SHIFT},
	{PMIC_XO_EXTBUF1_CKG_EN_M, PMIC_XO_EXTBUF1_CKG_EN_M_ADDR, PMIC_XO_EXTBUF1_CKG_EN_M_MASK,
	 PMIC_XO_EXTBUF1_CKG_EN_M_SHIFT},
	{PMIC_XO_EXTBUF2_CKG_MAN, PMIC_XO_EXTBUF2_CKG_MAN_ADDR, PMIC_XO_EXTBUF2_CKG_MAN_MASK,
	 PMIC_XO_EXTBUF2_CKG_MAN_SHIFT},
	{PMIC_XO_EXTBUF2_CKG_EN_M, PMIC_XO_EXTBUF2_CKG_EN_M_ADDR, PMIC_XO_EXTBUF2_CKG_EN_M_MASK,
	 PMIC_XO_EXTBUF2_CKG_EN_M_SHIFT},
	{PMIC_XO_EXTBUF3_CKG_MAN, PMIC_XO_EXTBUF3_CKG_MAN_ADDR, PMIC_XO_EXTBUF3_CKG_MAN_MASK,
	 PMIC_XO_EXTBUF3_CKG_MAN_SHIFT},
	{PMIC_XO_EXTBUF3_CKG_EN_M, PMIC_XO_EXTBUF3_CKG_EN_M_ADDR, PMIC_XO_EXTBUF3_CKG_EN_M_MASK,
	 PMIC_XO_EXTBUF3_CKG_EN_M_SHIFT},
	{PMIC_XO_EXTBUF4_CKG_MAN, PMIC_XO_EXTBUF4_CKG_MAN_ADDR, PMIC_XO_EXTBUF4_CKG_MAN_MASK,
	 PMIC_XO_EXTBUF4_CKG_MAN_SHIFT},
	{PMIC_XO_EXTBUF4_CKG_EN_M, PMIC_XO_EXTBUF4_CKG_EN_M_ADDR, PMIC_XO_EXTBUF4_CKG_EN_M_MASK,
	 PMIC_XO_EXTBUF4_CKG_EN_M_SHIFT},
	{PMIC_XO_INTBUF_MAN, PMIC_XO_INTBUF_MAN_ADDR, PMIC_XO_INTBUF_MAN_MASK,
	 PMIC_XO_INTBUF_MAN_SHIFT},
	{PMIC_XO_PBUF_EN_M, PMIC_XO_PBUF_EN_M_ADDR, PMIC_XO_PBUF_EN_M_MASK,
	 PMIC_XO_PBUF_EN_M_SHIFT},
	{PMIC_XO_IBUF_EN_M, PMIC_XO_IBUF_EN_M_ADDR, PMIC_XO_IBUF_EN_M_MASK,
	 PMIC_XO_IBUF_EN_M_SHIFT},
	{PMIC_XO_LPMBUF_MAN, PMIC_XO_LPMBUF_MAN_ADDR, PMIC_XO_LPMBUF_MAN_MASK,
	 PMIC_XO_LPMBUF_MAN_SHIFT},
	{PMIC_XO_LPM_PREBUF_EN_M, PMIC_XO_LPM_PREBUF_EN_M_ADDR, PMIC_XO_LPM_PREBUF_EN_M_MASK,
	 PMIC_XO_LPM_PREBUF_EN_M_SHIFT},
	{PMIC_XO_LPBUF_EN_M, PMIC_XO_LPBUF_EN_M_ADDR, PMIC_XO_LPBUF_EN_M_MASK,
	 PMIC_XO_LPBUF_EN_M_SHIFT},
	{PMIC_XO_BBLPM_CKSEL_M, PMIC_XO_BBLPM_CKSEL_M_ADDR, PMIC_XO_BBLPM_CKSEL_M_MASK,
	 PMIC_XO_BBLPM_CKSEL_M_SHIFT},
	{PMIC_XO_EN32K_MAN, PMIC_XO_EN32K_MAN_ADDR, PMIC_XO_EN32K_MAN_MASK,
	 PMIC_XO_EN32K_MAN_SHIFT},
	{PMIC_XO_EN32K_M, PMIC_XO_EN32K_M_ADDR, PMIC_XO_EN32K_M_MASK, PMIC_XO_EN32K_M_SHIFT},
	{PMIC_XO_XMODE_MAN, PMIC_XO_XMODE_MAN_ADDR, PMIC_XO_XMODE_MAN_MASK,
	 PMIC_XO_XMODE_MAN_SHIFT},
	{PMIC_XO_XMODE_M, PMIC_XO_XMODE_M_ADDR, PMIC_XO_XMODE_M_MASK, PMIC_XO_XMODE_M_SHIFT},
	{PMIC_XO_STRUP_MODE, PMIC_XO_STRUP_MODE_ADDR, PMIC_XO_STRUP_MODE_MASK,
	 PMIC_XO_STRUP_MODE_SHIFT},
	{PMIC_XO_AAC_FPM_TIME, PMIC_XO_AAC_FPM_TIME_ADDR, PMIC_XO_AAC_FPM_TIME_MASK,
	 PMIC_XO_AAC_FPM_TIME_SHIFT},
	{PMIC_XO_AAC_MODE_LPM, PMIC_XO_AAC_MODE_LPM_ADDR, PMIC_XO_AAC_MODE_LPM_MASK,
	 PMIC_XO_AAC_MODE_LPM_SHIFT},
	{PMIC_XO_AAC_MODE_FPM, PMIC_XO_AAC_MODE_FPM_ADDR, PMIC_XO_AAC_MODE_FPM_MASK,
	 PMIC_XO_AAC_MODE_FPM_SHIFT},
	{PMIC_XO_EN26M_OFFSQ_EN, PMIC_XO_EN26M_OFFSQ_EN_ADDR, PMIC_XO_EN26M_OFFSQ_EN_MASK,
	 PMIC_XO_EN26M_OFFSQ_EN_SHIFT},
	{PMIC_XO_LDOCAL_EN, PMIC_XO_LDOCAL_EN_ADDR, PMIC_XO_LDOCAL_EN_MASK,
	 PMIC_XO_LDOCAL_EN_SHIFT},
	{PMIC_XO_CBANK_SYNC_DYN, PMIC_XO_CBANK_SYNC_DYN_ADDR, PMIC_XO_CBANK_SYNC_DYN_MASK,
	 PMIC_XO_CBANK_SYNC_DYN_SHIFT},
	{PMIC_XO_26MLP_MAN_EN, PMIC_XO_26MLP_MAN_EN_ADDR, PMIC_XO_26MLP_MAN_EN_MASK,
	 PMIC_XO_26MLP_MAN_EN_SHIFT},
	{PMIC_XO_BUFLDOK_EN, PMIC_XO_BUFLDOK_EN_ADDR, PMIC_XO_BUFLDOK_EN_MASK,
	 PMIC_XO_BUFLDOK_EN_SHIFT},
	{PMIC_XO_EXTBUF5_CKG_MAN, PMIC_XO_EXTBUF5_CKG_MAN_ADDR, PMIC_XO_EXTBUF5_CKG_MAN_MASK,
	 PMIC_XO_EXTBUF5_CKG_MAN_SHIFT},
	{PMIC_XO_EXTBUF5_CKG_EN_M, PMIC_XO_EXTBUF5_CKG_EN_M_ADDR, PMIC_XO_EXTBUF5_CKG_EN_M_MASK,
	 PMIC_XO_EXTBUF5_CKG_EN_M_SHIFT},
	{PMIC_XO_EXTBUF6_CKG_MAN, PMIC_XO_EXTBUF6_CKG_MAN_ADDR, PMIC_XO_EXTBUF6_CKG_MAN_MASK,
	 PMIC_XO_EXTBUF6_CKG_MAN_SHIFT},
	{PMIC_XO_EXTBUF6_CKG_EN_M, PMIC_XO_EXTBUF6_CKG_EN_M_ADDR, PMIC_XO_EXTBUF6_CKG_EN_M_MASK,
	 PMIC_XO_EXTBUF6_CKG_EN_M_SHIFT},
	{PMIC_XO_EXTBUF7_CKG_MAN, PMIC_XO_EXTBUF7_CKG_MAN_ADDR, PMIC_XO_EXTBUF7_CKG_MAN_MASK,
	 PMIC_XO_EXTBUF7_CKG_MAN_SHIFT},
	{PMIC_XO_EXTBUF7_CKG_EN_M, PMIC_XO_EXTBUF7_CKG_EN_M_ADDR, PMIC_XO_EXTBUF7_CKG_EN_M_MASK,
	 PMIC_XO_EXTBUF7_CKG_EN_M_SHIFT},
	{PMIC_XO_LPM_ISEL_MAN, PMIC_XO_LPM_ISEL_MAN_ADDR, PMIC_XO_LPM_ISEL_MAN_MASK,
	 PMIC_XO_LPM_ISEL_MAN_SHIFT},
	{PMIC_XO_FPM_ISEL_MAN, PMIC_XO_FPM_ISEL_MAN_ADDR, PMIC_XO_FPM_ISEL_MAN_MASK,
	 PMIC_XO_FPM_ISEL_MAN_SHIFT},
	{PMIC_XO_CDAC_FPM, PMIC_XO_CDAC_FPM_ADDR, PMIC_XO_CDAC_FPM_MASK, PMIC_XO_CDAC_FPM_SHIFT},
	{PMIC_XO_CDAC_LPM, PMIC_XO_CDAC_LPM_ADDR, PMIC_XO_CDAC_LPM_MASK, PMIC_XO_CDAC_LPM_SHIFT},
	{PMIC_XO_32KDIV_NFRAC_FPM, PMIC_XO_32KDIV_NFRAC_FPM_ADDR, PMIC_XO_32KDIV_NFRAC_FPM_MASK,
	 PMIC_XO_32KDIV_NFRAC_FPM_SHIFT},
	{PMIC_XO_COFST_FPM, PMIC_XO_COFST_FPM_ADDR, PMIC_XO_COFST_FPM_MASK,
	 PMIC_XO_COFST_FPM_SHIFT},
	{PMIC_XO_32KDIV_NFRAC_LPM, PMIC_XO_32KDIV_NFRAC_LPM_ADDR, PMIC_XO_32KDIV_NFRAC_LPM_MASK,
	 PMIC_XO_32KDIV_NFRAC_LPM_SHIFT},
	{PMIC_XO_COFST_LPM, PMIC_XO_COFST_LPM_ADDR, PMIC_XO_COFST_LPM_MASK,
	 PMIC_XO_COFST_LPM_SHIFT},
	{PMIC_XO_CORE_MAN, PMIC_XO_CORE_MAN_ADDR, PMIC_XO_CORE_MAN_MASK, PMIC_XO_CORE_MAN_SHIFT},
	{PMIC_XO_CORE_EN_M, PMIC_XO_CORE_EN_M_ADDR, PMIC_XO_CORE_EN_M_MASK,
	 PMIC_XO_CORE_EN_M_SHIFT},
	{PMIC_XO_CORE_TURBO_EN_M, PMIC_XO_CORE_TURBO_EN_M_ADDR, PMIC_XO_CORE_TURBO_EN_M_MASK,
	 PMIC_XO_CORE_TURBO_EN_M_SHIFT},
	{PMIC_XO_CORE_AAC_EN_M, PMIC_XO_CORE_AAC_EN_M_ADDR, PMIC_XO_CORE_AAC_EN_M_MASK,
	 PMIC_XO_CORE_AAC_EN_M_SHIFT},
	{PMIC_XO_STARTUP_EN_M, PMIC_XO_STARTUP_EN_M_ADDR, PMIC_XO_STARTUP_EN_M_MASK,
	 PMIC_XO_STARTUP_EN_M_SHIFT},
	{PMIC_XO_CORE_VBFPM_EN_M, PMIC_XO_CORE_VBFPM_EN_M_ADDR, PMIC_XO_CORE_VBFPM_EN_M_MASK,
	 PMIC_XO_CORE_VBFPM_EN_M_SHIFT},
	{PMIC_XO_CORE_VBLPM_EN_M, PMIC_XO_CORE_VBLPM_EN_M_ADDR, PMIC_XO_CORE_VBLPM_EN_M_MASK,
	 PMIC_XO_CORE_VBLPM_EN_M_SHIFT},
	{PMIC_XO_LPMBIAS_EN_M, PMIC_XO_LPMBIAS_EN_M_ADDR, PMIC_XO_LPMBIAS_EN_M_MASK,
	 PMIC_XO_LPMBIAS_EN_M_SHIFT},
	{PMIC_XO_VTCGEN_EN_M, PMIC_XO_VTCGEN_EN_M_ADDR, PMIC_XO_VTCGEN_EN_M_MASK,
	 PMIC_XO_VTCGEN_EN_M_SHIFT},
	{PMIC_XO_IAAC_COMP_EN_M, PMIC_XO_IAAC_COMP_EN_M_ADDR, PMIC_XO_IAAC_COMP_EN_M_MASK,
	 PMIC_XO_IAAC_COMP_EN_M_SHIFT},
	{PMIC_XO_IFPM_COMP_EN_M, PMIC_XO_IFPM_COMP_EN_M_ADDR, PMIC_XO_IFPM_COMP_EN_M_MASK,
	 PMIC_XO_IFPM_COMP_EN_M_SHIFT},
	{PMIC_XO_ILPM_COMP_EN_M, PMIC_XO_ILPM_COMP_EN_M_ADDR, PMIC_XO_ILPM_COMP_EN_M_MASK,
	 PMIC_XO_ILPM_COMP_EN_M_SHIFT},
	{PMIC_XO_CORE_BYPCAS_FPM, PMIC_XO_CORE_BYPCAS_FPM_ADDR, PMIC_XO_CORE_BYPCAS_FPM_MASK,
	 PMIC_XO_CORE_BYPCAS_FPM_SHIFT},
	{PMIC_XO_CORE_GMX2_FPM, PMIC_XO_CORE_GMX2_FPM_ADDR, PMIC_XO_CORE_GMX2_FPM_MASK,
	 PMIC_XO_CORE_GMX2_FPM_SHIFT},
	{PMIC_XO_CORE_IDAC_FPM, PMIC_XO_CORE_IDAC_FPM_ADDR, PMIC_XO_CORE_IDAC_FPM_MASK,
	 PMIC_XO_CORE_IDAC_FPM_SHIFT},
	{PMIC_XO_AAC_COMP_MAN, PMIC_XO_AAC_COMP_MAN_ADDR, PMIC_XO_AAC_COMP_MAN_MASK,
	 PMIC_XO_AAC_COMP_MAN_SHIFT},
	{PMIC_XO_AAC_EN_M, PMIC_XO_AAC_EN_M_ADDR, PMIC_XO_AAC_EN_M_MASK, PMIC_XO_AAC_EN_M_SHIFT},
	{PMIC_XO_AAC_MONEN_M, PMIC_XO_AAC_MONEN_M_ADDR, PMIC_XO_AAC_MONEN_M_MASK,
	 PMIC_XO_AAC_MONEN_M_SHIFT},
	{PMIC_XO_COMP_EN_M, PMIC_XO_COMP_EN_M_ADDR, PMIC_XO_COMP_EN_M_MASK,
	 PMIC_XO_COMP_EN_M_SHIFT},
	{PMIC_XO_COMP_TSTEN_M, PMIC_XO_COMP_TSTEN_M_ADDR, PMIC_XO_COMP_TSTEN_M_MASK,
	 PMIC_XO_COMP_TSTEN_M_SHIFT},
	{PMIC_XO_AAC_HV_FPM, PMIC_XO_AAC_HV_FPM_ADDR, PMIC_XO_AAC_HV_FPM_MASK,
	 PMIC_XO_AAC_HV_FPM_SHIFT},
	{PMIC_XO_AAC_IBIAS_FPM, PMIC_XO_AAC_IBIAS_FPM_ADDR, PMIC_XO_AAC_IBIAS_FPM_MASK,
	 PMIC_XO_AAC_IBIAS_FPM_SHIFT},
	{PMIC_XO_AAC_VOFST_FPM, PMIC_XO_AAC_VOFST_FPM_ADDR, PMIC_XO_AAC_VOFST_FPM_MASK,
	 PMIC_XO_AAC_VOFST_FPM_SHIFT},
	{PMIC_XO_AAC_COMP_HV_FPM, PMIC_XO_AAC_COMP_HV_FPM_ADDR, PMIC_XO_AAC_COMP_HV_FPM_MASK,
	 PMIC_XO_AAC_COMP_HV_FPM_SHIFT},
	{PMIC_XO_AAC_VSEL_FPM, PMIC_XO_AAC_VSEL_FPM_ADDR, PMIC_XO_AAC_VSEL_FPM_MASK,
	 PMIC_XO_AAC_VSEL_FPM_SHIFT},
	{PMIC_XO_AAC_COMP_POL, PMIC_XO_AAC_COMP_POL_ADDR, PMIC_XO_AAC_COMP_POL_MASK,
	 PMIC_XO_AAC_COMP_POL_SHIFT},
	{PMIC_XO_CORE_BYPCAS_LPM, PMIC_XO_CORE_BYPCAS_LPM_ADDR, PMIC_XO_CORE_BYPCAS_LPM_MASK,
	 PMIC_XO_CORE_BYPCAS_LPM_SHIFT},
	{PMIC_XO_CORE_GMX2_LPM, PMIC_XO_CORE_GMX2_LPM_ADDR, PMIC_XO_CORE_GMX2_LPM_MASK,
	 PMIC_XO_CORE_GMX2_LPM_SHIFT},
	{PMIC_XO_CORE_IDAC_LPM, PMIC_XO_CORE_IDAC_LPM_ADDR, PMIC_XO_CORE_IDAC_LPM_MASK,
	 PMIC_XO_CORE_IDAC_LPM_SHIFT},
	{PMIC_XO_AAC_COMP_HV_LPM, PMIC_XO_AAC_COMP_HV_LPM_ADDR, PMIC_XO_AAC_COMP_HV_LPM_MASK,
	 PMIC_XO_AAC_COMP_HV_LPM_SHIFT},
	{PMIC_XO_AAC_VSEL_LPM, PMIC_XO_AAC_VSEL_LPM_ADDR, PMIC_XO_AAC_VSEL_LPM_MASK,
	 PMIC_XO_AAC_VSEL_LPM_SHIFT},
	{PMIC_XO_AAC_HV_LPM, PMIC_XO_AAC_HV_LPM_ADDR, PMIC_XO_AAC_HV_LPM_MASK,
	 PMIC_XO_AAC_HV_LPM_SHIFT},
	{PMIC_XO_AAC_IBIAS_LPM, PMIC_XO_AAC_IBIAS_LPM_ADDR, PMIC_XO_AAC_IBIAS_LPM_MASK,
	 PMIC_XO_AAC_IBIAS_LPM_SHIFT},
	{PMIC_XO_AAC_VOFST_LPM, PMIC_XO_AAC_VOFST_LPM_ADDR, PMIC_XO_AAC_VOFST_LPM_MASK,
	 PMIC_XO_AAC_VOFST_LPM_SHIFT},
	{PMIC_XO_AAC_FPM_SWEN, PMIC_XO_AAC_FPM_SWEN_ADDR, PMIC_XO_AAC_FPM_SWEN_MASK,
	 PMIC_XO_AAC_FPM_SWEN_SHIFT},
	{PMIC_XO_SWRST, PMIC_XO_SWRST_ADDR, PMIC_XO_SWRST_MASK, PMIC_XO_SWRST_SHIFT},
	{PMIC_XO_32KDIV_SWRST, PMIC_XO_32KDIV_SWRST_ADDR, PMIC_XO_32KDIV_SWRST_MASK,
	 PMIC_XO_32KDIV_SWRST_SHIFT},
	{PMIC_XO_32KDIV_RATIO_MAN, PMIC_XO_32KDIV_RATIO_MAN_ADDR, PMIC_XO_32KDIV_RATIO_MAN_MASK,
	 PMIC_XO_32KDIV_RATIO_MAN_SHIFT},
	{PMIC_XO_32KDIV_TEST_EN, PMIC_XO_32KDIV_TEST_EN_ADDR, PMIC_XO_32KDIV_TEST_EN_MASK,
	 PMIC_XO_32KDIV_TEST_EN_SHIFT},
	{PMIC_XO_CBANK_SYNC_MAN, PMIC_XO_CBANK_SYNC_MAN_ADDR, PMIC_XO_CBANK_SYNC_MAN_MASK,
	 PMIC_XO_CBANK_SYNC_MAN_SHIFT},
	{PMIC_XO_CBANK_SYNC_EN_M, PMIC_XO_CBANK_SYNC_EN_M_ADDR, PMIC_XO_CBANK_SYNC_EN_M_MASK,
	 PMIC_XO_CBANK_SYNC_EN_M_SHIFT},
	{PMIC_XO_CTL_SYNC_MAN, PMIC_XO_CTL_SYNC_MAN_ADDR, PMIC_XO_CTL_SYNC_MAN_MASK,
	 PMIC_XO_CTL_SYNC_MAN_SHIFT},
	{PMIC_XO_CTL_SYNC_EN_M, PMIC_XO_CTL_SYNC_EN_M_ADDR, PMIC_XO_CTL_SYNC_EN_M_MASK,
	 PMIC_XO_CTL_SYNC_EN_M_SHIFT},
	{PMIC_XO_LDO_MAN, PMIC_XO_LDO_MAN_ADDR, PMIC_XO_LDO_MAN_MASK, PMIC_XO_LDO_MAN_SHIFT},
	{PMIC_XO_LDOPBUF_EN_M, PMIC_XO_LDOPBUF_EN_M_ADDR, PMIC_XO_LDOPBUF_EN_M_MASK,
	 PMIC_XO_LDOPBUF_EN_M_SHIFT},
	{PMIC_XO_LDOPBUF_VSET_M, PMIC_XO_LDOPBUF_VSET_M_ADDR, PMIC_XO_LDOPBUF_VSET_M_MASK,
	 PMIC_XO_LDOPBUF_VSET_M_SHIFT},
	{PMIC_XO_LDOVTST_EN_M, PMIC_XO_LDOVTST_EN_M_ADDR, PMIC_XO_LDOVTST_EN_M_MASK,
	 PMIC_XO_LDOVTST_EN_M_SHIFT},
	{PMIC_XO_TEST_VCAL_EN_M, PMIC_XO_TEST_VCAL_EN_M_ADDR, PMIC_XO_TEST_VCAL_EN_M_MASK,
	 PMIC_XO_TEST_VCAL_EN_M_SHIFT},
	{PMIC_XO_VBIST_EN_M, PMIC_XO_VBIST_EN_M_ADDR, PMIC_XO_VBIST_EN_M_MASK,
	 PMIC_XO_VBIST_EN_M_SHIFT},
	{PMIC_XO_VTEST_SEL_MUX, PMIC_XO_VTEST_SEL_MUX_ADDR, PMIC_XO_VTEST_SEL_MUX_MASK,
	 PMIC_XO_VTEST_SEL_MUX_SHIFT},
	{PMIC_XO_EXTBUF5_MODE, PMIC_XO_EXTBUF5_MODE_ADDR, PMIC_XO_EXTBUF5_MODE_MASK,
	 PMIC_XO_EXTBUF5_MODE_SHIFT},
	{PMIC_XO_EXTBUF5_EN_M, PMIC_XO_EXTBUF5_EN_M_ADDR, PMIC_XO_EXTBUF5_EN_M_MASK,
	 PMIC_XO_EXTBUF5_EN_M_SHIFT},
	{PMIC_XO_EXTBUF6_MODE, PMIC_XO_EXTBUF6_MODE_ADDR, PMIC_XO_EXTBUF6_MODE_MASK,
	 PMIC_XO_EXTBUF6_MODE_SHIFT},
	{PMIC_XO_EXTBUF6_EN_M, PMIC_XO_EXTBUF6_EN_M_ADDR, PMIC_XO_EXTBUF6_EN_M_MASK,
	 PMIC_XO_EXTBUF6_EN_M_SHIFT},
	{PMIC_XO_EXTBUF7_MODE, PMIC_XO_EXTBUF7_MODE_ADDR, PMIC_XO_EXTBUF7_MODE_MASK,
	 PMIC_XO_EXTBUF7_MODE_SHIFT},
	{PMIC_XO_EXTBUF7_EN_M, PMIC_XO_EXTBUF7_EN_M_ADDR, PMIC_XO_EXTBUF7_EN_M_MASK,
	 PMIC_XO_EXTBUF7_EN_M_SHIFT},
	{PMIC_XO_BUFLDOK_MAN, PMIC_XO_BUFLDOK_MAN_ADDR, PMIC_XO_BUFLDOK_MAN_MASK,
	 PMIC_XO_BUFLDOK_MAN_SHIFT},
	{PMIC_XO_BUF1LDO_CAL_M, PMIC_XO_BUF1LDO_CAL_M_ADDR, PMIC_XO_BUF1LDO_CAL_M_MASK,
	 PMIC_XO_BUF1LDO_CAL_M_SHIFT},
	{PMIC_DCXO_CW11_SET, PMIC_DCXO_CW11_SET_ADDR, PMIC_DCXO_CW11_SET_MASK,
	 PMIC_DCXO_CW11_SET_SHIFT},
	{PMIC_DCXO_CW11_CLR, PMIC_DCXO_CW11_CLR_ADDR, PMIC_DCXO_CW11_CLR_MASK,
	 PMIC_DCXO_CW11_CLR_SHIFT},
	{PMIC_XO_BUFLDO_CAL_M, PMIC_XO_BUFLDO_CAL_M_ADDR, PMIC_XO_BUFLDO_CAL_M_MASK,
	 PMIC_XO_BUFLDO_CAL_M_SHIFT},
	{PMIC_XO_EXTBUF4_CLKSEL_MAN, PMIC_XO_EXTBUF4_CLKSEL_MAN_ADDR,
	 PMIC_XO_EXTBUF4_CLKSEL_MAN_MASK, PMIC_XO_EXTBUF4_CLKSEL_MAN_SHIFT},
	{PMIC_XO_VIO18PG_BUFEN, PMIC_XO_VIO18PG_BUFEN_ADDR, PMIC_XO_VIO18PG_BUFEN_MASK,
	 PMIC_XO_VIO18PG_BUFEN_SHIFT},
	{PMIC_XO_CAL_EN_MAN, PMIC_XO_CAL_EN_MAN_ADDR, PMIC_XO_CAL_EN_MAN_MASK,
	 PMIC_XO_CAL_EN_MAN_SHIFT},
	{PMIC_XO_CAL_EN_M, PMIC_XO_CAL_EN_M_ADDR, PMIC_XO_CAL_EN_M_MASK, PMIC_XO_CAL_EN_M_SHIFT},
	{PMIC_RG_XO_CORE_OSCTD, PMIC_RG_XO_CORE_OSCTD_ADDR, PMIC_RG_XO_CORE_OSCTD_MASK,
	 PMIC_RG_XO_CORE_OSCTD_SHIFT},
	{PMIC_RG_XO_THADC_EN, PMIC_RG_XO_THADC_EN_ADDR, PMIC_RG_XO_THADC_EN_MASK,
	 PMIC_RG_XO_THADC_EN_SHIFT},
	{PMIC_RG_XO_SYNC_CKPOL, PMIC_RG_XO_SYNC_CKPOL_ADDR, PMIC_RG_XO_SYNC_CKPOL_MASK,
	 PMIC_RG_XO_SYNC_CKPOL_SHIFT},
	{PMIC_RG_XO_CBANK_POL, PMIC_RG_XO_CBANK_POL_ADDR, PMIC_RG_XO_CBANK_POL_MASK,
	 PMIC_RG_XO_CBANK_POL_SHIFT},
	{PMIC_RG_XO_CBANK_SYNC_BYP, PMIC_RG_XO_CBANK_SYNC_BYP_ADDR, PMIC_RG_XO_CBANK_SYNC_BYP_MASK,
	 PMIC_RG_XO_CBANK_SYNC_BYP_SHIFT},
	{PMIC_RG_XO_CTL_POL, PMIC_RG_XO_CTL_POL_ADDR, PMIC_RG_XO_CTL_POL_MASK,
	 PMIC_RG_XO_CTL_POL_SHIFT},
	{PMIC_RG_XO_CTL_SYNC_BYP, PMIC_RG_XO_CTL_SYNC_BYP_ADDR, PMIC_RG_XO_CTL_SYNC_BYP_MASK,
	 PMIC_RG_XO_CTL_SYNC_BYP_SHIFT},
	{PMIC_RG_XO_LPBUF_INV, PMIC_RG_XO_LPBUF_INV_ADDR, PMIC_RG_XO_LPBUF_INV_MASK,
	 PMIC_RG_XO_LPBUF_INV_SHIFT},
	{PMIC_RG_XO_LDOPBUF_BYP, PMIC_RG_XO_LDOPBUF_BYP_ADDR, PMIC_RG_XO_LDOPBUF_BYP_MASK,
	 PMIC_RG_XO_LDOPBUF_BYP_SHIFT},
	{PMIC_RG_XO_LDOPBUF_ENCL, PMIC_RG_XO_LDOPBUF_ENCL_ADDR, PMIC_RG_XO_LDOPBUF_ENCL_MASK,
	 PMIC_RG_XO_LDOPBUF_ENCL_SHIFT},
	{PMIC_RG_XO_VGBIAS_VSET, PMIC_RG_XO_VGBIAS_VSET_ADDR, PMIC_RG_XO_VGBIAS_VSET_MASK,
	 PMIC_RG_XO_VGBIAS_VSET_SHIFT},
	{PMIC_RG_XO_PBUF_ISET, PMIC_RG_XO_PBUF_ISET_ADDR, PMIC_RG_XO_PBUF_ISET_MASK,
	 PMIC_RG_XO_PBUF_ISET_SHIFT},
	{PMIC_RG_XO_IBUF_ISET, PMIC_RG_XO_IBUF_ISET_ADDR, PMIC_RG_XO_IBUF_ISET_MASK,
	 PMIC_RG_XO_IBUF_ISET_SHIFT},
	{PMIC_RG_XO_BUFLDO1_ENCL, PMIC_RG_XO_BUFLDO1_ENCL_ADDR, PMIC_RG_XO_BUFLDO1_ENCL_MASK,
	 PMIC_RG_XO_BUFLDO1_ENCL_SHIFT},
	{PMIC_RG_XO_BUFLDO1_IBX2, PMIC_RG_XO_BUFLDO1_IBX2_ADDR, PMIC_RG_XO_BUFLDO1_IBX2_MASK,
	 PMIC_RG_XO_BUFLDO1_IBX2_SHIFT},
	{PMIC_RG_XO_BUFLDO1_IX2, PMIC_RG_XO_BUFLDO1_IX2_ADDR, PMIC_RG_XO_BUFLDO1_IX2_MASK,
	 PMIC_RG_XO_BUFLDO1_IX2_SHIFT},
	{PMIC_RG_XO_BUFLDO1_VSET, PMIC_RG_XO_BUFLDO1_VSET_ADDR, PMIC_RG_XO_BUFLDO1_VSET_MASK,
	 PMIC_RG_XO_BUFLDO1_VSET_SHIFT},
	{PMIC_RG_XO_BUFLDO24_ENCL, PMIC_RG_XO_BUFLDO24_ENCL_ADDR, PMIC_RG_XO_BUFLDO24_ENCL_MASK,
	 PMIC_RG_XO_BUFLDO24_ENCL_SHIFT},
	{PMIC_RG_XO_BUFLDO24_IBX2, PMIC_RG_XO_BUFLDO24_IBX2_ADDR, PMIC_RG_XO_BUFLDO24_IBX2_MASK,
	 PMIC_RG_XO_BUFLDO24_IBX2_SHIFT},
	{PMIC_RG_XO_BUFLDO24_VSET, PMIC_RG_XO_BUFLDO24_VSET_ADDR, PMIC_RG_XO_BUFLDO24_VSET_MASK,
	 PMIC_RG_XO_BUFLDO24_VSET_SHIFT},
	{PMIC_RG_XO_BUFLDO35_ENCL, PMIC_RG_XO_BUFLDO35_ENCL_ADDR, PMIC_RG_XO_BUFLDO35_ENCL_MASK,
	 PMIC_RG_XO_BUFLDO35_ENCL_SHIFT},
	{PMIC_RG_XO_BUFLDO35_IBX2, PMIC_RG_XO_BUFLDO35_IBX2_ADDR, PMIC_RG_XO_BUFLDO35_IBX2_MASK,
	 PMIC_RG_XO_BUFLDO35_IBX2_SHIFT},
	{PMIC_RG_XO_BUFLDO35_IX2, PMIC_RG_XO_BUFLDO35_IX2_ADDR, PMIC_RG_XO_BUFLDO35_IX2_MASK,
	 PMIC_RG_XO_BUFLDO35_IX2_SHIFT},
	{PMIC_RG_XO_BUFLDO35_VSET, PMIC_RG_XO_BUFLDO35_VSET_ADDR, PMIC_RG_XO_BUFLDO35_VSET_MASK,
	 PMIC_RG_XO_BUFLDO35_VSET_SHIFT},
	{PMIC_RG_XO_BUFLDO67_ENCL, PMIC_RG_XO_BUFLDO67_ENCL_ADDR, PMIC_RG_XO_BUFLDO67_ENCL_MASK,
	 PMIC_RG_XO_BUFLDO67_ENCL_SHIFT},
	{PMIC_RG_XO_BUFLDO67_IBX2, PMIC_RG_XO_BUFLDO67_IBX2_ADDR, PMIC_RG_XO_BUFLDO67_IBX2_MASK,
	 PMIC_RG_XO_BUFLDO67_IBX2_SHIFT},
	{PMIC_RG_XO_BUFLDO67_IX2, PMIC_RG_XO_BUFLDO67_IX2_ADDR, PMIC_RG_XO_BUFLDO67_IX2_MASK,
	 PMIC_RG_XO_BUFLDO67_IX2_SHIFT},
	{PMIC_RG_XO_BUFLDO67_VSET, PMIC_RG_XO_BUFLDO67_VSET_ADDR, PMIC_RG_XO_BUFLDO67_VSET_MASK,
	 PMIC_RG_XO_BUFLDO67_VSET_SHIFT},
	{PMIC_RG_XO_EXTBUF_INV, PMIC_RG_XO_EXTBUF_INV_ADDR, PMIC_RG_XO_EXTBUF_INV_MASK,
	 PMIC_RG_XO_EXTBUF_INV_SHIFT},
	{PMIC_RG_XO_RESERVED0, PMIC_RG_XO_RESERVED0_ADDR, PMIC_RG_XO_RESERVED0_MASK,
	 PMIC_RG_XO_RESERVED0_SHIFT},
	{PMIC_RG_XO_AUDIO_ATTEN, PMIC_RG_XO_AUDIO_ATTEN_ADDR, PMIC_RG_XO_AUDIO_ATTEN_MASK,
	 PMIC_RG_XO_AUDIO_ATTEN_SHIFT},
	{PMIC_RG_XO_AUDIO_ISET, PMIC_RG_XO_AUDIO_ISET_ADDR, PMIC_RG_XO_AUDIO_ISET_MASK,
	 PMIC_RG_XO_AUDIO_ISET_SHIFT},
	{PMIC_RG_XO_EXTBUF1_HD, PMIC_RG_XO_EXTBUF1_HD_ADDR, PMIC_RG_XO_EXTBUF1_HD_MASK,
	 PMIC_RG_XO_EXTBUF1_HD_SHIFT},
	{PMIC_RG_XO_EXTBUF2_HD, PMIC_RG_XO_EXTBUF2_HD_ADDR, PMIC_RG_XO_EXTBUF2_HD_MASK,
	 PMIC_RG_XO_EXTBUF2_HD_SHIFT},
	{PMIC_RG_XO_EXTBUF3_HD, PMIC_RG_XO_EXTBUF3_HD_ADDR, PMIC_RG_XO_EXTBUF3_HD_MASK,
	 PMIC_RG_XO_EXTBUF3_HD_SHIFT},
	{PMIC_RG_XO_EXTBUF4_HD, PMIC_RG_XO_EXTBUF4_HD_ADDR, PMIC_RG_XO_EXTBUF4_HD_MASK,
	 PMIC_RG_XO_EXTBUF4_HD_SHIFT},
	{PMIC_RG_XO_EXTBUF5_HD, PMIC_RG_XO_EXTBUF5_HD_ADDR, PMIC_RG_XO_EXTBUF5_HD_MASK,
	 PMIC_RG_XO_EXTBUF5_HD_SHIFT},
	{PMIC_RG_XO_EXTBUF6_HD, PMIC_RG_XO_EXTBUF6_HD_ADDR, PMIC_RG_XO_EXTBUF6_HD_MASK,
	 PMIC_RG_XO_EXTBUF6_HD_SHIFT},
	{PMIC_RG_XO_EXTBUF7_HD, PMIC_RG_XO_EXTBUF7_HD_ADDR, PMIC_RG_XO_EXTBUF7_HD_MASK,
	 PMIC_RG_XO_EXTBUF7_HD_SHIFT},
	{PMIC_XO_EXTBUF1_ISET_M, PMIC_XO_EXTBUF1_ISET_M_ADDR, PMIC_XO_EXTBUF1_ISET_M_MASK,
	 PMIC_XO_EXTBUF1_ISET_M_SHIFT},
	{PMIC_XO_EXTBUF2_ISET_M, PMIC_XO_EXTBUF2_ISET_M_ADDR, PMIC_XO_EXTBUF2_ISET_M_MASK,
	 PMIC_XO_EXTBUF2_ISET_M_SHIFT},
	{PMIC_XO_EXTBUF3_ISET_M, PMIC_XO_EXTBUF3_ISET_M_ADDR, PMIC_XO_EXTBUF3_ISET_M_MASK,
	 PMIC_XO_EXTBUF3_ISET_M_SHIFT},
	{PMIC_XO_EXTBUF4_ISET_M, PMIC_XO_EXTBUF4_ISET_M_ADDR, PMIC_XO_EXTBUF4_ISET_M_MASK,
	 PMIC_XO_EXTBUF4_ISET_M_SHIFT},
	{PMIC_XO_EXTBUF5_ISET_M, PMIC_XO_EXTBUF5_ISET_M_ADDR, PMIC_XO_EXTBUF5_ISET_M_MASK,
	 PMIC_XO_EXTBUF5_ISET_M_SHIFT},
	{PMIC_XO_EXTBUF6_ISET_M, PMIC_XO_EXTBUF6_ISET_M_ADDR, PMIC_XO_EXTBUF6_ISET_M_MASK,
	 PMIC_XO_EXTBUF6_ISET_M_SHIFT},
	{PMIC_XO_EXTBUF7_ISET_M, PMIC_XO_EXTBUF7_ISET_M_ADDR, PMIC_XO_EXTBUF7_ISET_M_MASK,
	 PMIC_XO_EXTBUF7_ISET_M_SHIFT},
	{PMIC_RG_XO_LPM_PREBUF_ISET, PMIC_RG_XO_LPM_PREBUF_ISET_ADDR,
	 PMIC_RG_XO_LPM_PREBUF_ISET_MASK, PMIC_RG_XO_LPM_PREBUF_ISET_SHIFT},
	{PMIC_RG_XO_RESERVED1, PMIC_RG_XO_RESERVED1_ADDR, PMIC_RG_XO_RESERVED1_MASK,
	 PMIC_RG_XO_RESERVED1_SHIFT},
	{PMIC_XO_STATIC_AUXOUT_SEL, PMIC_XO_STATIC_AUXOUT_SEL_ADDR, PMIC_XO_STATIC_AUXOUT_SEL_MASK,
	 PMIC_XO_STATIC_AUXOUT_SEL_SHIFT},
	{PMIC_XO_AUXOUT_SEL, PMIC_XO_AUXOUT_SEL_ADDR, PMIC_XO_AUXOUT_SEL_MASK,
	 PMIC_XO_AUXOUT_SEL_SHIFT},
	{PMIC_XO_STATIC_AUXOUT, PMIC_XO_STATIC_AUXOUT_ADDR, PMIC_XO_STATIC_AUXOUT_MASK,
	 PMIC_XO_STATIC_AUXOUT_SHIFT},
	{PMIC_AUXADC_ADC_OUT_CH0, PMIC_AUXADC_ADC_OUT_CH0_ADDR, PMIC_AUXADC_ADC_OUT_CH0_MASK,
	 PMIC_AUXADC_ADC_OUT_CH0_SHIFT},
	{PMIC_AUXADC_ADC_RDY_CH0, PMIC_AUXADC_ADC_RDY_CH0_ADDR, PMIC_AUXADC_ADC_RDY_CH0_MASK,
	 PMIC_AUXADC_ADC_RDY_CH0_SHIFT},
	{PMIC_AUXADC_ADC_OUT_CH1, PMIC_AUXADC_ADC_OUT_CH1_ADDR, PMIC_AUXADC_ADC_OUT_CH1_MASK,
	 PMIC_AUXADC_ADC_OUT_CH1_SHIFT},
	{PMIC_AUXADC_ADC_RDY_CH1, PMIC_AUXADC_ADC_RDY_CH1_ADDR, PMIC_AUXADC_ADC_RDY_CH1_MASK,
	 PMIC_AUXADC_ADC_RDY_CH1_SHIFT},
	{PMIC_AUXADC_ADC_OUT_CH2, PMIC_AUXADC_ADC_OUT_CH2_ADDR, PMIC_AUXADC_ADC_OUT_CH2_MASK,
	 PMIC_AUXADC_ADC_OUT_CH2_SHIFT},
	{PMIC_AUXADC_ADC_RDY_CH2, PMIC_AUXADC_ADC_RDY_CH2_ADDR, PMIC_AUXADC_ADC_RDY_CH2_MASK,
	 PMIC_AUXADC_ADC_RDY_CH2_SHIFT},
	{PMIC_AUXADC_ADC_OUT_CH3, PMIC_AUXADC_ADC_OUT_CH3_ADDR, PMIC_AUXADC_ADC_OUT_CH3_MASK,
	 PMIC_AUXADC_ADC_OUT_CH3_SHIFT},
	{PMIC_AUXADC_ADC_RDY_CH3, PMIC_AUXADC_ADC_RDY_CH3_ADDR, PMIC_AUXADC_ADC_RDY_CH3_MASK,
	 PMIC_AUXADC_ADC_RDY_CH3_SHIFT},
	{PMIC_AUXADC_ADC_OUT_CH4, PMIC_AUXADC_ADC_OUT_CH4_ADDR, PMIC_AUXADC_ADC_OUT_CH4_MASK,
	 PMIC_AUXADC_ADC_OUT_CH4_SHIFT},
	{PMIC_AUXADC_ADC_RDY_CH4, PMIC_AUXADC_ADC_RDY_CH4_ADDR, PMIC_AUXADC_ADC_RDY_CH4_MASK,
	 PMIC_AUXADC_ADC_RDY_CH4_SHIFT},
	{PMIC_AUXADC_ADC_OUT_CH5, PMIC_AUXADC_ADC_OUT_CH5_ADDR, PMIC_AUXADC_ADC_OUT_CH5_MASK,
	 PMIC_AUXADC_ADC_OUT_CH5_SHIFT},
	{PMIC_AUXADC_ADC_RDY_CH5, PMIC_AUXADC_ADC_RDY_CH5_ADDR, PMIC_AUXADC_ADC_RDY_CH5_MASK,
	 PMIC_AUXADC_ADC_RDY_CH5_SHIFT},
	{PMIC_AUXADC_ADC_OUT_CH6, PMIC_AUXADC_ADC_OUT_CH6_ADDR, PMIC_AUXADC_ADC_OUT_CH6_MASK,
	 PMIC_AUXADC_ADC_OUT_CH6_SHIFT},
	{PMIC_AUXADC_ADC_RDY_CH6, PMIC_AUXADC_ADC_RDY_CH6_ADDR, PMIC_AUXADC_ADC_RDY_CH6_MASK,
	 PMIC_AUXADC_ADC_RDY_CH6_SHIFT},
	{PMIC_AUXADC_ADC_OUT_CH7, PMIC_AUXADC_ADC_OUT_CH7_ADDR, PMIC_AUXADC_ADC_OUT_CH7_MASK,
	 PMIC_AUXADC_ADC_OUT_CH7_SHIFT},
	{PMIC_AUXADC_ADC_RDY_CH7, PMIC_AUXADC_ADC_RDY_CH7_ADDR, PMIC_AUXADC_ADC_RDY_CH7_MASK,
	 PMIC_AUXADC_ADC_RDY_CH7_SHIFT},
	{PMIC_AUXADC_ADC_OUT_CH8, PMIC_AUXADC_ADC_OUT_CH8_ADDR, PMIC_AUXADC_ADC_OUT_CH8_MASK,
	 PMIC_AUXADC_ADC_OUT_CH8_SHIFT},
	{PMIC_AUXADC_ADC_RDY_CH8, PMIC_AUXADC_ADC_RDY_CH8_ADDR, PMIC_AUXADC_ADC_RDY_CH8_MASK,
	 PMIC_AUXADC_ADC_RDY_CH8_SHIFT},
	{PMIC_AUXADC_ADC_OUT_CH9, PMIC_AUXADC_ADC_OUT_CH9_ADDR, PMIC_AUXADC_ADC_OUT_CH9_MASK,
	 PMIC_AUXADC_ADC_OUT_CH9_SHIFT},
	{PMIC_AUXADC_ADC_RDY_CH9, PMIC_AUXADC_ADC_RDY_CH9_ADDR, PMIC_AUXADC_ADC_RDY_CH9_MASK,
	 PMIC_AUXADC_ADC_RDY_CH9_SHIFT},
	{PMIC_AUXADC_ADC_OUT_CH10, PMIC_AUXADC_ADC_OUT_CH10_ADDR, PMIC_AUXADC_ADC_OUT_CH10_MASK,
	 PMIC_AUXADC_ADC_OUT_CH10_SHIFT},
	{PMIC_AUXADC_ADC_RDY_CH10, PMIC_AUXADC_ADC_RDY_CH10_ADDR, PMIC_AUXADC_ADC_RDY_CH10_MASK,
	 PMIC_AUXADC_ADC_RDY_CH10_SHIFT},
	{PMIC_AUXADC_ADC_OUT_CH11, PMIC_AUXADC_ADC_OUT_CH11_ADDR, PMIC_AUXADC_ADC_OUT_CH11_MASK,
	 PMIC_AUXADC_ADC_OUT_CH11_SHIFT},
	{PMIC_AUXADC_ADC_RDY_CH11, PMIC_AUXADC_ADC_RDY_CH11_ADDR, PMIC_AUXADC_ADC_RDY_CH11_MASK,
	 PMIC_AUXADC_ADC_RDY_CH11_SHIFT},
	{PMIC_AUXADC_ADC_OUT_CH12_15, PMIC_AUXADC_ADC_OUT_CH12_15_ADDR,
	 PMIC_AUXADC_ADC_OUT_CH12_15_MASK, PMIC_AUXADC_ADC_OUT_CH12_15_SHIFT},
	{PMIC_AUXADC_ADC_RDY_CH12_15, PMIC_AUXADC_ADC_RDY_CH12_15_ADDR,
	 PMIC_AUXADC_ADC_RDY_CH12_15_MASK, PMIC_AUXADC_ADC_RDY_CH12_15_SHIFT},
	{PMIC_AUXADC_ADC_OUT_THR_HW, PMIC_AUXADC_ADC_OUT_THR_HW_ADDR,
	 PMIC_AUXADC_ADC_OUT_THR_HW_MASK, PMIC_AUXADC_ADC_OUT_THR_HW_SHIFT},
	{PMIC_AUXADC_ADC_RDY_THR_HW, PMIC_AUXADC_ADC_RDY_THR_HW_ADDR,
	 PMIC_AUXADC_ADC_RDY_THR_HW_MASK, PMIC_AUXADC_ADC_RDY_THR_HW_SHIFT},
	{PMIC_AUXADC_ADC_OUT_LBAT, PMIC_AUXADC_ADC_OUT_LBAT_ADDR, PMIC_AUXADC_ADC_OUT_LBAT_MASK,
	 PMIC_AUXADC_ADC_OUT_LBAT_SHIFT},
	{PMIC_AUXADC_ADC_RDY_LBAT, PMIC_AUXADC_ADC_RDY_LBAT_ADDR, PMIC_AUXADC_ADC_RDY_LBAT_MASK,
	 PMIC_AUXADC_ADC_RDY_LBAT_SHIFT},
	{PMIC_AUXADC_ADC_OUT_LBAT2, PMIC_AUXADC_ADC_OUT_LBAT2_ADDR, PMIC_AUXADC_ADC_OUT_LBAT2_MASK,
	 PMIC_AUXADC_ADC_OUT_LBAT2_SHIFT},
	{PMIC_AUXADC_ADC_RDY_LBAT2, PMIC_AUXADC_ADC_RDY_LBAT2_ADDR, PMIC_AUXADC_ADC_RDY_LBAT2_MASK,
	 PMIC_AUXADC_ADC_RDY_LBAT2_SHIFT},
	{PMIC_AUXADC_ADC_OUT_CH7_BY_GPS, PMIC_AUXADC_ADC_OUT_CH7_BY_GPS_ADDR,
	 PMIC_AUXADC_ADC_OUT_CH7_BY_GPS_MASK, PMIC_AUXADC_ADC_OUT_CH7_BY_GPS_SHIFT},
	{PMIC_AUXADC_ADC_RDY_CH7_BY_GPS, PMIC_AUXADC_ADC_RDY_CH7_BY_GPS_ADDR,
	 PMIC_AUXADC_ADC_RDY_CH7_BY_GPS_MASK, PMIC_AUXADC_ADC_RDY_CH7_BY_GPS_SHIFT},
	{PMIC_AUXADC_ADC_OUT_CH7_BY_MD, PMIC_AUXADC_ADC_OUT_CH7_BY_MD_ADDR,
	 PMIC_AUXADC_ADC_OUT_CH7_BY_MD_MASK, PMIC_AUXADC_ADC_OUT_CH7_BY_MD_SHIFT},
	{PMIC_AUXADC_ADC_RDY_CH7_BY_MD, PMIC_AUXADC_ADC_RDY_CH7_BY_MD_ADDR,
	 PMIC_AUXADC_ADC_RDY_CH7_BY_MD_MASK, PMIC_AUXADC_ADC_RDY_CH7_BY_MD_SHIFT},
	{PMIC_AUXADC_ADC_OUT_CH7_BY_AP, PMIC_AUXADC_ADC_OUT_CH7_BY_AP_ADDR,
	 PMIC_AUXADC_ADC_OUT_CH7_BY_AP_MASK, PMIC_AUXADC_ADC_OUT_CH7_BY_AP_SHIFT},
	{PMIC_AUXADC_ADC_RDY_CH7_BY_AP, PMIC_AUXADC_ADC_RDY_CH7_BY_AP_ADDR,
	 PMIC_AUXADC_ADC_RDY_CH7_BY_AP_MASK, PMIC_AUXADC_ADC_RDY_CH7_BY_AP_SHIFT},
	{PMIC_AUXADC_ADC_OUT_CH4_BY_MD, PMIC_AUXADC_ADC_OUT_CH4_BY_MD_ADDR,
	 PMIC_AUXADC_ADC_OUT_CH4_BY_MD_MASK, PMIC_AUXADC_ADC_OUT_CH4_BY_MD_SHIFT},
	{PMIC_AUXADC_ADC_RDY_CH4_BY_MD, PMIC_AUXADC_ADC_RDY_CH4_BY_MD_ADDR,
	 PMIC_AUXADC_ADC_RDY_CH4_BY_MD_MASK, PMIC_AUXADC_ADC_RDY_CH4_BY_MD_SHIFT},
	{PMIC_AUXADC_ADC_OUT_WAKEUP_PCHR, PMIC_AUXADC_ADC_OUT_WAKEUP_PCHR_ADDR,
	 PMIC_AUXADC_ADC_OUT_WAKEUP_PCHR_MASK, PMIC_AUXADC_ADC_OUT_WAKEUP_PCHR_SHIFT},
	{PMIC_AUXADC_ADC_RDY_WAKEUP_PCHR, PMIC_AUXADC_ADC_RDY_WAKEUP_PCHR_ADDR,
	 PMIC_AUXADC_ADC_RDY_WAKEUP_PCHR_MASK, PMIC_AUXADC_ADC_RDY_WAKEUP_PCHR_SHIFT},
	{PMIC_AUXADC_ADC_OUT_WAKEUP_SWCHR, PMIC_AUXADC_ADC_OUT_WAKEUP_SWCHR_ADDR,
	 PMIC_AUXADC_ADC_OUT_WAKEUP_SWCHR_MASK, PMIC_AUXADC_ADC_OUT_WAKEUP_SWCHR_SHIFT},
	{PMIC_AUXADC_ADC_RDY_WAKEUP_SWCHR, PMIC_AUXADC_ADC_RDY_WAKEUP_SWCHR_ADDR,
	 PMIC_AUXADC_ADC_RDY_WAKEUP_SWCHR_MASK, PMIC_AUXADC_ADC_RDY_WAKEUP_SWCHR_SHIFT},
	{PMIC_AUXADC_ADC_OUT_CH0_BY_MD, PMIC_AUXADC_ADC_OUT_CH0_BY_MD_ADDR,
	 PMIC_AUXADC_ADC_OUT_CH0_BY_MD_MASK, PMIC_AUXADC_ADC_OUT_CH0_BY_MD_SHIFT},
	{PMIC_AUXADC_ADC_RDY_CH0_BY_MD, PMIC_AUXADC_ADC_RDY_CH0_BY_MD_ADDR,
	 PMIC_AUXADC_ADC_RDY_CH0_BY_MD_MASK, PMIC_AUXADC_ADC_RDY_CH0_BY_MD_SHIFT},
	{PMIC_AUXADC_ADC_OUT_CH0_BY_AP, PMIC_AUXADC_ADC_OUT_CH0_BY_AP_ADDR,
	 PMIC_AUXADC_ADC_OUT_CH0_BY_AP_MASK, PMIC_AUXADC_ADC_OUT_CH0_BY_AP_SHIFT},
	{PMIC_AUXADC_ADC_RDY_CH0_BY_AP, PMIC_AUXADC_ADC_RDY_CH0_BY_AP_ADDR,
	 PMIC_AUXADC_ADC_RDY_CH0_BY_AP_MASK, PMIC_AUXADC_ADC_RDY_CH0_BY_AP_SHIFT},
	{PMIC_AUXADC_ADC_OUT_CH1_BY_MD, PMIC_AUXADC_ADC_OUT_CH1_BY_MD_ADDR,
	 PMIC_AUXADC_ADC_OUT_CH1_BY_MD_MASK, PMIC_AUXADC_ADC_OUT_CH1_BY_MD_SHIFT},
	{PMIC_AUXADC_ADC_RDY_CH1_BY_MD, PMIC_AUXADC_ADC_RDY_CH1_BY_MD_ADDR,
	 PMIC_AUXADC_ADC_RDY_CH1_BY_MD_MASK, PMIC_AUXADC_ADC_RDY_CH1_BY_MD_SHIFT},
	{PMIC_AUXADC_ADC_OUT_CH1_BY_AP, PMIC_AUXADC_ADC_OUT_CH1_BY_AP_ADDR,
	 PMIC_AUXADC_ADC_OUT_CH1_BY_AP_MASK, PMIC_AUXADC_ADC_OUT_CH1_BY_AP_SHIFT},
	{PMIC_AUXADC_ADC_RDY_CH1_BY_AP, PMIC_AUXADC_ADC_RDY_CH1_BY_AP_ADDR,
	 PMIC_AUXADC_ADC_RDY_CH1_BY_AP_MASK, PMIC_AUXADC_ADC_RDY_CH1_BY_AP_SHIFT},
	{PMIC_AUXADC_ADC_OUT_BAT_TEMP, PMIC_AUXADC_ADC_OUT_BAT_TEMP_ADDR,
	 PMIC_AUXADC_ADC_OUT_BAT_TEMP_MASK, PMIC_AUXADC_ADC_OUT_BAT_TEMP_SHIFT},
	{PMIC_AUXADC_ADC_RDY_BAT_TEMP, PMIC_AUXADC_ADC_RDY_BAT_TEMP_ADDR,
	 PMIC_AUXADC_ADC_RDY_BAT_TEMP_MASK, PMIC_AUXADC_ADC_RDY_BAT_TEMP_SHIFT},
	{PMIC_AUXADC_ADC_OUT_FGADC1, PMIC_AUXADC_ADC_OUT_FGADC1_ADDR,
	 PMIC_AUXADC_ADC_OUT_FGADC1_MASK, PMIC_AUXADC_ADC_OUT_FGADC1_SHIFT},
	{PMIC_AUXADC_ADC_RDY_FGADC1, PMIC_AUXADC_ADC_RDY_FGADC1_ADDR,
	 PMIC_AUXADC_ADC_RDY_FGADC1_MASK, PMIC_AUXADC_ADC_RDY_FGADC1_SHIFT},
	{PMIC_AUXADC_ADC_OUT_FGADC2, PMIC_AUXADC_ADC_OUT_FGADC2_ADDR,
	 PMIC_AUXADC_ADC_OUT_FGADC2_MASK, PMIC_AUXADC_ADC_OUT_FGADC2_SHIFT},
	{PMIC_AUXADC_ADC_RDY_FGADC2, PMIC_AUXADC_ADC_RDY_FGADC2_ADDR,
	 PMIC_AUXADC_ADC_RDY_FGADC2_MASK, PMIC_AUXADC_ADC_RDY_FGADC2_SHIFT},
	{PMIC_AUXADC_ADC_OUT_BAT_PLUGIN1, PMIC_AUXADC_ADC_OUT_BAT_PLUGIN1_ADDR,
	 PMIC_AUXADC_ADC_OUT_BAT_PLUGIN1_MASK, PMIC_AUXADC_ADC_OUT_BAT_PLUGIN1_SHIFT},
	{PMIC_AUXADC_ADC_RDY_BAT_PLUGIN1, PMIC_AUXADC_ADC_RDY_BAT_PLUGIN1_ADDR,
	 PMIC_AUXADC_ADC_RDY_BAT_PLUGIN1_MASK, PMIC_AUXADC_ADC_RDY_BAT_PLUGIN1_SHIFT},
	{PMIC_AUXADC_ADC_OUT_BAT_PLUGIN2, PMIC_AUXADC_ADC_OUT_BAT_PLUGIN2_ADDR,
	 PMIC_AUXADC_ADC_OUT_BAT_PLUGIN2_MASK, PMIC_AUXADC_ADC_OUT_BAT_PLUGIN2_SHIFT},
	{PMIC_AUXADC_ADC_RDY_BAT_PLUGIN2, PMIC_AUXADC_ADC_RDY_BAT_PLUGIN2_ADDR,
	 PMIC_AUXADC_ADC_RDY_BAT_PLUGIN2_MASK, PMIC_AUXADC_ADC_RDY_BAT_PLUGIN2_SHIFT},
	{PMIC_AUXADC_ADC_OUT_IMP, PMIC_AUXADC_ADC_OUT_IMP_ADDR, PMIC_AUXADC_ADC_OUT_IMP_MASK,
	 PMIC_AUXADC_ADC_OUT_IMP_SHIFT},
	{PMIC_AUXADC_ADC_RDY_IMP, PMIC_AUXADC_ADC_RDY_IMP_ADDR, PMIC_AUXADC_ADC_RDY_IMP_MASK,
	 PMIC_AUXADC_ADC_RDY_IMP_SHIFT},
	{PMIC_AUXADC_ADC_OUT_IMP_AVG, PMIC_AUXADC_ADC_OUT_IMP_AVG_ADDR,
	 PMIC_AUXADC_ADC_OUT_IMP_AVG_MASK, PMIC_AUXADC_ADC_OUT_IMP_AVG_SHIFT},
	{PMIC_AUXADC_ADC_RDY_IMP_AVG, PMIC_AUXADC_ADC_RDY_IMP_AVG_ADDR,
	 PMIC_AUXADC_ADC_RDY_IMP_AVG_MASK, PMIC_AUXADC_ADC_RDY_IMP_AVG_SHIFT},
	{PMIC_AUXADC_ADC_OUT_RAW, PMIC_AUXADC_ADC_OUT_RAW_ADDR, PMIC_AUXADC_ADC_OUT_RAW_MASK,
	 PMIC_AUXADC_ADC_OUT_RAW_SHIFT},
	{PMIC_AUXADC_ADC_OUT_MDRT, PMIC_AUXADC_ADC_OUT_MDRT_ADDR, PMIC_AUXADC_ADC_OUT_MDRT_MASK,
	 PMIC_AUXADC_ADC_OUT_MDRT_SHIFT},
	{PMIC_AUXADC_ADC_RDY_MDRT, PMIC_AUXADC_ADC_RDY_MDRT_ADDR, PMIC_AUXADC_ADC_RDY_MDRT_MASK,
	 PMIC_AUXADC_ADC_RDY_MDRT_SHIFT},
	{PMIC_AUXADC_ADC_OUT_MDBG, PMIC_AUXADC_ADC_OUT_MDBG_ADDR, PMIC_AUXADC_ADC_OUT_MDBG_MASK,
	 PMIC_AUXADC_ADC_OUT_MDBG_SHIFT},
	{PMIC_AUXADC_ADC_RDY_MDBG, PMIC_AUXADC_ADC_RDY_MDBG_ADDR, PMIC_AUXADC_ADC_RDY_MDBG_MASK,
	 PMIC_AUXADC_ADC_RDY_MDBG_SHIFT},
	{PMIC_AUXADC_ADC_OUT_JEITA, PMIC_AUXADC_ADC_OUT_JEITA_ADDR, PMIC_AUXADC_ADC_OUT_JEITA_MASK,
	 PMIC_AUXADC_ADC_OUT_JEITA_SHIFT},
	{PMIC_AUXADC_ADC_RDY_JEITA, PMIC_AUXADC_ADC_RDY_JEITA_ADDR, PMIC_AUXADC_ADC_RDY_JEITA_MASK,
	 PMIC_AUXADC_ADC_RDY_JEITA_SHIFT},
	{PMIC_AUXADC_ADC_OUT_DCXO_BY_GPS, PMIC_AUXADC_ADC_OUT_DCXO_BY_GPS_ADDR,
	 PMIC_AUXADC_ADC_OUT_DCXO_BY_GPS_MASK, PMIC_AUXADC_ADC_OUT_DCXO_BY_GPS_SHIFT},
	{PMIC_AUXADC_ADC_RDY_DCXO_BY_GPS, PMIC_AUXADC_ADC_RDY_DCXO_BY_GPS_ADDR,
	 PMIC_AUXADC_ADC_RDY_DCXO_BY_GPS_MASK, PMIC_AUXADC_ADC_RDY_DCXO_BY_GPS_SHIFT},
	{PMIC_AUXADC_ADC_OUT_DCXO_BY_MD, PMIC_AUXADC_ADC_OUT_DCXO_BY_MD_ADDR,
	 PMIC_AUXADC_ADC_OUT_DCXO_BY_MD_MASK, PMIC_AUXADC_ADC_OUT_DCXO_BY_MD_SHIFT},
	{PMIC_AUXADC_ADC_RDY_DCXO_BY_MD, PMIC_AUXADC_ADC_RDY_DCXO_BY_MD_ADDR,
	 PMIC_AUXADC_ADC_RDY_DCXO_BY_MD_MASK, PMIC_AUXADC_ADC_RDY_DCXO_BY_MD_SHIFT},
	{PMIC_AUXADC_ADC_OUT_DCXO_BY_AP, PMIC_AUXADC_ADC_OUT_DCXO_BY_AP_ADDR,
	 PMIC_AUXADC_ADC_OUT_DCXO_BY_AP_MASK, PMIC_AUXADC_ADC_OUT_DCXO_BY_AP_SHIFT},
	{PMIC_AUXADC_ADC_RDY_DCXO_BY_AP, PMIC_AUXADC_ADC_RDY_DCXO_BY_AP_ADDR,
	 PMIC_AUXADC_ADC_RDY_DCXO_BY_AP_MASK, PMIC_AUXADC_ADC_RDY_DCXO_BY_AP_SHIFT},
	{PMIC_AUXADC_ADC_OUT_DCXO_MDRT, PMIC_AUXADC_ADC_OUT_DCXO_MDRT_ADDR,
	 PMIC_AUXADC_ADC_OUT_DCXO_MDRT_MASK, PMIC_AUXADC_ADC_OUT_DCXO_MDRT_SHIFT},
	{PMIC_AUXADC_ADC_RDY_DCXO_MDRT, PMIC_AUXADC_ADC_RDY_DCXO_MDRT_ADDR,
	 PMIC_AUXADC_ADC_RDY_DCXO_MDRT_MASK, PMIC_AUXADC_ADC_RDY_DCXO_MDRT_SHIFT},
	{PMIC_AUXADC_ADC_OUT_NAG, PMIC_AUXADC_ADC_OUT_NAG_ADDR, PMIC_AUXADC_ADC_OUT_NAG_MASK,
	 PMIC_AUXADC_ADC_OUT_NAG_SHIFT},
	{PMIC_AUXADC_ADC_RDY_NAG, PMIC_AUXADC_ADC_RDY_NAG_ADDR, PMIC_AUXADC_ADC_RDY_NAG_MASK,
	 PMIC_AUXADC_ADC_RDY_NAG_SHIFT},
	{PMIC_AUXADC_ADC_OUT_BATID, PMIC_AUXADC_ADC_OUT_BATID_ADDR, PMIC_AUXADC_ADC_OUT_BATID_MASK,
	 PMIC_AUXADC_ADC_OUT_BATID_SHIFT},
	{PMIC_AUXADC_ADC_RDY_BATID, PMIC_AUXADC_ADC_RDY_BATID_ADDR, PMIC_AUXADC_ADC_RDY_BATID_MASK,
	 PMIC_AUXADC_ADC_RDY_BATID_SHIFT},
	{PMIC_AUXADC_BUF_OUT_00, PMIC_AUXADC_BUF_OUT_00_ADDR, PMIC_AUXADC_BUF_OUT_00_MASK,
	 PMIC_AUXADC_BUF_OUT_00_SHIFT},
	{PMIC_AUXADC_BUF_RDY_00, PMIC_AUXADC_BUF_RDY_00_ADDR, PMIC_AUXADC_BUF_RDY_00_MASK,
	 PMIC_AUXADC_BUF_RDY_00_SHIFT},
	{PMIC_AUXADC_BUF_OUT_01, PMIC_AUXADC_BUF_OUT_01_ADDR, PMIC_AUXADC_BUF_OUT_01_MASK,
	 PMIC_AUXADC_BUF_OUT_01_SHIFT},
	{PMIC_AUXADC_BUF_RDY_01, PMIC_AUXADC_BUF_RDY_01_ADDR, PMIC_AUXADC_BUF_RDY_01_MASK,
	 PMIC_AUXADC_BUF_RDY_01_SHIFT},
	{PMIC_AUXADC_BUF_OUT_02, PMIC_AUXADC_BUF_OUT_02_ADDR, PMIC_AUXADC_BUF_OUT_02_MASK,
	 PMIC_AUXADC_BUF_OUT_02_SHIFT},
	{PMIC_AUXADC_BUF_RDY_02, PMIC_AUXADC_BUF_RDY_02_ADDR, PMIC_AUXADC_BUF_RDY_02_MASK,
	 PMIC_AUXADC_BUF_RDY_02_SHIFT},
	{PMIC_AUXADC_BUF_OUT_03, PMIC_AUXADC_BUF_OUT_03_ADDR, PMIC_AUXADC_BUF_OUT_03_MASK,
	 PMIC_AUXADC_BUF_OUT_03_SHIFT},
	{PMIC_AUXADC_BUF_RDY_03, PMIC_AUXADC_BUF_RDY_03_ADDR, PMIC_AUXADC_BUF_RDY_03_MASK,
	 PMIC_AUXADC_BUF_RDY_03_SHIFT},
	{PMIC_AUXADC_BUF_OUT_04, PMIC_AUXADC_BUF_OUT_04_ADDR, PMIC_AUXADC_BUF_OUT_04_MASK,
	 PMIC_AUXADC_BUF_OUT_04_SHIFT},
	{PMIC_AUXADC_BUF_RDY_04, PMIC_AUXADC_BUF_RDY_04_ADDR, PMIC_AUXADC_BUF_RDY_04_MASK,
	 PMIC_AUXADC_BUF_RDY_04_SHIFT},
	{PMIC_AUXADC_BUF_OUT_05, PMIC_AUXADC_BUF_OUT_05_ADDR, PMIC_AUXADC_BUF_OUT_05_MASK,
	 PMIC_AUXADC_BUF_OUT_05_SHIFT},
	{PMIC_AUXADC_BUF_RDY_05, PMIC_AUXADC_BUF_RDY_05_ADDR, PMIC_AUXADC_BUF_RDY_05_MASK,
	 PMIC_AUXADC_BUF_RDY_05_SHIFT},
	{PMIC_AUXADC_BUF_OUT_06, PMIC_AUXADC_BUF_OUT_06_ADDR, PMIC_AUXADC_BUF_OUT_06_MASK,
	 PMIC_AUXADC_BUF_OUT_06_SHIFT},
	{PMIC_AUXADC_BUF_RDY_06, PMIC_AUXADC_BUF_RDY_06_ADDR, PMIC_AUXADC_BUF_RDY_06_MASK,
	 PMIC_AUXADC_BUF_RDY_06_SHIFT},
	{PMIC_AUXADC_BUF_OUT_07, PMIC_AUXADC_BUF_OUT_07_ADDR, PMIC_AUXADC_BUF_OUT_07_MASK,
	 PMIC_AUXADC_BUF_OUT_07_SHIFT},
	{PMIC_AUXADC_BUF_RDY_07, PMIC_AUXADC_BUF_RDY_07_ADDR, PMIC_AUXADC_BUF_RDY_07_MASK,
	 PMIC_AUXADC_BUF_RDY_07_SHIFT},
	{PMIC_AUXADC_BUF_OUT_08, PMIC_AUXADC_BUF_OUT_08_ADDR, PMIC_AUXADC_BUF_OUT_08_MASK,
	 PMIC_AUXADC_BUF_OUT_08_SHIFT},
	{PMIC_AUXADC_BUF_RDY_08, PMIC_AUXADC_BUF_RDY_08_ADDR, PMIC_AUXADC_BUF_RDY_08_MASK,
	 PMIC_AUXADC_BUF_RDY_08_SHIFT},
	{PMIC_AUXADC_BUF_OUT_09, PMIC_AUXADC_BUF_OUT_09_ADDR, PMIC_AUXADC_BUF_OUT_09_MASK,
	 PMIC_AUXADC_BUF_OUT_09_SHIFT},
	{PMIC_AUXADC_BUF_RDY_09, PMIC_AUXADC_BUF_RDY_09_ADDR, PMIC_AUXADC_BUF_RDY_09_MASK,
	 PMIC_AUXADC_BUF_RDY_09_SHIFT},
	{PMIC_AUXADC_BUF_OUT_10, PMIC_AUXADC_BUF_OUT_10_ADDR, PMIC_AUXADC_BUF_OUT_10_MASK,
	 PMIC_AUXADC_BUF_OUT_10_SHIFT},
	{PMIC_AUXADC_BUF_RDY_10, PMIC_AUXADC_BUF_RDY_10_ADDR, PMIC_AUXADC_BUF_RDY_10_MASK,
	 PMIC_AUXADC_BUF_RDY_10_SHIFT},
	{PMIC_AUXADC_BUF_OUT_11, PMIC_AUXADC_BUF_OUT_11_ADDR, PMIC_AUXADC_BUF_OUT_11_MASK,
	 PMIC_AUXADC_BUF_OUT_11_SHIFT},
	{PMIC_AUXADC_BUF_RDY_11, PMIC_AUXADC_BUF_RDY_11_ADDR, PMIC_AUXADC_BUF_RDY_11_MASK,
	 PMIC_AUXADC_BUF_RDY_11_SHIFT},
	{PMIC_AUXADC_BUF_OUT_12, PMIC_AUXADC_BUF_OUT_12_ADDR, PMIC_AUXADC_BUF_OUT_12_MASK,
	 PMIC_AUXADC_BUF_OUT_12_SHIFT},
	{PMIC_AUXADC_BUF_RDY_12, PMIC_AUXADC_BUF_RDY_12_ADDR, PMIC_AUXADC_BUF_RDY_12_MASK,
	 PMIC_AUXADC_BUF_RDY_12_SHIFT},
	{PMIC_AUXADC_BUF_OUT_13, PMIC_AUXADC_BUF_OUT_13_ADDR, PMIC_AUXADC_BUF_OUT_13_MASK,
	 PMIC_AUXADC_BUF_OUT_13_SHIFT},
	{PMIC_AUXADC_BUF_RDY_13, PMIC_AUXADC_BUF_RDY_13_ADDR, PMIC_AUXADC_BUF_RDY_13_MASK,
	 PMIC_AUXADC_BUF_RDY_13_SHIFT},
	{PMIC_AUXADC_BUF_OUT_14, PMIC_AUXADC_BUF_OUT_14_ADDR, PMIC_AUXADC_BUF_OUT_14_MASK,
	 PMIC_AUXADC_BUF_OUT_14_SHIFT},
	{PMIC_AUXADC_BUF_RDY_14, PMIC_AUXADC_BUF_RDY_14_ADDR, PMIC_AUXADC_BUF_RDY_14_MASK,
	 PMIC_AUXADC_BUF_RDY_14_SHIFT},
	{PMIC_AUXADC_BUF_OUT_15, PMIC_AUXADC_BUF_OUT_15_ADDR, PMIC_AUXADC_BUF_OUT_15_MASK,
	 PMIC_AUXADC_BUF_OUT_15_SHIFT},
	{PMIC_AUXADC_BUF_RDY_15, PMIC_AUXADC_BUF_RDY_15_ADDR, PMIC_AUXADC_BUF_RDY_15_MASK,
	 PMIC_AUXADC_BUF_RDY_15_SHIFT},
	{PMIC_AUXADC_BUF_OUT_16, PMIC_AUXADC_BUF_OUT_16_ADDR, PMIC_AUXADC_BUF_OUT_16_MASK,
	 PMIC_AUXADC_BUF_OUT_16_SHIFT},
	{PMIC_AUXADC_BUF_RDY_16, PMIC_AUXADC_BUF_RDY_16_ADDR, PMIC_AUXADC_BUF_RDY_16_MASK,
	 PMIC_AUXADC_BUF_RDY_16_SHIFT},
	{PMIC_AUXADC_BUF_OUT_17, PMIC_AUXADC_BUF_OUT_17_ADDR, PMIC_AUXADC_BUF_OUT_17_MASK,
	 PMIC_AUXADC_BUF_OUT_17_SHIFT},
	{PMIC_AUXADC_BUF_RDY_17, PMIC_AUXADC_BUF_RDY_17_ADDR, PMIC_AUXADC_BUF_RDY_17_MASK,
	 PMIC_AUXADC_BUF_RDY_17_SHIFT},
	{PMIC_AUXADC_BUF_OUT_18, PMIC_AUXADC_BUF_OUT_18_ADDR, PMIC_AUXADC_BUF_OUT_18_MASK,
	 PMIC_AUXADC_BUF_OUT_18_SHIFT},
	{PMIC_AUXADC_BUF_RDY_18, PMIC_AUXADC_BUF_RDY_18_ADDR, PMIC_AUXADC_BUF_RDY_18_MASK,
	 PMIC_AUXADC_BUF_RDY_18_SHIFT},
	{PMIC_AUXADC_BUF_OUT_19, PMIC_AUXADC_BUF_OUT_19_ADDR, PMIC_AUXADC_BUF_OUT_19_MASK,
	 PMIC_AUXADC_BUF_OUT_19_SHIFT},
	{PMIC_AUXADC_BUF_RDY_19, PMIC_AUXADC_BUF_RDY_19_ADDR, PMIC_AUXADC_BUF_RDY_19_MASK,
	 PMIC_AUXADC_BUF_RDY_19_SHIFT},
	{PMIC_AUXADC_BUF_OUT_20, PMIC_AUXADC_BUF_OUT_20_ADDR, PMIC_AUXADC_BUF_OUT_20_MASK,
	 PMIC_AUXADC_BUF_OUT_20_SHIFT},
	{PMIC_AUXADC_BUF_RDY_20, PMIC_AUXADC_BUF_RDY_20_ADDR, PMIC_AUXADC_BUF_RDY_20_MASK,
	 PMIC_AUXADC_BUF_RDY_20_SHIFT},
	{PMIC_AUXADC_BUF_OUT_21, PMIC_AUXADC_BUF_OUT_21_ADDR, PMIC_AUXADC_BUF_OUT_21_MASK,
	 PMIC_AUXADC_BUF_OUT_21_SHIFT},
	{PMIC_AUXADC_BUF_RDY_21, PMIC_AUXADC_BUF_RDY_21_ADDR, PMIC_AUXADC_BUF_RDY_21_MASK,
	 PMIC_AUXADC_BUF_RDY_21_SHIFT},
	{PMIC_AUXADC_BUF_OUT_22, PMIC_AUXADC_BUF_OUT_22_ADDR, PMIC_AUXADC_BUF_OUT_22_MASK,
	 PMIC_AUXADC_BUF_OUT_22_SHIFT},
	{PMIC_AUXADC_BUF_RDY_22, PMIC_AUXADC_BUF_RDY_22_ADDR, PMIC_AUXADC_BUF_RDY_22_MASK,
	 PMIC_AUXADC_BUF_RDY_22_SHIFT},
	{PMIC_AUXADC_BUF_OUT_23, PMIC_AUXADC_BUF_OUT_23_ADDR, PMIC_AUXADC_BUF_OUT_23_MASK,
	 PMIC_AUXADC_BUF_OUT_23_SHIFT},
	{PMIC_AUXADC_BUF_RDY_23, PMIC_AUXADC_BUF_RDY_23_ADDR, PMIC_AUXADC_BUF_RDY_23_MASK,
	 PMIC_AUXADC_BUF_RDY_23_SHIFT},
	{PMIC_AUXADC_BUF_OUT_24, PMIC_AUXADC_BUF_OUT_24_ADDR, PMIC_AUXADC_BUF_OUT_24_MASK,
	 PMIC_AUXADC_BUF_OUT_24_SHIFT},
	{PMIC_AUXADC_BUF_RDY_24, PMIC_AUXADC_BUF_RDY_24_ADDR, PMIC_AUXADC_BUF_RDY_24_MASK,
	 PMIC_AUXADC_BUF_RDY_24_SHIFT},
	{PMIC_AUXADC_BUF_OUT_25, PMIC_AUXADC_BUF_OUT_25_ADDR, PMIC_AUXADC_BUF_OUT_25_MASK,
	 PMIC_AUXADC_BUF_OUT_25_SHIFT},
	{PMIC_AUXADC_BUF_RDY_25, PMIC_AUXADC_BUF_RDY_25_ADDR, PMIC_AUXADC_BUF_RDY_25_MASK,
	 PMIC_AUXADC_BUF_RDY_25_SHIFT},
	{PMIC_AUXADC_BUF_OUT_26, PMIC_AUXADC_BUF_OUT_26_ADDR, PMIC_AUXADC_BUF_OUT_26_MASK,
	 PMIC_AUXADC_BUF_OUT_26_SHIFT},
	{PMIC_AUXADC_BUF_RDY_26, PMIC_AUXADC_BUF_RDY_26_ADDR, PMIC_AUXADC_BUF_RDY_26_MASK,
	 PMIC_AUXADC_BUF_RDY_26_SHIFT},
	{PMIC_AUXADC_BUF_OUT_27, PMIC_AUXADC_BUF_OUT_27_ADDR, PMIC_AUXADC_BUF_OUT_27_MASK,
	 PMIC_AUXADC_BUF_OUT_27_SHIFT},
	{PMIC_AUXADC_BUF_RDY_27, PMIC_AUXADC_BUF_RDY_27_ADDR, PMIC_AUXADC_BUF_RDY_27_MASK,
	 PMIC_AUXADC_BUF_RDY_27_SHIFT},
	{PMIC_AUXADC_BUF_OUT_28, PMIC_AUXADC_BUF_OUT_28_ADDR, PMIC_AUXADC_BUF_OUT_28_MASK,
	 PMIC_AUXADC_BUF_OUT_28_SHIFT},
	{PMIC_AUXADC_BUF_RDY_28, PMIC_AUXADC_BUF_RDY_28_ADDR, PMIC_AUXADC_BUF_RDY_28_MASK,
	 PMIC_AUXADC_BUF_RDY_28_SHIFT},
	{PMIC_AUXADC_BUF_OUT_29, PMIC_AUXADC_BUF_OUT_29_ADDR, PMIC_AUXADC_BUF_OUT_29_MASK,
	 PMIC_AUXADC_BUF_OUT_29_SHIFT},
	{PMIC_AUXADC_BUF_RDY_29, PMIC_AUXADC_BUF_RDY_29_ADDR, PMIC_AUXADC_BUF_RDY_29_MASK,
	 PMIC_AUXADC_BUF_RDY_29_SHIFT},
	{PMIC_AUXADC_BUF_OUT_30, PMIC_AUXADC_BUF_OUT_30_ADDR, PMIC_AUXADC_BUF_OUT_30_MASK,
	 PMIC_AUXADC_BUF_OUT_30_SHIFT},
	{PMIC_AUXADC_BUF_RDY_30, PMIC_AUXADC_BUF_RDY_30_ADDR, PMIC_AUXADC_BUF_RDY_30_MASK,
	 PMIC_AUXADC_BUF_RDY_30_SHIFT},
	{PMIC_AUXADC_BUF_OUT_31, PMIC_AUXADC_BUF_OUT_31_ADDR, PMIC_AUXADC_BUF_OUT_31_MASK,
	 PMIC_AUXADC_BUF_OUT_31_SHIFT},
	{PMIC_AUXADC_BUF_RDY_31, PMIC_AUXADC_BUF_RDY_31_ADDR, PMIC_AUXADC_BUF_RDY_31_MASK,
	 PMIC_AUXADC_BUF_RDY_31_SHIFT},
	{PMIC_AUXADC_ADC_BUSY_IN, PMIC_AUXADC_ADC_BUSY_IN_ADDR, PMIC_AUXADC_ADC_BUSY_IN_MASK,
	 PMIC_AUXADC_ADC_BUSY_IN_SHIFT},
	{PMIC_AUXADC_ADC_BUSY_IN_LBAT, PMIC_AUXADC_ADC_BUSY_IN_LBAT_ADDR,
	 PMIC_AUXADC_ADC_BUSY_IN_LBAT_MASK, PMIC_AUXADC_ADC_BUSY_IN_LBAT_SHIFT},
	{PMIC_AUXADC_ADC_BUSY_IN_LBAT2, PMIC_AUXADC_ADC_BUSY_IN_LBAT2_ADDR,
	 PMIC_AUXADC_ADC_BUSY_IN_LBAT2_MASK, PMIC_AUXADC_ADC_BUSY_IN_LBAT2_SHIFT},
	{PMIC_AUXADC_ADC_BUSY_IN_BAT_TEMP, PMIC_AUXADC_ADC_BUSY_IN_BAT_TEMP_ADDR,
	 PMIC_AUXADC_ADC_BUSY_IN_BAT_TEMP_MASK, PMIC_AUXADC_ADC_BUSY_IN_BAT_TEMP_SHIFT},
	{PMIC_AUXADC_ADC_BUSY_IN_WAKEUP, PMIC_AUXADC_ADC_BUSY_IN_WAKEUP_ADDR,
	 PMIC_AUXADC_ADC_BUSY_IN_WAKEUP_MASK, PMIC_AUXADC_ADC_BUSY_IN_WAKEUP_SHIFT},
	{PMIC_AUXADC_ADC_BUSY_IN_DCXO_MDRT, PMIC_AUXADC_ADC_BUSY_IN_DCXO_MDRT_ADDR,
	 PMIC_AUXADC_ADC_BUSY_IN_DCXO_MDRT_MASK, PMIC_AUXADC_ADC_BUSY_IN_DCXO_MDRT_SHIFT},
	{PMIC_AUXADC_ADC_BUSY_IN_DCXO_GPS_AP, PMIC_AUXADC_ADC_BUSY_IN_DCXO_GPS_AP_ADDR,
	 PMIC_AUXADC_ADC_BUSY_IN_DCXO_GPS_AP_MASK, PMIC_AUXADC_ADC_BUSY_IN_DCXO_GPS_AP_SHIFT},
	{PMIC_AUXADC_ADC_BUSY_IN_DCXO_GPS_MD, PMIC_AUXADC_ADC_BUSY_IN_DCXO_GPS_MD_ADDR,
	 PMIC_AUXADC_ADC_BUSY_IN_DCXO_GPS_MD_MASK, PMIC_AUXADC_ADC_BUSY_IN_DCXO_GPS_MD_SHIFT},
	{PMIC_AUXADC_ADC_BUSY_IN_DCXO_GPS, PMIC_AUXADC_ADC_BUSY_IN_DCXO_GPS_ADDR,
	 PMIC_AUXADC_ADC_BUSY_IN_DCXO_GPS_MASK, PMIC_AUXADC_ADC_BUSY_IN_DCXO_GPS_SHIFT},
	{PMIC_AUXADC_ADC_BUSY_IN_JEITA, PMIC_AUXADC_ADC_BUSY_IN_JEITA_ADDR,
	 PMIC_AUXADC_ADC_BUSY_IN_JEITA_MASK, PMIC_AUXADC_ADC_BUSY_IN_JEITA_SHIFT},
	{PMIC_AUXADC_ADC_BUSY_IN_MDRT, PMIC_AUXADC_ADC_BUSY_IN_MDRT_ADDR,
	 PMIC_AUXADC_ADC_BUSY_IN_MDRT_MASK, PMIC_AUXADC_ADC_BUSY_IN_MDRT_SHIFT},
	{PMIC_AUXADC_ADC_BUSY_IN_MDBG, PMIC_AUXADC_ADC_BUSY_IN_MDBG_ADDR,
	 PMIC_AUXADC_ADC_BUSY_IN_MDBG_MASK, PMIC_AUXADC_ADC_BUSY_IN_MDBG_SHIFT},
	{PMIC_AUXADC_ADC_BUSY_IN_SHARE, PMIC_AUXADC_ADC_BUSY_IN_SHARE_ADDR,
	 PMIC_AUXADC_ADC_BUSY_IN_SHARE_MASK, PMIC_AUXADC_ADC_BUSY_IN_SHARE_SHIFT},
	{PMIC_AUXADC_ADC_BUSY_IN_IMP, PMIC_AUXADC_ADC_BUSY_IN_IMP_ADDR,
	 PMIC_AUXADC_ADC_BUSY_IN_IMP_MASK, PMIC_AUXADC_ADC_BUSY_IN_IMP_SHIFT},
	{PMIC_AUXADC_ADC_BUSY_IN_FGADC1, PMIC_AUXADC_ADC_BUSY_IN_FGADC1_ADDR,
	 PMIC_AUXADC_ADC_BUSY_IN_FGADC1_MASK, PMIC_AUXADC_ADC_BUSY_IN_FGADC1_SHIFT},
	{PMIC_AUXADC_ADC_BUSY_IN_FGADC2, PMIC_AUXADC_ADC_BUSY_IN_FGADC2_ADDR,
	 PMIC_AUXADC_ADC_BUSY_IN_FGADC2_MASK, PMIC_AUXADC_ADC_BUSY_IN_FGADC2_SHIFT},
	{PMIC_AUXADC_ADC_BUSY_IN_GPS_AP, PMIC_AUXADC_ADC_BUSY_IN_GPS_AP_ADDR,
	 PMIC_AUXADC_ADC_BUSY_IN_GPS_AP_MASK, PMIC_AUXADC_ADC_BUSY_IN_GPS_AP_SHIFT},
	{PMIC_AUXADC_ADC_BUSY_IN_GPS_MD, PMIC_AUXADC_ADC_BUSY_IN_GPS_MD_ADDR,
	 PMIC_AUXADC_ADC_BUSY_IN_GPS_MD_MASK, PMIC_AUXADC_ADC_BUSY_IN_GPS_MD_SHIFT},
	{PMIC_AUXADC_ADC_BUSY_IN_GPS, PMIC_AUXADC_ADC_BUSY_IN_GPS_ADDR,
	 PMIC_AUXADC_ADC_BUSY_IN_GPS_MASK, PMIC_AUXADC_ADC_BUSY_IN_GPS_SHIFT},
	{PMIC_AUXADC_ADC_BUSY_IN_THR_HW, PMIC_AUXADC_ADC_BUSY_IN_THR_HW_ADDR,
	 PMIC_AUXADC_ADC_BUSY_IN_THR_HW_MASK, PMIC_AUXADC_ADC_BUSY_IN_THR_HW_SHIFT},
	{PMIC_AUXADC_ADC_BUSY_IN_THR_MD, PMIC_AUXADC_ADC_BUSY_IN_THR_MD_ADDR,
	 PMIC_AUXADC_ADC_BUSY_IN_THR_MD_MASK, PMIC_AUXADC_ADC_BUSY_IN_THR_MD_SHIFT},
	{PMIC_AUXADC_ADC_BUSY_IN_BAT_PLUGIN1, PMIC_AUXADC_ADC_BUSY_IN_BAT_PLUGIN1_ADDR,
	 PMIC_AUXADC_ADC_BUSY_IN_BAT_PLUGIN1_MASK, PMIC_AUXADC_ADC_BUSY_IN_BAT_PLUGIN1_SHIFT},
	{PMIC_AUXADC_ADC_BUSY_IN_BAT_PLUGIN2, PMIC_AUXADC_ADC_BUSY_IN_BAT_PLUGIN2_ADDR,
	 PMIC_AUXADC_ADC_BUSY_IN_BAT_PLUGIN2_MASK, PMIC_AUXADC_ADC_BUSY_IN_BAT_PLUGIN2_SHIFT},
	{PMIC_AUXADC_ADC_BUSY_IN_BATID, PMIC_AUXADC_ADC_BUSY_IN_BATID_ADDR,
	 PMIC_AUXADC_ADC_BUSY_IN_BATID_MASK, PMIC_AUXADC_ADC_BUSY_IN_BATID_SHIFT},
	{PMIC_AUXADC_ADC_BUSY_IN_NAG, PMIC_AUXADC_ADC_BUSY_IN_NAG_ADDR,
	 PMIC_AUXADC_ADC_BUSY_IN_NAG_MASK, PMIC_AUXADC_ADC_BUSY_IN_NAG_SHIFT},
	{PMIC_AUXADC_RQST_CH0, PMIC_AUXADC_RQST_CH0_ADDR, PMIC_AUXADC_RQST_CH0_MASK,
	 PMIC_AUXADC_RQST_CH0_SHIFT},
	{PMIC_AUXADC_RQST_CH1, PMIC_AUXADC_RQST_CH1_ADDR, PMIC_AUXADC_RQST_CH1_MASK,
	 PMIC_AUXADC_RQST_CH1_SHIFT},
	{PMIC_AUXADC_RQST_CH2, PMIC_AUXADC_RQST_CH2_ADDR, PMIC_AUXADC_RQST_CH2_MASK,
	 PMIC_AUXADC_RQST_CH2_SHIFT},
	{PMIC_AUXADC_RQST_CH3, PMIC_AUXADC_RQST_CH3_ADDR, PMIC_AUXADC_RQST_CH3_MASK,
	 PMIC_AUXADC_RQST_CH3_SHIFT},
	{PMIC_AUXADC_RQST_CH4, PMIC_AUXADC_RQST_CH4_ADDR, PMIC_AUXADC_RQST_CH4_MASK,
	 PMIC_AUXADC_RQST_CH4_SHIFT},
	{PMIC_AUXADC_RQST_CH5, PMIC_AUXADC_RQST_CH5_ADDR, PMIC_AUXADC_RQST_CH5_MASK,
	 PMIC_AUXADC_RQST_CH5_SHIFT},
	{PMIC_AUXADC_RQST_CH6, PMIC_AUXADC_RQST_CH6_ADDR, PMIC_AUXADC_RQST_CH6_MASK,
	 PMIC_AUXADC_RQST_CH6_SHIFT},
	{PMIC_AUXADC_RQST_CH7, PMIC_AUXADC_RQST_CH7_ADDR, PMIC_AUXADC_RQST_CH7_MASK,
	 PMIC_AUXADC_RQST_CH7_SHIFT},
	{PMIC_AUXADC_RQST_CH8, PMIC_AUXADC_RQST_CH8_ADDR, PMIC_AUXADC_RQST_CH8_MASK,
	 PMIC_AUXADC_RQST_CH8_SHIFT},
	{PMIC_AUXADC_RQST_CH9, PMIC_AUXADC_RQST_CH9_ADDR, PMIC_AUXADC_RQST_CH9_MASK,
	 PMIC_AUXADC_RQST_CH9_SHIFT},
	{PMIC_AUXADC_RQST_CH10, PMIC_AUXADC_RQST_CH10_ADDR, PMIC_AUXADC_RQST_CH10_MASK,
	 PMIC_AUXADC_RQST_CH10_SHIFT},
	{PMIC_AUXADC_RQST_CH11, PMIC_AUXADC_RQST_CH11_ADDR, PMIC_AUXADC_RQST_CH11_MASK,
	 PMIC_AUXADC_RQST_CH11_SHIFT},
	{PMIC_AUXADC_RQST_CH12, PMIC_AUXADC_RQST_CH12_ADDR, PMIC_AUXADC_RQST_CH12_MASK,
	 PMIC_AUXADC_RQST_CH12_SHIFT},
	{PMIC_AUXADC_RQST_CH13, PMIC_AUXADC_RQST_CH13_ADDR, PMIC_AUXADC_RQST_CH13_MASK,
	 PMIC_AUXADC_RQST_CH13_SHIFT},
	{PMIC_AUXADC_RQST_CH14, PMIC_AUXADC_RQST_CH14_ADDR, PMIC_AUXADC_RQST_CH14_MASK,
	 PMIC_AUXADC_RQST_CH14_SHIFT},
	{PMIC_AUXADC_RQST_CH15, PMIC_AUXADC_RQST_CH15_ADDR, PMIC_AUXADC_RQST_CH15_MASK,
	 PMIC_AUXADC_RQST_CH15_SHIFT},
	{PMIC_AUXADC_RQST0_SET, PMIC_AUXADC_RQST0_SET_ADDR, PMIC_AUXADC_RQST0_SET_MASK,
	 PMIC_AUXADC_RQST0_SET_SHIFT},
	{PMIC_AUXADC_RQST0_CLR, PMIC_AUXADC_RQST0_CLR_ADDR, PMIC_AUXADC_RQST0_CLR_MASK,
	 PMIC_AUXADC_RQST0_CLR_SHIFT},
	{PMIC_AUXADC_RQST_CH0_BY_MD, PMIC_AUXADC_RQST_CH0_BY_MD_ADDR,
	 PMIC_AUXADC_RQST_CH0_BY_MD_MASK, PMIC_AUXADC_RQST_CH0_BY_MD_SHIFT},
	{PMIC_AUXADC_RQST_CH1_BY_MD, PMIC_AUXADC_RQST_CH1_BY_MD_ADDR,
	 PMIC_AUXADC_RQST_CH1_BY_MD_MASK, PMIC_AUXADC_RQST_CH1_BY_MD_SHIFT},
	{PMIC_AUXADC_RQST_RSV0, PMIC_AUXADC_RQST_RSV0_ADDR, PMIC_AUXADC_RQST_RSV0_MASK,
	 PMIC_AUXADC_RQST_RSV0_SHIFT},
	{PMIC_AUXADC_RQST_BATID, PMIC_AUXADC_RQST_BATID_ADDR, PMIC_AUXADC_RQST_BATID_MASK,
	 PMIC_AUXADC_RQST_BATID_SHIFT},
	{PMIC_AUXADC_RQST_CH4_BY_MD, PMIC_AUXADC_RQST_CH4_BY_MD_ADDR,
	 PMIC_AUXADC_RQST_CH4_BY_MD_MASK, PMIC_AUXADC_RQST_CH4_BY_MD_SHIFT},
	{PMIC_AUXADC_RQST_CH7_BY_MD, PMIC_AUXADC_RQST_CH7_BY_MD_ADDR,
	 PMIC_AUXADC_RQST_CH7_BY_MD_MASK, PMIC_AUXADC_RQST_CH7_BY_MD_SHIFT},
	{PMIC_AUXADC_RQST_CH7_BY_GPS, PMIC_AUXADC_RQST_CH7_BY_GPS_ADDR,
	 PMIC_AUXADC_RQST_CH7_BY_GPS_MASK, PMIC_AUXADC_RQST_CH7_BY_GPS_SHIFT},
	{PMIC_AUXADC_RQST_DCXO_BY_MD, PMIC_AUXADC_RQST_DCXO_BY_MD_ADDR,
	 PMIC_AUXADC_RQST_DCXO_BY_MD_MASK, PMIC_AUXADC_RQST_DCXO_BY_MD_SHIFT},
	{PMIC_AUXADC_RQST_DCXO_BY_GPS, PMIC_AUXADC_RQST_DCXO_BY_GPS_ADDR,
	 PMIC_AUXADC_RQST_DCXO_BY_GPS_MASK, PMIC_AUXADC_RQST_DCXO_BY_GPS_SHIFT},
	{PMIC_AUXADC_RQST_RSV1, PMIC_AUXADC_RQST_RSV1_ADDR, PMIC_AUXADC_RQST_RSV1_MASK,
	 PMIC_AUXADC_RQST_RSV1_SHIFT},
	{PMIC_AUXADC_RQST1_SET, PMIC_AUXADC_RQST1_SET_ADDR, PMIC_AUXADC_RQST1_SET_MASK,
	 PMIC_AUXADC_RQST1_SET_SHIFT},
	{PMIC_AUXADC_RQST1_CLR, PMIC_AUXADC_RQST1_CLR_ADDR, PMIC_AUXADC_RQST1_CLR_MASK,
	 PMIC_AUXADC_RQST1_CLR_SHIFT},
	{PMIC_AUXADC_CK_ON_EXTD, PMIC_AUXADC_CK_ON_EXTD_ADDR, PMIC_AUXADC_CK_ON_EXTD_MASK,
	 PMIC_AUXADC_CK_ON_EXTD_SHIFT},
	{PMIC_AUXADC_SRCLKEN_SRC_SEL, PMIC_AUXADC_SRCLKEN_SRC_SEL_ADDR,
	 PMIC_AUXADC_SRCLKEN_SRC_SEL_MASK, PMIC_AUXADC_SRCLKEN_SRC_SEL_SHIFT},
	{PMIC_AUXADC_ADC_PWDB, PMIC_AUXADC_ADC_PWDB_ADDR, PMIC_AUXADC_ADC_PWDB_MASK,
	 PMIC_AUXADC_ADC_PWDB_SHIFT},
	{PMIC_AUXADC_ADC_PWDB_SWCTRL, PMIC_AUXADC_ADC_PWDB_SWCTRL_ADDR,
	 PMIC_AUXADC_ADC_PWDB_SWCTRL_MASK, PMIC_AUXADC_ADC_PWDB_SWCTRL_SHIFT},
	{PMIC_AUXADC_STRUP_CK_ON_ENB, PMIC_AUXADC_STRUP_CK_ON_ENB_ADDR,
	 PMIC_AUXADC_STRUP_CK_ON_ENB_MASK, PMIC_AUXADC_STRUP_CK_ON_ENB_SHIFT},
	{PMIC_AUXADC_SRCLKEN_CK_EN, PMIC_AUXADC_SRCLKEN_CK_EN_ADDR, PMIC_AUXADC_SRCLKEN_CK_EN_MASK,
	 PMIC_AUXADC_SRCLKEN_CK_EN_SHIFT},
	{PMIC_AUXADC_CK_AON_GPS, PMIC_AUXADC_CK_AON_GPS_ADDR, PMIC_AUXADC_CK_AON_GPS_MASK,
	 PMIC_AUXADC_CK_AON_GPS_SHIFT},
	{PMIC_AUXADC_CK_AON_MD, PMIC_AUXADC_CK_AON_MD_ADDR, PMIC_AUXADC_CK_AON_MD_MASK,
	 PMIC_AUXADC_CK_AON_MD_SHIFT},
	{PMIC_AUXADC_CK_AON, PMIC_AUXADC_CK_AON_ADDR, PMIC_AUXADC_CK_AON_MASK,
	 PMIC_AUXADC_CK_AON_SHIFT},
	{PMIC_AUXADC_CON0_SET, PMIC_AUXADC_CON0_SET_ADDR, PMIC_AUXADC_CON0_SET_MASK,
	 PMIC_AUXADC_CON0_SET_SHIFT},
	{PMIC_AUXADC_CON0_CLR, PMIC_AUXADC_CON0_CLR_ADDR, PMIC_AUXADC_CON0_CLR_MASK,
	 PMIC_AUXADC_CON0_CLR_SHIFT},
	{PMIC_AUXADC_AVG_NUM_SMALL, PMIC_AUXADC_AVG_NUM_SMALL_ADDR, PMIC_AUXADC_AVG_NUM_SMALL_MASK,
	 PMIC_AUXADC_AVG_NUM_SMALL_SHIFT},
	{PMIC_AUXADC_AVG_NUM_LARGE, PMIC_AUXADC_AVG_NUM_LARGE_ADDR, PMIC_AUXADC_AVG_NUM_LARGE_MASK,
	 PMIC_AUXADC_AVG_NUM_LARGE_SHIFT},
	{PMIC_AUXADC_SPL_NUM, PMIC_AUXADC_SPL_NUM_ADDR, PMIC_AUXADC_SPL_NUM_MASK,
	 PMIC_AUXADC_SPL_NUM_SHIFT},
	{PMIC_AUXADC_AVG_NUM_SEL, PMIC_AUXADC_AVG_NUM_SEL_ADDR, PMIC_AUXADC_AVG_NUM_SEL_MASK,
	 PMIC_AUXADC_AVG_NUM_SEL_SHIFT},
	{PMIC_AUXADC_AVG_NUM_SEL_SHARE, PMIC_AUXADC_AVG_NUM_SEL_SHARE_ADDR,
	 PMIC_AUXADC_AVG_NUM_SEL_SHARE_MASK, PMIC_AUXADC_AVG_NUM_SEL_SHARE_SHIFT},
	{PMIC_AUXADC_AVG_NUM_SEL_LBAT, PMIC_AUXADC_AVG_NUM_SEL_LBAT_ADDR,
	 PMIC_AUXADC_AVG_NUM_SEL_LBAT_MASK, PMIC_AUXADC_AVG_NUM_SEL_LBAT_SHIFT},
	{PMIC_AUXADC_AVG_NUM_SEL_BAT_TEMP, PMIC_AUXADC_AVG_NUM_SEL_BAT_TEMP_ADDR,
	 PMIC_AUXADC_AVG_NUM_SEL_BAT_TEMP_MASK, PMIC_AUXADC_AVG_NUM_SEL_BAT_TEMP_SHIFT},
	{PMIC_AUXADC_AVG_NUM_SEL_WAKEUP, PMIC_AUXADC_AVG_NUM_SEL_WAKEUP_ADDR,
	 PMIC_AUXADC_AVG_NUM_SEL_WAKEUP_MASK, PMIC_AUXADC_AVG_NUM_SEL_WAKEUP_SHIFT},
	{PMIC_AUXADC_SPL_NUM_LARGE, PMIC_AUXADC_SPL_NUM_LARGE_ADDR, PMIC_AUXADC_SPL_NUM_LARGE_MASK,
	 PMIC_AUXADC_SPL_NUM_LARGE_SHIFT},
	{PMIC_AUXADC_SPL_NUM_SLEEP, PMIC_AUXADC_SPL_NUM_SLEEP_ADDR, PMIC_AUXADC_SPL_NUM_SLEEP_MASK,
	 PMIC_AUXADC_SPL_NUM_SLEEP_SHIFT},
	{PMIC_AUXADC_SPL_NUM_SLEEP_SEL, PMIC_AUXADC_SPL_NUM_SLEEP_SEL_ADDR,
	 PMIC_AUXADC_SPL_NUM_SLEEP_SEL_MASK, PMIC_AUXADC_SPL_NUM_SLEEP_SEL_SHIFT},
	{PMIC_AUXADC_SPL_NUM_SEL, PMIC_AUXADC_SPL_NUM_SEL_ADDR, PMIC_AUXADC_SPL_NUM_SEL_MASK,
	 PMIC_AUXADC_SPL_NUM_SEL_SHIFT},
	{PMIC_AUXADC_SPL_NUM_SEL_SHARE, PMIC_AUXADC_SPL_NUM_SEL_SHARE_ADDR,
	 PMIC_AUXADC_SPL_NUM_SEL_SHARE_MASK, PMIC_AUXADC_SPL_NUM_SEL_SHARE_SHIFT},
	{PMIC_AUXADC_SPL_NUM_SEL_LBAT, PMIC_AUXADC_SPL_NUM_SEL_LBAT_ADDR,
	 PMIC_AUXADC_SPL_NUM_SEL_LBAT_MASK, PMIC_AUXADC_SPL_NUM_SEL_LBAT_SHIFT},
	{PMIC_AUXADC_SPL_NUM_SEL_BAT_TEMP, PMIC_AUXADC_SPL_NUM_SEL_BAT_TEMP_ADDR,
	 PMIC_AUXADC_SPL_NUM_SEL_BAT_TEMP_MASK, PMIC_AUXADC_SPL_NUM_SEL_BAT_TEMP_SHIFT},
	{PMIC_AUXADC_SPL_NUM_SEL_WAKEUP, PMIC_AUXADC_SPL_NUM_SEL_WAKEUP_ADDR,
	 PMIC_AUXADC_SPL_NUM_SEL_WAKEUP_MASK, PMIC_AUXADC_SPL_NUM_SEL_WAKEUP_SHIFT},
	{PMIC_AUXADC_SPL_NUM_CH0, PMIC_AUXADC_SPL_NUM_CH0_ADDR, PMIC_AUXADC_SPL_NUM_CH0_MASK,
	 PMIC_AUXADC_SPL_NUM_CH0_SHIFT},
	{PMIC_AUXADC_SPL_NUM_CH3, PMIC_AUXADC_SPL_NUM_CH3_ADDR, PMIC_AUXADC_SPL_NUM_CH3_MASK,
	 PMIC_AUXADC_SPL_NUM_CH3_SHIFT},
	{PMIC_AUXADC_SPL_NUM_CH7, PMIC_AUXADC_SPL_NUM_CH7_ADDR, PMIC_AUXADC_SPL_NUM_CH7_MASK,
	 PMIC_AUXADC_SPL_NUM_CH7_SHIFT},
	{PMIC_AUXADC_AVG_NUM_LBAT, PMIC_AUXADC_AVG_NUM_LBAT_ADDR, PMIC_AUXADC_AVG_NUM_LBAT_MASK,
	 PMIC_AUXADC_AVG_NUM_LBAT_SHIFT},
	{PMIC_AUXADC_AVG_NUM_CH7, PMIC_AUXADC_AVG_NUM_CH7_ADDR, PMIC_AUXADC_AVG_NUM_CH7_MASK,
	 PMIC_AUXADC_AVG_NUM_CH7_SHIFT},
	{PMIC_AUXADC_AVG_NUM_CH3, PMIC_AUXADC_AVG_NUM_CH3_ADDR, PMIC_AUXADC_AVG_NUM_CH3_MASK,
	 PMIC_AUXADC_AVG_NUM_CH3_SHIFT},
	{PMIC_AUXADC_AVG_NUM_CH0, PMIC_AUXADC_AVG_NUM_CH0_ADDR, PMIC_AUXADC_AVG_NUM_CH0_MASK,
	 PMIC_AUXADC_AVG_NUM_CH0_SHIFT},
	{PMIC_AUXADC_TRIM_CH0_SEL, PMIC_AUXADC_TRIM_CH0_SEL_ADDR, PMIC_AUXADC_TRIM_CH0_SEL_MASK,
	 PMIC_AUXADC_TRIM_CH0_SEL_SHIFT},
	{PMIC_AUXADC_TRIM_CH1_SEL, PMIC_AUXADC_TRIM_CH1_SEL_ADDR, PMIC_AUXADC_TRIM_CH1_SEL_MASK,
	 PMIC_AUXADC_TRIM_CH1_SEL_SHIFT},
	{PMIC_AUXADC_TRIM_CH2_SEL, PMIC_AUXADC_TRIM_CH2_SEL_ADDR, PMIC_AUXADC_TRIM_CH2_SEL_MASK,
	 PMIC_AUXADC_TRIM_CH2_SEL_SHIFT},
	{PMIC_AUXADC_TRIM_CH3_SEL, PMIC_AUXADC_TRIM_CH3_SEL_ADDR, PMIC_AUXADC_TRIM_CH3_SEL_MASK,
	 PMIC_AUXADC_TRIM_CH3_SEL_SHIFT},
	{PMIC_AUXADC_TRIM_CH4_SEL, PMIC_AUXADC_TRIM_CH4_SEL_ADDR, PMIC_AUXADC_TRIM_CH4_SEL_MASK,
	 PMIC_AUXADC_TRIM_CH4_SEL_SHIFT},
	{PMIC_AUXADC_TRIM_CH5_SEL, PMIC_AUXADC_TRIM_CH5_SEL_ADDR, PMIC_AUXADC_TRIM_CH5_SEL_MASK,
	 PMIC_AUXADC_TRIM_CH5_SEL_SHIFT},
	{PMIC_AUXADC_TRIM_CH6_SEL, PMIC_AUXADC_TRIM_CH6_SEL_ADDR, PMIC_AUXADC_TRIM_CH6_SEL_MASK,
	 PMIC_AUXADC_TRIM_CH6_SEL_SHIFT},
	{PMIC_AUXADC_TRIM_CH7_SEL, PMIC_AUXADC_TRIM_CH7_SEL_ADDR, PMIC_AUXADC_TRIM_CH7_SEL_MASK,
	 PMIC_AUXADC_TRIM_CH7_SEL_SHIFT},
	{PMIC_AUXADC_TRIM_CH8_SEL, PMIC_AUXADC_TRIM_CH8_SEL_ADDR, PMIC_AUXADC_TRIM_CH8_SEL_MASK,
	 PMIC_AUXADC_TRIM_CH8_SEL_SHIFT},
	{PMIC_AUXADC_TRIM_CH9_SEL, PMIC_AUXADC_TRIM_CH9_SEL_ADDR, PMIC_AUXADC_TRIM_CH9_SEL_MASK,
	 PMIC_AUXADC_TRIM_CH9_SEL_SHIFT},
	{PMIC_AUXADC_TRIM_CH10_SEL, PMIC_AUXADC_TRIM_CH10_SEL_ADDR, PMIC_AUXADC_TRIM_CH10_SEL_MASK,
	 PMIC_AUXADC_TRIM_CH10_SEL_SHIFT},
	{PMIC_AUXADC_TRIM_CH11_SEL, PMIC_AUXADC_TRIM_CH11_SEL_ADDR, PMIC_AUXADC_TRIM_CH11_SEL_MASK,
	 PMIC_AUXADC_TRIM_CH11_SEL_SHIFT},
	{PMIC_AUXADC_ADC_2S_COMP_ENB, PMIC_AUXADC_ADC_2S_COMP_ENB_ADDR,
	 PMIC_AUXADC_ADC_2S_COMP_ENB_MASK, PMIC_AUXADC_ADC_2S_COMP_ENB_SHIFT},
	{PMIC_AUXADC_ADC_TRIM_COMP, PMIC_AUXADC_ADC_TRIM_COMP_ADDR, PMIC_AUXADC_ADC_TRIM_COMP_MASK,
	 PMIC_AUXADC_ADC_TRIM_COMP_SHIFT},
	{PMIC_AUXADC_SW_GAIN_TRIM, PMIC_AUXADC_SW_GAIN_TRIM_ADDR, PMIC_AUXADC_SW_GAIN_TRIM_MASK,
	 PMIC_AUXADC_SW_GAIN_TRIM_SHIFT},
	{PMIC_AUXADC_SW_OFFSET_TRIM, PMIC_AUXADC_SW_OFFSET_TRIM_ADDR,
	 PMIC_AUXADC_SW_OFFSET_TRIM_MASK, PMIC_AUXADC_SW_OFFSET_TRIM_SHIFT},
	{PMIC_AUXADC_RNG_EN, PMIC_AUXADC_RNG_EN_ADDR, PMIC_AUXADC_RNG_EN_MASK,
	 PMIC_AUXADC_RNG_EN_SHIFT},
	{PMIC_AUXADC_TEST_MODE, PMIC_AUXADC_TEST_MODE_ADDR, PMIC_AUXADC_TEST_MODE_MASK,
	 PMIC_AUXADC_TEST_MODE_SHIFT},
	{PMIC_AUXADC_BIT_SEL, PMIC_AUXADC_BIT_SEL_ADDR, PMIC_AUXADC_BIT_SEL_MASK,
	 PMIC_AUXADC_BIT_SEL_SHIFT},
	{PMIC_AUXADC_START_SW, PMIC_AUXADC_START_SW_ADDR, PMIC_AUXADC_START_SW_MASK,
	 PMIC_AUXADC_START_SW_SHIFT},
	{PMIC_AUXADC_START_SWCTRL, PMIC_AUXADC_START_SWCTRL_ADDR, PMIC_AUXADC_START_SWCTRL_MASK,
	 PMIC_AUXADC_START_SWCTRL_SHIFT},
	{PMIC_AUXADC_TS_VBE_SEL, PMIC_AUXADC_TS_VBE_SEL_ADDR, PMIC_AUXADC_TS_VBE_SEL_MASK,
	 PMIC_AUXADC_TS_VBE_SEL_SHIFT},
	{PMIC_AUXADC_TS_VBE_SEL_SWCTRL, PMIC_AUXADC_TS_VBE_SEL_SWCTRL_ADDR,
	 PMIC_AUXADC_TS_VBE_SEL_SWCTRL_MASK, PMIC_AUXADC_TS_VBE_SEL_SWCTRL_SHIFT},
	{PMIC_AUXADC_VBUF_EN, PMIC_AUXADC_VBUF_EN_ADDR, PMIC_AUXADC_VBUF_EN_MASK,
	 PMIC_AUXADC_VBUF_EN_SHIFT},
	{PMIC_AUXADC_VBUF_EN_SWCTRL, PMIC_AUXADC_VBUF_EN_SWCTRL_ADDR,
	 PMIC_AUXADC_VBUF_EN_SWCTRL_MASK, PMIC_AUXADC_VBUF_EN_SWCTRL_SHIFT},
	{PMIC_AUXADC_OUT_SEL, PMIC_AUXADC_OUT_SEL_ADDR, PMIC_AUXADC_OUT_SEL_MASK,
	 PMIC_AUXADC_OUT_SEL_SHIFT},
	{PMIC_AUXADC_DA_DAC, PMIC_AUXADC_DA_DAC_ADDR, PMIC_AUXADC_DA_DAC_MASK,
	 PMIC_AUXADC_DA_DAC_SHIFT},
	{PMIC_AUXADC_DA_DAC_SWCTRL, PMIC_AUXADC_DA_DAC_SWCTRL_ADDR, PMIC_AUXADC_DA_DAC_SWCTRL_MASK,
	 PMIC_AUXADC_DA_DAC_SWCTRL_SHIFT},
	{PMIC_AD_AUXADC_COMP, PMIC_AD_AUXADC_COMP_ADDR, PMIC_AD_AUXADC_COMP_MASK,
	 PMIC_AD_AUXADC_COMP_SHIFT},
	{PMIC_RG_VBUF_EXTEN, PMIC_RG_VBUF_EXTEN_ADDR, PMIC_RG_VBUF_EXTEN_MASK,
	 PMIC_RG_VBUF_EXTEN_SHIFT},
	{PMIC_RG_VBUF_CALEN, PMIC_RG_VBUF_CALEN_ADDR, PMIC_RG_VBUF_CALEN_MASK,
	 PMIC_RG_VBUF_CALEN_SHIFT},
	{PMIC_RG_VBUF_BYP, PMIC_RG_VBUF_BYP_ADDR, PMIC_RG_VBUF_BYP_MASK, PMIC_RG_VBUF_BYP_SHIFT},
	{PMIC_RG_AUX_RSV, PMIC_RG_AUX_RSV_ADDR, PMIC_RG_AUX_RSV_MASK, PMIC_RG_AUX_RSV_SHIFT},
	{PMIC_RG_AUXADC_CALI, PMIC_RG_AUXADC_CALI_ADDR, PMIC_RG_AUXADC_CALI_MASK,
	 PMIC_RG_AUXADC_CALI_SHIFT},
	{PMIC_AUXADC_ADCIN_VSEN_EN, PMIC_AUXADC_ADCIN_VSEN_EN_ADDR, PMIC_AUXADC_ADCIN_VSEN_EN_MASK,
	 PMIC_AUXADC_ADCIN_VSEN_EN_SHIFT},
	{PMIC_AUXADC_ADCIN_VBAT_EN, PMIC_AUXADC_ADCIN_VBAT_EN_ADDR, PMIC_AUXADC_ADCIN_VBAT_EN_MASK,
	 PMIC_AUXADC_ADCIN_VBAT_EN_SHIFT},
	{PMIC_AUXADC_ADCIN_VSEN_MUX_EN, PMIC_AUXADC_ADCIN_VSEN_MUX_EN_ADDR,
	 PMIC_AUXADC_ADCIN_VSEN_MUX_EN_MASK, PMIC_AUXADC_ADCIN_VSEN_MUX_EN_SHIFT},
	{PMIC_AUXADC_ADCIN_VSEN_EXT_BATON_EN, PMIC_AUXADC_ADCIN_VSEN_EXT_BATON_EN_ADDR,
	 PMIC_AUXADC_ADCIN_VSEN_EXT_BATON_EN_MASK, PMIC_AUXADC_ADCIN_VSEN_EXT_BATON_EN_SHIFT},
	{PMIC_AUXADC_ADCIN_CHR_EN, PMIC_AUXADC_ADCIN_CHR_EN_ADDR, PMIC_AUXADC_ADCIN_CHR_EN_MASK,
	 PMIC_AUXADC_ADCIN_CHR_EN_SHIFT},
	{PMIC_AUXADC_ADCIN_BATON_TDET_EN, PMIC_AUXADC_ADCIN_BATON_TDET_EN_ADDR,
	 PMIC_AUXADC_ADCIN_BATON_TDET_EN_MASK, PMIC_AUXADC_ADCIN_BATON_TDET_EN_SHIFT},
	{PMIC_AUXADC_ACCDET_ANASWCTRL_EN, PMIC_AUXADC_ACCDET_ANASWCTRL_EN_ADDR,
	 PMIC_AUXADC_ACCDET_ANASWCTRL_EN_MASK, PMIC_AUXADC_ACCDET_ANASWCTRL_EN_SHIFT},
	{PMIC_AUXADC_XO_THADC_EN, PMIC_AUXADC_XO_THADC_EN_ADDR, PMIC_AUXADC_XO_THADC_EN_MASK,
	 PMIC_AUXADC_XO_THADC_EN_SHIFT},
	{PMIC_AUXADC_ADCIN_BATID_SW_EN, PMIC_AUXADC_ADCIN_BATID_SW_EN_ADDR,
	 PMIC_AUXADC_ADCIN_BATID_SW_EN_MASK, PMIC_AUXADC_ADCIN_BATID_SW_EN_SHIFT},
	{PMIC_AUXADC_DIG0_RSV0, PMIC_AUXADC_DIG0_RSV0_ADDR, PMIC_AUXADC_DIG0_RSV0_MASK,
	 PMIC_AUXADC_DIG0_RSV0_SHIFT},
	{PMIC_AUXADC_CHSEL, PMIC_AUXADC_CHSEL_ADDR, PMIC_AUXADC_CHSEL_MASK,
	 PMIC_AUXADC_CHSEL_SHIFT},
	{PMIC_AUXADC_SWCTRL_EN, PMIC_AUXADC_SWCTRL_EN_ADDR, PMIC_AUXADC_SWCTRL_EN_MASK,
	 PMIC_AUXADC_SWCTRL_EN_SHIFT},
	{PMIC_AUXADC_SOURCE_LBAT_SEL, PMIC_AUXADC_SOURCE_LBAT_SEL_ADDR,
	 PMIC_AUXADC_SOURCE_LBAT_SEL_MASK, PMIC_AUXADC_SOURCE_LBAT_SEL_SHIFT},
	{PMIC_AUXADC_SOURCE_LBAT2_SEL, PMIC_AUXADC_SOURCE_LBAT2_SEL_ADDR,
	 PMIC_AUXADC_SOURCE_LBAT2_SEL_MASK, PMIC_AUXADC_SOURCE_LBAT2_SEL_SHIFT},
	{PMIC_AUXADC_START_EXTD, PMIC_AUXADC_START_EXTD_ADDR, PMIC_AUXADC_START_EXTD_MASK,
	 PMIC_AUXADC_START_EXTD_SHIFT},
	{PMIC_AUXADC_DAC_EXTD, PMIC_AUXADC_DAC_EXTD_ADDR, PMIC_AUXADC_DAC_EXTD_MASK,
	 PMIC_AUXADC_DAC_EXTD_SHIFT},
	{PMIC_AUXADC_DAC_EXTD_EN, PMIC_AUXADC_DAC_EXTD_EN_ADDR, PMIC_AUXADC_DAC_EXTD_EN_MASK,
	 PMIC_AUXADC_DAC_EXTD_EN_SHIFT},
	{PMIC_AUXADC_PMU_THR_PDN_SW, PMIC_AUXADC_PMU_THR_PDN_SW_ADDR,
	 PMIC_AUXADC_PMU_THR_PDN_SW_MASK, PMIC_AUXADC_PMU_THR_PDN_SW_SHIFT},
	{PMIC_AUXADC_PMU_THR_PDN_SEL, PMIC_AUXADC_PMU_THR_PDN_SEL_ADDR,
	 PMIC_AUXADC_PMU_THR_PDN_SEL_MASK, PMIC_AUXADC_PMU_THR_PDN_SEL_SHIFT},
	{PMIC_AUXADC_PMU_THR_PDN_STATUS, PMIC_AUXADC_PMU_THR_PDN_STATUS_ADDR,
	 PMIC_AUXADC_PMU_THR_PDN_STATUS_MASK, PMIC_AUXADC_PMU_THR_PDN_STATUS_SHIFT},
	{PMIC_AUXADC_DIG0_RSV1, PMIC_AUXADC_DIG0_RSV1_ADDR, PMIC_AUXADC_DIG0_RSV1_MASK,
	 PMIC_AUXADC_DIG0_RSV1_SHIFT},
	{PMIC_AUXADC_START_SHADE_NUM, PMIC_AUXADC_START_SHADE_NUM_ADDR,
	 PMIC_AUXADC_START_SHADE_NUM_MASK, PMIC_AUXADC_START_SHADE_NUM_SHIFT},
	{PMIC_AUXADC_START_SHADE_EN, PMIC_AUXADC_START_SHADE_EN_ADDR,
	 PMIC_AUXADC_START_SHADE_EN_MASK, PMIC_AUXADC_START_SHADE_EN_SHIFT},
	{PMIC_AUXADC_START_SHADE_SEL, PMIC_AUXADC_START_SHADE_SEL_ADDR,
	 PMIC_AUXADC_START_SHADE_SEL_MASK, PMIC_AUXADC_START_SHADE_SEL_SHIFT},
	{PMIC_AUXADC_ADC_RDY_WAKEUP_CLR, PMIC_AUXADC_ADC_RDY_WAKEUP_CLR_ADDR,
	 PMIC_AUXADC_ADC_RDY_WAKEUP_CLR_MASK, PMIC_AUXADC_ADC_RDY_WAKEUP_CLR_SHIFT},
	{PMIC_AUXADC_ADC_RDY_FGADC_CLR, PMIC_AUXADC_ADC_RDY_FGADC_CLR_ADDR,
	 PMIC_AUXADC_ADC_RDY_FGADC_CLR_MASK, PMIC_AUXADC_ADC_RDY_FGADC_CLR_SHIFT},
	{PMIC_AUXADC_ADC_RDY_BAT_PLUGIN_CLR, PMIC_AUXADC_ADC_RDY_BAT_PLUGIN_CLR_ADDR,
	 PMIC_AUXADC_ADC_RDY_BAT_PLUGIN_CLR_MASK, PMIC_AUXADC_ADC_RDY_BAT_PLUGIN_CLR_SHIFT},
	{PMIC_AUXADC_DATA_REUSE_SEL, PMIC_AUXADC_DATA_REUSE_SEL_ADDR,
	 PMIC_AUXADC_DATA_REUSE_SEL_MASK, PMIC_AUXADC_DATA_REUSE_SEL_SHIFT},
	{PMIC_AUXADC_CH0_DATA_REUSE_SEL, PMIC_AUXADC_CH0_DATA_REUSE_SEL_ADDR,
	 PMIC_AUXADC_CH0_DATA_REUSE_SEL_MASK, PMIC_AUXADC_CH0_DATA_REUSE_SEL_SHIFT},
	{PMIC_AUXADC_CH1_DATA_REUSE_SEL, PMIC_AUXADC_CH1_DATA_REUSE_SEL_ADDR,
	 PMIC_AUXADC_CH1_DATA_REUSE_SEL_MASK, PMIC_AUXADC_CH1_DATA_REUSE_SEL_SHIFT},
	{PMIC_AUXADC_DCXO_DATA_REUSE_SEL, PMIC_AUXADC_DCXO_DATA_REUSE_SEL_ADDR,
	 PMIC_AUXADC_DCXO_DATA_REUSE_SEL_MASK, PMIC_AUXADC_DCXO_DATA_REUSE_SEL_SHIFT},
	{PMIC_AUXADC_DATA_REUSE_EN, PMIC_AUXADC_DATA_REUSE_EN_ADDR, PMIC_AUXADC_DATA_REUSE_EN_MASK,
	 PMIC_AUXADC_DATA_REUSE_EN_SHIFT},
	{PMIC_AUXADC_CH0_DATA_REUSE_EN, PMIC_AUXADC_CH0_DATA_REUSE_EN_ADDR,
	 PMIC_AUXADC_CH0_DATA_REUSE_EN_MASK, PMIC_AUXADC_CH0_DATA_REUSE_EN_SHIFT},
	{PMIC_AUXADC_CH1_DATA_REUSE_EN, PMIC_AUXADC_CH1_DATA_REUSE_EN_ADDR,
	 PMIC_AUXADC_CH1_DATA_REUSE_EN_MASK, PMIC_AUXADC_CH1_DATA_REUSE_EN_SHIFT},
	{PMIC_AUXADC_DCXO_DATA_REUSE_EN, PMIC_AUXADC_DCXO_DATA_REUSE_EN_ADDR,
	 PMIC_AUXADC_DCXO_DATA_REUSE_EN_MASK, PMIC_AUXADC_DCXO_DATA_REUSE_EN_SHIFT},
	{PMIC_AUXADC_AUTORPT_PRD, PMIC_AUXADC_AUTORPT_PRD_ADDR, PMIC_AUXADC_AUTORPT_PRD_MASK,
	 PMIC_AUXADC_AUTORPT_PRD_SHIFT},
	{PMIC_AUXADC_AUTORPT_EN, PMIC_AUXADC_AUTORPT_EN_ADDR, PMIC_AUXADC_AUTORPT_EN_MASK,
	 PMIC_AUXADC_AUTORPT_EN_SHIFT},
	{PMIC_AUXADC_LBAT_DEBT_MAX, PMIC_AUXADC_LBAT_DEBT_MAX_ADDR, PMIC_AUXADC_LBAT_DEBT_MAX_MASK,
	 PMIC_AUXADC_LBAT_DEBT_MAX_SHIFT},
	{PMIC_AUXADC_LBAT_DEBT_MIN, PMIC_AUXADC_LBAT_DEBT_MIN_ADDR, PMIC_AUXADC_LBAT_DEBT_MIN_MASK,
	 PMIC_AUXADC_LBAT_DEBT_MIN_SHIFT},
	{PMIC_AUXADC_LBAT_DET_PRD_15_0, PMIC_AUXADC_LBAT_DET_PRD_15_0_ADDR,
	 PMIC_AUXADC_LBAT_DET_PRD_15_0_MASK, PMIC_AUXADC_LBAT_DET_PRD_15_0_SHIFT},
	{PMIC_AUXADC_LBAT_DET_PRD_19_16, PMIC_AUXADC_LBAT_DET_PRD_19_16_ADDR,
	 PMIC_AUXADC_LBAT_DET_PRD_19_16_MASK, PMIC_AUXADC_LBAT_DET_PRD_19_16_SHIFT},
	{PMIC_AUXADC_LBAT_VOLT_MAX, PMIC_AUXADC_LBAT_VOLT_MAX_ADDR, PMIC_AUXADC_LBAT_VOLT_MAX_MASK,
	 PMIC_AUXADC_LBAT_VOLT_MAX_SHIFT},
	{PMIC_AUXADC_LBAT_IRQ_EN_MAX, PMIC_AUXADC_LBAT_IRQ_EN_MAX_ADDR,
	 PMIC_AUXADC_LBAT_IRQ_EN_MAX_MASK, PMIC_AUXADC_LBAT_IRQ_EN_MAX_SHIFT},
	{PMIC_AUXADC_LBAT_EN_MAX, PMIC_AUXADC_LBAT_EN_MAX_ADDR, PMIC_AUXADC_LBAT_EN_MAX_MASK,
	 PMIC_AUXADC_LBAT_EN_MAX_SHIFT},
	{PMIC_AUXADC_LBAT_MAX_IRQ_B, PMIC_AUXADC_LBAT_MAX_IRQ_B_ADDR,
	 PMIC_AUXADC_LBAT_MAX_IRQ_B_MASK, PMIC_AUXADC_LBAT_MAX_IRQ_B_SHIFT},
	{PMIC_AUXADC_LBAT_VOLT_MIN, PMIC_AUXADC_LBAT_VOLT_MIN_ADDR, PMIC_AUXADC_LBAT_VOLT_MIN_MASK,
	 PMIC_AUXADC_LBAT_VOLT_MIN_SHIFT},
	{PMIC_AUXADC_LBAT_IRQ_EN_MIN, PMIC_AUXADC_LBAT_IRQ_EN_MIN_ADDR,
	 PMIC_AUXADC_LBAT_IRQ_EN_MIN_MASK, PMIC_AUXADC_LBAT_IRQ_EN_MIN_SHIFT},
	{PMIC_AUXADC_LBAT_EN_MIN, PMIC_AUXADC_LBAT_EN_MIN_ADDR, PMIC_AUXADC_LBAT_EN_MIN_MASK,
	 PMIC_AUXADC_LBAT_EN_MIN_SHIFT},
	{PMIC_AUXADC_LBAT_MIN_IRQ_B, PMIC_AUXADC_LBAT_MIN_IRQ_B_ADDR,
	 PMIC_AUXADC_LBAT_MIN_IRQ_B_MASK, PMIC_AUXADC_LBAT_MIN_IRQ_B_SHIFT},
	{PMIC_AUXADC_LBAT_DEBOUNCE_COUNT_MAX, PMIC_AUXADC_LBAT_DEBOUNCE_COUNT_MAX_ADDR,
	 PMIC_AUXADC_LBAT_DEBOUNCE_COUNT_MAX_MASK, PMIC_AUXADC_LBAT_DEBOUNCE_COUNT_MAX_SHIFT},
	{PMIC_AUXADC_LBAT_DEBOUNCE_COUNT_MIN, PMIC_AUXADC_LBAT_DEBOUNCE_COUNT_MIN_ADDR,
	 PMIC_AUXADC_LBAT_DEBOUNCE_COUNT_MIN_MASK, PMIC_AUXADC_LBAT_DEBOUNCE_COUNT_MIN_SHIFT},
	{PMIC_AUXADC_ACCDET_AUTO_SPL, PMIC_AUXADC_ACCDET_AUTO_SPL_ADDR,
	 PMIC_AUXADC_ACCDET_AUTO_SPL_MASK, PMIC_AUXADC_ACCDET_AUTO_SPL_SHIFT},
	{PMIC_AUXADC_ACCDET_AUTO_RQST_CLR, PMIC_AUXADC_ACCDET_AUTO_RQST_CLR_ADDR,
	 PMIC_AUXADC_ACCDET_AUTO_RQST_CLR_MASK, PMIC_AUXADC_ACCDET_AUTO_RQST_CLR_SHIFT},
	{PMIC_AUXADC_ACCDET_DIG1_RSV0, PMIC_AUXADC_ACCDET_DIG1_RSV0_ADDR,
	 PMIC_AUXADC_ACCDET_DIG1_RSV0_MASK, PMIC_AUXADC_ACCDET_DIG1_RSV0_SHIFT},
	{PMIC_AUXADC_ACCDET_DIG0_RSV0, PMIC_AUXADC_ACCDET_DIG0_RSV0_ADDR,
	 PMIC_AUXADC_ACCDET_DIG0_RSV0_MASK, PMIC_AUXADC_ACCDET_DIG0_RSV0_SHIFT},
	{PMIC_AUXADC_THR_DEBT_MAX, PMIC_AUXADC_THR_DEBT_MAX_ADDR, PMIC_AUXADC_THR_DEBT_MAX_MASK,
	 PMIC_AUXADC_THR_DEBT_MAX_SHIFT},
	{PMIC_AUXADC_THR_DEBT_MIN, PMIC_AUXADC_THR_DEBT_MIN_ADDR, PMIC_AUXADC_THR_DEBT_MIN_MASK,
	 PMIC_AUXADC_THR_DEBT_MIN_SHIFT},
	{PMIC_AUXADC_THR_DET_PRD_15_0, PMIC_AUXADC_THR_DET_PRD_15_0_ADDR,
	 PMIC_AUXADC_THR_DET_PRD_15_0_MASK, PMIC_AUXADC_THR_DET_PRD_15_0_SHIFT},
	{PMIC_AUXADC_THR_DET_PRD_19_16, PMIC_AUXADC_THR_DET_PRD_19_16_ADDR,
	 PMIC_AUXADC_THR_DET_PRD_19_16_MASK, PMIC_AUXADC_THR_DET_PRD_19_16_SHIFT},
	{PMIC_AUXADC_THR_VOLT_MAX, PMIC_AUXADC_THR_VOLT_MAX_ADDR, PMIC_AUXADC_THR_VOLT_MAX_MASK,
	 PMIC_AUXADC_THR_VOLT_MAX_SHIFT},
	{PMIC_AUXADC_THR_IRQ_EN_MAX, PMIC_AUXADC_THR_IRQ_EN_MAX_ADDR,
	 PMIC_AUXADC_THR_IRQ_EN_MAX_MASK, PMIC_AUXADC_THR_IRQ_EN_MAX_SHIFT},
	{PMIC_AUXADC_THR_EN_MAX, PMIC_AUXADC_THR_EN_MAX_ADDR, PMIC_AUXADC_THR_EN_MAX_MASK,
	 PMIC_AUXADC_THR_EN_MAX_SHIFT},
	{PMIC_AUXADC_THR_MAX_IRQ_B, PMIC_AUXADC_THR_MAX_IRQ_B_ADDR, PMIC_AUXADC_THR_MAX_IRQ_B_MASK,
	 PMIC_AUXADC_THR_MAX_IRQ_B_SHIFT},
	{PMIC_AUXADC_THR_VOLT_MIN, PMIC_AUXADC_THR_VOLT_MIN_ADDR, PMIC_AUXADC_THR_VOLT_MIN_MASK,
	 PMIC_AUXADC_THR_VOLT_MIN_SHIFT},
	{PMIC_AUXADC_THR_IRQ_EN_MIN, PMIC_AUXADC_THR_IRQ_EN_MIN_ADDR,
	 PMIC_AUXADC_THR_IRQ_EN_MIN_MASK, PMIC_AUXADC_THR_IRQ_EN_MIN_SHIFT},
	{PMIC_AUXADC_THR_EN_MIN, PMIC_AUXADC_THR_EN_MIN_ADDR, PMIC_AUXADC_THR_EN_MIN_MASK,
	 PMIC_AUXADC_THR_EN_MIN_SHIFT},
	{PMIC_AUXADC_THR_MIN_IRQ_B, PMIC_AUXADC_THR_MIN_IRQ_B_ADDR, PMIC_AUXADC_THR_MIN_IRQ_B_MASK,
	 PMIC_AUXADC_THR_MIN_IRQ_B_SHIFT},
	{PMIC_AUXADC_THR_DEBOUNCE_COUNT_MAX, PMIC_AUXADC_THR_DEBOUNCE_COUNT_MAX_ADDR,
	 PMIC_AUXADC_THR_DEBOUNCE_COUNT_MAX_MASK, PMIC_AUXADC_THR_DEBOUNCE_COUNT_MAX_SHIFT},
	{PMIC_AUXADC_THR_DEBOUNCE_COUNT_MIN, PMIC_AUXADC_THR_DEBOUNCE_COUNT_MIN_ADDR,
	 PMIC_AUXADC_THR_DEBOUNCE_COUNT_MIN_MASK, PMIC_AUXADC_THR_DEBOUNCE_COUNT_MIN_SHIFT},
	{PMIC_EFUSE_GAIN_CH4_TRIM, PMIC_EFUSE_GAIN_CH4_TRIM_ADDR, PMIC_EFUSE_GAIN_CH4_TRIM_MASK,
	 PMIC_EFUSE_GAIN_CH4_TRIM_SHIFT},
	{PMIC_EFUSE_OFFSET_CH4_TRIM, PMIC_EFUSE_OFFSET_CH4_TRIM_ADDR,
	 PMIC_EFUSE_OFFSET_CH4_TRIM_MASK, PMIC_EFUSE_OFFSET_CH4_TRIM_SHIFT},
	{PMIC_EFUSE_GAIN_CH0_TRIM, PMIC_EFUSE_GAIN_CH0_TRIM_ADDR, PMIC_EFUSE_GAIN_CH0_TRIM_MASK,
	 PMIC_EFUSE_GAIN_CH0_TRIM_SHIFT},
	{PMIC_EFUSE_OFFSET_CH0_TRIM, PMIC_EFUSE_OFFSET_CH0_TRIM_ADDR,
	 PMIC_EFUSE_OFFSET_CH0_TRIM_MASK, PMIC_EFUSE_OFFSET_CH0_TRIM_SHIFT},
	{PMIC_EFUSE_GAIN_CH7_TRIM, PMIC_EFUSE_GAIN_CH7_TRIM_ADDR, PMIC_EFUSE_GAIN_CH7_TRIM_MASK,
	 PMIC_EFUSE_GAIN_CH7_TRIM_SHIFT},
	{PMIC_EFUSE_OFFSET_CH7_TRIM, PMIC_EFUSE_OFFSET_CH7_TRIM_ADDR,
	 PMIC_EFUSE_OFFSET_CH7_TRIM_MASK, PMIC_EFUSE_OFFSET_CH7_TRIM_SHIFT},
	{PMIC_AUXADC_FGADC_START_SW, PMIC_AUXADC_FGADC_START_SW_ADDR,
	 PMIC_AUXADC_FGADC_START_SW_MASK, PMIC_AUXADC_FGADC_START_SW_SHIFT},
	{PMIC_AUXADC_FGADC_START_SEL, PMIC_AUXADC_FGADC_START_SEL_ADDR,
	 PMIC_AUXADC_FGADC_START_SEL_MASK, PMIC_AUXADC_FGADC_START_SEL_SHIFT},
	{PMIC_AUXADC_FGADC_R_SW, PMIC_AUXADC_FGADC_R_SW_ADDR, PMIC_AUXADC_FGADC_R_SW_MASK,
	 PMIC_AUXADC_FGADC_R_SW_SHIFT},
	{PMIC_AUXADC_FGADC_R_SEL, PMIC_AUXADC_FGADC_R_SEL_ADDR, PMIC_AUXADC_FGADC_R_SEL_MASK,
	 PMIC_AUXADC_FGADC_R_SEL_SHIFT},
	{PMIC_AUXADC_BAT_PLUGIN_START_SW, PMIC_AUXADC_BAT_PLUGIN_START_SW_ADDR,
	 PMIC_AUXADC_BAT_PLUGIN_START_SW_MASK, PMIC_AUXADC_BAT_PLUGIN_START_SW_SHIFT},
	{PMIC_AUXADC_BAT_PLUGIN_START_SEL, PMIC_AUXADC_BAT_PLUGIN_START_SEL_ADDR,
	 PMIC_AUXADC_BAT_PLUGIN_START_SEL_MASK, PMIC_AUXADC_BAT_PLUGIN_START_SEL_SHIFT},
	{PMIC_AUXADC_DBG_DIG0_RSV2, PMIC_AUXADC_DBG_DIG0_RSV2_ADDR, PMIC_AUXADC_DBG_DIG0_RSV2_MASK,
	 PMIC_AUXADC_DBG_DIG0_RSV2_SHIFT},
	{PMIC_AUXADC_DBG_DIG1_RSV2, PMIC_AUXADC_DBG_DIG1_RSV2_ADDR, PMIC_AUXADC_DBG_DIG1_RSV2_MASK,
	 PMIC_AUXADC_DBG_DIG1_RSV2_SHIFT},
	{PMIC_AUXADC_IMPEDANCE_CNT, PMIC_AUXADC_IMPEDANCE_CNT_ADDR, PMIC_AUXADC_IMPEDANCE_CNT_MASK,
	 PMIC_AUXADC_IMPEDANCE_CNT_SHIFT},
	{PMIC_AUXADC_IMPEDANCE_CHSEL, PMIC_AUXADC_IMPEDANCE_CHSEL_ADDR,
	 PMIC_AUXADC_IMPEDANCE_CHSEL_MASK, PMIC_AUXADC_IMPEDANCE_CHSEL_SHIFT},
	{PMIC_AUXADC_IMPEDANCE_IRQ_CLR, PMIC_AUXADC_IMPEDANCE_IRQ_CLR_ADDR,
	 PMIC_AUXADC_IMPEDANCE_IRQ_CLR_MASK, PMIC_AUXADC_IMPEDANCE_IRQ_CLR_SHIFT},
	{PMIC_AUXADC_IMPEDANCE_IRQ_STATUS, PMIC_AUXADC_IMPEDANCE_IRQ_STATUS_ADDR,
	 PMIC_AUXADC_IMPEDANCE_IRQ_STATUS_MASK, PMIC_AUXADC_IMPEDANCE_IRQ_STATUS_SHIFT},
	{PMIC_AUXADC_CLR_IMP_CNT_STOP, PMIC_AUXADC_CLR_IMP_CNT_STOP_ADDR,
	 PMIC_AUXADC_CLR_IMP_CNT_STOP_MASK, PMIC_AUXADC_CLR_IMP_CNT_STOP_SHIFT},
	{PMIC_AUXADC_IMPEDANCE_MODE, PMIC_AUXADC_IMPEDANCE_MODE_ADDR,
	 PMIC_AUXADC_IMPEDANCE_MODE_MASK, PMIC_AUXADC_IMPEDANCE_MODE_SHIFT},
	{PMIC_AUXADC_IMP_AUTORPT_PRD, PMIC_AUXADC_IMP_AUTORPT_PRD_ADDR,
	 PMIC_AUXADC_IMP_AUTORPT_PRD_MASK, PMIC_AUXADC_IMP_AUTORPT_PRD_SHIFT},
	{PMIC_AUXADC_IMP_AUTORPT_EN, PMIC_AUXADC_IMP_AUTORPT_EN_ADDR,
	 PMIC_AUXADC_IMP_AUTORPT_EN_MASK, PMIC_AUXADC_IMP_AUTORPT_EN_SHIFT},
	{PMIC_AUXADC_BAT_TEMP_FROZE_EN, PMIC_AUXADC_BAT_TEMP_FROZE_EN_ADDR,
	 PMIC_AUXADC_BAT_TEMP_FROZE_EN_MASK, PMIC_AUXADC_BAT_TEMP_FROZE_EN_SHIFT},
	{PMIC_AUXADC_BAT_TEMP_DEBT_MAX, PMIC_AUXADC_BAT_TEMP_DEBT_MAX_ADDR,
	 PMIC_AUXADC_BAT_TEMP_DEBT_MAX_MASK, PMIC_AUXADC_BAT_TEMP_DEBT_MAX_SHIFT},
	{PMIC_AUXADC_BAT_TEMP_DEBT_MIN, PMIC_AUXADC_BAT_TEMP_DEBT_MIN_ADDR,
	 PMIC_AUXADC_BAT_TEMP_DEBT_MIN_MASK, PMIC_AUXADC_BAT_TEMP_DEBT_MIN_SHIFT},
	{PMIC_AUXADC_BAT_TEMP_DET_PRD_15_0, PMIC_AUXADC_BAT_TEMP_DET_PRD_15_0_ADDR,
	 PMIC_AUXADC_BAT_TEMP_DET_PRD_15_0_MASK, PMIC_AUXADC_BAT_TEMP_DET_PRD_15_0_SHIFT},
	{PMIC_AUXADC_BAT_TEMP_DET_PRD_19_16, PMIC_AUXADC_BAT_TEMP_DET_PRD_19_16_ADDR,
	 PMIC_AUXADC_BAT_TEMP_DET_PRD_19_16_MASK, PMIC_AUXADC_BAT_TEMP_DET_PRD_19_16_SHIFT},
	{PMIC_AUXADC_BAT_TEMP_VOLT_MAX, PMIC_AUXADC_BAT_TEMP_VOLT_MAX_ADDR,
	 PMIC_AUXADC_BAT_TEMP_VOLT_MAX_MASK, PMIC_AUXADC_BAT_TEMP_VOLT_MAX_SHIFT},
	{PMIC_AUXADC_BAT_TEMP_IRQ_EN_MAX, PMIC_AUXADC_BAT_TEMP_IRQ_EN_MAX_ADDR,
	 PMIC_AUXADC_BAT_TEMP_IRQ_EN_MAX_MASK, PMIC_AUXADC_BAT_TEMP_IRQ_EN_MAX_SHIFT},
	{PMIC_AUXADC_BAT_TEMP_EN_MAX, PMIC_AUXADC_BAT_TEMP_EN_MAX_ADDR,
	 PMIC_AUXADC_BAT_TEMP_EN_MAX_MASK, PMIC_AUXADC_BAT_TEMP_EN_MAX_SHIFT},
	{PMIC_AUXADC_BAT_TEMP_MAX_IRQ_B, PMIC_AUXADC_BAT_TEMP_MAX_IRQ_B_ADDR,
	 PMIC_AUXADC_BAT_TEMP_MAX_IRQ_B_MASK, PMIC_AUXADC_BAT_TEMP_MAX_IRQ_B_SHIFT},
	{PMIC_AUXADC_BAT_TEMP_VOLT_MIN, PMIC_AUXADC_BAT_TEMP_VOLT_MIN_ADDR,
	 PMIC_AUXADC_BAT_TEMP_VOLT_MIN_MASK, PMIC_AUXADC_BAT_TEMP_VOLT_MIN_SHIFT},
	{PMIC_AUXADC_BAT_TEMP_IRQ_EN_MIN, PMIC_AUXADC_BAT_TEMP_IRQ_EN_MIN_ADDR,
	 PMIC_AUXADC_BAT_TEMP_IRQ_EN_MIN_MASK, PMIC_AUXADC_BAT_TEMP_IRQ_EN_MIN_SHIFT},
	{PMIC_AUXADC_BAT_TEMP_EN_MIN, PMIC_AUXADC_BAT_TEMP_EN_MIN_ADDR,
	 PMIC_AUXADC_BAT_TEMP_EN_MIN_MASK, PMIC_AUXADC_BAT_TEMP_EN_MIN_SHIFT},
	{PMIC_AUXADC_BAT_TEMP_MIN_IRQ_B, PMIC_AUXADC_BAT_TEMP_MIN_IRQ_B_ADDR,
	 PMIC_AUXADC_BAT_TEMP_MIN_IRQ_B_MASK, PMIC_AUXADC_BAT_TEMP_MIN_IRQ_B_SHIFT},
	{PMIC_AUXADC_BAT_TEMP_DEBOUNCE_COUNT_MAX, PMIC_AUXADC_BAT_TEMP_DEBOUNCE_COUNT_MAX_ADDR,
	 PMIC_AUXADC_BAT_TEMP_DEBOUNCE_COUNT_MAX_MASK,
	 PMIC_AUXADC_BAT_TEMP_DEBOUNCE_COUNT_MAX_SHIFT},
	{PMIC_AUXADC_BAT_TEMP_DEBOUNCE_COUNT_MIN, PMIC_AUXADC_BAT_TEMP_DEBOUNCE_COUNT_MIN_ADDR,
	 PMIC_AUXADC_BAT_TEMP_DEBOUNCE_COUNT_MIN_MASK,
	 PMIC_AUXADC_BAT_TEMP_DEBOUNCE_COUNT_MIN_SHIFT},
	{PMIC_AUXADC_LBAT2_DEBT_MAX, PMIC_AUXADC_LBAT2_DEBT_MAX_ADDR,
	 PMIC_AUXADC_LBAT2_DEBT_MAX_MASK, PMIC_AUXADC_LBAT2_DEBT_MAX_SHIFT},
	{PMIC_AUXADC_LBAT2_DEBT_MIN, PMIC_AUXADC_LBAT2_DEBT_MIN_ADDR,
	 PMIC_AUXADC_LBAT2_DEBT_MIN_MASK, PMIC_AUXADC_LBAT2_DEBT_MIN_SHIFT},
	{PMIC_AUXADC_LBAT2_DET_PRD_15_0, PMIC_AUXADC_LBAT2_DET_PRD_15_0_ADDR,
	 PMIC_AUXADC_LBAT2_DET_PRD_15_0_MASK, PMIC_AUXADC_LBAT2_DET_PRD_15_0_SHIFT},
	{PMIC_AUXADC_LBAT2_DET_PRD_19_16, PMIC_AUXADC_LBAT2_DET_PRD_19_16_ADDR,
	 PMIC_AUXADC_LBAT2_DET_PRD_19_16_MASK, PMIC_AUXADC_LBAT2_DET_PRD_19_16_SHIFT},
	{PMIC_AUXADC_LBAT2_VOLT_MAX, PMIC_AUXADC_LBAT2_VOLT_MAX_ADDR,
	 PMIC_AUXADC_LBAT2_VOLT_MAX_MASK, PMIC_AUXADC_LBAT2_VOLT_MAX_SHIFT},
	{PMIC_AUXADC_LBAT2_IRQ_EN_MAX, PMIC_AUXADC_LBAT2_IRQ_EN_MAX_ADDR,
	 PMIC_AUXADC_LBAT2_IRQ_EN_MAX_MASK, PMIC_AUXADC_LBAT2_IRQ_EN_MAX_SHIFT},
	{PMIC_AUXADC_LBAT2_EN_MAX, PMIC_AUXADC_LBAT2_EN_MAX_ADDR, PMIC_AUXADC_LBAT2_EN_MAX_MASK,
	 PMIC_AUXADC_LBAT2_EN_MAX_SHIFT},
	{PMIC_AUXADC_LBAT2_MAX_IRQ_B, PMIC_AUXADC_LBAT2_MAX_IRQ_B_ADDR,
	 PMIC_AUXADC_LBAT2_MAX_IRQ_B_MASK, PMIC_AUXADC_LBAT2_MAX_IRQ_B_SHIFT},
	{PMIC_AUXADC_LBAT2_VOLT_MIN, PMIC_AUXADC_LBAT2_VOLT_MIN_ADDR,
	 PMIC_AUXADC_LBAT2_VOLT_MIN_MASK, PMIC_AUXADC_LBAT2_VOLT_MIN_SHIFT},
	{PMIC_AUXADC_LBAT2_IRQ_EN_MIN, PMIC_AUXADC_LBAT2_IRQ_EN_MIN_ADDR,
	 PMIC_AUXADC_LBAT2_IRQ_EN_MIN_MASK, PMIC_AUXADC_LBAT2_IRQ_EN_MIN_SHIFT},
	{PMIC_AUXADC_LBAT2_EN_MIN, PMIC_AUXADC_LBAT2_EN_MIN_ADDR, PMIC_AUXADC_LBAT2_EN_MIN_MASK,
	 PMIC_AUXADC_LBAT2_EN_MIN_SHIFT},
	{PMIC_AUXADC_LBAT2_MIN_IRQ_B, PMIC_AUXADC_LBAT2_MIN_IRQ_B_ADDR,
	 PMIC_AUXADC_LBAT2_MIN_IRQ_B_MASK, PMIC_AUXADC_LBAT2_MIN_IRQ_B_SHIFT},
	{PMIC_AUXADC_LBAT2_DEBOUNCE_COUNT_MAX, PMIC_AUXADC_LBAT2_DEBOUNCE_COUNT_MAX_ADDR,
	 PMIC_AUXADC_LBAT2_DEBOUNCE_COUNT_MAX_MASK, PMIC_AUXADC_LBAT2_DEBOUNCE_COUNT_MAX_SHIFT},
	{PMIC_AUXADC_LBAT2_DEBOUNCE_COUNT_MIN, PMIC_AUXADC_LBAT2_DEBOUNCE_COUNT_MIN_ADDR,
	 PMIC_AUXADC_LBAT2_DEBOUNCE_COUNT_MIN_MASK, PMIC_AUXADC_LBAT2_DEBOUNCE_COUNT_MIN_SHIFT},
	{PMIC_AUXADC_MDBG_DET_PRD, PMIC_AUXADC_MDBG_DET_PRD_ADDR, PMIC_AUXADC_MDBG_DET_PRD_MASK,
	 PMIC_AUXADC_MDBG_DET_PRD_SHIFT},
	{PMIC_AUXADC_MDBG_DET_EN, PMIC_AUXADC_MDBG_DET_EN_ADDR, PMIC_AUXADC_MDBG_DET_EN_MASK,
	 PMIC_AUXADC_MDBG_DET_EN_SHIFT},
	{PMIC_AUXADC_MDBG_R_PTR, PMIC_AUXADC_MDBG_R_PTR_ADDR, PMIC_AUXADC_MDBG_R_PTR_MASK,
	 PMIC_AUXADC_MDBG_R_PTR_SHIFT},
	{PMIC_AUXADC_MDBG_W_PTR, PMIC_AUXADC_MDBG_W_PTR_ADDR, PMIC_AUXADC_MDBG_W_PTR_MASK,
	 PMIC_AUXADC_MDBG_W_PTR_SHIFT},
	{PMIC_AUXADC_MDBG_BUF_LENGTH, PMIC_AUXADC_MDBG_BUF_LENGTH_ADDR,
	 PMIC_AUXADC_MDBG_BUF_LENGTH_MASK, PMIC_AUXADC_MDBG_BUF_LENGTH_SHIFT},
	{PMIC_AUXADC_MDRT_DET_PRD, PMIC_AUXADC_MDRT_DET_PRD_ADDR, PMIC_AUXADC_MDRT_DET_PRD_MASK,
	 PMIC_AUXADC_MDRT_DET_PRD_SHIFT},
	{PMIC_AUXADC_MDRT_DET_EN, PMIC_AUXADC_MDRT_DET_EN_ADDR, PMIC_AUXADC_MDRT_DET_EN_MASK,
	 PMIC_AUXADC_MDRT_DET_EN_SHIFT},
	{PMIC_AUXADC_MDRT_DET_WKUP_START_CNT, PMIC_AUXADC_MDRT_DET_WKUP_START_CNT_ADDR,
	 PMIC_AUXADC_MDRT_DET_WKUP_START_CNT_MASK, PMIC_AUXADC_MDRT_DET_WKUP_START_CNT_SHIFT},
	{PMIC_AUXADC_MDRT_DET_WKUP_START_CLR, PMIC_AUXADC_MDRT_DET_WKUP_START_CLR_ADDR,
	 PMIC_AUXADC_MDRT_DET_WKUP_START_CLR_MASK, PMIC_AUXADC_MDRT_DET_WKUP_START_CLR_SHIFT},
	{PMIC_AUXADC_MDRT_DET_WKUP_START, PMIC_AUXADC_MDRT_DET_WKUP_START_ADDR,
	 PMIC_AUXADC_MDRT_DET_WKUP_START_MASK, PMIC_AUXADC_MDRT_DET_WKUP_START_SHIFT},
	{PMIC_AUXADC_MDRT_DET_WKUP_START_SEL, PMIC_AUXADC_MDRT_DET_WKUP_START_SEL_ADDR,
	 PMIC_AUXADC_MDRT_DET_WKUP_START_SEL_MASK, PMIC_AUXADC_MDRT_DET_WKUP_START_SEL_SHIFT},
	{PMIC_AUXADC_MDRT_DET_WKUP_EN, PMIC_AUXADC_MDRT_DET_WKUP_EN_ADDR,
	 PMIC_AUXADC_MDRT_DET_WKUP_EN_MASK, PMIC_AUXADC_MDRT_DET_WKUP_EN_SHIFT},
	{PMIC_AUXADC_MDRT_DET_SRCLKEN_IND, PMIC_AUXADC_MDRT_DET_SRCLKEN_IND_ADDR,
	 PMIC_AUXADC_MDRT_DET_SRCLKEN_IND_MASK, PMIC_AUXADC_MDRT_DET_SRCLKEN_IND_SHIFT},
	{PMIC_AUXADC_MDRT_DET_RDY_ST_PRD, PMIC_AUXADC_MDRT_DET_RDY_ST_PRD_ADDR,
	 PMIC_AUXADC_MDRT_DET_RDY_ST_PRD_MASK, PMIC_AUXADC_MDRT_DET_RDY_ST_PRD_SHIFT},
	{PMIC_AUXADC_MDRT_DET_RDY_ST_EN, PMIC_AUXADC_MDRT_DET_RDY_ST_EN_ADDR,
	 PMIC_AUXADC_MDRT_DET_RDY_ST_EN_MASK, PMIC_AUXADC_MDRT_DET_RDY_ST_EN_SHIFT},
	{PMIC_AUXADC_MDRT_DET_START_SEL, PMIC_AUXADC_MDRT_DET_START_SEL_ADDR,
	 PMIC_AUXADC_MDRT_DET_START_SEL_MASK, PMIC_AUXADC_MDRT_DET_START_SEL_SHIFT},
	{PMIC_AUXADC_JEITA_IRQ_EN, PMIC_AUXADC_JEITA_IRQ_EN_ADDR, PMIC_AUXADC_JEITA_IRQ_EN_MASK,
	 PMIC_AUXADC_JEITA_IRQ_EN_SHIFT},
	{PMIC_AUXADC_JEITA_EN, PMIC_AUXADC_JEITA_EN_ADDR, PMIC_AUXADC_JEITA_EN_MASK,
	 PMIC_AUXADC_JEITA_EN_SHIFT},
	{PMIC_AUXADC_JEITA_DET_PRD, PMIC_AUXADC_JEITA_DET_PRD_ADDR, PMIC_AUXADC_JEITA_DET_PRD_MASK,
	 PMIC_AUXADC_JEITA_DET_PRD_SHIFT},
	{PMIC_AUXADC_JEITA_DEBT, PMIC_AUXADC_JEITA_DEBT_ADDR, PMIC_AUXADC_JEITA_DEBT_MASK,
	 PMIC_AUXADC_JEITA_DEBT_SHIFT},
	{PMIC_AUXADC_JEITA_MIPI_DIS, PMIC_AUXADC_JEITA_MIPI_DIS_ADDR,
	 PMIC_AUXADC_JEITA_MIPI_DIS_MASK, PMIC_AUXADC_JEITA_MIPI_DIS_SHIFT},
	{PMIC_AUXADC_JEITA_FROZE_EN, PMIC_AUXADC_JEITA_FROZE_EN_ADDR,
	 PMIC_AUXADC_JEITA_FROZE_EN_MASK, PMIC_AUXADC_JEITA_FROZE_EN_SHIFT},
	{PMIC_AUXADC_JEITA_VOLT_HOT, PMIC_AUXADC_JEITA_VOLT_HOT_ADDR,
	 PMIC_AUXADC_JEITA_VOLT_HOT_MASK, PMIC_AUXADC_JEITA_VOLT_HOT_SHIFT},
	{PMIC_AUXADC_JEITA_HOT_IRQ, PMIC_AUXADC_JEITA_HOT_IRQ_ADDR, PMIC_AUXADC_JEITA_HOT_IRQ_MASK,
	 PMIC_AUXADC_JEITA_HOT_IRQ_SHIFT},
	{PMIC_AUXADC_JEITA_VOLT_WARM, PMIC_AUXADC_JEITA_VOLT_WARM_ADDR,
	 PMIC_AUXADC_JEITA_VOLT_WARM_MASK, PMIC_AUXADC_JEITA_VOLT_WARM_SHIFT},
	{PMIC_AUXADC_JEITA_WARM_IRQ, PMIC_AUXADC_JEITA_WARM_IRQ_ADDR,
	 PMIC_AUXADC_JEITA_WARM_IRQ_MASK, PMIC_AUXADC_JEITA_WARM_IRQ_SHIFT},
	{PMIC_AUXADC_JEITA_VOLT_COOL, PMIC_AUXADC_JEITA_VOLT_COOL_ADDR,
	 PMIC_AUXADC_JEITA_VOLT_COOL_MASK, PMIC_AUXADC_JEITA_VOLT_COOL_SHIFT},
	{PMIC_AUXADC_JEITA_COOL_IRQ, PMIC_AUXADC_JEITA_COOL_IRQ_ADDR,
	 PMIC_AUXADC_JEITA_COOL_IRQ_MASK, PMIC_AUXADC_JEITA_COOL_IRQ_SHIFT},
	{PMIC_AUXADC_JEITA_VOLT_COLD, PMIC_AUXADC_JEITA_VOLT_COLD_ADDR,
	 PMIC_AUXADC_JEITA_VOLT_COLD_MASK, PMIC_AUXADC_JEITA_VOLT_COLD_SHIFT},
	{PMIC_AUXADC_JEITA_COLD_IRQ, PMIC_AUXADC_JEITA_COLD_IRQ_ADDR,
	 PMIC_AUXADC_JEITA_COLD_IRQ_MASK, PMIC_AUXADC_JEITA_COLD_IRQ_SHIFT},
	{PMIC_AUXADC_JEITA_DEBOUNCE_COUNT_COLD, PMIC_AUXADC_JEITA_DEBOUNCE_COUNT_COLD_ADDR,
	 PMIC_AUXADC_JEITA_DEBOUNCE_COUNT_COLD_MASK, PMIC_AUXADC_JEITA_DEBOUNCE_COUNT_COLD_SHIFT},
	{PMIC_AUXADC_JEITA_DEBOUNCE_COUNT_COOL, PMIC_AUXADC_JEITA_DEBOUNCE_COUNT_COOL_ADDR,
	 PMIC_AUXADC_JEITA_DEBOUNCE_COUNT_COOL_MASK, PMIC_AUXADC_JEITA_DEBOUNCE_COUNT_COOL_SHIFT},
	{PMIC_AUXADC_JEITA_DEBOUNCE_COUNT_WARM, PMIC_AUXADC_JEITA_DEBOUNCE_COUNT_WARM_ADDR,
	 PMIC_AUXADC_JEITA_DEBOUNCE_COUNT_WARM_MASK, PMIC_AUXADC_JEITA_DEBOUNCE_COUNT_WARM_SHIFT},
	{PMIC_AUXADC_JEITA_DEBOUNCE_COUNT_HOT, PMIC_AUXADC_JEITA_DEBOUNCE_COUNT_HOT_ADDR,
	 PMIC_AUXADC_JEITA_DEBOUNCE_COUNT_HOT_MASK, PMIC_AUXADC_JEITA_DEBOUNCE_COUNT_HOT_SHIFT},
	{PMIC_AUXADC_DCXO_MDRT_DET_PRD, PMIC_AUXADC_DCXO_MDRT_DET_PRD_ADDR,
	 PMIC_AUXADC_DCXO_MDRT_DET_PRD_MASK, PMIC_AUXADC_DCXO_MDRT_DET_PRD_SHIFT},
	{PMIC_AUXADC_DCXO_MDRT_DET_EN, PMIC_AUXADC_DCXO_MDRT_DET_EN_ADDR,
	 PMIC_AUXADC_DCXO_MDRT_DET_EN_MASK, PMIC_AUXADC_DCXO_MDRT_DET_EN_SHIFT},
	{PMIC_AUXADC_DCXO_MDRT_DET_WKUP_START_CNT, PMIC_AUXADC_DCXO_MDRT_DET_WKUP_START_CNT_ADDR,
	 PMIC_AUXADC_DCXO_MDRT_DET_WKUP_START_CNT_MASK,
	 PMIC_AUXADC_DCXO_MDRT_DET_WKUP_START_CNT_SHIFT},
	{PMIC_AUXADC_DCXO_MDRT_DET_WKUP_START_CLR, PMIC_AUXADC_DCXO_MDRT_DET_WKUP_START_CLR_ADDR,
	 PMIC_AUXADC_DCXO_MDRT_DET_WKUP_START_CLR_MASK,
	 PMIC_AUXADC_DCXO_MDRT_DET_WKUP_START_CLR_SHIFT},
	{PMIC_AUXADC_DCXO_MDRT_DET_WKUP_EN, PMIC_AUXADC_DCXO_MDRT_DET_WKUP_EN_ADDR,
	 PMIC_AUXADC_DCXO_MDRT_DET_WKUP_EN_MASK, PMIC_AUXADC_DCXO_MDRT_DET_WKUP_EN_SHIFT},
	{PMIC_AUXADC_DCXO_MDRT_DET_WKUP_START_SEL, PMIC_AUXADC_DCXO_MDRT_DET_WKUP_START_SEL_ADDR,
	 PMIC_AUXADC_DCXO_MDRT_DET_WKUP_START_SEL_MASK,
	 PMIC_AUXADC_DCXO_MDRT_DET_WKUP_START_SEL_SHIFT},
	{PMIC_AUXADC_DCXO_MDRT_DET_WKUP_START, PMIC_AUXADC_DCXO_MDRT_DET_WKUP_START_ADDR,
	 PMIC_AUXADC_DCXO_MDRT_DET_WKUP_START_MASK, PMIC_AUXADC_DCXO_MDRT_DET_WKUP_START_SHIFT},
	{PMIC_AUXADC_DCXO_MDRT_DET_SRCLKEN_IND, PMIC_AUXADC_DCXO_MDRT_DET_SRCLKEN_IND_ADDR,
	 PMIC_AUXADC_DCXO_MDRT_DET_SRCLKEN_IND_MASK, PMIC_AUXADC_DCXO_MDRT_DET_SRCLKEN_IND_SHIFT},
	{PMIC_AUXADC_DCXO_CH4_MUX_AP_SEL, PMIC_AUXADC_DCXO_CH4_MUX_AP_SEL_ADDR,
	 PMIC_AUXADC_DCXO_CH4_MUX_AP_SEL_MASK, PMIC_AUXADC_DCXO_CH4_MUX_AP_SEL_SHIFT},
	{PMIC_AUXADC_NAG_EN, PMIC_AUXADC_NAG_EN_ADDR, PMIC_AUXADC_NAG_EN_MASK,
	 PMIC_AUXADC_NAG_EN_SHIFT},
	{PMIC_AUXADC_NAG_CLR, PMIC_AUXADC_NAG_CLR_ADDR, PMIC_AUXADC_NAG_CLR_MASK,
	 PMIC_AUXADC_NAG_CLR_SHIFT},
	{PMIC_AUXADC_NAG_VBAT1_SEL, PMIC_AUXADC_NAG_VBAT1_SEL_ADDR, PMIC_AUXADC_NAG_VBAT1_SEL_MASK,
	 PMIC_AUXADC_NAG_VBAT1_SEL_SHIFT},
	{PMIC_AUXADC_NAG_PRD, PMIC_AUXADC_NAG_PRD_ADDR, PMIC_AUXADC_NAG_PRD_MASK,
	 PMIC_AUXADC_NAG_PRD_SHIFT},
	{PMIC_AUXADC_NAG_IRQ_EN, PMIC_AUXADC_NAG_IRQ_EN_ADDR, PMIC_AUXADC_NAG_IRQ_EN_MASK,
	 PMIC_AUXADC_NAG_IRQ_EN_SHIFT},
	{PMIC_AUXADC_NAG_C_DLTV_IRQ, PMIC_AUXADC_NAG_C_DLTV_IRQ_ADDR,
	 PMIC_AUXADC_NAG_C_DLTV_IRQ_MASK, PMIC_AUXADC_NAG_C_DLTV_IRQ_SHIFT},
	{PMIC_AUXADC_NAG_ZCV, PMIC_AUXADC_NAG_ZCV_ADDR, PMIC_AUXADC_NAG_ZCV_MASK,
	 PMIC_AUXADC_NAG_ZCV_SHIFT},
	{PMIC_AUXADC_NAG_C_DLTV_TH_15_0, PMIC_AUXADC_NAG_C_DLTV_TH_15_0_ADDR,
	 PMIC_AUXADC_NAG_C_DLTV_TH_15_0_MASK, PMIC_AUXADC_NAG_C_DLTV_TH_15_0_SHIFT},
	{PMIC_AUXADC_NAG_C_DLTV_TH_26_16, PMIC_AUXADC_NAG_C_DLTV_TH_26_16_ADDR,
	 PMIC_AUXADC_NAG_C_DLTV_TH_26_16_MASK, PMIC_AUXADC_NAG_C_DLTV_TH_26_16_SHIFT},
	{PMIC_AUXADC_NAG_CNT_15_0, PMIC_AUXADC_NAG_CNT_15_0_ADDR, PMIC_AUXADC_NAG_CNT_15_0_MASK,
	 PMIC_AUXADC_NAG_CNT_15_0_SHIFT},
	{PMIC_AUXADC_NAG_CNT_25_16, PMIC_AUXADC_NAG_CNT_25_16_ADDR, PMIC_AUXADC_NAG_CNT_25_16_MASK,
	 PMIC_AUXADC_NAG_CNT_25_16_SHIFT},
	{PMIC_AUXADC_NAG_DLTV, PMIC_AUXADC_NAG_DLTV_ADDR, PMIC_AUXADC_NAG_DLTV_MASK,
	 PMIC_AUXADC_NAG_DLTV_SHIFT},
	{PMIC_AUXADC_NAG_C_DLTV_15_0, PMIC_AUXADC_NAG_C_DLTV_15_0_ADDR,
	 PMIC_AUXADC_NAG_C_DLTV_15_0_MASK, PMIC_AUXADC_NAG_C_DLTV_15_0_SHIFT},
	{PMIC_AUXADC_NAG_C_DLTV_26_16, PMIC_AUXADC_NAG_C_DLTV_26_16_ADDR,
	 PMIC_AUXADC_NAG_C_DLTV_26_16_MASK, PMIC_AUXADC_NAG_C_DLTV_26_16_SHIFT},
	{PMIC_AUXADC_EFUSE_DEGC_CALI, PMIC_AUXADC_EFUSE_DEGC_CALI_ADDR,
	 PMIC_AUXADC_EFUSE_DEGC_CALI_MASK, PMIC_AUXADC_EFUSE_DEGC_CALI_SHIFT},
	{PMIC_AUXADC_EFUSE_ADC_CALI_EN, PMIC_AUXADC_EFUSE_ADC_CALI_EN_ADDR,
	 PMIC_AUXADC_EFUSE_ADC_CALI_EN_MASK, PMIC_AUXADC_EFUSE_ADC_CALI_EN_SHIFT},
	{PMIC_AUXADC_EFUSE_1RSV0, PMIC_AUXADC_EFUSE_1RSV0_ADDR, PMIC_AUXADC_EFUSE_1RSV0_MASK,
	 PMIC_AUXADC_EFUSE_1RSV0_SHIFT},
	{PMIC_AUXADC_EFUSE_O_VTS, PMIC_AUXADC_EFUSE_O_VTS_ADDR, PMIC_AUXADC_EFUSE_O_VTS_MASK,
	 PMIC_AUXADC_EFUSE_O_VTS_SHIFT},
	{PMIC_AUXADC_EFUSE_2RSV0, PMIC_AUXADC_EFUSE_2RSV0_ADDR, PMIC_AUXADC_EFUSE_2RSV0_MASK,
	 PMIC_AUXADC_EFUSE_2RSV0_SHIFT},
	{PMIC_AUXADC_EFUSE_O_SLOPE, PMIC_AUXADC_EFUSE_O_SLOPE_ADDR, PMIC_AUXADC_EFUSE_O_SLOPE_MASK,
	 PMIC_AUXADC_EFUSE_O_SLOPE_SHIFT},
	{PMIC_AUXADC_EFUSE_O_SLOPE_SIGN, PMIC_AUXADC_EFUSE_O_SLOPE_SIGN_ADDR,
	 PMIC_AUXADC_EFUSE_O_SLOPE_SIGN_MASK, PMIC_AUXADC_EFUSE_O_SLOPE_SIGN_SHIFT},
	{PMIC_AUXADC_EFUSE_3RSV0, PMIC_AUXADC_EFUSE_3RSV0_ADDR, PMIC_AUXADC_EFUSE_3RSV0_MASK,
	 PMIC_AUXADC_EFUSE_3RSV0_SHIFT},
	{PMIC_AUXADC_EFUSE_AUXADC_RSV, PMIC_AUXADC_EFUSE_AUXADC_RSV_ADDR,
	 PMIC_AUXADC_EFUSE_AUXADC_RSV_MASK, PMIC_AUXADC_EFUSE_AUXADC_RSV_SHIFT},
	{PMIC_AUXADC_EFUSE_ID, PMIC_AUXADC_EFUSE_ID_ADDR, PMIC_AUXADC_EFUSE_ID_MASK,
	 PMIC_AUXADC_EFUSE_ID_SHIFT},
	{PMIC_AUXADC_EFUSE_4RSV0, PMIC_AUXADC_EFUSE_4RSV0_ADDR, PMIC_AUXADC_EFUSE_4RSV0_MASK,
	 PMIC_AUXADC_EFUSE_4RSV0_SHIFT},
	{PMIC_AUXADC_RSV_1RSV0, PMIC_AUXADC_RSV_1RSV0_ADDR, PMIC_AUXADC_RSV_1RSV0_MASK,
	 PMIC_AUXADC_RSV_1RSV0_SHIFT},
	{PMIC_DA_ADCIN_VBAT_EN, PMIC_DA_ADCIN_VBAT_EN_ADDR, PMIC_DA_ADCIN_VBAT_EN_MASK,
	 PMIC_DA_ADCIN_VBAT_EN_SHIFT},
	{PMIC_DA_AUXADC_VBAT_EN, PMIC_DA_AUXADC_VBAT_EN_ADDR, PMIC_DA_AUXADC_VBAT_EN_MASK,
	 PMIC_DA_AUXADC_VBAT_EN_SHIFT},
	{PMIC_DA_ADCIN_VSEN_MUX_EN, PMIC_DA_ADCIN_VSEN_MUX_EN_ADDR, PMIC_DA_ADCIN_VSEN_MUX_EN_MASK,
	 PMIC_DA_ADCIN_VSEN_MUX_EN_SHIFT},
	{PMIC_DA_ADCIN_VSEN_EN, PMIC_DA_ADCIN_VSEN_EN_ADDR, PMIC_DA_ADCIN_VSEN_EN_MASK,
	 PMIC_DA_ADCIN_VSEN_EN_SHIFT},
	{PMIC_DA_ADCIN_CHR_EN, PMIC_DA_ADCIN_CHR_EN_ADDR, PMIC_DA_ADCIN_CHR_EN_MASK,
	 PMIC_DA_ADCIN_CHR_EN_SHIFT},
	{PMIC_DA_BATON_TDET_EN, PMIC_DA_BATON_TDET_EN_ADDR, PMIC_DA_BATON_TDET_EN_MASK,
	 PMIC_DA_BATON_TDET_EN_SHIFT},
	{PMIC_DA_ADCIN_BATID_SW_EN, PMIC_DA_ADCIN_BATID_SW_EN_ADDR, PMIC_DA_ADCIN_BATID_SW_EN_MASK,
	 PMIC_DA_ADCIN_BATID_SW_EN_SHIFT},
	{PMIC_RG_VCDT_HV_EN, PMIC_RG_VCDT_HV_EN_ADDR, PMIC_RG_VCDT_HV_EN_MASK,
	 PMIC_RG_VCDT_HV_EN_SHIFT},
	{PMIC_RGS_CHR_LDO_DET, PMIC_RGS_CHR_LDO_DET_ADDR, PMIC_RGS_CHR_LDO_DET_MASK,
	 PMIC_RGS_CHR_LDO_DET_SHIFT},
	{PMIC_RG_VCDT_DEB_SEL, PMIC_RG_VCDT_DEB_SEL_ADDR, PMIC_RG_VCDT_DEB_SEL_MASK,
	 PMIC_RG_VCDT_DEB_SEL_SHIFT},
	{PMIC_RG_PCHR_FT_CTRL, PMIC_RG_PCHR_FT_CTRL_ADDR, PMIC_RG_PCHR_FT_CTRL_MASK,
	 PMIC_RG_PCHR_FT_CTRL_SHIFT},
	{PMIC_RGS_CHRDET, PMIC_RGS_CHRDET_ADDR, PMIC_RGS_CHRDET_MASK, PMIC_RGS_CHRDET_SHIFT},
	{PMIC_RGS_VCDT_LV_DET, PMIC_RGS_VCDT_LV_DET_ADDR, PMIC_RGS_VCDT_LV_DET_MASK,
	 PMIC_RGS_VCDT_LV_DET_SHIFT},
	{PMIC_RGS_VCDT_HV_DET, PMIC_RGS_VCDT_HV_DET_ADDR, PMIC_RGS_VCDT_HV_DET_MASK,
	 PMIC_RGS_VCDT_HV_DET_SHIFT},
	{PMIC_RG_VCDT_LV_VTH, PMIC_RG_VCDT_LV_VTH_ADDR, PMIC_RG_VCDT_LV_VTH_MASK,
	 PMIC_RG_VCDT_LV_VTH_SHIFT},
	{PMIC_RG_VCDT_HV_VTH, PMIC_RG_VCDT_HV_VTH_ADDR, PMIC_RG_VCDT_HV_VTH_MASK,
	 PMIC_RG_VCDT_HV_VTH_SHIFT},
	{PMIC_RGS_PCHR_FLAG_OUT, PMIC_RGS_PCHR_FLAG_OUT_ADDR, PMIC_RGS_PCHR_FLAG_OUT_MASK,
	 PMIC_RGS_PCHR_FLAG_OUT_SHIFT},
	{PMIC_RG_PCHR_FLAG_EN, PMIC_RG_PCHR_FLAG_EN_ADDR, PMIC_RG_PCHR_FLAG_EN_MASK,
	 PMIC_RG_PCHR_FLAG_EN_SHIFT},
	{PMIC_RG_PCHR_FLAG_SEL, PMIC_RG_PCHR_FLAG_SEL_ADDR, PMIC_RG_PCHR_FLAG_SEL_MASK,
	 PMIC_RG_PCHR_FLAG_SEL_SHIFT},
	{PMIC_RG_PCHR_RSV, PMIC_RG_PCHR_RSV_ADDR, PMIC_RG_PCHR_RSV_MASK, PMIC_RG_PCHR_RSV_SHIFT},
	{PMIC_RG_ENVTEM_D, PMIC_RG_ENVTEM_D_ADDR, PMIC_RG_ENVTEM_D_MASK, PMIC_RG_ENVTEM_D_SHIFT},
	{PMIC_RG_ENVTEM_EN, PMIC_RG_ENVTEM_EN_ADDR, PMIC_RG_ENVTEM_EN_MASK,
	 PMIC_RG_ENVTEM_EN_SHIFT},
	{PMIC_RG_BATON_EN, PMIC_RG_BATON_EN_ADDR, PMIC_RG_BATON_EN_MASK, PMIC_RG_BATON_EN_SHIFT},
	{PMIC_RG_BATON_HT_EN_RSV0, PMIC_RG_BATON_HT_EN_RSV0_ADDR, PMIC_RG_BATON_HT_EN_RSV0_MASK,
	 PMIC_RG_BATON_HT_EN_RSV0_SHIFT},
	{PMIC_BATON_TDET_EN, PMIC_BATON_TDET_EN_ADDR, PMIC_BATON_TDET_EN_MASK,
	 PMIC_BATON_TDET_EN_SHIFT},
	{PMIC_RG_BATON_HT_TRIM, PMIC_RG_BATON_HT_TRIM_ADDR, PMIC_RG_BATON_HT_TRIM_MASK,
	 PMIC_RG_BATON_HT_TRIM_SHIFT},
	{PMIC_RG_BATON_HT_TRIM_SET, PMIC_RG_BATON_HT_TRIM_SET_ADDR, PMIC_RG_BATON_HT_TRIM_SET_MASK,
	 PMIC_RG_BATON_HT_TRIM_SET_SHIFT},
	{PMIC_RG_BATON_TDET_EN, PMIC_RG_BATON_TDET_EN_ADDR, PMIC_RG_BATON_TDET_EN_MASK,
	 PMIC_RG_BATON_TDET_EN_SHIFT},
	{PMIC_RG_OTG_BVALID_EN, PMIC_RG_OTG_BVALID_EN_ADDR, PMIC_RG_OTG_BVALID_EN_MASK,
	 PMIC_RG_OTG_BVALID_EN_SHIFT},
	{PMIC_RGS_OTG_BVALID_DET, PMIC_RGS_OTG_BVALID_DET_ADDR, PMIC_RGS_OTG_BVALID_DET_MASK,
	 PMIC_RGS_OTG_BVALID_DET_SHIFT},
	{PMIC_RG_PCHR_RV, PMIC_RG_PCHR_RV_ADDR, PMIC_RG_PCHR_RV_MASK, PMIC_RG_PCHR_RV_SHIFT},
	{PMIC_RG_USBDL_RST, PMIC_RG_USBDL_RST_ADDR, PMIC_RG_USBDL_RST_MASK,
	 PMIC_RG_USBDL_RST_SHIFT},
	{PMIC_RG_USBDL_SET, PMIC_RG_USBDL_SET_ADDR, PMIC_RG_USBDL_SET_MASK,
	 PMIC_RG_USBDL_SET_SHIFT},
	{PMIC_RG_ADCIN_VSEN_MUX_EN, PMIC_RG_ADCIN_VSEN_MUX_EN_ADDR, PMIC_RG_ADCIN_VSEN_MUX_EN_MASK,
	 PMIC_RG_ADCIN_VSEN_MUX_EN_SHIFT},
	{PMIC_RG_ADCIN_VSEN_EXT_BATON_EN, PMIC_RG_ADCIN_VSEN_EXT_BATON_EN_ADDR,
	 PMIC_RG_ADCIN_VSEN_EXT_BATON_EN_MASK, PMIC_RG_ADCIN_VSEN_EXT_BATON_EN_SHIFT},
	{PMIC_RG_ADCIN_VBAT_EN, PMIC_RG_ADCIN_VBAT_EN_ADDR, PMIC_RG_ADCIN_VBAT_EN_MASK,
	 PMIC_RG_ADCIN_VBAT_EN_SHIFT},
	{PMIC_RG_ADCIN_VSEN_EN, PMIC_RG_ADCIN_VSEN_EN_ADDR, PMIC_RG_ADCIN_VSEN_EN_MASK,
	 PMIC_RG_ADCIN_VSEN_EN_SHIFT},
	{PMIC_RG_ADCIN_CHR_EN, PMIC_RG_ADCIN_CHR_EN_ADDR, PMIC_RG_ADCIN_CHR_EN_MASK,
	 PMIC_RG_ADCIN_CHR_EN_SHIFT},
	{PMIC_RG_UVLO_VTHL, PMIC_RG_UVLO_VTHL_ADDR, PMIC_RG_UVLO_VTHL_MASK,
	 PMIC_RG_UVLO_VTHL_SHIFT},
	{PMIC_RG_UVLO_VH_LAT, PMIC_RG_UVLO_VH_LAT_ADDR, PMIC_RG_UVLO_VH_LAT_MASK,
	 PMIC_RG_UVLO_VH_LAT_SHIFT},
	{PMIC_RG_LBAT_INT_VTH, PMIC_RG_LBAT_INT_VTH_ADDR, PMIC_RG_LBAT_INT_VTH_MASK,
	 PMIC_RG_LBAT_INT_VTH_SHIFT},
	{PMIC_RG_BGR_RSEL, PMIC_RG_BGR_RSEL_ADDR, PMIC_RG_BGR_RSEL_MASK, PMIC_RG_BGR_RSEL_SHIFT},
	{PMIC_RG_BGR_UNCHOP_PH, PMIC_RG_BGR_UNCHOP_PH_ADDR, PMIC_RG_BGR_UNCHOP_PH_MASK,
	 PMIC_RG_BGR_UNCHOP_PH_SHIFT},
	{PMIC_RG_BGR_UNCHOP, PMIC_RG_BGR_UNCHOP_ADDR, PMIC_RG_BGR_UNCHOP_MASK,
	 PMIC_RG_BGR_UNCHOP_SHIFT},
	{PMIC_RGS_BATON_HV, PMIC_RGS_BATON_HV_ADDR, PMIC_RGS_BATON_HV_MASK,
	 PMIC_RGS_BATON_HV_SHIFT},
	{PMIC_RG_HW_VTH_CTRL, PMIC_RG_HW_VTH_CTRL_ADDR, PMIC_RG_HW_VTH_CTRL_MASK,
	 PMIC_RG_HW_VTH_CTRL_SHIFT},
	{PMIC_RG_HW_VTH2, PMIC_RG_HW_VTH2_ADDR, PMIC_RG_HW_VTH2_MASK, PMIC_RG_HW_VTH2_SHIFT},
	{PMIC_RG_HW_VTH1, PMIC_RG_HW_VTH1_ADDR, PMIC_RG_HW_VTH1_MASK, PMIC_RG_HW_VTH1_SHIFT},
	{PMIC_RG_BGR_TRIM_EN, PMIC_RG_BGR_TRIM_EN_ADDR, PMIC_RG_BGR_TRIM_EN_MASK,
	 PMIC_RG_BGR_TRIM_EN_SHIFT},
	{PMIC_RG_ICHRG_TRIM, PMIC_RG_ICHRG_TRIM_ADDR, PMIC_RG_ICHRG_TRIM_MASK,
	 PMIC_RG_ICHRG_TRIM_SHIFT},
	{PMIC_RG_BGR_TRIM, PMIC_RG_BGR_TRIM_ADDR, PMIC_RG_BGR_TRIM_MASK, PMIC_RG_BGR_TRIM_SHIFT},
	{PMIC_RG_CHR_OSC_TRIM, PMIC_RG_CHR_OSC_TRIM_ADDR, PMIC_RG_CHR_OSC_TRIM_MASK,
	 PMIC_RG_CHR_OSC_TRIM_SHIFT},
	{PMIC_DA_QI_BGR_EXT_BUF_EN, PMIC_DA_QI_BGR_EXT_BUF_EN_ADDR, PMIC_DA_QI_BGR_EXT_BUF_EN_MASK,
	 PMIC_DA_QI_BGR_EXT_BUF_EN_SHIFT},
	{PMIC_RG_BGR_TEST_EN, PMIC_RG_BGR_TEST_EN_ADDR, PMIC_RG_BGR_TEST_EN_MASK,
	 PMIC_RG_BGR_TEST_EN_SHIFT},
	{PMIC_RG_BGR_TEST_RSTB, PMIC_RG_BGR_TEST_RSTB_ADDR, PMIC_RG_BGR_TEST_RSTB_MASK,
	 PMIC_RG_BGR_TEST_RSTB_SHIFT},
	{PMIC_RG_QI_BATON_LT_EN, PMIC_RG_QI_BATON_LT_EN_ADDR, PMIC_RG_QI_BATON_LT_EN_MASK,
	 PMIC_RG_QI_BATON_LT_EN_SHIFT},
	{PMIC_RGS_BATON_UNDET, PMIC_RGS_BATON_UNDET_ADDR, PMIC_RGS_BATON_UNDET_MASK,
	 PMIC_RGS_BATON_UNDET_SHIFT},
	{PMIC_RG_VCDT_TRIM, PMIC_RG_VCDT_TRIM_ADDR, PMIC_RG_VCDT_TRIM_MASK,
	 PMIC_RG_VCDT_TRIM_SHIFT},
	{PMIC_RG_INDICATOR_TRIM, PMIC_RG_INDICATOR_TRIM_ADDR, PMIC_RG_INDICATOR_TRIM_MASK,
	 PMIC_RG_INDICATOR_TRIM_SHIFT},
	{PMIC_EOSC_CALI_START, PMIC_EOSC_CALI_START_ADDR, PMIC_EOSC_CALI_START_MASK,
	 PMIC_EOSC_CALI_START_SHIFT},
	{PMIC_EOSC_CALI_TD, PMIC_EOSC_CALI_TD_ADDR, PMIC_EOSC_CALI_TD_MASK,
	 PMIC_EOSC_CALI_TD_SHIFT},
	{PMIC_EOSC_CALI_TEST, PMIC_EOSC_CALI_TEST_ADDR, PMIC_EOSC_CALI_TEST_MASK,
	 PMIC_EOSC_CALI_TEST_SHIFT},
	{PMIC_EOSC_CALI_DCXO_RDY_TD, PMIC_EOSC_CALI_DCXO_RDY_TD_ADDR,
	 PMIC_EOSC_CALI_DCXO_RDY_TD_MASK, PMIC_EOSC_CALI_DCXO_RDY_TD_SHIFT},
	{PMIC_FRC_VTCXO0_ON, PMIC_FRC_VTCXO0_ON_ADDR, PMIC_FRC_VTCXO0_ON_MASK,
	 PMIC_FRC_VTCXO0_ON_SHIFT},
	{PMIC_EOSC_CALI_RSV, PMIC_EOSC_CALI_RSV_ADDR, PMIC_EOSC_CALI_RSV_MASK,
	 PMIC_EOSC_CALI_RSV_SHIFT},
	{PMIC_VRTC_PWM_MODE, PMIC_VRTC_PWM_MODE_ADDR, PMIC_VRTC_PWM_MODE_MASK,
	 PMIC_VRTC_PWM_MODE_SHIFT},
	{PMIC_VRTC_PWM_RSV, PMIC_VRTC_PWM_RSV_ADDR, PMIC_VRTC_PWM_RSV_MASK,
	 PMIC_VRTC_PWM_RSV_SHIFT},
	{PMIC_VRTC_PWM_L_DUTY, PMIC_VRTC_PWM_L_DUTY_ADDR, PMIC_VRTC_PWM_L_DUTY_MASK,
	 PMIC_VRTC_PWM_L_DUTY_SHIFT},
	{PMIC_VRTC_PWM_H_DUTY, PMIC_VRTC_PWM_H_DUTY_ADDR, PMIC_VRTC_PWM_H_DUTY_MASK,
	 PMIC_VRTC_PWM_H_DUTY_SHIFT},
	{PMIC_VRTC_CAP_SEL, PMIC_VRTC_CAP_SEL_ADDR, PMIC_VRTC_CAP_SEL_MASK,
	 PMIC_VRTC_CAP_SEL_SHIFT},
	{PMIC_GPIO_DIR0, PMIC_GPIO_DIR0_ADDR, PMIC_GPIO_DIR0_MASK, PMIC_GPIO_DIR0_SHIFT},
	{PMIC_GPIO_DIR0_SET, PMIC_GPIO_DIR0_SET_ADDR, PMIC_GPIO_DIR0_SET_MASK,
	 PMIC_GPIO_DIR0_SET_SHIFT},
	{PMIC_GPIO_DIR0_CLR, PMIC_GPIO_DIR0_CLR_ADDR, PMIC_GPIO_DIR0_CLR_MASK,
	 PMIC_GPIO_DIR0_CLR_SHIFT},
	{PMIC_GPIO_PULLEN0, PMIC_GPIO_PULLEN0_ADDR, PMIC_GPIO_PULLEN0_MASK,
	 PMIC_GPIO_PULLEN0_SHIFT},
	{PMIC_GPIO_PULLEN0_SET, PMIC_GPIO_PULLEN0_SET_ADDR, PMIC_GPIO_PULLEN0_SET_MASK,
	 PMIC_GPIO_PULLEN0_SET_SHIFT},
	{PMIC_GPIO_PULLEN0_CLR, PMIC_GPIO_PULLEN0_CLR_ADDR, PMIC_GPIO_PULLEN0_CLR_MASK,
	 PMIC_GPIO_PULLEN0_CLR_SHIFT},
	{PMIC_GPIO_PULLSEL0, PMIC_GPIO_PULLSEL0_ADDR, PMIC_GPIO_PULLSEL0_MASK,
	 PMIC_GPIO_PULLSEL0_SHIFT},
	{PMIC_GPIO_PULLSEL0_SET, PMIC_GPIO_PULLSEL0_SET_ADDR, PMIC_GPIO_PULLSEL0_SET_MASK,
	 PMIC_GPIO_PULLSEL0_SET_SHIFT},
	{PMIC_GPIO_PULLSEL0_CLR, PMIC_GPIO_PULLSEL0_CLR_ADDR, PMIC_GPIO_PULLSEL0_CLR_MASK,
	 PMIC_GPIO_PULLSEL0_CLR_SHIFT},
	{PMIC_GPIO_DINV0, PMIC_GPIO_DINV0_ADDR, PMIC_GPIO_DINV0_MASK, PMIC_GPIO_DINV0_SHIFT},
	{PMIC_GPIO_DINV0_SET, PMIC_GPIO_DINV0_SET_ADDR, PMIC_GPIO_DINV0_SET_MASK,
	 PMIC_GPIO_DINV0_SET_SHIFT},
	{PMIC_GPIO_DINV0_CLR, PMIC_GPIO_DINV0_CLR_ADDR, PMIC_GPIO_DINV0_CLR_MASK,
	 PMIC_GPIO_DINV0_CLR_SHIFT},
	{PMIC_GPIO_DOUT0, PMIC_GPIO_DOUT0_ADDR, PMIC_GPIO_DOUT0_MASK, PMIC_GPIO_DOUT0_SHIFT},
	{PMIC_GPIO_DOUT0_SET, PMIC_GPIO_DOUT0_SET_ADDR, PMIC_GPIO_DOUT0_SET_MASK,
	 PMIC_GPIO_DOUT0_SET_SHIFT},
	{PMIC_GPIO_DOUT0_CLR, PMIC_GPIO_DOUT0_CLR_ADDR, PMIC_GPIO_DOUT0_CLR_MASK,
	 PMIC_GPIO_DOUT0_CLR_SHIFT},
	{PMIC_GPIO_PI0, PMIC_GPIO_PI0_ADDR, PMIC_GPIO_PI0_MASK, PMIC_GPIO_PI0_SHIFT},
	{PMIC_GPIO_POE0, PMIC_GPIO_POE0_ADDR, PMIC_GPIO_POE0_MASK, PMIC_GPIO_POE0_SHIFT},
	{PMIC_GPIO0_MODE, PMIC_GPIO0_MODE_ADDR, PMIC_GPIO0_MODE_MASK, PMIC_GPIO0_MODE_SHIFT},
	{PMIC_GPIO1_MODE, PMIC_GPIO1_MODE_ADDR, PMIC_GPIO1_MODE_MASK, PMIC_GPIO1_MODE_SHIFT},
	{PMIC_GPIO2_MODE, PMIC_GPIO2_MODE_ADDR, PMIC_GPIO2_MODE_MASK, PMIC_GPIO2_MODE_SHIFT},
	{PMIC_GPIO3_MODE, PMIC_GPIO3_MODE_ADDR, PMIC_GPIO3_MODE_MASK, PMIC_GPIO3_MODE_SHIFT},
	{PMIC_GPIO4_MODE, PMIC_GPIO4_MODE_ADDR, PMIC_GPIO4_MODE_MASK, PMIC_GPIO4_MODE_SHIFT},
	{PMIC_GPIO_MODE0_SET, PMIC_GPIO_MODE0_SET_ADDR, PMIC_GPIO_MODE0_SET_MASK,
	 PMIC_GPIO_MODE0_SET_SHIFT},
	{PMIC_GPIO_MODE0_CLR, PMIC_GPIO_MODE0_CLR_ADDR, PMIC_GPIO_MODE0_CLR_MASK,
	 PMIC_GPIO_MODE0_CLR_SHIFT},
	{PMIC_GPIO5_MODE, PMIC_GPIO5_MODE_ADDR, PMIC_GPIO5_MODE_MASK, PMIC_GPIO5_MODE_SHIFT},
	{PMIC_GPIO6_MODE, PMIC_GPIO6_MODE_ADDR, PMIC_GPIO6_MODE_MASK, PMIC_GPIO6_MODE_SHIFT},
	{PMIC_GPIO7_MODE, PMIC_GPIO7_MODE_ADDR, PMIC_GPIO7_MODE_MASK, PMIC_GPIO7_MODE_SHIFT},
	{PMIC_GPIO8_MODE, PMIC_GPIO8_MODE_ADDR, PMIC_GPIO8_MODE_MASK, PMIC_GPIO8_MODE_SHIFT},
	{PMIC_GPIO9_MODE, PMIC_GPIO9_MODE_ADDR, PMIC_GPIO9_MODE_MASK, PMIC_GPIO9_MODE_SHIFT},
	{PMIC_GPIO_MODE1_SET, PMIC_GPIO_MODE1_SET_ADDR, PMIC_GPIO_MODE1_SET_MASK,
	 PMIC_GPIO_MODE1_SET_SHIFT},
	{PMIC_GPIO_MODE1_CLR, PMIC_GPIO_MODE1_CLR_ADDR, PMIC_GPIO_MODE1_CLR_MASK,
	 PMIC_GPIO_MODE1_CLR_SHIFT},
	{PMIC_GPIO_RSV, PMIC_GPIO_RSV_ADDR, PMIC_GPIO_RSV_MASK, PMIC_GPIO_RSV_SHIFT},
};


unsigned short mt6335_set_register_value(PMU_FLAGS_LIST_ENUM flagname, unsigned int val)
{
	const PMU_FLAG_TABLE_ENTRY *pFlag = &pmu_flags_table[flagname];
	unsigned int ret = 0;


	if (pFlag->flagname != flagname) {
		pr_notice("[pmu_set_register_value]pmic flag idx error\n");
		return 1;
	}

	ret = pmic_config_interface((pFlag->offset),
				    (unsigned int)(val),
				    (unsigned int)(pFlag->mask), (unsigned int)(pFlag->shift)
	    );

	return 0;
}


unsigned short mt6335_get_register_value(PMU_FLAGS_LIST_ENUM flagname)
{
	const PMU_FLAG_TABLE_ENTRY *pFlag = &pmu_flags_table[flagname];
	unsigned int val;
	unsigned int ret;

	ret =
	    pmic_read_interface((unsigned int)pFlag->offset, &val, (unsigned int)(pFlag->mask),
				(unsigned int)(pFlag->shift));

	return val;
}

unsigned short mt6335_get_register_value_nolock(PMU_FLAGS_LIST_ENUM flagname)
{
	const PMU_FLAG_TABLE_ENTRY *pFlag = &pmu_flags_table[flagname];
	unsigned int val;
	unsigned int ret;

	ret =
	    pmic_read_interface_nolock((unsigned int)pFlag->offset, &val, (unsigned int)(pFlag->mask),
				(unsigned int)(pFlag->shift));

	return val;
}



unsigned short pmic_set_register_value(PMU_FLAGS_LIST_ENUM flagname, unsigned int val)
{
	return mt6335_set_register_value(flagname, val);
}


unsigned short pmic_get_register_value(PMU_FLAGS_LIST_ENUM flagname)
{
	return mt6335_get_register_value(flagname);
}

unsigned short pmic_get_register_value_nolock(PMU_FLAGS_LIST_ENUM flagname)
{
	return mt6335_get_register_value_nolock(flagname);
}


unsigned short bc11_set_register_value(PMU_FLAGS_LIST_ENUM flagname, unsigned int val)
{
	return mt6335_set_register_value(flagname, val);
}


unsigned short bc11_get_register_value(PMU_FLAGS_LIST_ENUM flagname)
{
	return mt6335_get_register_value(flagname);
}
